package DBEditor;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# カウントデータ編集モジュール
# DBEditor.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;

    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';

    #ページ設定読み込み
    $this->import_db( $default::preference_dir . '/page',
        \%setting::page, 1, 'ranking' );

    # 削除
    push( @main::target, $main::target )
        if defined $main::target && $main::target;
    if ( $main::num && $this->is_nan($main::num) ) {
        push(
            @{ $this->{'dialog'} },
            'アクセス数修正値を確認してください。半角数字で入力してください。'
        );
    }
    elsif (@main::target) {
        my (%counter);
        tie( %counter, $env::dbm_file,
            $default::access_counter . '/access_counter',
            O_RDWR, $env::dbm_mode );
        if ( ref tied %counter ) {
            foreach my $location ( keys %counter ) {
                my $url = $this->query_decode($location);
                if ( grep( $_ eq $url, @main::target ) ) {
                    if ( defined $main::num && $main::num ) {
                        eval '$counter{$location} = $main::num';
                    }
                    else {
                        delete $counter{$location};
                    }
                }
            }
            if ( defined $main::num && $main::num ) {
                push(
                    @{ $this->{'dialog'} },
                    'アクセス数情報を修正しました。'
                );
            }
            else {
                push(
                    @{ $this->{'dialog'} },
                    'アクセス数情報を削除しました。'
                );
            }
        }
        else {
            push(
                @{ $this->{'dialog'} },
                'データベースに接続できませんでした。'
            );
        }
        untie %counter;
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print
        '<h1 title="アクセス数情報を一括削除・修正します。">カウントデータの編集</h1>';

    $this->print_file_list;

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# リスト表示
sub print_file_list ($) {
    my $this = shift;
    my ( @search, @url, $lastpage, $this_table );

    #検索
    @search
        = ( defined $main::search && $main::search ne '' )
        ? grep( $_ ne '', split( /[\x09\x0A\x0C\x0D\x20]+/, $main::search ) )
        : ();
    map( $_ = quotemeta $_, @search )
        if !defined $main::regexp || !$main::regexp;

    $main::u = 20 if !defined $main::u || !$main::u;

    #フォーム開始
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'required' => \@setting::form
    );
    print '<fieldset><legend><label for="search">URL 検索</label></legend>';
    print '<input type="text" size="50" name="search" id="search" value="'
        . $this->escape($main::search)
        . '" title="パス中に入力された文字列を含むURLのみ抽出します。" />';
    print ' <select name="regexp">';
    print '<option value="">正規表現不使用</option>';
    print '<option value="1"'
        . ( ($main::regexp) ? ' selected="selected"' : '' )
        . '>正規表現使用</option>';
    print '</select>';
    print ' <input type="submit" value="検索" />';
    print '</fieldset>';

    #解析結果テーブル
    $this_table = Table->new;
    if ( ref $table::obj{ $setting::page{'ranking'}{'table'} } ) {
        %{$this_table}
            = %{ $table::obj{ $setting::page{'ranking'}{'table'} } };
        $this_table->{'fixed'} = 1;
    }
    else {
        $this_table->set(
            'fixed'      => 1,
            'tableclass' => 'tooltable',
            'trclass'    => 'odd,even',
            'class0'     => 'odd,even',
            'thenabled'  => 'true'
        );
    }

    # アクセスカウント情報読み書き
    my ( %counter, %data );
    tie( %counter, $env::dbm_file,
        $default::access_counter . '/access_counter',
        O_RDONLY, $env::dbm_mode );
    if ( ref tied %counter ) {
        while ( my ( $location, $data ) = each %counter ) {
            utf8::decode($location) if !utf8::is_utf8($location);
            utf8::decode($data)     if !utf8::is_utf8($data);

            #解析対象ファイルチェック
            next
                if @search
                && grep( eval { $location =~ /$_/ }, @search ) == 0;

            my (@data);
            @data = split( /\x09/, $data );
            $data{$location} = $data[0] + 0;
        }
    }
    untie %counter;

    #幅
    $this_table->set(
        'styleH' =>
            'text-align: center; width: 80px,text-align: center; width: 320px,text-align: center; width: 100px',
        'style0' => 'text-align: center,text-align: center,text-align: right'
    );

    #ページ数
    @url = sort keys %data;
    $main::u = '' if $this->is_nan($main::u);
    $lastpage
        = ($main::u)
        ? (
        ( @url % $main::u ) ? int( @url / $main::u ) + 1 : @url / $main::u )
        : 1;
    if ( !$main::p || $main::p < 1 ) {
        $main::p = 1;
    }
    elsif ( $main::p > $lastpage ) {
        $main::p = $lastpage;
    }

    print '<fieldset><legend>カウンター設置 URL '
        . ( (@search) ? '検索結果' : '一覧' )
        . '</legend>';

    print ' (' . $main::p . '/' . $lastpage . 'ページ)' if $lastpage > 1;

    print $this_table->table;
    print $this_table->tr;
    print $this_table->td('削除/修正');
    print $this_table->td('URL');
    print $this_table->td('総アクセス数');
    print $this_table->_tr;

    #解析結果
    my ( $count, $location, $pageoption );
    foreach $location (@url) {
        $count++;

        #ページ
        if ($main::u) {
            if ( $main::p > 0 && $main::u * ( $main::p - 1 ) >= $count ) {
                $pageoption = 1;
                next;
            }
            elsif ( $main::p > 0 && $main::u * $main::p < $count ) {
                $pageoption = 1;
                last;
            }
        }

        print $this_table->tr;
        print $this_table->td( '<input type="checkbox" name="target" value="'
                . $this->escape( $this->query_decode($location) )
                . '" />' );
        print $this_table->td(
            '<strong>' . $this->data_key($location) . '</strong>' );
        print $this_table->td( ( $data{$location} ) ? $data{$location} : 0 );
        print $this_table->_tr;
    }
    print $this_table->_table;

    if ( defined $pageoption && $pageoption ) {
        $this->use_deco;
        print '<ul class="page-navi" id="counter-page-navi">';
        print
            '<li><label for="u" class="datatitle">表示数</label><input type="text" name="u" id="u" value="'
            . $main::u
            . '" size="3" maxlength="3" /></li>';
        print '<li>'
            . $cgi::Deco->submit_button( 'prev',
            $setting::page{'ranking'}{'prevButton'},
            '前のページ', '', { 'p' => ( $main::p - 1 ) } )
            . '</li>';
        print '<li>'
            . $cgi::Deco->submit_button( 'next',
            $setting::page{'ranking'}{'nextButton'},
            '次のページ', '', { 'p' => ( $main::p + 1 ) } )
            . '</li>';
        print '</ul>';
    }
    print '</fieldset><fieldset class="submit">';
    print
        '<label for="num" class="datatitle">アクセス数修正値</label>';
    print '<input type="text" size="18" name="num" id="num" value="'
        . $this->escape($main::num)
        . '" title="チェックされたURLへの総アクセス数を設定された値に変更します(0または無記入=削除)。" />';
    print
        ' <input type="submit" value="削除/修正" onclick="return confirm(\'この操作は取り消せません。本当に実行しますか？\');" />';
    print '</fieldset>';
    print '</form>';
}

######################################################################
# キー項目調整
sub data_key ($$) {
    my ( $this, $key ) = @_;
    return
          '<a href="'
        . $key
        . '" target="_blank">'
        . $this->jfold( $this->query_decode($key), 40, 'r' ) . '</a>';
}

sub query_decode ($$) {
    my ( $this, $query ) = @_;
    utf8::encode($query);
    $query =~ s/\+/ /g;
    $query =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
    return $this->jcode_convert( \$query, 'utf8' );
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 'c200', 'DBEditor' );

    #メニュー登録
    $setting::jump_menu{'c200'} = 'カウントデータの編集';

}

1;
