package AccessReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# アクセス数推移レポートモジュール
# AccessReport.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	利用可能なスクリーン幅
# 	$$data[9]	利用可能なスクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;

    # 月ごとに集計する
    if ( defined $setting::stat{'access'}{'mon'}
        && $setting::stat{'access'}{'mon'} )
    {
        $date = substr( $date, 0, 6 );
    }
    else {
        $date = substr( $date, 0, 8 );
    }

    #return 0 if $date !~ /^[0-9]{6,8}$/;
    $this->{'access_fluctuation'}->{$date}++;
    return 1;
}

######################################################################
# 解析環境（追加）
sub print_sub_stat_env ($) {
    my $this = shift;
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('表示順');
    print $this->{'infoTable'}->td(
              '<select name="setting:access_reverse">'
            . '<option value="">過去→現在</option>'
            . '<option value="1"'
            . (
            ( $setting::stat{'access'}{'reverse'} )
            ? ' selected="selected"'
            : ''
            )
            . '>現在→過去</option>'
            . '</select> '
            . ' <select name="setting:access_mon">'
            . '<option value="">日単位</option>'
            . '<option value="1"'
            . (
            ( $setting::stat{'access'}{'mon'} ) ? ' selected="selected"' : ''
            )
            . '>月単位</option>'
            . '</select>'
            . $this->submit_button(
            $env::image_dir . '/stat/access.gif',
            20, 20, 'アクセス数の推移', '', 'm', 'AccessReport', 'k',
            'access'
            )
    );
    print $this->{'infoTable'}->_tr;
}

######################################################################
# キー項目調整
sub stat_key ($$;$) {
    my ( $this, $key ) = @_;
    return '<strong>'
        . $this->date(
        $key,
        $setting::page{'stat'}{'datelimit'},
        $setting::page{'stat'}{'datetype'}
        ) . '</strong>';
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return 'アクセス数の推移';
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    return if defined $this->{'noLog'} && $this->{'noLog'};

    my (@date);
    @date
        = ( !@{ $this->{'access_log'} } )
        ? ()
        : $this->date_span( $this->{'access_log'}->[0],
        $this->{'access_log'}->[ $#{ $this->{'access_log'} } ] );
    @date = reverse @date
        if defined $setting::stat{'access'}{'reverse'}
        && $setting::stat{'access'}{'reverse'};
    $this->print_stat_report(
        (   ( $setting::stat{'access'}{'mon'} )
            ? '月別アクセス数'
            : '日付別アクセス数'
        ),
        \@date,
        $this->{'access_fluctuation'},
        '', '',
        $this->{'access_fluctuation'}
            ->{ $this->maximum( $this->{'access_fluctuation'} ) },
        'access', 0,
        ( ( $setting::stat{'access'}{'mon'} ) ? '月' : '日' ),
        'center', 180,
        'アクセス数',
        100, '件'
    );

}

# 期間配列
sub date_span ($$$) {
    my ( $this, $from, $to ) = @_;

    # 月ごとの場合
    if ( defined $setting::stat{'access'}{'mon'}
        && $setting::stat{'access'}{'mon'} )
    {
        my ( @date, $year, $mon, $mday, $y, $m, $d );
        ( $y,    $m )   = ( $1, $2 ) if $from =~ /^([0-9]{4})([0-9]{2})/;
        ( $year, $mon ) = ( $1, $2 ) if $to =~ /^([0-9]{4})([0-9]{2})/;

        while ( $y < $year ) {
            foreach ( $m .. 12 ) {
                push( @date, sprintf( "%.4d", $y ) . sprintf( "%.2d", $_ ) );
            }
            $m = 1;
            $y++;
        }
        while ( $m <= $mon ) {
            push( @date, sprintf( "%.4d", $y ) . sprintf( "%.2d", $m ) );
            $m++;
        }
        return @date;
    }

    # 日ごとの場合
    my ( @date, $year, $mon, $mday, $y, $m, $d, @month );
    ( $y, $m, $d ) = ( $1, $2, $3 )
        if $from =~ /^([0-9]{4})([0-9]{2})([0-9]{2})/;
    ( $year, $mon, $mday ) = ( $1, $2, $3 )
        if $to =~ /^([0-9]{4})([0-9]{2})([0-9]{2})/;

    while ( $y < $year ) {
        my ($month);
        @month = (
            0, 31, ( ( $this->is_leap_year($y) ) ? 29 : 28 ),
            31, 30, 31, 30, 31, 31, 30, 31, 30, 31
        );
        foreach $month ( $m .. 12 ) {
            foreach ( $d .. $month[$month] ) {
                push( @date,
                          sprintf( "%.4d", $y )
                        . sprintf( "%.2d", $month )
                        . sprintf( "%.2d", $_ ) );
            }
            $d = 1;
        }
        $m = 1;
        $y++;
    }
    @month = (
        0, 31, ( ( $this->is_leap_year($y) ) ? 29 : 28 ),
        31, 30, 31, 30, 31, 31, 30, 31, 30, 31
    );
    while ( $m < $mon ) {
        foreach ( $d .. $month[$m] ) {
            push( @date,
                      sprintf( "%.4d", $y )
                    . sprintf( "%.2d", $m )
                    . sprintf( "%.2d", $_ ) );
        }
        $d = 1;
        $m++;
    }
    while ( $d <= $mday ) {
        push( @date,
                  sprintf( "%.4d", $y )
                . sprintf( "%.2d", $m )
                . sprintf( "%.2d", $d ) );
        $d++;
    }

    return @date;
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'access' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'access', 'AccessReport' );

}

1;

