package Accestat;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# アクセス解析モジュール
# Accestat.pm ver1.1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-12-01
######################################################################
# 言語判定
sub get_lang ($$) {
    my ( $this, $key ) = @_;
    return '' if !$key;
    $key = lc $key;

    # 言語コード
    my %lang = (
        'ja'    => '日本語',
        'ja-jp' => '日本語',
        'af'    => 'アフリカーンス語',
        'sq'    => 'アラビア語',
        'eu'    => 'バスク語',
        'bg'    => 'ブルガリア語',
        'be'    => 'ベラルーシ語',
        'ca'    => 'カタロニア語',
        'zh'    => '中国語',
        'zh-tw' => '中国語(台湾)',
        'zh-cn' => '中国語(中国)',
        'zh-hk' => '中国語(香港)',
        'zh-sg' => '中国語(シンガポール)',
        'hr'    => 'クロアチア語',
        'cs'    => 'チェコ語',
        'da'    => 'デンマーク語',
        'nl'    => 'オランダ語(オランダ)',
        'nl-be' => 'オランダ語(ベルギー)',
        'en'    => '英語',
        'en-us' => '英語(アメリカ)',
        'en-gb' => '英語(イギリス)',
        'en-au' => '英語(オーストラリア)',
        'en-ca' => '英語(カナダ)',
        'en-nz' => '英語(ニュージーランド)',
        'en-ie' => '英語(アイルランド)',
        'en-jm' => '英語(ジャマイカ)',
        'en-bz' => '英語(ベリーズ)',
        'en-tt' => '英語(トリニダード)',
        'et'    => 'エストニア語',
        'fo'    => 'フェロー語',
        'fa'    => 'ペルシア語',
        'fi'    => 'フィンランド語',
        'fr'    => 'フランス語(フランス)',
        'fr-be' => 'フランス語(ベルギー)',
        'fr-ca' => 'フランス語(カナダ)',
        'fr-ch' => 'フランス語(スイス)',
        'fr-lu' => 'フランス語(ルクセンブルグ)',
        'gd'    => 'ゲール語',
        'de'    => 'ドイツ語(ドイツ)',
        'de-ch' => 'ドイツ語(スイス)',
        'de-at' => 'ドイツ語(オーストリア)',
        'de-lu' => 'ドイツ語(ルクセンブルグ)',
        'de-li' => 'ドイツ語(リヒテンシュタイン)',
        'el'    => 'ギリシヤ語',
        'hi'    => 'ヒンディー語',
        'hu'    => 'ハンガリー語',
        'is'    => 'アイスランド語',
        'in'    => 'インドネシア語',
        'it'    => 'イタリア語(イタリア)',
        'it-ch' => 'イタリア語(スイス)',
        'ko'    => '韓国語',
        'ko-kr' => '韓国語(大韓民国)',
        'lv'    => 'ラトビア語',
        'lt'    => 'リトアニア語',
        'mk'    => 'マケドニア語',
        'ms'    => 'マレー語',
        'mt'    => 'マルタ語',
        'no'    => 'ノルウェー語',
        'pl'    => 'ポーランド語',
        'pt-br' => 'ポルトガル語(ブラジル)',
        'pt'    => 'ポルトガル語(ポルトガル)',
        'rm'    => 'レートロマンス語',
        'ro'    => 'ルーマニア語',
        'ro-mo' => 'ルーマニア語(モルドバ)',
        'ru'    => 'ロシア語',
        'ru-mo' => 'ロシア語(モルドバ)',
        'sr'    => 'セルビア語',
        'sl'    => 'スロベニア語',
        'sb'    => 'ソルビア語',
        'es'    => 'スペイン語',
        'es-mx' => 'スペイン語(メキシコ)',
        'es-gt' => 'スペイン語(グアテマラ)',
        'es-cr' => 'スペイン語(コスタリカ)',
        'es-pa' => 'スペイン語(パナマ)',
        'es-do' => 'スペイン語',
        'es-ve' => 'スペイン語(ベネズエラ)',
        'es-co' => 'スペイン語(コロンビア)',
        'es-pe' => 'スペイン語(ペルー)',
        'es-ar' => 'スペイン語(アルゼンチン)',
        'es-ec' => 'スペイン語(エクアドル)',
        'es-cl' => 'スペイン語(チリ)',
        'es-uy' => 'スペイン語(ウルグアイ)',
        'es-py' => 'スペイン語(パラグアイ)',
        'es-bo' => 'スペイン語(ボリビア)',
        'es-sv' => 'スペイン語(エルサルバドル)',
        'es-hn' => 'スペイン語(ホンジュラス)',
        'es-ni' => 'スペイン語(ニカラグア)',
        'es-pr' => 'スペイン語(プエルトリコ)',
        'sx'    => 'スツ語',
        'sv'    => 'スウェーデン語',
        'sv-fi' => 'スウェーデン語(フィンランド)',
        'th'    => 'タイ語',
        'ts'    => 'ツォンガ語',
        'tn'    => 'ツワナ語',
        'tr'    => 'トルコ語',
        'uk'    => 'ウクライナ語',
        'ur'    => 'ウルドゥー語',
        'vi'    => 'ベトナム語',
        'xh'    => 'コーサ語',
        'ji'    => 'イディッシュ語',
        'zu'    => 'ズールー語'
    );
    if ( defined $lang{$key} && $lang{$key} ne '' ) {
        return $lang{$key} . ' [' . $key . ']';
    }
    else {
        my ($lang);
        $lang = $1 if $key =~ /^([0-9a-zA-Z_]+)\-/;
        return ( $lang && $lang{$lang} )
            ? $lang{$lang} . '(?) [' . $key . ']'
            : $key;
    }
}

sub query_to_keywords ($$;$$) {
    my ( $this, $query, $join, $cgi_url ) = @_;
    $join = 1 if !wantarray;
    my @keywords = ();

    # よくある検索フォームシンボリックネーム
    my @name = (
        's',     'search',      'p',       'MT',
        'kw',    'key',         'keyword', 'Keywords',
        'q',     'qt',          'qry',     'QUERY',
        'query', 'QueryString', 'target',  'general',
        'word',  'as_q',        'Text'
    );
    my @name2 = (
        '^search%3A[0-9a-zA-Z_]+%3A[\x09\x0A\x0C\x0D\x20]tr$',
        '^search:[0-9a-zA-Z_]+:str',
        '^FI_[0-9]+$'
    );
    if ( $query =~ /(^|&)m=[A-Za-z]+/ ) {
        @name = (
            'search',  'MT',       'kw',     'key',
            'keyword', 'Keywords', 'q',      'qt',
            'qry',     'QUERY',    'query',  'QueryString',
            'word',    'as_q',     'as_epq', 'as_oq'
        );
    }

    utf8::encode($query);
    $query =~ s/#(.*?)$//g;
    my $icode;
    if ( $query =~ /(?:\&|^)(?:_|\%5F)=([^\&=]+)(?:\&|$)/i ) {
        $icode = $this->get_charset_from_hint($1);
    }
    elsif ( $query =~ /(?:\&|^)ie=([^\&=]+)(?:\&|$)/i ) {
        $icode = $this->ename_to_jname($1);
    }
    elsif ( $query =~ /(?:\&|^)ie=([^\&=]+)(?:\&|$)/i ) {
        $icode = $this->ename_to_jname($1);
    }
    elsif ( $cgi_url && $cgi_url =~ /^[^\?]+\.(?:google|yahoo|bing)\./i ) {
        $icode = 'utf8';
    }
    $query =~ s/\+/ /g;
    while ( $query =~ /\G([^=\&]*)[=]?([^\&]*)[\&]?/sg ) {
        my ( $k, $v ) = ( $1, $2 );
        if (   ( grep( $k eq $_, @name ) || grep( $k =~ /$_/, @name2 ) )
            && $v ne 'all'
            && $v ne 'combined'
            &&    # 一部のCGI対策
            (      $v =~ /%u?[A-Fa-f0-9][A-Fa-f0-9]%[A-Fa-f0-9][A-Fa-f0-9]/
                || $v =~ /[a-zA-Z]{2}/
            )
            )
        {
            $v =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
            if ( $v =~ /(?:%[A-Fa-f0-9][A-Fa-f0-9]){2}/ ) {
                $v =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
                $v =~ s/\+/ /g;
            }
            $v = $this->jcode_convert( $v, 'utf8', $icode );
            utf8::encode($v);
            $v
                =~ s/\%u([A-Fa-f0-9][A-Fa-f0-9])([A-Fa-f0-9][A-Fa-f0-9])/$this->jcode_convert(pack("C",hex($1)) . pack("C",hex($2)),'utf8','ucs2')/ge;
            utf8::decode($v);
            $v = $this->z_to_h( $v, 'utf8' );
            $v
                =~ s/(^|[\x09\x0A\x0C\x0D\x20])cache:[^\x09\x0A\x0C\x0D\x20]+/ /g;
            $v =~ s/["']//sg;
            $v =~ s/\&quot;//sg;
            $v =~ s/^[\x09\x0A\x0C\x0D\x20]+//sg;
            $v =~ s/[\x09\x0A\x0C\x0D\x20]+$//sg;
            $v =~ s/[\x09\x0A\x0C\x0D\x20]{2,}/ /sg;
            $v = lc $v;

            if ($join) {
                push( @keywords, $v );
            }
            else {
                $v
                    =~ s/( [\&\|]+ | [\{\(\[]|[\]\)\}] | \- | and | or |\|)/ /gi;
                push( @keywords, split( /[\x09\x0A\x0C\x0D\x20]+/, $v ) );
            }
        }
    }
    if ($join) {
        my $kywd = join( ' ', @keywords );
        $kywd =~ s/[\x09\x0A\x0C\x0D\x20]+/ /g;
        return (wantarray) ? ($kywd) : $kywd;
    }
    else {
        return @keywords;
    }
}

sub url_filter ($$;$) {
    my ( $this, $url, $self ) = @_;
    utf8::decode($url);
    if ( $url =~ /^(.*)\?(.*)$/ ) {
        my ( $query, $count, %q, @del );
        ( $url, $query ) = ( $1, $2 );
        $query =~ s/\x20/\+/g;
        $url =~ s!(/search(?:;|%3B))[^/\?]+$!$1!;
        if ( !$self ) {
            my %filter = (
                'yahoo\\.' => [
                    'search\\.x', 'fr',    'tid',    'ei',
                    'meta',       'x',     'y',      'SpellState',
                    'search_x',   'sp',    'pstart', 'qrw',
                    'b',          'xargs', 'yuragi', 'fl'
                ],
                'search\\.(msn|live)\\.' =>
                    [ 'FORM', 'first', 'cp', 'noredir', 'meta' ],
                'google\\.' => [
                    'hl',     'rls',       'hs',  'btnG',
                    'lr',     'revid',     'ei',  'ie',
                    'oi',     'oe',        'sa',  'resnum',
                    'ct',     'cd',        'rlz', 'adsafe',
                    'safe',   'channel',   'aq',  'sourceid',
                    'tab',    'meta',      'x',   'sitesearch',
                    'gs_l',   'newwindow', 'tbo', 'num',
                    'bvm',    'cad',       'rct', 'source',
                    'client', 'usg',       'ved', 'esrc',
                    'sig2',   'gbv'
                ],
                'goo\\.ne\\.jp'  => [ 'SM',            'FR', 'from' ],
                'mixi\\.jp'      => [ 'comment_count', 'page' ],
                '/viewtopic.php' => ['sid'],
                '/start\\.cgi'   => [
                    'search%3A[0-9a-zA-Z_]+%3A[0-9a-zA-Z_]+',
                    'search:[0-9a-zA-Z_]+:[0-9a-zA-Z_]+', 'ct', 'f', 'u',
                    'uni', 's', 'w', 'mod', 'and', 'search', 'q', 'p'
                ]
            );
            while ( my ( $site, $queries ) = each %filter ) {
                if ( $url =~ /$site/i ) {
                    @del = @$queries;
                    last;
                }
            }
        }
    QUERY:
        while ( $query =~ /\G([^=\&]*)=([^=\&]*)\&?/g ) {
            my ( $k, $v ) = ( $1, $2 );
            if ( $url =~ m!/start.cgi$! && $k =~ /\%3D/i ) {
                next QUERY if $k =~ /\.y$/s;
                $k =~ s/\.[xy]$//;
                utf8::encode($k);
                $k =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
                utf8::decode($k);
            KEY:
                while ( $k =~ /\G([^=\&]*)=([^=\&]*)\&?/g ) {
                    my ( $k2, $v2 ) = ( $1, $2 );
                    next KEY if @del && grep( $k2 =~ /^$_$/, @del );
                    if ( $v2 eq '%03' ) {
                        $q{$k2} = $v;
                    }
                    elsif ( ref $q{$k2} eq 'ARRAY' ) {
                        push( @{ $q{$k2} }, $v2 );
                    }
                    elsif ( defined $q{$k2} ) {
                        $q{$k2} = [ $q{$k2}, $v2 ];
                    }
                    else {
                        $v2 =~ s/[\x00-\x1f]//sg;
                        $q{$k2} = $v2;
                    }
                }
            }
            else {
                next QUERY
                    if $v eq '' || ( @del && grep( $k =~ /^$_$/, @del ) );
                $v =~ s/[\x00-\x19]//sg;
                if ( ref $q{$k} eq 'ARRAY' ) {
                    push( @{ $q{$k} }, $v );
                }
                elsif ( defined $q{$k} ) {
                    $q{$k} = [ $q{$k}, $v ];
                }
                else {
                    $q{$k} = $v;
                }
            }
        }
        foreach my $k ( sort { $a cmp $b } keys %q ) {
            if ( ref $q{$k} eq 'ARRAY' ) {
                foreach my $v ( @{ $q{$k} } ) {
                    $url .= ( ( $count++ ) ? '&' : '?' ) . $k . '=' . $v;
                }
            }
            else {
                $url .= ( ( $count++ ) ? '&' : '?' ) . $k . '=' . $q{$k};
            }
        }
    }
    return $url;
}

sub query_decode ($$;$) {
    my ( $this, $query, $icode ) = @_;
    utf8::encode($query);
    $query =~ s/\+/ /g;
    $query =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
    $query
        =~ s/\%u([A-Fa-f0-9][A-Fa-f0-9])([A-Fa-f0-9][A-Fa-f0-9])/$this->jcode_convert(pack("C",hex($1)) . pack("C",hex($2)),'utf8','ucs2')/ge;
    return $this->jcode_convert( \$query, 'utf8',
        ( ($icode) ? $icode : ( $query =~ /=utf-8/i ) ? 'utf8' : '' ) );
}

######################################################################
# メイン
sub run ($) {
    my $this = shift;

    #設定保存
    $this->use_deco;
    $this->stat_setting;

    #ログファイルチェック
    $this->log_check;

    #ヘッダ
    $this->print_header;

    #解析
    $this->access_stat;

    #解析結果ヘッダ
    $this->contents_header;

    #解析結果報告
    $this->print_report;

    #フッタ
    $this->contents_footer;
    $this->print_footer;

}

######################################################################
# 解析
sub access_stat ($) {
    my $this = shift;
    my ($access_log);
    $this->{'access_count'} = 0;
    return if defined $this->{'noLog'} && $this->{'noLog'};
    foreach $access_log ( @{ $this->{'access_log'} } ) {
        local (*LOG);
        if ( open( LOG, $default::access_log . '/' . $access_log ) ) {
            binmode LOG, ":utf8";
            flock( LOG, LOCK_SH );

            while (<LOG>) {
                my (@data);
                $_ =~ s/\x0D?\x0A?$//s;
                @data = $this->svsplit( ',', $_, -1 );

                #解析対象ファイルチェック
                next
                    if @setting::target
                    && grep( eval { $data[2] =~ /$_/ }, @setting::target )
                    == 0;

                #RSSマーカー処理
                my ( $s, $m, $t );
                if ( $data[2] ) {
                    $s = $1 if $data[2] =~ s/[\?\&]utm_source=([^\&#=]*)//;
                    $m = $1 if $data[2] =~ s/[\?\&]utm_medium=([^\&#=]*)//;
                    $t = $1 if $data[2] =~ s/[\?\&]utm_term=([^\&#=]*)//;
                    $data[2] =~ s/\&/\?/
                        if ( $s || $m || $t ) && $data[2] !~ /\?/;
                }
                if ( !$data[5] && ( $s || $m || $t ) ) {
                    $data[5] = join(
                        ':',
                        map( $this->query_decode( $_, 'utf8' ),
                            grep( defined $_ && $_ ne '', $m, $t, $s ) )
                    );
                }
                elsif ( $data[5] ) {
                    $data[5] =~ s/\&/\?/
                        if $data[5] =~ s/[\?\&]utm_[0-9a-zA-Z_]+=[^\&#=]*//ig
                        && $data[5] !~ /\?/;
                }
                $this->{'access_count'}++
                    if !$this->is_skippable( \@data )
                    && $this->log_stat( $access_log, \@data );
            }
            close(LOG);
        }
    }
}

sub log_stat ($$$) {
    my ( $this, $access_log, $data ) = @_;
    return 0;
}

sub is_skippable ($$$) {
    my ( $this, $data ) = @_;
    if (   $$data[1]
        && $$data[1] =~ /(?:crawl|spider|\w+bot|nemui\.org|arege\.jp)/i )
    {
        return 1;
    }
    elsif ($$data[3]
        && $$data[3]
        =~ /(?:crawl|spider|\w+bot|https?:\/\/| Qt\/|ScreenShot|FC2BS|archive\.org|ia_archiver|wget)/i
        )
    {
        return 1;
    }
    return 0;
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    return;
}

sub print_stat_report ($$$$;$$$$$$$$$$$$$$) {
    my ($this,        $title,    $array,     $hash,
        $subhash,     $grafhash, $max,       $setting_key,
        $order,       $name,     $namealign, $namewidth,
        $num,         $numwidth, $unit,      $option,
        $optionwidth, $grafname
    ) = @_;
    my ($lastpage);

    #ヘッダ
    if ( ( !defined $setting::download || $setting::download ne 'no' )
        && $setting::stat{'download'} )
    {
        print $this->jcode_convert( $title, 'sjis', 'utf8' ) . "\x0D\x0A"
            if defined $title && $title ne '';
        if ( $setting::stat{'download'} eq 'csv' ) {
            print $this->jcode_convert(
                $this->svjoin( ',', $name, $num, $unit ),
                'sjis', 'utf8' )
                . "\x0D\x0A";
        }
        elsif ( $setting::stat{'download'} eq 'tsv' ) {
            print $this->jcode_convert(
                $this->svjoin( "\x09", $name, $num, $unit ),
                'sjis', 'utf8' )
                . "\x0D\x0A";
        }
    }
    else {
        $namewidth   = 300 if !defined $namewidth   || !$namewidth;
        $numwidth    = 50  if !defined $numwidth    || !$numwidth;
        $optionwidth = 30  if !defined $optionwidth || !$optionwidth;

        $this->{'Table'}->set(
            'styleH' => ( ($order) ? 'text-align: center; width: 30px,' : '' )
                . 'text-align: center; width: '
                . $namewidth
                . 'px,text-align: center; width: '
                . $numwidth
                . 'px,text-align: center; width: '
                . $this->{'BarGraph'}->{'barWidth'} . 'px'
                . (
                ($option)
                ? ',text-align: center; width: ' . $optionwidth . 'px'
                : ''
                ),
            'style0' => ( ($order) ? 'text-align: center; width: 30px,' : '' )
                . 'text-align: '
                . ( ($namealign) ? $namealign : 'center' )
                . '; width: '
                . $namewidth
                . 'px,text-align: right; width: '
                . $numwidth
                . 'px,text-align: left; width: '
                . $this->{'BarGraph'}->{'barWidth'} . 'px'
                . (
                ($option)
                ? ',text-align: center; width: ' . $optionwidth . 'px'
                : ''
                )
        );

        print '<div class="accestat-report">';
        print '<h2>' . $title . '</h2>' if defined $title && $title;
        if ( defined $setting_key && $setting_key ) {
            $setting::stat{$setting_key}{'u'} = ''
                if $this->is_nan( $setting::stat{$setting_key}{'u'} );
            my $cnt = @$array;
            $lastpage
                = ( $setting::stat{$setting_key}{'u'} )
                ? ( ( $cnt % $setting::stat{$setting_key}{'u'} )
                ? int( $cnt / $setting::stat{$setting_key}{'u'} ) + 1
                : $cnt / $setting::stat{$setting_key}{'u'} )
                : 1;
            if ( !$main::p || $main::p < 1 ) {
                $main::p = 1;
            }
            elsif ( $main::p > $lastpage ) {
                $main::p = $lastpage;
            }
            print '全'
                . ( ($cnt) ? $cnt : '0' ) . '件 ('
                . $main::p . '/'
                . $lastpage
                . 'ページ)';
        }
        print $this->{'Table'}->table;

        print $this->{'Table'}->tr;
        print $this->{'Table'}->td('No.') if defined $order && $order;
        print $this->{'Table'}->td($name);
        print $this->{'Table'}->td($num);
        print $this->{'Table'}->td(
            (   ( defined $grafname && $grafname ne '' )
                ? $grafname
                : 'グラフ'
            )
            . ( ($unit) ? ' (' . $unit . ')' : '' )
        );
        print $this->{'Table'}->td($option) if defined $option && $option;
        print $this->{'Table'}->_tr;
    }

    #解析結果
    my ( $count, $ranking, $prev );
    foreach my $k (@$array) {
        my $n = ( ref $grafhash ) ? $$grafhash{$k} : $$hash{$k};
        if ( ( !defined $setting::download || $setting::download ne 'no' )
            && $setting::stat{'download'} )
        {
            if ( $setting::stat{'download'} eq 'csv' ) {
                print $this->jcode_convert(
                    $this->svjoin(
                        ',', $k,
                        $this->get_num( $hash->{$k} ),
                        $this->get_ratio( $n, $max, $unit )
                    ),
                    'sjis', 'utf8'
                ) . "\x0D\x0A";
            }
            elsif ( $setting::stat{'download'} eq 'tsv' ) {
                print $this->jcode_convert(
                    $this->svjoin(
                        "\x09", $k,
                        $this->get_num( $hash->{$k} ),
                        $this->get_ratio( $n, $max, $unit )
                    ),
                    'sjis', 'utf8'
                ) . "\x0D\x0A";
            }
        }
        else {
            $count++;

            #ページ
            if (   $setting_key
                && $setting::stat{$setting_key}{'u'}
                && $setting::stat{$setting_key}{'u'} > 0 )
            {
                if (   $main::p > 0
                    && $setting::stat{$setting_key}{'u'} * ( $main::p - 1 )
                    >= $count )
                {
                    $ranking = $count
                        if defined $order
                        && $order
                        && ( !$prev || $n != $prev || !$ranking );
                    $prev = $n;
                    next;
                }
                elsif ($main::p > 0
                    && $setting::stat{$setting_key}{'u'} * $main::p < $count )
                {
                    last;
                }
            }

            print $this->{'Table'}->tr;
            if ( defined $order && $order ) {
                $ranking = $count if !$prev || $n != $prev || !$ranking;
                print $this->{'Table'}->td($ranking);
            }
            print $this->{'Table'}->td( $this->stat_key($k) );
            print $this->{'Table'}->td( $this->get_num( $$hash{$k} ) );
            print $this->{'Table'}
                ->td( $this->{'BarGraph'}->bargraph( $n, $max, $unit ) );
            print $this->{'Table'}->td( $this->stat_option( $k, $$hash{$k} ) )
                if defined $option && $option;
            print '</tr>';
            $prev = $n;
        }
        if ( ref $subhash && ref $$subhash{$k} ) {
            no warnings qw(numeric);
            foreach my $subkey (
                sort { $$subhash{$k}{$b} <=> $$subhash{$k}{$a} || $a cmp $b }
                keys %{ $$subhash{$k} } )
            {
                if ((   !defined $setting::download
                        || $setting::download ne 'no'
                    )
                    && $setting::stat{'download'}
                    )
                {
                    if ( $setting::stat{'download'} eq 'csv' ) {
                        print $this->jcode_convert(
                            $this->svjoin(
                                ',', $subkey,
                                $this->get_num( $subhash->{$k}{$subkey} ),
                                $this->get_ratio(
                                    (   ( ref $grafhash )
                                        ? $$grafhash{$subkey}
                                        : $subhash->{$k}{$subkey}
                                    ),
                                    $max, $unit
                                )
                            ),
                            'sjis', 'utf8'
                        ) . "\x0D\x0A";
                    }
                    elsif ( $setting::stat{'download'} eq 'tsv' ) {
                        print $this->jcode_convert(
                            $this->svjoin(
                                "\x09", $subkey,
                                $this->get_num( $subhash->{$k}{$subkey} ),
                                $this->get_ratio(
                                    (   ( ref $grafhash )
                                        ? $$grafhash{$subkey}
                                        : $subhash->{$k}{$subkey}
                                    ),
                                    $max, $unit
                                )
                            ),
                            'sjis', 'utf8'
                        ) . "\x0D\x0A";
                    }
                }
                else {
                    print $this->{'Table'}->tr;
                    print $this->{'Table'}->td('')
                        if defined $order && $order;
                    print $this->{'Table'}
                        ->td( $this->stat_key( $subkey, 1 ) );
                    print $this->{'Table'}
                        ->td( $this->get_num( $$subhash{$k}{$subkey} ) );
                    print $this->{'Table'}->td(
                        $this->{'BarGraph2'}->bargraph(
                            (   ( ref $grafhash )
                                ? $$grafhash{$subkey}
                                : $$subhash{$k}{$subkey}
                            ),
                            $max, $unit
                        )
                    );
                    print $this->{'Table'}->td(
                        $this->stat_option( $subkey, $$subhash{$k}{$subkey} )
                    ) if defined $option && $option;
                    print '</tr>';
                }
            }
        }
        $this->{'Table'}->{'row'}++ if ref $this->{'Table'};
    }

    if (   ( defined $setting::download && $setting::download eq 'no' )
        || !$setting::stat{'download'}
        || (   $setting::stat{'download'} ne 'csv'
            && $setting::stat{'download'} ne 'tsv' )
        )
    {
        my $form_flag = 0;
        if ( $setting::general{'postMethod'} ) {
            $this->use_deco;
            $form_flag = 1;
        }
        print $this->{'Table'}->_table;
        print '<ul class="tools">';
        print '<li>';
        $setting::page{'stat'}{'indexButton'} = ''
            if !defined $setting::page{'stat'}{'indexButton'};
        if ($form_flag) {
            print $cgi::Deco->submit_button( 'index',
                $setting::page{'stat'}{'indexButton'},
                '環境設定', '',
                { 'm' => 'StatSetting', 'k' => 'setting' } );
        }
        else {
            print $cgi::Deco->button(
                'index',
                $setting::page{'stat'}{'indexButton'},
                './start.cgi?m=StatSetting&amp;k=setting',
                '環境設定'
            );
        }
        print '</li>';
        if ( defined $setting_key && $setting_key ) {
            if ($form_flag) {
                print '<li class="isolated"><label for="setting:'
                    . $setting_key
                    . '_u" class="datatitle">表示数</label><input type="text" name="setting:'
                    . $setting_key
                    . '_u" id="setting:'
                    . $setting_key
                    . '_u" value="'
                    . $this->escape( $setting::stat{$setting_key}{'u'} )
                    . '" size="3" maxlength="3" />';
                if ( defined $order && $order ) {
                    print '<select name="setting:' . $setting_key . '_sort">';
                    print '<option value="descend"'
                        . (
                        ( $setting::stat{$setting_key}{'sort'} eq 'descend' )
                        ? ' selected="selected"'
                        : '' )
                        . '>降順</option>';
                    print '<option value="ascend"'
                        . (
                        ( $setting::stat{$setting_key}{'sort'} eq 'ascend' )
                        ? ' selected="selected"'
                        : '' )
                        . '>昇順</option>';
                    print '</select>';
                }
                print '<input type="submit" name="m='
                    . $main::m
                    . '&amp;k='
                    . $main::k
                    . '" value="再読み込み" /></li>';
            }
            if ( $main::p > 1 ) {
                print '<li>';
                if ($form_flag) {
                    print $cgi::Deco->submit_button(
                        'prev',
                        $setting::page{'stat'}{'prevButton'},
                        '前のページ',
                        '',
                        {   'm' => $main::m,
                            'k' => $main::k,
                            'p' => ( $main::p - 1 )
                        }
                    );
                }
                else {
                    print $cgi::Deco->button(
                        'prev',
                        $setting::page{'stat'}{'prevButton'},
                        './start.cgi'
                            . $this->set_query( 'url', [ 'm', 'k' ] )
                            . $this->add_query(
                            'url', { 'p' => $main::p - 1 }
                            ),
                        '前のページ'
                    );
                }
                print '</li>';
            }
            if ( $main::p < $lastpage ) {
                print '<li>';
                if ($form_flag) {
                    print $cgi::Deco->submit_button(
                        'next',
                        $setting::page{'stat'}{'nextButton'},
                        '次のページ',
                        '',
                        {   'm' => $main::m,
                            'k' => $main::k,
                            'p' => ( $main::p + 1 )
                        }
                    );
                }
                else {
                    print $cgi::Deco->button(
                        'next',
                        $setting::page{'stat'}{'nextButton'},
                        './start.cgi'
                            . $this->set_query( 'url', [ 'm', 'k' ] )
                            . $this->add_query(
                            'url', { 'p' => $main::p + 1 }
                            ),
                        '次のページ'
                    );
                }
                print '</li>';
            }
        }
        print '</ul></div>';

    }
    else {
        print "\x0D\x0A";
    }
}

sub stat_option ($$$) {
    my ( $this, $key, $value ) = @_;
}

sub stat_key ($$;$) {
    my ( $this, $key, $normal_weight ) = @_;
    return ($normal_weight)
        ? $this->escape_except_html_entity($key)
        : '<strong>' . $this->escape_except_html_entity($key) . '</strong>';
}

sub get_num ($$) {
    my ( $this, $num ) = @_;
    return ($num) ? $num : 0;
}

sub get_ratio ($$$) {
    my ( $this, $point, $total, $unit ) = @_;
    if ( $unit eq '%' ) {
        return (
            int( ( ( ($total) ? $point / $total : 0 ) * 10000 ) + 0.5 )
                / 100 );
    }
    else {
        return $this->get_num($point);
    }
}

######################################################################
# 解析メニュー
sub print_stat_menu ($) {
    my $this = shift;

    #return if defined $this->{'noLog'} && $this->{'noLog'};

    print '<ul class="tools">';
    print '<li><img src="'
        . $env::image_dir
        . '/stat/leftend.gif" width="5" height="20" alt="ここからツールメニュー" border="0" /></li>';

    # 設定ページ
    print '<li>'
        . $this->submit_button( $env::image_dir . '/stat/setting.gif',
        20, 20, '環境設定', '', 'm', 'StatSetting', 'k', 'setting' )
        . '</li>';

    # アクセスの概要
    print '<li>'
        . $this->submit_button( $env::image_dir . '/stat/general.gif',
        20, 20, 'アクセスの概要', '', 'm', 'GeneralReport', 'k',
        'general' )
        . '</li>';

    # アクセス数の推移
    print '<li>'
        . $this->submit_button( $env::image_dir . '/stat/access.gif',
        20, 20, 'アクセス数の推移', '', 'm', 'AccessReport', 'k',
        'access' )
        . '</li>';

    # ファイル別アクセス数ランキング
    print '<li>'
        . $this->submit_button(
        $env::image_dir . '/stat/request.gif',
        20,
        20,
        'ファイル別アクセス数ランキング',
        '',
        'm',
        'RequestReport',
        'k',
        'request'
        ) . '</li>';

    # クリック率
    print '<li>'
        . $this->submit_button( $env::image_dir . '/stat/click.gif',
        20, 20, 'クリック率', '', 'm', 'ClickRatioReport', 'k', 'click' )
        . '</li>';

    # リンク元ランキング
    print '<li>'
        . $this->submit_button( $env::image_dir . '/stat/referer.gif',
        20, 20, 'リンク元ランキング', '', 'm', 'RefererReport', 'k',
        'referer' )
        . '</li>';

    # アクセス元ランキング
    print '<li>'
        . $this->submit_button(
        $env::image_dir . '/stat/client.gif',
        20, 20, 'アクセス元ランキング',
        '', 'm', 'ClientReport', 'k', 'client'
        ) . '</li>';

    # 検索キーワードランキング
    print '<li>'
        . $this->submit_button(
        $env::image_dir . '/stat/keyword.gif',
        20, 20, '検索キーワードランキング',
        '', 'm', 'KeywordReport', 'k', 'keyword'
        ) . '</li>';

    # 訪問者の足跡再現
    print '<li>'
        . $this->submit_button( $env::image_dir . '/stat/tracer.gif',
        20, 20, '訪問者の足跡再現', '', 'm', 'TracerReport', 'k',
        'tracer' )
        . '</li>';

    # 訪問者の閲覧環境
    print '<li>'
        . $this->submit_button( $env::image_dir . '/stat/agent.gif',
        20, 20, '訪問者の閲覧環境', '', 'm', 'AgentReport', 'k',
        'agent' )
        . '</li>';

    # 訪問者の言語環境
    print '<li>'
        . $this->submit_button( $env::image_dir . '/stat/language.gif',
        20, 20, '訪問者の言語環境', '', 'm', 'LanguageReport', 'k',
        'language' )
        . '</li>';

    # 訪問者のモニター環境
    print '<li>'
        . $this->submit_button(
        $env::image_dir . '/stat/monitor.gif',
        20, 20, '訪問者のモニター環境',
        '', 'm', 'MonitorReport', 'k', 'monitor'
        ) . '</li>';

    # 管理モード
    if ( $this->is_secure('tools') ) {
        print '<li>'
            . $this->submit_button( $env::image_dir . '/stat/tools.gif',
            20, 20, '管理モード', '', 'm', 'Tools', 'k', '0000' )
            . '</li>';
    }

    print '<li><img src="'
        . $env::image_dir
        . '/stat/rightend.gif" width="5" height="20" alt="ツールメニュー終わり" border="0" /></li>';
    print '</ul>';
}

######################################################################
# コンテンツヘッダ
sub contents_header ($) {
    my $this = shift;
    return
           if ( !defined $setting::download || $setting::download ne 'no' )
        && defined $setting::stat{'download'}
        && ( $setting::stat{'download'} eq 'csv'
        || $setting::stat{'download'} eq 'tsv' );

    #フォーム開始
    print $this->form_start(
        'key'  => 'stat',
        'name' => 'input'
    );

    #解析メニュー
    $this->print_stat_menu;

    #解析条件テーブル
    $this->{'infoTable'} = Table->new;
    if ( defined $setting::page{'stat'}{'infoTable'}
        && ref $table::obj{ $setting::page{'stat'}{'infoTable'} } )
    {
        %{ $this->{'infoTable'} }
            = %{ $table::obj{ $setting::page{'stat'}{'infoTable'} } };
        $this->{'infoTable'}->{'fixed'} = 0;
    }
    else {
        $this->{'infoTable'}->set(
            'fixed'      => 0,
            'tableclass' => 'datatable',
            'trclass'    => 'odd,even',
            'class0'     => 'odd,even',
        );
    }

    #解析結果テーブル
    $this->{'Table'} = Table->new;
    if ( defined $setting::page{'stat'}{'table'}
        && ref $table::obj{ $setting::page{'stat'}{'table'} } )
    {
        %{ $this->{'Table'} }
            = %{ $table::obj{ $setting::page{'stat'}{'table'} } };
        $this->{'Table'}->{'fixed'} = 0;
    }
    else {
        $this->{'Table'}->set(
            'fixed'      => 0,
            'tableclass' => 'datatable',
            'trclass'    => 'odd,even',
            'class0'     => 'odd,even',
            'thenabled'  => 'true'
        );
    }

    #グラフモジュール読み込み
    require CMONOS::CGI_Lib::BarGraph;
    $this->{'BarGraph'} = BarGraph->new(
        \%setting::deco,
        $setting::page{'stat'}{'barColor'},
        $setting::page{'stat'}{'bgColor'},
        $setting::page{'stat'}{'barImage'},
        $setting::page{'stat'}{'bgImage'},
        $setting::page{'stat'}{'barWidth'}
    );
    $this->{'BarGraph2'} = BarGraph->new(
        \%setting::deco,
        $setting::page{'stat'}{'barColor2'},
        $setting::page{'stat'}{'bgColor2'},
        $setting::page{'stat'}{'barImage2'},
        $setting::page{'stat'}{'bgImage2'},
        $setting::page{'stat'}{'barWidth'}
    );

    #解析条件
    $this->stat_env;

    #タイトル
    print '<h1>' . $this->stat_title . '</h1>';
}

sub stat_title ($) {
    my $this = shift;
    return 'アクセス解析';
}

sub contents_footer ($) {
    my $this = shift;
    return
           if ( !defined $setting::download || $setting::download ne 'no' )
        && defined $setting::stat{'download'}
        && ( $setting::stat{'download'} eq 'csv'
        || $setting::stat{'download'} eq 'tsv' );

    #解析メニュー
    $this->print_stat_menu;

    print '</form>';
}

######################################################################
# 解析条件
sub stat_env ($) {
    my $this = shift;

    print $this->{'infoTable'}->table( 'id', 'accestat-env' );

    #解析対象
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('解析対象ファイル');
    print $this->{'infoTable'}->td( $this->stat_target );
    print $this->{'infoTable'}->_tr;

    #解析対象期間
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('解析対象期間');
    print $this->{'infoTable'}->td( $this->stat_span );
    print $this->{'infoTable'}->_tr;

    #解析対象期間
    if ( defined $this->{'access_count'} && $this->{'access_count'} ne '' ) {
        print $this->{'infoTable'}->tr;
        print $this->{'infoTable'}->th('解析対象アクセス');
        print $this->{'infoTable'}->td( $this->{'access_count'},
            'title', '解析したアクセスの総数です。' );
        print $this->{'infoTable'}->_tr;
    }

    #解析別条件
    $this->print_sub_stat_env;

    print $this->{'infoTable'}->_table;
}

sub print_sub_stat_env ($) {
    my $this = shift;
}

sub stat_target ($) {
    my $this = shift;
    if (@setting::target) {
        my $setting = join( '<br />', @setting::target );
        if ( defined $setting::stat{'target'}{'regexp'}
            && $setting::stat{'target'}{'regexp'} )
        {
            return ( $setting, 'title',
                'パスがいずれかに一致するファイルのみ解析対象となります（正規表現を含んでいます）。'
            );
        }
        else {
            $setting =~ s/\\(?!\\)//g;
            return ( $setting, 'title',
                'パスがいずれかに一致するファイルのみ解析対象となります（正規表現を含みません）。'
            );
        }
    }
    else {
        return ('全てのファイル');
    }
}

sub stat_span ($) {
    my $this = shift;
    return '解析可能なログファイルが存在しません。'
        if defined $this->{'noLog'} && $this->{'noLog'};

    my ($span_start);
    if ( defined $setting::stat{'span'}{'start'}
        && $setting::stat{'span'}{'start'} )
    {
        $span_start = $setting::stat{'span'}{'start'};
    }
    else {
        $span_start = $1 if $this->{'access_log'}->[0] =~ /^([0-9]{8})/;
    }
    return $this->date(
        $span_start,
        $setting::page{'stat'}{'datelimit'},
        $setting::page{'stat'}{'datetype'}
        )
        . '〜'
        . $this->date(
        $setting::stat{'span'}{'end'},
        $setting::page{'stat'}{'datelimit'},
        $setting::page{'stat'}{'datetype'}
        );
}

######################################################################
# 設定書き込み/読み込み
sub stat_setting ($) {
    my $this = shift;
    $this->_stat_setting;

    #ターゲットファイル
    if ( !@default::target ) {
        my $home_location = $this->home_location;
        $home_location =~ s!^http[s]?://(?:www\.)?!!si;
        push( @default::target, 'http://' . $home_location, 'https://' . $home_location, 'http://www.' . $home_location, 'https://www.' . $home_location );
    }

    #書き込み/読み込み
    if (( %main::setting || %main::span )
        && (!$main::setting{'target_path'}
            || (   $main::setting{'target_path'}
                && $main::setting{'target_path'} !~ /[<>]/
                && $main::setting{'target_path'} !~ /\[url/i )
        )
        )
    {
        if (%main::span) {
            $main::setting{'span_start'}
                = $this->get_stat_span( %{ $main::span{'start'} } );
            $main::setting{'span_end'}
                = $this->get_stat_span( %{ $main::span{'end'} } );
            ( $main::setting{'span_start'}, $main::setting{'span_end'} )
                = ( $main::setting{'span_end'}, $main::setting{'span_start'} )
                if $main::setting{'span_start'}
                && $main::setting{'span_end'}
                && $main::setting{'span_start'} > $main::setting{'span_end'};
        }
        $main::setting{'target_path'} =~ s/[<>"]//gs
            if defined $main::setting{'target_path'}
            && $main::setting{'target_path'};
        $main::setting{'client_target'} =~ s/[<>"]//gs
            if defined $main::setting{'client_target'}
            && $main::setting{'client_target'};
        $main::setting{'client_referer'} =~ s/[<>"]//gs
            if defined $main::setting{'client_referer'}
            && $main::setting{'client_referer'};
        $main::setting{'click_target'} =~ s/[<>"]//gs
            if defined $main::setting{'click_target'}
            && $main::setting{'click_target'};
        $main::setting{'tracer_target'} =~ s/\+/ /gs
            if defined $main::setting{'tracer_target'}
            && $main::setting{'tracer_target'};

        if ( defined $main::targetreset && $main::targetreset ) {
            foreach ( 0 .. 10 ) {
                $main::setting{ 'recenttarget_' . $_ } = '';
            }
        }
        elsif ( defined $main::setting{'target_path'}
            && $main::setting{'target_path'} )
        {
            my (%temp);
            $this->import_db( $default::preference_dir . '/stat',
                \%temp, 1, 'recenttarget' );
            my ( $n, $e, @recenttarget );
            foreach ( sort keys %{ $temp{'recenttarget'} } ) {
                push( @recenttarget, $temp{'recenttarget'}{$_} )
                    if defined $temp{'recenttarget'}{$_}
                    && $temp{'recenttarget'}{$_} ne '';
                $e++
                    if $temp{'recenttarget'}{$_} eq
                    $main::setting{'target_path'};
            }
            unshift( @recenttarget, $main::setting{'target_path'} )
                if !defined $e || !$e;
            $n = 0;
            foreach (@recenttarget) {
                last if $n > 9;
                $main::setting{ 'recenttarget_' . $n++ } = $_;
            }
        }
        foreach my $k ( keys %main::setting ) {
            delete $main::setting{$k}
                if $k =~ /_u$/
                && $main::setting{$k}
                && $this->is_nan( $main::setting{$k} );
        }

        $this->write_db( $default::preference_dir . '/stat',
            \%main::setting, '', '', \%setting::stat, 1 );

        #読み込みのみ
    }
    else {
        $this->import_db( $default::preference_dir . '/stat',
            \%setting::stat, 1 );
    }

    if ( defined $setting::stat{'target'}{'path'}
        && $setting::stat{'target'}{'path'} )
    {
        @setting::target = $this->mk_regexp(
            $setting::stat{'target'}{'path'},
            $setting::stat{'target'}{'regexp'}
        );
    }
    if ( defined $setting::stat{'client'}{'target'}
        && $setting::stat{'client'}{'target'} )
    {
        @setting::target_client = $this->mk_regexp(
            $setting::stat{'client'}{'target'},
            $setting::stat{'client'}{'regexp'}
        );
    }
    if ( defined $setting::stat{'client'}{'referer'}
        && $setting::stat{'client'}{'referer'} )
    {
        @setting::target_client_referer = $this->mk_regexp(
            $setting::stat{'client'}{'referer'},
            $setting::stat{'client'}{'refererRegexp'}
        );
    }
    if ( defined $setting::stat{'client'}{'url'}
        && $setting::stat{'client'}{'url'} )
    {
        @setting::target_client_url = $this->mk_regexp(
            $setting::stat{'client'}{'url'},
            $setting::stat{'client'}{'urlRegexp'}
        );
    }
    if ( ref $setting::stat{'recenttarget'} eq 'HASH'
        && %{ $setting::stat{'recenttarget'} } )
    {
        foreach ( sort keys %{ $setting::stat{'recenttarget'} } ) {
            push( @setting::recenttarget, $setting::stat{'recenttarget'}{$_} )
                if defined $setting::stat{'recenttarget'}{$_}
                && $setting::stat{'recenttarget'}{$_} ne '';
        }
        undef %{ $setting::stat{'recenttarget'} };
    }
}

sub mk_regexp ($$;$) {
    my ( $this, $key, $regexp ) = @_;
    my @target = grep( $_ ne '', split( /[\x09\x0A\x0C\x0D\x20]+/, $key ) );
    if ($regexp) {
        map( $_ = '^' . quotemeta($_) . '$', @target ) if $regexp == 2;
    }
    else {
        map( $_ = quotemeta $_, @target );
    }
    return @target;
}

sub get_stat_span ($@) {
    my ( $this, %span ) = @_;
    return '' unless $span{'year'} && $span{'mon'} && $span{'mday'};
    if ($this->is_impossible_mday(
            $span{'year'}, $span{'mon'}, $span{'mday'}
        )
        )
    {
        my @mdays = ( 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
        $span{'mday'} = $mdays[ $span{'mon'} ];
    }
    return
          sprintf( "%.4d", $span{'year'} )
        . sprintf( "%.2d", $span{'mon'} )
        . sprintf( "%.2d", $span{'mday'} );
}

sub _stat_setting ($) {
    my $this = shift;
    return;
}

######################################################################
# ヘッダ/フッタ
sub print_header ($) {
    my $this = shift;
    if (   ( !defined $setting::download || $setting::download ne 'no' )
        && defined $setting::stat{'download'}
        && $setting::stat{'download'} eq 'csv' )
    {
        binmode STDOUT;

        #$this->print_http_header('text/comma-separated-values');
        $this->print_http_header('application/csv');
    }
    elsif (( !defined $setting::download || $setting::download ne 'no' )
        && defined $setting::stat{'download'}
        && $setting::stat{'download'} eq 'tsv' )
    {
        binmode STDOUT;

        #$this->print_http_header('text/tab-separated-values');
        $this->print_http_header('application/tab-separated-values');
    }
    else {
        binmode STDOUT, ":utf8";
        $this->set_og_url( $this->my_url );
        $this->push_window_title( $this->stat_title );
        $this->print_page_header('stat');
    }
}

sub print_footer ($) {
    my $this = shift;

    # ページフッタ
    $this->print_page_footer('stat')
        if ( defined $setting::download && $setting::download eq 'no' )
        || !$setting::stat{'download'}
        || ( $setting::stat{'download'} ne 'csv'
        && $setting::stat{'download'} ne 'tsv' );
}

######################################################################
# ログファイルチェック
sub log_check ($) {
    no warnings qw(numeric);
    my $this = shift;
    my ( $time, @localtime, $logend, $from, $to );
    $this->{'access_log_size'} = 0;

    # アクセスログディレクトリのサイズを求める
    opendir( LOG, $default::access_log );
    @{ $this->{'access_log'} }
        = grep( $_ =~ /[0-9]{8}\.csv$/
            && ( $this->{'access_log_size'}
            += -s $default::access_log . '/' . $_ ),
        readdir(LOG) );
    closedir(LOG);

    # 当日のログファイルを除外/ソート
    $time = time;
    $time += 86400
        if defined $setting::general{'realtime'}
        && $setting::general{'realtime'};
    @localtime = localtime($time);
    $logend
        = ( $localtime[5] + 1900 )
        . sprintf( "%.2d", $localtime[4] + 1 )
        . sprintf( "%.2d", $localtime[3] );
    @{ $this->{'access_log'} } = sort { $a <=> $b }
        grep( eval { $_ < $logend }, @{ $this->{'access_log'} } );

    if ( @{ $this->{'access_log'} } ) {
        $this->{'year_start'} = $1
            if ${ $this->{'access_log'} }[0] =~ /^([0-9]{4})/;
        $this->{'year_end'} = $1
            if ${ $this->{'access_log'} }[ $#{ $this->{'access_log'} } ]
            =~ /^([0-9]{4})/;

        $this->{'access_log_start'} = $1
            if $this->{'access_log'}->[0] =~ /^([0-9]{8})/;
        $this->{'access_log_end'} = $1
            if $this->{'access_log'}->[ $#{ $this->{'access_log'} } ]
            =~ /^([0-9]{8})/;

        if (   $setting::stat{'span'}{'end'} == 0
            || $setting::stat{'span'}{'end'} >= $logend )
        {
            $setting::stat{'span'}{'end'} = $this->{'access_log_end'};
            $this->{'noEnd'} = 1;
        }
        @{ $this->{'access_log'} }
            = grep( eval { $_ >= $setting::stat{'span'}{'start'} }
                && eval { $_ <= $setting::stat{'span'}{'end'} },
            @{ $this->{'access_log'} } );

    }
    else {
        push(
            @{ $this->{'dialog'} },
            '解析可能なログファイルが存在しません。'
        );
        $this->{'noLog'} = 1;
    }
}

sub submit_button ($$$$$@) {
    my ( $this, $src, $width, $height, $alt, $onclick, %query ) = @_;
    my ( $name, $form );
    if (%query) {
        $name = $this->add_query( 'url', \%query );
        $name =~ s/^\&//;
        $name =~ s/\&/\&amp;/g;
    }
    if ( $src =~ /\.(gif|jpg|jpeg|png)$/i ) {
        $form = '<input type="image"';
        $form .= ' name="' . $name . '"'     if $name;
        $form .= ' width="' . $width . '"'   if $width;
        $form .= ' height="' . $height . '"' if $height;
        $form
            .= ' src="'
            . ( ($cgi::output)
            ? $this->base_change( $src, $cgi::output, $cgi::output_option )
            : $src )
            . '"';
        $form .= ' title="' . $alt . '" alt="' . $alt . '"'
            if defined $alt && $alt ne '';
        $form .= ' onclick="' . $onclick . '"'
            if defined $onclick && $onclick ne '';
        $form .= ' border="0" value="1" />';
    }
    else {
        $form
            = '<input type="submit" name="'
            . $name
            . '" value="'
            . $alt . '"'
            . ( ( defined $onclick && $onclick ne '' )
            ? ' onclick="' . $onclick . '"'
            : '' )
            . ' />';
    }
    return $form;
}

sub tie_title_db {
    my $this = shift;
    return if !$setting::general{'getTitle'};
    require CMONOS::CGI_Lib::HTTP_Client;
    eval 'use ' . $env::dbm_file . ';';
    tie(%{ $this->{'url_to_title'} },
        $env::dbm_file,
        $default::preference_dir . '/url_to_title',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
}

sub untie_title_db {
    my $this = shift;
    return if !$setting::general{'getTitle'};
    untie( %{ $this->{'url_to_title'} } );
}

sub url_to_title ($$$;$) {
    my ( $this, $url, $site, $redirect ) = @_;
    return $this->jfold( $this->escape($url), 40, 'r', 0, '<wbr />' )
        if !$url || $url !~ m!^https?://!i;
    my ( $keyword, $cgi_url, $target_url, $title );
    $url =~ s/\/search;[^\/]*?\?/\/search?/;
    if ( $url =~ /^(.*)\?(.*)$/ ) {
        $cgi_url = $1;
        $keyword = $this->query_to_keywords( $2, 0, $cgi_url );
        $target_url
            = (
            $keyword
                || ( $setting::general{'getTitle'}
                && $setting::general{'getTitle'} eq 'static' )
            )
            ? (
            (   $url
                    =~ m!^https?://www\.google\.[a-z\.]+/hws/search\?.*\&?(client=[^=\&]+)!
            ) ? $cgi_url . '?' . $1 : $cgi_url
            )
            : $this->url_filter($url);
    }
    else {
        $target_url = $url;
    }
    if ( $setting::general{'getTitle'}
        && ref tied %{ $this->{'url_to_title'} } )
    {
        if ( $this->{'url_to_title'}{ $this->encode_utf8($target_url) } ) {
            $title = $this->decode_utf8(
                $this->{'url_to_title'}{ $this->encode_utf8($target_url) } );
        }
        else {
            my $res = HTTP_Client->new->request( 'GET' => $target_url );
            my $content;
            $content = $res->content('utf8') if $res->is_success;
            if ($content
                && ((      $setting::general{'getShortTitle'}
                        && $content
                        =~ m/<meta name="title" content="([^">]+?)" \/>/si
                    )
                    || $content
                    =~ m/<title(?![0-9a-zA-Z_])[^>]*>(.*?)<\/title(?![0-9a-zA-Z_])[^>]*>/si
                    || $content
                    =~ m/<h[1-6](?![0-9a-zA-Z_])[^>]*>(.*?)<\/h[1-6]>/si
                )
                )
            {
                $title = $this->unescape($1);
                if (   $target_url =~ m!/new/$!
                    && $content =~ /<base href="[^"]+" \/>/
                    && $content
                    =~ m/<meta name="title" content="([^">]+?)" \/>/si )
                {
                    my $shorttitle = $this->unescape($1);
                    $title =~ s/$shorttitle/最新記事/g;
                }
                elsif ( $target_url =~ m!/start\.cgi$! ) {
                    unless ( $title
                        =~ s/([\x09\x0A\x0C\x0D\x20]*[:\|\-][\x09\x0A\x0C\x0D\x20]*|)[0-9]{4}年[0-9]{1,2}月( 第[0-9]週|[0-9]{1,2}日\(.*?\)|)//
                        )
                    {
                        if (   $setting::general{'getTitle'}
                            && $setting::general{'getTitle'} eq 'static'
                            && $content
                            =~ m/<meta name="title" content="([^">]+?)" \/>/si
                            )
                        {
                            my $shorttitle = $1;
                            $title
                                =~ s/([\x09\x0A\x0C\x0D\x20]*[:\|\-][\x09\x0A\x0C\x0D\x20]*|)$shorttitle//g;
                        }
                    }
                }
            }
            if ($title) {
                $title = $this->cut_string( $title, 128, 'no' );
            }
            else {
                $title = $this->query_decode($target_url);
            }
            undef $content;
            undef $res;
            $this->{'url_to_title'}{ $this->encode_utf8($target_url) }
                = $this->encode_utf8($title);
        }
    }
    else {
        $title = $cgi_url || $url;
        $title =~ s!^https?://!!i if $site;
        $title = $this->query_decode($title);
    }
    $title .= '(' . $keyword . ')' if $keyword;
    $url =~ s!^(http://www.google.[\.0-9a-zA-Z_]+/)url\?!$1search\?!;
    $url =~ s/\%2B/\+/sg
        if $url !~ /\%2503/ && $url =~ s/\%25([0-9a-fA-F][0-9a-fA-F])/\%$1/sg;
    return
          '<div class="url-info"><strong><a href="'
        . $this->escape( ($redirect) ? $this->redirect($url) : $url )
        . '" target="_blank">'
        . $this->jfold( $this->escape_except_html_entity($title),
        40, 'r', 0, '<wbr />' )
        . '</a></strong>'
        . ( ( $title ne $url )
        ? '<div class="url">'
            . $this->jfold( $this->escape($url), 40, 'r', 0, '<wbr />' )
            . '</div>'
        : '' );
}

sub redirect ($$) {
    my ( $this, $url ) = @_;
    return $url if !$setting::general{'redirect'};
    my $encoded_url  = $this->url_encode($url);
    my $redirect_url = $setting::general{'redirect'};
    $encoded_url =~ s/%2Fsearch%3B%3F/%2Fsearch%3F/ig;
    return ( $redirect_url =~ s/\%u/$encoded_url/g ) ? $redirect_url : $url;
}

sub domain_info ($$;$) {
    my ( $this, $host, $marker ) = @_;
    $host = $1 if $host =~ /^(.*?):/;
    my $text
        = ($marker) ? $marker : $this->jfold( $host, 30, 'r', 0, '<wbr />' );
    if ( $host =~ /\.([a-zA-Z]+)$/ ) {
        my $domain = 'http://www.' . $this->host_to_domain($host) . '/';
        $text
            = '<a href="'
            . $this->redirect($domain)
            . '" target="_blank" title="ドメインを元にウェブサイトを確認する(存在しない場合もあります)">'
            . $text . '</a>';
    }
    elsif ($marker) {
        $text = '';
    }
    if ( $setting::general{'whois'} && $setting::general{'whois'} =~ /\%s/ ) {
        my $url = $setting::general{'whois'};
        my $domain
            = ( $setting::stat{'client'}{'option'} )
            ? $host
            : $this->host_to_domain($host);
        $url =~ s/\%s/$domain/g;
        $text
            .= ' <a href="'
            . $this->escape( $this->redirect($url) )
            . '" target="_blank" title="ドメインについて WHOIS で調べる">[?]</a>';
    }
    return $text;
}

sub host_to_domain ($$) {
    my ( $this, $host ) = @_;
    return (   $host =~ /([%0-9a-zA-Z_\-]+\.[a-zA-Z]{3,})$/
            || $host =~ /([%0-9a-zA-Z_\-]+\.[a-zA-Z]{2,3}\.[a-zA-Z]{2})$/
            || $host =~ /([%0-9a-zA-Z_\-]+\.[a-zA-Z]{2})$/ ) ? $1 : $host;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #設定DB読み込み設定
    $setting::general  = 1;    #一般設定読み込み
    $setting::deco     = 1;    #テキスト装飾設定読み込み
    $setting::security = 1;    #アクセス権設定読み込み
    $setting::table    = 1;    #テーブル設定読み込み
    $setting::page     = 1;    #ページ設定読み込み

    #設定種キー設定
    $setting::security_key = 'stat';  #デフォルトセキュリティキー
    $setting::key          = 'stat';  #ページ設定キー

    #クッキー
    $cookie::id_enabled = 1
        if !$cookie::id_enabled;      #ID情報クッキー読み込み

    #必須フォーム
    @setting::form = ( 'k', 'm' );

}

1;

