package AgentReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# 訪問者の閲覧環境モジュール
# AgentReport.pm ver1.0.4
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2013-07-04
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	利用可能なスクリーン幅
# 	$$data[9]	利用可能なスクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;
    return 0
        if !defined $$data[3] || $$data[3] eq '' || $$data[3] =~ /[<>\@]/;
    my $key
        = ( ( $$data[1] =~ /proxy[0-9a-zA-Z_]+\.docomo\.ne\.jp/ )
        ? ''
        : $$data[1] )
        . $$data[3];
    if ( !$this->{'is_counted'}{$key} ) {
        my ($platform,    $platform_ver, $browser,
            $browser_ver, $limitsize,    $device
        ) = $cgi::UserAgent->get_user_agent_info( $$data[3] );

        # プラットフォーム集計
        if ( defined $platform && $platform ) {
            $platform = '携帯電話・タブレット'
                if $platform eq '携帯電話'
                || $platform_ver =~ /^Android|iPhone|iPad|iPod$/;
            $this->{'user_platform'}->{$platform}++;
            $this->{'user_platform_ver'}->{$platform}->{$platform_ver}++
                if defined $platform_ver && $platform_ver;
        }
        else {
            return 0;
        }

        # ブラウザ集計
        if ($browser) {
            $this->{'user_browser'}->{$platform}->{$browser}++;
            $this->{'user_browser_ver'}->{$platform}->{$browser}
                ->{$browser_ver}++
                if defined $browser_ver && $browser_ver;
        }

        # 機種集計
        $this->{'user_device'}->{$platform_ver}->{$device}++
            if $platform eq '携帯電話・タブレット' && $device;
        $this->{'is_counted'}{$key} = 1;
        $this->{'user_count'}++;
    }
    return 1;
}

sub is_skippable ($$$) {
    my ( $this, $data ) = @_;
    return 0;
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return '訪問者の閲覧環境';
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    undef %{ $this->{'is_counted'} };
    delete $this->{'is_counted'};
    return if defined $this->{'noLog'} && $this->{'noLog'};

    # プラットフォームランキング
    if ( ref $this->{'user_platform'} ) {
        my @ranking = sort {
            $this->sort_by( \$this->{'user_platform'}->{$a},
                \$this->{'user_platform'}->{$b}, 'descend' )
                || $a cmp $b
        } keys %{ $this->{'user_platform'} };
        $this->print_stat_report(
            'プラットフォームランキング',
            \@ranking,
            $this->{'user_platform'},
            $this->{'user_platform_ver'},
            '',
            $this->{'user_count'},
            '',
            1,
            'プラットフォーム',
            'center',
            200,
            'ユーザー数',
            100,
            '%'
        ) if @ranking;
    }

    # ブラウザランキング（Windows）
    if (   ref $this->{'user_browser'}
        && ref $this->{'user_browser'}->{'Windows'} )
    {
        my @ranking = sort {
            $this->sort_by( \$this->{'user_browser'}->{'Windows'}->{$a},
                \$this->{'user_browser'}->{'Windows'}->{$b}, 'descend' )
                || $a cmp $b
        } keys %{ $this->{'user_browser'}->{'Windows'} };
        $this->print_stat_report(
            'ブラウザランキング（Windows）',
            \@ranking,
            $this->{'user_browser'}->{'Windows'},
            $this->{'user_browser_ver'}->{'Windows'},
            '',
            $this->{'user_count'},
            '',
            1,
            'ブラウザ',
            'center',
            200,
            'ユーザー数',
            100,
            '%'
        ) if @ranking;
    }

    # ブラウザランキング（Macintosh）
    if (   ref $this->{'user_browser'}
        && ref $this->{'user_browser'}->{'Macintosh'} )
    {
        my @ranking = sort {
            $this->sort_by( \$this->{'user_browser'}->{'Macintosh'}->{$a},
                \$this->{'user_browser'}->{'Macintosh'}->{$b}, 'descend' )
                || $a cmp $b
        } keys %{ $this->{'user_browser'}->{'Macintosh'} };
        $this->print_stat_report(
            'ブラウザランキング（Macintosh）',
            \@ranking,
            $this->{'user_browser'}->{'Macintosh'},
            $this->{'user_browser_ver'}->{'Macintosh'},
            '',
            $this->{'user_count'},
            '',
            1,
            'ブラウザ',
            'center',
            200,
            'ユーザー数',
            100,
            '%'
        ) if @ranking;
    }

    # ブラウザランキング（UNIX）
    if (   ref $this->{'user_browser'}
        && ref $this->{'user_browser'}->{'UNIX'} )
    {
        my @ranking = sort {
            $this->sort_by( \$this->{'user_browser'}->{'UNIX'}->{$a},
                \$this->{'user_browser'}->{'UNIX'}->{$b}, 'descend' )
                || $a cmp $b
        } keys %{ $this->{'user_browser'}->{'UNIX'} };
        $this->print_stat_report(
            'ブラウザランキング（UNIX）',
            \@ranking,
            $this->{'user_browser'}->{'UNIX'},
            $this->{'user_browser_ver'}->{'UNIX'},
            '',
            $this->{'user_count'},
            '',
            1,
            'ブラウザ',
            'center',
            200,
            'ユーザー数',
            100,
            '%'
        ) if @ranking;
    }

    # 携帯機種ランキング
    if (   ref $this->{'user_platform_ver'}
        && ref $this->{'user_platform_ver'}
        ->{'携帯電話・タブレット'} )
    {
        my @ranking = sort {
            $this->sort_by(
                \$this->{'user_platform_ver'}
                    ->{'携帯電話・タブレット'}->{$a},
                \$this->{'user_platform_ver'}
                    ->{'携帯電話・タブレット'}->{$b},
                'descend'
                )
                || $a cmp $b
            } keys %{ $this->{'user_platform_ver'}
                ->{'携帯電話・タブレット'} };
        $this->print_stat_report(
            '携帯電話・タブレット 機種ランキング',
            \@ranking,
            $this->{'user_platform_ver'}->{'携帯電話・タブレット'},
            $this->{'user_device'},
            '',
            $this->{'user_count'},
            '',
            1,
            '機種',
            'center',
            200,
            'ユーザー数',
            100,
            '%'
        ) if @ranking;
    }

    # 携帯ブラウザランキング
    if (   ref $this->{'user_browser'}
        && ref $this->{'user_browser'}->{'携帯電話・タブレット'} )
    {
        my @ranking = sort {
            $this->sort_by(
                \$this->{'user_browser'}->{'携帯電話・タブレット'}
                    ->{$a},
                \$this->{'user_browser'}->{'携帯電話・タブレット'}
                    ->{$b},
                'descend'
                )
                || $a cmp $b
            } keys
            %{ $this->{'user_browser'}->{'携帯電話・タブレット'} };
        $this->print_stat_report(
            'ブラウザランキング（携帯電話・タブレット）',
            \@ranking,
            $this->{'user_browser'}->{'携帯電話・タブレット'},
            $this->{'user_browser_ver'}->{'携帯電話・タブレット'},
            '',
            $this->{'user_count'},
            '',
            1,
            'ブラウザ',
            'center',
            200,
            'ユーザー数',
            100,
            '%'
        ) if @ranking;
    }

    # その他デバイスブラウザランキング
    if (   ref $this->{'user_browser'}
        && ref $this->{'user_browser'}->{'その他'} )
    {
        my @ranking = sort {
            $this->sort_by( \$this->{'user_browser'}->{'その他'}->{$a},
                \$this->{'user_browser'}->{'その他'}->{$b}, 'descend' )
                || $a cmp $b
        } keys %{ $this->{'user_browser'}->{'その他'} };
        $this->print_stat_report(
            'ブラウザランキング（その他）',
            \@ranking,
            $this->{'user_browser'}->{'その他'},
            $this->{'user_browser_ver'}->{'その他'},
            '',
            $this->{'user_count'},
            '',
            1,
            'ブラウザ',
            'center',
            200,
            'ユーザー数',
            100,
            '%'
        ) if @ranking;
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'agent' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'agent', 'AgentReport' );

    unless ( ref $cgi::UserAgent ) {
        require CMONOS::CGI_Lib::UserAgent;
        $cgi::UserAgent = UserAgent->new;
    }
}

1;

