package ClientReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# アクセス元ランキングモジュール
# ClientReport.pm ver1.0.5
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-12-01
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	利用可能なスクリーン幅
# 	$$data[9]	利用可能なスクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;
    my $host = ( defined $$data[1] ) ? $$data[1] : '';

    # ホスト検索
    return 0
        if @setting::target_client
        && !grep( eval { $host =~ /$_/ }, @setting::target_client );

    # ドメイン名で集計
    if ( defined $setting::stat{'client'}{'option'}
        && $setting::stat{'client'}{'option'} )
    {
        if ( $host =~ /[^0-9\.]/ ) {
            $host = $this->host_to_domain($host);
        }
        else {
            $host = 'ドメイン名不明';
        }
    }
    $host .= ':' . $$data[3] if $$data[3];

    # リンク元で検索
    if (   @setting::target_client_referer
        && $$data[5]
        && !$this->{'referer_is_target'}->{$host}
        && (   !$setting::stat{'client'}{'refererLanding'}
            || !$this->{'remote_host'}->{$host} )
        )
    {
        my $referer = $this->url_filter( $$data[5] );
        $this->{'referer_is_target'}->{$host} = 1
            if
            grep( ( eval { $$data[5] =~ /$_/ } || eval { $referer =~ /$_/ } ),
            @setting::target_client_referer );
    }

    # アクセスファイル検索
    if (   @setting::target_client_url
        && $$data[2]
        && !$this->{'url_is_target'}->{$host} )
    {
        $this->{'url_is_target'}->{$host} = 1
            if
            grep( eval { $$data[2] =~ /$_/ }, @setting::target_client_url );
    }

    $this->{'remote_host'}->{$host}++;
    $this->{'client_in'}->{$host} = $$data[0]
        if !defined $this->{'client_in'}->{$host};
    $this->{'client_out'}->{$host} = $$data[0];
    return 1;
}

######################################################################
# 解析環境（追加）
sub print_sub_stat_env ($) {
    my $this          = shift;
    my $client_target = $setting::stat{'client'}{'target'};
    $client_target =~ s/\\(?!\\)//g
        if !defined $setting::stat{'client'}{'regexp'}
        || !$setting::stat{'client'}{'regexp'};
    my $client_referer = $setting::stat{'client'}{'referer'};
    $client_referer =~ s/\\(?!\\)//g
        if !defined $setting::stat{'client'}{'refererRegexp'}
        || !$setting::stat{'client'}{'refererRegexp'};
    my $client_url = $setting::stat{'client'}{'url'};
    $client_url =~ s/\\(?!\\)//g
        if !defined $setting::stat{'client'}{'urlRegexp'}
        || !$setting::stat{'client'}{'urlRegexp'};

    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('集計方法');
    print $this->{'infoTable'}->td(
        '<div id="hostsearch"><label for="setting:client_target" class="datatitle">ホスト名検索</label>'
            . '<input type="text" name="setting:client_target" size="30" value="'
            . $this->escape($client_target) . '">'
            . '<select name="setting:client_regexp">'
            . '<option value="">部分一致</option>'
            . '<option value="2"'
            . (
            (          $setting::stat{'client'}{'regexp'}
                    && $setting::stat{'client'}{'regexp'} == 2
            ) ? ' selected="selected"' : ''
            )
            . '>完全一致</option>'
            . '<option value="1"'
            . (
            (          $setting::stat{'client'}{'regexp'}
                    && $setting::stat{'client'}{'regexp'} == 1
            ) ? ' selected="selected"' : ''
            )
            . '>正規表現使用</option>'
            . '</select>'
            . '<select name="setting:client_option">'
            . '<option value="">ホスト名で集計する</option>'
            . '<option value="1"'
            . (
            ( $setting::stat{'client'}{'option'} )
            ? ' selected="selected"'
            : ''
            )
            . '>ドメイン名で集計する</option>'
            . '</select></div>'

            . '<div id="referersearch"><label for="setting:client_referer" class="datatitle">リンク元検索</label>'
            . '<input type="text" name="setting:client_referer" size="30" value="'
            . $this->escape($client_referer) . '">'
            . '<select name="setting:client_refererRegexp">'
            . '<option value="">部分一致</option>'
            . '<option value="2"'
            . (
            (          $setting::stat{'client'}{'refererRegexp'}
                    && $setting::stat{'client'}{'refererRegexp'} == 2
            ) ? ' selected="selected"' : ''
            )
            . '>完全一致</option>'
            . '<option value="1"'
            . (
            (          $setting::stat{'client'}{'refererRegexp'}
                    && $setting::stat{'client'}{'refererRegexp'} == 1
            ) ? ' selected="selected"' : ''
            )
            . '>正規表現使用</option>'
            . '</select>'
            . '<select name="setting:client_refererLanding">'
            . '<option value="">再訪問時のリンク元を含む</option>'
            . '<option value="1"'
            . (
            ( $setting::stat{'client'}{'refererLanding'} )
            ? ' selected="selected"'
            : ''
            )
            . '>到着時のリンク元のみ</option>'
            . '</select>'

            . '<div id="urlsearch"><label for="setting:client_url" class="datatitle">アクセスURL検索</label>'
            . '<input type="text" name="setting:client_url" size="30" value="'
            . $this->escape($client_url) . '">'
            . '<select name="setting:client_urlRegexp">'
            . '<option value="">部分一致</option>'
            . '<option value="2"'
            . (
            (          $setting::stat{'client'}{'urlRegexp'}
                    && $setting::stat{'client'}{'urlRegexp'} == 2
            ) ? ' selected="selected"' : ''
            )
            . '>完全一致</option>'
            . '<option value="1"'
            . (
            (          $setting::stat{'client'}{'urlRegexp'}
                    && $setting::stat{'client'}{'urlRegexp'} == 1
            ) ? ' selected="selected"' : ''
            )
            . '>正規表現使用</option>'
            . '</select> '
            . $this->submit_button(
            $env::image_dir . '/stat/client.gif',
            20, 20, 'アクセス元ランキング',
            '', 'm', 'ClientReport', 'k', 'client'
            )
            . '</div>'
    );
    print $this->{'infoTable'}->_tr;
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('リセット');
    print $this->{'infoTable'}->td(
        '<input type="button" value="解析条件をリセット" onclick="cmonos.form.item(this.form).resetAll()" />'
    );
    print $this->{'infoTable'}->_tr;
}

######################################################################
# キー項目調整
sub stat_key ($$;$) {
    my ( $this, $key ) = @_;
    return '<strong>' . $this->domain_info($key) . '</strong>';
}

######################################################################
# 追加項目
sub stat_option ($$$) {
    my ( $this, $key, $value ) = @_;
    return
          $this->hhmmss_to_time( $this->{'client_in'}->{$key} ) . '-'
        . $this->hhmmss_to_time( $this->{'client_out'}->{$key} ) . ' '
        . $this->submit_button(
        $env::image_dir . '/stat/dog.gif', 14,
        14,                                '足跡を再現します。',
        '',                                'm',
        'TracerReport',                    'k',
        'tracer',                          'setting:tracer_target',
        $key
        );
}

sub hhmmss_to_time ($$) {
    my ( $this, $time ) = @_;
    return
          ( !defined $time )                   ? ''
        : ( $time =~ /^([0-9]{2})([0-9]{2})/ ) ? $1 . ':' . $2
        :                                        $time;
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return 'アクセス元'
        . ( ( $setting::stat{'referer'}{'option'} )
        ? 'ドメイン'
        : 'ホスト' )
        . 'ランキング';
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    return if defined $this->{'noLog'} && $this->{'noLog'};

    my (@ranking);
    $setting::stat{'client'}{'sort'} = 'descend'
        if !defined $setting::stat{'client'}{'sort'}
        || $setting::stat{'client'}{'sort'} eq '';
    @ranking = keys %{ $this->{'remote_host'} };
    @ranking = grep( $this->{'referer_is_target'}->{$_}, @ranking )
        if @setting::target_client_referer;
    @ranking = grep( $this->{'url_is_target'}->{$_}, @ranking )
        if @setting::target_client_url;
    @ranking = sort {
        $this->sort_by(
            \$this->{'remote_host'}->{$a},
            \$this->{'remote_host'}->{$b},
            $setting::stat{'client'}{'sort'}
            )
            || $this->sort_by( \$this->{'client_out'}->{$a},
            \$this->{'client_out'}->{$b}, 'ascend' )
            || $a cmp $b
    } @ranking;
    $this->print_stat_report(
        '',
        \@ranking,
        $this->{'remote_host'},
        '', '',
        (     ( !@ranking ) ? ''
            : ( $setting::stat{'client'}{'sort'} eq 'ascend' )
            ? $this->{'remote_host'}->{ $ranking[$#ranking] }
            : $this->{'remote_host'}->{ $ranking[0] }
        ),
        'client', 1,
        'アクセス元'
            . (
            ( $setting::stat{'client'}{'option'} ) ? 'ドメイン'
            : 'ホスト'
            ),
        'left', 250,
        'アクセス数',
        100, '件',
        (   ( $setting::stat{'client'}{'option'} ) ? ''
            : '滞在時間/追跡'
        ),
        ( ( $setting::stat{'client'}{'option'} ) ? '' : 120 )
    );

}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'client' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'client', 'ClientReport' );

}

1;

