package GeneralReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# アクセス概要レポートモジュール
# GeneralReport.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	利用可能なスクリーン幅
# 	$$data[9]	利用可能なスクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;
    my ( $wday, $hour );

    #時間別集計
    $hour = $1 + 0 if $$data[0] && $$data[0] =~ /^([0-9]{2})/;
    if ( defined $hour && $hour ne '' ) {
        $this->{'access_hour'}->{$hour}++;
        $this->{'count_hour'}++
            if !defined $this->{'date'} || $this->{'date'} ne $date;
    }

    #曜日別集計
    @{ $this->{'week'} }
        = ( '日', '月', '火', '水', '木', '金', '土' );
    $wday = $this->{'week'}->[ $this->get_wday( $1, $2, $3 ) ]
        if $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})/;
    if ( defined $wday && $wday ne '' ) {
        $this->{'access_wday'}->{$wday}++;
        $this->{'count_wday'}->{$wday}++
            if !defined $this->{'date'} || $this->{'date'} ne $date;
    }

    $this->{'date'} = $date;
    return (   ( !defined $hour || $hour eq '' )
            && ( !defined $wday || $wday eq '' ) ) ? 0 : 1;
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return 'アクセスの概要';
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    return if defined $this->{'noLog'} && $this->{'noLog'};
    my ( $k, $v );

    #時間別集計

    #平均値化
    while ( ( $k, $v ) = each %{ $this->{'access_hour'} } ) {
        $this->{'access_hour'}->{$k}
            = ( $this->{'count_hour'} )
            ? int( ( ( $v / $this->{'count_hour'} ) * 100 ) + 0.5 ) / 100
            : 0;
    }

    my (@hour);
    @hour = ( 0 .. 23 );
    $this->print_stat_report(
        '時間別集計',
        \@hour,
        $this->{'access_hour'},
        '', '',
        (   ( %{ $this->{'access_hour'} } )
            ? $this->{'access_hour'}
                ->{ $this->maximum( $this->{'access_hour'} ) }
            : ''
        ),
        '', 0, '時間', 'center', 50,
        '平均アクセス数',
        100, '件'
    );

    #曜日別集計

    #平均値化
    while ( ( $k, $v ) = each %{ $this->{'access_wday'} } ) {
        $this->{'access_wday'}->{$k}
            = ( $this->{'count_wday'}->{$k} )
            ? int( ( ( $v / $this->{'count_wday'}->{$k} ) * 100 ) + 0.5 )
            / 100
            : 0;
    }

    $this->print_stat_report(
        '曜日別集計',
        $this->{'week'},
        $this->{'access_wday'},
        '', '',
        (   ( %{ $this->{'access_wday'} } )
            ? $this->{'access_wday'}
                ->{ $this->maximum( $this->{'access_wday'} ) }
            : ''
        ),
        '', 0, '曜日', 'center', 50,
        '平均アクセス数',
        100, '件'
    );

}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'general' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'general', 'GeneralReport' );

}

1;

