package KeywordReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# 検索キーワードランキングモジュール
# KeywordReport.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-08-07
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	利用可能なスクリーン幅
# 	$$data[9]	利用可能なスクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;
    my ( $query, @name, @name2, @keywords );

    if ( defined $$data[5] && $$data[5] =~ /\?(.*)$/ ) {
        @keywords
            = $this->query_to_keywords( $1,
            $setting::stat{'keyword'}{'option'},
            $$data[5] );
    }
    else {
        return 0;
    }
    return 0 if !@keywords;

    if ( !$this->{'is_counted'}{ $$data[1] . $$data[3] } ) {
        if ( $setting::stat{'keyword'}{'option'} ) {
            return 0 if !@keywords;
            $this->{'search_keyword'}->{ $keywords[0] }++
                if defined $keywords[0]
                && $keywords[0] ne ''
                && $keywords[0] =~ /[^\x09\x0A\x0C\x0D\x20]/;
        }
        else {
            foreach my $k (@keywords) {
                $this->{'search_keyword'}->{$k}++
                    if defined $k
                    && $k ne ''
                    && $k =~ /[^\x09\x0A\x0C\x0D\x20]/;
            }
        }
        $this->{'is_counted'}{ $$data[1] . $$data[3] } = 1;
    }
    return 1;
}

######################################################################
# 解析環境（追加）
sub print_sub_stat_env ($) {
    my $this = shift;
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('集計方法');
    print $this->{'infoTable'}->td(
              '<select name="setting:keyword_option">'
            . '<option value="">単語ごとに集計する</option>'
            . '<option value="1"'
            . (
            ( $setting::stat{'keyword'}{'option'} )
            ? ' selected="selected"'
            : ''
            )
            . '>検索条件で集計する</option>'
            . '</select> '
            . $this->submit_button(
            $env::image_dir . '/stat/keyword.gif',
            20,
            20,
            '検索キーワードランキング',
            '',
            'm',
            'KeywordReport',
            'k',
            'keyword'
            )
    );
    print $this->{'infoTable'}->_tr;
}

######################################################################
# キー項目調整
sub stat_key ($$;$) {
    my ( $this, $key ) = @_;
    my $limit = ( $setting::stat{'keyword'}{'option'} ) ? 40 : 24;
    return '<strong>'
        . (
        ( bytes::length $key < $limit )
        ? $this->escape_except_html_entity($key)
        : $this->jfold(
            $this->escape_except_html_entity($key),
            $limit, 'r', 0, '<wbr />'
        )
        ) . '</strong>';
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return '検索キーワードランキング';
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    undef %{ $this->{'is_counted'} };
    delete $this->{'is_counted'};
    return if defined $this->{'noLog'} && $this->{'noLog'};

    my (@ranking);
    $setting::stat{'keyword'}{'sort'} = 'descend'
        if !defined $setting::stat{'keyword'}{'sort'}
        || $setting::stat{'keyword'}{'sort'} eq '';
    @ranking = sort {
        $this->sort_by(
            \$this->{'search_keyword'}->{$a},
            \$this->{'search_keyword'}->{$b},
            $setting::stat{'keyword'}{'sort'}
            )
            || $a cmp $b
    } keys %{ $this->{'search_keyword'} };

    $this->print_stat_report(
        '',
        \@ranking,
        $this->{'search_keyword'},
        '', '',
        (   (          !ref $this->{'search_keyword'}
                    || !%{ $this->{'search_keyword'} }
            ) ? 0
            : (        $setting::stat{'keyword'}{'sort'}
                    && $setting::stat{'keyword'}{'sort'} eq 'ascend' )
            ? $this->{'search_keyword'}->{ $ranking[$#ranking] }
            : $this->{'search_keyword'}->{ $ranking[0] }
        ),
        'keyword',
        1,
        '検索キーワード',
        'left',
        ( ( $setting::stat{'keyword'}{'option'} ) ? 300 : 180 ),
        'アクセス数',
        100, '件'
    );

}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'keyword' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'keyword', 'KeywordReport' );

}

1;

