package RefererReport;
require CMONOS::Accestat::Accestat;
@ISA = qw(Accestat);
use strict;
use utf8;
######################################################################
# リンク元ランキングモジュール
# RefererReport.pm ver1.0.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-01-30
######################################################################
# ログ解析
# 	$$data[0]	時間（hhmmss）
# 	$$data[1]	IP（REMOTE_HOST）
# 	$$data[2]	アクセスファイル
# 	$$data[3]	クライアント環境（HTTP_USER_AGENT）
# 	$$data[4]	クライアント主言語（HTTP_ACCEPT_LANGUAGE）
# 	$$data[5]	リンク元（HTTP_REFERER）
# 	$$data[6]	ウィンドウ内側の幅
# 	$$data[7]	ウィンドウ内側の高さ
# 	$$data[8]	利用可能なスクリーン幅
# 	$$data[9]	利用可能なスクリーン高さ
sub log_stat ($$$) {
    my ( $this, $date, $data ) = @_;
    my $referer = ( defined $$data[5] ) ? $$data[5] : '';

    # サイト内からのリンクを除外
    return 0
        if grep( eval { $referer =~ /\Q$_\E/ }, @default::target )
        || !$referer;

    # referer登録
    if ( $referer =~ m!^https?://!i ) {
        $referer = $this->url_filter($referer);
        my $mtime = substr( $date, 0, 8 ) . $$data[0];
        my $url = $this->rm_query($referer);
        $this->{'referer_mtime'}->{$url} = $mtime
            if !$this->{'referer_mtime'}->{$url}
            || $mtime < $this->{'referer_mtime'}->{$url};
    }

    # ランディングページのみカウント
    if ( $setting::stat{'referer'}{'landing'} ) {
        return 0 if $this->{'is_counted'}{ $$data[1] };
        $this->{'is_counted'}{ $$data[1] } = 1;
    }

    # CGIからのリンクを除外
    if ( $setting::stat{'referer'}{'option'} =~ /nocgi$/ ) {
        return 0 if $referer =~ /\?/ || $referer =~ /\.cgi$/i;
        $referer =~ s/#.*$//s;

        # CGIの条件部分を無視
    }
    elsif ( $setting::stat{'referer'}{'option'} =~ /ignore$/ ) {
        $referer = $this->rm_query($referer);

        # サイトで集計
    }
    elsif ( $setting::stat{'referer'}{'option'} eq 'site' ) {
        $referer = $1 if $referer =~ /^(http[s]?:\/\/[^\/]+)/is;
    }

    $this->{'http_referer'}->{$referer}++
        if !$this->{'referer_ip'}->{$referer}->{ $$data[1] };
    $this->{'referer_ip'}->{$referer}->{ $$data[1] }++;
    return 1;
}

sub rm_query ($$) {
    my ( $this, $url ) = @_;
    $url =~ s/[#\?].*$//s;
    $url =~ s/(\.(cgi|php|pl|asp))(?=[^0-9a-zA-Z_]).*$/$1/s;
    $url =~ s/(\.[s]?htm[l]?)(?=[^0-9a-zA-Z_]).*?$/$1/s;
    return $url;
}

######################################################################
# 解析環境（追加）
sub print_sub_stat_env ($) {
    my $this = shift;
    print $this->{'infoTable'}->tr;
    print $this->{'infoTable'}->th('集計方法');
    print $this->{'infoTable'}->td(
              '<select name="setting:referer_option">'
            . '<option value="">URL単位で集計する</option>'
            . '<option value="ignore"'
            . (
            ( $setting::stat{'referer'}{'option'} eq 'ignore' )
            ? ' selected="selected"'
            : ''
            )
            . '>CGIの条件部分とアンカーを無視する</option>'
            . '<option value="time_ignore"'
            . (
            ( $setting::stat{'referer'}{'option'} eq 'time_ignore' )
            ? ' selected="selected"'
            : ''
            )
            . '>CGIの条件部分とアンカーを無視する（新着順）</option>'
            . '<option value="nocgi"'
            . (
            ( $setting::stat{'referer'}{'option'} eq 'nocgi' )
            ? ' selected="selected"'
            : ''
            )
            . '>CGIからのリンクとアンカーを無視する</option>'
            . '<option value="time_nocgi"'
            . (
            ( $setting::stat{'referer'}{'option'} eq 'time_nocgi' )
            ? ' selected="selected"'
            : ''
            )
            . '>CGIからのリンクとアンカーを無視する（新着順）</option>'
            . '<option value="site"'
            . (
            ( $setting::stat{'referer'}{'option'} eq 'site' )
            ? ' selected="selected"'
            : ''
            )
            . '>サイト単位で集計する</option>'
            . '</select>'
            . '<select name="setting:referer_landing">'
            . '<option value="">再訪問時のリンク元もカウントする</option>'
            . '<option value="1"'
            . (
            ( $setting::stat{'referer'}{'landing'} ) ? ' selected="selected"'
            : ''
            )
            . '>到着時のリンク元のみカウントする</option>'
            . '</select> '
            . $this->submit_button(
            $env::image_dir . '/stat/referer.gif',
            20, 20, 'リンク元ランキング', '', 'm', 'RefererReport',
            'k', 'referer'
            )
    );
    print $this->{'infoTable'}->_tr;
}

######################################################################
# キー項目調整
sub stat_key ($$) {
    my ( $this, $key ) = @_;
    return $this->url_to_title(
        $key,
        (   (          $setting::stat{'referer'}{'option'}
                    && $setting::stat{'referer'}{'option'} eq 'site'
            ) ? 1 : 0
        ),
        1
    );
}

######################################################################
# 追加項目
sub stat_option ($$$) {
    my ( $this, $key, $value ) = @_;
    return $this->submit_button(
        $env::image_dir . '/stat/visitor.gif',
        14,
        14,
        '訪問者リストを表示します。',
        '',
        'm',
        'ClientReport',
        'k',
        'client',
        'setting:client_referer',
        $key,
        'setting:client_target',
        '',
        'setting:client_option',
        '',
        'setting:client_refererRegexp',
        ( ( $setting::stat{'referer'}{'option'} ) ? '' : 2 ),
        'setting:client_refererLanding',
        ( ( $setting::stat{'referer'}{'landing'} ) ? 1 : 0 )
    );
}

######################################################################
# 解析タイトル
sub stat_title ($) {
    my $this = shift;
    return 'リンク元'
        . ( ( $setting::stat{'referer'}{'option'} eq 'site' ) ? 'サイト'
        : 'URL' )
        . ( ( $setting::stat{'referer'}{'option'} =~ /^time/ )
        ? '一覧（新着順）'
        : 'ランキング' );
}

######################################################################
# 解析結果表示
sub print_report ($) {
    my $this = shift;
    return if defined $this->{'noLog'} && $this->{'noLog'};
    $this->tie_title_db;

    # referer登録
    my (%referer);
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    tie(%referer, $env::dbm_file,
        $default::preference_dir . '/referer_log',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %referer ) {
        while ( my ( $url, $t ) = each %{ $this->{'referer_mtime'} } ) {
            my $ref_t = $referer{$url} || 0;
            if ( !$ref_t || $ref_t > $t ) {
                utf8::encode($url) if utf8::is_utf8($url);
                utf8::encode($t)   if utf8::is_utf8($t);
                $referer{$url} = $t;
            }
            else {
                $this->{'referer_mtime'}->{$url} = $ref_t;
            }
        }
    }
    untie %referer;

    my (@ranking);
    $setting::stat{'referer'}{'sort'} = 'descend'
        if !defined $setting::stat{'referer'}{'sort'}
        || $setting::stat{'referer'}{'sort'} eq '';

    # 新着順
    if ( $setting::stat{'referer'}{'option'} =~ /^time/ ) {
        @ranking = sort {
            $this->sort_by(
                \$this->{'referer_mtime'}->{$a},
                \$this->{'referer_mtime'}->{$b},
                $setting::stat{'referer'}{'sort'}
                )
                || $this->sort_by(
                \$this->{'http_referer'}->{$a},
                \$this->{'http_referer'}->{$b},
                $setting::stat{'referer'}{'sort'}
                )
                || $a cmp $b
        } keys %{ $this->{'http_referer'} };
        $this->print_stat_report(
            '',
            \@ranking,
            $this->{'referer_mtime'},
            '',
            $this->{'http_referer'},
            $this->{'http_referer'}
                ->{ $this->maximum( $this->{'http_referer'} ) },
            'referer',
            1,
            'リンク元URL',
            'left',
            300,
            '初アクセス',
            200,
            '件',
            '訪問者',
            50
        );

        # その他
    }
    else {
        @ranking = sort {
            $this->sort_by(
                \$this->{'http_referer'}->{$a},
                \$this->{'http_referer'}->{$b},
                $setting::stat{'referer'}{'sort'}
                )
                || $a cmp $b
        } keys %{ $this->{'http_referer'} };
        $this->print_stat_report(
            '',
            \@ranking,
            $this->{'http_referer'},
            '', '',
            (   (          !ref $this->{'http_referer'}
                        || !%{ $this->{'http_referer'} }
                ) ? 0
                : (        $setting::stat{'referer'}{'sort'}
                        && $setting::stat{'referer'}{'sort'} eq 'ascend' )
                ? $this->{'http_referer'}->{ $ranking[$#ranking] }
                : $this->{'http_referer'}->{ $ranking[0] }
            ),
            'referer',
            1,
            'リンク元'
                . (
                (          $setting::stat{'referer'}{'option'}
                        && $setting::stat{'referer'}{'option'} eq 'site'
                ) ? 'サイト' : 'URL'
                ),
            'left', 300,
            'アクセス数',
            100, '件',
            '訪問者',
            50
        );
    }
    $this->untie_title_db;
}

sub get_num ($$) {
    my ( $this, $num ) = @_;
    if ( $setting::stat{'referer'}{'option'} =~ /^time/ ) {
        return $this->date(
            $num,
            $setting::page{'stat'}{'datelimit'},
            $setting::page{'stat'}{'datetype'}
        );
    }
    else {
        return ($num) ? $num : 0;
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #デフォルトキー
    $main::k = 'referer' if !defined $main::k || !$main::k;

    #クラス登録
    Common::class_register( 'referer', 'RefererReport' );
}

1;

