package Stat;
require CMONOS::CGI_Lib::UrlDecode;
@ISA = qw(UrlDecode);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# アクセスログ記録モジュール
# Stat.pm ver1.0.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-01-18
######################################################################
# オブジェクトを作る。
sub new ($) {
    my $pkg  = shift;
    my $this = {};
    bless $this;
    $this->fix_env;
    $this;
}

######################################################################
# メイン
sub run ($) {
    my $this = shift;
    my ( $location, $date, $time, $access_log );

    # リンク先指定がある場合
    if ( defined $main::location && $main::location ne '' ) {
        $location = $main::location;
        $main::referer = ( $ENV{'HTTP_REFERER'} ) ? $ENV{'HTTP_REFERER'} : '';
    }
    elsif ( defined $main::url && $main::url ne '' ) {
        $location = $main::url;

        # その他
    }
    elsif ( defined $ENV{'HTTP_REFERER'}
        && $ENV{'HTTP_REFERER'} !~ /\/stat\.cgi/ )
    {
        $location = $ENV{'HTTP_REFERER'};
    }
    else {
        $location = '';
    }
    $main::referer = '' if !$main::referer || $main::referer =~ m!^file://!i;

# パスワードの埋め込まれたURL→安全のため、パスワード部分を削除
    $location =~ s!^(https?://)[0-9a-zA-Z_:]+\@!$1!si;
    $main::referer =~ s!^(https?://)[0-9a-zA-Z_:]+\@!$1!si;

    $location =~ s!/index\.s?htm[l]?$!/!si;
    $main::referer =~ s!/index\.s?htm[l]?$!/!si;

    $location =~ s/%7e/~/sgi;
    $main::referer =~ s/%7e/~/sgi;
    $location =~ s/ /\+/g;
    $main::referer =~ s/ /\+/g;

    # ドメインを小文字に統一
    $location =~ s!^(https?://[^/]+)!lc($1)!ie;
    $main::referer =~ s!^(https?://[^/]+)!lc($1)!ie;

    # スラッシュの補完
    $location      .= '/' if $location =~ m!^https?://[^/]+$!;
    $main::referer .= '/' if $main::referer =~ m!^https?://[^/]+$!;

    # ホスト名へ変換
    $location =~ s!^(https?://)([[0-9]+\.]+)/!$1 . $this->to_host($2) . '/'!e;
    $main::referer
        =~ s!^(https?://)([[0-9]+\.]+)/!$1 . $this->to_host($2) . '/'!e;

    # ターゲットサイト以外の場合/管理人等記録対象外者
    if ( !@default::target ) {
        my $home_location = $this->home_location;
        if ( defined $env::doublename && $env::doublename ) {
            $home_location =~ s!^https?://www\.!!si;
            $location =~ s!^(https?://)www\.$home_location!$1$home_location!si
                if defined $env::doublename && $env::doublename;
        }
        push( @default::target, $home_location );
    }
    ( defined $main::location && $main::location ne '' )
        ? $this->location($main::location)
        : $this->invisible_gif
        if grep( eval { $location =~ /\Q$_\E/ }, @default::target ) == 0
        || $this->read_cookie_direct( 'NOCHECK' . $default::id ) eq
        $default::nocheck_pswd;

    # アクセス日時
    ( $date, $time ) = $this->date_id;

    # ログファイル入れ替え
    $access_log = $default::access_log . '/' . $date . '.csv';
    $this->log_sweeper unless -f $access_log;

    # アクセスログ書き込み
    #if (open(LOG, ">>$access_log")) {
    if ( $this->filelock( $access_log, 2 ) && open( LOG, ">>$access_log" ) ) {
        binmode LOG, ":utf8";
        flock( LOG, LOCK_EX );

        print LOG $this->csv(
            1,
            $time,
            ( $ENV{'REMOTE_HOST'} || $this->to_host( $ENV{'REMOTE_ADDR'} ) ),
            $location,
            $ENV{'HTTP_USER_AGENT'},
            $ENV{'HTTP_ACCEPT_LANGUAGE'},
            $main::referer,
            $main::innerWidth,
            $main::innerHeight,
            $main::screenWidth,
            $main::screenHeight
        );
        print LOG "\x0D\x0A";

        close(LOG);
        $this->filelock( $access_log, 8 );
    }

    # 透過GIF/ジャンプ
    ( defined $main::location && $main::location ne '' )
        ? $this->location($main::location)
        : $this->invisible_gif;
}

#ファイルロック
sub filelock ($$$) {
    my ( $this, $file, $code ) = @_;
    my ( $lockdir, $i, $lock );
    $lockdir = $file . '.lock';

    $SIG{'TERM'} = $SIG{'PIPE'} = $SIG{'HUP'} = \&unlock;

    if ( $code == 2 ) {
        for ( $i = 0; $i <= 10; $i++ ) {
            if ( mkdir( $lockdir, $env::dir_mode ) ) {
                $locked::file{$lockdir} = 1;
                return 1;
            }
            else {
                my @stat = stat($lockdir);
                if ( $stat[9] && time - $stat[9] > 5 ) {
                    rmdir($lockdir);
                    delete $locked::file{$lockdir};
                    next;
                }
                sleep(1);
            }
        }
        return 0;

    }
    elsif ( $code == 8 ) {
        delete $locked::file{$lockdir};
        return rmdir($lockdir);
    }
}

sub unlock {
    my ($lockdir);
    while ( ($lockdir) = each %locked::file ) {
        rmdir($lockdir);
    }
}

######################################################################
# アクセスログ順送り
sub log_sweeper ($) {
    my $this = shift;
    my (@logfiles, $size,   %setting, $cginame, $mastername,
        $master,   $mailto, $limit,   $oldest
    );

    # アクセスログディレクトリのサイズを求める
    opendir( LOG, $default::access_log );
    @logfiles
        = grep( $_ =~ /[0-9]{8}\.csv$/
            && ( $size += -s $default::access_log . '/' . $_ ),
        readdir(LOG) );
    closedir(LOG);

    #ファイルが存在しないときは何もしない
    return if !@logfiles;

    # 設定を読み込む
    eval 'use ' . $env::dbm_file . ';';
    tie( %setting, $env::dbm_file, $default::preference_dir . '/general',
        O_RDONLY, $env::dbm_mode );
    $cginame    = $this->decode_utf8( $setting{'cginame'} );
    $mastername = $this->decode_utf8( $setting{'mastername'} );
    $master     = $this->decode_utf8( $setting{'master'} );
    $mailto     = $this->decode_utf8( $setting{'mailto'} );
    $limit      = $this->decode_utf8( $setting{'limit'} );
    untie %setting;

    $limit = $default::limit if !defined $limit || !$limit;
    $limit = 100000          if !defined $limit || !$limit;

    # 容量が最大容量以内の時は何もしない
    $size += $size / @logfiles;
    return if $size < $limit;

    # 一番古いログ
    @logfiles = sort @logfiles;
    $oldest   = $default::access_log . '/' . $logfiles[0];

    # ログ送付先が指定されているときはメールを送る
    if ( defined $mailto && $mailto ne '' ) {
        my ($msg);
        $msg = '【報告】' . $cginame . "\x0A";
        $msg .= "\x0A";
        $msg
            .= 'アクセスログ容量が制限値に近づいてきたので、'
            . "\x0A";
        $msg .= '古いアクセスログを削除しました。' . "\x0A";

        $msg .= "\x0A";
        $msg
            .= '※添付ファイルは削除されたアクセスログです。'
            . "\x0A";
        $msg .= 'ファイル名：日付（yyyymmdd）' . "\x0A";
        $msg
            .= '形　　　式：csv形式（カンマ区切りテキスト）'
            . "\x0A";
        $msg .= "\x0A";
        $msg .= '時間（hhmmss）,' . "\x0A";
        $msg .= 'IP（REMOTE_HOST）,' . "\x0A";
        $msg .= 'アクセスファイル,' . "\x0A";
        $msg .= 'クライアント環境（HTTP_USER_AGENT）,' . "\x0A";
        $msg .= 'クライアント主言語（HTTP_ACCEPT_LANGUAGE）,'
            . "\x0A";
        $msg .= 'リンク元（HTTP_REFERER）,' . "\x0A";
        $msg .= 'ウィンドウ内側の幅,' . "\x0A";
        $msg .= 'ウィンドウ内側の高さ,' . "\x0A";
        $msg .= 'スクリーン幅,' . "\x0A";
        $msg .= 'スクリーン高さ' . "\x0A";
        $msg .= "\x0A";

        unless ( ref $cgi::SendMail ) {
            require CMONOS::CGI_Lib::SendMail;
            $cgi::SendMail
                = SendMail->new( $env::sendmail, $env::master_email, '',
                $cgi::input, 1 );
        }

        $cgi::SendMail->send_mail(
            'Bcc'  => $mailto,
            'From' => (
                ($mastername) ? $mastername . ' <' . $master . '>' : $master
            ),
            'Subject' => '古いアクセスログを削除しました。',
            'message' => $msg,
            'file'    => $oldest,
            'info'    => 0
        );
    }

    unlink $oldest;
}

1;
