package NoticeMsg;
require CMONOS::CGI_Lib::Tools::Preference;
@ISA = qw(Preference);
use strict;
use utf8;
######################################################################
# 投稿通知設定モジュール
# NoticeMsg.pm ver1.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 設定環境
sub set_preferences ($) {
    my $this = shift;
    $preference::db   = 'mail';               # 記録用データベース名
    $preference::name = '投稿通知設定'; # 設定パネル名
    $preference::title
        = '新規投稿を管理人やスレッド参加者に通知するメールの内容を設定をします。';
}

######################################################################
# 設定パネル
sub print_preference_panel ($) {
    my $this = shift;

    #フィールド設定読み込み
    $this->use_field( 'field', $default::field_pref_dir . '/inputfield', 2 );

    my ( $mail_field_flag, $name_field, $setting_flag, $titlefield_name );
    $mail_field_flag = 1
        if $field::for{'mail'}
        && ref $field::obj{ $field::for{'mail'} }
        && $field::obj{ $field::for{'mail'} }->{'e-mail'};
    $name_field
        = ( ( $field::for{'name'} && ref $field::obj{ $field::for{'name'} } )
        ? '（' . $field::obj{ $field::for{'name'} }->name . '）'
        : '' );
    $titlefield_name
        = (
        ( $field::for{'title'} && ref $field::obj{ $field::for{'title'} } )
        ? '（' . $field::obj{ $field::for{'title'} }->name . '）'
        : '' );

    print
        '<fieldset><legend>新規投稿通知メール設定 <span class="note">(新規投稿があったとき管理人・共同オーナー・スタッフに自動的に送信されます)</span></legend>';
    if ( defined $setting::general{'newmail'}
        && $setting::general{'newmail'} )
    {
        print '<table class="field-table"><tbody>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:newsubject">メールの表題</label></td>';
        print
            '<td class="field"><input type="text" name="data:newsubject" id="data:newsubject" size="72" value="'
            . $this->escape( $main::data{'newsubject'} )
            . '" /></td>';
        print '</tr>';

        print '<tr>';
        print
            '<td class="field-name"><label for="data:newmsg">通知内容</label></td>';
        print
            '<td class="field"><textarea name="data:newmsg" id="data:newmsg" cols="72" rows="15" wrap="soft">'
            . $this->escape( $main::data{'newmsg'} )
            . '</textarea>';
        print '<div class="toolbox">';
        print
            '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:newmsg\').quotationByMenu(this)" title="これらのタグは通知メール送信時に対応する値に変換されます。">';
        print '<option value="">▼挿入する</option>';
        print
            '<option value="&lt;name&gt;">&lt;name&gt;[投稿者名フィールド'
            . $name_field
            . 'の値]</option>';
        print
            '<option value="&lt;title&gt;">&lt;title&gt;[タイトルフィールド'
            . $titlefield_name
            . 'の値]</option>';
        print
            '<option value="&lt;url&gt;">&lt;url&gt;[新しい記事のURL]</option>';
        print
            '<option value="&lt;data&gt;">&lt;data&gt;[データの一覧]</option>'
            if $setting::general{'newmail'} eq 'data';
        print '</select>';
        print '</div>';
        print '</td>';
        print '</tr>';

        print '</tbody></table>';

        $setting_flag = 1;
    }
    else {

        print
            '新規投稿通知機能停止中（「一般設定」で変更できます。）';
    }
    print '</fieldset>';

    print
        '<fieldset><legend>返信通知メール設定 <span class="note">(レスがあったときスレッド参加者の '
        . $field::obj{ $field::for{'mail'} }->name
        . ' に送信されます)</span></legend>';
    if ( !defined $mail_field_flag || !$mail_field_flag ) {
        print
            '代表メールアドレスフィールドが存在しないので、返信通知機能は利用できません。<br />入力フィールド設定で代表メールアドレスフィールドを指定してください。';

    }
    else {
        if ( defined $setting::general{'resmail'}
            && $setting::general{'resmail'} )
        {
            print '<table class="field-table"><tbody>';
            print '<tr>';
            print
                '<td class="field-name"><label for="data:resdelivery">通知の範囲</label></td>';
            print
                '<td class="field"><select name="data:resdelivery" id="data:resdelivery">';
            print '<option value="upper"'
                . (
                (   defined $main::data{'resdelivery'}
                        && $main::data{'resdelivery'} eq 'upper'
                ) ? ' selected="selected"' : ''
                ) . '>上流記事の投稿者にのみ通知</option>';
            print '<option value="thread"'
                . (
                (   defined $main::data{'resdelivery'}
                        && $main::data{'resdelivery'} eq 'thread'
                ) ? ' selected="selected"' : ''
                ) . '>関連記事の投稿者全員に通知</option>';
            print '</select>';
            print '</td>';
            print '</tr>';

            print '<tr>';
            print
                '<td class="field-name"><label for="data:ressubject">メールの表題</label></td>';
            print
                '<td class="field"><input type="text" name="data:ressubject" id="data:ressubject" size="72" value="'
                . $this->escape( $main::data{'ressubject'} )
                . '" /></td>';
            print '</tr>';

            print '<tr>';
            print
                '<td class="field-name"><label for="data:resmsg">返信通知メールの本文</label></td>';
            print
                '<td class="field"><textarea name="data:resmsg" id="data:resmsg" cols="72" rows="15" wrap="soft">'
                . $this->escape( $main::data{'resmsg'} )
                . '</textarea>';
            print '<div class="toolbox">';
            print
                '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:resmsg\').quotationByMenu(this)" title="これらのタグは通知メール送信時に対応する値に変換されます。">';
            print '<option value="">▼挿入する</option>';
            print
                '<option value="&lt;name&gt;">&lt;name&gt;[投稿者名フィールド'
                . $name_field
                . 'の値]</option>';
            print
                '<option value="&lt;title&gt;">&lt;title&gt;[タイトルフィールド'
                . $titlefield_name
                . 'の値]</option>';
            print
                '<option value="&lt;url&gt;">&lt;url&gt;[新しい記事のURL]</option>';
            print
                '<option value="&lt;data&gt;">&lt;data&gt;[データの一覧]</option>'
                if $setting::general{'newmail'} eq 'data';
            print '</select>';
            print '</div>';
            print '</td>';
            print '</tr>';

            print '</tbody></table>';
            $setting_flag = 1;
        }
        else {

            print
                '返信通知機能停止中（「一般設定」で変更できます。）';
        }
    }
    print '</fieldset>';
    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>'
        if defined $setting_flag && $setting_flag;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {

        #クラス登録
        Common::class_register( 'b200', 'NoticeMsg' );

        #ツール登録
        $setting::jump_menu{'b200'} = '投稿通知設定';
    }
}

1;
