package Deco;
require CMONOS::CGI_Lib::Common;
@ISA = qw(Common);
use strict;
use utf8;
######################################################################
# 装飾モジュール
# Deco.pm 1.1.8
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-02-27
######################################################################
#
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	(0) オブジェクトを作る。
#		require CMONOS::CGI_Lib::Deco;
#		obj = Deco->new(image_dir,image_lib,output,[\x09\x0A\x0C\x0D\x20]etting,deco_pref);
# 			image_dir			画像ディレクトリ(デフォルトは、$env::image_dir)
# 			imageLib			画像素材ディレクトリ(デフォルトは、$env::image_lib)
# 			output				書き出し先ファイルのパス
# 			setting				修飾設定ハッシュのリファレンス(デフォルトは、%setting::deco)
# 			deco_pref			修飾設定DB(デフォルトは、$env::preference_dir . '/deco')
#
#	(0)-2 obj->set_output(output);
# 			output				書き出し先ファイルのパス
#
#	(0)-3 obj->restore_output;
# 								書き出し先ファイルのパスを一つ前に戻す
#
# ---------------------------------------------------------------------------------------
#	(1)-1 form = obj->background_menu(name,dir[,maxwidth,default,add,list,lib_dir]);
#		背景画選択フォームを返します。
# 			name				シンボリックネーム
# 			maxwidth			画像幅制限
# 			default				現在の画像番号
# 			add					1=新規作成ボタンの追加
# 			list				1=空選択肢を用意しない
# 			lib_dir				1=画像素材ディレクトリのメニュー
#
#	(1)-2 file_name = obj->background_file_name(dir,image,lib_dir);
#		背景画像情報を返します。
# 			dir					画像ディレクトリ
# 			image				画像番号
# 			file_name			画像ファイル名
#
# ---------------------------------------------------------------------------------------
#	(2)-1 form = obj->image_menu(name,dir[,maxwidth,default,add,list,lib_dir]);
#		画像選択フォームを返します。
#
#	(2)-2 (width,height,name) = obj->image_info(dir,image,lib_dir);
#		画像情報を返します。
#
#	(2)-3 path = obj->image_path(dir,image,lib_dir);
#		画像の相対パスを返します。
#
# ---------------------------------------------------------------------------------------
#	(3)-1 obj->button(dir,image,href,alt,target,option,abs,lib_dir);
#		ボタンタグを返します。
# 			href				リンク先URL
# 			alt					ボタン代替テキスト
# 			target				リンクターゲットウィンドウ名
# 			option				<a> のオプション
# 			abs					画像のパスを絶対パスにする(=abs)
#
#	(3)-2 obj->button_menu(name,dir[,maxwidth,default,add,list,lib_dir]);
#		ボタン選択フォームを返します。
#
# ---------------------------------------------------------------------------------------
#	(4)-1 obj->frame(dir,image,\contents,\title,$tag,$abs,lib_dir,%option);
#		フレームを返します。
# 			contents			フレームコンテンツのリファレンス
# 			title				フレームタイトルのリファレンス
# 			tag					no=タグ無し
# 			%option				フレームオプション
#
#	(4)-2 obj->frame_menu(name,dir[,cols,default,add,list,lib_dir]);
#		フレーム選択フォームを返します。
# 			list				ID=ID情報付きのリスト
#
# ---------------------------------------------------------------------------------------
#	(5)-1 obj->numbers(dir,image,lib_dir);
#		数字GIFのサンプルを返します。
#
#	(5)-2 obj->number_menu(name,dir[,default,add,list,lib_dir]);
#		数字GIF選択フォームを返します。
#
# ---------------------------------------------------------------------------------------
#	(6)-1 obj->tab_sample(dir,image,text,lib_dir);
#		タブのサンプルを返します。
# 			text				タブのテキスト
#
#	(6)-2 obj->tab_menu(name,dir[,default,add,list,lib_dir]);
#		タブ選択フォームを返します。
#
######################################################################
# 	■特殊変数
#	$env::spacer			スペーサーGIFの位置（相対パス）
#	$env::image_lib			共通素材画像ライブラリの位置（相対パス）
#	$env::image_dir			画像ディレクトリ（相対パス）
#
######################################################################
# 	■画像命名規則
#	background				自由
#	button					out.gif	通常画像
#							down.gif	マウスクリック時の画像
#							over.gif	マウスを重ねたときの画像
#							up.gif	マウスを上げたときの画像
#							※ 通常はup.gifはout.gifで代用
#	frame					Frame.pm参照
#
######################################################################
# 	■ディレクトリ配置規則
#	ディレクトリ名			英字のみ可
#	サブディレクトリ名		数字のみ可
#							background/imageの画像名も数字のみ可
#	background/image		画像ディレクトリ直下
#							例）../imageLib/background/1.gif
#	button					サブディレクトリを作りその中に格納
#							例）../imageLib/button/1/out.gif
#	frame					サブディレクトリを作りその中に格納
#							例）../imageLib/frame/1/ml.gif
#
######################################################################
#オブジェクト作成
sub new ($;$$$$$) {
    my ( $pkg, $image_dir, $image_lib, $output, $setting, $deco_pref ) = @_;
    my $this = {};
    bless $this;
    $this->{'output'} = ( defined $output && $output ) ? $output : '';
    if ( ref $setting ) {
        $this->{'setting'} = $setting;
    }
    elsif ($image_dir) {
        $this->{'setting'} = {};
    }
    else {
        $this->{'setting'} = \%setting::deco;
    }
    if ( defined $deco_pref ) {
        $this->{'deco_pref'} = $deco_pref;
    }
    elsif ($image_dir) {
        $this->{'deco_pref'} = '';
    }
    else {
        $this->{'deco_pref'} = $env::preference_dir . '/deco';
    }
    if ( defined $image_dir && $image_dir ) {
        $this->{'image_dir'} = $image_dir;
    }
    else {
        $this->{'image_dir'} = $env::image_dir;
    }
    if ( defined $image_lib && $image_lib ) {
        $this->{'image_lib'} = $image_lib;
    }
    else {
        $this->{'image_lib'} = $env::image_lib;
    }
    unless ( ref $cgi::Image ) {
        require CMONOS::CGI_Lib::Image;
        $cgi::Image = Image->new;
    }
    require CMONOS::CGI_Lib::Table;
    require CMONOS::CGI_Lib::Frame;
    $this;
}

#書き出し先の設定
sub set_output ($$) {
    my ( $this, $output ) = @_;
    $this->{'old_output'} = $this->{'output'} if defined $this->{'output'};
    $this->{'output'} = ( defined $output ) ? $output : '';
    return $this;
}

sub output ($) {
    my $this = shift;
    return ( defined $this->{'output'} ) ? $this->{'output'} : '';
}

sub restore_output ($) {
    my $this = shift;
    $this->{'output'}
        = ( defined $this->{'old_output'} ) ? $this->{'old_output'} : '';
    delete $this->{'old_output'};
    return $this;
}

#メニュー自動判別
sub menu ($$$;$$$$$$) {
    my ($this,    $type, $name, $dir, $width,
        $default, $add,  $list, $is_image_lib
    ) = @_;
    if ( $type eq 'background' ) {
        return $this->background_menu( $name, $dir, $width, $default, $add,
            $list, $is_image_lib );
    }
    elsif ( $type eq 'image' ) {
        return $this->image_menu( $name, $dir, $width, $default, $add, $list,
            $is_image_lib );
    }
    elsif ( $type eq 'button' ) {
        return $this->button_menu( $name, $dir, $width, $default, $add,
            $list, $is_image_lib );
    }
    elsif ( $type eq 'frame' || $type eq 'bar' ) {
        return $this->frame_menu( $name, $type, $dir, $default, $add, $list,
            $is_image_lib );
    }
    elsif ( $type eq 'number' ) {
        return $this->number_menu( $name, $type, $dir, $default, $add, $list,
            $is_image_lib );
    }
    elsif ( $type eq 'tab' ) {
        return $this->tab_menu( $name, $type, $dir, $default, $add, $list,
            $is_image_lib );
    }
}

#装飾設定のパス
sub preference_db ($$) {
    my $this = shift;
    return ( $this->{'deco_pref'} ) ? $this->{'deco_pref'} : '';
}

######################################################################
#	(1)-2 obj->background_menu;
sub background_menu ($$$;$$$$$) {
    no warnings qw(numeric);
    my ($this, $name, $dir,          $width, $default,
        $add,  $list, $is_image_lib, $is_tabbg
    ) = @_;
    my ( @img, $out, $image_dir, $height );
    $image_dir
        = ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'};
    if ( opendir( DIR, "$image_dir/$dir" ) ) {
        @img
            = grep( -f "$image_dir/$dir/$_"
                && $_ =~ /^[0-9a-zA-Z_]+\.(jpg|gif|png)$/i,
            readdir(DIR) );
        closedir(DIR);
    }
    return '<div class="note">画像が見つかりません。</div>'
        if !@img;

    $out
        .= '<div class="note">画像をダブルクリックすると背景見本ウィンドウが開きます。</div>';
    $width = 180 if !$width;
    $height = int( $width * 0.75 );

    $out .= '<ul class="materials">';

    #新規作成
    $out
        .= '<li'
        . (
        ( !defined $default || $default eq '' ) ? ' class="selected"' : '' )
        . '><label class="bgimg" style="display: block; width: '
        . $width
        . 'px; height: '
        . $height . 'px;">'
        . '<input name="'
        . $name
        . '" type="radio" value=""'
        . (
        ( !defined $default || $default eq '' ) ? ' checked="checked"' : '' )
        . ' />'
        . ( ($add) ? '<span class="add">新規作成</span>' : '' )
        . '</label>'
        . ( ( !defined $default || $default eq '' )
        ? '<div class="marker">▲</div>'
        : '' )
        . '</li>'
        if !defined $list || !$list;

    foreach my $img ( sort { $a <=> $b || $a cmp $b } @img ) {
        my ( $path, $w, $h, $type );
        $path = $image_dir . '/' . $dir . '/' . $img;
        ( $w, $h ) = $cgi::Image->info($path);
        $path = $this->base_change( $path, $this->{'output'} )
            if $this->{'output'};
        $img =~ s/\.(jpg|jpeg|gif|png)$//i;
        $type = uc $1;
        $out
            .= '<li'
            . ( ( defined $default && $img eq $default ) ? ' class="selected"'
            : '' )
            . ' title="'
            . $w . '×'
            . $h . '('
            . $type . ')'
            . '" ondblclick="cmonos.window.openBgSample(\''
            . $path . '\')">'
            . '<label class="bgimg" style="display: block; width: '
            . $width
            . 'px; height: '
            . $height
            . 'px; background-image: url(&quot;'
            . $path
            . '&quot;);'
            . ( ($is_tabbg)
            ? ' background-repeat: repeat-x; background-position: bottom;'
            : '' )
            . '">'
            . '<input name="'
            . $name
            . '" type="radio" value="'
            . $img . '"'
            . (
            ( defined $default && $img eq $default ) ? ' checked="checked"'
            : '' )
            . ' />'
            . '</label>'
            . ( ( defined $default && $img eq $default )
            ? '<div class="marker">▲</div>'
            : '' )
            . '</li>';
    }

    $out .= '</ul>';
    return $out;
}

# (1)-3 obj->background_info;
sub background_file_name ($$$;$) {
    my ( $this, $dir, $image, $is_image_lib ) = @_;
    my ($ext);
    $ext = $this->get_ext( $dir, $image, $is_image_lib );
    return if !defined $ext || $ext eq '';
    return $image . '.' . $ext;
}

sub get_ext ($$$) {
    my ( $this, $dir, $image, $is_image_lib ) = @_;
    my $image_name
        = ( ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'} )
        . '/'
        . $dir . '/'
        . $image;
    if ( -f $image_name . '.gif' ) {
        unlink( $image_name . '.jpg', $image_name . '.png' );
        return 'gif';
    }
    elsif ( -f $image_name . '.jpg' ) {
        unlink( $image_name . '.gif', $image_name . '.png' );
        return 'jpg';
    }
    elsif ( -f $image_name . '.png' ) {
        unlink( $image_name . '.jpg', $image_name . '.gif' );
        return 'png';
    }
    return '';
}

######################################################################
#	(2)-1 obj->image_menu;
sub image_menu ($$$;$$$$$$) {
    no warnings qw(numeric);
    my ($this, $name, $dir,          $width, $default,
        $add,  $list, $is_image_lib, $target_id
    ) = @_;
    my ( @img, $out, $image_dir, $target_width, $target_height );
    local (*DIR);
    $width = 120 if !$width;
    $image_dir
        = ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'};
    if ( opendir( DIR, "$image_dir/$dir" ) ) {
        @img
            = grep( -f "$image_dir/$dir/$_"
                && $_ =~ /^[0-9a-zA-Z_]+\.(jpg|gif|png)$/i,
            readdir(DIR) );
        closedir(DIR);
    }
    return '<div class="note">画像が見つかりません。</div>'
        if !@img;
    if ( $is_image_lib && $target_id ) {
        $out
            .= '<div class="note">背景が<span class="samesizecolor">■</span>の画像は画像サイズが一致します。</div>';
        ( $target_width, $target_height )
            = $this->image_info( $dir, $target_id, 0 );
    }
    $out .= '<ul class="materials">';

    #新規作成
    $out
        .= '<li'
        . (
        ( !defined $default || $default eq '' ) ? ' class="selected"' : '' )
        . '><label class="img"><input name="'
        . $name
        . '" type="radio" value=""'
        . (
        ( !defined $default || $default eq '' ) ? ' checked="checked"' : '' )
        . ' />'
        . ( ($add) ? '<span class="add">新規作成</span>' : '' )
        . '</label>'
        . ( ( !defined $default || $default eq '' )
        ? '<div class="marker">▲</div>'
        : '' )
        . '</li>'
        if !defined $list || !$list;

    foreach my $img ( sort { $a <=> $b || $a cmp $b } @img ) {
        my ( $path, $w, $h );
        $path = $image_dir . '/' . $dir . '/' . $img;
        $img =~ s/\.(jpg|gif|png)$//i;
        ( $w, $h ) = $cgi::Image->info($path);
        $out .= '<li'
            . (
            ( defined $default && $default eq $img ) ? ' class="selected"'
            : (        $is_image_lib
                    && $target_id
                    && $target_width
                    && $target_width == $w
                    && $target_height
                    && $target_height == $h ) ? ' class="samesize"'
            : ''
            )
            . '>'
            . '<label class="img"><input name="'
            . $name
            . '" type="radio" value="'
            . $img . '"'
            . (
            ( defined $default && $img eq $default ) ? ' checked="checked"'
            : '' )
            . ' />'
            . $cgi::Image->image_tag(
            $path, $w, $h, 0,
            (   ( $this->{'setting'}{$dir}{$img}{'alt'} )
                ? $this->{'setting'}{$dir}{$img}{'alt'}
                : ''
                )
                . '['
                . $w . '×'
                . $h . ']',
            '', $width, '',
            $this->{'output'}
            )
            . '</label>'
            . ( ( defined $default && $default eq $img )
            ? '<div class="marker">▲</div>'
            : '' )
            . '</li>';
    }

    $out .= '</ul>';
    return $out;
}

# (2)-2 obj->image_info;
sub image_info ($$$;$) {
    my ( $this, $dir, $image, $is_image_lib ) = @_;
    my ( @image, $ext );
    $ext = $this->get_ext( $dir, $image, $is_image_lib );
    return () if !$ext;
    @image
        = $cgi::Image->info(
          ( ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'} )
        . '/'
            . $dir . '/'
            . $image . '.'
            . $ext );
    return () if !defined $image[2] || $image[2] eq '';
    $image[2] = $image . '.' . $image[2];
    return @image;
}

# (2)-3 obj->image_path;
sub image_path ($$$;$) {
    my ( $this, $dir, $image, $is_image_lib ) = @_;
    my $ext = $this->get_ext( $dir, $image, $is_image_lib );
    return ($ext)
        ? ( ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'} )
        . '/'
        . $dir . '/'
        . $image . '.'
        . $ext
        : '';
}

######################################################################
#	(3)-1-1 obj->button;
sub button ($$$;$$$$$$) {
    my ($this,   $dir,    $id,        $location,     $title,
        $target, $option, $path_type, $is_image_lib, $escaped
    ) = @_;
    my ($class_name);
    $location
        = ( !defined $location || $location eq '' ) ? '#'
        : (    ( $path_type || $this->{'output'} )
            && $location !~ /:/
            && $location !~ /\&#58;/ )
        ? $this->base_change( $location, $this->{'output'}, $path_type )
        : $location;
    $option = $this->get_attribute_from_hash($option);
    $this->escape_except_html_entity( \$location ) if !$escaped;
    $this->escape_except_html_entity( \$title );
    $title = ($title) ? $title : ($default::english) ? 'Link' : 'リンク';
    $title =~ s/[\x0A\x0D]//sg;
    return
          '<a href="'
        . $location . '"'
        . ( ( $dir && $id )
        ? ' class="button-'
            . ( ($is_image_lib) ? 'lib-' : '' )
            . $dir . '-'
            . $id . '"'
        : '' )
        . ( ($target)           ? ' target="' . $target . '"' : '' )
        . ( ($title)            ? ' title="' . $title . '"'   : '' )
        . ( ( defined $option ) ? $option                     : '' ) . '>'
        . $title . '</a>';
}

sub get_attribute_from_hash ($$) {
    my ( $this, $hash_ref ) = @_;
    if ( ref $hash_ref ) {
        my ( $k, $v );
        my $attribute = '';
        while ( ( $k, $v ) = each %$hash_ref ) {
            $attribute
                .= ' '
                . $this->escape($k) . '="'
                . $this->escape_except_html_entity($v) . '"';
        }
        return $attribute;
    }
    elsif ($hash_ref) {
        return ( ( $hash_ref && $hash_ref =~ /^[0-9a-zA-Z_]/ ) ? ' ' : '' )
            . $hash_ref;
    }
    return '';
}

#	(3)-1-2 obj->submit_button;
sub submit_button ($$$$;$$$$) {
    my ( $this, $dir, $id, $alt, $option, $query, $path_type, $suffix ) = @_;
    my ( $class_name, $alt_class_name, $name, $form );
    $alt = $this->escape_except_html_entity($alt) if $alt;
    if ( $dir && $id ) {
        $class_name = 'button-' . $dir . '-' . $id;
        $id =~ s/-(selected|active|visited)$//;
    }
    if ( ref $query ) {
        $name = $this->url_encoded_query($query);
        $name =~ s/\&/\&amp;/g;
    }
    return '<input type="submit"'
        . ( ($class_name)
        ? ' class="' . $class_name . ( ($suffix) ? $suffix : '' ) . '"'
        : '' )
        . ( ($name) ? ' name="' . $name . '"' : '' )
        . ( ( defined $alt && $alt ne '' )
        ? ' value="' . $alt . '" title="' . $alt . '"'
        : '' )
        . $this->get_attribute_from_hash($option) . ' />';
}

#	(3)-2 obj->button_menu;
sub button_menu ($$$;$$$$$$$) {
    no warnings qw(numeric);
    my ($this, $name, $dir,          $width, $default,
        $add,  $list, $is_image_lib, $target_id
    ) = @_;
    my ( @img, $out, $image_dir );
    $width = 120 if !$width;
    $image_dir
        = ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'};
    if ( opendir( DIR, "$image_dir/$dir" ) ) {
        @img = grep( -d "$image_dir/$dir/$_" && $_ !~ /[^0-9a-zA-Z_]/,
            readdir(DIR) );
        closedir(DIR);
    }
    else {
        return
            '<div class="note">画像ディレクトリが見つかりません。</div>';
    }
    return '<div class="note">画像が見つかりません。</div>'
        if !@img;

    $out .= '<ul class="materials">';

    #新規作成
    $out
        .= '<li'
        . (
        ( !defined $default || $default eq '' ) ? ' class="selected"' : '' )
        . '><label class="button"><input name="'
        . $name
        . '" type="radio" value=""'
        . (
        ( !defined $default || $default eq '' ) ? ' checked="checked"' : '' )
        . ' />'
        . ( ($add) ? '<span class="add">新規作成</span>' : '' )
        . '</label>'
        . ( ( !defined $default || $default eq '' )
        ? '<div class="marker">▲</div>'
        : '' )
        . '</li>'
        if !defined $list || !$list;

    foreach my $img ( sort { $a <=> $b || $a cmp $b } @img ) {
        $out
            .= '<li'
            . ( ( defined $default && $img eq $default ) ? ' class="selected"'
            : '' )
            . '><label class="button"><input name="'
            . $name
            . '" type="radio" value="'
            . $img . '"'
            . (
            ( defined $default && $img eq $default ) ? ' checked="checked"'
            : '' )
            . ' />'
            . $this->button( $dir, $img, $env::dummy_window, $img,
            'dummyViewer',
            'onclick="cmonos.window.open(\'dummyViewer\',250,50)"',
            '', $is_image_lib )
            . '</label>'
            . ( ( defined $default && $img eq $default )
            ? '<div class="marker">▲</div>'
            : '' )
            . '</li>';
    }

    $out .= '</ul>';
    return $out;
}

######################################################################
#	(4)-1 obj->frame;
sub frame ($$$;$$$$@) {
    my ($this, $dir, $image,        $contents, $title,
        $tag,  $abs, $is_image_lib, %option
    ) = @_;
    my ($frame);
    if ( !$is_image_lib && ref $frame::obj{$dir}{$image} ) {
        if ( %option || $this->{'output'} ) {
            $frame = Frame->new;
            %{$frame} = %{ $frame::obj{$dir}{$image} };
            $frame->set_direct(%option);
            $frame->frame_base_change( $this->{'output'}, $abs );
        }
        else {
            $frame = $frame::obj{$dir}{$image};
        }
    }
    else {
        $frame = Frame->new;
        $frame->set_direct(
            'library' => ( ($is_image_lib) ? $dir . '-lib' : $dir ),
            'id'      => $image,
            'cssbase' => 1,
            %option
        );
        $frame->frame_base_change( $this->{'output'}, $abs );
    }
    $tag = 'no' if !defined $tag || !$tag;
    return $frame->frame( $contents, $title, $tag );
}

#	(4)-2 obj->frame_menu;
sub frame_menu ($$$;$$$$$) {
    no warnings qw(numeric);
    my ( $this, $name, $dir, $default, $add, $list, $is_image_lib,
        $target_id )
        = @_;
    my ( $out, @img, $image_dir );
    $image_dir
        = ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'};
    if ( opendir( DIR, "$image_dir/$dir" ) ) {
        @img = grep( -d "$image_dir/$dir/$_" && $_ !~ /[^0-9a-zA-Z_]/,
            readdir(DIR) );
        closedir(DIR);
    }
    else {
        return
            '<div class="note">画像ディレクトリが見つかりません。</div>';
    }
    return '<div class="note">画像が見つかりません。</div>'
        if !@img;

    $out .= '<ul class="materials">';

    #新規作成
    $out
        .= '<li'
        . (
        ( !defined $default || $default eq '' ) ? ' class="selected"' : '' )
        . '><label><input name="'
        . $name
        . '" type="radio" value=""'
        . (
        ( !defined $default || $default eq '' ) ? ' checked="checked"' : '' )
        . ' />'
        . ( ($add) ? '<span class="add">新規作成</span>' : '' )
        . '</label>'
        . ( ( !defined $default || $default eq '' )
        ? '<div class="marker">▲</div>'
        : '' )
        . '</li>'
        if !defined $list || !$list;

    foreach my $img ( sort { $a <=> $b || $a cmp $b } @img ) {
        my $sample
            = ( $list && $list eq 'ID' ) ? 'ID:' . $img
            : '<input name="'
            . $name
            . '" type="radio" value="'
            . $img . '"'
            . (
            ( defined $default && $img eq $default ) ? ' checked="checked"'
            : '' )
            . ' />';
        $out
            .= '<li'
            . ( ( defined $default && $img eq $default ) ? ' class="selected"'
            : '' )
            . '>'
            . $this->frame( $dir, $img, \$sample, '', 'no', '',
            $is_image_lib )
            . ( ( defined $default && $img eq $default )
            ? '<div class="marker">▲</div>'
            : '' )
            . '</li>';
    }

    $out .= '</ul>';
    return $out;
}

######################################################################
#	(5)-1 obj->numbers;
sub numbers ($$$;$$) {
    my ( $this, $dir, $image, $is_image_lib, $short_flag ) = @_;
    my ( $gif, $number, $vector, @num_imgs );
    $gif
        = ( ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'} )
        . '/'
        . $dir . '/'
        . $image;
    $vector = ( -f $gif . '/v.gif' ) ? 'v' : 'h';

    $number
        = '<table cellpadding="0" cellspacing="0" border="0" title="ID:'
        . $image
        . '"><tbody>';
    $number .= '<tr>' if $vector ne 'v';
    @num_imgs = ($short_flag)
        ? (
        'start', 'total', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0',
        'total_end', 'kiriban', 'end'
        )
        : (
        'start',     'total',  '1',             '2',
        '3',         '4',      '5',             'total_end',
        'kiriban',   'spacer', 'thismonth',     '6',
        '7',         '8',      'thismonth_end', 'spacer',
        'today',     '9',      '0',             'today_end',
        'yesterday', '8',      '8',             'yesterday_end',
        'end'
        );
    foreach (@num_imgs) {
        $number .= $this->number_cell( $gif, $_, $vector );
    }
    $number .= '</tr>' if $vector ne 'v';
    $number .= '</tbody></table>';
    return $number;
}

sub number_cell ($$$$) {
    my ( $this, $gif, $num, $vector ) = @_;
    my ($number);
    return '' unless -f $gif . '/' . $num . '.gif';
    $number = '<tr>' if $vector eq 'v';
    $number .= '<td>'
        . $cgi::Image->tag( $gif . '/' . $num . '.gif',
        0, $num, ' style="display: block"' )
        . '</td>';
    $number .= '</tr>' if $vector eq 'v';
    return $number;
}

#	(5)-2 obj->number_menu;
sub number_menu ($$$;$$$$) {
    no warnings qw(numeric);
    my ( $this, $name, $dir, $default, $add, $list, $is_image_lib ) = @_;
    my ( $out, @img, $image_dir );
    $image_dir
        = ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'};
    if ( opendir( DIR, "$image_dir/$dir" ) ) {
        @img = grep( -d "$image_dir/$dir/$_" && $_ !~ /[^0-9a-zA-Z_]/,
            readdir(DIR) );
        closedir(DIR);
    }
    else {
        return
            '<div class="note">画像ディレクトリが見つかりません。</div>';
    }
    return '<div class="note">画像が見つかりません。</div>'
        if !@img;

    $out .= '<ul class="materials">';

    $out
        .= '<li'
        . (
        ( !defined $default || $default eq '' ) ? ' class="selected"' : '' )
        . '><label><input name="'
        . $name
        . '" type="radio" value=""'
        . (
        ( !defined $default || $default eq '' ) ? ' checked="checked"' : '' )
        . ' />'
        . ( ($add) ? '<span class="add">新規作成</span>' : '' )
        . '</label>'
        . ( ( !defined $default || $default eq '' )
        ? '<div class="marker">▲</div>'
        : '' )
        . '</li>'
        if !$list;
    foreach my $img ( sort { $a <=> $b || $a cmp $b } @img ) {
        $out
            .= '<li'
            . ( ( defined $default && $img eq $default ) ? ' class="selected"'
            : '' )
            . '><input name="'
            . $name
            . '" type="radio" value="'
            . $img . '"'
            . (
            ( defined $default && $img eq $default ) ? ' checked="checked"'
            : '' )
            . ' />'
            . $this->numbers( $dir, $img, $is_image_lib, 1 )
            . ( ( defined $default && $img eq $default )
            ? '<div class="marker">▲</div>'
            : '' )
            . '</li>';
    }

    $out .= '</ul>';
    return $out;
}

######################################################################
#	(6)-1 obj->tab_sample;
sub tab_sample ($$$;$$) {
    my ( $this, $dir, $image, $text, $is_image_lib ) = @_;
    my ( $image_folder, $tab, $left_image, $right_image, $padding,
        $font_color, $max_padding );
    $image_folder
        = ( ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'} )
        . '/'
        . $dir . '/'
        . $image;
    ( $left_image, $right_image, $padding, $font_color )
        = (    !$is_image_lib
            && defined $this->{'setting'}{$dir}{$image}{'padding'}
            && $this->{'setting'}{$dir}{$image}{'padding'} > 0 )
        ? (
        $this->{'setting'}{$dir}{$image}{'left'},
        $this->{'setting'}{$dir}{$image}{'right'},
        $this->{'setting'}{$dir}{$image}{'padding'},
        $this->{'setting'}{$dir}{$image}{'color'}
        )
        : $this->tab_info( $dir, $image, $is_image_lib );
    return '' if !$padding;
    $max_padding = ( $padding > 15 ) ? $padding : 15;
    $tab = '<label class="tabsample" style="';
    $tab .= 'margin: 0;';
    $tab .= ' padding: 0 0 0 ' . $padding . 'px;';
    $tab
        .= ' background-image: url(&quot;'
        . $image_folder . '/'
        . $left_image
        . '&quot;);';
    $tab .= ' background-repeat: no-repeat;';
    $tab .= ' background-position: left top;';
    $tab .= ' text-decoration: none;';
    $tab .= ' color: ' . $font_color . ';';
    $tab .= '"><span style="';
    $tab .= 'display: block;';
    $tab
        .= ' background-image: url(&quot;'
        . $image_folder . '/'
        . $right_image
        . '&quot;);';
    $tab .= ' background-repeat: no-repeat;';
    $tab .= ' background-position: right top;';
    $tab
        .= ' padding:15px '
        . $max_padding
        . 'px 15px '
        . ( ( $max_padding == $padding )
        ? '0;'
        : ( $max_padding - $padding ) . 'px;' );
    $tab
        .= '">'
        . ( ( defined $text && $text ne '' ) ? $text : 'Tab' )
        . '</span></label>';
    return $tab;
}

#	(6)-2 obj->tab_menu;
sub tab_menu ($$$;$$$$) {
    no warnings qw(numeric);
    my ( $this, $name, $dir, $default, $add, $list, $is_image_lib ) = @_;
    my ( $out, @tabs, $image_dir );
    $image_dir
        = ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'};
    if ( opendir( DIR, "$image_dir/$dir" ) ) {
        @tabs = grep( -d "$image_dir/$dir/$_" && $_ !~ /[^0-9a-zA-Z_]/,
            readdir(DIR) );
        closedir(DIR);
    }
    else {
        return
            '<div class="note">タブ画像ディレクトリが見つかりません。</div>';
    }
    return '<div class="note">タブ画像が見つかりません。</div>'
        if !@tabs;

    $out .= '<ul class="materials">';

    #新規作成
    $out
        .= '<li'
        . (
        ( !defined $default || $default eq '' ) ? ' class="selected"' : '' )
        . '><label><input name="'
        . $name
        . '" type="radio" value=""'
        . (
        ( !defined $default || $default eq '' ) ? ' checked="checked"' : '' )
        . ' />'
        . ( ($add) ? '<span class="add">新規作成</span>' : '' )
        . '</label>'
        . ( ( !defined $default || $default eq '' )
        ? '<div class="marker">▲</div>'
        : '' )
        . '</li>'
        if !defined $list || !$list;

    foreach my $img ( sort { $a <=> $b || $a cmp $b } @tabs ) {
        $out
            .= '<li'
            . ( ( defined $default && $img eq $default ) ? ' class="selected"'
            : '' )
            . '>'
            . $this->tab_sample(
            $dir, $img,
            '<input name="'
                . $name
                . '" type="radio" value="'
                . $img . '"'
                . (
                ( defined $default && $img eq $default )
                ? ' checked="checked"'
                : ''
                )
                . ' />タブ['
                . $img . ']',
            $is_image_lib
            )
            . ( ( defined $default && $img eq $default )
            ? '<div class="marker">▲</div>'
            : '' )
            . '</li>';
    }

    $out .= '</ul>';
    return $out;
}

sub tab_info ($$$;$) {
    my ( $this, $dir, $image, $is_image_lib ) = @_;
    my ( @image, $left_ext, $right_ext );
    $left_ext  = $this->get_ext( $dir, $image . '/left',  $is_image_lib );
    $right_ext = $this->get_ext( $dir, $image . '/right', $is_image_lib );
    @image
        = $cgi::Image->info(
          ( ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'} )
        . '/'
            . $dir . '/'
            . $image
            . '/left.'
            . $left_ext );
    return (   !defined $image[2]
            || $image[2] eq ''
            || !defined $right_ext
            || $right_ext eq '' )
        ? ()
        : ( 'left.' . $left_ext, 'right.' . $right_ext, $image[0],
        '#000000' );
}

######################################################################
#	(7)-1 obj->toggler;
sub toggler ($$$$$;$$$) {
    my ( $this, $dir, $id, $side, $value, $eid, $fixed, $is_image_lib ) = @_;
    return '' if !$side || ( $side ne 'a' && $side ne 'b' && $side ne 'c' );
    $this->escape_except_html_entity( \$value );
    $value =~ s/[\x0A\x0D]//sg;
    my $class
        = ( $dir && $id )
        ? $dir . '-' . ( ($is_image_lib) ? 'lib-' : '' ) . $id
        : '';
    if ($fixed) {
        return
              '<span'
            . ( ($eid) ? ' id="' . $eid . '"' : '' )
            . ( ($class) ? ' class="' . $class . '-' . $side . '"' : '' )
            . ( ( $fixed == 2 )
            ? ' onclick="this.className=((this.className==\''
                . $class
                . '-a\') ? \''
                . $class
                . '-b\' : (this.className==\''
                . $class
                . '-b\') ? \''
                . $class
                . '-c\' : \''
                . $class
                . '-a\')"'
            : '' )
            . ( ($value) ? ' title="' . $value . '">' . $value : '>' )
            . '</span>';
    }
    else {
        return
              '<input type="submit"'
            . ( ($eid) ? ' id="' . $eid . '"' : '' )
            . ( ($class) ? ' class="' . $class . '-' . $side . '"' : '' )
            . (
            ($value) ? ' value="' . $value . '" title="' . $value . '"' : '' )
            . ' />';
    }
}

#	(6)-2 obj->toggler_menu;
sub toggler_menu ($$$;$$$$) {
    no warnings qw(numeric);
    my ( $this, $name, $dir, $default, $add, $list, $is_image_lib ) = @_;
    my ( $out, @togglers, $image_dir );
    $image_dir
        = ($is_image_lib) ? $this->{'image_lib'} : $this->{'image_dir'};
    if ( opendir( DIR, "$image_dir/$dir" ) ) {
        @togglers = grep( -d "$image_dir/$dir/$_" && $_ !~ /[^0-9a-zA-Z_]/,
            readdir(DIR) );
        closedir(DIR);
    }
    else {
        return
            '<div class="note">トグルボタン画像ディレクトリが見つかりません。</div>';
    }
    return
        '<div class="note">トグルボタン画像が見つかりません。</div>'
        if !@togglers;

    $out .= '<ul class="materials">';

    #新規作成
    $out
        .= '<li'
        . (
        ( !defined $default || $default eq '' ) ? ' class="selected"' : '' )
        . '><label><input name="'
        . $name
        . '" type="radio" value=""'
        . (
        ( !defined $default || $default eq '' ) ? ' checked="checked"' : '' )
        . ' />'
        . ( ($add) ? '<span class="add">新規作成</span>' : '' )
        . '</label>'
        . ( ( !defined $default || $default eq '' )
        ? '<div class="marker">▲</div>'
        : '' )
        . '</li>'
        if !defined $list || !$list;

    foreach my $img ( sort { $a <=> $b || $a cmp $b } @togglers ) {
        $out
            .= '<li'
            . ( ( defined $default && $img eq $default )
            ? ' class="selected"'
            : '' )
            . '><label>'
            . $this->toggler( $dir, $img, 'a', 'A', '', 2, $is_image_lib );
        $out
            .= '<input name="'
            . $name
            . '" type="radio" value="'
            . $img . '"'
            . (
            ( defined $default && $img eq $default ) ? ' checked="checked"'
            : '' )
            . ' />'
            . ( ( defined $default && $img eq $default )
            ? '<div class="marker">▲</div>'
            : '' )
            . '</label></li>';
    }

    $out .= '</ul>';
    return $out;
}

1;
