package RSS2_0;
require CMONOS::CGI_Lib::Common;
@ISA = qw(Common);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# RSS2.0更新
# CMONOS::CGI_Lib::Feed::RSS2_0.pm ver1.0.7
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-01-16
######################################################################
# 使い方
# require CMONOS::CGI_Lib::Feed::RSS2_0;
# $rss = RSS2_0->new(
#	'path'					=>	RSSのパス（../index.xml etc）,
#	'maxitem'				=>	最大記事数,
#	'description_maxlength'	=>	説明文の最大文字数（半角換算）,
#	'delete_only'			=>	削除しかしない場合=1,
#	'rss_marker'			=>	link の URL に RSS からのアクセスを示す情報を付加する=1,
#	'utm_medium'			=>	link の URL に RSS からのアクセスを示す情報を付加する場合の RSS の種類
#	'channel_element'		=>	{
# 									チャンネル要素名 	=>	設定,
# 									チャンネル要素名 	=>	設定,
#								},
#	'xmlns'					=>	{
# 									'content' 			=>	'http://purl.org/rss/1.0/modules/content/',
# 									'itunes' 			=>	'http://www.itunes.com/DTDs/Podcast-1.0.dtd',
# 									'dc' 				=>	'http://purl.org/dc/elements/1.1/',
#								},
# );
#
# チャンネル要素一覧
# http://blogs.law.harvard.edu/tech/rss
# http://wiki.koshigoe.jp/doku.php?id=koshigoewiki:feed:rss2.0%E4%BB%95%E6%A7%98（日本語）
#	title			チャンネルのタイトル
#	link			チャンネルを表現するコンテンツのURI
#	description		チャンネルの概要; 大抵、純粋なテキストのみで記述され、改行も意味を持ちません
#	language		言語コード; Netscapeが定めるコードが利用可能です。コード仕様はW3Cで定められています。（en-us,ja 等）
#	copyright		チャンネルの著作権
#	managingEditor	コンテンツ責任者のEmailアドレス
#	webMaster		システム責任者のEmailアドレス
#	pubDate			チャンネルが構築された日時; RFC822に従います
#	lastBuildDate	チャンネルの最終更新日時; pubDateと同規則
#	category		チャンネルが属すカテゴリ; 要素レベルでの複数定義が可能です
#	generator		チャンネルを生成したプログラムの識別名
#	docs			RSS2.0のドキュメントを表すURI; 原則http://blogs.law.harvard.edu/tech/rss
#	cloud			XML-RPCなどを利用した更新通知などのサービス定義; HTTP-POST/XML-RPC/SOAPが利用可能です
#	ttl				キャッシュを保持する時間(分); 整数値のみ利用できます
#	image			チャンネルを表現する画像のURI
#	rating			PICSのレート
#	textInput		入力可能なテキストボックス; ドラフト
#	skipHours		アグリゲータに対して与える無視時間
#	skipDays		アグリゲータに対して与える無視日数
#
# アイテム追加
# $rss->update({
# 	アイテム要素名 	=>	設定,
# 	アイテム要素名 	=>	設定,
# },keep_pubDate);
#
# アイテム要素一覧
#	title			アイテムのタイトル
#	link			アイテムを表現するコンテンツのURI
#	description		アイテムの概要
#	author			アイテムの著者
#	category		アイテムが属するカテゴリ
#	comments		アイテムに関連するコメントコンテンツのURI
#	enclosure		メディアコンテンツのURI	<enclosure url="http://www.scripting.com/mp3s/weatherReportSuite.mp3" length="12216320" type="audio/mpeg" />
#	guid			アイテム固有の文字列	<guid isPermaLink="true">http://inessential.com/2002/09/01.php#a2</guid>
#	pubDate			アイテムが発行された日時	Sun, 19 May 2002 15:21:36 GMT
#	source			アイテムのソース	<source url="http://www.tomalak.org/links2.xml">Tomalak's Realm</source>
#
# アイテム削除
# $rss->update({
# 	'delete' 			=>	[削除する記事のlinkリスト],
# 	'delete_prefix' 	=>	[削除する記事のlinkリスト(前方一致)],
# });
#
# オプション
#	keep_pubDate	1=前回更新日時を保持する
#
######################################################################
# オブジェクトを作る。
if ( !ref $cgi::Jcode ) {
    use Jcode;
    $cgi::Jcode = Jcode->new;
    $Jcode::FALLBACK = &Jcode::FB_XMLCREF if defined $Jcode::FALLBACK;
}

sub new ($;@) {
    my $pkg  = shift;
    my $this = {@_};
    bless $this, $pkg;
    $this->{'maxitem'} = 15 if !$this->{'maxitem'};
    $this->{'description_maxlength'} = 128
        if !$this->{'decription_maxlength'};
    $this->{'rss_marker'} = 1
        if !defined $this->{'rss_marker'} && $env::rss_marker;
    $env::file_mode = 0644 if !$env::file_mode;

    #現在時刻
    $this->{'channel_element'}{'pubDate'} = $this->{'gmt'}
        = $this->current_gmt;

    #チャンネル要素のチェック
    $this->{'channel_element'} = {} if !ref $this->{'channel_element'};
    $this->{'channel_element'}{'link'} = $this->home_location
        if !defined $this->{'channel_element'}{'link'}
        || $this->{'channel_element'}{'link'} eq '';
    $this->{'channel_element'}{'link'} =~ s/[\?\&]$//;
    $this->{'channel_element'}{'title'}
        = ( defined $env::title && $env::title ne '' )
        ? $env::title
        : $this->{'channel_element'}{'link'}
        if !defined $this->{'channel_element'}{'title'}
        || $this->{'channel_element'}{'title'} eq '';
    $this->{'channel_element'}{'description'}
        = ( defined $env::description && $env::description ne '' )
        ? $env::description
        : $this->{'channel_element'}{'title'}
        if !defined $this->{'channel_element'}{'description'}
        || $this->{'channel_element'}{'description'} eq '';
    $this->{'channel_element'}{'language'} = 'ja-JP'
        if !defined $this->{'channel_element'}{'language'}
        || $this->{'channel_element'}{'language'} eq '';
    $this->{'channel_element'}{'copyright'}
        = 'Copyright '
        . ( ( localtime() )[5] + 1900 )
        . ( ( $this->{'channel_element'}{'copyright'} )
        ? ', ' . $this->{'channel_element'}{'copyright'}
        : '' )
        if !defined $this->{'channel_element'}{'copyright'}
        || $this->{'channel_element'}{'copyright'} eq ''
        || $this->{'channel_element'}{'copyright'} !~ /Copyright [0-9]{4}/i;
    $this->{'channel_element'}{'lastBuildDate'}
        = $this->{'channel_element'}{'pubDate'}
        if !$this->{'delete_only'}
        && ( !defined $this->{'channel_element'}{'lastBuildDate'}
        || $this->{'channel_element'}{'lastBuildDate'} eq '' );
    $this->{'channel_element'}{'generator'} = 'http://cmonos.jp/'
        if !defined $this->{'channel_element'}{'generator'}
        || $this->{'channel_element'}{'generator'} eq '';
    $this->{'channel_element'}{'docs'}
        = 'http://blogs.law.harvard.edu/tech/rss'
        if !defined $this->{'channel_element'}{'docs'}
        || $this->{'channel_element'}{'docs'} eq '';
    $this->{'channel_element'}{'description'}
        = $this->cut_string( $this->{'channel_element'}{'description'},
        $this->{'description_maxlength'}, 'no' );
    $this->_image_init( $this->{'channel_element'} );
    $this->_init();
    return $this;
}

sub current_gmt ($;$) {
    my ( $this, $time ) = @_;
    $time = time if !$time;
    my $gmt = gmtime($time);
    $gmt
        =~ s/^([a-zA-Z]+)[\x09\x0A\x0C\x0D\x20]+([a-zA-Z]+)[\x09\x0A\x0C\x0D\x20]+([0-9]{1,2})[\x09\x0A\x0C\x0D\x20]+([0-9]{2}:[0-9]{2}:[0-9]{2})[\x09\x0A\x0C\x0D\x20]+([0-9]{4})/$1, $3 $2 $5 $4/;
    return $gmt . ' GMT';
}

sub to_utf8 ($) {
    my ( $this, $str ) = @_;
    if ( !utf8::is_utf8($str) ) {
        $str = $cgi::Jcode->set($str)->utf8;
        utf8::decode($str);
    }
    return $str;
}

sub _init ($) {
    my $this = shift;
}

sub _image_init ($) {
    my ( $this, $item ) = @_;
    if ( ref $item->{'image'} ) {
        if (   $item->{'image'}{'url'}
            && !ref $item->{'image'}{'value'}
            && !defined $item->{'image'}{'value'}{'url'} )
        {
            $item->{'image'}{'value'}{'url'} = $item->{'image'}{'url'};
        }
        if (   !ref $item->{'image'}{'value'}
            || !defined $item->{'image'}{'value'}{'url'} )
        {
            undef $item->{'image'};
        }
        else {
            $item->{'image'}{'value'}{'link'} = $item->{'link'}
                if !defined $item->{'image'}{'value'}{'link'}
                || $item->{'image'}{'value'}{'link'} eq '';
            $item->{'image'}{'value'}{'title'} = $item->{'title'}
                if !defined $item->{'image'}{'value'}{'title'}
                || $item->{'image'}{'value'}{'title'} eq '';
        }
    }
}

sub push_rss_marker ($$$;$) {
    my ( $this, $link, $rss_marker, $is_escaped ) = @_;
    $link = $this->unescape($link) if $is_escaped;
    $link =~ s/[\?\&]utm_source=RSS\&utm_medium=[^\&#=]+//;
    if ($rss_marker) {
        $rss_marker = ( ( $link =~ /\?/ ) ? '&' : '?' ) . $rss_marker;
        $link .= $rss_marker unless $link =~ s/#/$rss_marker#/;
    }
    $link = $this->escape($link) if $is_escaped;
    return $link;
}

sub update ($$;$) {
    my ( $this, $item_ref, $keep_pubdate ) = @_;
    return 0 if !$this->{'path'};
    my $item = {%$item_ref};

    #アイテム要素のチェック
    return 0
        if !$item->{'link'}
        && !ref $item->{'delete'}
        && !ref $item->{'delete_prefix'}
        && !ref $item->{'delete_regexp'};
    my $rss_marker
        = ( $this->{'rss_marker'} )
        ? 'utm_source=RSS&utm_medium='
        . $this->url_encode( ( $this->{'utm_medium'} )
        ? $this->{'utm_medium'}
        : $this->to_utf8( $this->{'channel_element'}{'title'} ) )
        : '';
    if ( $item->{'link'} ) {
        $item->{'description'} = $item->{'content:encoded'}
            if ( !defined $item->{'description'}
            || $item->{'description'} eq '' )
            && defined $item->{'content:encoded'}
            && $item->{'content:encoded'} ne '';
        if ( !defined $item->{'title'} || $item->{'title'} eq '' ) {
            if ( $item->{'description'} ) {
                $item->{'title'} = $this->cut_string(
                    $this->detag_lite( $item->{'description'} ), 32 );
            }
            else {
                $item->{'title'} = $item->{'link'};
            }
        }
        $item->{'description'} = $item->{'title'}
            if !defined $item->{'description'}
            || $item->{'description'} eq '';
        if ( $item->{'pubDate'} && $item->{'pubDate'} eq 'KEEP' ) {
            $keep_pubdate = 1;
            $item->{'pubDate'} = $this->{'gmt'};
        }
        elsif ( !$item->{'pubDate'} ) {
            $item->{'pubDate'} = $this->{'gmt'};
        }
        if (   !defined $item->{'guid'}
            || $item->{'guid'} eq ''
            || ( ref $item->{'guid'} && !$item->{'guid'}{'value'} ) )
        {
            $item->{'guid'} = {
                'isPermaLink' => 'true',
                'value'       => $item->{'link'},
            };
        }
        elsif (defined $item->{'guid'}
            && !ref $item->{'guid'}
            && $item->{'guid'} ne '' )
        {
            $item->{'guid'} = { 'value' => $item->{'guid'} };
        }
        if ( $item->{'content:encoded'} ) {
            $this->{'xmlns'}{'content'}
                = 'http://purl.org/rss/1.0/modules/content/'
                if !$this->{'xmlns'}{'content'};
            my $path = $item->{'guid'}{'value'};
            $path =~ s!^https?://[^/]+!!ig;
            $item->{'content:encoded'}
                = $this->rel_to_abs( $item->{'content:encoded'}, $path, 1,
                1 );
        }
        if ( defined $item->{'g:price'} ) {
            $this->{'xmlns'}{'g'} = 'http://base.google.com/ns/1.0'
                if !$this->{'xmlns'}{'g'};
        }
        if ( defined $item->{'dc:creator'} ) {
            $this->{'xmlns'}{'dc'} = 'http://purl.org/dc/elements/1.1/'
                if !$this->{'xmlns'}{'dc'};
        }
        $item->{'description'}
            = $this->cut_string( $this->detag_lite( $item->{'description'} ),
            $this->{'description_maxlength'} );
        $item->{'link'}
            = $this->push_rss_marker( $item->{'link'}, $rss_marker )
            if $rss_marker;
        $this->_image_init($item);
        $this->_item_init($item);
    }
    elsif ( !-f $this->{'path'} ) {
        return 0;
    }

    my @delete_regexp
        = ( ref $item->{'delete_prefix'} )
        ? map( quotemeta($_), @{ $item->{'delete_prefix'} } )
        : ();
    push( @delete_regexp, @{ $item->{'delete_regexp'} } )
        if ref $item->{'delete_regexp'};
    my $temp_path  = $this->{'path'} . '.tmp';
    my $is_updated = 0;
    local ( *RSS, *TEMP );
    if ( open( TEMP, ">>$temp_path" ) ) {
        binmode TEMP, ":utf8";
        flock( TEMP, LOCK_EX );
        truncate( TEMP, 0 );
        seek( TEMP, 0, 0 );
        print TEMP '<?xml version="1.0" encoding="UTF-8"?>', "\x0A";
        print TEMP '<rss version="2.0"';
        foreach my $xmlns ( sort keys %{ $this->{'xmlns'} } ) {
            print TEMP ' xmlns:'
                . $xmlns . '="'
                . $this->{'xmlns'}{$xmlns} . '"';
        }
        print TEMP '>',         "\x0A";
        print TEMP '<channel>', "\x0A";
        my %done               = ();
        my $is_channel_element = 1;
        my $item_elements      = '';
        my $item_counter       = 0;
        print TEMP $this->_hash_to_xml( $this->{'channel_element'}, \%done,
            1 );

        if ( !-f $this->{'path'} ) {
            print TEMP '<item>', "\x0A";
            print TEMP $this->_hash_to_xml($item);
            print TEMP '</item>', "\x0A";
            $is_updated = 1;
        }
        elsif ( open( RSS, $this->{'path'} ) ) {
            binmode RSS, ":utf8";
            flock( RSS, LOCK_SH );
            my $done = 0;
        ITEM:
            while (<RSS>) {
                my $line = $_;
                if ( $item_counter >= $this->{'maxitem'} ) {
                    last ITEM;
                }
                elsif ( $line =~ /<item>/ || $line =~ /<\/channel>/ ) {
                    $done = 1 if !$keep_pubdate;
                    if ($is_channel_element) {
                        $is_channel_element = 0;
                        if (   $item->{'link'}
                            && !$keep_pubdate
                            && (!ref $item->{'delete'}
                                || !grep( $_ eq $item->{'guid'}{'value'},
                                    @{ $item->{'delete'} } )
                            )
                            && (!@delete_regexp
                                || !grep( $item->{'guid'}{'value'} =~ /^$_/,
                                    @delete_regexp )
                            )
                            )
                        {
                            print TEMP '<item>', "\x0A";
                            print TEMP $this->_hash_to_xml($item);
                            print TEMP '</item>', "\x0A";
                            $item_counter++;
                            last ITEM if $line =~ /<\/channel>/;
                        }
                    }
                    $item_elements = $line;
                }
                elsif ( $line =~ m!</item>! ) {
                    $item_elements .= $line;
                    if ( $item_elements
                        =~ /<guid(?![0-9a-zA-Z_])[^>]*>(.*?)<\/guid>/ )
                    {
                        my $guid = $this->unescape($1);
                        if (   defined $item->{'guid'}
                            && ref $item->{'guid'}
                            && defined $item->{'guid'}{'value'}
                            && $item->{'guid'}{'value'} eq $guid )
                        {
                            if ( !$done && $keep_pubdate && $item->{'link'} )
                            {
                                $item->{'pubDate'} = $this->unescape($1)
                                    if $item_elements
                                    =~ /<pubDate(?![0-9a-zA-Z_])[^>]*>(.*?)<\/pubDate>/;
                                print TEMP '<item>', "\x0A";
                                print TEMP $this->_hash_to_xml($item);
                                print TEMP '</item>', "\x0A";
                                $item_counter++;
                                $done = 1;
                            }
                        }
                        elsif (
                            (      !ref $item->{'delete'}
                                || !
                                grep( $_ eq $guid, @{ $item->{'delete'} } )
                            )
                            && (   !@delete_regexp
                                || !grep( $guid =~ /^$_/, @delete_regexp ) )
                            )
                        {
                            $item_elements
                                =~ s/(<link(?![0-9a-zA-Z_])[^>]*>)(.*?)(<\/link>)/$1 . $this->push_rss_marker($2,$rss_marker,1) . $3/ges;
                            print TEMP $item_elements;
                            $item_counter++;
                        }
                    }
                    $item_elements = '';
                }
                elsif ($is_channel_element) {
                    print TEMP $line
                        if $line =~ /<([0-9a-zA-Z_:]+)(?![0-9a-zA-Z_])[^>]*>/
                        && !$done{$1}
                        && $1 ne 'rss'
                        && $1 ne 'channel';
                }
                elsif ($item_elements) {
                    $item_elements .= $line;
                }
            }
            close(RSS);
            if ( !$done && !$keep_pubdate ) {
                print TEMP '<item>', "\x0A";
                print TEMP $this->_hash_to_xml($item);
                print TEMP '</item>', "\x0A";
            }
            $is_updated = 1;
        }
        print TEMP '</channel>', "\x0A";
        print TEMP '</rss>';
        close(TEMP);
        if ($is_updated) {
            rename( $temp_path, $this->{'path'} );
            chmod( $env::file_mode, $this->{'path'} );
        }
    }
    return $is_updated;
}

sub _hash_to_xml ($$;$$) {
    my ( $this, $hash, $done, $oneline ) = @_;
    my $xml = '';
ELEMENT:
    foreach my $key ( sort keys %{$hash} ) {
        next ELEMENT if $key eq 'delete';
        my $value     = '';
        my $attribute = '';
        if ( ref $hash->{$key} ) {
            if ( ref $hash->{$key} eq 'HASH' && %{ $hash->{$key} } ) {
            ATTRIBUTE:
                foreach my $k ( sort keys %{ $hash->{$key} } ) {
                    next ATTRIBUTE
                        if !defined $hash->{$key}{$k}
                        || $hash->{$key}{$k} eq '';
                    if ( $k eq 'value' ) {
                        if ( ref $hash->{$key}{$k} eq 'HASH' ) {
                            $value = "\x0A"
                                . $this->_hash_to_xml( $hash->{$key}{$k} );
                            $value = $this->oneline($value) if $oneline;
                        }
                        else {
                            $value = $this->to_utf8( $hash->{$key}{$k} );
                            $value
                                = ( $key eq 'content:encoded' )
                                ? '<![CDATA['
                                . $this->oneline($value) . ']]>'
                                : $this->detag($value);
                        }
                    }
                    else {
                        $attribute
                            .= ' '
                            . $k . '="'
                            . $this->detag(
                            $this->to_utf8( $hash->{$key}{$k} ) )
                            . '"';
                    }
                }
                $xml
                    .= '<'
                    . $key
                    . $attribute
                    . ( ( $attribute && $value eq '' )
                    ? ' />'
                    : '>' . $value . '</' . $key . '>' )
                    . "\x0A"
                    if $attribute || $value ne '';
            }
        }
        else {
            $value = $this->to_utf8( $hash->{$key} ) if defined $hash->{$key};
            $xml
                .= '<'
                . $key . '>'
                . ( ( $key eq 'content:encoded' )
                ? '<![CDATA[' . $this->oneline($value) . ']]>'
                : $this->detag($value) )
                . '</'
                . $key . '>' . "\x0A"
                if $value ne '';
        }
        $done->{$key} = 1 if ref $done;
    }
    return $xml;
}

sub _item_init ($$) {
    my $this = shift;
}

sub detag ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/<(.*?)>/(($1 =~ m! alt="(.*?)"!i) ? $1 : '')/egs;
    $str
        =~ s/\&(?!(?:#[0-9]{2,6}|#x[a-fA-F0-9]+|[a-zA-Z0-9]{2,10});)/\&amp;/g;
    $str =~ s/"/&quot;/g;
    $str =~ s/</&lt;/g;
    $str =~ s/>/&gt;/g;
    $str =~ s/[\x0D\x0A]+/ /gs;
    return $str;
}

sub detag_lite ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/<(.*?)>/(($1 =~ m! alt="(.*?)"!i) ? $1 : '')/egs;
    $str =~ s/[\x0D\x0A]+/ /gs;
    return $str;
}

sub oneline ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/[\x0D\x0A]+/ /gs;
    $str =~ s/<!--.*?-->//gs;
    return $str;
}

sub escape ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/&/&amp;/gs;
    $str =~ s/"/&quot;/g;
    $str =~ s/</&lt;/g;
    $str =~ s/>/&gt;/g;
    return $str;
}

sub unescape ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/\&quot;/"/g;
    $str =~ s/\&lt;/</g;
    $str =~ s/\&gt;/>/g;
    $str =~ s/\&amp;/&/gs;
    return $str;
}

1;
