package ImageMenu;
require CMONOS::CGI_Lib::Field;
@ISA = qw(Field);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# フィールド定義モジュール（画像メニュー）
# ImageSelector.pm 1.1.9
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-04-10
######################################################################
sub _set ($$) {
    my ( $this, $setting ) = @_;
    unless ( ref $cgi::Image ) {
        require CMONOS::CGI_Lib::Image;
        $cgi::Image = Image->new;
    }

    #複数の要素を登録
    $this->{'elements'} = 1;
    return;
}

#	(0)-8 obj->align;
sub align ($) {
    my $this = shift;
    return 'center';
}

# --------------------------------------------------------------------
#	(0)-5 データ要素の取得
sub elements ($$) {
    my ( $this, $str ) = @_;
    return () if !defined $str;
    my (%data);
    while ( $str =~ s/^([0-9a-zA-Z_\.\,\-\+\/]+)\:([^\x0A\x0D]*)[\x0A]*//s ) {
        @{ $data{$1} } = split( /,/, $2 ) if defined $2 && $2 ne '';
    }
    return %data;
}

#---------------------------------------------------------------------
# データ整形
# データベース記録前処理
sub data ($;@) {
    my ( $this, @ignore ) = @_;
    $main::data{ $this->{'id'} }{'str'}
        = $this->hash_to_data( $main::data{ $this->{'id'} } );
}

sub hash_to_data ($;@) {
    my ( $this, $hash ) = @_;
    my $str = '';
    while ( my ( $k, $v ) = each %$hash ) {
        next if $k eq 'str';
        $str .= $k . ':' . join( ",", @$v ) . "\x0A";
    }
    return $str;
}

######################################################################
#	(1)-0 obj->match_enabled;
sub match_enabled ($) {
    my $this = shift;
    return (
               defined $this->{'dir'}
            && $this->{'dir'} ne ''
            && (
            (   ref $main::search{ $this->{'id'} }{'images'} > 0
                && @{ $main::search{ $this->{'id'} }{'images'} } > 0
            )
            || ( defined $main::search{ $this->{'id'} }{'images'}
                && $main::search{ $this->{'id'} }{'images'} ne '' )
            )
    ) ? 1 : 0;
}

#	(1) obj->match(str);
sub match ($;$) {
    my ( $this, $str ) = @_;
    return 1
        unless @{ $main::search{ $this->{'id'} }{'images'} } > 0
        || $main::search{ $this->{'id'} }{'images'} ne '';
    $this->scalar_to_array( \$main::search{ $this->{'id'} }{'images'} );

    my ( %data, $img );

    #データを分解
    %data = $this->elements($$str);

    foreach $img ( @{ $main::search{ $this->{'id'} }{'images'} } ) {
        if ( grep( $_ eq $img, @{ $data{'images'} } ) ) {
            return 1
                if !defined $main::search{ $this->{'id'} }{'mode'}
                || $main::search{ $this->{'id'} }{'mode'} ne 'and';
        }
        elsif ( defined $main::search{ $this->{'id'} }{'mode'}
            && $main::search{ $this->{'id'} }{'mode'} eq 'and' )
        {
            return 0;
        }
    }
    if ( defined $main::search{ $this->{'id'} }{'mode'}
        && $main::search{ $this->{'id'} }{'mode'} eq 'and' )
    {
        return 1;
    }
    else {
        return 0;
    }
}

######################################################################
#	(2) obj->form;
sub input_enabled ($) {
    my $this = shift;
    return ( defined $this->{'dir'} && $this->{'dir'} ne '' ) ? 1 : 0;
}

sub form {
    my ( $this, $status, $original, $path ) = @_;
    my ( $form, $sample_id );
    return '' if !defined $this->{'dir'} || $this->{'dir'} eq '';
    if ( !ref $main::data{ $this->{'id'} }{'images'} ) {
        if ( $main::data{ $this->{'id'} }{'images'} ) {
            $this->scalar_to_array( \$main::data{ $this->{'id'} }{'images'} );
            $this->scalar_to_array( \$main::data{ $this->{'id'} }{'width'} );
            $this->scalar_to_array( \$main::data{ $this->{'id'} }{'height'} );
        }
        elsif ( $main::data{ $this->{'id'} }{'str'} ) {
            %{ $main::data{ $this->{'id'} } }
                = $this->elements( $main::data{ $this->{'id'} }{'str'} );
        }
    }

    #フィールドネーム+ヘッダ
    $form = $this->field_header;

    #画像選択フォーム
    $sample_id = 'sample' . $this->{'id'}
        if !defined $this->{'multiple'} || !$this->{'multiple'};

    #現在の設定
    #複数選択
    if ( defined $this->{'multiple'} && $this->{'multiple'} ) {
        if ( ref $main::data{ $this->{'id'} }{'images'} eq 'ARRAY'
            && $main::data{ $this->{'id'} }{'images'}[0] )
        {
            $form
                .= '<div class="field-image-sample">'
                . $this->get_images( $main::data{ $this->{'id'} }, 1 )
                . '</div>';
        }
    }
    else {
        my $path;
        if (   ref $main::data{ $this->{'id'} }{'images'} eq 'ARRAY'
            && defined $main::data{ $this->{'id'} }{'images'}[0]
            && $main::data{ $this->{'id'} }{'images'}[0] ne ''
            && -f $main::data{ $this->{'id'} }{'images'}[0] )
        {
            $path = $main::data{ $this->{'id'} }{'images'}[0];
        }
        else {
            $path = $env::spacer;
        }
        $form
            .= '<div class="field-image-sample"><img id="'
            . $sample_id
            . '" name="'
            . $sample_id
            . '" src="'
            . $path
            . '" alt="" /></div>';
    }

    #画像選択フォーム
    $form .= $this->get_image_menu(
        'data:' . $this->{'id'} . ':images',
        $this->{'multiple'},
        $this->{'required'},
        $main::data{ $this->{'id'} }{'images'},
        0,
        $sample_id,
        $this->{'is_invalid_field'}{'images'},
        $this->{'required'}
    );
    $form .= $this->field_footer;
    return $form;
}

#画像選択フォーム
sub get_image_menu ($$;$$$$$$) {
    my ($this,          $name,             $multiple_flag,
        $required_flag, $default,          $nocheck_flag,
        $sample_id,     $is_invalid_field, $required
    ) = @_;
    return '' if !defined $this->{'dir'} || $this->{'dir'} eq '';
    my ( $form, $dir, $file, %image );
    local ( *DIR, *NAME );
    $dir = $env::image_dir . '/imagemenu/' . $this->{'dir'};
    return '' unless opendir( DIR, $dir );
    $dir .= '/';
    while ( $file = readdir(DIR) ) {

        if (   $file =~ /\.(gif|jpg|png)$/i
            && $file !~ /^\./
            && -f $dir . $file )
        {
            $image{$file} = 1;
        }
    }
    closedir(DIR);
    my $script
        = ($is_invalid_field) ? 'cmonos.validate.field(this).bailOut()' : '';
    $form
        = '<select name="'
        . $name
        . '" id="'
        . $name . '"'
        . ( ($is_invalid_field) ? ' class="invalid-field"' : '' )
        . (
        ($multiple_flag) ? ' multiple="multiple"'
        : ( ($sample_id)
            ? ' onchange="cmonos.element.item(\''
                . $sample_id
                . '\').setSrc(this.options[this.selectedIndex].value);'
                . $script . '"'
            : '' )
        )
        . ( ($is_invalid_field) ? ' onchange="' . $script . '"' : '' )
        . ( ($required)         ? ' required="required"'        : '' ) . '>';
    $form .= '<option value="' . $env::spacer . '"></option>'
        if !$multiple_flag;
    if ( open( NAME, $dir . 'name.txt' ) ) {
        binmode NAME, ":utf8";
        flock( NAME, LOCK_SH );
        while ( my $line = <NAME> ) {
            chomp $line;
            next if $line !~ /:/;
            my ( $img, $name, $reserved ) = split( /:/, $line, 3 );
            next if !defined $image{$img} || !$image{$img};
            delete $image{$img};
            my $path = $dir . $img;
            if ( !$nocheck_flag && $reserved ) {
                if ( $this->is_secure('tools') ) {
                    $name = '*' . $name;
                }
                else {
                    next;
                }
            }
            $form
                .= '<option value="'
                . $path . '"'
                . (
                ( ref $default eq 'ARRAY' && grep( $_ eq $path, @$default ) )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $name
                . '</option>';
        }
        close(NAME);
    }
    foreach my $img ( sort keys %image ) {
        my $path = $dir . $img;
        $form
            .= '<option value="'
            . $path . '"'
            . ( ( ref $default eq 'ARRAY' && grep( $_ eq $path, @$default ) )
            ? ' selected="selected"'
            : '' )
            . '>'
            . $img
            . '</option>';
    }
    $form .= '</select>';
    return $form;
}

######################################################################
#	(3) obj->is_valid_data(path);
sub is_valid_data ($;$) {
    my $this = shift;
    my ( $dir, $is_valid, $is_invalid_image, $image, @images );
    $dir = $env::image_dir . '/imagemenu/' . $this->{'dir'}
        if defined $this->{'dir'} && $this->{'dir'} ne '';
    $is_valid = 1;
    if ( !defined $this->{'dir'} || $this->{'dir'} eq '' || !-d $dir ) {
        push(
            @{ $cgi::action->{'dialog'} },
            'フィールド設定に不足があります。「'
                . ( $this->name || '画像メニュー' )
                . '」フィールドの「詳細設定」を開き、「画像フォルダ」を指定してください。'
        ) if ref $cgi::action;
        return 0;
    }
    if ( ref $main::data{ $this->{'id'} }{'images'} eq 'ARRAY' ) {
        @images = @{ $main::data{ $this->{'id'} }{'images'} };
        undef @{ $main::data{ $this->{'id'} }{'images'} };
    }
    else {
        push( @images, $main::data{ $this->{'id'} }{'images'} )
            if defined $main::data{ $this->{'id'} }{'images'}
            && $main::data{ $this->{'id'} }{'images'} ne '';
        undef $main::data{ $this->{'id'} }{'images'};
    }
    %{ $main::data{ $this->{'id'} } } = ();
    foreach $image (@images) {
        my ( $width, $height, $ext );
        if ( $image eq '' || $image eq $env::spacer ) {
            next;
        }
        elsif ( $this->is_master_image_disabled($image) ) {
            $is_invalid_image = 1;
            next;
        }
        elsif ($image =~ /^$dir/
            && -f $image
            && $image =~ /\.(gif|jpg|png)$/i )
        {
            ( $width, $height, $ext ) = $cgi::Image->info($image);
            $is_invalid_image = 1 if !defined $ext || !$ext;
        }
        else {
            $is_invalid_image = 1;
            next;
        }
        push( @{ $main::data{ $this->{'id'} }{'images'} }, $image );
        push( @{ $main::data{ $this->{'id'} }{'width'} },  $width );
        push( @{ $main::data{ $this->{'id'} }{'height'} }, $height );
    }
    if ( defined $is_invalid_image && $is_invalid_image ) {
        push(
            @{ $cgi::action->{'dialog'} },
            ( ( $this->{'name'} ne '' ) ? $this->{'name'} . 'で' : '' )
                . '指定された画像がみつかりませんでした。'
        ) if ref $cgi::action;
        $this->{'is_invalid_field'}{'images'} = 1;
        $is_valid = 0;
    }
    if (   !ref $main::data{ $this->{'id'} }{'images'}
        || !@{ $main::data{ $this->{'id'} }{'images'} } )
    {
        if ( defined $this->{'required'} && $this->{'required'} ) {
            push(
                @{ $cgi::action->{'dialog'} },
                (   ( $this->{'name'} ne '' )
                    ? $this->{'name'} . 'が'
                    : '画像が'
                    )
                    . '指定されていません。'
            ) if ref $cgi::action;
            $this->{'is_invalid_field'}{'images'} = 1;
            $is_valid = 0;
        }
        else {
            @{ $main::data{ $this->{'id'} }{'images'} } = ('');
            @{ $main::data{ $this->{'id'} }{'width'} }  = ('');
            @{ $main::data{ $this->{'id'} }{'height'} } = ('');
            $main::data{ $this->{'id'} }{'str'} = '';
        }
    }
    elsif ( defined $is_valid && $is_valid ) {
        $this->data;
        $cookie::data{ 'data:' . $this->{'id'} . ':str' }
            = $main::data{ $this->{'id'} }{'str'}
            if $this->{'cookie'} && $main::set_cookie;
    }

    return $is_valid;
}

sub is_master_image_disabled ($$) {
    my ( $this, $path ) = @_;
    $this->set_image_info_hash;
    my $img = ( $path =~ /([^\/]+)$/ ) ? $1 : '';
    return ( $this->{'image'}{$img}{'reserved'}
            && !$this->is_secure('tools') ) ? 1 : 0;
}

######################################################################
#	(4) obj->usual_format(str,path,category,no);
#	str=表示するデータ/path=ファイル保存ディレクトリ
sub usual_format_enabled ($) {
    my $this = shift;
    return ( defined $this->{'dir'} && $this->{'dir'} ne '' ) ? 1 : 0;
}

sub usual_format {
    my ( $this, $str, $path ) = @_;
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return $this->field_tag(
        $this->data_tag_from_hash( 'str',
            ( ( ref $str ) ? $this->hash_to_data( \%data ) : $str ) )
            . $this->get_images( \%data )
    );
}

sub image_file_url ($$$$) {
    my ( $this, $str, $path, $c, $n ) = @_;
    return '' if !$str;
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return ( ref $data{'image'} eq 'ARRAY' && $data{'image'}[0] )
        ? $this->full_path( $data{'image'}[0] )
        : '';
}

sub has_content {
    my ( $this, $str ) = @_;
    return 0 if !$this->usual_format_enabled || !defined $str || $str eq '';
    my %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return (   %data
            && defined $data{'images'}
            && ref $data{'images'}
            && @{ $data{'images'} } ) ? 1 : 0;
}

sub get_images ($$;$) {
    my ( $this, $data, $master_image_check ) = @_;
    return '' if ref $data->{'images'} ne 'ARRAY';
    my ( $img, $n, $path );
    $n = 0;
    foreach $path ( @{ $data->{'images'} } ) {
        next if $path !~ /\.(gif|jpg|png)$/;
        next if $master_image_check && $this->is_master_image_disabled($path);
        $img .= $cgi::Image->image_tag(
            $path,
            $data->{'width'}[$n],
            $data->{'height'}[$n],
            $this->{'border'},
            (   ( $this->{'getName'} )
                ? $this->get_image_name($path)
                : $this->name
            ),
            '',
            $this->{'maxwidth'},
            $this->{'maxheight'},
            $cgi::output,
            $cgi::output_option
        );
        $n++;
    }
    return $img;
}

#---------------------------------------------------------------------
#	(4) obj->format_for_mail(str);
sub format_for_mail_enabled ($) {
    my $this = shift;
    return ( defined $this->{'dir'} && $this->{'dir'} ne '' ) ? 1 : 0;
}

sub format_for_mail {
    my ( $this, $str ) = @_;
    return '' if !defined $str || $str eq '';
    my (%data);
    %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return '' if !ref $data{'images'} || !@{ $data{'images'} };
    return
          '['
        . $this->name . "]\x0A"
        . $this->get_image_name( $data{'images'} )
        . "\x0A\x0A";
}

#---------------------------------------------------------------------
#	(4)-4 obj->format_for_text(str);
sub format_for_text_enabled ($) {
    my $this = shift;
    return ( defined $this->{'dir'} && $this->{'dir'} ne '' ) ? 1 : 0;
}

sub format_for_text {
    my ( $this, $str ) = @_;
    return '' if !defined $str || $str eq '';
    my (%data);
    %data
        = ( ref $str eq 'HASH' )
        ? ( defined $$str{'str'} )
            ? $this->elements( $$str{'str'} )
            : %$str
        : $this->elements($str);
    return '' if !ref $data{'images'} || !@{ $data{'images'} };
    return $this->get_image_name( $data{'images'} );
}

sub format_for_detail_enabled ($) {
    my $this = shift;
    return ( defined $this->{'dir'} && $this->{'dir'} ne '' ) ? 1 : 0;
}

sub get_image_name ($$) {
    my ( $this, $data ) = @_;
    $this->set_image_info_hash;
    if ( ref $data eq 'ARRAY' ) {
        my $imgs = '';
        foreach my $img (@$data) {
            if ( $img =~ /([^\/]+)$/ ) {
                $img = $1;
            }
            else {
                next;
            }
            $imgs .= ',' if $imgs;
            $imgs
                .= ( $this->{'image'}{$img}{'name'} )
                ? $this->{'image'}{$img}{'name'}
                : $img;
        }
        return $imgs;
    }
    elsif ( $data =~ /([^\/]+)$/ ) {
        my $img = $1;
        return ( $this->{'image'}{$img}{'name'} )
            ? $this->{'image'}{$img}{'name'}
            : $img;
    }
    return $this->name;
}

sub set_image_info_hash ($) {
    my $this = shift;
    return
           if ref $this->{'image'}
        || !defined $this->{'dir'}
        || $this->{'dir'} eq '';
    local (*NAME);
    if (open( NAME,
                  $env::image_dir
                . '/imagemenu/'
                . $this->{'dir'} . '/'
                . 'name.txt'
        )
        )
    {
        binmode NAME, ":utf8";
        flock( NAME, LOCK_SH );
        while ( my $line = <NAME> ) {
            next if $line !~ /:/;
            chomp $line;
            my ( $img, $name, $reserved ) = split( /:/, $line, 3 );
            if ( defined $img && $img ne '' ) {
                $this->{'image'}{$img}{'name'}     = $name;
                $this->{'image'}{$img}{'reserved'} = $reserved;
            }
        }
        close(NAME);
    }
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my $this = shift;

    #名前
    my $name = 'data:' . $this->{'id'};

    $this->print_field_info_form($name);

    print '<fieldset>';
    print '<legend>画像表示設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_maxwidth">最大横幅</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_maxwidth" id="'
        . $name
        . '_maxwidth" value="'
        . $this->escape( $this->{'maxwidth'} )
        . '" size="3" /> <span class="unit">ピクセル</span></td>';
    print '<td class="field-name"><label for="' . $name
        . '_maxheight">最大高さ</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_maxheight" id="'
        . $name
        . '_maxheight" value="'
        . $this->escape( $this->{'maxheight'} )
        . '" size="3" /> <span class="unit">ピクセル</span></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_border">枠線幅</label></td>';
    print '<td class="field" colspan="3"><input type="text" name="'
        . $name
        . '_border" id="'
        . $name
        . '_border" value="'
        . $this->escape( $this->{'border'} )
        . '" size="3" /> <span class="unit">ピクセル</span></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">代替テキスト</span></td>';
    print '<td class="field" colspan="3"><label><input type="checkbox" name="'
        . $name
        . '_getName" id="'
        . $name
        . '_getName" value="1"'
        . ( ( defined $this->{'getName'} && $this->{'getName'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 選択画像表示時に画像の名前を取得する</label></td>';
    print '</tr>';
    print '</tbody></table>';
    print '</fieldset>';
    print '<fieldset>';
    print '<legend>入力欄設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_dir">画像フォルダ</label></td>';
    print '<td class="field">';
    print '<select name="' . $name . '_dir" id="' . $name . '_dir">';

    local ( *DIR, *NAME );
    if ( opendir( DIR, $env::image_dir . '/imagemenu' ) ) {
        my @dir
            = grep( $_ !~ /^\./ && -d $env::image_dir . '/imagemenu/' . $_,
            readdir(DIR) );
        closedir(DIR);
        my ($dir_name);
        foreach $dir_name (@dir) {
            my ($name);
            if (open( NAME,
                          $env::image_dir
                        . '/imagemenu/'
                        . $dir_name . '/'
                        . 'name.txt'
                )
                )
            {
                binmode NAME, ":utf8";
                flock( NAME, LOCK_SH );
                $name = <NAME>;
                close(NAME);
                chomp $name;
            }
            $name = $dir_name
                if !defined $name || $name eq '' || $name =~ /:/;
            print '<option value="'
                . $dir_name . '"'
                . ( ( defined $this->{'dir'} && $dir_name eq $this->{'dir'} )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $name
                . '</option>';
        }
    }

    print '</select></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_multiple">複数選択可</label></td>';
    print '<td class="field"><input type="checkbox" name="'
        . $name
        . '_multiple" id="'
        . $name
        . '_multiple" value="1"'
        . ( ( defined $this->{'multiple'} && $this->{'multiple'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 画像を複数選択できるようにする</td>';
    print '</tr>';

    print '</tbody></table></fieldset>';
    $this->print_setting_save_button;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this     = shift;
    my $is_valid = 1;

    #チェックボックス
    $main::data{ $this->{'id'} . '_getName' } = ''
        if !defined $main::data{ $this->{'id'} . '_getName' }
        || $main::data{ $this->{'id'} . '_getName' } eq '';
    $main::data{ $this->{'id'} . '_multiple' } = ''
        if !defined $main::data{ $this->{'id'} . '_multiple' }
        || $main::data{ $this->{'id'} . '_multiple' } eq '';

    #数字チェック
    if (   $main::data{ $this->{'id'} . '_maxwidth' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_maxwidth' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '最大横幅を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_maxwidth' };
        $is_valid = 0;
    }
    if (   $main::data{ $this->{'id'} . '_maxheight' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_maxheight' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '最大高さを更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_maxheight' };
        $is_valid = 0;
    }

    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };
    return $is_valid;
}

######################################################################
#	(6) obj->search_form;
sub search_form ($) {
    my $this = shift;
    my ($form);
    $this->scalar_to_array( \$main::search{ $this->{'id'} }{'images'} );
    $form
        = '<tr><td class="field-name"><label for="search:'
        . $this->{'id'}
        . ':images">'
        . $this->{'name'}
        . '</label></td><td class="field">';
    $form .= $this->get_image_menu( 'search:' . $this->{'id'} . ':images',
        1, 0, $main::search{ $this->{'id'} }{'images'}, 1 );

    if ( defined $this->{'multiple'} && $this->{'multiple'} ) {

        #英語表記
        if ( defined $default::english && $default::english ) {
            $form .= $this->mk_select_menu(
                'search:' . $this->{'id'} . ':mode', '', '',
                $main::search{ $this->{'id'} }{'mode'},
                'and' => 'Matches on all selection',
                'or'  => 'Matches on any selection'
            );

            #日本語
        }
        else {
            $form .= $this->mk_select_menu(
                'search:' . $this->{'id'} . ':mode', '', '',
                $main::search{ $this->{'id'} }{'mode'},
                'and' => '全ての選択肢に一致',
                'or'  => 'いずれかの選択肢に一致'
            );
        }
    }

    $form .= '</td></tr>';

    return $form;
}

######################################################################
#	sample
sub sample ($$) {
    my ( $this, $str ) = @_;
    return '' if !defined $str || $str eq '';
    my %data = $this->elements($str);
    return '' if !ref $data{'images'} || ref $data{'images'} ne 'ARRAY';
    return
        join( ',', map( $this->get_image_name($_), @{ $data{'images'} } ) );
}

######################################################################
#sub import {
BEGIN {
    if ( !$setting::field_for_mail ) {
        Field::class_register( 'select_imagemenu', 'ImageMenu',
            '画像メニュー' );
    }
}

1;
