package NumBarGraph;
require CMONOS::CGI_Lib::Field::Num;
@ISA = qw(Num);
use strict;
use utf8;
######################################################################
# フィールド定義モジュール（棒グラフ）
# NumBarGraph.pm 1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2012-06-17
######################################################################
sub _set ($$) {
    my $this = shift;
    $this->{'cols'} = '8' if !$this->{'cols'};
    unless ( ref $cgi::Image ) {
        require CMONOS::CGI_Lib::Image;
        $cgi::Image = Image->new;
    }
    unless ( ref $cgi::Deco ) {
        require CMONOS::CGI_Lib::Deco;
        $cgi::Deco = Deco->new;
    }
    unless ( ref $cgi::BarGraph ) {
        require CMONOS::CGI_Lib::BarGraph;
        $cgi::BarGraph = BarGraph->new;
    }

    #初期設定
    $this->{'barColor'} = '#6699ff'
        if !defined $this->{'barColor'} || !$this->{'barColor'};
    $this->{'bgColor'} = '#eeeeee'
        if !defined $this->{'bgColor'} || !$this->{'bgColor'};
    $this->{'barWidth'} = 150
        if !defined $this->{'barWidth'} || !$this->{'barWidth'};
    $this->{'max'} = 100 if !defined $this->{'max'} || !$this->{'max'};
    $this->{'unit'} = '%'
        if ( !defined $this->{'unit'} || $this->{'unit'} eq '' )
        && $this->{'max'} == 100;
    delete $this->{'maxwidth'}
        if defined $this->{'maxwidth'} && $this->{'maxwidth'};

    return;
}

######################################################################
#	(4) obj->usual_format(str,path,category,no);
#	str=表示するデータ/path=ファイル保存ディレクトリ
sub usual_format {
    my ( $this, $str, $path ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';

    #グラフ表示
    return $this->field_tag(
        $this->data_tag_from_hash( 'str', $str )
            . $cgi::BarGraph->bargraph(
            $str,
            $this->{'max'},
            $this->{'unit'},
            '',
            $this->{'barColor'},
            $this->{'bgColor'},
            $this->{'barImage'},
            $this->{'bgImage'},
            (   (          $this->{'maxwidth'}
                        && $this->{'maxwidth'} < $this->{'barWidth'}
                ) ? $this->{'maxwidth'} : $this->{'barWidth'}
            )
            )
    );
}

sub has_content {
    my $this = shift;
    return 1;
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my ( $this, $html_disabled ) = @_;
    my ($name);

    #名前
    $name = 'data:' . $this->{'id'};
    $this->{'default'} =~ s/\x0A//sg if defined $this->{'default'};

    $this->print_field_info_form($name);

    print '<fieldset>';
    print '<legend>入力欄設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_cols">入力欄幅</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_cols" id="'
        . $name
        . '_cols" value="'
        . $this->escape( $this->{'cols'} )
        . '" size="3" title="入力欄の横幅を設定します。" /> <span class="unit">文字(半角)</span></td>';
    print '<td class="field-name"><label for="' . $name
        . '_maxlength">最大文字数</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_maxlength" id="'
        . $name
        . '_maxlength" value="'
        . $this->escape( $this->{'maxlength'} )
        . '" size="3" title="入力欄の文字数を制限します。" /> <span class="unit">文字(半角)</span></td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_unit">単位</label></td>';
    print '<td class="q-field"><input type="text" name="'
        . $name
        . '_unit" id="'
        . $name
        . '_unit" value="'
        . $this->escape( $this->{'unit'} )
        . '" size="8" title="入力する内容に適した単位を設定します。" /> <label><input type="checkbox" name="'
        . $name
        . '_int" value="1"'
        . ( ( defined $this->{'int'} && $this->{'int'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 整数</label></td>';
    print '<td class="field-name"><label for="' . $name
        . '_max">最大値</label></td>';
    print '<td class="q-field"><input type="number" name="'
        . $name
        . '_max" id="'
        . $name
        . '_max" value="'
        . $this->escape( $this->{'max'} )
        . '" size="16" title="棒グラフが満たされたときの最大値を設定します。" />'
        . ( ( defined $this->{'unit'} )
        ? ' <span class="unit">' . $this->{'unit'} . '</span>'
        : '' )
        . '</td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_default">デフォルト</label></td>';
    print '<td class="field" colspan="3"><input type="text" name="'
        . $name
        . '_default" id="'
        . $name
        . '_default" value="'
        . $this->escape( $this->{'default'} )
        . '" size="'
        . ( ( $this->{'cols'} ) ? $this->{'cols'} : '8' )
        . '" title="入力欄のデフォルトの値を設定します。" />'
        . ( ( defined $this->{'unit'} )
        ? ' <span class="unit">' . $this->{'unit'} . '</span>'
        : '' )
        . '</td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_placeholder">プレースホルダ</label></td>';
    print '<td class="field" colspan="3"><input type="text" name="'
        . $name
        . '_placeholder" id="'
        . $name
        . '_placeholder" size="'
        . ( ( $this->{'cols'} ) ? $this->{'cols'} : '72' )
        . '" title="入力欄の placeholder 属性の値を設定します。" value="'
        . $this->escape( $this->{'placeholder'} )
        . '" /></td>';
    print '</tr>';
    print '</tbody></table>';
    print '</fieldset>';

    #棒グラフ設定
    $cgi::BarGraph->print_bargraph_setting_form( $this->{'id'}, '', '', 0,
        $this );

    if ( !$default::static && !$html_disabled ) {
        print '<fieldset>';
        print '<legend>検索設定</legend>';
        print '<label><input type="checkbox" name="'
            . $name
            . '_searchOption" value="1"'
            . ( ( defined $this->{'searchOption'} && $this->{'searchOption'} )
            ? ' checked="checked"'
            : '' )
            . ' /> より詳しい検索</label>';
        print '</fieldset>';
    }

    $this->print_setting_save_button;
    return;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this = shift;
    $main::data{ $this->{'id'} . '_int' } = 0
        if !$main::data{ $this->{'id'} . '_int' };

    #棒グラフ設定チェック
    my $is_valid = $cgi::BarGraph->is_valid_bargraph_setting( $this->{'id'} );

    #タグの除去
    if ( defined $main::data{ $this->{'id'} . '_default' }
        && $main::data{ $this->{'id'} . '_default' } )
    {
        $main::data{ $this->{'id'} . '_default' } =~ tr/\x0A\x0D//;
        $this->detag( \$main::data{ $this->{'id'} . '_default' } );
    }
    $main::data{ $this->{'id'} . '_placeholder' } =~ s/[\x0A\x0D]+//sg
        if defined $main::data{ $this->{'id'} . '_placeholder' };
    if (   defined $main::data{ $this->{'id'} . '_default' }
        && $main::data{ $this->{'id'} . '_default' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_default' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'デフォルトを更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_default' };
        $is_valid = 0;
    }
    elsif ($main::data{ $this->{'id'} . '_int' }
        && $main::data{ $this->{'id'} . '_default' }
        && $main::data{ $this->{'id'} . '_default' } =~ /\./ )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'デフォルト設定を確認してください。小数点以下を四捨五入しました。'
        ) if ref $cgi::action;
        $main::data{ $this->{'id'} . '_default' }
            = int( $main::data{ $this->{'id'} . '_default' } + 0.5 );
    }
    if (   defined $main::data{ $this->{'id'} . '_max' }
        && $main::data{ $this->{'id'} . '_max' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_max' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '最大値設定を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_max' };
        $is_valid = 0;
    }
    elsif ($main::data{ $this->{'id'} . '_int' }
        && $main::data{ $this->{'id'} . '_max' }
        && $main::data{ $this->{'id'} . '_max' } =~ /\./ )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            'デフォルト設定を確認してください。小数点以下を四捨五入しました。'
        ) if ref $cgi::action;
        $main::data{ $this->{'id'} . '_max' }
            = int( $main::data{ $this->{'id'} . '_max' } + 0.5 );
    }

    $this->tag_fix( \$main::data{ $this->{'id'} . '_unit' }, 2 )
        if defined $main::data{ $this->{'id'} . '_unit' }
        && $main::data{ $this->{'id'} . '_unit' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };
    $main::data{ $this->{'id'} . '_searchOption' } = ''
        if !defined $main::data{ $this->{'id'} . '_searchOption' }
        || $main::data{ $this->{'id'} . '_searchOption' } eq '';

    return $is_valid;
}

######################################################################
#sub import {
BEGIN {
    if ( !$setting::field_for_mail ) {
        Field::class_register( 'num_bargraph', 'NumBarGraph',
            '棒グラフ' );
        $setting::deco_dir{'bargraph'}    = '棒グラフ画像';
        $setting::deco_dir{'bargraphbg'}  = '棒グラフ枠画像';
        $setting::deco_type{'bargraph'}   = 'image';
        $setting::deco_type{'bargraphbg'} = 'image';
    }
}

1;
