package WeeklySchedule;
require CMONOS::CGI_Lib::Field;
@ISA = qw(Field);
use strict;
use utf8;
use Time::Local;
######################################################################
# フィールド定義モジュール（週間予定表）
# WeeklySchedule.pm 1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-04-10
######################################################################
sub _set ($$) {
    my ( $this, $setting ) = @_;
    $this->{'lang'} = (
        (          $setting::general{'datetype'}
                && $setting::general{'datetype'} =~ /e/
        ) ? 'en' : 'ja'
    ) if !$this->{'lang'};
    $this->{'short'} = ( ( $this->{'lang'} eq 'ja' ) ? 1 : 0 )
        if !$this->{'short'};
    $this->{'hourStart'} = 0  if !$this->{'hourStart'};
    $this->{'hourEnd'}   = 24 if !$this->{'hourEnd'};
    return;
}

sub mk_table_object ($$) {
    my ( $this, $id ) = @_;
    my $table;
    if ($id) {
        if ( ref $table::obj{$id} ) {
            $table = $table::obj{$id};
        }
        elsif ( ref $setting::table{$id} ) {
            $this->use_table;
            $table = $table::obj{$id};
        }
    }
    if ( !ref $table ) {
        $table = Table->new;
        $table->set(
            'tableclass' => 'datatable',
            'trclass'    => 'odd,even',
            'class0'     => 'odd,even',
        );
    }
    return $table;
}

######################################################################
#	(1) obj->match(str);
sub match_enabled ($) {
    my $this = shift;
    if ( ref $this->{'pattern'} && @{ $this->{'pattern'} } ) {
        return 1;
    }
    else {
        my $match_enabled = 0;
        foreach my $w ( 'su', 'mo', 'tu', 'we', 'th', 'fr', 'sa' ) {
            return 1
                if ( defined $main::search{ $this->{'id'} }{ $w . 'fh' }
                && $main::search{ $this->{'id'} }{ $w . 'fh' } ne '' )
                || ( defined $main::search{ $this->{'id'} }{ $w . 'th' }
                && $main::search{ $this->{'id'} }{ $w . 'th' } ne '' )
                || ( defined $main::search{ $this->{'id'} }{$w}
                && $main::search{ $this->{'id'} }{$w} ne '' );
        }
    }
    return 0;
}

sub match ($;$) {
    no warnings qw(numeric);
    my ( $this, $str ) = @_;
    return 0 if !ref $str || $$str eq '';
    $this->get_search_patterns if !ref $this->{'pattern'};
    return 1 if !@{ $this->{'pattern'} };
    my @timetable = split( /\x0A/, $$str );
    foreach my $pats ( @{ $this->{'pattern'} } ) {
        my $t = shift @timetable || '';
        my ( $from, $to )
            = ( $t !~ / - / ) ? ( $t, $t ) : split( / \- /, $t );
        $from = $this->str_to_hour($from);
        $to   = $this->str_to_hour($to);
        foreach my $p (@$pats) {
            next if !defined $p || $p eq '';
            if ( $p =~ /^(.*?)\-(.*)$/ ) {
                my ( $f, $t ) = ( $1, $2 );
                if ((   (      !defined $f
                            || $f eq ''
                            || ( defined $from && $from ne '' && $f <= $from )
                        )
                        && (   !defined $t
                            || $t eq ''
                            || ( defined $to && $to ne '' && $to <= $t ) )
                    )
                    || (!$main::search{ $this->{'id'} }{'o'}
                        && ((   (      !defined $f
                                    || $f eq ''
                                    || (   defined $from
                                        && $from ne ''
                                        && $f <= $from )
                                )
                                && (   !defined $t
                                    || $t eq ''
                                    || (   defined $from
                                        && $from ne ''
                                        && $from < $t )
                                )
                            )
                            || ((      !defined $f
                                    || $f eq ''
                                    || (   defined $to
                                        && $to ne ''
                                        && $f < $to )
                                )
                                && (   !defined $t
                                    || $t eq ''
                                    || (   defined $to
                                        && $to ne ''
                                        && $to <= $t ) )
                            )
                            || ((      defined $f
                                    && $f ne ''
                                    && defined $from
                                    && $from ne ''
                                    && $from <= $f
                                )
                                && (   defined $t
                                    && $t ne ''
                                    && defined $to
                                    && $to ne ''
                                    && $t <= $to )
                            )
                        )
                    )
                    )
                {
                    return 1
                        if !defined $main::search{ $this->{'id'} }{'mode'}
                        || $main::search{ $this->{'id'} }{'mode'} ne 'and';
                }
                elsif ( defined $main::search{ $this->{'id'} }{'mode'}
                    && $main::search{ $this->{'id'} }{'mode'} eq 'and' )
                {
                    return 0;
                }
            }
            elsif ($p ne ''
                && defined $from
                && $from ne ''
                && $from <= $p
                && defined $to
                && $to ne ''
                && $p <= $to )
            {
                return 1
                    if !defined $main::search{ $this->{'id'} }{'mode'}
                    || $main::search{ $this->{'id'} }{'mode'} ne 'and';
            }
            elsif ( defined $main::search{ $this->{'id'} }{'mode'}
                && $main::search{ $this->{'id'} }{'mode'} eq 'and' )
            {
                return 0;
            }
        }
    }
    if ( defined $main::search{ $this->{'id'} }{'mode'}
        && $main::search{ $this->{'id'} }{'mode'} eq 'and' )
    {
        return 1;
    }
    else {
        return 0;
    }
}

sub str_to_hour ($$) {
    my ( $this, $str ) = @_;
    return
          ( !defined $str || $str eq '' ) ? ''
        : ( $str =~ /(\d{1,2})\s*[:h時]\s*(\d{1,2})/ ) ? $1 * 100 + $2
        : ( $str =~ /(\d{3,})/ )                        ? $1
        : ( $str =~ /(\d{1,2})/ )                       ? $1 * 100
        :                                                 '';
}

sub get_search_patterns ($) {
    my $this = shift;
    @{ $this->{'pattern'} } = ();
    my $has_pattern = 0;
    foreach my $w ( 'su', 'mo', 'tu', 'we', 'th', 'fr', 'sa' ) {
        my $pat = [];
        undef $main::search{ $this->{'id'} }{ $w . 'fm' }
            if !defined $main::search{ $this->{'id'} }{ $w . 'fh' }
            || $main::search{ $this->{'id'} }{ $w . 'fh' } eq '';
        undef $main::search{ $this->{'id'} }{ $w . 'tm' }
            if !defined $main::search{ $this->{'id'} }{ $w . 'th' }
            || $main::search{ $this->{'id'} }{ $w . 'th' } eq '';
        my ( $fh, $fm, $th, $tm, $q ) = (
            $main::search{ $this->{'id'} }{ $w . 'fh' },
            $main::search{ $this->{'id'} }{ $w . 'fm' },
            $main::search{ $this->{'id'} }{ $w . 'th' },
            $main::search{ $this->{'id'} }{ $w . 'tm' },
            $main::search{ $this->{'id'} }{$w}
        );
        if ( ( defined $fh && $fh ne '' ) || ( defined $th && $th ne '' ) ) {
            push(
                @$pat,
                (   ( defined $fh && !$this->is_nan($fh) )
                    ? $fh * 100
                        + ( ( defined $fm && !$this->is_nan($fm) ) ? $fm : 0 )
                    : ''
                    )
                    . '-'
                    . (
                    ( defined $th && !$this->is_nan($th) )
                    ? $th * 100
                        + ( ( defined $tm && !$this->is_nan($tm) ) ? $tm : 0 )
                    : ''
                    )
            );
        }
        else {
            undef $main::search{ $this->{'id'} }{ $w . 'fh' };
            undef $main::search{ $this->{'id'} }{ $w . 'fm' };
            undef $main::search{ $this->{'id'} }{ $w . 'th' };
            undef $main::search{ $this->{'id'} }{ $w . 'tm' };
        }
        if ( defined $q ) {
            $q =~ s/\s*[\-~ー〜]+\s*/-/sg;
            $q = $this->z_to_h($q);
            foreach my $p ( split( /[\s,]+/, $q ) ) {
                push(
                    @$pat,
                    (   ( $p =~ /^(.*?)\-(.*)$/ )
                        ? $this->str_to_hour($1) . '-'
                            . $this->str_to_hour($2)
                        : $p
                    )
                ) if $p ne '';
            }
        }
        $has_pattern = 1 if @$pat;
        push( @{ $this->{'pattern'} }, $pat );
    }
    $this->{'pattern'} = [] if !$has_pattern;
}

######################################################################
#	(2) obj->form;
sub form {
    my ( $this, $status ) = @_;
    my ($form);

    #フィールドネーム+ヘッダ
    $form = $this->field_header;

    #デフォルト作成
    if (   ( !defined $status || $status eq '' )
        && !defined $main::data{ $this->{'id'} }{'str'}
        && !$main::data{ $this->{'id'} }{'su'}
        && !$main::data{ $this->{'id'} }{'mo'}
        && !$main::data{ $this->{'id'} }{'tu'}
        && !$main::data{ $this->{'id'} }{'we'}
        && !$main::data{ $this->{'id'} }{'th'}
        && !$main::data{ $this->{'id'} }{'fr'}
        && !$main::data{ $this->{'id'} }{'sa'} )
    {
        $main::data{ $this->{'id'} }{'su'} = $this->{'defaultsu'};
        $main::data{ $this->{'id'} }{'mo'} = $this->{'defaultmo'};
        $main::data{ $this->{'id'} }{'tu'} = $this->{'defaulttu'};
        $main::data{ $this->{'id'} }{'we'} = $this->{'defaultwe'};
        $main::data{ $this->{'id'} }{'th'} = $this->{'defaultth'};
        $main::data{ $this->{'id'} }{'fr'} = $this->{'defaultfr'};
        $main::data{ $this->{'id'} }{'sa'} = $this->{'defaultsa'};
    }
    elsif ( $main::data{ $this->{'id'} }{'str'} ) {
        (   $main::data{ $this->{'id'} }{'su'},
            $main::data{ $this->{'id'} }{'mo'},
            $main::data{ $this->{'id'} }{'tu'},
            $main::data{ $this->{'id'} }{'we'},
            $main::data{ $this->{'id'} }{'th'},
            $main::data{ $this->{'id'} }{'fr'},
            $main::data{ $this->{'id'} }{'sa'}
        ) = split( /\x0A/, $main::data{ $this->{'id'} }{'str'} );
    }
    $form .= $this->input_form( 'data', '', '', 1 );
    $form .= $this->field_footer;
    return $form;
}

sub input_form ($$;$$$) {
    my ( $this, $p, $prefix, $setting, $invalidcheck ) = @_;
    no strict "refs";
    $prefix = '' if !defined $prefix;
    my $name
        = $p . ':' . $this->{'id'} . ( ($setting) ? '_' : ':' ) . $prefix;
    my $table     = $this->mk_table_object( $this->{'table'} );
    my $form      = $table->table;
    my @week_name = $this->week_name;
    my $searching = ( !$setting && $p eq 'search' ) ? 1 : 0;
    foreach my $w ( 'su', 'mo', 'tu', 'we', 'th', 'fr', 'sa' ) {
        $form .= $table->tr;
        $form .= $table->th( shift @week_name );
        if ( $searching && !$this->{'searchOption'} ) {
            my $menu
                = '<label><select name="'
                . $name
                . $w
                . 'fh"><option value=""></option>';
            my $fh
                = ( ($setting)
                ? $this->{ $prefix . 'fh' }
                : ${ 'main::' . $p }{ $this->{'id'} }{ $prefix . $w . 'fh' } )
                || '';
            my $hour_end
                = ( $this->{'minScale'} && $this->{'minScale'} eq 'none' )
                ? $this->{'hourEnd'} + 1
                : $this->{'hourEnd'};
            foreach my $h ( $this->{'hourStart'} .. $hour_end ) {
                $menu
                    .= '<option value="'
                    . $h . '"'
                    . ( ( $fh eq $h ) ? ' selected="selected"' : '' ) . '>'
                    . $h
                    . '</option>';
            }
            $menu .= '</select>';
            $menu .= ($default::english) ? ':' : '時';
            $menu .= '</label>';
            if ( !$this->{'minScale'} || $this->{'minScale'} ne 'none' ) {
                $menu
                    .= '<label><select name="'
                    . $name
                    . $w
                    . 'fm"><option value=""></option>';
                my $fm
                    = ( ($setting)
                    ? $this->{ $prefix . 'fm' }
                    : ${ 'main::' . $p }{ $this->{'id'} }
                        { $prefix . $w . 'fm' } )
                    || '';
                foreach my $m ( 0 .. 59 ) {
                    if (  !$this->{'minScale'}
                        || $m % $this->{'minScale'} == 0 )
                    {
                        $menu
                            .= '<option value="'
                            . $m . '"'
                            . ( ( $fm eq $m ) ? ' selected="selected"' : '' )
                            . '>'
                            . $m
                            . '</option>';
                    }
                }
                $menu .= '</select>';
                $menu .= ($default::english) ? '' : '分';
                $menu .= '</label>';
            }
            $menu .= ($default::english) ? ' - ' : ' 〜 ';
            $menu
                .= '<label><select name="'
                . $name
                . $w
                . 'th"><option value=""></option>';
            my $th
                = ( ($setting)
                ? $this->{ $prefix . 'th' }
                : ${ 'main::' . $p }{ $this->{'id'} }{ $prefix . $w . 'th' } )
                || '';
            foreach my $h ( $this->{'hourStart'} .. $hour_end ) {
                $menu
                    .= '<option value="'
                    . $h . '"'
                    . ( ( $th eq $h ) ? ' selected="selected"' : '' ) . '>'
                    . $h
                    . '</option>';
            }
            $menu .= '</select>';
            $menu .= ($default::english) ? ':' : '時';
            $menu .= '</label>';
            if ( !$this->{'minScale'} || $this->{'minScale'} ne 'none' ) {
                $menu
                    .= '<label><select name="'
                    . $name
                    . $w
                    . 'tm"><option value=""></option>';
                my $tm
                    = ( ($setting)
                    ? $this->{ $prefix . 'tm' }
                    : ${ 'main::' . $p }{ $this->{'id'} }
                        { $prefix . $w . 'tm' } )
                    || '';
                foreach my $m ( 0 .. 59 ) {
                    if (  !$this->{'minScale'}
                        || $m % $this->{'minScale'} == 0 )
                    {
                        $menu
                            .= '<option value="'
                            . $m . '"'
                            . ( ( $tm eq $m ) ? ' selected="selected"' : '' )
                            . '>'
                            . $m
                            . '</option>';
                    }
                }
                $menu .= '</select>';
                $menu .= ($default::english) ? '' : '分';
                $menu .= '</label>';
            }
            $form .= $table->td($menu);
        }
        else {
            $form .= $table->td(
                      '<input type="text" name="'
                    . $name
                    . $w
                    . '" size="40" value="'
                    . $this->escape(
                    ($setting) ? $this->{ $prefix . $w }
                    : ${ 'main::' . $p }{ $this->{'id'} }{ $prefix . $w }
                    )
                    . '" placeholder="hh:mm - hh:mm, hh:mm - hh:mm, ..."'
                    . (
                    ( $invalidcheck && $this->{'is_invalid_field'}{$w} )
                    ? $this->set_invalid_class
                    : ''
                    )
                    . ' />'
            );
        }
        $form .= $table->_tr;
    }
    $form .= $table->_table;
    return $form;
}

sub week_name ($) {
    my $this = shift;
    return
          ( $this->{'lang'} eq 'ja' )
        ? ( $this->{'short'} )
            ? ( '日', '月', '火', '水', '木', '金', '土' )
            : ( '日曜日', '月曜日', '火曜日', '水曜日',
                '木曜日', '金曜日', '土曜日'
            )
        : ( $this->{'short'} )
        ? ( 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat' )
        : (
        'Sunday',   'Monday', 'Tuesday', 'Wednesday',
        'Thursday', 'Friday', 'Saturday'
        );
}

######################################################################
#	(3) obj->is_valid_data(path);
sub is_valid_data ($;$) {
    my $this = shift;
    delete $this->{'is_invalid_field'} if ref $this->{'is_invalid_field'};
    my @week_name     = $this->week_name;
    my $start         = $this->{'hourStart'} * 100;
    my $end           = ( $this->{'hourEnd'} + 1 ) * 100;
    my $str           = '';
    my $is_valid_data = 1;
    foreach my $w ( 'su', 'mo', 'tu', 'we', 'th', 'fr', 'sa' ) {
        my $time = $main::data{ $this->{'id'} }{$w};
        $time = '' if !defined $time;
        $time =~ s/\s*[\-~ー〜]+\s*/-/sg;
        $time = $this->z_to_h($time);
        my $is_invalid       = 0;
        my $is_not_available = 0;
        my $h                = '';
        foreach my $span ( split( /[\s,]+/, $time ) ) {

            if ( $span =~ /^(.*?)\-(.*)$/ ) {
                my ( $f, $t )
                    = ( $this->str_to_hour($1), $this->str_to_hour($2) );
                if ( $f ne '' || $t ne '' ) {
                    ( $f, $t ) = ( $t, $f )
                        if $f ne '' && $t ne '' && $f > $t;
                    $h
                        .= ( ( $h ne '' ) ? ', ' : '' )
                        . $this->num_to_hour($f) . ' - '
                        . $this->num_to_hour($t);
                    $is_not_available = 1 if $f < $start || $t >= $end;
                }
                else {
                    $is_invalid = 1;
                }
            }
            elsif ( $span ne '' ) {
                $span = $this->str_to_hour($span);
                if ( $span ne '' ) {
                    $h .= ( ( $h ne '' ) ? ', ' : '' )
                        . $this->num_to_hour($span);
                    $is_not_available = 1 if $span < $start || $span >= $end;
                }
                else {
                    $is_invalid = 1;
                }
            }
        }
        $main::data{ $this->{'id'} }{$w} = $h;
        $str .= $h . "\x0A";
        my $wname = shift @week_name;
        if ( $is_invalid || $is_not_available ) {
            $this->{'is_invalid_field'}{$w} = 1;
            $is_valid_data = 0;
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '入力' ) . '('
                    . $wname
                    . ')を確認してください。'
                    . $this->{'hourStart'}
                    . '時台から'
                    . $this->{'hourEnd'}
                    . '時台の間を入力してください。'
            ) if ref $cgi::action && $is_not_available;
            push(
                @{ $cgi::action->{'dialog'} },
                ( ( $this->{'name'} ) ? $this->{'name'} : '入力' ) . '('
                    . $wname
                    . ')を確認してください。'
            ) if ref $cgi::action && $is_invalid;
        }
    }
    return 0 if !$is_valid_data;

    $str =~ s/^\s+$//s;
    if ($str) {
        $main::data{ $this->{'id'} }{'str'} = $str;

        #必須項目チェック
    }
    elsif ( $this->{'required'} && !$str ) {
        push(
            @{ $cgi::action->{'dialog'} },
            (   ( defined $this->{'name'} && $this->{'name'} )
                ? $this->{'name'} . 'が'
                : ''
                )
                . '入力されていません。'
        ) if ref $cgi::action;
        foreach my $w ( 'su', 'mo', 'tu', 'we', 'th', 'fr', 'sa' ) {
            $this->{'is_invalid_field'}{$w} = 1
                if !defined $main::data{ $this->{'id'} }{$w}
                || $main::data{ $this->{'id'} }{$w} eq '';
        }
        return 0;
    }

    $cookie::data{ 'data:' . $this->{'id'} . ':str' }
        = $main::data{ $this->{'id'} }{'str'}
        if $this->{'cookie'} && $main::set_cookie;
    return 1;
}

sub num_to_hour ($$) {
    my ( $this, $num ) = @_;
    $num = sprintf( "%.3d", $num ) if $num < 100;
    $num =~ s/(\d{2})$/:$1/;
    return $num;
}

######################################################################
#	(4) obj->usual_format(str);
sub usual_format {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return $this->field_tag('') if !defined $str || $str eq '';
    return $this->field_tag( $this->data_tag_from_hash( 'str', $str )
            . $this->weekly_schedule($str) );
}

sub weekly_schedule ($$) {
    my ( $this, $str ) = @_;
    my $table     = $this->mk_table_object( $this->{'table'} );
    my $schedule  = $table->table;
    my @week_name = $this->week_name;
    my @timetable = split( /\x0A/, $str );
    if ( $this->{'column'} ) {
        $schedule .= $table->tr;
        foreach my $w (@week_name) {
            $schedule .= $table->th($w);
        }
        $schedule .= $table->_tr;
        $schedule .= $table->tr;
        foreach my $w (@week_name) {
            my $t = shift @timetable || '';
            if ($t) {
                if ( $this->{'uselist'} ) {
                    $t =~ s/\s*,\s*/<\/li><li>/gs;
                    $t = '<ul><li>' . $t . '</li></ul>';
                }
                else {
                    $t =~ s/\s*,\s*/<br \/>/gs;
                }
            }
            $schedule .= $table->td($t);
        }
        $schedule .= $table->_tr;
    }
    else {
        foreach my $w (@week_name) {
            my $t = shift @timetable || '';
            if ( $this->{'uselist'} && $t ) {
                $t =~ s/\s*,\s*/<\/li><li>/gs;
                $t = '<ul><li>' . $t . '</li></ul>';
            }
            $schedule .= $table->tr;
            $schedule .= $table->th($w);
            $schedule .= $table->td($t);
            $schedule .= $table->_tr;
        }
    }
    $schedule .= $table->_table;
    return $schedule;
}

#---------------------------------------------------------------------
#	(4) obj->format_for_mail(str);
sub format_for_mail {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return '' if !defined $str || $str eq '';
    return
          '['
        . $this->name . "]\x0A"
        . $this->format_for_text($str)
        . "\x0A\x0A";
}

#	(4)-3 obj->format_for_text(str);
sub format_for_text {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return '' if !defined $str || $str eq '';
    my @week_name = $this->week_name;
    my @timetable = split( /\x0A/, $str );
    my $schedule  = '';
    foreach my $w (@week_name) {
        my $t = shift @timetable || '';
        $schedule .= '(' . $w . ') ' . $t . "\x0A";
    }
    return $schedule;
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my ( $this, $html_disabled ) = @_;

    #名前
    my $name = 'data:' . $this->{'id'};

    $this->print_field_info_form($name);

    print '<fieldset>';
    print '<legend>入力欄設定</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_lang">週表示</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        $name . '_lang', '', '', $this->{'lang'},
        'ja' =>
            '日曜日,月曜日,火曜日,水曜日,木曜日,金曜日,土曜日',
        'en' => 'Sunday,Monday,Tuesday,Wednesday,Thursday,Friday,Saturday',
    );
    print ' <label><input type="checkbox" name="'
        . $name
        . '_short" value="1"'
        . ( ( defined $this->{'short'} && $this->{'short'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 略記（日,Sun）</label>';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">「時」指定範囲</span></td>';
    print '<td class="field">';
    print '<label><select name="'
        . $name
        . '_hourStart" id="'
        . $name
        . '_hourStart">';
    print '<option value=""></option>';

    foreach my $hour ( 0 .. 30 ) {
        print '<option value="'
            . $hour . '"'
            . (
            (          defined $this->{'hourStart'}
                    && $this->{'hourStart'} ne ''
                    && $hour == $this->{'hourStart'}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $hour
            . '</option>';
    }
    print '</select>時台</label>〜';
    print '<label><select name="'
        . $name
        . '_hourEnd" id="'
        . $name
        . '_hourEnd">';
    print '<option value=""></option>';
    foreach my $hour ( 0 .. 30 ) {
        print '<option value="'
            . $hour . '"'
            . (
            (          defined $this->{'hourEnd'}
                    && $this->{'hourEnd'} ne ''
                    && $hour == $this->{'hourEnd'}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $hour
            . '</option>';
    }
    print '</select>時台</label>';
    print '</td>';
    print '</tr>';
    print '<td class="field-name"><label for="' . $name
        . '_minScale">「分」指定（検索時）</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        $name . '_minScale', '', '', $this->{'minScale'},
        ''     => '1分きざみ',
        '5'    => '5分きざみ',
        '10'   => '10分きざみ',
        '15'   => '15分きざみ',
        '20'   => '20分きざみ',
        '30'   => '30分きざみ',
        'none' => '非表示',
    );
    print '</td>';
    print '</tr>';

    print '<tr>';
    print '<td class="field-name"><label for="' . $name
        . '_default">デフォルト</label></td>';
    print '<td class="field">';
    print $this->input_form( 'data', 'default', 1 );
    print '</td>';
    print '</tr>';
    print '</tbody></table>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>表示設定</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_column" value="1"'
        . ( ( defined $this->{'column'} && $this->{'column'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 列を週にする。</label></li>';
    print '<li><label><input type="checkbox" name="'
        . $name
        . '_uselist" value="1"'
        . ( ( defined $this->{'uselist'} && $this->{'uselist'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 時間をリストで表示する。</label></li>';
    print '</ol>';
    $this->print_table_setting_form( $name, ' (週間予定表)', 'none' )
        if !$html_disabled;
    print '</fieldset>';

    if ( !$html_disabled ) {

        print '<fieldset>';
        print '<legend>よりくわしい検索</legend>';
        print '<label><input type="checkbox" name="'
            . $name
            . '_searchOption" value="1"'
            . ( ( defined $this->{'searchOption'} && $this->{'searchOption'} )
            ? ' checked="checked"'
            : '' )
            . ' /> よりくわしい検索を有効にする。</label></li>';
        print '</fieldset>';
    }
    $this->print_setting_save_button;
    return;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this     = shift;
    my $is_valid = 1;

    #チェックボックス
    $main::data{ $this->{'id'} . '_short' } = ''
        if !$main::data{ $this->{'id'} . '_short' };
    $main::data{ $this->{'id'} . '_column' } = ''
        if !$main::data{ $this->{'id'} . '_column' };
    $main::data{ $this->{'id'} . '_uselist' } = ''
        if !$main::data{ $this->{'id'} . '_uselist' };
    $main::data{ $this->{'id'} . '_searchOption' } = ''
        if !$main::data{ $this->{'id'} . '_searchOption' };

    if (   defined $main::data{ $this->{'id'} . '_hourStart' }
        && $main::data{ $this->{'id'} . '_hourStart' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_hourStart' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '「時」指定範囲の開始時を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_hourStart' };
        $is_valid = 0;
    }
    elsif (
           defined $main::data{ $this->{'id'} . '_hourStart' }
        && $main::data{ $this->{'id'} . '_hourStart' } ne ''
        && (   $main::data{ $this->{'id'} . '_hourStart' } < 0
            || $main::data{ $this->{'id'} . '_hourStart' } > 30 )
        )
    {
        delete $main::data{ $this->{'id'} . '_hourStart' };
        $is_valid = 0;
    }
    if (   defined $main::data{ $this->{'id'} . '_hourEnd' }
        && $main::data{ $this->{'id'} . '_hourEnd' } ne ''
        && $this->is_nan( $main::data{ $this->{'id'} . '_hourEnd' } ) )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '「時」指定範囲の終了時を更新できませんでした。半角数字にしてください。'
        ) if ref $cgi::action;
        delete $main::data{ $this->{'id'} . '_hourEnd' };
        $is_valid = 0;
    }
    elsif (
           defined $main::data{ $this->{'id'} . '_hourEnd' }
        && $main::data{ $this->{'id'} . '_hourEnd' } ne ''
        && (   $main::data{ $this->{'id'} . '_hourEnd' } < 0
            || $main::data{ $this->{'id'} . '_hourEnd' } > 30 )
        )
    {
        delete $main::data{ $this->{'id'} . '_hourEnd' };
        $is_valid = 0;
    }
    if (   defined $main::data{ $this->{'id'} . '_hourStart' }
        && $main::data{ $this->{'id'} . '_hourStart' } ne ''
        && defined $main::data{ $this->{'id'} . '_hourEnd' }
        && $main::data{ $this->{'id'} . '_hourEnd' } ne ''
        && $main::data{ $this->{'id'} . '_hourEnd' }
        - $main::data{ $this->{'id'} . '_hourStart' } >= 24 )
    {
        $main::data{ $this->{'id'} . '_hourStart' }
            = $main::data{ $this->{'id'} . '_hourEnd' } - 23;
    }
    if (   $main::data{ $this->{'id'} . '_hourStart' }
        && $main::data{ $this->{'id'} . '_hourEnd' }
        && $main::data{ $this->{'id'} . '_hourStart' }
        > $main::data{ $this->{'id'} . '_hourEnd' } )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '「時」指定範囲を確認してください。開始時が終了時より後になっています。'
        ) if ref $cgi::action;
        my $tmp = $main::data{ $this->{'id'} . '_hourEnd' };
        $main::data{ $this->{'id'} . '_hourEnd' }
            = $main::data{ $this->{'id'} . '_hourStart' };
        $main::data{ $this->{'id'} . '_hourStart' } = $tmp;
    }

    if ($main::data{ $this->{'id'} . '_minScale' }
        && !grep( $_ eq $main::data{ $this->{'id'} . '_minScale' },
            5, 10, 15, 20, 30, 'none' )
        )
    {
        $main::data{ $this->{'id'} . '_minScale' } = '';
    }

    my @week_name = $this->week_name;
    my $hour_start
        = (    $is_valid
            && defined $main::data{ $this->{'id'} . '_hourStart' }
            && $main::data{ $this->{'id'} . '_hourStart' } ne '' )
        ? $main::data{ $this->{'id'} . '_hourStart' }
        : $this->{'hourStart'};
    my $hour_end
        = (    $is_valid
            && defined $main::data{ $this->{'id'} . '_hourEnd' }
            && $main::data{ $this->{'id'} . '_hourEnd' } ne '' )
        ? $main::data{ $this->{'id'} . '_hourEnd' }
        : $this->{'hourEnd'};
    my $start = $hour_start * 100;
    my $end   = ( $hour_end + 1 ) * 100;
    foreach my $w ( 'su', 'mo', 'tu', 'we', 'th', 'fr', 'sa' ) {
        my $time = $main::data{ $this->{'id'} . '_default' . $w };
        $time =~ s/\s*[\-~ー〜]+\s*/-/sg;
        $time = $this->z_to_h($time);
        my $is_invalid       = 0;
        my $is_not_available = 0;
        my $h                = '';
        foreach my $span ( split( /[\s,]+/, $time ) ) {
            if ( $span =~ /^(.*?)\-(.*)$/ ) {
                my ( $f, $t )
                    = ( $this->str_to_hour($1), $this->str_to_hour($2) );
                if ( $f ne '' || $t ne '' ) {
                    ( $f, $t ) = ( $t, $f )
                        if $f ne '' && $t ne '' && $f > $t;
                    $h
                        .= ( ( $h ne '' ) ? ', ' : '' )
                        . $this->num_to_hour($f) . ' - '
                        . $this->num_to_hour($t);
                    $is_not_available = 1 if $f < $start || $t >= $end;
                }
                else {
                    $is_invalid = 1;
                }
            }
            elsif ( $span ne '' ) {
                $span = $this->str_to_hour($span);
                if ( $span ne '' ) {
                    $h .= ( ( $h ne '' ) ? ', ' : '' )
                        . $this->num_to_hour($span);
                    $is_not_available = 1 if $span < $start || $span >= $end;
                }
                else {
                    $is_invalid = 1;
                }
            }
        }
        $main::data{ $this->{'id'} . '_default' . $w } = $h;
        my $wname = shift @week_name;
        if ( $is_invalid || $is_not_available ) {
            $is_valid = 0;
            push(
                @{ $cgi::action->{'dialog'} },
                'デフォルトの入力('
                    . $wname
                    . ')を確認してください。'
                    . $hour_start
                    . '時台から'
                    . $hour_end
                    . '時台の間を入力してください。'
            ) if ref $cgi::action && $is_not_available;
            push(
                @{ $cgi::action->{'dialog'} },
                'デフォルトの入力('
                    . $wname
                    . ')を確認してください。'
            ) if ref $cgi::action && $is_invalid;
        }
    }

    $this->tag_fix( \$main::data{ $this->{'id'} . '_description' } )
        if defined $main::data{ $this->{'id'} . '_description' }
        && $main::data{ $this->{'id'} . '_description' };
    $this->tag_fix( \$main::data{ $this->{'id'} . '_note' } )
        if defined $main::data{ $this->{'id'} . '_note' }
        && $main::data{ $this->{'id'} . '_note' };
    return $is_valid;
}

######################################################################
#	(7) obj->search_form;
sub search_form ($) {
    my $this = shift;

    my $form
        = '<tr><td class="field-name"><span class="datatitle">'
        . $this->{'name'}
        . '</span></td><td class="field">';
    $form .= $this->input_form('search');

    #英語表記
    if ( defined $default::english && $default::english ) {
        $form .= '<div class="sub-field">';
        $form .= $this->mk_select_menu(
            'search:' . $this->{'id'} . ':mode', '', '',
            $main::search{ $this->{'id'} }{'mode'},
            'and' => 'Matches on all',
            'or'  => 'Matches on any'
        );
        $form .= '</div>';

        #日本語
    }
    else {
        $form .= '<div class="sub-field">';
        $form .= $this->mk_select_menu(
            'search:' . $this->{'id'} . ':mode', '', '',
            $main::search{ $this->{'id'} }{'mode'},
            'and' => '全ての条件に一致',
            'or'  => 'いずれかの条件に一致'
        );
        $form .= '</div>';
    }
    $form
        .= '<div class="extra-field"><label><input type="checkbox" name="search:'
        . $this->{'id'}
        . ':o" value="1"'
        . (
        (   defined $main::search{ $this->{'id'} }{'o'}
                && $main::search{ $this->{'id'} }{'o'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> '
        . ( ($default::english)
        ? 'Strictly.'
        : '条件に重ならない期間を許容しない' )
        . '</label></div>';

    $form .= '</td></tr>';

    return $form;
}

######################################################################
#	(8) obj->sort_enabled;
sub sort_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#sub import {
BEGIN {
    Field::class_register( 'time_wschedule', 'WeeklySchedule',
        '週間予定表' );
}

1;
