package HTTP_Client;
use Fcntl qw(:flock);
use strict;
{ use bytes; }
use utf8;
######################################################################
# HTTP クライアントモジュール
# HTTP_Client.pm 1.1.1
# Copyright (C) 2007-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-09-01
######################################################################
#	$res = HTTP_Client->new->request(
#		'GET'	=>	url,
#		{
#			'Accept'	=>	'*/*',
#		}
#		content,
#		jcode,
#		size,
#		content_file,
#	);
######################################################################
$HTTP_Client::UserAgent = 'CMONOS_HTTP_CLIENT';
######################################################################
use Jcode;

sub new ($;@) {
    my $pkg  = shift;
    my $this = {@_};
    bless $this;

    #if ($this->{'lwp'} && eval{require LWP::UserAgent}) {
    if ( eval { require LWP::UserAgent } ) {
        $this->{'ua'} = LWP::UserAgent->new;
        $this->{'ua'}->agent($HTTP_Client::UserAgent);
        $this->{'ua'}->timeout(10);
    }
    else {
        require CMONOS::CGI_Lib::TcpClient;
    }
    $this;
}

sub jcode_convert($$$$;$) {
    my ( $this, $str, $ocode, $icode, $was_entity ) = @_;
    my $str_ref = ( ref $str ) ? $str : \$str;
    $ocode = 'utf8' if !$ocode;
    if ( !$icode || $ocode ne $icode ) {
        if ( defined $Jcode::FALLBACK ) {
            if ( $ocode eq 'utf8' ) {
                $$str_ref = Jcode->new( $str_ref, $icode )
                    ->fallback(&Jcode::FB_XMLCREF)->utf8;
            }
            elsif ( $ocode eq 'euc' ) {
                $$str_ref = Jcode->new( $str_ref, $icode )
                    ->fallback(&Jcode::FB_XMLCREF)->euc;
            }
            elsif ( $ocode eq 'jis' ) {
                $$str_ref = Jcode->new( $str_ref, $icode )
                    ->fallback(&Jcode::FB_XMLCREF)->jis;
            }
            elsif ( $ocode eq 'sjis' ) {
                $$str_ref = Jcode->new( $str_ref, $icode )
                    ->fallback(&Jcode::FB_XMLCREF)->sjis;
            }
            elsif ( $ocode eq 'ucs2' ) {
                $$str_ref = Jcode->new( $str_ref, $icode )
                    ->fallback(&Jcode::FB_XMLCREF)->ucs2;
            }
            else {
                Jcode::convert( $str_ref, $ocode, $icode );
            }
        }
        else {
            Jcode::convert( $str_ref, $ocode, $icode );
        }
    }
    if ( $ocode eq 'utf8' ) {
        utf8::decode($$str_ref) if !utf8::is_utf8($$str_ref);
    }
    elsif ( utf8::is_utf8($$str_ref) ) {
        utf8::encode($$str_ref);
    }
    $$str_ref
        =~ s/\&\#([0-9]{4,}|x[a-fA-F0-9]+);/$this->_entity_to_jcode($1,$ocode)/seg
        if !$was_entity && $ocode ne 'ucs2';
        $$str_ref =~ s/(?:\x{200b}|\&#x200b;)/ /sig if !$was_entity && $ocode eq 'utf8';
    return $$str_ref;
}

sub _entity_to_jcode ($$$) {
    my ( $this, $code, $ocode ) = @_;
    utf8::encode($code) if utf8::is_utf8($code);
    $code = sprintf( "%X", $code ) unless $code =~ s/^x//;
    return $this->jcode_convert( pack( "U", hex($code) ), $ocode, 'utf8', 1 );
}

# meta 情報から文字コード取得
sub get_charset_from_html ($$) {
    my ( $this, $html ) = @_;
    my $html_ref = ( ref $html ) ? $html : \$html;
    if ( $$html_ref =~ /<head(?![0-9a-zA-Z_])[^>]*>(.*?)<\/head>/is ) {
        my $header = $1;
        if ( $header
            =~ /<meta(?![0-9a-zA-Z_])[^>]*charset=["']?([0-9a-zA-Z_\-]+)["']?[^>]*>/si
            )
        {
            return $this->ename_to_jname($1);
        }
        else {
            while ( $header
                =~ /<meta(?![0-9a-zA-Z_])([^>]*)http-equiv=["']?Content-Type["']?([^>]*)>/sgi
                )
            {
                my $contenttype = $1 . $2;
                if ( $contenttype
                    =~ /[\x20\x09\x0A\x0D]content=["']?(.*?)["']?/si )
                {
                    $contenttype = $1;
                    if ( $contenttype =~ /charset=([0-9a-zA-Z_\-]+)/is ) {
                        return $this->ename_to_jname($1);
                    }
                }
            }
        }
    }
    if ( $$html_ref
        =~ /<\?xml(?=[\x20\x09\x0A\x0D])[^>]*[\x20\x09\x0A\x0D]encoding=["']?([0-9a-zA-Z_\-]+)["']?[^>]*\?>/i
        )
    {
        return $this->ename_to_jname($1);
    }
    elsif ( $$html_ref =~ /\@charset\x20+["']?([a-zA-Z0-9\-_]+)["']?;/i ) {
        return $this->ename_to_jname($1);
    }
    elsif ( $$html_ref =~ /^[\x20\x09]*use utf8;/m ) {
        return 'utf8';
    }
    return '';
}

sub ename_to_jname ($$) {
    my ( $this, $ename ) = @_;
    return 'utf8' if !$ename;
    $ename = lc $ename;
    my %ename2j = (
        'shiftjis'    => 'sjis',
        'x-sjis'      => 'sjis',
        'shift_jis'   => 'sjis',
        'euc-jp'      => 'euc',
        'x-euc-jp'    => 'euc',
        '7bit-jis'    => 'jis',
        'x-jis'       => 'jis',
        'iso-2022-jp' => 'jis',
        'utf-16be'    => 'ucs2',
        'utf-8'       => 'utf8',
    );
    if ( $ename2j{$ename} ) {
        return $ename2j{$ename};
    }
    else {
        $ename =~ tr/-/_/;
        return $ename;
    }
}

######################################################################
#ソケットを開いて接続する
sub request ($$$;$$$$$$) {
    my ($this,  $method, $url,          $header,  $content,
        $jcode, $size,   $content_file, $account, $password
    ) = @_;
    undef $content_file if defined $content_file && $content_file eq '';
    $url =~ s/#.*?$//;
    ( $account, $password ) = ( $2, $3 )
        if $url =~ s/^(https?:\/\/)([^:\/\@]+):([^:\/\@]+)\@/$1/i;
    $this->{'url'} = $url;
    $header = {} if !ref $header;
    if ( ref $content eq 'HASH' ) {
        $content = $this->url_encoded_query( $content, $jcode );
        $header->{'Content-Type'}
            = 'application/x-www-form-urlencoded; charset='
            . $this->j2e($jcode)
            if !$header->{'Content-Type'};
    }
    elsif ( $content && $jcode ) {
        $this->jcode_convert( \$content, $jcode );
    }
    utf8::encode($content) if $content && utf8::is_utf8($content);
    $method = uc $method;
    $method = 'POST'
        if $method eq 'GET' && defined $content && $content ne '';
    if ( ref $this->{'ua'} ) {
        delete $this->{'res'};
        my $request = HTTP::Request->new( $method => $url );
        $this->{'ua'}->max_size( ($size) ? $size : undef );
        $request->authorization_basic( $account, $password )
            if defined $account && $account ne '';
        $request->header(%$header) if %$header;
        $request->content($content) if defined $content && $content ne '';
        if ( ref $this->{'requests_redirectable_if'} ) {
            $this->{'ua'}->requests_redirectable( [] );
        }
        elsif ( ref $this->{'requests_redirectable'} ) {
            $this->{'ua'}
                ->requests_redirectable( $this->{'requests_redirectable'} );
        }
        else {
            $this->{'ua'}->requests_redirectable( [ 'POST', 'GET' ] );
        }
        $this->{'res'} = $this->{'ua'}->request( $request, $content_file );
        if ( ref $this->{'requests_redirectable_if'} ) {
            my $location = $this->{'res'}->header('Location');
            if (   $location
                && $this->{'requests_redirectable_if'}
                ->( $method => $location, $this ) )
            {
                return $this->request( $method, $location, $header, $content,
                    $jcode );
            }
        }
        elsif ( $this->{'res'}->previous ) {

            #$url = $this->{'res'}->previous->header('Location');
            $url = $this->{'res'}->request->uri;
            $this->{'url'} = $url if $url;
        }
    }
    else {
        delete $this->{'response'};
        my ( $host, $path )
            = ( $url =~ m!^https?://([^/]+)(/?.*)$!is )
            ? ( $1, $2 )
            : ( '', '' );
        $path = '/' if !$path;
        if ( !$host ) {
            $this->{'response'}{'is_error'} = 1;
            $this->{'response'}{'message'}  = '';
            return $this;
        }
        delete $header->{'Content-Length'};
        $header->{'User-Agent'} = $HTTP_Client::UserAgent
            if $HTTP_Client::UserAgent;
        $header->{'Accept'} = '*/*' if !$header->{'Accept'};
        $header->{'Host'} = $host;
        local (*HTTPCLIENT);
        if (!TcpClient->new(
                'handle'   => \*HTTPCLIENT,
                'host'     => $host,
                'portname' => 'http',
            )->tcp_connect
            )
        {
            $this->{'response'}{'is_error'} = 1;
            return $this;
        }
        print HTTPCLIENT "$method $path HTTP/1.0\x0D\x0A";
        while ( my ( $k, $v ) = each %$header ) {
            print HTTPCLIENT $k . ': ' . $v . "\x0D\x0A";
        }
        print HTTPCLIENT 'Authorization: BASIC '
            . $this->data_to_base64( $account . ':' . $password )
            . "\x0D\x0A"
            if defined $account && $account ne '';
        if ( defined $content && $content ne '' ) {
            print HTTPCLIENT 'Content-Length: '
                . bytes::length($content)
                . "\x0D\x0A";
            print HTTPCLIENT "\x0D\x0A";
            print HTTPCLIENT $content;
        }
        else {
            print HTTPCLIENT "\x0D\x0A";
        }
        my $res = <HTTPCLIENT>;
        $res =~ s/[\x09\x0A\x0C\x0D\x20]+$//gs;
        if ( $res
            =~ /HTTP\/[0-9\.]+[\x09\x0A\x0C\x0D\x20]+([0-9]+)[\x09\x0A\x0C\x0D\x20]*(.*)$/
            )
        {
            my ( $code, $msg ) = ( $1, $2 );
            $this->{'response'}{'code'}    = $code;
            $this->{'response'}{'message'} = $msg;
            if ( $code == 200 ) {
                $this->{'response'}{'is_success'} = 1;
            }
            else {
                $this->{'response'}{'is_error'} = 1;
            }
        }
        else {
            $this->{'response'}{'is_error'} = 1;
        }
        my $content_flag = 0;
        my $res_header   = '';
        while ( $res = <HTTPCLIENT> ) {
            if ($content_flag) {
                $this->{'response'}{'content'} .= $res;
            }
            else {
                $res =~ s/[\x09\x0A\x0C\x0D\x20]+$//gs;
                if ( $res ne '' ) {
                    if ( $res
                        =~ /^(.*?):[\x09\x0A\x0C\x0D\x20]*(.*)[\x09\x0A\x0C\x0D\x20]*$/s
                        )
                    {
                        $res_header .= $res;
                        my ( $name, $value ) = ( $1, $2 );
                        if ( defined $this->{'response'}{'header'}{$name} ) {
                            if (ref $this->{'response'}{'header'}{$name} eq
                                'ARRAY' )
                            {
                                push(
                                    @{  $this->{'response'}{'header'}{$name}
                                    },
                                    $value
                                );
                            }
                            else {
                                $this->{'response'}{'header'}{$name} = [
                                    $this->{'response'}{'header'}{$name},
                                    $value
                                ];
                            }
                        }
                        else {
                            $this->{'response'}{'header'}{$name} = $value;
                        }
                    }
                }
                elsif ( $size || $content_file ) {
                    last;
                }
                else {
                    $content_flag = 1;
                }
            }
        }
        if ($content_file) {
            if ( open( FILE, ">>$content_file" ) ) {
                binmode FILE;
                flock( FILE, LOCK_EX );
                truncate( FILE, 0 );
                seek( FILE, 0, 0 );
                my $buf = '';
                my $buffer_size = $env::buffer_size || 8192;
                while ( read( HTTPCLIENT, $buf, $buffer_size ) ) {
                    print FILE $buf;
                }
                close(FILE);
                $this->{'response'}{'content_file'} = $content_file;
                $this->{'response'}{'header'}{'Content-Length'}
                    = -s $content_file;
            }
        }
        elsif ($size) {
            read( HTTPCLIENT, $this->{'response'}{'content'}, $size );
            my $content_length = bytes::length $this->{'response'}{'content'};
            $this->{'response'}{'header'}{'Content-Range'}
                = 'bytes 0-'
                . ( $content_length - 1 ) . '/'
                . $this->{'response'}{'header'}{'Content-Length'}
                if $this->{'response'}{'header'}{'Content-Length'};
            $this->{'response'}{'header'}{'Content-Length'} = $content_length;
        }
        close(HTTPCLIENT);
        if (   $this->{'response'}{'is_error'}
            && $this->{'response'}{'header'}{'Location'}
            && $this->{'response'}{'header'}{'Location'} ne $url
            && ( !ref $this->{'requests_redirectable'}
                || grep( $_ eq $method,
                    @{ $this->{'requests_redirectable'} } ) )
            && (!ref $this->{'requests_redirectable_if'}
                || $this->{'requests_redirectable_if'}->(
                    $method => $this->{'response'}{'header'}{'Location'},
                    $this
                )
            )
            )
        {
            return $this->request( $method,
                $this->{'response'}{'header'}{'Location'},
                $header, $content, $jcode );
        }
    }
    return $this;
}

sub requests_redirectable ($$) {
    my ( $this, $protocols_ref ) = @_;
    $this->{'requests_redirectable'}
        = ( ref $protocols_ref eq 'ARRAY' )
        ? $protocols_ref
        : [ 'POST', 'GET' ];
    return $this;
}

sub requests_redirectable_if ($$) {
    my ( $this, $function_ref ) = @_;
    $this->{'requests_redirectable_if'}
        = ( ref $function_ref eq 'CODE' ) ? $function_ref : undef;
    return $this;
}

sub read_content_file ($) {
    my $this = shift;
    return 1 if ref $this->{'response'} && $this->{'response'}{'content'};
    local (*FILE);
    if (   ref $this->{'response'}
        && $this->{'response'}{'content_file'}
        && open( FILE, $this->{'response'}{'content_file'} ) )
    {
        binmode FILE;
        flock( FILE, LOCK_SH );
        $this->{'response'}{'content'} = join( '', <FILE> );
        close(FILE);
        return 1;
    }
    return 0;
}

sub url ($) {
    my $this = shift;
    return ( $this->{'url'} ) ? $this->{'url'} : '';
}

sub is_success ($) {
    my $this = shift;
    if ( ref $this->{'ua'} ) {
        return ( ref $this->{'res'} ) ? $this->{'res'}->is_success : 0;
    }
    else {
        return ( ref $this->{'response'}
                && $this->{'response'}{'is_success'} ) ? 1 : 0;
    }
}

sub is_error ($) {
    my $this = shift;
    if ( ref $this->{'ua'} ) {
        return ( ref $this->{'res'} ) ? $this->{'res'}->is_error : 0;
    }
    else {
        return ( ref $this->{'response'} && $this->{'response'}{'is_error'} )
            ? 1
            : 0;
    }
}

sub content ($;$) {
    my ( $this, $jcode ) = @_;
    if ( ref $this->{'ua'} ) {
        return
              ( ref $this->{'res'} )
            ? ($jcode)
                ? $this->jcode_convert(
                    $this->{'res'}->content,
                    $jcode,
                    $this->get_charset_from_html(
                        $this->{'res'}->content_ref
                    )
                )
                : $this->{'res'}->content
            : '';
    }
    else {
        $this->read_content_file;
        return
              ( ref $this->{'response'} && $this->{'response'}{'content'} )
            ? ($jcode)
                ? $this->jcode_convert(
                    $this->{'response'}{'content'},
                    $jcode,
                    $this->get_charset_from_html(
                        \$this->{'response'}{'content'}
                    )
                )
                : $this->{'response'}{'content'}
            : '';
    }
}

sub hash ($;$$) {
    my ( $this, $jcode, $ignore_attribute ) = @_;
    my $content = $this->content('utf8');
    if ( $content =~ /<.*?>/ ) {
        return $this->get_hash_from_xml( $content, $ignore_attribute,
            $jcode );
    }
    elsif ( $content !~ /[\x0A\x0D]/ && $content =~ /=/ ) {
        return $this->url_decode( $content, $jcode );
    }
    else {
        return $this->get_hash_from_key_and_value( $content, $jcode );
    }
}

sub content_ref ($) {
    my $this  = shift;
    my $empty = '';
    if ( ref $this->{'ua'} ) {
        return ( ref $this->{'res'} ) ? $this->{'res'}->content_ref : \$empty;
    }
    else {
        $this->read_content_file;
        return ( ref $this->{'response'} && $this->{'response'}{'content'} )
            ? \$this->{'response'}{'content'}
            : \$empty;
    }
}

sub code ($) {
    my $this = shift;
    if ( ref $this->{'ua'} ) {
        return ( ref $this->{'res'} ) ? $this->{'res'}->code : '';
    }
    else {
        return ( ref $this->{'response'} && $this->{'response'}{'code'} )
            ? $this->{'response'}{'code'}
            : '';
    }
}

sub message ($) {
    my $this = shift;
    if ( ref $this->{'ua'} ) {
        return ( ref $this->{'res'} ) ? $this->{'res'}->message : '';
    }
    else {
        return ( ref $this->{'response'} && $this->{'response'}{'message'} )
            ? $this->{'response'}{'message'}
            : '';
    }
}

sub header ($;$) {
    my ( $this, $key ) = @_;
    if ($key) {
        if ( ref $this->{'ua'} ) {
            return
                  ( ref $this->{'res'} )
                ? (wantarray)
                    ? ( $this->{'res'}->header($key) )
                    : $this->{'res'}->header($key)
                : (wantarray) ? ()
                :               '';
        }
        elsif (ref $this->{'response'}
            && ref $this->{'response'}{'header'}
            && defined $this->{'response'}{'header'}{$key} )
        {
            if (wantarray) {
                return ( ref $this->{'response'}{'header'}{$key} )
                    ? @{ $this->{'response'}{'header'}{$key} }
                    : ( $this->{'response'}{'header'}{$key} );
            }
            elsif ( ref $this->{'response'}{'header'}{$key} ) {
                return join(
                    ',',
                    map( $_ =~ s/,/\%2C/g,
                        @{ $this->{'response'}{'header'}{$key} } )
                );
            }
            else {
                return $this->{'response'}{'header'}{$key};
            }
        }
        else {
            return (wantarray) ? () : '';
        }
    }
    else {
        if ( ref $this->{'ua'} ) {
            return ( ref $this->{'res'} )
                ? $this->get_hash_from_key_and_value(
                $this->{'res'}->as_string )
                : {};
        }
        else {
            return (   ref $this->{'response'}
                    && ref $this->{'response'}{'header'} )
                ? $this->{'response'}{'header'}
                : {};
        }
    }
}

sub content_type ($) {
    my $this = shift;
    if ( ref $this->{'ua'} ) {
        return ( ref $this->{'res'} ) ? $this->{'res'}->content_type : '';
    }
    elsif (ref $this->{'response'}
        && ref $this->{'response'}{'header'}
        && $this->{'response'}{'header'}{'Content-Type'} )
    {
        return ( $this->{'response'}{'header'}{'Content-Type'} =~ /^(.*?);/ )
            ? lc($1)
            : lc( $this->{'response'}{'header'}{'Content-Type'} );
    }
    else {
        return '';
    }
}

########################################################################
#簡易xml処理
sub get_hash_from_xml ($$;$) {
    my ( $this, $xml, $ignore_attribute, $jcode ) = @_;
    my $hash = {};
    my $name;
    my @hash_ref;
    my @keys;
    my $param;
    my $start;
    my %had_attributes;
    $this->jcode_convert( \$xml, 'utf8' ) if !utf8::is_utf8($xml);
    $xml
        =~ s/<!\[CDATA\[(.*?)\]\]>/'<![CDATA[' . $this->escape($1) . ']]>'/seg;

    while ( $xml =~ /\G(.*?)<(\/?)([0-9a-zA-Z_:\-]+)([^>]*)>/sg ) {
        my ( $value, $closed, $key, $attribute ) = ( $1, $2, $3, $4 );
        if ($closed) {
            if ( $start && $key eq $start ) {
                if ($name) {
                    $this->_set_hash_value( $hash, $name, $value );
                }
                elsif ( $key eq 'value' ) {
                    $hash = pop(@hash_ref);
                    pop(@keys);
                    $this->_set_hash_value( $hash, $param, $value );
                }
                else {
                    $this->_set_hash_value( $hash, $key, $value,
                        $had_attributes{$hash} );
                }
                undef $param;
                undef $start;
                undef $name;
            }
            elsif ( @keys && $key eq $keys[$#keys] ) {
                $hash = pop(@hash_ref);
                pop(@keys);
            }
            delete $had_attributes{$hash};
        }
        else {
            if ( $start && $start ne 'value' && $start ne 'name' ) {
                push( @hash_ref, $hash );
                push( @keys,     $start );
                if ( !defined $hash->{$start} ) {
                    $hash = $hash->{$start} = {};
                }
                elsif ( ref $hash->{$start} eq 'ARRAY'
                    && $had_attributes{$hash} )
                {
                    $hash = $hash->{$start}[ $#{ $hash->{$start} } ];
                }
                elsif ( ref $hash->{$start} eq 'ARRAY' ) {
                    $hash = $hash->{$start}[ @{ $hash->{$start} } ] = {};
                }
                elsif ( ref $hash->{$start} eq 'HASH'
                    && !%{ $hash->{$start} } )
                {
                    $hash = $hash->{$start} = {};
                }
                elsif ( ref $hash->{$start} eq 'HASH'
                    && $had_attributes{$hash} )
                {
                    $hash = $hash->{$start};
                }
                else {
                    $hash->{$start} = [ $hash->{$start}, {} ];
                    $hash = $hash->{$start}[1];
                }
                $param = $start;
            }
            if ( $attribute && $attribute =~ m!/$! ) {
                if ( $attribute =~ /[\x09\x0A\x0C\x0D\x20]+name="(.*?)"/is ) {
                    $key = $1;
                    $value
                        = ( $attribute
                            =~ /[\x09\x0A\x0C\x0D\x20]+value="(.*?)"/is )
                        ? $1
                        : '';
                    $this->_set_hash_value( $hash, $key, $value );
                }
                else {
                    my %v = ();
                    while ( $attribute
                        =~ /[\x09\x0A\x0C\x0D\x20]+([0-9a-zA-Z_:\-]+)="([^"]*)"/g
                        )
                    {
                        $v{$1} = $this->unescape($2);
                    }
                    $this->_set_hash_value( $hash, $key, \%v ) if %v;
                }
                undef $start;
            }
            else {
                if ($attribute) {
                    if ( $attribute
                        =~ /[\x09\x0A\x0C\x0D\x20]+name="(.*?)"/is )
                    {
                        $key = $1;
                        $value
                            = ( $attribute
                                =~ /[\x09\x0A\x0C\x0D\x20]+value="(.*?)"/is )
                            ? $1
                            : '';
                        $this->_set_hash_value( $hash, $key, $value );
                        undef $start;
                        next;
                    }
                    elsif ( !$ignore_attribute ) {
                        my $temp = $hash;
                        if ( !defined $hash->{$key} ) {
                            $hash = $hash->{$key} = {};
                        }
                        elsif ( ref $hash->{$key} eq 'ARRAY' ) {
                            $hash = $hash->{$key}[ @{ $hash->{$key} } ] = {};
                        }
                        elsif ( ref $hash->{$key} eq 'HASH'
                            && !%{ $hash->{$key} } )
                        {
                            $hash = $hash->{$key} = {};
                        }
                        else {
                            $hash->{$key} = [ $hash->{$key}, {} ];
                            $hash = $hash->{$key}[1];
                        }
                        while ( $attribute
                            =~ /[\x09\x0A\x0C\x0D\x20]+([0-9a-zA-Z_:\-]+)="([^"]*)"/g
                            )
                        {
                            $this->_set_hash_value( $hash, $1, $2 );
                            $had_attributes{$temp} = 1;
                        }
                        $hash = $temp;
                    }
                }
                $start = $key;
            }
        }
    }
    if ( $jcode && $jcode ne 'utf8' ) {
        $hash = $this->hash_jcode_convert( $hash, $jcode );
    }
    return (%$hash) ? $hash : $xml;
}

sub _set_hash_value ($$$$) {
    my ( $this, $hash, $k, $v, $is_value ) = @_;
    return if !defined $k || !defined $v;
    if ( !ref $v ) {
        $v =~ s/<!\[CDATA\[(.*?)\]\]>/$1/sg;
        $v = $this->unescape($v);
    }
    if ( defined $hash->{$k} ) {
        if ( ref $hash->{$k} eq 'ARRAY' ) {
            @{ $hash->{$k} } = grep( !ref $_ || ( ref $_ eq 'HASH' && %$_ ),
                @{ $hash->{$k} } );
            if ( ref $v eq 'ARRAY' ) {
                push( @{ $hash->{$k} }, @$v );
            }
            elsif ($is_value) {
                $hash->{$k}->[ $#{ $hash->{$k} } ]->{'value'} = $v;
            }
            else {
                push( @{ $hash->{$k} }, $v );
            }
        }
        elsif ( ref $hash->{$k} eq 'HASH' ) {
            if ( !%{ $hash->{$k} } ) {
                $hash->{$k} = $v;
            }
            elsif ( ref $v eq 'ARRAY' ) {
                $this->_set_hash_value( $hash->{$k}, 'ARRAY', $v );
            }
            elsif ( ref $v eq 'HASH' ) {
                $hash->{$k} = [ $hash->{$k}, $v ];
            }
            elsif ( $v ne '' ) {
                $this->_set_hash_value( $hash->{$k}, 'value', $v );
            }
        }
        else {
            $hash->{$k} = [ $hash->{$k}, $v ];
        }
    }
    else {
        $hash->{$k} = $v;
    }
}

sub hash_jcode_convert ($$$) {
    my ( $this, $hash, $jcode ) = @_;
    if ( ref $hash eq 'ARRAY' ) {
        return [
            grep( $_ = $this->hash_jcode_convert( $_, $jcode ), @$hash ) ];
    }
    elsif ( ref $hash eq 'HASH' ) {
        my $conv = {};
        foreach my $k ( keys %$hash ) {
            $conv->{ $this->jcode_convert( $k, $jcode, 'utf8' ) }
                = $this->hash_jcode_convert( $hash->{$k}, $jcode );
        }
        return $conv;
    }
    elsif ( defined $hash && $hash ne '' ) {
        return $this->jcode_convert( $hash, $jcode, 'utf8' );
    }
    return $hash;
}

sub escape ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/&/&amp;/gs;
    $str =~ s/"/&quot;/g;
    $str =~ s/</&lt;/g;
    $str =~ s/>/&gt;/g;
    return $str;
}

sub unescape ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/\&quot;/"/g;
    $str =~ s/\&lt;/</g;
    $str =~ s/\&gt;/>/g;
    $str =~ s/\&amp;/&/gs;
    return $str;
}

sub get_hash_from_key_and_value ($$;$) {
    my ( $this, $str, $jcode ) = @_;
    $str = $1 if $str =~ /^(.*?)(\x0D\x0A|\x0A){2}/s;
    my $hash = {};
    while ( $str
        =~ /^([^:]+):[\x09\x0A\x0C\x0D\x20]*(.*?)[\x09\x0A\x0C\x0D\x20]*$/mg )
    {
        my ( $k, $v ) = ( $1, $2 );
        $hash->{ $this->jcode_convert( $k, $jcode, 'utf8' ) }
            = $this->jcode_convert( $v, $jcode, 'utf8' );
    }
    return $hash;
}

######################################################################
#BASE64エンコード
sub data_to_base64 ($$;$) {
    my ( $this, $data, $return ) = @_;
    my (@charset) = ( 'A' .. 'Z', 'a' .. 'z', '0' .. '9', '+', '/' );
    utf8::encode($data) if utf8::is_utf8($data);
    $data = unpack( "B*", $data );
    $data =~ s!(.{1,6})!$charset[ord(pack("B*", '00' . $1))]!g;
    $data .= '=' x ( 3 - ( bytes::length($1) / 2 ) );
    $data =~ s/(.{72})/$1$this->{'CR+LF'}/gs if $return;
    return $data;
}

######################################################################
#BASE64デコード
sub base64_to_data ($$) {
    my ( $this, $base64 ) = @_;
    $base64 =~ tr!A-Za-z0-9+=/!!cd;
    return $_[1] if bytes::length($base64) % 4;
    utf8::encode($base64) if utf8::is_utf8($base64);
    $base64 =~ s/=+$//;
    $base64 =~ tr!A-Za-z0-9+/! -_!;
    my $data = '';
    utf8::encode($data) if utf8::is_utf8($data);

    while ( $base64 =~ /(.{1,60})/g ) {
        my $len = pack( "C", 32 + int( bytes::length($1) * 3 / 4 ) );
        $data .= unpack( "u", $len . $1 );
    }
    return $data;
}

########################################################################
#URLエンコード
sub url_encode ($$;$) {
    my ( $this, $text, $op ) = @_;
    return '' if !defined $text || $text eq '';
    utf8::encode($text) if utf8::is_utf8($text);
    $text
        =~ s/([\x00-\x1F\x21-\x29\x2B\x2C\x2F\x3A-\x3F\x5B-\x5E\x60\x7B-\xFF])/'%' . sprintf("%.2X", unpack("C", $1))/eg;
    $text =~ s/\x20/+/g;
    return $text;
}

sub url_decode ($$) {
    my ( $this, $query, $jcode ) = @_;
    my $hash = {};
    $query =~ s/\+/ /g;
    while ( $query =~ /\G([^=\&]*)=([^\&]*)[\&]?/sgo ) {
        my ( $k, $v ) = ( $1, $2 );
        utf8::encode($k) if utf8::is_utf8($k);
        utf8::encode($v) if utf8::is_utf8($v);
        $k =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
        $v =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
        $hash->{ $this->jcode_convert( $k, $jcode ) }
            = $this->jcode_convert( $v, $jcode );
    }
    return $hash;
}

sub make_query ($$$) {
    my ( $this, $key, $value ) = @_;
    my $query = '';
    if ( ref $value eq 'ARRAY' ) {
        foreach (@$value) {
            $query .= $this->make_query( $key, $_ );
        }
    }
    elsif ( ref $value eq 'HASH' ) {
        my ( $k, $v );
        while ( ( $k, $v ) = each %$value ) {
            $query .= $this->make_query( $key . ':' . $k, $v );
        }
    }
    elsif ( ref $value eq 'SCALAR' ) {
        return $query;
    }
    else {
        $query
            .= '&'
            . $this->url_encode($key) . '='
            . $this->url_encode($value);
    }
    return $query;
}

sub url_encoded_query ($$;$) {
    my ( $this, $hash, $jcode ) = @_;
    my $query = '';
    while ( my ( $key, $value ) = each %$hash ) {
        $query .= ($jcode)
            ? $this->make_query(
            $this->jcode_convert( $key,   $jcode ),
            $this->jcode_convert( $value, $jcode )
            )
            : $this->make_query( $key, $value );
    }
    $query =~ s/^&//;
    return $query;
}

sub j2e ($$) {
    my ( $this, $jname ) = @_;
    return 'utf-8' if !$jname;
    my %j2e = (
        'sjis' => 'shift_jis',
        'euc'  => 'euc-jp',
        'jis'  => 'iso_2022_jp',
        'ucs2' => 'utf-16be',
        'utf8' => 'utf-8',
    );
    if ( $j2e{$jname} ) {
        return $j2e{$jname};
    }
    else {
        $jname =~ tr/_/-/;
        return $jname;
    }
}

1;
