package Holiday;
use strict;
use utf8;
######################################################################
# 祝日モジュール
# Holiday.pm 1.0.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
#　support@cmonos.jp
#　http://cmonos.jp/
#  最終更新日：2008-02-01
######################################################################
#
#		require CMONOS::CGI_Lib::Holiday;
#		obj = Calendar->new(year);
#
#		obj->{'mon'}{'mday'} = name;
#			mon		月[1-12]
#			mday	日
#			name	休日名
#
######################################################################
#	■参考資料
#	http://www.asahi-net.or.jp/~CI5M-NMR/misc/holiday.html
#	http://www.asahi-net.or.jp/~CI5M-NMR/misc/equinox.html
#	http://www.top.or.jp/~cpop/syunbun.htm
#	貴重な資料をまとめてくださった諸兄に心から感謝いたします。
######################################################################
#	■利用規約
#	このスクリプトの著作権はCMONOSが保有しています。
#	このサブルーチン集は、以下の条件に従うかぎり自由に再配布できます。
#		・著作権表示を変更してはいけません。
#		・修正を加えた場合は、修正者名・修正者の連絡先・修正個所・修正日
#		　時を明記して下さい。
#	このスクリプトを利用したことによって生じた、いかなる損害にも
#	CMONOSは責任を負いません。
######################################################################
#オブジェクト作成
sub new ($$) {
    my ( $pkg, $year ) = @_;
    my $this = {};
    bless $this;

    #日本の祝日
    #if ($ENV{'TZ'} =~ /JST/i) {
    $this->jp($year) if $year;

    #}
    return $this;
}

#閏年判定
sub is_leap_year ($$) {
    my ( $this, $year ) = @_;
    return 1 if ( $year % 4 == 0 && $year % 100 != 0 ) || $year % 400 == 0;
}

#日付チェック
sub is_impossible_mday ($$$$) {
    my ( $this, $y, $m, $d ) = @_;
    if ( ( ( $m == 4 || $m == 6 || $m == 9 || $m == 11 ) && $d > 30 )
        || (   $m == 2
            && $d > ( 28 + ( ( $this->is_leap_year($y) ) ? 1 : 0 ) ) )
        || $d > 31 )
    {
        return 1;
    }
    else {
        return 0;
    }
}

#ツェラーの公式
sub get_wday ($$$$) {
    my ( $this, $year, $mon, $mday ) = @_;
    if ( $mon == 1 || $mon == 2 ) {
        $year--;
        $mon += 12;
    }
    return
        int( $year
            + int( $year / 4 )
            - int( $year / 100 )
            + int( $year / 400 )
            + int( 2.6 * $mon + 1.6 )
            + $mday )
        % 7;
}

#第n w曜日
sub week ($$$$$) {
    my ( $this, $year, $mon, $week, $wday ) = @_;
    my ( $first, $mday );
    $first = $this->get_wday( $year, $mon, 1 );
    $mday = ( $week * 7 ) + $wday - $first - 6;
    return ( $wday >= $first ) ? $mday : 7 + $mday;
}

#春分の日　1900≦Y≦2099
sub s_equinox ($$) {
    my ( $this, $year ) = @_;
    return $this->gauss(
        0.24242 * $year - $this->gauss( $year / 4 ) + 35.84 );
}

#秋分の日　1900≦Y≦2099
sub a_equinox ($$) {
    my ( $this, $year ) = @_;
    return $this->gauss(
        0.24204 * $year - $this->gauss( $year / 4 ) + 39.01 );
}

sub gauss ($$) {
    my ( $this, $n ) = @_;
    return ( $n >= 0 ) ? int($n) : int( $n - 1 );
}

# --------------------------------------------------------------------
#	日本の祝日
sub jp ($$) {
    my ( $this, $year ) = @_;
    $this->holidays_j(
        $year, 1,
        1 => '元日',
        "New Year's Day",
        1,
        ( ( $year < 2000 ) ? 15 : $this->week( $year, 1, 2, 1 ) ) =>
            '成人の日',
        'Coming of Age Day',
        2,
        11 => '建国記念の日',
        'National Foundation Day',
        (   ( $year > 2019 )
            ? ( 2,
                23 => '天皇誕生日(令和)',
                "Emperor's Birthday (Naruhito)"
                )
            : ()
        ),
        3,
        $this->s_equinox($year) => '春分の日',
        'Vernal Equinox',
        4,
        29 => (
              ( $year < 1989 ) ? '天皇誕生日(昭和)'
            : ( $year < 2007 ) ? 'みどりの日'
            :                    '昭和の日'
        ),
        ( ( $year < 2007 ) ? 'Greenery Day' : 'Showa Day' ),
        5,
        3 => '憲法記念日',
        'Constitution Memorial Day',
        (   ( $year >= 1988 )
            ? ( 5,
                4 => (
                    ( $year < 2007 ) ? '国民の休日' : 'みどりの日'
                ),
                (   ( $year < 2007 )
                    ? 'Holiday for a Nation'
                    : 'Greenery Day'
                )
                )
            : ()
        ),
        5,
        5 => 'こどもの日',
        'Child Day',
        (   ( $year >= 1996 )
            ? ( 7,
                ( ( $year < 2003 ) ? 20 : $this->week( $year, 7, 3, 1 ) ) =>
                    '海の日',
                'Marine Day'
                )
            : ()
        ),
        (   ( $year >= 2016 )
            ? ( 8,
                11 =>
                    '山の日',
                'Mountain Day'
                )
            : ()
        ),
        (   ( $year >= 1966 )
            ? ( 9,
                ( ( $year < 2003 ) ? 15 : $this->week( $year, 9, 3, 1 ) ) =>
                    '敬老の日',
                'Respect for the Aged Day'
                )
            : ()
        ),
        9,
        $this->a_equinox($year) => '秋分の日',
        'Autumnal Equinox',
        (
            ( $year >= 1966 )
            ? (
                ( ( $year == 2020 || $year == 2021 ) ? 7 : 10 ),
                (
                      ( $year < 2000 )  ? 10
                    : ( $year == 2020 ) ? 24
                    : ( $year == 2021 ) ? 23
                    :                     $this->week( $year, 10, 2, 1 )
                  ) => (
                    ( $year < 2020 )
                    ? ( '体育の日', 'Health and Sports Day' )
                    : ( 'スポーツの日', 'Sports Day' )
                  )
              )
            : ()
          ),
        11,
        3 => '文化の日',
        'National Culture Day',
        11,
        23 => '勤労感謝の日',
        'Labor Thanksgiving Day',
        (   ( $year >= 1989 && $year <= 2019 )
            ? ( 12,
                23 => '天皇誕生日(平成)',
                "Emperor's Birthday (Akihito)"
                )
            : ()
        )
    );
    return $this;
}

sub holidays_j ($$) {
    my $this = shift;
    my $year = shift;
    my ( $m, $d, $j, $e );
    $this->{'holiday'} = {};
    while ( ( $m, $d, $j, $e ) = splice( @_, 0, 4 ) ) {
        if ($year >= 1973
            && ( defined $this->{'holiday'}{$m}{$d}
                && $this->{'holiday'}{$m}{$d} ne '' )
            || $this->get_wday( $year, $m, $d ) == 0
            )
        {
            my ( $fm, $fd )
                = ( $this->is_impossible_mday( $year, $m, $d + 1 ) )
                ? ( $m + 1, 1 )
                : ( $m, $d + 1 );
            $this->{'holiday'}{$fm}{$fd}
                = ($default::english) ? 'Holiday' : '振り替え休日';
        }
        elsif ($year >= 2003
            && $m == 9
            && $d == 23
            && $this->{'holiday'}{$m}{21}
            && !$this->{'holiday'}{$m}{22} )
        {
            $this->{'holiday'}{$m}{ $d - 1 }
                = ($default::english)
                ? 'Holiday'
                : '国民祝日法による休日';
        }
        $this->{'holiday'}{$m}{$d} = ($default::english) ? $e : $j;
    }
}

sub is_holiday ($$$) {
    my ( $this, $m, $d ) = @_;
    return (   $m
            && $d
            && ref $this->{'holiday'}{$m}
            && $this->{'holiday'}{$m}{$d} ) ? 1 : 0;
}

sub name ($$$) {
    my ( $this, $m, $d ) = @_;
    return (   $m
            && $d
            && ref $this->{'holiday'}{$m}
            && $this->{'holiday'}{$m}{$d} ) ? $this->{'holiday'}{$m}{$d} : '';
}

1;
