package IndexSweeper;
require CMONOS::CGI_Lib::Base;
@ISA = qw(Base);
use strict;
use utf8;
######################################################################
# インデックスなどから関連リンクを削除
# CMONOS::CGI_Lib::IndexSweeper.pm ver1.0.1
# Copyright (C) 2008-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2012-05-16
######################################################################
#
#	IndexSweeper->new(
#		'index_files'	=>	[
#								$env::externalfiles . '/metaindex.ssi',
#								$env::externalfiles . '/recentcomments.ssi',
#								$env::externalfiles . '/recenttrackbacks.ssi'
#							],
#		'rss_files'		=>	[
#								'../../index.xml',
#								'../../comment.xml',
#								'../../trackback.xml',
#								'../../podcast.xml'
#							],
#		'kml_files'		=>	[
#								'../../news/index.kml',
#							],
#	)->sweep(
#		[
#			url
#		],
#		{
#			cgi_id		=>	[
#								category_id
#							]
#		},
#		regexp_enabled
#	);
#
######################################################################
sub new ($;@) {
    my $pkg  = shift;
    my $this = {@_};
    bless $this;
}

sub sweep ($$$;$) {
    my ( $this, $url_ref, $cgi_id_ref, $regexp_enabled ) = @_;
    if (   ref $this->{'index_files'}
        && @{ $this->{'index_files'} }
        && ref $cgi_id_ref )
    {
        foreach my $i ( @{ $this->{'index_files'} } ) {
            $this->_sweep_index( $i, $cgi_id_ref );
        }
    }
    if ( ref $url_ref && @$url_ref ) {
        my @url
            = ($regexp_enabled) ? @$url_ref : map( quotemeta($_), @$url_ref );
        if ( ref $this->{'rss_files'} && @{ $this->{'rss_files'} } ) {
            foreach my $i ( @{ $this->{'rss_files'} } ) {
                $this->_sweep_rss( $i, \@url );
            }
        }
        if ( ref $this->{'kml_files'} && @{ $this->{'kml_files'} } ) {
            foreach my $i ( @{ $this->{'kml_files'} } ) {
                $this->_sweep_kml( $i, \@url );
            }
        }
    }
}

######################################################################
sub _mk_template_from ($$) {
    my ( $this, $template ) = @_;
    my %count = ();
    if ( $template
        =~ s/<!--([0-9a-zA-Z_]+)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<!--\/\1-->[\x09\x0A\x0C\x0D\x20]*/(($count{$1}++) ? '' : $this->_mk_template_tag($1,$2,$3))/egs
        )
    {
        return $template;
    }
    else {
        return '';
    }
}

sub _mk_template_tag ($$$$) {
    my ( $this, $tag, $setting, $value ) = @_;
    return '' if $tag eq 'data';
    $setting =~ s/[\x09\x0A\x0C\x0D\x20]+(id|mtime|value)="([^"]+)"//g;
    if ( $value
        =~ s/\A[\x09\x0A\x0C\x0D\x20]*<\!--head(?![0-9a-zA-Z_])[^>]*-->\n*(.*?)\n*<\!--\/head(?![0-9a-zA-Z_])[^>]*-->[\x09\x0A\x0C\x0D\x20]*//s
        )
    {
        my $header = $1;
        $setting
            =~ s/[\x09\x0A\x0C\x0D\x20]+head[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"//sg;
        $setting .= ' head="' . $this->_escape($header) . '"';
    }
    if ( $value
        =~ s/[\x09\x0A\x0C\x0D\x20]*<\!--foot(?![0-9a-zA-Z_])[^>]*-->\n*((.(?!<\!--foot-->))*?)\n*<\!--\/foot(?![0-9a-zA-Z_])[^>]*-->[\x09\x0A\x0C\x0D\x20]*\Z//s
        )
    {
        my $footer = $1;
        $setting
            =~ s/[\x09\x0A\x0C\x0D\x20]+foot[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"//sg;
        $setting .= ' foot="' . $this->_escape($footer) . '"';
    }
    return
          '<!--'
        . $tag
        . $setting . '-->'
        . $this->_mk_template_from($value) . '<!--/'
        . $tag . '-->';
}

sub _escape ($$) {
    my ( $this, $html ) = @_;
    $html =~ s/\&/\&amp;/g;
    $html =~ s/"/\&quot;/g;
    $html =~ s/</\&lt;/g;
    $html =~ s/>/\&gt;/g;
    $html =~ s/(\x0D\x0A|\x0A|\x0D)/&#10;/gs;
    return $html;
}

sub _unescape ($$) {
    my ( $this, $str ) = @_;
    $str =~ s/&#10;/\x0A/gs;
    $str =~ s/\&quot;/"/g;
    $str =~ s/\&lt;/</g;
    $str =~ s/\&gt;/>/g;
    $str =~ s/\&amp;/&/gs;
    return $str;
}

######################################################################
sub _sweep_index ($$$) {
    my ( $this, $path, $cgi_id_ref ) = @_;
    my ( $data, $sweeped, $template );
    $data = $this->read_file( $path, 1 );
    return 0 if !$data;
    $sweeped  = 0;
    $template = $this->_mk_template_from($data);
    if ( $data
        =~ s/(<\!--index(?![0-9a-zA-Z_])([^>]*)-->.*?<\!--\/index-->)/$this->_sweep_subindex($1,$2,$cgi_id_ref,\$sweeped,\$template)/sge
        )
    {
        if ($data =~ /<\!--index(?![0-9a-zA-Z_])[^>]*-->.*?<\!--\/index-->/s )
        {
            $data =~ s/<\!--DELETED-->[\x09\x0A\x0C\x0D\x20]*//sg;
        }
        else {
            my $index_template
                = ( $template
                    =~ /(<\!--index(?![0-9a-zA-Z_])[^>]*-->.*?<\!--\/index-->)/s
                ) ? $1 : '';
            $data =~ s/<\!--DELETED-->/$index_template/s;
            $data =~ s/<\!--DELETED-->[\x09\x0A\x0C\x0D\x20]*//sg;
        }
    }
    while ( my ( $cgi_id, $delete_categories ) = each %{$cgi_id_ref} ) {
        $data
            =~ s/(<\!--(metaindex|recommendedindex|recentcomments|recenttrackbacks)(?![0-9a-zA-Z_])[^>]*-->.*?<\!--\/\2-->)/$this->_sweep_article($1,$2,$cgi_id,$delete_categories,\$sweeped,\$template)/sge;
    }
    return 0 if !$sweeped;
    $this->write_file( $path, \$data, 1 );
    $this->mk_external_javascript($path);
}

sub _sweep_subindex ($$$$$$) {
    my ( $this, $index, $setting, $cgi_id_ref, $sweeped, $template_ref ) = @_;
    if ( $setting
        =~ /[\x09\x0A\x0C\x0D\x20]+id[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/s
        )
    {
        my $cid = $1;
        if ( defined $cgi_id_ref->{$cid} ) {
            if ( ref $cgi_id_ref->{$cid} && @{ $cgi_id_ref->{$cid} } ) {
                if ( $index
                    =~ s/(<\!--subindex(?![0-9a-zA-Z_])([^>]*)-->.*?<\!--\/subindex-->)/$this->__sweep_subindex($1,$2,$cgi_id_ref->{$cid},$sweeped,$template_ref)/sge
                    )
                {
                    if ( $index
                        =~ /<\!--subindex(?![0-9a-zA-Z_])[^>]*-->.*?<\!--\/subindex-->/s
                        )
                    {
                        $index
                            =~ s/<\!--DELETED-->[\x09\x0A\x0C\x0D\x20]*//sg;
                    }
                    else {
                        my $index_template
                            = ( $$template_ref
                                =~ /(<\!--subindex(?![0-9a-zA-Z_])[^>]*-->.*?<\!--\/subindex-->)/s
                            ) ? $1 : '';
                        $index =~ s/<\!--DELETED-->/$index_template/s;
                        $index
                            =~ s/<\!--DELETED-->[\x09\x0A\x0C\x0D\x20]*//sg;
                    }
                }
            }
            else {
                $$sweeped++;
                return '<!--DELETED-->';
            }
        }
    }
    return $index;
}

sub __sweep_subindex ($$$$$) {
    my ( $this, $subindex, $setting, $delete_categories, $sweeped ) = @_;
    if ( $setting
        =~ /[\x09\x0A\x0C\x0D\x20]+id[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/s
        && grep( $_ eq $1, @$delete_categories ) )
    {
        $$sweeped++;
        return '<!--DELETED-->';
    }
    else {
        return $subindex;
    }
}

sub _sweep_article ($$$$$$$) {
    my ( $this, $index, $tag, $cgi_id, $delete_categories, $sweeped,
        $template_ref )
        = @_;
    $cgi_id =~ s/^_+//;
    $cgi_id =~ s/_/-/g;
    if ( $index
        =~ s/(<\!--article(?![0-9a-zA-Z_])([^>]*)-->.*?<\!--\/article-->)/$this->__sweep_article($1,$2,$cgi_id,$delete_categories,$sweeped)/sge
        )
    {
        if ( $index
            =~ /<\!--article(?![0-9a-zA-Z_])[^>]*-->.*?<\!--\/article-->/s )
        {
            $index =~ s/<\!--DELETED-->[\x09\x0A\x0C\x0D\x20]*//sg;
        }
        elsif ( $$template_ref
            =~ /(<\!--$tag(?![0-9a-zA-Z_])[^>]*-->.*?<\!--\/$tag-->)/s )
        {
            return $1;
        }
        else {
            my $index_template
                = ( $$template_ref
                    =~ /(<\!--article(?![0-9a-zA-Z_])[^>]*-->.*?<\!--\/article-->)/s
                ) ? $1 : '';
            $index =~ s/<\!--DELETED-->/$index_template/s;
            $index =~ s/<\!--DELETED-->[\x09\x0A\x0C\x0D\x20]*//sg;
        }
        return $index;
    }
    else {
        return ( $$template_ref
                =~ /(<\!--$tag(?![0-9a-zA-Z_])[^>]*-->.*?<\!--\/$tag-->)/s )
            ? $1
            : $index;
    }
}

sub __sweep_article ($$$$$$) {
    my ( $this, $article, $setting, $cgi_id, $delete_categories, $sweeped )
        = @_;
    if ($setting
        =~ /[\x09\x0A\x0C\x0D\x20]+id[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"/s
        && ( ( !@$delete_categories && $1 =~ /^$cgi_id\-/ )
            || grep( $1 =~ /^$cgi_id-$_(-|$)/, @$delete_categories ) )
        )
    {
        $$sweeped++;
        return '<!--DELETED-->';
    }
    else {
        return $article;
    }
}

######################################################################
sub _sweep_rss ($$$) {
    my ( $this, $path, $url_ref ) = @_;
    my $data = $this->read_file( $path, 1 );
    return 0 if !$data;
    my $sweeped = 0;
    $data
        =~ s/([\t\x20]*<item(?![0-9a-zA-Z_])[^>]*>.*?<\/item>[\x0A\x0D]*)/$this->__sweep_rss($1,$url_ref,\$sweeped)/sge;
    return 0 if !$sweeped;
    $this->write_file( $path, \$data, 1 );
}

sub __sweep_rss ($$$$) {
    my ( $this, $item, $url_ref, $sweeped ) = @_;
    if ( $item =~ /<guid(?![0-9a-zA-Z_])[^>]*>(.*?)<\/guid>/s ) {
        my $guid = $1;
        $guid =~ s/^[\x09\x0A\x0C\x0D\x20]+//s;
        $guid =~ s/[\x09\x0A\x0C\x0D\x20]+$//s;
        $guid = $this->_unescape($guid);
        if ( grep( $guid =~ /^$_/, @$url_ref ) ) {
            $$sweeped++;
            return '';
        }
    }
    return $item;
}

######################################################################
sub _sweep_kml ($$$) {
    my ( $this, $path, $url_ref ) = @_;
    my $data = $this->read_file( $path, 1 );
    return 0 if !$data;
    my $sweeped = 0;
    $data
        =~ s/([\t\x20]*<(Placemark|NetworkLink)>.*?<\/\2>[\x0A\x0D]*)/$this->__sweep_kml($1,$url_ref,\$sweeped)/sge;
    return 0 if !$sweeped;
    $this->write_file( $path, \$data, 1 );
}

sub __sweep_kml ($$$$) {
    my ( $this, $place_mark, $url_ref, $sweeped ) = @_;
    if ( $place_mark
        =~ /<atom:link(?![0-9a-zA-Z_])[^>]*[\x09\x0A\x0C\x0D\x20]+href="([^"]*?)"[^>]*\/>/
        )
    {
        my $link = $this->_unescape($1);
        if ( grep( $link =~ /^$_/, @$url_ref ) ) {
            $$sweeped++;
            return '';
        }
    }
    return $place_mark;
}

1;
