package SetUp;
require CMONOS::CGI_Lib::Base;
@ISA = qw(Base);
use strict;
use utf8;
######################################################################
# CGIに共通する処理を行うモジュール
# SetUp.pm ver1.3.22
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新：2023-02-27
######################################################################
#
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	obj->print_html_header(handle);
#		ファイルヘッダ(PrintHTTPHeader)とHTMLヘッダを出力します。
#		事前にobj->set_cookie()が施してある場合、Set-Cookieヘッダも作成します。
#			handle	出力先ハンドル（設定の無い場合はSTDOUTになります。）
#
#	obj->set_alert(@alert);
#		JavaScript:alertの設定をします。
#
#	obj->set_title(title);
#		<title>の設定をします。
#
#	obj->set_description(description);
#		<meta name="description">の設定をします。全角100文字程度で切られます。
#
#	obj->set_body(
#				'text' => '#000000',
#				'link' => '#0000ff',
#				'alink' => '#00ffff',
#				'vlink' => '#000099',
#				'bgcolor' => '#ffffff',
#				'background' => 'background.gif',
#				'bgproperties' => 'fixed',
#				'topmargin' => '0',
#				'leftmargin' => '0'
#			 );
#		<body>の設定をします。
#
#	obj->set_javascript(javascript);
#		JavaScriptを設定します。
#
#	obj->set_css(css);
#		CSSを設定します。
#
#	obj->set_style_sheet_link(path);
#			path		スタイルシートファイルのパス
#
#	obj->set_base(href);
#			href		ベースURLの設定
#
#	obj->set_html_header(header);
#			header	上記以外の追加ヘッダ
#
#	obj->reset_html_header;
#			ヘッダのクリア
#
# ---------------------------------------------------------------------------------------
#	obj->print_password_check_form(key)
#		パスワードチェック画面を書き出して終了します。
#			key		ページ設定キー
#
#	obj->print_error_dialog(key)
#		エラーメッセージを書き出して終了します。
#
# ---------------------------------------------------------------------------------------
#	obj->print_page_header(key,\*handle,output,remain,cginame,title)
#		パッケージの設定値に基づきページヘッダを出力します。
#			key			ページ設定キー
#			*handle	出力先ハンドル
#			output		出力先パス
#			remain		1=出力終了後もヘッダ情報を保持する
#			cginame		CGIの名前
#			title		記事タイトル
#
#	obj->print_dialog(key,@msg)
#		パッケージの設定値に基づきダイアログページを出力します（出力後即終了）。
#			msg			メッセージ
#
# ---------------------------------------------------------------------------------------
#	obj->print_page_footer(key,\*handle)
#		パッケージの設定値に基づきページフッタを出力します。
#			key			ページ設定キー
#			*handle	出力先ハンドル
#
# ---------------------------------------------------------------------------------------
#	obj->frame_start(key,width,class,id,title,path)
#		装飾スタートタグを返します。
#			key			装飾キー
#			width		装飾幅
#			class		クラス
#			id			ID
#			title		タイトル部分
#			path		ファイルの出力先
#
#	obj->frame_end(key,width,class,id,title)
#		装飾終了タグを返します。
#
# ---------------------------------------------------------------------------------------
#	obj->get_frame(key,contents,title,tag)
#		装飾タグを返します。
#
# ---------------------------------------------------------------------------------------
#	obj->use_field(package,DB,setting,NoStyle,Tag,TAG,all)
#		フィールド設定を読み込みます
#			package		フィールドオブジェクトの属するパッケージ $package::obj{'key'}
#			DB			フィールド設定の格納されたデータベースのパス
#			setting		読み込みセッティング（1=読み込み/2=CGIオリジナルも読み込み）
#			NoStyle		1=フォント設定を無視/2=フォント設定スタイル設定を無視
#			Tag			tag=タグのみ/all=全ての設定を付記/no=タグ無し
#			TAG			フィールドタグ指定 <!--field-->
#			all			1=全てのフィールドを読み込む
#
# ---------------------------------------------------------------------------------------
#	obj->use_table
#		テーブル設定を読み込みます
#
# ---------------------------------------------------------------------------------------
#	obj->print_tool_icon
#		ツールへのジャンプアイコンを設定します。アイコンサイズは20×20ピクセルにして下さい。
#
#		アイコン画像ファイルの名前
#		モジュールキー-モジュール名-ウィンドウ幅-ウィンドウ高さ.gif
#		ex)index-Index-640-480.gif
#
# ---------------------------------------------------------------------------------------
#	obj->notice_mail_for_master(subject,msg[,\%field,\%data,[@addr]]);
#		新規入力通知メールを管理人他に発送します。また、送信した内容を返します。
#			subject		メールのタイトル
#			msg			メールの内容（指定がない場合自動的に作成します。）
#			\%field		フィールドオブジェクトハッシュのリファレンス
#			\%data		データハッシュのリファレンス
#			@addr			追加の送信先
#
#			※ $mail::file を添付します。
#
#	obj->notice_mail(subject,msg,\%field,\%data,@addr);
#		新規入力通知メールを発送します。また、送信した内容を返します。
#
######################################################################
#	特殊変数
#		$cgi::output	ファイルを書き出す位置
#
######################################################################
sub unauthorized_alert ($) {
    my $this = shift;
    if ( $this->{'access_forbidden'} ) {
        $this->error403(1);
    }
    else {
        $this->print_password_check_form($setting::key);
    }
}

######################################################################
# ヘッダの出力
sub print_html_header ($*;$$) {
    my ( $this, $handle, $encoding, $lang ) = @_;
    my ( $is_xhtml, $is_html5 );
    $encoding = 'UTF-8' if !$encoding;
    $lang = ( $env::locale && $env::locale =~ /^([a-zA-Z]{2})/ ) ? lc $1 : 'ja'
      if !$lang;
    if ( !$handle || scalar(*$handle) eq scalar(*STDOUT) ) {
        $handle = \*STDOUT;
        $this->print_http_header( 'text/html; charset=' . $encoding );
    }
    if ( $encoding =~ /utf-8/i ) {
        binmode $handle, ":utf8";
    }
    else {
        binmode $handle;
    }
    if ( defined $default::doctype && $default::doctype ) {
        if ( $default::doctype =~ /<!DOCTYPE html>/i ) {
            $is_html5 = 1;
        }
        elsif ( $default::doctype =~ /XHTML/ ) {

       #		Win-IE6 で表示に不具合が出る
       #		print $handle '<?xml version="1.0" encoding="',$encoding,'"?>',"\x0A";
            $is_xhtml = 1;
        }
        print $handle $default::doctype, "\x0A";
    }
    elsif ( defined $env::doctype && $env::doctype ) {
        if ( $env::doctype =~ /<!DOCTYPE html>/i ) {
            $is_html5 = 1;
        }
        elsif ( $env::doctype =~ /XHTML/ ) {

       #		Win-IE6 で表示に不具合が出る
       #		print $handle '<?xml version="1.0" encoding="',$encoding,'"?>',"\x0A";
            $is_xhtml = 1;
        }
        print $handle $env::doctype, "\x0A";
    }
    else {
        print $handle
'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">',
          "\x0A";
    }
    if ( defined $is_xhtml && $is_xhtml ) {
        print $handle
'<html xmlns="http://www.w3.org/1999/xhtml" xmlns:og="http://ogp.me/ns#"'
          . (
            ($env::fb_appid)
            ? ' xmlns:fb="http://www.facebook.com/2008/fbml"'
            : ''
          )
          . ' xml:lang="'
          . $lang
          . '" lang="'
          . $lang
          . '">', "\x0A";
    }
    else {
        print $handle '<html xmlns:og="http://ogp.me/ns#"'
          . (
            ($env::fb_appid)
            ? ' xmlns:fb="http://www.facebook.com/2008/fbml"'
            : ''
          )
          . ' lang="'
          . $lang
          . '">', "\x0A";
    }

    print $handle '<head>', "\x0A";
    if ($is_html5) {
        print $handle '<meta charset="', $encoding, '" />', "\x0A";
    }
    else {
        print $handle
          '<meta http-equiv="Content-Type" content="text/html;charset=',
          $encoding, '" />', "\x0A";
        print $handle
          '<meta http-equiv="Content-Script-Type" content="text/javascript" />',
          "\x0A";
        print $handle
          '<meta http-equiv="Content-Style-Type" content="text/css" />',
          "\x0A";
    }
    print $handle $this->meta_last_modified, "\x0A"
      if $this->{'Last_Modified'};
    print $handle '<meta name="robots" content="noindex,nofollow" />', "\x0A"
      if $env::noindex;
    print $handle '<meta name="generator" content="http://cmonos.jp/" />',
      "\x0A"
      if !$env::no_generator;
    print $handle $env::html_header . "\x0A"
      if $env::html_header && !$this->{'common_html_header_disabled'};

    print $handle '<title lang="' . $lang . '">',
      $this->escape_except_html_entity( $this->{'page_title'} ),
      '</title>', "\x0A";
    print $handle $this->{'basehref'}, "\x0A"
      if defined $this->{'basehref'} && $this->{'basehref'};

    #favicon.ico
    if ( !defined $this->{'html_header'}
        || $this->{'html_header'} !~ / rel="shortcut icon"/i )
    {
        my $favicon_path = $this->favicon_path;
        print $handle '<link rel="shortcut icon" href="'
          . (
            ($cgi::output)
            ? $this->base_change( $favicon_path, $cgi::output )
            : $favicon_path
          )
          . '" />', "\x0A"
          if $favicon_path ne '';
    }

    print $handle $this->{'stylesheetlink'}
      if defined $this->{'stylesheetlink'};
    print $handle $this->{'javascriptlink'}
      if defined $this->{'javascriptlink'};
    print $handle $this->{'html_header'},
      ( ( $this->{'html_header'} !~ /\x0A$/s ) ? "\x0A" : '' )
      if defined $this->{'html_header'} && $this->{'html_header'};
    print $handle $this->javascript( $this->{'JavaScript'}, $is_xhtml ), "\x0A"
      if $this->{'JavaScript'};
    if ( $this->{'ga'} ) {
        if ( $env::google_analytics_uacct =~ /^UA-/ ) {
            print $handle "<!-- Google Analytics -->\x0A";
            print $handle "<script>\x0A";
            print $handle
"  (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){\x0A";
            print $handle
"  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),\x0A";
            print $handle
"  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)\x0A";
            print $handle
"  })(window,document,'script','https://www.google-analytics.com/analytics.js','ga');\x0A";
            print $handle "  ga('create', '"
              . $env::google_analytics_uacct
              . "', 'auto');\x0A";
            print $handle "  ga('send', 'pageview');\x0A";
            print $handle "</script>\x0A";
            print $handle "<!-- End Google Analytics -->\x0A";
        }
        elsif ( $env::google_analytics_uacct =~ /^G-/ ) {
            print $handle
              '<script async src="https://www.googletagmanager.com/gtag/js?id='
              . $env::google_analytics_uacct
              . '"></script>';
            print $handle "\x0A";
            print $handle "<script>\x0A";
            print $handle "  window.dataLayer = window.dataLayer || [];\x0A";
            print $handle "  function gtag(){dataLayer.push(arguments);}\x0A";
            print $handle "  gtag('js', new Date());\x0A";
            print $handle "  gtag('config', '"
              . $env::google_analytics_uacct
              . "');\x0A";
            print $handle "</script>\x0A";
        }
    }
    if ( defined $this->{'CSS'} && $this->{'CSS'} ) {
        print $handle '<style type="text/css">', "\x0A";
        print $handle $this->{'CSS'};
        print $handle '</style>', "\x0A";
    }
    print $handle '</head>', "\x0A";
    print $handle '<body'
      . ( ( defined $this->{'body_property'} ) ? $this->{'body_property'} : '' )
      . (
        ( defined $this->{'onload'} && $this->{'onload'} ne '' )
        ? ' onload="' . $this->{'onload'} . '"'
        : ''
      ) . '>', "\x0A";
}

sub meta_last_modified ($$) {
    my ( $this, $time ) = @_;
    $time = $this->{'Last_Modified'} if !$time && $this->{'Last_Modified'};
    return ($time)
      ? '<meta http-equiv="Last-Modified" content="'
      . $this->server_gmtime($time)
      . '" data-unix-timestamp="'
      . $time . '" />'
      : '';
}

# onloadの設定
sub set_onload ($;@) {
    my ( $this, @onload ) = @_;
    return if !@onload;
    my ($onload);
    foreach $onload (@onload) {
        next if $onload eq '';
        $onload =~ s/"/'/g;
        $this->{'onload'} .= $onload . ( ( $onload !~ /;$/ ) ? ';' : '' );
    }
}

# titleの設定
sub set_title ($;$) {
    my ( $this, $title ) = @_;
    $this->{'page_title'} = $title if defined $title && $title ne '';
}

sub set_page_title ($;$) {
    my ( $this, $title ) = @_;
    $this->{'page_title_html'} = $title if defined $title && $title ne '';
}

# meta og の設定
sub set_description ($$;$) {
    my ( $this, $description, $check ) = @_;
    return if $check && !$description;
    $this->set_og_metadata( 'description', $this->detag_text($description) );
}

sub set_og_title ($$;$) {
    my ( $this, $title, $check ) = @_;
    return if $check && !$title;
    $this->set_og_metadata( 'title', $title );
}

sub set_og_image ($$;$) {
    my ( $this, $image, $check ) = @_;
    return if $check && !$image;
    $this->set_og_metadata( 'image', $image );
}

sub set_og_latlng ($$$;$) {
    my ( $this, $lat, $lng, $check ) = @_;
    return if $check && ( !$lat || !$lng );
    $this->set_og_metadata( 'latitude',  $lat );
    $this->set_og_metadata( 'longitude', $lng );
}

sub set_og_url ($$;$) {
    my ( $this, $url, $check ) = @_;
    return if $check && !$url;
    $url =~ s!/index\.s?html?(\#[^/]+|)$!/$1!is if $url;
    $this->set_og_metadata( 'url', $url );
}

sub set_og_metadata ($@) {
    my ( $this, @metadata ) = @_;
    if (@metadata) {
        $this->{'meta_data'} = {} if !ref $this->{'meta_data'};
        for ( my $i = 0 ; $i < @metadata ; $i += 2 ) {
            $this->{'meta_data'}{ $metadata[$i] } = $metadata[ $i + 1 ];
        }
    }
}

sub set_og_cgi_summary ($;$) {
    my ( $this, $is_article ) = @_;
    if (   %setting::general
        && $setting::general{'ogimage'}
        && ( $default::file || $default::home ) )
    {
        $this->set_og_image(
            $this->full_path(
                ( $default::file || $default::home ) . '/'
                  . $setting::general{'ogimage'}
            )
        );
    }
    elsif (%setting::general
        && $setting::general{'icon'}
        && $setting::general{'iconwidth'}
        && $setting::general{'iconheight'}
        && $setting::general{'iconwidth'} >= 50
        && $setting::general{'iconheight'} >= 50
        && ( $default::file || $default::home ) )
    {
        $this->set_og_image(
            $this->full_path(
                ( $default::file || $default::home ) . '/'
                  . $setting::general{'icon'}
            )
        );
    }
    elsif (%master::general
        && $master::general{'ogimage'}
        && ( $default::file || $default::home ) )
    {
        $this->set_og_image(
            $this->full_path(
                ( $default::file || $default::home ) . '/'
                  . $master::general{'ogimage'}
            )
        );
    }
    elsif (%master::general
        && $master::general{'icon'}
        && $master::general{'iconwidth'}
        && $master::general{'iconheight'}
        && $master::general{'iconwidth'} >= 50
        && $master::general{'iconheight'} >= 50
        && ( $default::master_file_dir || $master::home ) )
    {
        $this->set_og_image(
            $this->full_path(
                ( $default::master_file_dir || $master::home ) . '/'
                  . $setting::general{'icon'}
            )
        );
    }
    if ( %setting::general && $setting::general{'description'} ) {
        $this->set_description( $setting::general{'description'} );
    }
    elsif ( %master::general && $master::general{'description'} ) {
        $this->set_description( $master::general{'description'} );
    }
    if ($is_article) {
        my $type = $setting::general{'ogtype'} || $env::og_type;
        $this->set_og_metadata(
            'type',
            (
                ( $type eq 'blog' || $type eq 'website' )
                ? 'article'
                : $type
            )
        );
    }
}

# BODYの設定
sub set_body ($;@) {
    my $this = shift;
    my (%body_property);
    %body_property = @_;

    $this->{'body_property'} = '';
    my ( $body, $link, $alink, $vlink, $hover, $css, $legacy_mode );
    $legacy_mode = (
        (
            defined $body_property{'marginwidth'}
              && $body_property{'marginwidth'} ne ''
        )
          || ( defined $body_property{'marginheight'}
            && $body_property{'marginheight'} ne '' )
    ) ? 1 : 0;
    $css = '';

    #BODYのスタイル
    if (   defined $body_property{'text'}
        && $body_property{'text'} ne ''
        && !$this->is_invalid_color( $body_property{'text'} ) )
    {
        $body .= '  color: ' . $body_property{'text'} . ';' . "\x0A";
        $this->{'body_property'} .= ' text="' . $body_property{'text'} . '"'
          if $legacy_mode;
    }
    if (   defined $body_property{'bgcolor'}
        && $body_property{'bgcolor'} ne ''
        && !$this->is_invalid_color( $body_property{'bgcolor'} ) )
    {
        $body .=
          '  background-color: ' . $body_property{'bgcolor'} . ';' . "\x0A";
        $this->{'body_property'} .=
          ' bgcolor="' . $body_property{'bgcolor'} . '"'
          if $legacy_mode;
    }
    if (   defined $body_property{'background'}
        && $body_property{'background'} ne ''
        && !$this->is_invalid_image( $body_property{'background'} ) )
    {
        $body .= '  background-image: url("'
          . $body_property{'background'} . '");' . "\x0A";
        $this->{'body_property'} .=
          ' background="' . $body_property{'background'} . '"'
          if $legacy_mode;
    }
    if ( defined $body_property{'bgproperties'}
        && $body_property{'bgproperties'} =~ /^(fixed|scroll)$/i )
    {
        $body .= '  background-attachment: '
          . $body_property{'bgproperties'} . ';' . "\x0A";
        $this->{'body_property'} .=
          ' bgproperties="' . $body_property{'bgproperties'} . '"'
          if $legacy_mode;
    }

    #余白
    if ( defined $body_property{'topmargin'}
        && $body_property{'topmargin'} ne '' )
    {
        $body .=
            '  margin-top: '
          . $body_property{'topmargin'}
          . (
            (
                     $body_property{'topmargin'}
                  && $body_property{'topmargin'} !~ /[a-zA-Z\%]$/
            ) ? 'px' : ''
          ) . ';' . "\x0A";
        $this->{'body_property'} .=
          ' topmargin="' . $this->escape( $body_property{'topmargin'} ) . '"'
          if $legacy_mode;
    }
    if ( defined $body_property{'leftmargin'}
        && $body_property{'leftmargin'} ne '' )
    {
        $body .=
            '  margin-left: '
          . $body_property{'leftmargin'}
          . (
            (
                     $body_property{'leftmargin'}
                  && $body_property{'leftmargin'} !~ /[a-zA-Z\%]$/
            ) ? 'px' : ''
          ) . ';' . "\x0A";
        $this->{'body_property'} .=
          ' leftmargin="' . $this->escape( $body_property{'leftmargin'} ) . '"'
          if $legacy_mode;
    }
    if ( defined $body_property{'rightmargin'}
        && $body_property{'rightmargin'} ne '' )
    {
        $body .=
            '  margin-right: '
          . $body_property{'rightmargin'}
          . (
            (
                     $body_property{'rightmargin'}
                  && $body_property{'rightmargin'} !~ /[a-zA-Z\%]$/
            ) ? 'px' : ''
          ) . ';' . "\x0A";
        $this->{'body_property'} .=
          ' rightmargin="'
          . $this->escape( $body_property{'rightmargin'} ) . '"'
          if $legacy_mode;
    }
    if ( defined $body_property{'bottommargin'}
        && $body_property{'bottommargin'} ne '' )
    {
        $body .=
            '  margin-bottom: '
          . $body_property{'bottommargin'}
          . (
            (
                     $body_property{'bottommargin'}
                  && $body_property{'bottommargin'} !~ /[a-zA-Z\%]$/
            ) ? 'px' : ''
          ) . ';' . "\x0A";
        $this->{'body_property'} .=
          ' bottommargin="'
          . $this->escape( $body_property{'bottommargin'} ) . '"'
          if $legacy_mode;
    }
    if ($legacy_mode) {
        $this->{'body_property'} .=
          ' marginwidth="'
          . $this->escape( $body_property{'marginwidth'} ) . '"'
          if defined $body_property{'marginwidth'}
          && $body_property{'marginwidth'} ne '';
        $this->{'body_property'} .=
          ' marginheight="'
          . $this->escape( $body_property{'marginheight'} ) . '"'
          if defined $body_property{'marginheight'}
          && $body_property{'marginheight'} ne '';
    }
    $body .= $body_property{'bodystyle'} . "\x0A"
      if defined $body_property{'bodystyle'}
      && $body_property{'bodystyle'} ne '';
    if ( defined $body && $body ne '' ) {
        $css .= 'body {' . "\x0A" . $body . '}' . "\x0A";
        undef $body;
    }

    #Pの行間隔
    $css .= 'p {line-height: ' . $body_property{'lineheight'} . '}' . "\x0A"
      if defined $body_property{'lineheight'}
      && $body_property{'lineheight'} ne '';

    #リンク
    if (   defined $body_property{'link'}
        && $body_property{'link'} ne ''
        && !$this->is_invalid_color( $body_property{'link'} ) )
    {
        $link .= 'color: ' . $body_property{'link'} . ';';
        $this->{'body_property'} .= ' link="' . $body_property{'link'} . '"'
          if $legacy_mode;
    }
    $link .= ' text-decoration: ' . $body_property{'linkdecoration'}
      if defined $body_property{'linkdecoration'}
      && $body_property{'linkdecoration'} ne '';
    $css .= 'a:link {' . $link . '}' . "\x0A" if defined $link && $link;
    undef $link;

    if (   defined $body_property{'vlink'}
        && $body_property{'vlink'} ne ''
        && !$this->is_invalid_color( $body_property{'vlink'} ) )
    {
        $vlink .= 'color: ' . $body_property{'vlink'} . ';';
        $this->{'body_property'} .= ' vlink="' . $body_property{'vlink'} . '"'
          if $legacy_mode;
    }
    $vlink .= ' text-decoration: ' . $body_property{'vlinkdecoration'}
      if defined $body_property{'vlinkdecoration'}
      && $body_property{'vlinkdecoration'} ne '';
    $css .= 'a:visited {' . $vlink . '}' . "\x0A"
      if defined $vlink && $vlink;
    undef $vlink;

    $hover .= 'color: ' . $body_property{'hover'} . ';'
      if defined $body_property{'hover'}
      && $body_property{'hover'} ne ''
      && !$this->is_invalid_color( $body_property{'hover'} );
    $hover .= ' text-decoration: ' . $body_property{'hoverdecoration'}
      if defined $body_property{'hoverdecoration'}
      && $body_property{'hoverdecoration'} ne '';
    $css .= 'a:hover {' . $hover . '}' . "\x0A"
      if defined $hover && $hover;
    undef $hover;

    if (   defined $body_property{'alink'}
        && $body_property{'alink'} ne ''
        && !$this->is_invalid_color( $body_property{'alink'} ) )
    {
        $alink .= 'color: ' . $body_property{'alink'} . ';';
        $this->{'body_property'} .= ' alink="' . $body_property{'alink'} . '"'
          if $legacy_mode;
    }
    $alink .= ' text-decoration: ' . $body_property{'alinkdecoration'}
      if defined $body_property{'alinkdecoration'}
      && $body_property{'alinkdecoration'} ne '';
    $css .= 'a:active {' . $alink . '}' . "\x0A"
      if defined $alink && $alink;
    undef $alink;

    $this->{'CSS'} = $css . ( ( defined $this->{'CSS'} ) ? $this->{'CSS'} : '' )
      if $css ne '';
    $this->{'body_property'} .=
      ' id="' . $this->escape( $body_property{'id'} ) . '"'
      if defined $body_property{'id'} && $body_property{'id'} ne '';
    $this->{'body_property'} .=
      ' class="' . $this->escape( $body_property{'class'} ) . '"'
      if defined $body_property{'class'} && $body_property{'class'} ne '';

}

# JavaScriptの設定
sub set_javascript ($;@) {
    my ( $this, @javascript ) = @_;
    return if !@javascript;
    my $js;
    foreach $js (@javascript) {
        next if !$js;
        $this->{'JavaScript'} .=
          $js . ( ( $js =~ /\x0A$/s ) ? '' : "\x0A" );
    }
}

# 外部スクリプトパスの設定
sub set_javascript_link ($@) {
    my ( $this, @uri ) = @_;
    return if !@uri;
    my $uri;
    foreach $uri (@uri) {
        next if !defined $uri || !$uri;
        $this->{'javascriptlink'} .= $this->javascript_link($uri) . "\x0A";
    }
}

# CSSの設定
sub set_css ($;@) {
    my ( $this, @css ) = @_;
    return if !@css;
    my $css;
    foreach $css (@css) {
        next if !defined $css || $css eq '';
        $this->{'CSS'} .= $css . ( ( $css =~ /\x0A$/s ) ? '' : "\x0A" );
    }
}

# LINKの設定
sub set_link_rel ($$;$) {
    my ( $this, $link, $clear ) = @_;
    delete $this->{'page_link'} if $clear;
    if ( ref $link eq 'HASH' ) {
        if ( !ref $this->{'page_link'} ) {
            $this->{'page_link'} = $link;
        }
        else {
            while ( my ( $rel, $values ) = each %$link ) {
                $this->{'page_link'}->{$rel} = $values;
            }
        }
    }
    else {
        delete $this->{'page_link'};
    }
}

sub set_canonical ($$;$) {
    my ( $this, $href, $raw_url ) = @_;
    return if !$href;
    $href =~ s!/index\.s?html?(\#[^/]+|)$!/$1!is if !$raw_url;
    $this->set_link_rel(
        { 'canonical' => { 'href' => $this->escape($href) } } );
    $this->set_og_metadata( 'url', $href );
}

sub replace_meta_data ($$;$$) {
    my ( $this, $html, $metadata_hash, $template ) = @_;
    my $metadata = '';
    my $og       = '';
    $metadata_hash = {} if !ref $metadata_hash;
    $metadata_hash->{'app_id'} =
         $this->get_metadata_by_name( $template, 'fb:app_id' )
      || $env::fb_appid
      || ''
      if !defined $metadata_hash->{'app_id'};
    $metadata_hash->{'locale'} =
         $this->get_metadata_by_name( $template, 'og:locale' )
      || $env::locale
      || 'ja_JP'
      if !defined $metadata_hash->{'locale'};
    $metadata_hash->{'type'} =
         $this->get_metadata_by_name( $template, 'og:type' )
      || $setting::general{'ogtype'}
      || $env::og_type
      || 'article'
      if !defined $metadata_hash->{'type'};
    $metadata_hash->{'site_name'} =
         $this->get_metadata_by_name( $template, 'og:site_name' )
      || $this->get_metadata_by_name( $html,     'og:site_name' )
      || $env::title
      || ''
      if !defined $metadata_hash->{'site_name'};
    $metadata_hash->{'title'} =
         $this->get_metadata_by_name( $template, 'og:title' )
      || $this->get_metadata_by_name( $template, 'title' )
      || $this->get_metadata_by_name( $html,     'og:title' )
      || $this->get_metadata_by_name( $html,     'title' )
      || ''
      if !defined $metadata_hash->{'title'};
    $metadata_hash->{'description'} =
         $this->get_metadata_by_name( $template, 'og:description' )
      || $this->get_metadata_by_name( $template, 'description' )
      || $this->get_metadata_by_name( $html,     'og:description' )
      || $this->get_metadata_by_name( $html,     'description' )
      || ''
      if !defined $metadata_hash->{'description'};

    if ( !defined $metadata_hash->{'image'} ) {
        my $ogimage = $this->get_metadata_by_name( $template, 'og:image' );
        if ($ogimage) {
            $metadata_hash->{'image'} = $ogimage;
        }
        else {
            $ogimage = $this->image_file_whole_path( $default::file . '/og' )
              if $default::file;
            if ( !$ogimage ) {
                $ogimage =
                  $this->image_file_whole_path( $default::home . '/og' )
                  if $default::home;
                if ( !$ogimage ) {
                    $ogimage =
                      $this->image_file_whole_path(
                        $default::master_file_dir . '/og' )
                      if $default::master_file_dir;
                    if ( !$ogimage ) {
                        $ogimage =
                          $this->image_file_whole_path( $master::home . '/og' )
                          if $master::home;
                        $ogimage = $this->image_file_whole_path('../../og')
                          if !$ogimage;
                    }
                }
            }
            $metadata_hash->{'image'} = $this->full_path($ogimage)
              if $ogimage;
        }
    }
    foreach my $name ( sort { $a cmp $b } keys %$metadata_hash ) {
        if ( $name eq 'title' || $name eq 'description' ) {
            $$html =~
s/[\x20\x09]*<meta[^>]+name=["']$name["'][^>]*>[\x20\x09]*\x0A?/<og\/>/sig
              if ref $html && $$html;
            $metadata .=
                '<meta name="'
              . $name
              . '" content="'
              . $this->escape_except_html_entity(
                ( $name eq 'description' )
                ? $this->cut_string( $metadata_hash->{$name}, 201, 'i' )
                : $this->lf_to_space(
                    $this->escape_except_html_entity( $metadata_hash->{$name} )
                )
              )
              . '" />' . "\x0A"
              if defined $metadata_hash->{$name}
              && $metadata_hash->{$name} ne '';
        }
        my $prefix = ( $name eq 'app_id' ) ? 'fb' : 'og';
        $og .=
            '<meta property="'
          . $prefix . ':'
          . $name
          . '" content="'
          . $this->lf_to_space(
            $this->escape_except_html_entity( $metadata_hash->{$name} ) )
          . '" />' . "\x0A"
          if defined $metadata_hash->{$name}
          && $metadata_hash->{$name} ne '';
        $$html =~
s/[\x20\x09]*<meta[^>]+property=["']$prefix:$name["'][^>]*>[\x20\x09]*\x0A?//sig
          if ref $html && $$html;
    }
    $metadata .= $og;
    if ( ref $html ) {
        $$html = $metadata . $$html
          unless $$html =~ s!<og/>!$metadata!s
          || $$html =~ s!(<title[^>]*>.*?</title>)\x0A?!$1\x0A$metadata!si
          || $$html =~ s!(\x0A?)(<link[^>]*>)!$1$metadata$2!si;
        $$html =~ s!<og/>!!sg;
        $$html =~
s/(<html)(?![a-z])([^>]*)>/$1 . $this->supplement_xmlns($2,(($metadata_hash->{'app_id'}) ? 1 : 0)) . '>'/ies;
    }
    return $metadata;
}

sub supplement_xmlns ($$;$) {
    my ( $this, $attributes, $fb ) = @_;
    if (   $fb
        && $attributes !~
        m!xmlns:fb=["']http://www.facebook.com/2008/fbml["']!s )
    {
        $attributes =
          ' xmlns:fb="http://www.facebook.com/2008/fbml"' . $attributes;
    }
    if ( $attributes !~ m!xmlns:og=["']http://ogp.me/ns#["']!s ) {
        $attributes = ' xmlns:og="http://ogp.me/ns#"' . $attributes;
    }
    return $attributes;
}

sub get_metadata_by_name ($$$) {
    my ( $this, $html, $name ) = @_;
    return '' if !ref $html;
    if ( $$html =~ /<meta([^>]+)(?:property|name)=["']$name["']([^>]*)>/si ) {
        my $attributes = $1 . $2;
        if ( $attributes =~ /[\x20\x09\x0A\x0D]content=["']([^>]*?)["']/si ) {
            return $this->unescape($1);
        }
    }
    return '';
}

sub get_og_latlng ($$) {
    my ( $this, $html ) = @_;
    return (
        $this->get_metadata_by_name( $html, 'og:latitude' ),
        $this->get_metadata_by_name( $html, 'og:longitude' )
    );
}

sub get_last_modified ($$;$) {
    my ( $this, $html, $path ) = @_;
    return '' if !ref $html;
    if ( $$html =~ /<meta([^>]+)http-equiv=["']Last-Modified["']([^>]*)>/si ) {
        my $attributes = $1 . $2;
        if ( $attributes =~ /[\x20\x09\x0A\x0D]content=["']([^>]*?)["']/si ) {
            return $this->date_to_gmt( $this->unescape($1) );
        }
    }
    if ( $$html =~ /<!--mtime(?!\w)([^>]*)-->/si ) {
        my $attributes = $1;
        if ( $attributes =~ /[\x20\x09\x0A\x0D]value=["']([^>]*?)["']/si ) {
            return $this->date_to_gmt( $this->unescape($1) );
        }
    }
    if ( $$html =~ /<!--date(?!\w)([^>]*)-->/si ) {
        my $attributes = $1;
        if ( $attributes =~ /[\x20\x09\x0A\x0D]value=["']([^>]*?)["']/si ) {
            return $this->date_to_gmt( $this->unescape($1) );
        }
    }
    if ($path) {
        return ( ( stat $path )[9] || '' );
    }
    return '';
}

sub replace_link_rel ($$;$) {
    my ( $this, $html, $link, $appendable ) = @_;
    foreach my $rel ( sort { $a cmp $b } keys %{$link} ) {
        my $tag = '';
        if ( ref $link->{$rel} ) {
            my %link_attributes =
              $this->_link_attributes( $rel, $link->{$rel} );
            foreach my $name ( sort { $a cmp $b } keys %link_attributes ) {
                $tag .= ' '
                  . $name . '="'
                  . $this->escape_except_html_entity( $link_attributes{$name} )
                  . '"';
            }
            $tag = '<link rel="' . $rel . '"' . $tag . ' />' . "\x0A"
              if $tag;
        }
        if ( !defined $$html || $$html eq '' ) {
            $$html = $tag if $appendable;
        }
        else {
            unless ( $$html =~
s/<link(?![0-9a-zA-Z_])[^>]* rel="$rel"[^>]*>[\x09\x0A\x0C\x0D\x20]*/$tag/si
                || $$html =~
s/(<link(?![0-9a-zA-Z_])[^>]* rel="(prev|next|last|first|index|home|contents|canonical)"[^>]*>)[\x09\x0A\x0C\x0D\x20]*/$1\x0A$tag/si
                || $$html =~
                s/[\x09\x0A\x0C\x0D\x20]*<\/head>/\x0A$tag<\/head>/si )
            {
                $$html .= ( ( $$html !~ /\x0A$/s ) ? "\x0A" : '' ) . $tag
                  if $appendable;
            }
            $$html =~
s/<a(?![0-9a-zA-Z_])([^>]*)[\x09\x0A\x0C\x0D\x20]+rel="$rel"([^>]*)>/$this->_replace_anchor_rel($rel,$1 . $2,$link->{$rel})/iges;
            $$html =~
s/&lt;a(?![0-9a-zA-Z_])([^"]*?)[\x09\x0A\x0C\x0D\x20]+rel=&quot;$rel&quot;([^"]*?)&gt;/$this->_replace_anchor_rel($rel,$1 . $2,$link->{$rel},1)/iges;
            if ( $rel eq 'prev' ) {

#$$html =~ s/^(.*?)(<a(?![0-9a-zA-Z_])([^>]*[\x09\x0A\x0C\x0D\x20]+rel="prevsection"[^>]*)>|&lt;a(?![0-9a-zA-Z_])([^"]*?[\x09\x0A\x0C\x0D\x20]+rel=&quot;prevsection&quot;[^"]*?)&gt;)/$1 . $this->_replace_anchor_rel('prevsection',(($3) ? $3 : $4),$link->{'prev'},(($4) ? 1 : 0))/ies;
                $$html =~
s/^(.*?)<a(?![0-9a-zA-Z_])([^>]*[\x09\x0A\x0C\x0D\x20]+rel="prevsection"[^>]*)>/$1 . $this->_replace_anchor_rel('prevsection',$2,$link->{'prev'})/ies
                  || $$html =~
s/^(.*?)&lt;a(?![0-9a-zA-Z_])([^"]*?[\x09\x0A\x0C\x0D\x20]+rel=&quot;prevsection&quot;[^"]*?)&gt;/$1 . $this->_replace_anchor_rel('prevsection',$2,$link->{'prev'},1)/ies;
            }
            elsif ( $rel eq 'next' ) {

#$$html =~ s/(<a(?![0-9a-zA-Z_])([^>]*[\x09\x0A\x0C\x0D\x20]+rel="nextsection"[^>]*)>|&lt;a(?![0-9a-zA-Z_])([^"]*?[\x09\x0A\x0C\x0D\x20]+rel=&quot;nextsection&quot;[^"]*?)&gt;)((?:(?!(?:<a(?![0-9a-zA-Z_])[^>]*[\x09\x0A\x0C\x0D\x20]+rel="nextsection"[^>]*>|&lt;a(?![0-9a-zA-Z_])[^"]*?[\x09\x0A\x0C\x0D\x20]+rel=&quot;nextsection&quot;[^"]*?&gt;)).)*?)$/$this->_replace_anchor_rel('nextsection',(($2) ? $2 : $3),$link->{'next'},(($3) ? 1 : 0)) . $4/ies;
                $$html =~
s/<a(?![0-9a-zA-Z_])([^>]*[\x09\x0A\x0C\x0D\x20]+rel="nextsection"[^>]*)>((?:(?!<a(?![0-9a-zA-Z_])[^>]*[\x09\x0A\x0C\x0D\x20]+rel="nextsection"[^>]*>).)*?)$/$this->_replace_anchor_rel('nextsection',$1,$link->{'next'}) . $2/ies
                  || $$html =~
s/&lt;a(?![0-9a-zA-Z_])([^"]*?[\x09\x0A\x0C\x0D\x20]+rel=&quot;nextsection&quot;[^"]*?)&gt;((?:(?!(?:<a(?![0-9a-zA-Z_])[^>]*[\x09\x0A\x0C\x0D\x20]+rel="nextsection"[^>]*>|&lt;a(?![0-9a-zA-Z_])[^"]*?[\x09\x0A\x0C\x0D\x20]+rel=&quot;nextsection&quot;[^"]*?&gt;)).)*?)$/$this->_replace_anchor_rel('nextsection',$1,$link->{'next'},1) . $2/ies;
            }
        }
    }
    return $$html;
}

sub _link_attributes ($$$$;$) {
    my ( $this, $rel, $attributes_hash ) = @_;
    my %temp = %$attributes_hash;
    $temp{'href'} .= '#' . $temp{'anchor'}
      if $rel =~ /section$/i && $temp{'anchor'};
    delete $temp{'anchor'};
    return %temp;
}

sub _replace_anchor_rel ($$$$;$) {
    my ( $this, $rel, $attributes, $attributes_hash, $escape ) = @_;
    my $tag = '';
    $attributes = $this->unescape($attributes) if $escape;
    $attributes =~ s/[\x09\x0A\x0C\x0D\x20]+rel="[^"]*"//isg;
    if ( ref $attributes_hash ) {
        my %link_attributes = $this->_link_attributes( $rel, $attributes_hash );
        foreach my $name ( sort { $a cmp $b } keys %link_attributes ) {
            $attributes =~ s/[\x09\x0A\x0C\x0D\x20]+$name="[^"]*"//isg;
            $attributes .= ' '
              . $name . '="'
              . $this->escape_except_html_entity( $link_attributes{$name} )
              . '"';
        }
    }
    $tag = '<a' . $attributes . ' rel="' . $rel . '">';
    return ($escape) ? $this->escape($tag) : $tag;
}

# スタイルシートパスの設定
sub set_style_sheet_link ($;@) {
    my ( $this, @uri ) = @_;
    return if !@uri;
    my $uri;
    foreach $uri (@uri) {
        next if !defined $uri || !$uri;
        $this->{'stylesheetlink'} .=
          '<link rel="stylesheet" type="text/css" href="'
          . $uri . '" />' . "\x0A";
    }
}

# ベースURLパスの設定
sub set_base ($$) {
    my ( $this, $uri ) = @_;
    return if !defined $uri || !$uri;
    $this->{'basehref'} = '<base href="' . $uri . '" />' . "\x0A";
}

# GoogleAnalyticsの設定
sub set_google_analytics ($$) {
    my ( $this, $flag ) = @_;
    $this->{'ga'} = $flag;
}

# その他ヘッダの設定
sub set_html_header ($$) {
    my ( $this, $header ) = @_;
    return if !defined $header || !$header;
    $this->{'html_header'} .= $header;
}

# ヘッダのクリア
sub reset_html_header ($) {
    my $this = shift;
    delete $this->{'meta_data'};
    delete $this->{'html_header'};
    delete $this->{'common_html_header_disabled'};
    delete $this->{'basehref'};
    delete $this->{'stylesheetlink'};
    delete $this->{'CSS'};
    delete $this->{'javascriptlink'};
    delete $this->{'JavaScript'};
    delete $this->{'body_property'};
    delete $this->{'page_link'};
    delete $this->{'page_title'};
    delete $this->{'page_title_html'};
    delete $this->{'ga'};
    delete $this->{'onload'};
    delete $this->{'Set_Cookie'};
    delete $this->{'additional_topic_path'};
    delete $this->{'Last_Modified'};
}

# 承認待ち項目の警告
sub set_pending_item_alert ($) {
    my $this = shift;
    return 0 if !$default::preference_dir;
    my %pending;
    my $count = 0;
    my $alert = '';
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    tie( %pending, $env::dbm_file, $default::preference_dir . '/pending',
        O_RDONLY, $env::dbm_mode );

    if ( ref tied %pending ) {
        foreach my $editor_url (
            sort {
                     $this->sort_by( \$pending{$b}, \$pending{$a} )
                  || $this->sort_by( \$a,           \$b )
            } keys %pending
          )
        {
            my ( $mtime, $title ) = split(
                /\x09/,
                $this->decode_utf8(
                    $pending{ $this->encode_utf8($editor_url) }
                )
            );
            $alert .=
                '<li><a href="'
              . $this->escape($editor_url)
              . '" target="UserGeneratedFieldEditor" onclick="cmonos.window.open(\'UserGeneratedFieldEditor\',640,600)">'
              . ( ($title) ? $title : '未読の承認待ち項目' )
              . '</a> ('
              . $this->date( $mtime, 7, 'gmt/' )
              . ')</li>';
            $count++;
        }
    }
    untie %pending;
    push(
        @{ $this->{'dialog'} },
        '未読の承認待ち項目が '
          . $count
          . ' 件みつかりました。<ol>'
          . $alert . '</ol>'
    ) if $count;
    return 1;
}

########################################################################
#HTMLフッタの出力
sub print_html_footer ($) {
    my ( $this, $handle ) = @_;
    $handle = \*STDOUT if !$handle;
    print $handle '</body></html>';
    binmode $handle;
}

######################################################################
#	obj->print_password_check_form
sub print_password_check_form ($$;$$$$) {
    my ( $this, $key, $new_addr, $need_new_pswd, $query_params,
        $shift_required ) = @_;
    my ( $name, $additional_form, $hidden_item, $cancel_button, @query,
        $email_id, $pswd_id );
    $key = $setting::security_key if !$key && $setting::security_key;
    $setting::page{$key}{'title'}      = 'ログイン';
    $setting::page{$key}{'titleImage'} = '';
    $setting::page{$key}{'titleText'}  = '';
    $setting::page{$key}{'headText'}   = '';
    $email_id                          = 'email';
    $pswd_id                           = 'pswd';
    @query                             = @setting::form;

    if ($new_addr) {
        my @form = @setting::form;
        unshift( @form, 'email', 'pswd' )
          if !$env::session_enabled
          && defined $main::pswd
          && !ref $main::pswd
          && $main::pswd ne ''
          && !$cookie::read{ $this->login_cookie_name };
        $cancel_button =
          ' <form method="post" id="cancel" name="cancel" action="'
          . $cgi::path . '">';
        $cancel_button .= '<input type="hidden" name="_" value="＿" />';
        $cancel_button .= $this->set_query( 'post', \@form );
        $cancel_button .=
'<input type="button" value="&laquo; 戻る" onclick="history.back()" />'
          if !$this->is_ssl;
        $cancel_button .= '<input type="submit" value="元に戻す" /></form>';
        if ( $new_addr eq 'none' ) {
            push(
                @{ $this->{'dialog'} },
'認証に必要なメールアドレスがみつかりません。'
            );
            $this->print_page_header($key);
            print $cancel_button;
            $this->print_page_footer($key);
            exit;
        }
        else {
            while ( my ( $id, $field ) = each %{$field::obj} ) {
                delete $main::data{$id}{'str'}
                  if defined $field->{'elements'} && $field->{'elements'};
            }
            push( @query, 'a', 'data' );
            if ($need_new_pswd) {
                $additional_form .=
'<li><label><input type="checkbox" name="shift_flag" id="shift_flag" value="1" /> これまで登録されていたメールアドレスを抹消する</label></li>'
                  if !$shift_required;
                push(
                    @{ $this->{'dialog'} },
'新しいメールアドレスと登録パスワードを入力してください。'
                      . (
                        ( $field::NAMECHECK && $field::for{'name'} )
                        ? $field::obj{ $field::for{'name'} }->name
                          . ' を引き継ぐには、これまで登録されていたメールアドレスを抹消する必要があります。'
                        : ''
                      )
                );
                $email_id = 'new_email';
                $pswd_id  = 'new_pswd';
                if (   !$env::session_enabled
                    && !$cookie::read{ $this->login_cookie_name } )
                {
                    $hidden_item .=
                      '<input type="hidden" name="pswd" value="'
                      . $this->escape($main::pswd) . '" />';
                    $hidden_item .=
                      '<input type="hidden" name="email" value="'
                      . $this->escape($main::email) . '" />'
                      if defined $main::email;
                }
            }
            else {
                push(
                    @{ $this->{'dialog'} },
'元のメールアドレスと登録パスワードを入力してください。'
                );
                $main::email = $new_addr;
                $main::pswd  = '';
            }
        }
    }
    push( @query, @$query_params ) if ref $query_params;

    push(
        @{ $this->{'dialog'} },
'メールアドレスまたはパスワードを確認してください。'
      )
      if !$need_new_pswd
      && defined $main::pswd
      && ( ref $main::pswd || $main::pswd ne '' );

    $this->print_page_header($key);

    $main::referer_url = $ENV{'HTTP_REFERER'} || ''
      if !$main::referer_url || $main::referer_url !~ m!^https?://!;
    print '<form name="gate" id="gate" method="post" action="'
      . $cgi::path . '">';
    print '<input type="hidden" name="_" value="＿" />';
    print $this->set_query( 'post', \@query );
    print '<input type="hidden" name="referer_url" value="'
      . $this->escape($main::referer_url) . '" />';
    print
'<table class="field-table"><tbody id="login-form"><tr><td class="field-name">';
    print '<label for="'
      . $email_id
      . '">メールアドレス</label></td><td class="field"><input type="email" name="'
      . $email_id
      . '" id="'
      . $email_id
      . '" size="32" value="'
      . (
        ( $need_new_pswd && $main::new_email ) ? $this->escape($main::new_email)
        : ($new_addr) ? $this->escape($new_addr)
        :               $this->escape($main::email)
      ) . '" /></td></tr>';
    print '<tr><td class="field-name"><label for="'
      . $pswd_id
      . '">パスワード</label></td><td class="field"><input type="password" name="'
      . $pswd_id
      . '" id="'
      . $pswd_id
      . '" size="16" />';
    print ' <a href="'
      . $env::register_cgi
      . (
        ( $new_addr && $need_new_pswd )
        ? '?addr='
          . $this->escape( ($main::new_email) ? $main::new_email : $new_addr )
          . '&amp;transition=1'
        : ''
      )
      . '"'
      . (
        ($env::register_in_same_window)
        ? ''
        : ' target="LoginWindow" onclick="cmonos.window.open(\'LoginWindow\',450,300)"'
      ) . ' title="メールアドレス登録">';
    print '<img src="'
      . $env::image_dir
      . '/tool/login.gif" width="20" height="20" border="0" alt="メールアドレス登録" />';
    print '</a>';
    print '</td></tr>';
    $additional_form .=
'<li><label><input type="checkbox" name="set_cookie_id" id="set_cookie_id" value="1"'
      . (
        ( $cookie::read{ $this->login_cookie_name } )
        ? ' checked="checked"'
        : ''
      )
      . ' /> ブラウザに記憶させる(ログイン)</label></li>'
      if !$env::session_enabled;

    if ($additional_form) {
        print
'<tr><td class="field-name"><span class="datatitle">後処理</span></td><td class="field"><ol class="checkbox">';
        print $additional_form;
        print '</ol></td></tr>';
    }
    print '</tbody><tbody>';
    print '<tr><td class="close-box">';
    print $hidden_item if defined $hidden_item && $hidden_item;
    print $this->close_button(
        (
            $main::referer_url || $cgi::path
              . (
                ( defined $main::c && $main::c ne '' )
                ? '?c=' . $main::c
                : ''
              )
        ),
        $setting::page{'closeButton'},
        $setting::page{'backButton'}
    );
    print '</td><td class="submit-box">';
    print
'<label><input type="checkbox" name="use_memorized_id" id="use_memorized_id" value="1" onclick="cmonos.element.item(\'login-form\').disappear(this.checked)" /> ログイン済み</label> '
      if $env::openid_enabled;
    print '<input type="submit" value="ログイン" />';
    print '</td></tr></tbody></table></form>';
    print $cancel_button if defined $cancel_button && $cancel_button;

    $this->print_page_footer($key);
    exit;
}

#	obj->print_error_dialog
sub print_error_dialog ($$) {
    my ( $this, $key ) = @_;
    $setting::page{$key}{'title'} = 'エラー！';
    $this->print_page_header($key);
    print '<strong>エラー！</strong>';
    $this->print_page_footer($key);
    exit;
}

######################################################################
#	(8) obj->print_page_header(key,handle,output,remain,cginame,title,info,class)
sub print_page_header ($$;$$$$$$$$) {
    my (
        $this,    $key,   $handle,      $output, $remain,
        $cginame, $title, $description, $info,   $class
    ) = @_;
    $key    = caller       if !defined $key    || $key eq '';
    $output = $cgi::output if !defined $output || !$output;
    my ( $is_not_stdout, @dialog, $background );
    if ( !$handle || scalar(*$handle) eq scalar(*STDOUT) ) {
        $handle = \*STDOUT;
        $this->set_base( $env::file_server . $ENV{'SCRIPT_NAME'} )
          if defined $env::file_server && $env::file_server;
        $this->mkcookie;
        @dialog = $this->dialog;
        $this->set_onload('cmonos.dialog.set()') if @dialog;
    }
    else {
        $is_not_stdout = 1;
    }

    #タイトル
    $this->set_description($description)
      if defined $description && $description ne '';
    $this->set_title( $this->mk_page_title( $key, $cginame, $title, $info ) )
      if !defined $this->{'page_title'}
      || $this->{'page_title'} eq ''
      || ( defined $title && $title ne '' );

    #BODYオプション
    $background = '';
    if ( $setting::page{$key}{'background'} ) {
        if ( $setting::page{$key}{'background'} =~ /^[0-9a-zA-Z_]+$/i ) {
            $background =
              $this->image_file_whole_path( $env::image_dir
                  . '/background/'
                  . $setting::page{$key}{'background'} );
        }
        elsif ( $setting::page{$key}{'background'} =~ /\.(gif|jpg|jpeg|png)$/i )
        {
            $background = $setting::page{$key}{'background'};
        }
    }
    $background = $this->base_change( $background, $output )
      if $background && $output;

    $this->set_body(
        'class' => (
            ($class)
            ? $class . ( ( $setting::page{$key}{'bodyClass'} ) ? ' ' : '' )
            : ''
          )
          . (
            ( $setting::page{$key}{'bodyClass'} )
            ? $setting::page{$key}{'bodyClass'}
            : ''
          ),
        'text'            => $setting::page{$key}{'text'},
        'lineheight'      => $setting::page{$key}{'lineheight'},
        'link'            => $setting::page{$key}{'link'},
        'linkdecoration'  => $setting::page{$key}{'linkdecoration'},
        'alink'           => $setting::page{$key}{'alink'},
        'alinkdecoration' => $setting::page{$key}{'alinkdecoration'},
        'vlink'           => $setting::page{$key}{'vlink'},
        'vlinkdecoration' => $setting::page{$key}{'vlinkdecoration'},
        'hover'           => $setting::page{$key}{'hover'},
        'hoverdecoration' => $setting::page{$key}{'hoverdecoration'},
        'bgcolor'         => $setting::page{$key}{'bgcolor'},
        'background'      => $background,
        'bgproperties'    => $setting::page{$key}{'bgproperties'},
        'topmargin'       => $setting::page{$key}{'topmargin'},
        'leftmargin'      => $setting::page{$key}{'leftmargin'},
        'bottommargin'    => $setting::page{$key}{'bottommargin'},
        'rightmargin'     => $setting::page{$key}{'rightmargin'},
        'marginwidth'     => $setting::page{$key}{'marginwidth'},
        'marginheight'    => $setting::page{$key}{'marginheight'},
        'bodystyle'       => (
            (
                defined $setting::page{$key}{'align'}
                  && ( $setting::page{$key}{'align'} eq 'center'
                    || $setting::page{$key}{'align'} eq 'right' )
            )
            ? '  text-align: ' . $setting::page{$key}{'align'} . ';'
            : ''
        ),
        'id' => (
            ( $setting::page{$key}{'bodyId'} )
            ? $setting::page{$key}{'bodyId'}
            : ($default::id) ? 'document' . $default::id
            :                  ''
        )
    );
    undef $background;

    #HTMLヘッダ情報
    $this->set_javascript( $setting::page{$key}{'javascript'} );
    $this->{'javascriptlink'} = $this->javascript_link(
        ($output)
        ? $this->base_change( $env::javascript . '/cmonos.js', $output )
        : $env::javascript . '/cmonos.js'
      )
      . "\x0A"
      . ( ( $this->{'javascriptlink'} ) ? $this->{'javascriptlink'} : '' )
      if $env::javascript;

    #個別CSS
    #title-image
    if ( defined $setting::page{$key}{'titleImage'}
        && $setting::page{$key}{'titleImage'} )
    {
        $this->{'CSS'} .= 'h1#title-image, h1#title-image img {' . "\x0A";
        $this->{'CSS'} .= '  display: block;' . "\x0A";
        $this->{'CSS'} .= '  margin-top: 0;' . "\x0A";
        $this->{'CSS'} .= '  margin-bottom: 0;' . "\x0A";
        if ( defined $setting::page{$key}{'titleImageAlign'}
            && $setting::page{$key}{'titleImageAlign'} ne '' )
        {
            $this->{'CSS'} .=
              '  text-align: ' . {'titleImageAlign'} . ';' . "\x0A";
            $this->{'CSS'} .= '  margin-left: auto;' . "\x0A"
              if $setting::page{$key}{'titleImageAlign'} eq 'center'
              || $setting::page{$key}{'titleImageAlign'} eq 'right';
            $this->{'CSS'} .= '  margin-right: auto;' . "\x0A"
              if $setting::page{$key}{'titleImageAlign'} eq 'center'
              || $setting::page{$key}{'titleImageAlign'} eq 'left';
        }
        $this->{'CSS'} .= '}' . "\x0A";
    }

    #container
    if (
        (
            defined $setting::page{$key}{'textAlign'}
            && $setting::page{$key}{'textAlign'} ne ''
        )
        || ( defined $setting::page{$key}{'margintop'}
            && $setting::page{$key}{'margintop'} ne '' )
        || ( defined $setting::page{$key}{'marginbottom'}
            && $setting::page{$key}{'marginbottom'} ne '' )
        || ( defined $setting::page{$key}{'align'}
            && $setting::page{$key}{'align'} ne '' )
        || ( defined $setting::page{$key}{'width'}
            && $setting::page{$key}{'width'} ne '' )
      )
    {
        $this->{'CSS'} .= 'div' if !$setting::page{$key}{'deco'};
        $this->{'CSS'} .= '#container {' . "\x0A";
        $this->{'CSS'} .= '  text-align: '
          . (
            (
                defined $setting::page{$key}{'textAlign'}
                  && $setting::page{$key}{'textAlign'} ne ''
            ) ? $setting::page{$key}{'textAlign'} : 'left'
          ) . ';' . "\x0A";
        $this->{'CSS'} .=
          '  margin-top: ' . $setting::page{$key}{'margintop'} . ';' . "\x0A"
          if defined $setting::page{$key}{'margintop'}
          && $setting::page{$key}{'margintop'} ne '';
        $this->{'CSS'} .=
          '  margin-bottom: '
          . $setting::page{$key}{'marginbottom'} . ';' . "\x0A"
          if defined $setting::page{$key}{'marginbottom'}
          && $setting::page{$key}{'marginbottom'} ne '';
        if ( defined $setting::page{$key}{'align'}
            && $setting::page{$key}{'align'} ne '' )
        {
            $this->{'CSS'} .= '  margin-left: auto;' . "\x0A"
              if $setting::page{$key}{'align'} eq 'center'
              || $setting::page{$key}{'align'} eq 'right';
            $this->{'CSS'} .= '  margin-right: auto;' . "\x0A"
              if $setting::page{$key}{'align'} eq 'center'
              || $setting::page{$key}{'align'} eq 'left';
        }
        $this->{'CSS'} .=
            '  width: '
          . $setting::page{$key}{'width'}
          . ( ( $setting::page{$key}{'width'} !~ /[^0-9]+$/ ) ? 'px' : '' )
          . "\x0A"
          if defined $setting::page{$key}{'width'}
          && $setting::page{$key}{'width'} ne '';
        $this->{'CSS'} .= '}' . "\x0A";
    }

    #page-title
    if (
           defined $setting::page{$key}{'titleText'}
        && $setting::page{$key}{'titleText'} ne ''
        && !$setting::page{$key}{'titleTextDeco'}
        && (   $setting::page{$key}{'titleTextAlign'}
            || $setting::page{$key}{'titleTextWidth'} )
      )
    {
        $this->{'CSS'} .= 'h1#page-title {' . "\x0A";
        if ( defined $setting::page{$key}{'titleTextAlign'}
            && $setting::page{$key}{'titleTextAlign'} )
        {
            $this->{'CSS'} .= '  margin-left: auto;' . "\x0A"
              if $setting::page{$key}{'titleTextAlign'} eq 'center'
              || $setting::page{$key}{'titleTextAlign'} eq 'right';
            $this->{'CSS'} .= '  margin-right: auto;' . "\x0A"
              if $setting::page{$key}{'titleTextAlign'} eq 'center'
              || $setting::page{$key}{'titleTextAlign'} eq 'left';
            $this->{'CSS'} .= '  text-align: '
              . $setting::page{$key}{'titleTextAlign'} . ';' . "\x0A";
        }
        $this->{'CSS'} .=
            '  width: '
          . $setting::page{$key}{'titleTextWidth'}
          . (
            ( $setting::page{$key}{'titleTextWidth'} !~ /[^0-9]+$/ )
            ? 'px'
            : ''
          )
          . "\x0A"
          if defined $setting::page{$key}{'titleTextWidth'}
          && $setting::page{$key}{'titleTextWidth'} ne '';
        $this->{'CSS'} .= '}' . "\x0A";
    }

    #subcontainer
    if ( defined $setting::page{$key}{'contentsAlign'}
        && $setting::page{$key}{'contentsAlign'} )
    {
        $this->{'CSS'} .=
            'div#subcontainer, div#subcontainer div, div#subcontainer table {'
          . "\x0A";
        $this->{'CSS'} .= '  margin-left: auto;' . "\x0A"
          if $setting::page{$key}{'contentsAlign'} eq 'center'
          || $setting::page{$key}{'contentsAlign'} eq 'right';
        $this->{'CSS'} .= '  margin-right: auto;' . "\x0A"
          if $setting::page{$key}{'contentsAlign'} eq 'center'
          || $setting::page{$key}{'contentsAlign'} eq 'left';
        $this->{'CSS'} .= '}' . "\x0A";
    }

    #page-introduction
    if (   defined $setting::page{$key}{'headText'}
        && $setting::page{$key}{'headText'} ne ''
        && !$setting::page{$key}{'headTextDeco'}
        && $setting::page{$key}{'headTextWidth'} )
    {
        $this->{'CSS'} .= 'div#page-introduction {' . "\x0A";
        $this->{'CSS'} .=
            '  width: '
          . $setting::page{$key}{'headTextWidth'}
          . (
            ( $setting::page{$key}{'headTextWidth'} !~ /[^0-9]+$/ )
            ? 'px'
            : ''
          )
          . "\x0A"
          if defined $setting::page{$key}{'headTextWidth'}
          && $setting::page{$key}{'headTextWidth'} ne '';
        $this->{'CSS'} .= '}' . "\x0A";
    }

    $this->set_css( $setting::page{$key}{'CSS'} );

    #共通CSS
    if ($env::css) {
        my $env_css =
          ($output)
          ? $this->base_change( $env::css, $output )
          : $env::css;
        $this->{'stylesheetlink'} =
            '<link rel="stylesheet" type="text/css" href="'
          . $env_css
          . '/common.css' . '" />' . "\x0A"
          . '<link rel="stylesheet" type="text/css" href="'
          . $env_css
          . '/image.css' . '" />' . "\x0A"
          . (
            ($setting::image_css)
            ? '<link rel="stylesheet" type="text/css" href="'
              . $env_css . '/'
              . $setting::image_css . '" />' . "\x0A"
            : ''
          )
          . (
            ($setting::image_css2)
            ? '<link rel="stylesheet" type="text/css" href="'
              . $env_css . '/'
              . $setting::image_css2 . '" />' . "\x0A"
            : ''
          )
          . (
            ( $this->{'stylesheetlink'} ) ? $this->{'stylesheetlink'}
            : ''
          );
    }

    #インラインサイトナビゲーションCSS
    if (   $env::inline_navi
        && $setting::page{$key}{'naviEnabled'}
        && -f $env::css . '/site-navi.css' )
    {
        my $stylesheet = $env::css . '/site-navi.css';
        $stylesheet = $this->base_change( $stylesheet, $output )
          if defined $output && $output;
        $this->{'stylesheetlink'} =
            '<link rel="stylesheet" type="text/css" href="'
          . $stylesheet . '" />' . "\x0A"
          . (
            ( $this->{'stylesheetlink'} )
            ? $this->{'stylesheetlink'}
            : ''
          );
    }

    #デフォルトCSS
    if (   $setting::page{$key}{'defaultcss'}
        && $env::css
        && -f $env::css . '/default.css' )
    {
        my $defaultcss = $env::css . '/default.css';
        $defaultcss = $this->base_change( $defaultcss, $output )
          if $defaultcss && $output;
        $this->set_style_sheet_link($defaultcss);
    }

    #オリジナルCSS
    if ( $setting::page{$key}{'cssFileName'}
        && -f $env::css . '/' . $setting::page{$key}{'cssFileName'} )
    {
        my $stylesheet = $env::css . '/' . $setting::page{$key}{'cssFileName'};
        $stylesheet = $this->base_change( $stylesheet, $output )
          if defined $output && $output;
        $this->set_style_sheet_link($stylesheet);
    }
    if ( $setting::page{$key}{'cssFileName2'}
        && -f $env::css . '/' . $setting::page{$key}{'cssFileName2'} )
    {
        my $stylesheet = $env::css . '/' . $setting::page{$key}{'cssFileName2'};
        $stylesheet = $this->base_change( $stylesheet, $output )
          if defined $output && $output;
        $this->set_style_sheet_link($stylesheet);
    }

    #Google-Analytics
    if ( $env::google_analytics_uacct && $key ne 'tools' ) {
        if (
            (
                $env::google_analytics_embedded
                && (  !$setting::page{$key}{'googleAnalytics'}
                    || $setting::page{$key}{'googleAnalytics'} ne 'disabled' )
            )
            || (  !$env::google_analytics_embedded
                && $setting::page{$key}{'googleAnalytics'}
                && $setting::page{$key}{'googleAnalytics'} eq 'enabled' )
          )
        {
            $this->set_google_analytics(1);
        }
    }

    #RSS検索
    $this->set_rss_link($output);
    $this->_set_rss_link( '../../',
        ( ($env::title) ? $env::title : 'This site' ), $output );

    $this->{'common_html_header_disabled'} = 1
      if $setting::page{$key}{'commonHtmlHeaderDisabled'};
    $this->set_html_header( $setting::page{$key}{'htmlHeader'} );
    $this->replace_meta_data( \$this->{'html_header'},
        $this->{'meta_data'}, \$this->{'html_header'} )
      if ref $this->{'meta_data'};
    $this->replace_link_rel( \$this->{'html_header'}, $this->{'page_link'}, 1 )
      if ref $this->{'page_link'};
    $this->print_html_header($handle);

    #警告ダイアログ
    if (@dialog) {
        print $handle '<div id="cgi-dialog-box">';
        print $handle '<div id="cgi-dialog">';
        print $handle '<ol id="cgi-dialog-list">';
        foreach my $msg (@dialog) {
            if ($msg) {
                $msg =~ s/\\n/<br \/>/g;
                print $handle '<li>';
                print $handle $msg;
                print $handle '</li>';
            }
        }
        print $handle '</ol>';
        print $handle
          '<p id="hide-cgi-dialog"><a href="JavaScript:cmonos.dialog.hide()">'
          . ( ($default::english) ? 'Close.' : '閉じる' )
          . '</a></p>';
        print $handle
          '<p id="show-cgi-dialog"><a href="JavaScript:cmonos.dialog.show()">'
          . ( ($default::english) ? 'Open.' : '開く' )
          . '</a></p>';
        print $handle '</div>';
        print $handle '</div>';
    }

    #インラインサイトナビゲーション
    if (   $env::inline_navi
        && $setting::page{$key}{'naviEnabled'}
        && -f $env::externalfiles . '/site-navi.ssi' )
    {
        print $handle $this->include_external_file(
            $env::externalfiles . '/site-navi.ssi',
            $setting::page{$key}{'externalNavi'}, $output );
    }

    #ページヘッダ
    print $handle $this->include_external_file(
        $env::externalfiles . '/' . $setting::page{$key}{'headerFileName'},
        $setting::page{$key}{'externalHeader'}, $output )
      if !$setting::page{$key}{'insideExternalHeader'}
      && defined $setting::page{$key}{'headerFileName'}
      && $setting::page{$key}{'headerFileName'} ne '';
    print $handle $this->page_item_replace( $setting::page{$key}{'header'},
        $is_not_stdout, $output, $setting::page{$key}{'title'} )
      if !$setting::page{$key}{'insideHeader'}
      && defined $setting::page{$key}{'header'}
      && $setting::page{$key}{'header'} ne '';

    #メニュー
    print $handle '<div id="infobar">'
      if defined $setting::page{$key}{'topicpath'}
      && $setting::page{$key}{'topicpath'} ne ''
      && defined $setting::page{$key}{'menu'}
      && $setting::page{$key}{'menu'};
    print $handle '<div id="topicpath">'
      . $setting::page{$key}{'topicpath'}
      . '</div>'
      if defined $setting::page{$key}{'topicpath'}
      && $setting::page{$key}{'topicpath'} ne '';
    if ( defined $setting::page{$key}{'menu'}
        && $setting::page{$key}{'menu'} )
    {
        print $handle '<div id="jumpmenu">';
        print $handle $setting::page{$key}{'menu'};
        if ( defined $setting::page{$key}{'subMenu'}
            && $setting::page{$key}{'subMenu'} )
        {
            print $handle ' &gt; ';
            print $handle $setting::page{$key}{'subMenu'};
        }
        print $handle '</div>';
    }
    print $handle '</div>'
      if defined $setting::page{$key}{'topicpath'}
      && $setting::page{$key}{'topicpath'} ne ''
      && defined $setting::page{$key}{'menu'}
      && $setting::page{$key}{'menu'};

    #ページタイトル画像
    $this->print_page_title_image( $handle, $key, $output )
      if !$setting::page{$key}{'insideTitleImage'}
      && defined $setting::page{$key}{'titleImage'}
      && $setting::page{$key}{'titleImage'};

    #ページ全体装飾
    if ( defined $setting::page{$key}{'deco'}
        && $setting::page{$key}{'deco'} )
    {
        print $handle $this->frame_start(
            $setting::page{$key}{'deco'},
            $setting::page{$key}{'width'},
            '', 'container', '', $output
        );
    }
    else {
        print $handle '<div id="container">';
    }
    print $handle $this->include_external_file(
        $env::externalfiles . '/' . $setting::page{$key}{'headerFileName'},
        $setting::page{$key}{'externalHeader'}, $output )
      if $setting::page{$key}{'insideExternalHeader'}
      && defined $setting::page{$key}{'headerFileName'}
      && $setting::page{$key}{'headerFileName'} ne '';
    print $handle $this->page_item_replace( $setting::page{$key}{'header'},
        $is_not_stdout, $output, $setting::page{$key}{'title'} )
      if $setting::page{$key}{'insideHeader'}
      && $setting::page{$key}{'header'} ne '';
    $this->print_page_title_image( $handle, $key, $output )
      if $setting::page{$key}{'insideTitleImage'}
      && defined $setting::page{$key}{'titleImage'}
      && $setting::page{$key}{'titleImage'};

    #タイトルテキスト
    if ( defined $setting::page{$key}{'titleText'}
        && $setting::page{$key}{'titleText'} )
    {
        if ( defined $setting::page{$key}{'titleTextDeco'}
            && $setting::page{$key}{'titleTextDeco'} )
        {
            $this->use_deco($output);
            print $handle $cgi::Deco->frame(
                'frame',
                $setting::page{$key}{'titleTextDeco'},
                \$setting::page{$key}{'titleText'},
                '',
                'no',
                '',
                0,
                'framewidth',
                $setting::page{$key}{'titleTextWidth'},
                'frameid',
                'page-titlebar'
            );
        }
        else {
            print $handle '<h1 id="page-title">'
              . $setting::page{$key}{'titleText'} . '</h1>';
        }
    }

    #序文
    if ( defined $setting::page{$key}{'headText'}
        && $setting::page{$key}{'headText'} ne '' )
    {
        my $head_text =
          $this->page_item_replace( $setting::page{$key}{'headText'},
            $is_not_stdout, $output, $setting::page{$key}{'title'} );
        if ( $head_text =~ /[^\x09\x0A\x0C\x0D\x20]/ ) {
            if ( defined $setting::page{$key}{'headTextDeco'}
                && $setting::page{$key}{'headTextDeco'} )
            {
                $this->use_deco($output);
                print $handle $cgi::Deco->frame(
                    'frame',
                    $setting::page{$key}{'headTextDeco'},
                    \$head_text,
                    '',
                    'no',
                    '',
                    0,
                    'framewidth',
                    $setting::page{$key}{'headTextWidth'},
                    'frameid',
                    'page-introduction'
                );
            }
            else {
                print $handle '<div id="page-introduction">'
                  . $head_text
                  . '</div>';
            }
        }
    }
    return if defined $remain && $remain;

    #ヘッダ情報リセット
    $this->reset_html_header;
}

sub set_rss_link ($;$) {
    my ( $this, $output ) = @_;
    if ($default::home) {
        if ( $default::home !~ m!/[0-9a-zA-Z_]+\.s?html?$! ) {
            $this->_set_rss_link(
                $default::home,
                (
                    (
                        defined $setting::general{'cginame'}
                          && $setting::general{'cginame'} ne ''
                    ) ? $setting::general{'cginame'} : ''
                ),
                $output
              )
              if $default::home ne '../../'
              && ( !$output || $output =~ /^$default::home/ );
            $this->_set_rss_link(
                $master::home,
                (
                    (
                        defined $master::general{'cginame'}
                          && $master::general{'cginame'} ne ''
                    ) ? $master::general{'cginame'} : ''
                ),
                $output
              )
              if $default::subdivided
              && $master::home
              && $master::home ne $default::home;
        }
    }
    elsif ($default::file) {
        $this->_set_rss_link(
            $default::file,
            (
                (
                    defined $setting::general{'cginame'}
                      && $setting::general{'cginame'} ne ''
                ) ? $setting::general{'cginame'} : ''
            ),
            $output
        );
        $this->_set_rss_link(
            $default::master_file_dir,
            (
                (
                    defined $master::general{'cginame'}
                      && $master::general{'cginame'} ne ''
                ) ? $master::general{'cginame'} : ''
            ),
            $output
          )
          if $default::subdivided
          && $default::master_file_dir
          && $default::master_file_dir ne $default::file;
    }
}

sub _set_rss_link ($$$;$) {
    my ( $this, $dir, $title, $output ) = @_;
    $dir .= '/' if $dir !~ m!/$!;
    if ( !defined $title ) {
        $title = '';
    }
    elsif ( $title ne '' ) {
        $title .= ' - ';
    }
    $this->set_html_header(
            '<link rel="alternate" type="application/rss+xml" title="'
          . $title
          . 'RSS2.0" href="'
          . (
            ($output)
            ? $this->base_change( $dir . 'index.xml', $output )
            : $dir . 'index.xml'
          )
          . '" />' . "\x0A"
    ) if -f $dir . 'index.xml';
    $this->set_html_header(
            '<link rel="alternate" type="application/rss+xml" title="'
          . $title
          . 'RSS2.0(Podcast)" href="itpc://'
          . $env::server_name
          . (
            ( $ENV{'SERVER_PORT'} != 80 )
            ? ':' . $ENV{'SERVER_PORT'}
            : ''
          )
          . $this->path_change(
            $dir . 'podcast.xml', $ENV{'SCRIPT_NAME'}, 'abs'
          )
          . '" />' . "\x0A"
    ) if -f $dir . 'podcast.xml';
    $this->set_html_header(
            '<link rel="alternate" type="application/rss+xml" title="'
          . $title
          . 'RSS2.0(Comments)" href="'
          . (
            ($output)
            ? $this->base_change( $dir . 'comment.xml', $output )
            : $dir . 'comment.xml'
          )
          . '" />' . "\x0A"
    ) if -f $dir . 'comment.xml';
}

sub image_file_whole_path ($$) {
    my ( $this, $path ) = @_;
    if ( -f $path . '.gif' ) {
        return $path . '.gif';
    }
    elsif ( -f $path . '.jpg' ) {
        return $path . '.jpg';
    }
    elsif ( -f $path . '.png' ) {
        return $path . '.png';
    }
    else {
        return '';
    }
}

#タイトル画像
sub print_page_title_image ($$$$;$) {
    my ( $this, $handle, $key, $output ) = @_;
    return if !$setting::page{$key}{'titleImage'};
    my $title =
      ( ref $setting::deco{'title'}{ $setting::page{$key}{'titleImage'} }
          && defined $setting::deco{'title'}
          { $setting::page{$key}{'titleImage'} }{'alt'}
          && $setting::deco{'title'}{ $setting::page{$key}{'titleImage'} }
          {'alt'} ne '' )
      ? $setting::deco{'title'}{ $setting::page{$key}{'titleImage'} }{'alt'}
      : ( defined $setting::page{$key}{'title'}
          && $setting::page{$key}{'title'} ne '' )
      ? $setting::page{$key}{'title'}
      : ( defined $setting::page{$key}{'titleText'}
          && $setting::page{$key}{'titleText'} ne '' )
      ? $setting::page{$key}{'titleText'}
      : ( defined $setting::general{'cginame'}
          && $setting::general{'cginame'} ne '' ) ? $setting::general{'cginame'}
      : (    $default::subdivided
          && defined $master::general{'cginame'}
          && $master::general{'cginame'} ne '' ) ? $master::general{'cginame'}
      : ( defined $env::title && $env::title ne '' ) ? $env::title
      :                                                'Title Image';
    if ( $setting::page{$key}{'titleImage'} =~ /^[0-9]+$/ ) {
        print $handle '<h1 class="title-'
          . $setting::page{$key}{'titleImage'} . '">'
          . $title . '</h1>';
    }
    elsif ( -f $setting::page{$key}{'titleImage'} ) {
        unless ( ref $cgi::Image ) {
            require CMONOS::CGI_Lib::Image;
            $cgi::Image = Image->new;
        }
        print $handle '<h1 id="page-title-image">';
        print $handle $cgi::Image->image_tag(
            $setting::page{$key}{'titleImage'},
            $setting::page{$key}{'titleImageWidth'},
            $setting::page{$key}{'titleImageHeight'},
            0,
            (
                ( $setting::page{$key}{'titleImageAlt'} )
                ? $setting::page{$key}{'titleImageAlt'}
                : $title
            ),
            '', '', '', $output
        );
        print $handle '</h1>';
    }
}

#タイトル調整
sub mk_page_title ($;$$$$) {
    my ( $this, $key, $cginame, $article_title, $info ) = @_;
    my @title_path;
    if ( defined $cginame && $cginame ne '' ) {
        push( @title_path, $cginame ) if $cginame ne '<>';
    }
    else {
        push( @title_path, $master::general{'cginame'} )
          if $default::subdivided
          && defined $main::c
          && $main::c ne ''
          && !$master::general{'notforwindowtitle'}
          && defined $master::general{'cginame'}
          && $master::general{'cginame'} ne '';
        push( @title_path, $setting::general{'cginame'} )
          if !$setting::general{'notforwindowtitle'}
          && defined $setting::general{'cginame'}
          && $setting::general{'cginame'} ne '';
    }
    $this->push_optional_title( \@title_path );
    push( @title_path, $setting::page{$key}{'title'} )
      if defined $key
      && defined $setting::page{$key}
      && defined $setting::page{$key}{'title'}
      && $setting::page{$key}{'title'} ne '';
    if ( defined $article_title && $article_title ne '' ) {
        push( @title_path, $article_title . ( ($info) ? $info : '' ) );
    }
    elsif ( ref $this->{'additional_topic_path'}
        && @{ $this->{'additional_topic_path'} } )
    {
        $article_title = $this->{'additional_topic_path'}
          ->[ $#{ $this->{'additional_topic_path'} } ];
    }
    elsif (@title_path) {
        $article_title = $title_path[$#title_path];
    }
    push( @title_path, @{ $this->{'additional_topic_path'} } )
      if ref $this->{'additional_topic_path'}
      && @{ $this->{'additional_topic_path'} };
    if ( defined $article_title && $article_title ne '' ) {
        $this->set_og_metadata( 'title', $article_title );
        $article_title = '<meta name="title" content="'
          . $this->escape_except_html_entity($article_title) . '" />';
        $this->{'html_header'} =
          $article_title . "\x0A"
          . ( ( $this->{'html_header'} ) ? $this->{'html_header'} : '' )
          unless $this->{'html_header'}
          && $this->{'html_header'} =~
          s/<meta name="title" content="[^">]*?" \/>/$article_title/sig;
    }
    return $this->window_title( $env::window_title_format, $env::title,
        @title_path );
}

sub push_optional_title ($$) {
    my ( $this, $title_path ) = @_;
    return;
}

sub push_window_title ($;@) {
    my ( $this, @titles ) = @_;
    @titles = grep( defined $_ && $_ ne '', @titles );
    push( @{ $this->{'additional_topic_path'} }, @titles );
}

#ウィンドウタイトル
sub window_title ($;$@) {
    my ( $this, $title_format, $site_title, @title_path ) = @_;
    $title_format = ':,:' if !$title_format;
    my ( $site_title_separator, $category_separator, $reverse ) =
      split( /,/, $title_format );
    @title_path =
      grep( defined $_ && $_ ne '' && $_ =~ /[^\x09\x0A\x0C\x0D\x20]/,
        @title_path );
    return ( ( defined $site_title ) ? $site_title : '' ) if !@title_path;
    my $title = '';
    if ($category_separator) {
        @title_path = reverse @title_path if $reverse;
        $title = join( $category_separator, @title_path );
    }
    else {
        $title = $title_path[$#title_path];
    }
    if ( defined $site_title && $site_title ne '' ) {
        if ($reverse) {
            $title .= $site_title_separator . $site_title;
        }
        else {
            $title = $site_title . $site_title_separator . $title;
        }
    }
    return $this->detag_text($title);
}

#ヘッダ・フッタ埋め込みツール変換
sub page_item_replace ($$;$$$$) {
    my ( $this, $str, $file_flag, $output, $title, $info ) = @_;
    $str =~
s/<\!--(sitetitle)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/sitetitle(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$env::title,$file_flag)/iges;
    $str =~
s/<\!--(sitedescription)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/sitedescription(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$env::description,$file_flag)/iges;

    $str =~
s/<\!--(cginame)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/cginame(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$setting::general{'cginame'},$file_flag)/iges;
    $str =~
s/<\!--(cgidescription)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/cgidescription(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$setting::general{'description'},$file_flag)/iges;

    $str =~
s/<\!--(ogdescription)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/ogdescription(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$setting::general{'description'},$file_flag)/iges;
    $str =~
s/<\!--(ogimage)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/ogimage(?![0-9a-zA-Z_])[^>]*-->/$this->ogimage_tag_replace($1,$2,$3,$file_flag,$output)/iges;
    $str =~
s/<\!--(pageicon)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/pageicon(?![0-9a-zA-Z_])[^>]*-->/$this->ogimage_tag_replace($1,$2,$3,$file_flag,$output,1)/iges;

    my $pagetitle = $this->detag_text(
        (
            defined $this->{'page_title_html'}
              && $this->{'page_title_html'} ne ''
        )
        ? $this->{'page_title_html'}
        : ( ref $this->{'additional_topic_path'}
              && @{ $this->{'additional_topic_path'} } )
        ? join( ' &gt; ', @{ $this->{'additional_topic_path'} } )
        : ( defined $title && $title ne '' )
        ? $title . ( ( defined $info && $info ne '' ) ? $info : '' )
        : ( defined $setting::general{'cginame'} )
        ? $setting::general{'cginame'}
        : ''
    );
    $str =~
s/<\!--(pagetitle)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/pagetitle(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$pagetitle,$file_flag)/iges;

    $this->site_tool_replace( \$str, $file_flag, $output );
    $this->topicpath_replace( \$str, $file_flag, $output, $title )
      if $str =~ /<\!--topicpath(?![0-9a-zA-Z_])[^>]*-->/
      || $str =~ /<\!--breadcrumbs(?![0-9a-zA-Z_])[^>]*-->/;

    $str =~
s/<\!--(masterrss)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/masterrss-->/$this->rss_link_replace($1,$2,$3,'..\/..\/index.xml',$file_flag,$output)/iges;
    $str =~
s/<\!--(mastercommentrss)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/mastercommentrss-->/$this->rss_link_replace($1,$2,$3,'..\/..\/comment.xml',$file_flag,$output)/iges;
    $str =~
s/<\!--(mastertrackbackrss)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/mastertrackbackrss-->/$this->rss_link_replace($1,$2,$3,'..\/..\/trackback.xml',$file_flag,$output)/iges;
    $str =~
s/<\!--(masterpodcast)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/masterpodcast-->/$this->rss_link_replace($1,$2,$3,'..\/..\/podcast.xml',$file_flag,$output)/iges;

    my $sitemap =
      ( -f $env::externalfiles . '/site-map.ssi' )
      ? '<!--#include virtual="'
      . $this->path_change( $env::externalfiles . '/site-map.ssi',
        $ENV{'SCRIPT_NAME'}, 'abs' )
      . '"-->'
      : '';
    $str =~
s/<\!--(sitemap)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/sitemap(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$sitemap,$file_flag)/iges;

    $this->sim_ssi( \$str )
      if !$default::static || !$env::ssi || !$env::ssi_ext;

    $this->_page_item_replace( \$str, $file_flag, $output, $title, $info );
    return $str;
}

#ヘッダ・フッタ埋め込みツール変換
sub _page_item_replace ($$;$$$$) {
    my ( $this, $str, $file_flag, $output, $title, $info ) = @_;
}

sub ogimage_tag_replace ($$$$;$$) {
    my ( $this, $tag, $attributes, $content, $file_flag, $output, $og_disabled )
      = @_;
    my $img =
      (      ( !$og_disabled || !$setting::general{'iconwidth'} )
          && $setting::general{'og'}
          && $setting::general{'ogwidth'} )
      ? $cgi::Image->image_tag(
        ( $default::file || $default::home ) . '/' . $setting::general{'og'},
        $setting::general{'ogwidth'},
        $setting::general{'ogheight'},
        0,
        $setting::general{'cginame'},
        '',
        (
            (
                     $attributes
                  && $attributes =~ /[\x20\x09\x0A\x0D]maxwidth=["']?([0-9]+)/
            ) ? $1 : ''
        ),
        (
            (
                     $attributes
                  && $attributes =~ /[\x20\x09\x0A\x0D]maxheight=["']?([0-9]+)/
            ) ? $1 : ''
        ),
        $output,
        $cgi::output_option
      )
      : ( $setting::general{'icon'} && $setting::general{'iconwidth'} )
      ? $cgi::Image->image_tag(
        ( $default::file || $default::home ) . '/' . $setting::general{'icon'},
        $setting::general{'iconwidth'},
        $setting::general{'iconheight'},
        0,
        $setting::general{'cginame'},
        '',
        (
            (
                     $attributes
                  && $attributes =~ /[\x20\x09\x0A\x0D]maxwidth=["']?([0-9]+)/
            ) ? $1 : ''
        ),
        (
            (
                     $attributes
                  && $attributes =~ /[\x20\x09\x0A\x0D]maxheight=["']?([0-9]+)/
            ) ? $1 : ''
        ),
        $output,
        $cgi::output_option
      )
      : '';
    return $this->item_tag_replace( $tag, $attributes, $content, $img,
        $file_flag );
}

sub rss_link_replace ($$$$$;$$$) {
    my ( $this, $tag, $attributes, $content, $path, $tag_flag, $output,
        $option ) = @_;
    my ( $header, $footer ) =
      $this->get_header_and_footer( $attributes, $content );
    $attributes =~
s/[\x09\x0A\x0C\x0D\x20]+head[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"//sg;
    $attributes =~
s/[\x09\x0A\x0C\x0D\x20]+foot[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]+)"//sg;
    $attributes = ''
      if $attributes eq '' || $attributes !~ /[^\x09\x0A\x0C\x0D\x20]/;
    my $nocheck = ( $attributes =~ / nocheck="([01])"/i ) ? $1 : '';
    if ( !$nocheck && !-f $path ) {
        if ($tag_flag) {
            return
                '<!--'
              . $tag
              . $attributes
              . (
                ( $header ne '' )
                ? ' head="'
                  . $this->lf_to_entity( $this->escape($header) ) . '"'
                : ''
              )
              . (
                ( $footer ne '' )
                ? ' foot="'
                  . $this->lf_to_entity( $this->escape($footer) ) . '"'
                : ''
              )
              . '--><!--/'
              . $tag . '-->';
        }
        else {
            return '';
        }
    }
    my $class = ( $attributes =~ / class="([0-9a-zA-Z_\-\x20]+)"/i ) ? $1 : '';
    my $id = ( $attributes =~ / id="([0-9a-zA-Z_\-]+)"/i ) ? $1 : '';
    my $title = ( $attributes =~ / title="([^"]+)"/i ) ? $1 : 'RSS2.0';
    my $link = '<a href="'
      . $this->escape_except_html_entity(
        ( $tag =~ /podcast/i )
        ? 'itpc://'
          . $env::server_name
          . (
            ( $ENV{'SERVER_PORT'} != 80 )
            ? ':' . $ENV{'SERVER_PORT'}
            : ''
          )
          . $this->path_change( $path, $ENV{'SCRIPT_NAME'}, 'abs' )
        : ( $output || $option )
        ? $this->base_change( $path, $output, $option )
        : $path
      )
      . '"'
      . ( ($class) ? ' class="' . $class . '"' : '' )
      . ( ($id)    ? ' id="' . $id . '"'       : '' ) . '>'
      . $this->escape_except_html_entity($title) . '</a>';
    if ($tag_flag) {
        return
            '<!--'
          . $tag
          . $attributes . '-->'
          . (
            ( $header ne '' ) ? '<!--head-->' . $header . '<!--/head-->'
            : ''
          )
          . '<!--data-->'
          . $link
          . '<!--/data-->'
          . (
            ( $footer ne '' ) ? '<!--foot-->' . $footer . '<!--/foot-->'
            : ''
          )
          . '<!--/'
          . $tag . '-->';
    }
    else {
        return $header . $link . $footer;
    }
}

#パンくず
sub topicpath_replace ($$;$$$) {
    my ( $this, $str, $file_flag, $output, $title ) = @_;
    my @topicpath_pair = ();
    if ( $file_flag && $default::home ) {
        if ( defined $default::subdivided && $default::subdivided ) {
            if ( $output =~ m!^$master::home/index\.[0-9a-zA-Z_]+$! ) {
                $title =
                  ( defined $master::general{'cginame'} )
                  ? $master::general{'cginame'}
                  : '';
            }
            elsif ( $output =~ m!^$default::home/index\.[0-9a-zA-Z_]+$! ) {
                $title =
                  ( defined $setting::general{'cginame'} )
                  ? $setting::general{'cginame'}
                  : '';
                @topicpath_pair = (
                    $master::home
                      . '/' => (
                        ( defined $master::general{'cginame'} )
                        ? $master::general{'cginame'}
                        : ''
                      )
                );
            }
            else {
                @topicpath_pair = (
                    $master::home
                      . '/' => (
                        ( defined $master::general{'cginame'} )
                        ? $master::general{'cginame'}
                        : ''
                      ),
                    $default::home
                      . '/' => (
                        ( defined $setting::general{'cginame'} )
                        ? $setting::general{'cginame'}
                        : ''
                      )
                );
            }
        }
        else {
            if ( $output =~ m!^$default::home(/index\.[0-9a-zA-Z_]+|)$! ) {
                $title =
                  ( defined $setting::general{'cginame'} )
                  ? $setting::general{'cginame'}
                  : '';
            }
            else {
                @topicpath_pair = (
                    $default::home
                      . '/' => (
                        ( defined $setting::general{'cginame'} )
                        ? $setting::general{'cginame'}
                        : ''
                      )
                );
            }
        }
    }
    else {
        if ( defined $default::subdivided && $default::subdivided ) {
            if ( defined $main::c && $main::c ne '' ) {
                @topicpath_pair = (
                    './start.cgi' => (
                        ( defined $master::general{'cginame'} )
                        ? $master::general{'cginame'}
                        : ''
                    ),
                    './start.cgi?c='
                      . $main::c => (
                        ( defined $setting::general{'cginame'} )
                        ? $setting::general{'cginame'}
                        : ''
                      )
                );
            }
            else {
                $title =
                  ( defined $setting::general{'cginame'} )
                  ? $setting::general{'cginame'}
                  : '';
            }
        }
        elsif ($default::lib
            && $default::lib eq 'TableMaker'
            && $default::type
            && $default::type eq 'order' )
        {
            @topicpath_pair = (
                './start.cgi?iframe_disabled=1' => (
                    ( defined $setting::general{'cginame'} )
                    ? $setting::general{'cginame'}
                    : ''
                )
            );
        }
        elsif (
            !$default::lib
            || (   $default::lib ne 'GateKeeper'
                && $default::lib ne 'NaviMaker' )
          )
        {
            @topicpath_pair = (
                './start.cgi' => (
                    ( defined $setting::general{'cginame'} )
                    ? $setting::general{'cginame'}
                    : ''
                )
            );
            $this->push_optional_topicpath( \@topicpath_pair );
        }
    }
    $$str =~
s/<\!--topicpath(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/topicpath(?![0-9a-zA-Z_])[^>]*-->/$this->_topicpath_replace($1,$2,\@topicpath_pair,$this->{'additional_topic_path'},$output,$title,$file_flag)/gies;
    $$str =~
s/<\!--breadcrumbs(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/breadcrumbs(?![0-9a-zA-Z_])[^>]*-->/$this->_topicpath_replace($1,$2,\@topicpath_pair,$this->{'additional_topic_path'},$output,$title,$file_flag)/gies;
}

sub push_optional_topicpath ($$) {
    my ( $this, $topicpath_pair ) = @_;
    return;
}

sub _topicpath_replace ($$$$$$) {
    my ( $this, $attributes, $data, $topicpath_pair,
        $additional_topicpath, $output, $title, $file_flag )
      = @_;
    my $topicpath = '';
    my $is_list =
      ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+type="list"/i ) ? 1 : 0;
    my $id =
      ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+id="([^"]+)"/i ) ? $1 : "";
    my $class =
      ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+class="([^"]+)"/i )
      ? $1
      : "";
    my $shema_type =
      ' itemscope="itemscope" itemtype="http://schema.org/BreadcrumbList"';
    my $list_type =
' itemtype="http://schema.org/ListItem" itemscope="itemscope" itemprop="itemListElement"';
    my $position = 1;

    if ($is_list) {
        $topicpath = '<ul'
          . ( ( $id ne '' )    ? ' id="' . $id . '"'       : '' )
          . ( ( $class ne '' ) ? ' class="' . $class . '"' : '' )
          . $shema_type
          . '><li class="home"'
          . $list_type . '>';
    }
    elsif ( $id ne '' || $class ne '' ) {
        $topicpath = '<p'
          . ( ( $id ne '' )    ? ' id="' . $id . '"'       : '' )
          . ( ( $class ne '' ) ? ' class="' . $class . '"' : '' )
          . $shema_type
          . '><span'
          . $list_type . '>';
    }
    $topicpath .=
        '<a href="'
      . ( ($output) ? $this->base_change( '../../', $output ) : '../../' )
      . '" rel="home" itemprop="item" target="_top"><span itemprop="name">'
      . (
          ( $env::title ne '' ) ? $this->escape_except_html_entity($env::title)
        : ($default::english)   ? 'Home'
        :                         'ホーム'
      ) . '</span></a><meta itemprop="position" content="1" />';
    $topicpath .= ($is_list) ? '</li>' : '</span>';
    if (@$topicpath_pair) {
        for ( my $i = 0 ; $i < @$topicpath_pair ; $i += 2 ) {
            if (   $topicpath_pair->[$i]
                && defined $topicpath_pair->[ $i + 1 ]
                && $topicpath_pair->[ $i + 1 ] ne '' )
            {
                $topicpath .= ( ($is_list) ? '<li' : ' &gt; <span' )
                  . (
                    (
                        ( !defined $title || $title eq '' )
                          && ( !ref $additional_topicpath
                            || !@{$additional_topicpath} )
                          && $i == @$topicpath_pair - 2
                    ) ? ' class="current"' : ''
                  ) . $list_type . '>';
                $topicpath .= '<a href="'
                  . (
                    ($output)
                    ? $this->base_change( $topicpath_pair->[$i], $output )
                    : $topicpath_pair->[$i]
                  )
                  . '" itemprop="item"><span itemprop="name">'
                  . $this->detag_text( $topicpath_pair->[ $i + 1 ] )
                  . '</span></a><meta itemprop="position" content="'
                  . ( ++$position ) . '" />';
                $topicpath .= ($is_list) ? '</li>' : '</span>';
            }
        }
    }
    if ( ref $additional_topicpath && @{$additional_topicpath} ) {
        my @additional_topicpath = @{$additional_topicpath};
        unshift( @additional_topicpath, $title )
          if defined $title && $title ne '';
        $title = pop @additional_topicpath;
        foreach my $t (@additional_topicpath) {
            if ( defined $t && $t ne '' ) {
                $topicpath .= ( ($is_list) ? '<li' : ' &gt; <span' ) . '>';
                $topicpath .= '<span>' . $this->detag_text($t) . '</span>';
                $topicpath .= ($is_list) ? '</li>' : '</span>';
            }
        }
    }
    if ( defined $title && $title ne '' ) {
        $topicpath .=
          ( ($is_list) ? '<li' : ' &gt; <span' ) . ' class="current">';
        $topicpath .= '<span>' . $this->detag_text($title) . '</span>';
        $topicpath .= ($is_list) ? '</li>' : '</span>';
    }
    if ($is_list) {
        $topicpath .= '</ul>';
    }
    elsif ( $id ne '' || $class ne '' ) {
        $topicpath .= '</p>';
    }
    return $this->item_tag_replace( 'topicpath', $attributes, $data,
        $topicpath, $file_flag );
}

#ツール類
sub site_tool_replace ($$;$$) {
    my ( $this, $str, $file_flag, $output ) = @_;
    $this->use_deco($output);
    my $checkout_tool =
      ($env::checkout)
      ? $cgi::Deco->button(
        'check',
        $setting::page{'checkButton'},
        $env::checkout . '?k=cartviewer&amp;m=CartViewer',
        (
            ( $setting::page{'checkButtonName'} )
            ? $setting::page{'checkButtonName'}
            : ($default::english) ? 'Check Out.'
            :                       'カートを見る'
        ),
        '',
        { 'onclick' => 'cmonos.tool.shoppingcart.window.open(this)' }
      )
      : '';
    $$str =~
s/<\!--(checkout)(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/checkout(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$checkout_tool,$file_flag)/iges;
    $$str =~
s/<\!--username(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/username-->/$this->user_name_replace($1,$2,$file_flag)/gse;
    $$str =~
s/<\!--welcome(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/welcome-->/$this->welcome_msg_replace($1,$2,$output,$file_flag)/gse;
    $$str =~
s/<\!--login(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/login-->/$this->login_tool_replace($1,$2,$output,$file_flag)/gse;
}

#ユーザー名
sub user_name_replace ($$$;$) {
    my ( $this, $attributes, $content, $file_flag ) = @_;
    return '' if $file_flag;
    my $user_name = $this->get_user_name;
    return '' if !defined $user_name || $user_name eq '';
    my ( $header, $footer ) =
      $this->get_header_and_footer( $attributes, $content );
    return $header . $this->escape($user_name) . $footer;
}

#ウェルカムメッセージ
sub welcome_msg_replace ($$$;$$) {
    my ( $this, $attributes, $content, $output, $file_flag ) = @_;
    return $this->login_tool_replace( $attributes, $content, $output,
        $file_flag )
      if $file_flag;
    $this->get_user_name if !defined $cgi::user_name;
    return $this->login_tool_replace( $attributes, $content, $output,
        $file_flag )
      if !defined $cgi::user_name || $cgi::user_name eq '';
    my ( $header, $footer, $hour, $hello, $user_name, $tool );
    $user_name = $this->escape($cgi::user_name);
    ( $header, $footer ) =
      $this->get_header_and_footer( $attributes, $content );
    $hour = ( localtime() )[2];

    if ( $hour > 3 && $hour <= 10 ) {
        $hello =
          ($default::english)
          ? 'Good morning'
          : 'おはようございます';
    }
    elsif ( $hour > 10 && $hour <= 16 ) {
        $hello = ($default::english) ? 'Hello' : 'こんにちは';
    }
    elsif ( $hour > 16 || $hour <= 3 ) {
        $hello = ($default::english) ? 'Good evening' : 'こんばんは';
    }
    $tool =
      $this->login_link( $output,
        ( ($default::english) ? 'LOG IN' : 'ログイン' ),
        '', '', 1 );
    my $data = '';
    if ( defined $default::english && $default::english ) {
        $data =
            $hello
          . ', Mr/Ms. <strong>'
          . $user_name
          . '</strong>. If you are NOT Mr/Ms. <strong>'
          . $user_name
          . '</strong>, please <strong>'
          . $tool
          . '</strong>.';
    }
    else {
        $data =
            $hello
          . '、<strong>'
          . $user_name
          . '</strong> さん。もしあなたが <strong>'
          . $user_name
          . '</strong> さんではない場合、<strong>'
          . $tool
          . '</strong>してください。';
    }
    return $this->make_item_tag( 'welcome', $attributes, $data, $header,
        $footer, $file_flag );
}

sub login_link ($$$$) {
    my ( $this, $output, $text, $id, $class, $login ) = @_;
    return '<a href="'
      . (
        ($output) ? $this->base_change( $env::register_cgi, $output )
        : $env::register_cgi
      )
      . '?k=pswd&amp;m=Password'
      . ( ($login) ? '&amp;login=1' : '' ) . '"'
      . (
        ($env::register_in_same_window) ? ''
        : ' target="LoginWindow" onclick="cmonos.window.open(\'LoginWindow\',450,300)"'
      )
      . ( ( defined $id && $id ne '' ) ? ' id="' . $id . '"' : '' )
      . (
        ( defined $class && $class ne '' ) ? ' class="' . $class . '"'
        : ''
      )
      . '>'
      . (
          ($text)             ? $text
        : ($default::english) ? 'LOG IN'
        :                       'ログイン'
      ) . '</a>';
}

sub login_tool_replace ($$$;$) {
    my ( $this, $attributes, $content, $output, $file_flag ) = @_;
    my ( $header, $footer ) =
      $this->get_header_and_footer( $attributes, $content );
    my $login = ( $file_flag || !$main::email || !defined $main::pswd ) ? 1 : 0;
    my $login_text =
        ( $attributes =~ /\s(?:login|)text="([^"]+)"/is ) ? $this->unescape($1)
      : ($default::english)                               ? 'Log In.'
      :                                                     'ログイン';
    my $logout_text =
        ( $attributes =~ /\slogouttext="([^"]+)"/is ) ? $this->unescape($1)
      : ($default::english)                           ? 'Log Out.'
      :                                                 'ログアウト';
    my $login_id = ( $attributes =~ /\s(?:login|)id="([^"]+)"/is ) ? $1 : '';
    my $login_class =
      ( $attributes =~ /\s(?:login|)class="([^"]+)"/is ) ? $1 : '';
    my $logout_id    = ( $attributes =~ /\slogoutid="([^"]+)"/is )    ? $1 : '';
    my $logout_class = ( $attributes =~ /\slogoutclass="([^"]+)"/is ) ? $1 : '';
    my $data         = '';

    if (
        $attributes =~ /\slink="([^"]+)"/is
        || ( $login
            && ( $login_class || !$setting::page{'loginButton'} ) )
        || ( !$login
            && ( $logout_class || !$setting::page{'logoutButton'} ) )
      )
    {
        $data = (
            ($login)
            ? $this->login_link( $output, $login_text, $login_id,
                $login_class, 1 )
            : $this->login_link(
                $output, $logout_text, $logout_id, $logout_class
            )
        );
    }
    else {
        $this->use_deco($output);
        my %attr        = ();
        my $window_name = '';
        if ($env::register_in_same_window) {
            $attr{'onclick'} = 'cmonos.window.open(\'LoginWindow\',450,300)"';
            $window_name = 'LoginWindow';
        }
        if ($login) {
            $attr{'id'} = $login_id if $login_id ne '';
            $data = $cgi::Deco->button(
                'login',
                $setting::page{'loginButton'},
                $env::register_cgi . '?k=pswd&amp;m=Password&amp;login=1',
                $login_text,
                $window_name,
                \%attr
            );
        }
        else {
            $attr{'id'} = $logout_id if $logout_id ne '';
            $data = $cgi::Deco->button(
                'logout',
                $setting::page{'logoutButton'},
                $env::register_cgi . '?k=pswd&amp;m=Password',
                $logout_text,
                $window_name,
                \%attr
            );
        }
    }
    return $this->make_item_tag( 'login', $attributes, $data, $header,
        $footer, $file_flag );
}

######################################################################
#ツールメニュー
sub jump_menu ($$;$) {
    my ( $this, $key, $subkey ) = @_;
    return if !%setting::jump_menu;
    my ( @required, $code );
    push( @required, 'm' );
    push( @required, 'c' ) if defined $main::c && $main::c ne '';

    #設定選択メニュー
    $setting::page{$key}{'menu'} = $this->form_start(
        'key'      => $setting::security_key,
        'name'     => '',
        'required' => \@required
    );
    $code = '';
    $setting::page{$key}{'menu'} .= '<select name="k" onchange="submit()">';
    foreach ( sort keys %setting::jump_menu ) {
        next if $setting::master_check_option{$_} && !$this->is_master;
        if ( $_ =~ /^([\.\-0-9a-zA-Z_]{2})/ && $code ne $1 ) {
            $setting::page{$key}{'menu'} .=
              '<option value="' . $main::k . '"></option>'
              if defined $code && $code ne '';
            $code = $1;
        }
        $setting::page{$key}{'menu'} .=
            '<option value="'
          . $_ . '"'
          . (
            (
                $_ eq $main::k
                  || ( defined $setting::selected_jump_menu{$main::k}
                    && $_ eq $setting::selected_jump_menu{$main::k} )
            ) ? ' selected="selected"' : ''
          )
          . '>'
          . $setting::jump_menu{$_}
          . '</option>';
    }
    $setting::page{$key}{'menu'} .= '</select>';
    $setting::page{$key}{'menu'} .=
      '<input name="jumpMenu" type="submit" value="選択" /></form>';

    $setting::page{$key}{'topicpath'} = $env::title
      if defined $env::title && $env::title ne '';
    $setting::page{$key}{'topicpath'} .= (
        (
            defined $setting::page{$key}{'topicpath'}
              && $setting::page{$key}{'topicpath'} ne ''
        ) ? ' &gt; ' : ''
      )
      . $master::general{'cginame'}
      if $default::subdivided
      && defined $main::c
      && $main::c ne ''
      && defined $master::general{'cginame'}
      && $master::general{'cginame'} ne '';
    $setting::page{$key}{'topicpath'} .= (
        (
            defined $setting::page{$key}{'topicpath'}
              && $setting::page{$key}{'topicpath'} ne ''
        ) ? ' &gt; ' : ''
      )
      . $setting::general{'cginame'}
      if defined $setting::general{'cginame'}
      && $setting::general{'cginame'} ne '';
    $setting::page{$key}{'topicpath'} .= (
        (
            defined $setting::page{$key}{'topicpath'}
              && $setting::page{$key}{'topicpath'} ne ''
        ) ? ' &gt; ' : ''
      )
      . $setting::page{$key}{'title'}
      if defined $setting::page{$key}{'title'}
      && $setting::page{$key}{'title'} ne '';
    if ( defined $setting::jump_menu{$main::k}
        && $setting::jump_menu{$main::k} ne '' )
    {
        $setting::page{$key}{'topicpath'} .= (
            (
                defined $setting::page{$key}{'topicpath'}
                  && $setting::page{$key}{'topicpath'} ne ''
            ) ? ' &gt; ' : ''
        ) . $setting::jump_menu{$main::k};
        $this->push_window_title( $setting::jump_menu{$main::k} );
    }
    elsif (defined $setting::selected_jump_menu{$main::k}
        && defined $setting::jump_menu{ $setting::selected_jump_menu{$main::k} }
        && $setting::jump_menu{ $setting::selected_jump_menu{$main::k} } ne '' )
    {
        $setting::page{$key}{'topicpath'} .= (
            (
                defined $setting::page{$key}{'topicpath'}
                  && $setting::page{$key}{'topicpath'} ne ''
            ) ? ' &gt; ' : ''
        ) . $setting::jump_menu{ $setting::selected_jump_menu{$main::k} };
        $this->push_window_title(
            $setting::jump_menu{ $setting::selected_jump_menu{$main::k} } );
    }

    if (   $subkey
        && ref $setting::sub_jump_menu{$subkey}
        && %{ $setting::sub_jump_menu{$subkey} } )
    {
        $code = '';
        $setting::page{$key}{'subMenu'} = $this->form_start(
            'key'      => $setting::security_key,
            'name'     => '',
            'required' => \@required
        );
        $setting::page{$key}{'subMenu'} .=
          '<select name="k" onchange="submit()">';
        foreach ( sort keys %{ $setting::sub_jump_menu{$subkey} } ) {
            next
              if $setting::master_check_option{$_}
              && $this->is_master == 0;
            if ( $_ =~ /^([\.\-0-9a-zA-Z_]{3})/ && $code ne $1 ) {
                $setting::page{$key}{'subMenu'} .=
                  '<option value="' . $main::k . '"></option>'
                  if defined $code && $code ne '';
                $code = $1;
            }
            $setting::page{$key}{'subMenu'} .=
                '<option value="'
              . $_ . '"'
              . ( ( $_ eq $main::k ) ? ' selected="selected"' : '' ) . '>'
              . $setting::sub_jump_menu{$subkey}{$_}
              . '</option>';
        }
        $setting::page{$key}{'subMenu'} .= '</select>';
        $setting::page{$key}{'subMenu'} .=
          '<input name="subJumpMenu" type="submit" value="選択" /></form>';
        if ( defined $setting::sub_jump_menu{$subkey}{$main::k}
            && $setting::sub_jump_menu{$subkey}{$main::k} ne '' )
        {
            $setting::page{$key}{'topicpath'} .= (
                ( $setting::page{$key}{'topicpath'} ne '' )
                ? ' &gt; '
                : ''
            ) . $setting::sub_jump_menu{$subkey}{$main::k};
            $this->push_window_title(
                $setting::sub_jump_menu{$subkey}{$main::k} );
        }
    }
}

######################################################################
# obj->print_dialog(key,@msg)
sub print_dialog ($$@) {
    my ( $this, $key, @msg ) = @_;
    $this->print_page_header($key);
    if ( @msg > 1 ) {
        print '<ul class="dialog">';
        foreach (@msg) {
            next if !defined $_ || $_ eq '';
            $_ =~ s/\\r\\n/<br \/>/g;
            $_ =~ s/\\r/<br \/>/g;
            $_ =~ s/\\n/<br \/>/g;
            print '<li>' . $_ . '</li>';
        }
        print '</ul>';
    }
    else {
        $msg[0] =~ s/\\r\\n/<br \/>/g;
        $msg[0] =~ s/\\r/<br \/>/g;
        $msg[0] =~ s/\\n/<br \/>/g;
        my $p = ( $msg[0] !~ m!<\w+[^/]*>!s ) ? 1 : 0;
        print '<p>' if $p;
        print $msg[0];
        print '</p>' if $p;
    }
    print '<div id="close-button">'
      . $this->close_button(
        '',
        $setting::page{'closeButton'},
        $setting::page{'backButton'}
      ) . '</div>';
    $this->print_page_footer($key);
    exit;
}

######################################################################
#	obj->print_page_footer(key,handle)
sub print_page_footer ($$;$$$) {
    my ( $this, $key, $handle, $output, $remain ) = @_;
    my ($file);
    $key = caller if !defined $key || $key eq '';
    if ( !$handle || scalar(*$handle) eq scalar(*STDOUT) ) {
        $handle = \*STDOUT;
    }
    else {
        $file = 1;
    }

    #ページフッタ
    print $handle $this->page_item_replace( $setting::page{$key}{'footer'},
        $file, $output, $setting::page{$key}{'title'} )
      if $setting::page{$key}{'insideFooter'}
      && $setting::page{$key}{'footer'} ne '';
    print $handle $this->include_external_file(
        $env::externalfiles . '/' . $setting::page{$key}{'footerFileName'},
        $setting::page{$key}{'externalFooter'}, $output )
      if $setting::page{$key}{'insideExternalFooter'}
      && defined $setting::page{$key}{'footerFileName'}
      && $setting::page{$key}{'footerFileName'} ne '';

    #終了タグ
    if ( defined $setting::page{$key}{'deco'}
        && $setting::page{$key}{'deco'} )
    {
        print $handle $this->frame_end(
            $setting::page{$key}{'deco'},
            $setting::page{$key}{'width'},
            '', 'container'
        );
    }
    else {
        print $handle '</div>';
    }

    #ページフッタ
    print $handle $this->page_item_replace( $setting::page{$key}{'footer'},
        $file, $output, $setting::page{$key}{'title'} )
      if !$setting::page{$key}{'insideFooter'}
      && defined $setting::page{$key}{'footer'}
      && $setting::page{$key}{'footer'} ne '';
    print $handle $this->include_external_file(
        $env::externalfiles . '/' . $setting::page{$key}{'footerFileName'},
        $setting::page{$key}{'externalFooter'}, $output )
      if !$setting::page{$key}{'insideExternalFooter'}
      && defined $setting::page{$key}{'footerFileName'}
      && $setting::page{$key}{'footerFileName'} ne '';
    print $handle $setting::page{$key}{'maintain'}
      if defined $setting::page{$key}{'maintain'}
      && $setting::page{$key}{'maintain'};

    $this->print_html_footer($handle);

    return if defined $remain && $remain;
    undef %frame::start;
    undef %frame::end;
}

######################################################################
#	obj->frame_start(key,width,class,id,title,output)
sub frame_start ($$;$$$$) {
    my ( $this, $key, $width, $class, $id, $title, $output ) = @_;
    my ($fkey);
    $this->use_deco($output);
    $fkey =
        ( ( defined $key )   ? $key   : '' ) . '+'
      . ( ( defined $width ) ? $width : '' ) . '+'
      . ( ( defined $class ) ? $class : '' ) . '+'
      . ( ( defined $id )    ? $id    : '' );
    if ( defined $frame::start{$fkey} && $frame::start{$fkey} ) {
        $frame::start{$fkey} =~
s/<!--titlebar-->.*?<!--\/titlebar-->/<!--titlebar-->$title<!--\/titlebar-->/;
    }
    elsif (
           defined $key
        && $key ne ''
        && (
            ref $frame::obj{'frame'}{$key}
            || ( defined $setting::deco{'frame'}{$key}{'tlimagewidth'}
                && $setting::deco{'frame'}{$key}{'tlimagewidth'} ne '' )
        )
      )
    {
        my ( $temp, $contents );
        $contents = '_C_';
        $temp     = $cgi::Deco->frame(
            'frame',    $key,
            \$contents, \$title,
            ( ($output) ? 'tag' : 'no' ), '',
            0,      'framewidth',
            $width, 'frameheight',
            '',     'frameclass',
            $class, 'frameid',
            $id
        );
        ( $frame::start{$fkey}, $frame::end{$fkey} ) =
          split( /_C_/, $temp );
    }
    else {
        $frame::start{$fkey} = '<div'
          . (
            ( defined $class && $class ne '' ) ? ' class="' . $class . '"'
            : ''
          )
          . ( ( defined $id && $id ne '' ) ? ' id="' . $id . '"' : '' )
          . (
            ( defined $width && $width ne '' )
            ? ' style="width: ' . $width . 'px"'
            : ''
          )
          . '>'
          . ( ($output)          ? '<!--titlebar-->'  : '' )
          . ( ( defined $title ) ? $title             : '' )
          . ( ($output)          ? '<!--/titlebar-->' : '' );
        $frame::end{$fkey} = '</div>';
    }
    return $frame::start{$fkey};
}

#	obj->frame_end(key,width,class,id,delete_flag)
sub frame_end ($$;$$$$) {
    my ( $this, $key, $width, $class, $id, $delete_flag ) = @_;
    my $fkey =
        ( ( defined $key )   ? $key   : '' ) . '+'
      . ( ( defined $width ) ? $width : '' ) . '+'
      . ( ( defined $class ) ? $class : '' ) . '+'
      . ( ( defined $id )    ? $id    : '' );
    if ( defined $delete_flag && $delete_flag ) {
        my $frame_end = $frame::end{$fkey};
        delete $frame::start{$fkey};
        delete $frame::end{$fkey};
        return ($frame_end) ? $frame_end : '';
    }
    else {
        return ( $frame::end{$fkey} ) ? $frame::end{$fkey} : '';
    }
}

######################################################################
#	obj->get_frame(key)
sub get_frame ($;$$$$$$) {
    my ( $this, $key, $contents, $title, $tag, $class, $id ) = @_;
    $contents = ''   if !defined $contents;
    $title    = ''   if !defined $title;
    $tag      = 'no' if !defined $tag || $tag eq '';
    if ( defined $key && $key ne '' && ref $frame::obj{'frame'}{$key} ) {
        return $frame::obj{'frame'}{$key}
          ->frame( \$contents, \$title, $tag, '', $class, $id );
    }
    else {
        if ( $title ne '' ) {
            $title = '<h2>' . $title . '</h2>' if $title !~ /<[^<>]*>/;
            $contents = $title . $contents;
        }
        if ( $class || $id ) {
            return
                '<div'
              . ( ($id)    ? ' class="' . $id . '"'    : '' ) . ''
              . ( ($class) ? ' class="' . $class . '"' : '' ) . '>'
              . $contents
              . '</div>';
        }
        else {
            return $contents;
        }
    }
}

######################################################################
sub table_menu ($$$) {
    my ( $this, $name, $default ) = @_;
    my ($menu);
    $menu = '<select name="' . $name . '" id="' . $name . '">';
    $menu .= '<option value=""></option>';
    foreach ( sort { $a <=> $b || $a cmp $b } keys %table::obj ) {
        next unless ref $table::obj{$_};
        $menu .=
            '<option value="'
          . $_ . '"'
          . (
            ( defined $default && $_ eq $default )
            ? ' selected="selected"'
            : ''
          )
          . '>'
          . $table::obj{$_}->{'name'}
          . '</option>';
    }
    $menu .= '</select>';
    return $menu;
}

######################################################################
#	obj->print_tool_icon
sub print_tool_icon ($) {
    my $this = shift;
    print '<ul class="tools panel">';
    print '<li class="isolated"><a href="'
      . $env::register_cgi
      . '?m=Password&amp;k=pswd"'
      . (
        ($env::register_in_same_window)
        ? ''
        : ' target="LoginWindow" onclick="cmonos.window.open(\'LoginWindow\',450,300)"'
      )
      . ' title="ログイン"><img src="'
      . $env::image_dir
      . '/tool/login.gif" width="20" height="20" border="0" alt="ログイン" /></a></li>';
    print '<li class="isolated"><a href="'
      . $cgi::path
      . '?m=Tools" target="_blank" title="環境設定"><img src="'
      . $env::image_dir
      . '/tool/tools.gif" width="20" height="20" border="0" alt="環境設定" /></a></li>';
    print '</ul>';
}

######################################################################
#	obj->notice_mail
sub notice_mail_for_master ($;$$$$@) {
    my ( $this, $subject, $msg, $field, $data, @addr ) = @_;

    #機能停止中
    return if $setting::general{'mail'} eq '' && @addr == 0;

    unshift( @addr, $setting::general{'staff'} )
      if defined $setting::general{'mail'}
      && $setting::general{'mail'} eq 'staff';
    unshift( @addr, $setting::general{'owners'} )
      if defined $setting::general{'mail'}
      && ( $setting::general{'mail'} eq 'staff'
        || $setting::general{'mail'} eq 'owners' );
    unshift( @addr, $setting::general{'master'} )
      if defined $setting::general{'mail'}
      && $setting::general{'mail'} ne '';

    return if !@addr;
    return $this->notice_mail( $subject, $msg, $field, $data,
        $setting::general{'master'}, @addr );
}

sub notice_mail ($$$$;$$$@) {
    my ( $this, $subject, $msg, $field, $data, $from, @addr ) = @_;
    my ($bcc);
    @addr = $this->get_address(@addr);
    return '' if !@addr;

    $subject = 'データが更新されました。'
      if !defined $subject || !$subject;
    $msg = 'データが更新されました。'
      if !defined $msg || !$msg;
    $msg .= "\x0A\x0A\x0A" if defined $msg && $msg;

    #送信先
    $bcc = join( ',', @addr );

    #差出人
    $from = $env::master_email if !defined $from || !$from;

    my $list = '';
    $list = $this->data_for_notice_mail( $field, $data )
      if ref $field && ref $data;

    #CGI情報
    my $info =
"\x0A\x0A============================================================ \x0A";
    $info .=
      $setting::general{'cginame'} . "\x0A" . $this->my_default_url . "\x0A";
    $info .=
      "============================================================ \x0A\x0A";

    unless ( ref $cgi::SendMail ) {
        require CMONOS::CGI_Lib::SendMail;
        $cgi::SendMail =
          SendMail->new( $env::sendmail, $env::master_email, '',
            $cgi::input, 1 );
    }

    $cgi::SendMail->send_mail(
        'Bcc'     => $bcc,
        'From'    => $from,
        'Subject' => $subject,
        'message' => $msg . $list . $info,
        'file'    => ( ( ref $data && defined $mail::file ) ? $mail::file : '' )
    );
    push( @{ $this->{'dialog'} }, $cgi::SendMail->dialog );
    return $list;
}

sub my_default_url ($) {
    my $this = shift;
    return $this->my_url
      . ( ( defined $main::c && $main::c ) ? '?c=' . $main::c : '' );
}

sub data_for_notice_mail ($$$) {
    my ( $this, $field, $data ) = @_;
    my $list = '';
    foreach my $id (
        sort {
            (        ref $field->{$a}
                  && ref $field->{$b}
                  && $field->{$a}->{'ORDER'} <=> $field->{$b}->{'ORDER'} )
              || $a cmp $b
        } keys %$data
      )
    {
        next
          if !ref $field->{$id}
          || !$field->{$id}->format_for_mail_enabled
          || $field->{$id}->{'Hidden'}
          || ( defined $field->{$id}->{'Open'}
            && !$field->{$id}->{'Open'} );
        $list .= $field->{$id}->format_for_mail( $$data{$id}{'str'} );
    }
    return $list;
}

######################################################################
# 戻る/インデックスへのリンク
sub print_page_return_form ($;$$) {
    my ( $this, $index_key, $index_module ) = @_;
    return
      unless ( defined $setting::page{'returnUrl'}
        && $setting::page{'returnUrl'} ne '' )
      || ( defined $index_key
        && defined $index_module
        && $index_key ne ''
        && $index_module ne '' );
    if ( defined $this->{'PageReturnForm'} && $this->{'PageReturnForm'} ) {
        print $this->{'PageReturnForm'};
        return;
    }
    my ( @required, $ul_flag );
    $this->use_deco($cgi::output);
    @required = grep( $_ ne 'm' && $_ ne 'k', @setting::form );
    $ul_flag = 1
      if defined $setting::page{'returnUrl'}
      && $setting::page{'returnUrl'} ne ''
      && defined $index_key
      && defined $index_module
      && $index_key ne ''
      && $index_module ne '';

    $this->{'PageReturnForm'} = '<div class="jump-list">';
    $this->{'PageReturnForm'} = '<ul class="tools">'
      if defined $ul_flag && $ul_flag;

    if ( defined $setting::page{'returnUrl'}
        && $setting::page{'returnUrl'} ne '' )
    {
        $this->{'PageReturnForm'} .= '<li class="isolated">'
          if defined $ul_flag && $ul_flag;
        $this->{'PageReturnForm'} .= $cgi::Deco->button(
            'return',
            $setting::page{'returnButton'},
            $setting::page{'returnUrl'},
            ( ($default::english) ? 'Return' : '戻る' )
        );
        $this->{'PageReturnForm'} .= '</li>'
          if defined $ul_flag && $ul_flag;
    }

    if (   defined $index_key
        && defined $index_module
        && $index_key ne ''
        && $index_module ne '' )
    {
        $this->{'PageReturnForm'} .= '<li class="isolated">'
          if defined $ul_flag && $ul_flag;
        if ( $setting::general{'postMethod'} ) {
            $this->{'PageReturnForm'} .= $this->form_start(
                'key'      => $setting::security_key,
                'name'     => 'index',
                'required' => \@required
            );
            $this->{'PageReturnForm'} .= $cgi::Deco->submit_button(
                'index',
                $setting::page{'indexButton'},
                ( ($default::english) ? 'Index' : 'インデックス' ),
                '',
                { 'm' => $index_module, 'k' => $index_key }
            );
            $this->{'PageReturnForm'} .= '</form>';
        }
        else {
            $this->{'PageReturnForm'} .= $cgi::Deco->button(
                'index',
                $setting::page{'indexButton'},
                './start.cgi'
                  . $this->set_query( 'url', \@required )
                  . $this->add_query(
                    'url', { 'm' => $index_module, 'k' => $index_key }
                  ),
                ( ($default::english) ? 'Index' : 'インデックス' )
            );
        }
        $this->{'PageReturnForm'} .= '</li>'
          if defined $ul_flag && $ul_flag;
    }

    $this->{'PageReturnForm'} .= '</ul>' if defined $ul_flag && $ul_flag;
    $this->{'PageReturnForm'} .= '</div>';

    print $this->{'PageReturnForm'};
}

1;
