package TaketoriLegacy;
use strict;
use utf8;
######################################################################
# HTML/プレーンTEXTを縦書き表示HTMLに変換するモジュール
# TaketoriLegacy.pm 1.1.4
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
# 謝辞：このスクリプトは竹原英知氏（http://www.yume.net/）の依頼と資金
# 　　　協力がきっかけとなって開発されました。竹原英知氏に感謝いたしま
# 　　　す。
# 最終更新日：2011-09-10
######################################################################
#
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	(0)-0 オブジェクトを作る。
#		require CMONOS::CGI_Lib::TaketoriLegacy;
#		obj = TaketoriLegacy->new(
#			'name' => '',										# この縦書き区画の名前
#			'type' => 'text',									# text=改行コードでも改行
#			'rows' => '18',										# 一行字数
#			'cols' => '20',										# 一段行数
#			'marginEnabled' => 1,								# 空白と記号で小書き文字を調節せず、margin を設定して位置を調整する
#			'body' => 'bgcolor="#ffffff" link="#00ffff"',		# <body>タグオプションの初期値
#			'Set_Cookie' => '',									# Set-Cookieヘッダ
#			'html_header' => '',								# HTMLHeader追加分
#			'page_title' => 'タイトル',							# タイトル
#			'basehref' => 'url',								# BASE href="..."
#			'basefontsize' => '3',								# 標準フォントサイズ（1-7）
#			'fontOption' => 'face="osaka,ＭＳ−ゴシック"',		# 標準フォントオプション
#			'header' => '',										# ページヘッダ（広告等挿入）
#			'footer' => '',										# ページフッタ（広告等挿入）
#			'align' => 'right',									# 位置
#			'separator' => '<hr size="1" color="#000000" />',	# 段区切り
#			'jcode' => 'utf8',									# 漢字コード（euc/jis/sjis/utf8）
#			'numOption' => 1,									# 1=数字は横書きにする
#			'script' => 'no',							# オリジナルファイルのスクリプト（no=無視する）
#			'css' => 'no',								# オリジナルファイルのCSS（no=無視する）
#			'kinsokuLevel' => '1',						# 禁則処理設定（1=句読点のみ/2=句読点+ゃゅょ等）
#			'upwardChar' => '▲',						# 上段移動文字
#			'downwardChar' => '▼',						# 下段移動文字
#			'Mac' => '1'					# "1"=マック版NNに最適化する/"auto"=クライアントに合わせる
#		);
#
#	(0)-1 初期設定を変更する。
#		obj->set(
#			...設定は(0)-0に準じます...
#		);
#
#	(0)-2 設定をリセットする
#		obj->reset_all;
#
# ---------------------------------------------------------------------------------------
#	(1) obj->tategaki(text,out[,clear,option]);
#		縦書き表示HTMLに変換したtextをoutに出力します。
# 			text		変換するテキスト
# 								スカラー変数のリファレンス->オリジナル変数を縦書き化
# 								ファイルハンドルのリファレンス->ハンドルを縦書き化
# 								スカラー変数->変換元ファイルのパス
# 			out			縦書き表示に変換されたテキストの出力先
# 								ファイルハンドルのリファレンス->ハンドルに出力
# 								スカラー変数のリファレンス->出力先変数
# 								スカラー変数->出力先ファイルのパス
# 								"<return>"->変換したテキストを返す
# 								設定無し->標準出力に出力するかtextのオリジナルを縦書き化
# 			clear		1=呼び出した時点で段数を初期化します。
# 						2=呼び出した時点で段数を初期化し、nameに1を加算します。
# 			option		1=HTMLヘッダ/フッタを追加します。
# 								第一引数がスカラー変数のときのみ有効。それ以外では常に"1"。
# 								第二引数がスカラー変数のリファレンスの場合は常に"0"です。
#
# ---------------------------------------------------------------------------------------
#	(2)-1 obj->text_to_tategaki(text,tag);
#		テキスト中の<!--tate--><!--/tate-->で囲まれた部分を縦書き表示に変換します。
# 			text			テキスト
# 			tag				"1"=タグ（<!--_tate-->tate<!--/_tate-->）を付ける
#
# 			※	タグに設定値を埋め込むことができます。
# 			例）	<!--tate row="18" col="20"-->text<!--/tate-->
#
######################################################################
#	CSS
#
#	table.tategaki {
#	  empty-cells: show !important;
#	  background-color: transparent !important;
#	  background-image: none !important;
#	  border: none !important;
#	  border-collapse: collapse !important;
#	  border-spacing: 0 !important;
#	  padding: 0 !important;
#	}
#	table.tategaki td.gyo {
#	  margin: 0 !important;
#	  padding: 0 !important;
#	  background-color: transparent !important;
#	  background-image: none !important;
#	  border: none !important;
#	  line-height: normal !important;
#	  text-align: center !important;
#	  white-space: nowrap !important;
#	  vertical-align: top;
#	}
#	table.tategaki td.gyo a {
#	  text-decoration: none !important;
#	}
#
######################################################################
#	(0)-0 オブジェクトを作る。
use Jcode;

sub jcode_convert($$$$;$) {
    my ( $this, $str, $ocode, $icode, $was_entity ) = @_;
    my $str_ref = ( ref $str ) ? $str : \$str;
    $ocode = 'utf8' if !$ocode;
    if ( !$icode || $ocode ne $icode ) {
        if ( defined $Jcode::FALLBACK ) {
            if ( $ocode eq 'utf8' ) {
                $$str_ref = Jcode->new( $str_ref, $icode )
                    ->fallback(&Jcode::FB_XMLCREF)->utf8;
            }
            elsif ( $ocode eq 'euc' ) {
                $$str_ref = Jcode->new( $str_ref, $icode )
                    ->fallback(&Jcode::FB_XMLCREF)->euc;
            }
            elsif ( $ocode eq 'jis' ) {
                $$str_ref = Jcode->new( $str_ref, $icode )
                    ->fallback(&Jcode::FB_XMLCREF)->jis;
            }
            elsif ( $ocode eq 'sjis' ) {
                $$str_ref = Jcode->new( $str_ref, $icode )
                    ->fallback(&Jcode::FB_XMLCREF)->sjis;
            }
            elsif ( $ocode eq 'ucs2' ) {
                $$str_ref = Jcode->new( $str_ref, $icode )
                    ->fallback(&Jcode::FB_XMLCREF)->ucs2;
            }
            else {
                Jcode::convert( $str_ref, $ocode, $icode );
            }
        }
        else {
            Jcode::convert( $str_ref, $ocode, $icode );
        }
    }
    if ( $ocode eq 'utf8' ) {
        utf8::decode($$str_ref) if !utf8::is_utf8($$str_ref);
    }
    elsif ( utf8::is_utf8($$str_ref) ) {
        utf8::encode($$str_ref);
    }
    $$str_ref
        =~ s/\&\#([0-9]{4,}|x[a-fA-F0-9]+);/$this->_entity_to_jcode($1,$ocode)/seg
        if !$was_entity && $ocode ne 'ucs2';
    return $$str_ref;
}

sub _entity_to_jcode ($$$) {
    my ( $this, $code, $ocode ) = @_;
    utf8::encode($code) if utf8::is_utf8($code);
    $code = sprintf( "%X", $code ) unless $code =~ s/^x//;
    return $this->jcode_convert( pack( "U", hex($code) ), $ocode, 'utf8', 1 );
}

sub ename_to_jname ($$) {
    my ( $this, $ename ) = @_;
    return 'utf8' if !$ename;
    $ename = lc $ename;
    my %ename2j = (
        'shiftjis'    => 'sjis',
        'x-sjis'      => 'sjis',
        'shift_jis'   => 'sjis',
        'euc-jp'      => 'euc',
        'x-euc-jp'    => 'euc',
        '7bit-jis'    => 'jis',
        'x-jis'       => 'jis',
        'iso-2022-jp' => 'jis',
        'utf-16be'    => 'ucs2',
        'utf-8'       => 'utf8',
    );
    if ( $ename2j{$ename} ) {
        return $ename2j{$ename};
    }
    else {
        $ename =~ tr/-/_/;
        return $ename;
    }
}

sub h_to_z ($$;$$) {
    my ( $this, $text, $icode, $ascii ) = @_;
    return '' if !defined $text || $text eq '';
    if ( !$icode || ( $icode ne 'utf8' && $icode ne 'ascii' ) ) {
        my $jcode = Jcode->new;
        $text = $jcode->set( \$text, $icode )->utf8;
        $icode = $this->ename_to_jname( $jcode->icode ) if !$icode;
    }
    utf8::decode($text) if !utf8::is_utf8($text);
    if ($ascii) {
        $text =~ s/\x20/\x{3000}/gs;
        $text =~ s/([\x{0020}-\x{007e}])/pack("U",unpack("U",$1) + 65248)/ges;
    }
    $text =~ s/･/・/gs;
    $text =~ s/ｦ/ヲ/gs;
    $text =~ s/ｧ/ァ/gs;
    $text =~ s/ｨ/ィ/gs;
    $text =~ s/ｩ/ゥ/gs;
    $text =~ s/ｪ/ェ/gs;
    $text =~ s/ｫ/ォ/gs;
    $text =~ s/ｬ/ャ/gs;
    $text =~ s/ｭ/ュ/gs;
    $text =~ s/ｮ/ョ/gs;
    $text =~ s/ｯ/ッ/gs;
    $text =~ s/ｰ/ー/gs;
    $text =~ s/ｱ/ア/gs;
    $text =~ s/ｲ/イ/gs;
    $text =~ s/ｳﾞ/ヴ/gs;
    $text =~ s/ｳ/ウ/gs;
    $text =~ s/ｴ/エ/gs;
    $text =~ s/ｵ/オ/gs;
    $text =~ s/ｶﾞ/ガ/gs;
    $text =~ s/ｶ/カ/gs;
    $text =~ s/ｷﾞ/ギ/gs;
    $text =~ s/ｷ/キ/gs;
    $text =~ s/ｸﾞ/グ/gs;
    $text =~ s/ｸ/ク/gs;
    $text =~ s/ｹﾞ/ゲ/gs;
    $text =~ s/ｹ/ケ/gs;
    $text =~ s/ｺﾞ/ゴ/gs;
    $text =~ s/ｺ/コ/gs;
    $text =~ s/ｻﾞ/ザ/gs;
    $text =~ s/ｻ/サ/gs;
    $text =~ s/ｼﾞ/ジ/gs;
    $text =~ s/ｼ/シ/gs;
    $text =~ s/ｽﾞ/ズ/gs;
    $text =~ s/ｽ/ス/gs;
    $text =~ s/ｾﾞ/ゼ/gs;
    $text =~ s/ｾ/セ/gs;
    $text =~ s/ｿﾞ/ゾ/gs;
    $text =~ s/ｿ/ソ/gs;
    $text =~ s/ﾀﾞ/ダ/gs;
    $text =~ s/ﾀ/タ/gs;
    $text =~ s/ﾁﾞ/ヂ/gs;
    $text =~ s/ﾁ/チ/gs;
    $text =~ s/ﾂﾞ/ヅ/gs;
    $text =~ s/ﾂ/ツ/gs;
    $text =~ s/ﾃﾞ/デ/gs;
    $text =~ s/ﾃ/テ/gs;
    $text =~ s/ﾄﾞ/ド/gs;
    $text =~ s/ﾄ/ト/gs;
    $text =~ s/ﾅ/ナ/gs;
    $text =~ s/ﾆ/ニ/gs;
    $text =~ s/ﾇ/ヌ/gs;
    $text =~ s/ﾈ/ネ/gs;
    $text =~ s/ﾉ/ノ/gs;
    $text =~ s/ﾊﾞ/バ/gs;
    $text =~ s/ﾊﾟ/パ/gs;
    $text =~ s/ﾊ/ハ/gs;
    $text =~ s/ﾋﾞ/ビ/gs;
    $text =~ s/ﾋﾟ/ピ/gs;
    $text =~ s/ﾋ/ヒ/gs;
    $text =~ s/ﾌﾞ/ブ/gs;
    $text =~ s/ﾌﾟ/プ/gs;
    $text =~ s/ﾌ/フ/gs;
    $text =~ s/ﾍﾞ/ベ/gs;
    $text =~ s/ﾍﾟ/ペ/gs;
    $text =~ s/ﾍ/ヘ/gs;
    $text =~ s/ﾎﾞ/ボ/gs;
    $text =~ s/ﾎﾟ/ポ/gs;
    $text =~ s/ﾎ/ホ/gs;
    $text =~ s/ﾏ/マ/gs;
    $text =~ s/ﾐ/ミ/gs;
    $text =~ s/ﾑ/ム/gs;
    $text =~ s/ﾒ/メ/gs;
    $text =~ s/ﾓ/モ/gs;
    $text =~ s/ﾔ/ヤ/gs;
    $text =~ s/ﾕ/ユ/gs;
    $text =~ s/ﾖ/ヨ/gs;
    $text =~ s/ﾗ/ラ/gs;
    $text =~ s/ﾘ/リ/gs;
    $text =~ s/ﾙ/ル/gs;
    $text =~ s/ﾚ/レ/gs;
    $text =~ s/ﾛ/ロ/gs;
    $text =~ s/ﾜ/ワ/gs;
    $text =~ s/ﾝ/ン/gs;
    $this->jcode_convert( \$text, $icode, 'utf8' )
        if $icode ne 'utf8' && $icode ne 'ascii';
    return $text;
}

sub new ($@) {
    my ( $pkg, @setting ) = @_;
    my $this = {};
    bless $this;

    @setting = (
        'rows'         => '18',
        'cols'         => '20',
        'body'         => 'bgcolor="#ffffff"',
        'basefontsize' => '3',
        'separator'    => '<hr size="1" color="#000000" /><br /><br />',
        'script'       => 'no',
        'css'          => 'no',
        'jcode'        => 'utf8',
        'align'        => 'right',
        'kinsokuLevel' => '1'
    ) if !@setting;

    $this->set(@setting);
    $this;
}

#	(0)-1 初期設定
sub set ($@) {
    my ( $this, @setting ) = @_;

    #文字列設定の処理
    if ( @setting == 1 ) {
        my $setting = $setting[0];
        @setting = ();
        while ( $setting
            =~ /[\x09\x0A\x0C\x0D\x20]*([0-9a-zA-Z_]+)[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"([^"]*)"/gs
            )
        {
            my ( $k, $v ) = ( $1, $2 );
            $v =~ s/&lt;/</sg;
            $v =~ s/&gt;/>/sg;
            $v =~ s/&quot;/"/sg;
            $v =~ s/&#10;/\x0A/sg;
            $v =~ s/&amp;/\&/sg;
            push( @setting, $k, $v );
        }
    }

    my ( $k, $v );
    while ( ( $k, $v ) = splice( @setting, 0, 2 ) ) {
        next if !defined $k || $k eq '';
        $this->{$k} = $v;
    }

    #必須設定のチェック
    $this->{'rows'} = '18' if !$this->{'rows'} || $this->{'rows'} < 1;
    $this->{'upwardChar'} = "<strong>↑</strong>"
        if !defined $this->{'upwardChar'} || $this->{'upwardChar'} eq '';
    $this->{'downwardChar'} = "<strong>↓</strong>"
        if !defined $this->{'downwardChar'} || $this->{'downwardChar'} eq '';
    $this->{'jcode'} = 'utf8'
        if !defined $this->{'jcode'} || $this->{'jcode'} eq '';
    $this->{'body'} = 'bgcolor="#ffffff"'
        if !defined $this->{'body'} || $this->{'body'} eq '';
    $this->{'kinsokuLevel'} = 1
        if !defined $this->{'kinsokuLevel'} || $this->{'kinsokuLevel'} eq '';
    $this->{'name'} = 'tategaki'
        if !defined $this->{'name'} || $this->{'name'} eq '';
    $this->{'name'} = 'tategaki-' . $this->{'name'}
        if $this->{'name'} =~ /^[0-9]/;

    #本文漢字コード統一
    $this->{'fontOption'}
        = $this->jcode_convert( \$this->{'fontOption'}, 'utf8' )
        if defined $this->{'fontOption'} && $this->{'fontOption'} ne '';
    $this->{'upwardChar'}
        = $this->jcode_convert( \$this->{'upwardChar'}, 'utf8' )
        if defined $this->{'upwardChar'} && $this->{'upwardChar'} ne '';
    $this->{'downwardChar'}
        = $this->jcode_convert( \$this->{'downwardChar'}, 'utf8' )
        if defined $this->{'downwardChar'} && $this->{'downwardChar'} ne '';
    $this->{'separator'}
        = $this->jcode_convert( \$this->{'separator'}, 'utf8' )
        if defined $this->{'separator'} && $this->{'separator'} ne '';

    if ( defined $this->{'Mac'} && $this->{'Mac'} eq '1' ) {
        return;
    }
    elsif (
           defined $this->{'Mac'}
        && $this->{'Mac'} eq 'auto'
        && $ENV{'HTTP_USER_AGENT'} =~ /mac/i
        && ((      $ENV{'HTTP_USER_AGENT'} =~ /Mozilla\/([0-9\.]+)/
                && $1 < 5
                && $ENV{'HTTP_USER_AGENT'} =~ /[NIU];/
            )
            || ( $ENV{'HTTP_USER_AGENT'}
                =~ /MSIE[\x09\x0A\x0C\x0D\x20]+([0-9\.]+)/ && $1 > 5 )
        )
        )
    {
        $this->{'Mac'} = 1;
    }
    else {
        $this->{'Mac'} = '';
    }
    return $this;
}

#	(0)-2 設定のリセット
sub reset_all ($) {
    my $this = shift;
    undef %{$this};
    return $this;
}

#	(0)-3 設定の取得
sub get_setting ($) {
    my $this = shift;
    my ( $k, $v, $option );

    while ( ( $k, $v ) = each %{$this} ) {
        next if $k eq '' || $v eq '';
        next if ref $v;
        $v =~ s/\&/&amp;/sg;
        $v =~ s/</&lt;/sg;
        $v =~ s/>/&gt;/sg;
        $v =~ s/"/&quot;/sg;
        $v =~ s/\x0A/&#10;/g;
        $option .= ' ' . $k . '="' . $v . '"';
    }

    return $option;
}

######################################################################
#	(1) obj->tategaki(text[,option]);
#		textを縦書き表示HTMLに変換します。
sub tategaki ($$$;$$) {
    local ( *TEXT, *OUT );
    my ( $this, $text, $out, $clear, $option ) = @_;
    my ( $opened, $temp );

    # 各変数の初期化
    $this->{'handle_flag'} = 0,

        #$this->{'output'} = '',
        $this->{'textcolor'} = '';
    @{ $this->{'fsize'} } = ();
    $this->{'col_count'} = 0;
    @{ $this->{'string'} } = ();
    $this->{'toptags'} = '';
    @{ $this->{'toptag'} } = ();
    $this->{'bottomtags'} = '';
    $this->{'alt_text'}   = '';
    @{ $this->{'tate_align'} } = ();
    $this->{'subalign'}        = '';
    $this->{'colwidth'}        = 0;
    $this->{'rowheight'}       = 0;
    $this->{'valign'}          = '';
    $this->{'blockquote_flag'} = 0;
    $this->{'li_type'}         = '';
    $this->{'li_num'}          = 0;
    $this->{'dl_type'}         = '';
    $this->{'table_flag'}      = 0;
    $this->{'yoko_flag'}       = 0;
    $this->{'txt_flag'}        = 0;

    # 入力先を開く
    if ( ref $text eq 'SCALAR' ) {
        $this->{'output'} = $text if !defined $out || $out eq '';
        $text = $$text;
    }
    elsif ( ref $text eq 'GLOB' ) {
        *TEXT = $$text;
        $text = join( '', <TEXT> );
        close(TEXT);
        $option = 1;
    }
    elsif ( open( TEXT, $text ) ) {
        binmode TEXT;
        $text = join( '', <TEXT> );
        close(TEXT);
        $option = 1;
    }
    $text = '' if !defined $text;

    # 各変数の初期化
    $this->{'basefontsize'}
        = ( defined $this->{'basefontsize'} && $this->{'basefontsize'} )
        ? $this->{'basefontsize'}
        : 3;
    $this->{'fontsize'}  = 1;
    $this->{'cc'}        = 0;    # 行カウント
    $this->{'rc'}        = 0;    # 字数カウント
    $this->{'col_count'} = 0;    # 実際の行数
    if (   defined $clear
        && $clear == 2
        && $this->{'row_no'}
        && $this->{'name'} )
    {
        $this->{'name'} .= '-2'
            unless $this->{'name'} =~ s/([0-9]+)$/($1+1)/e;    #名前変更
    }
    $this->{'row_no'} = 0 if $clear || $option;                # 段数
    if ( defined $this->{'fontOption'} && $this->{'fontOption'} ne '' ) {
        $this->{'toptags'}    = '<font ' . $this->{'fontOption'} . '>';
        $this->{'bottomtags'} = '</font>';
        unshift( @{ $this->{'toptag'} }, $this->{'toptags'} );
    }

    # utf8/LF形式に変換
    $text = $this->h_to_z( $this->jcode_convert( $text, 'utf8' ) );
    $text =~ s/\x0D\x0A/\x0A/gs;
    $text =~ s/\x0D/\x0A/gs;

    # オリジナルファイルのスクリプトの除去
    $text =~ s!<script[^>]*>.*?</script>!!igs if $this->{'script'} eq 'no';

    # オリジナルファイルのスタイルの除去
    $text =~ s!<style[^>]*>.*?</style>!!igs if $this->{'css'} eq 'no';

    # 出力先を開く
    if ( ref $this->{'output'} ) {
        ${ $this->{'output'} } = '';
    }
    elsif ( $out eq '<return>' ) {
        $temp                  = '';
        $this->{'output'}      = \$temp;
        $this->{'handle_flag'} = 0;
    }
    elsif ( ref $out eq 'SCALAR' ) {
        $this->{'output'}      = $out;
        $this->{'handle_flag'} = 0;
    }
    elsif ( ref $out eq 'GLOB' ) {
        *OUT = $$out;
        $this->{'handle_flag'} = 1;
    }
    elsif ( $out ne '' && open( OUT, ">>$out" ) ) {
        use Fcntl qw(:flock);
        flock( OUT, LOCK_EX );
        truncate( OUT, 0 );
        seek( OUT, 0, 0 );
        $opened = 1;
        $this->{'handle_flag'} = 1;
    }
    else {
        *OUT = *STDOUT;
        print OUT 'Content-Type: text/html' . "\n";
        print OUT $this->{'Set_Cookie'}
            if defined $this->{'Set_Cookie'} && $this->{'Set_Cookie'} ne '';
        print OUT "\n";
        $this->{'handle_flag'} = 1;
    }

    # HTMLヘッダ部分の抽出
    if ( $text =~ /\A(.*)<body(?![0-9a-zA-Z_])([^>]*)>(.*)\Z/igs ) {
        $this->{'body'} = $2;
        $text = $3;
        $this->get_header($1);
    }

    # HTMLヘッダの出力
    $this->print_html_header if $this->{'handle_flag'} && $option;

    # ページヘッダの出力
    if ( defined $this->{'header'} ) {
        if ( defined $this->{'handle_flag'} && $this->{'handle_flag'} ) {
            print OUT $this->{'header'};
        }
        else {
            ${ $this->{'output'} } .= $this->{'header'};
        }
    }
    $this->{'textcolor'} = $1
        if $this->{'body'}
        =~ /text[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20"]*([0-9a-zA-Z_#]+)[\x09\x0A\x0C\x0D\x20"]+/is;
    $this->{'textcolor'} = '#000000'
        if !defined $this->{'textcolor'} || $this->{'textcolor'} eq '';

    # 区切り文字の除去
    $text =~ s/\t/ /g;

# 縦書き化（数字は横書き）
# タグ→エスケープ文字→エスケープ文字→2バイト文字（→桁数字）→1バイト文字→スペース→改行文字（3）
    my ( $num, $tag, $escape, $abc, $crlf );
    $num
        = ( defined $this->{'numOption'} && $this->{'numOption'} )
        ? '|[0-9][\,\.0-9]*'
        : '';
    $tag    = '<\!--.*?-->|<[^>]+>';
    $escape = '|&#[0-9]{2,6};|&#x[a-fA-F0-9]+;|&[a-zA-Z]+;';
    $abc    = '|[\?\!]{1,3}|[\x21-\x7f]|[\x20]+';
    $crlf   = '|\x0A';
    $text =~ s/($tag$escape$num$abc$crlf|.)/$this->yoko_to_tate($1)/egs;

    $this->print_text(1)
        if ref $this->{'string'} && @{ $this->{'string'} } > 0;

    if ( defined $this->{'handle_flag'} && $this->{'handle_flag'} ) {
        print OUT $this->{'separator'} if defined $this->{'separator'};
        print OUT $this->{'footer'}    if defined $this->{'footer'};
    }
    else {
        ${ $this->{'output'} } .= $this->{'separator'}
            if defined $this->{'separator'};
        ${ $this->{'output'} } .= $this->{'footer'}
            if defined $this->{'footer'};
    }

    $this->print_html_footer if $this->{'handle_flag'} && $option;
    close(OUT) if $opened;

    foreach my $name (

        #	'body',
        #	'page_title',
        #	'basehref',
        #	'html_header',
        'handle_flag',
        'output',
        'textcolor',
        'fsize',
        'col_count',
        'string',
        'toptags',
        'toptag',
        'bottomtags',
        'str',
        'align',
        'subalign',
        'colwidth',
        'rowheight',
        'valign',
        'blockquote_flag',
        'LI_type',
        'LI_flag',
        'DL_type',
        'table_flag',
        'yoko_flag',
        'txt_flag',
        'basefontsize',
        'fontsize',
        'cc',
        'rc',
        'col_count'
        )
    {
        delete $this->{$name} if defined $this->{$name};
    }
    return $temp if defined $temp && $temp;
    return 1;
}

######################################################################
# 縦書き化
sub yoko_to_tate {
    my ( $tmp_rc, $oidashi_flag, $kinsoku_flag, $kaigyo_flag, $h, $tmptags,
        $textdeco );
    $tmp_rc = $oidashi_flag = $kinsoku_flag = $kaigyo_flag = 0;
    $h = $tmptags = $textdeco = '';
    my ( $this, $w ) = @_;

    #テーブルタグ・横書き指定タグの中は処理しない
    if ( $this->{'table_flag'} <= 0 && $this->{'yoko_flag'} <= 0 ) {

        #タグの場合の処理
        if ( $w =~ /^<[^>]+>$/s ) {
            if ( $w =~ /^<br(?![0-9a-zA-Z_])[^>]*>$/is ) {
                $w            = '';
                $oidashi_flag = 1;
            }
            elsif ( $w
                =~ /^<(span|b|i|u|tt|em|strong|cite|kbd|var|dfn|code|samp|sup|sub|s)(?![0-9a-zA-Z_])[^>]*>$/is
                )
            {
                $textdeco = $1;
                unshift( @{ $this->{'toptag'} }, $w );
                $this->{'bottomtags'}
                    = "</$textdeco>" . $this->{'bottomtags'};
            }
            elsif ( $w
                =~ /^<\/(span|b|i|u|tt|em|strong|cite|kbd|var|dfn|code|samp|sup|sub|s)(?![0-9a-zA-Z_])[^>]*>$/is
                )
            {
                $textdeco = $1;

                # 行頭タグの削除
                $tmptags = join( "\t", @{ $this->{'toptag'} } );
                $tmptags =~ s/<$textdeco(?![0-9a-zA-Z_])[^>]*>//is;
                @{ $this->{'toptag'} } = split( /\t+/, $tmptags );

                $this->{'bottomtags'} =~ s!</$textdeco>!!i;
            }
            elsif ( $w =~ /^<font(?![0-9a-zA-Z_])[^>]*>$/is ) {
                $this->{'fontsize'}
                    = $this->font_size( $1, $this->{'basefontsize'} )
                    if $w
                    =~ /^<font[^>]*size[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20"]*([0-9])[^>]*>$/is;
                $w =~ s/^<font/<font/i;
                unshift( @{ $this->{'fsize'} },  $this->{'fontsize'} );
                unshift( @{ $this->{'toptag'} }, $w );
                $this->{'bottomtags'} = '</font>' . $this->{'bottomtags'};
            }
            elsif ($w =~ m!^</font>$!i
                || $w =~ /^<\/h[0-9](?![0-9a-zA-Z_])[^>]*>$/is )
            {
                shift( @{ $this->{'fsize'} } );
                $this->{'fontsize'} = $this->{'fsize'}->[0];
                $this->{'fontsize'} = 1 if !$this->{'fontsize'};

                # 行頭フォントタグの削除
                $tmptags = join( "\t", @{ $this->{'toptag'} } );
                $tmptags =~ s/<font[^>]*>//i;
                @{ $this->{'toptag'} } = split( /\t+/, $tmptags );

                $this->{'bottomtags'} =~ s!</font>!!i;

                if ( $w =~ /^<\/h[0-9](?![0-9a-zA-Z_])[^>]*>$/is ) {
                    shift( @{ $this->{'tate_align'} } );
                    $w           = '</font>';
                    $kaigyo_flag = 1;
                }
            }
            elsif ( $w =~ /^<h([0-9])(?![0-9a-zA-Z_])[^>]*>$/is ) {
                $h = $1;

                unshift( @{ $this->{'tate_align'} }, $this->align($w) );
                $this->kaigyo
                    if defined $this->{'txt_flag'} && $this->{'txt_flag'};

                $h = abs( $h - 7 );
                $this->{'fontsize'}
                    = $this->font_size( $h, $this->{'basefontsize'} );
                unshift( @{ $this->{'fsize'} }, $this->{'fontsize'} );

                $w = "<font size=\"$h\">";
                unshift( @{ $this->{'toptag'} }, $w );
                $this->{'bottomtags'} = '</font>' . $this->{'bottomtags'};
            }
            elsif ( $w =~ /^(<a(?![0-9a-zA-Z_])[^>]*)>$/is ) {
                unshift( @{ $this->{'toptag'} }, $w );
                $this->{'bottomtags'} = '</a>' . $this->{'bottomtags'};
            }
            elsif ( $w =~ /^<\/a(?![0-9a-zA-Z_])[^>]*>$/is ) {

                #リンクの解除
                $tmptags = join( "\t", @{ $this->{'toptag'} } );
                $tmptags =~ s/<a(?![0-9a-zA-Z_])[^>]*>//is;
                @{ $this->{'toptag'} } = split( /\t+/, $tmptags );

                $this->{'bottomtags'} =~ s!</a>!!i;
            }
            elsif ( $w =~ /^<img(?![0-9a-zA-Z_])[^>]*>$/is ) {
                $this->{'subalign'} = $this->align($w);

                # 画像の幅を抽出
                $this->{'colwidth'}
                    = $this->col_width( $w, $this->{'basefontsize'} );
                $this->{'cc'} += $this->{'colwidth'};
                $this->{'rowheight'}
                    = $this->row_height( $w, $this->{'basefontsize'} );
                $tmp_rc = $this->{'rc'} + $this->{'rowheight'};

                $w
                    =~ s/align[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20"]*[0-9a-zA-Z_]+[\x09\x0A\x0C\x0D\x20"]*//isg;
                $w .= '<br />';
            }
            elsif ( $w =~ /^<(p|div|center)(?![0-9a-zA-Z_])[^>]*>$/is ) {
                $this->kaigyo
                    if $w !~ /^<(div|center>)/i && $this->{'txt_flag'};

                # 位置指定
                unshift(
                    @{ $this->{'tate_align'} },
                    (   ( $w =~ /^<center(?![0-9a-zA-Z_])[^>]*>$/is )
                        ? 'middle'
                        : $this->align($w)
                    )
                );
                $w            = '';
                $oidashi_flag = 1
                    if defined $this->{'col_count'} && $this->{'col_count'};
            }
            elsif ( $w =~ /^<\/(p|div|span|center)(?![0-9a-zA-Z_])[^>]*>$/i )
            {
                # 位置指定の解除
                shift( @{ $this->{'tate_align'} } );
                $w            = '';
                $oidashi_flag = 1;
            }
            elsif ( $w =~ /^<hr(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # 線(行末に追加)
                ( $w, $this->{'subalign'} )
                    = $this->hr( $w, $this->{'basefontsize'} );
                $oidashi_flag = 1;
                $kaigyo_flag  = 1;
            }
            elsif ( $w =~ /^<table(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # テーブルの幅を抽出
                $this->{'colwidth'}
                    = $this->col_width( $w, $this->{'basefontsize'} );
                $this->{'cc'} += $this->{'colwidth'};

                $this->{'subalign'} = $this->align($w);
                $w
                    =~ s/align[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?[0-9a-zA-Z_]+"?[\x09\x0A\x0C\x0D\x20]*//isg;

                $this->{'table_flag'}++;
                $oidashi_flag = 1;
            }
            elsif ( $w =~ /^<blockquote(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # 引用
                $this->{'blockquote_flag'} = 1;
                $w                         = '';
                $oidashi_flag              = 1
                    if defined $this->{'txt_flag'} && $this->{'txt_flag'};
            }
            elsif ( $w =~ /^<\/blockquote(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # 引用解除
                $this->{'blockquote_flag'} = 0;
                $w                         = '';
                $oidashi_flag              = 1;
            }
            elsif ( $w =~ /^<(ul|menu|dir)(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # リスト表示
                $this->{'blockquote_flag'} = 0;
                $this->{'li_type'}         = $1
                    if $w
                    =~ /type[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?([0-9a-zA-Z_]+)("|[\x09\x0A\x0C\x0D\x20]|>)/is;
                $w = '';
            }
            elsif ( $w =~ /^<ol(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # リスト表示
                $this->{'blockquote_flag'} = 0;
                if ( $w
                    =~ / type[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20"]*([^\x09\x0A\x0C\x0D\x20"']+)("|[\x09\x0A\x0C\x0D\x20]|>)/is
                    )
                {
                    $this->{'li_type'} = $1;
                }
                else {
                    $this->{'li_type'} = '1';
                }
                $this->{'li_num'} = $1
                    if $w
                    =~ /start[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?([0-9]+)("|[\x09\x0A\x0C\x0D\x20]|>)/is;
                $w = '';
            }
            elsif ( $w =~ /^<\/(ol|ul|menu|dir)(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # リスト表示解除
                $this->{'blockquote_flag'} = 0;
                $this->{'li_type'}         = '';
                $this->{'li_num'}          = 0;
                $w                         = '';
                $oidashi_flag              = 1;
                $kaigyo_flag               = 1;
            }
            elsif ( $w =~ /^<li(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # リスト表示
                $this->{'blockquote_flag'} = 0;
                $this->{'li_type'}         = $1
                    if $w
                    =~ /type[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?([^\x09\x0A\x0C\x0D\x20"']+)("|[\x09\x0A\x0C\x0D\x20]|>)/is;
                $this->{'li_num'}++;
                $w            = '';
                $oidashi_flag = 1
                    if defined $this->{'col_count'}
                    && (
                    $this->{'col_count'}
                    || ( defined $this->{'string'}->[ $this->{'col_count'} ]
                        && $this->{'string'}->[ $this->{'col_count'} ] ne '' )
                    );
            }
            elsif ( $w =~ /^<\/(li|dt|dd)(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # 閉じタグ無視
                $w = '';
            }
            elsif ( $w =~ /^<dl(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # リスト表示
                $this->{'blockquote_flag'} = 0;
                $this->{'dl_type'}         = 'compact' if $w =~ /compact/i;
                $w                         = '';
            }
            elsif ( $w =~ /^<\/dl(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # リスト表示
                $this->{'blockquote_flag'} = 0;
                $this->{'dl_type'}         = '';
                $w                         = '';
                $oidashi_flag              = 1;
                $kaigyo_flag               = 1;
            }
            elsif ( $w =~ /^<dt(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # リスト表示
                $this->{'blockquote_flag'} = 0;
                $w                         = '';
                $oidashi_flag              = 1
                    if defined $this->{'col_count'}
                    && (
                    $this->{'col_count'}
                    || ( defined $this->{'string'}->[ $this->{'col_count'} ]
                        && $this->{'string'}->[ $this->{'col_count'} ] ne '' )
                    );
            }
            elsif ( $w =~ /^<dd(?![0-9a-zA-Z_])[^>]*>$/is ) {

                # リスト表示
                if ( $this->{'dl_type'} eq 'compact' ) {
                    $w      = '&nbsp;' . '<br />';
                    $tmp_rc = $this->{'rc'} + $this->{'fontsize'};
                }
                else {
                    $w            = '';
                    $oidashi_flag = 1;
                }
                $this->{'blockquote_flag'} = 1;
            }
            elsif ( $w
                =~ /^(<script(?![0-9a-zA-Z_])[^>]*>|<object(?![0-9a-zA-Z_])[^>]*>|<embed(?![0-9a-zA-Z_])[^>]*>|<style(?![0-9a-zA-Z_])[^>]*>|<yoko(?![0-9a-zA-Z_])[^>]*>|<y(?![0-9a-zA-Z_])[^>]*>)$/is
                )
            {
                #横書きタグのチェック
                $this->{'yoko_flag'}++;
                $w = '' if $w =~ /^(<yoko(?![0-9a-zA-Z_])[^>]*>|<y>)$/is;
            }
            elsif ( $w =~ s/^(<basefont(?![0-9a-zA-Z_]))([^>]*>)$/<font$2/is )
            {
                if ( $w
                    =~ /^(<font(?![0-9a-zA-Z_])[^>]*)size[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?([+\-]*[0-9])"?[\x09\x0A\x0C\x0D\x20]*([^>]*>)$/is
                    )
                {
                    $this->{'basefontsize'}
                        = ( $2 =~ /[+\-]/ )
                        ? eval $this->{'basefontsize'} . $2
                        : $2;
                    $this->{'basefontsize'} = 1
                        if $this->{'basefontsize'} < 1;
                    $this->{'basefontsize'} = 7
                        if $this->{'basefontsize'} > 7;
                    $w = $1 . ' size="' . $this->{'basefontsize'} . '"' . $3;
                    $this->{'fontsize'} = 1;
                }
                unshift( @{ $this->{'fsize'} },  $this->{'fontsize'} );
                unshift( @{ $this->{'toptag'} }, $w );
                $this->{'bottomtags'} = '</font>' . $this->{'bottomtags'};
            }
            elsif ( $w =~ /^<(\/body|\/html)(?![0-9a-zA-Z_])[^>]*>$/is ) {
                return '';
            }

     #テキストファイルの場合改行コードがあれば改行する
        }
        elsif (defined $this->{'type'}
            && $this->{'type'} eq 'text'
            && $w =~ /^[\x0D\x0A]+$/s )
        {
            $w           = '';
            $kaigyo_flag = 1;

            #エスケープの場合の処理
        }
        elsif ( $w =~ /^(\&#[0-9]{2,6};|&#x[a-fA-F0-9]+;|\&[a-zA-Z]+;)$/s ) {
            $w .= '<br />';
            $this->{'alt_text'} .= $w;

            #文字の場合の処理
        }
        elsif ( $w !~ /^[\x0A\x0D]+$/s ) {
            $this->{'alt_text'} .= $w;
            $tmp_rc = $this->{'rc'} + $this->{'fontsize'};
            if ($tmp_rc > $this->{'rows'}
                && (   $this->{'kinsokuLevel'} == 1
                    && $w =~ /[,﹐，\.﹒．、﹑､。｡〜ー～∼]/ )
                || (   $this->{'kinsokuLevel'} == 2
                    && $w
                    =~ /[,﹐，\.﹒．、﹑､。｡〜ー～∼\)\}\]〉》」』】〕〗〙〛ぁぃぅぇぉっゃゅょゎゕゖゝゞァィゥェォッャュョヮヵヶヽヾ]/
                )
                )
            {
                $kinsoku_flag = 1;
            }
            elsif (
                ( $tmp_rc >= ( $this->{'rows'} - $this->{'fontsize'} ) )
                && (   $this->{'kinsokuLevel'} == 2
                    && $w =~ /[\(\{\[〈《「『【〔〖〘〚]/ )
                )
            {
                $oidashi_flag = 1;
            }

            # 縦書き記号化+改行追加
            $w = $this->to_tategaki($w);
            $this->{'txt_flag'} = 1;
        }

        # 入れ子の横書きタグ
    }
    elsif ( $w
        =~ /^(<script(?![0-9a-zA-Z_])[^>]*>|<object(?![0-9a-zA-Z_])[^>]*>|<embed(?![0-9a-zA-Z_])[^>]*>|<style(?![0-9a-zA-Z_])[^>]*>|<yoko(?![0-9a-zA-Z_])[^>]*>|<y(?![0-9a-zA-Z_])[^>]*>)$/is
        )
    {
        $kinsoku_flag = 1;
        $this->{'yoko_flag'}++;
        $w = ''
            if $w
            =~ /^(<yoko(?![0-9a-zA-Z_])[^>]*>|<y(?![0-9a-zA-Z_])[^>]*>)$/is;

        #横書きタグの終了
    }
    elsif ( $w
        =~ /^(<\/script(?![0-9a-zA-Z_])[^>]*>|<\/object(?![0-9a-zA-Z_])[^>]*>|<\/embed(?![0-9a-zA-Z_])[^>]*>|<\/style(?![0-9a-zA-Z_])[^>]*>|<\/yoko(?![0-9a-zA-Z_])[^>]*>|<\/y(?![0-9a-zA-Z_])[^>]*>)$/is
        )
    {
        $kinsoku_flag = 1;
        $this->{'yoko_flag'}--;
        if ( $w
            =~ /^(<\/yoko(?![0-9a-zA-Z_])[^>]*>|<\/y(?![0-9a-zA-Z_])[^>]*>)$/is
            )
        {
            $w = '<br />';
            $this->{'rc'} += $this->{'fontsize'};
        }

        # 入れ子のテーブル
    }
    elsif ( $w =~ /^<table(?![0-9a-zA-Z_])[^>]*>$/is ) {
        $kinsoku_flag = 1;
        $this->{'table_flag'}++;

        #テーブルタグの終了
    }
    elsif ( $w =~ /^<\/table(?![0-9a-zA-Z_])[^>]*>$/is ) {
        $this->{'table_flag'}--;

        # テーブルタグが閉じられたら改行する
        if ( $this->{'table_flag'} == 0 ) {
            $this->{'string'}->[ $this->{'col_count'} ] .= $w . '<br />';
            $w            = '';
            $oidashi_flag = 1;
        }
        else {
            $kinsoku_flag = 1;
        }
    }
    else {
        $kinsoku_flag = 1;
    }

    # 字数が設定数を超えたら改行する
    $oidashi_flag = 1
        if $kinsoku_flag == 0
        && $tmp_rc > $this->{'rows'}
        && $this->{'string'}->[ $this->{'col_count'} ] ne '';
    $this->kaigyo if $oidashi_flag == 1;

    # 行数が設定数を超えたら改段する
    if (   $this->{'cols'}
        && $this->{'cc'} >= $this->{'cols'}
        && !$kinsoku_flag )
    {
        $this->print_text
            if ref $this->{'string'} && @{ $this->{'string'} } > 0;
        @{ $this->{'string'} } = ();
        $this->{'alt_text'}  = '';
        $this->{'col_count'} = 0;
        $this->{'cc'}        = $this->{'colwidth'};
        $this->{'colwidth'}  = 0;
    }

    if ( $oidashi_flag == 1 ) {
        if ( $this->{'blockquote_flag'} == 1 ) {
            $this->{'rc'} += 2 * $this->{'fontsize'};
            $this->{'string'}->[ $this->{'col_count'} ]
                = '&nbsp;' . '<br />' . '&nbsp;' . '<br />';
        }
        elsif ( $this->{'li_num'} > 0 ) {
            $this->{'rc'} += 2 * $this->{'fontsize'};
            if ( $this->{'li_type'} =~ /^disc$/i ) {
                $this->{'string'}->[ $this->{'col_count'} ]
                    = '&nbsp;<br />●<br />';
            }
            elsif ( $this->{'li_type'} =~ /^circle$/i ) {
                $this->{'string'}->[ $this->{'col_count'} ]
                    = '&nbsp;<br />○<br />';
            }
            elsif ( $this->{'li_type'} =~ /^square$/i ) {
                $this->{'string'}->[ $this->{'col_count'} ]
                    = '&nbsp;<br />□<br />';
            }
            elsif ( $this->{'li_type'} eq 'A' || $this->{'li_type'} eq 'ア' )
            {
                my @list = (
                    'ア', 'ア', 'イ', 'ウ', 'エ', 'オ', 'カ', 'キ',
                    'ク', 'ケ', 'コ', 'サ', 'シ', 'ス', 'セ', 'ソ',
                    'タ', 'チ', 'ツ', 'テ', 'ト', 'ナ', 'ニ', 'ヌ',
                    'ネ', 'ノ', 'ハ', 'ヒ', 'フ', 'ヘ', 'ホ', 'マ',
                    'ミ', 'ム', 'メ', 'モ', 'ヤ', 'ユ', 'ヨ', 'ラ',
                    'リ', 'ル', 'レ', 'ロ', 'ワ', 'ヰ', 'ヱ', 'ヲ',
                    'ン'
                );
                $this->{'string'}->[ $this->{'col_count'} ]
                    = '&nbsp;<br />('
                    . $list[ $this->{'li_num'} ]
                    . ')<br />';
            }
            elsif ( $this->{'li_type'} eq 'a' || $this->{'li_type'} eq 'あ' )
            {
                my @list = (
                    'あ', 'あ', 'い', 'う', 'え', 'お', 'か', 'き',
                    'く', 'け', 'こ', 'さ', 'し', 'す', 'せ', 'そ',
                    'た', 'ち', 'つ', 'て', 'と', 'な', 'に', 'ぬ',
                    'ね', 'の', 'は', 'ひ', 'ふ', 'へ', 'ほ', 'ま',
                    'み', 'む', 'め', 'も', 'や', 'ゆ', 'よ', 'ら',
                    'り', 'る', 'れ', 'ろ', 'わ', 'ゐ', 'ゑ', 'を',
                    'ん'
                );
                $this->{'string'}->[ $this->{'col_count'} ]
                    = '&nbsp;<br />('
                    . $list[ $this->{'li_num'} ]
                    . ')<br />';
            }
            elsif ( $this->{'li_type'} eq 'イ' ) {
                my @list = (
                    'イ', 'イ', 'ロ', 'ハ', 'ニ', 'ホ', 'ヘ', 'ト',
                    'チ', 'リ', 'ヌ', 'ル', 'ヲ', 'ワ', 'カ', 'ヨ',
                    'タ', 'レ', 'ソ', 'ツ', 'ネ', 'ナ', 'ラ', 'ム',
                    'ウ', 'ヰ', 'ノ', 'オ', 'ク', 'ヤ', 'マ', 'ケ',
                    'フ', 'コ', 'エ', 'テ', 'ア', 'サ', 'キ', 'ユ',
                    'メ', 'ミ', 'シ', 'ヱ', 'ヒ', 'モ', 'セ', 'ス'
                );
                $this->{'string'}->[ $this->{'col_count'} ]
                    = '&nbsp;<br />('
                    . $list[ $this->{'li_num'} ]
                    . ')<br />';
            }
            elsif ( $this->{'li_type'} eq 'い' ) {
                my @list = (
                    'い', 'い', 'ろ', 'は', 'に', 'ほ', 'へ', 'と',
                    'ち', 'り', 'ぬ', 'る', 'を', 'わ', 'か', 'よ',
                    'た', 'れ', 'そ', 'つ', 'ね', 'な', 'ら', 'む',
                    'う', 'ゐ', 'の', 'お', 'く', 'や', 'ま', 'け',
                    'ふ', 'こ', 'え', 'て', 'あ', 'さ', 'き', 'ゆ',
                    'め', 'み', 'し', 'ゑ', 'ひ', 'も', 'せ', 'す'
                );
                $this->{'string'}->[ $this->{'col_count'} ]
                    = '&nbsp;<br />('
                    . $list[ $this->{'li_num'} ]
                    . ')<br />';
            }
            elsif ( $this->{'li_type'} eq 'I' ) {
                my @list = (
                    'I',     'I',    'II',   'III', 'IV',  'V',
                    'VI',    'VII',  'VIII', 'IX',  'X',   'XI',
                    'XII',   'XIII', 'XIV',  'XV',  'XVI', 'XVII',
                    'XVIII', 'XIX',  'XX'
                );
                $this->{'string'}->[ $this->{'col_count'} ]
                    = '&nbsp;<br />('
                    . $list[ $this->{'li_num'} ]
                    . ')<br />';
            }
            elsif ( $this->{'li_type'} eq 'i' ) {
                my @list = (
                    'i',     'i',    'ii',   'iii', 'iv',  'v',
                    'vi',    'vii',  'viii', 'ix',  'x',   'xi',
                    'xii',   'xiii', 'xiv',  'xv',  'xvi', 'xvii',
                    'xviii', 'xix',  'xx'
                );
                $this->{'string'}->[ $this->{'col_count'} ]
                    = '&nbsp;<br />('
                    . $list[ $this->{'li_num'} ]
                    . ')<br />';
            }
            elsif ($this->{'li_type'} eq '1'
                || $this->{'li_type'} eq '一'
                || $this->{'li_type'} eq '壱' )
            {
                if ( $this->{'li_num'} <= 10 ) {
                    my @list = ( $this->{'li_type'} eq '壱' )
                        ? (
                        '壱', '壱', '弐', '参', '四', '伍',
                        '六', '七', '八', '九', '拾'
                        )
                        : (
                        '一', '一', '二', '三', '四', '五',
                        '六', '七', '八', '九', '十'
                        );
                    $this->{'string'}->[ $this->{'col_count'} ]
                        = '&nbsp;<br />('
                        . $list[ $this->{'li_num'} ]
                        . ')<br />';
                }
                else {
                    $this->{'string'}->[ $this->{'col_count'} ]
                        = '&nbsp;<br />(' . $this->{'li_num'} . ')<br />';
                }
            }
            else {
                $this->{'string'}->[ $this->{'col_count'} ]
                    = '&nbsp;<br />・<br />';
            }
            $this->{'blockquote_flag'} = 1;
        }
    }

    # 行末に一字ずつ追加
    $this->{'string'}->[ $this->{'col_count'} ] .= $w
        if defined $w && $w ne '';
    if ( defined $this->{'rowheight'} && $this->{'rowheight'} != 0 ) {
        $this->{'rc'} += $this->{'rowheight'};
        $this->{'rowheight'} = 0;
    }
    elsif ( $tmp_rc > 0 ) {
        $this->{'rc'} += $this->{'fontsize'};
    }

    $this->kaigyo if $kaigyo_flag == 1;

    $tmp_rc       = 0;
    $oidashi_flag = 0;
    $kinsoku_flag = 0;
    $kaigyo_flag  = 0;

    return '';
}

######################################################################
# 一段ずつ表示
sub print_text ($;$) {
    my ( $this, $row_end ) = @_;
    my ( $va, $txt );
    return
           if ref $this->{'string'}
        && @{ $this->{'string'} } == 1
        && $this->{'string'}->[0] !~ /[^\x0A\x0D\x09\x0A\x0C\x0D\x20]/;

    if ( $this->{'string'}->[ $#{ $this->{'string'} } ] !~ /\t/ ) {
        if ( $this->{'string'}->[ $#{ $this->{'string'} } ]
            =~ /^(<[^<>]+>)+$/ )
        {
            pop( @{ $this->{'string'} } );
            $this->{'toptags'} = join( '', reverse @{ $this->{'toptag'} } );
            if ( defined $this->{'subalign'} && $this->{'subalign'} ne '' ) {
                $this->{'valign'}   = $this->{'subalign'};
                $this->{'subalign'} = '';
            }
            else {
                $this->{'valign'}
                    = ( defined $this->{'tate_align'}->[0] )
                    ? $this->{'tate_align'}->[0]
                    : '';
            }
        }
        elsif ( $this->{'string'}->[ $#{ $this->{'string'} } ]
            =~ /[^\x09\x0A\x0C\x0D\x20]/ )
        {
            $this->kaigyo;
        }
        else {
            pop( @{ $this->{'string'} } );
        }
    }
    @{ $this->{'string'} } = reverse @{ $this->{'string'} };
    if ( defined $this->{'cols'} && $this->{'cols'} ne '' ) {
        my ( $max, $next_row, $k );
        $max      = $this->{'rows'} + 1;
        $next_row = $this->{'row_no'} + 2;
        if ( $this->{'row_no'} > 0 ) {
            $k
                = "<a href=\"#$this->{'name'}\-$this->{'row_no'}\">$this->{'upwardChar'}</a><br />";
            $max--;
        }
        if ( !$row_end ) {
            $k
                .= "<a href=\"#$this->{'name'}\-$next_row\">$this->{'downwardChar'}</a><br />";
            $max--;
        }
        if ( defined $k && $k ne '' ) {
            push(
                @{ $this->{'string'} },
                $k . ( '&nbsp;<br />' x $max ) . "\t" . 'top'
            );
            unshift( @{ $this->{'string'} }, $k . "&nbsp;\t" . 'bottom' );
        }
    }

    $this->{'row_no'}++;
    my $margin
        = ( $this->{'align'} )
        ? ( $this->{'align'} eq 'right' )
            ? ' style="margin-left: auto; margin-right: 0;"'
            : ( $this->{'align'} eq 'center' )
        ? ' style="margin-left: auto; margin-right: auto;"'
        : ''
        : '';
    if ( defined $this->{'handle_flag'} && $this->{'handle_flag'} ) {
        print OUT $this->{'separator'} if defined $this->{'separator'};
        print OUT '<a name="';
        print OUT $this->{'name'} . '-' . $this->{'row_no'};
        print OUT '" id="';
        print OUT $this->{'name'} . '-' . $this->{'row_no'};
        print OUT '"></a>';
        print OUT
            '<table class="tategaki" cellpadding="0" cellspacing="0" border="0" summary="'
            . $this->{'alt_text'} . '"'
            . ( ($margin) ? $margin : '' )
            . '><tbody><tr>';
    }
    else {
        ${ $this->{'output'} }
            .= (
            ( defined $this->{'separator'} ) ? $this->{'separator'} : '' )
            . '<a name="'
            . $this->{'name'} . '-'
            . $this->{'row_no'}
            . '" id="'
            . $this->{'name'} . '-'
            . $this->{'row_no'}
            . '"></a><table class="tategaki" cellpadding="0" cellspacing="0" border="0" summary="'
            . $this->{'alt_text'} . '"'
            . ( ($margin) ? $margin : '' )
            . '><tbody><tr>';
    }

    foreach $txt ( @{ $this->{'string'} } ) {
        ( $txt, $va ) = split( /\t/, $txt );
        $va = ' style="vertical-align:' . $va . ';"'
            if defined $va && $va ne '';
        $this->jcode_convert( \$txt, $this->{'jcode'}, 'utf8' )
            if $this->{'jcode'} && $this->{'jcode'} ne 'utf8';
        if ( defined $this->{'handle_flag'} && $this->{'handle_flag'} ) {
            print OUT '<td class="gyo"'
                . ( ( defined $va )  ? $va  : '' ) . '>'
                . ( ( defined $txt ) ? $txt : '' ) . '</td>';
        }
        else {
            ${ $this->{'output'} }
                .= '<td class="gyo"'
                . ( ( defined $va )  ? $va  : '' ) . '>'
                . ( ( defined $txt ) ? $txt : '' ) . '</td>';
        }
    }
    if ( defined $this->{'handle_flag'} && $this->{'handle_flag'} ) {
        print OUT '</tr></tbody></table>';
    }
    else {
        ${ $this->{'output'} } .= '</tr></tbody></table>';
    }
    return $this;
}

######################################################################
# 改行
sub kaigyo {
    my $this = shift;
    $this->{'string'}->[ $this->{'col_count'} ]
        = ( ( defined $this->{'toptags'} ) ? $this->{'toptags'} : '' )
        . ( ( defined $this->{'string'}->[ $this->{'col_count'} ] )
        ? $this->{'string'}->[ $this->{'col_count'} ]
        : '' )
        . ( ( !$this->{'marginEnabled'} )
        ? '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
        : '' )
        . ( ( defined $this->{'bottomtags'} ) ? $this->{'bottomtags'} : '' )
        . "\t"
        . ( ( defined $this->{'valign'} ) ? $this->{'valign'} : '' );
    $this->{'col_count'}++;
    $this->{'cc'} += $this->{'fontsize'};
    $this->{'rc'}       = 0;
    $this->{'txt_flag'} = 0;
    $this->{'alt_text'} .= ' ';
    $this->{'toptags'} = join( '', reverse @{ $this->{'toptag'} } );

    if ( defined $this->{'subalign'} && $this->{'subalign'} ne '' ) {
        $this->{'valign'}   = $this->{'subalign'};
        $this->{'subalign'} = '';
    }
    else {
        $this->{'valign'}
            = ( defined $this->{'tate_align'}->[0] )
            ? $this->{'tate_align'}->[0]
            : '';
    }
    return $this;
}

######################################################################
# 線
sub hr {
    my ( $align, $width, $size, $color, $u, $rw, $ns );
    my ( $this, $w, $b ) = @_;
    $align = $this->align($w);

    #$align = 'middle' if !defined $align || $align eq '';

    ( $width, $u ) = ( $1, $2 )
        if $w
        =~ /width[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?([0-9]+)([\%\x09\x0A\x0C\x0D\x20">])/i;
    $b = $this->s_to_w($b);
    $b += 3;
    $rw    = $this->{'rows'} * $b;
    $width = $rw * $width / 100 if $u eq '%' && $width ne '';
    $width = $rw if $width > $rw || $width <= 0;
    $width = ' size="' . $width . '"';
    $size  = $1
        if $w
        =~ /size[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?([0-9]+)[\x09\x0A\x0C\x0D\x20">]/i;
    $size  = 1 if $size <= 0;
    $size  = ' width="' . $size . '"';
    $color = $1
        if $w
        =~ /color[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?([0-9a-zA-Z_]+)[\x09\x0A\x0C\x0D\x20">]/i;
    $color = $this->{'textcolor'} if !defined $color || $color eq '';
    $color = ' color="' . $color . '"';
    $ns    = ' noshade="noshade"' if $w =~ /noshade/i;
    $w     = '<hr' . $width . $size . $color . $ns . ' />';
    return ( $w, $align );
}

######################################################################
# 位置
sub align {
    my $align;
    my ( $this, $w ) = @_;
    $align = $1
        if $w
        =~ /align[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?([0-9a-zA-Z_]+)[\x09\x0A\x0C\x0D\x20">]/i;
    $align = $1
        if $w
        =~ /text-align[\x09\x0A\x0C\x0D\x20]*:[\x09\x0A\x0C\x0D\x20]*([0-9a-zA-Z_]+)/i;
    if ( $align =~ /^left$/i ) {
        return 'top';
    }
    elsif ( $align =~ /^right$/i ) {
        return 'bottom';
    }
    elsif ( $align =~ /^center$/i ) {
        return 'middle';
    }
    return '';
}

######################################################################
# 行幅
sub col_width {
    my ( $this, $width, $b ) = @_;
    $width = $1
        if $width
        =~ /width[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?([0-9]+)(pt|"|[\x09\x0A\x0C\x0D\x20]|>)/i;
    return $this->{'cols'} if !$width;

    $b = $this->s_to_w($b);
    $b = ( $b * 2 ) + 4;

    return $width / $b;
}

######################################################################
# 列高
sub row_height {
    my ( $this, $height, $b ) = @_;
    $height = $1
        if $height
        =~ /height[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"?([0-9]+)(pt|"|[\x09\x0A\x0C\x0D\x20]|>)/i;
    return ( $this->{'rows'} ) ? $this->{'rows'} : 0 if !$height;
    return $height / ( $this->s_to_w($b) + 3 );
}

######################################################################
# サイズをポイントに変換
sub s_to_w {
    my ( $this, $s ) = @_;
    if ( !$s || $s <= 1 ) {
        return 9;
    }
    elsif ( $s == 2 ) {
        return 10;
    }
    elsif ( $s == 3 ) {
        return 12;
    }
    elsif ( $s == 4 ) {
        return 14;
    }
    elsif ( $s == 5 ) {
        return 18;
    }
    elsif ( $s == 6 ) {
        return 24;
    }
    elsif ( $s >= 7 ) {
        return 36;
    }
}

######################################################################
# フォントサイズ
sub font_size {
    my ( $this, $s, $b, $o ) = @_;
    if ( $o ne 'w' ) {
        $s = eval $b . $s if $s =~ /[+\-]/;
        $s = $this->s_to_w($s);
    }
    $b = $this->s_to_w($b);
    return $s / $b;
}

######################################################################
# 縦書き記号化
sub to_tategaki {
    my ( $this, $w ) = @_;

    # 英数字
    if ($w =~ /^[a-zA-Z0-9\+\%\*\$\&\#\@\?\!\･\/]$/

        # 数字
        || $w =~ /^[\,\.0-9]+$/

        # 変換
        || $w =~ s/(?:&lt;|＜|<)/∧/
        || $w =~ s/(?:&lt;|＞|>)/∨/
        || $w =~ s/↑/→/
        || $w =~ s/→/↓/
        || $w =~ s/↓/←/
        || $w =~ s/←/↑/
        || $w =~ s/[〜ー～∼]/｜/
        || $w =~ s/[=＝]/‖/
        || $w =~ s/…/︙/
        || $w =~ s/[:;：；]/‥/
        || $w =~ s/[\-−–]/︲/
        || $w =~ s/[_＿]/︳/
        || $w =~ s/[\(（]/︵/
        || $w =~ s/[\)）]/︶/
        || $w =~ s/[\{﹛｛]/︷/
        || $w =~ s/[\}﹜｝]/︸/
        || $w =~ s/[〔﹝]/︹/
        || $w =~ s/[〕﹞]/︺/
        || $w =~ s/【/︻/
        || $w =~ s/】/︼/
        || $w =~ s/[«《]/︽/
        || $w =~ s/[»》]/︾/
        || $w =~ s/[‹〈〈]/︿/
        || $w =~ s/[›〉〉]/﹀/
        || $w =~ s/[「｢]/﹁/
        || $w =~ s/[」｣]/﹂/
        || $w =~ s/『/﹃/
        || $w =~ s/』/﹄/
        || $w =~ s/[\[［]/﹇/
        || $w =~ s/[\]］]/﹈/
        )
    {
    }
    elsif ( $this->{'marginEnabled'} ) {
        if ( $w
            =~ s/、/<div style="margin: -0.7em -0.64em 0.7em 0.64em">、<\/div>/
            || $w
            =~ s/。/<div style="margin: -0.7em -0.64em 0.7em 0.64em">。<\/div>/
            || $w
            =~ s/([ぁぃぅぇぉっゃゅょゎゕゖ])/<div style="margin: -0.2em -0.15em 0.2em 0.15em">$1<\/div>/
            || $w
            =~ s/([ァィゥェォッャュョヮヵヶ])/<div style="margin: -0.2em -0.15em 0.2em 0.15em">$1<\/div>/
            )
        {
            return $w;
        }
    }
    elsif ($w =~ s/、/︑/
        || $w =~ s/。/︒/
        || $w
        =~ s/([ぁぃぅぇぉっゃゅょゎゕゖァィゥェォッャュョヮヵヶ])/&nbsp;$1/
        )
    {
    }
    return $w . '<br />';
}

######################################################################
# HTMLヘッダの抽出
sub get_header {
    my ( $this, $head ) = @_;

    #タイトル・ベースURLの抽出
    $this->{'page_title'} = $1
        if $head =~ s/<title(?![0-9a-zA-Z_])[^>]*>([^><]*)<\/title>//is;
    $this->{'basehref'} = $1
        if $head =~ s/<base[\x09\x0A\x0C\x0D\x20]+href="([^">]+)"[^>]*>//is;

    #その他のヘッダ情報の抽出
    $head =~ s/^.*<head>//is;
    $head =~ s!</head>.*$!!is;

    #漢字コードの指定を外す・余分な改行を削除
    $head
        =~ s/<meta[\x09\x0A\x0C\x0D\x20]*http-equiv[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"Content-Type"[\x09\x0A\x0C\x0D\x20]*content[\x09\x0A\x0C\x0D\x20]*=[\x09\x0A\x0C\x0D\x20]*"[^">]+"[^>]*>//igs;
    $head =~ s/[\x0A\x0D]+/\x0A/g;

    $this->{'html_header'} .= $head;
    return $this;
}

######################################################################
#HTMLフッタ
sub print_html_footer {
    my $this = shift;
    print OUT '</body></html>';
    binmode OUT;
}

######################################################################
#HTMLヘッダの出力
sub print_html_header {
    my $this  = shift;
    my $jcode = '';

    #ヘッダ漢字コード統一
    if ( $this->{'jcode'} ) {
        my $ocode = $this->{'jcode'};
        $this->jcode_convert( \$this->{'page_title'}, $ocode )
            if defined $this->{'page_title'} && $this->{'page_title'} ne '';
        $this->jcode_convert( \$this->{'html_header'}, $ocode )
            if defined $this->{'html_header'} && $this->{'html_header'} ne '';
    }
    if ( $this->{'jcode'} eq 'utf8' || $this->{'jcode'} eq '' ) {
        $jcode
            = '<meta http-equiv="Content-Type" CONTENT="text/html; charset=UTF-8" />';
        binmode OUT, ":utf8";
    }
    elsif ( $this->{'jcode'} eq 'sjis' ) {
        $jcode
            = '<meta http-equiv="Content-Type" CONTENT="text/html; charset=Shift_JIS" />';
        binmode OUT;
    }
    elsif ( $this->{'jcode'} eq 'jis' ) {
        $jcode
            = '<meta http-equiv="Content-Type" CONTENT="text/html; charset=ISO-2022-JP" />';
        binmode OUT;
    }
    elsif ( $this->{'jcode'} eq 'euc' ) {
        $jcode
            = '<meta http-equiv="Content-Type" content="text/html;charset=EUC-JP" />';
        binmode OUT;
    }

    print OUT "<html><head>\x0A<title>";
    print OUT $this->{'page_title'};
    print OUT "</title>\x0A";
    print OUT $jcode, "\x0A";

    print OUT $this->{'html_header'}, "\x0A"
        if defined $this->{'html_header'} && $this->{'html_header'} ne '';
    print OUT '<base href="', $this->{'basehref'}, "\" />\x0A"
        if defined $this->{'basehref'} && $this->{'basehref'} ne '';
    print OUT '<body ', $this->{'body'}, '>';
    print OUT '<br /><br />';
    return $this;
}

######################################################################
#	(2)-1 obj->text_to_tategaki(text,option);
sub text_to_tategaki ($$;$) {
    my ( $this, $text, $tag ) = @_;
    $text
        =~ s/<!--tate(?![0-9a-zA-Z_])([^>]*)-->(.*?)<!--\/tate-->/$this->_text_to_tategaki($1,$2,$tag)/egsi;
    return $text;
}

sub _text_to_tategaki ($$$;$) {
    my ( $this, $setting, $text, $tag ) = @_;
    my ( $obj, $undef );
    if ( defined $setting && $setting =~ /\=/ ) {
        $obj = TaketoriLegacy->new;
        $obj->set($setting);
        $undef = 1;
    }
    else {
        $obj = $this;
    }

    $obj->tategaki( \$text );
    $text = '<!--_tate-->' . $text . '<!--/_tate-->' if defined $tag && $tag;
    undef %$obj if defined $undef && $undef;
    return $text;
}

1;
