package ButtonLib;
require CMONOS::CGI_Lib::Tools::ImageLib;
@ISA = qw(ImageLib);
use strict;
use utf8;
######################################################################
# ボタンライブラリ編集モジュール
# ButtonLib.pm ver1.1.6
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-12-20
######################################################################
# 削除
sub erase ($) {
    my $this = shift;
    use File::Path;
    rmtree "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id";
    $this->erace_image_css;
}

sub erace_image_css ($) {
    my $this = shift;
    $this->replace_image_css( 'button-' . $main::dir . '-' . $main::id );
}

######################################################################
# 素材ライブラリCSSチェック
sub image_css_is_ready ($$$$) {
    my ( $this, $image_lib, $dir, $setting ) = @_;
    my $image_css = $image_lib . '/' . $dir . '/image.css';
    $this->get_sync_button;
    if ( !-f $image_css ) {
        my $css = $this->make_image_css( $image_lib, $dir, 1 );
        $this->write_file( $image_css, \$css );
    }
    return 1;
}

sub make_image_css ($$$;$$) {
    my ( $this, $image_dir, $dir, $is_image_lib, $setting ) = @_;
    my @dirs = $this->get_image_dir_list( $image_dir, $dir );
    my $css = '';
    my $id;
    foreach $id (@dirs) {
        my %image_info = ();
        my $class_name
            = 'button-'
            . ( ($is_image_lib) ? 'lib-' : '' )
            . $dir . '-'
            . $id;
        %image_info = $this->get_button_settings( $dir, $id, $is_image_lib );
        $css .= "\x0A\x0A"
            . $this->mk_image_css_for_button( $class_name, \%image_info,
            $dir, $id, $is_image_lib );
        $this->add_setting_hash( $setting, \%image_info, $dir, $id )
            if !$is_image_lib
            && ref $setting
            && defined $setting::deco_option{$dir}
            && $setting::deco_option{$dir} eq 'save_all'
            && %image_info;
    }
    return $css;
}

sub get_sync_button ($) {
    my $this = shift;
    return @{ $this->{'sync_button'} } if ref $this->{'sync_button'};
    my $cmonos_js = $this->read_file( $env::javascript . '/cmonos.js', 1 );
    if ( $cmonos_js =~ /var sync_button_classes = new Array\(([^\)]*)\);/si )
    {
        my $sync_buttons = $1;
        $sync_buttons =~ s/^'//;
        $sync_buttons =~ s/'$//;
        @{ $this->{'sync_button'} } = split( /','/, $sync_buttons );
    }
    else {
        @{ $this->{'sync_button'} } = ();
    }
    return @{ $this->{'sync_button'} };
}

sub push_sync_button ($$) {
    my ( $this, $class_name ) = @_;
    $this->get_sync_button;
    push( @{ $this->{'sync_button'} }, $class_name )
        if !grep( $_ eq $class_name, @{ $this->{'sync_button'} } );
}

sub pop_sync_button ($$) {
    my ( $this, $class_name ) = @_;
    @{ $this->{'sync_button'} }
        = grep( $_ ne $class_name, @{ $this->{'sync_button'} } )
        if ref $this->{'sync_button'};
}

sub replace_sync_button ($) {
    my $this = shift;
    return if !ref $this->{'sync_button'};
    @{ $this->{'sync_button'} }
        = sort { $a cmp $b } @{ $this->{'sync_button'} };
    my $sync_buttons
        = ( @{ $this->{'sync_button'} } )
        ? "'" . join( "','", @{ $this->{'sync_button'} } ) . "'"
        : '';
    my $cmonos_js = $this->read_file( $env::javascript . '/cmonos.js', 1 );
    if ( $cmonos_js
        =~ s/(var sync_button_classes = new Array\()[^\)]*(\);)/$1$sync_buttons$2/si
        )
    {
        $this->write_file( $env::javascript . '/cmonos.js', \$cmonos_js );
    }
}

sub mk_image_css_for_button ($$$$$;$) {
    my ( $this, $class_name, $setting, $dir, $id, $is_image_lib ) = @_;
    return '' if !$setting->{'link'};
    my $image_dir = (
        ($is_image_lib) ? '.' : $this->base_change(
            $this->{'deco_obj'}->{'image_dir'} . '/' . $dir,
            $this->{'image_css'}
        )
        )
        . '/'
        . $id;
    my $css = '/* ' . $class_name . ' start */' . "\x0A";
    $css
        .= 'a.'
        . $class_name . ', a.'
        . $class_name
        . '-selected, a.'
        . $class_name
        . '-active, a.'
        . $class_name
        . '-visited, ';
    $css
        .= 'input.'
        . $class_name
        . ', input.'
        . $class_name
        . '-selected, input.'
        . $class_name
        . '-active, input.'
        . $class_name
        . '-visited';
    $css .= ' {' . "\x0A";
    $css .= '  display: block;' . "\x0A"
        if grep( $_ eq $dir, ( 'bbs', 'db', 'page', 'equipment' ) );
    $css .= '  border: none !important;' . "\x0A";
    $css .= '  width: ' . $setting->{'linkwidth'} . 'px;' . "\x0A";
    $css .= '  height: ' . $setting->{'linkheight'} . 'px;' . "\x0A";
    $css
        .= '  background: transparent url("'
        . $image_dir . '/'
        . $setting->{'link'}
        . '") no-repeat center !important;' . "\x0A";
    $css .= '  vertical-align: middle;' . "\x0A";
    $css .= '  cursor: pointer !important;' . "\x0A";
    $css .= '  overflow: hidden;' . "\x0A";
    $css .= '  text-align: left;' . "\x0A";
    $css .= '  text-indent: -999px;' . "\x0A";
    $css .= '  font-size: 1px;' . "\x0A";
    $css .= '  line-height: 1px;' . "\x0A";
    $css .= '  /text-indent: 0;' . "\x0A";
    $css .= '  /font-size: 0;' . "\x0A";
    $css .= '  /line-height: 0;' . "\x0A";
    $css .= '  _text-indent: 0;' . "\x0A";
    $css .= '  _font-size: 0;' . "\x0A";
    $css .= '  _line-height: 0;' . "\x0A";
    $css .= '}' . "\x0A";

    if ( !grep( $_ eq $dir, ( 'bbs', 'db', 'page', 'equipment' ) ) ) {
        $css
            .= 'a.'
            . $class_name . ', a.'
            . $class_name
            . '-selected, a.'
            . $class_name
            . '-active, a.'
            . $class_name
            . '-visited ';
        $css .= ' {' . "\x0A";
        $css .= '  display: -moz-inline-box;' . "\x0A";
        $css .= '  display: inline-block;' . "\x0A";
        $css .= '  /display: inline;' . "\x0A";
        $css .= '  /zoom: 1;' . "\x0A";
        $css .= '  _display: inline;' . "\x0A";
        $css .= '  _zoom: 1;' . "\x0A";
        $css .= '}' . "\x0A";
    }
    $css
        .= 'input.'
        . $class_name
        . ', input.'
        . $class_name
        . '-selected, input.'
        . $class_name
        . '-active, input.'
        . $class_name
        . '-visited';
    $css .= ' {' . "\x0A";
    $css .= '  /padding-left: 999px;' . "\x0A";
    $css .= '  _padding-left: 999px;' . "\x0A";
    $css .= '}' . "\x0A";
    if ( $setting->{'visited'} ) {
        $css
            .= 'a.'
            . $class_name
            . ':visited, a.'
            . $class_name
            . '-visited, input.'
            . $class_name
            . '-visited {' . "\x0A";
        $css .= '  width: ' . $setting->{'visitedwidth'} . 'px;' . "\x0A";
        $css .= '  height: ' . $setting->{'visitedheight'} . 'px;' . "\x0A";
        $css
            .= '  background: transparent url("'
            . $image_dir . '/'
            . $setting->{'visited'}
            . '") no-repeat center !important;' . "\x0A";
        $css .= '}' . "\x0A";
    }
    if ( $setting->{'hover'} ) {
        $css
            .= 'a.'
            . $class_name
            . ':hover, a.'
            . $class_name
            . '-selected, a.'
            . $class_name
            . '-visited:hover, ';
        $css
            .= 'input.'
            . $class_name
            . ':hover, input.'
            . $class_name
            . '-selected, input.'
            . $class_name
            . '-visited:hover {' . "\x0A";
        $css .= '  width: ' . $setting->{'hoverwidth'} . 'px;' . "\x0A";
        $css .= '  height: ' . $setting->{'hoverheight'} . 'px;' . "\x0A";
        $css
            .= '  background: transparent url("'
            . $image_dir . '/'
            . $setting->{'hover'}
            . '") no-repeat center !important;' . "\x0A";
        $css .= '}' . "\x0A";
    }
    if ( $setting->{'active'} ) {
        $css
            .= 'a.'
            . $class_name
            . ':active, a.'
            . $class_name
            . '-active, a.'
            . $class_name
            . '-visited:active, a.'
            . $class_name
            . '-selected:active, ';
        $css
            .= 'input.'
            . $class_name
            . ':active, input.'
            . $class_name
            . '-active, input.'
            . $class_name
            . '-visited:active, input.'
            . $class_name
            . '-selected:active {' . "\x0A";
        $css .= '  width: ' . $setting->{'activewidth'} . 'px;' . "\x0A";
        $css .= '  height: ' . $setting->{'activeheight'} . 'px;' . "\x0A";
        $css
            .= '  background: transparent url("'
            . $image_dir . '/'
            . $setting->{'active'}
            . '") no-repeat center !important;' . "\x0A";
        $css .= '}' . "\x0A";
    }
    $css .= '/* ' . $class_name . ' end */' . "\x0A";
    return $css;
}

######################################################################
# メニュー
sub menu ($$;$$$) {
    my ( $this, $name, $lib_dir, $id, $list ) = @_;
    return $this->{'deco_obj'}
        ->button_menu( $name, $main::dir, 100, $id, 1, $list, $lib_dir,
        $main::id );
}

######################################################################
# 画像設定登録
sub set_image ($) {
    my $this       = shift;
    my $class_name = 'button-' . $main::dir . '-' . $main::id;
    my %image_info = $this->get_button_settings( $main::dir, $main::id );
    if ( $main::setting{'link'} ) {
        $this->push_sync_button($class_name);
    }
    else {
        $this->pop_sync_button($class_name);
    }
    $this->replace_sync_button;
    $this->replace_image_css(
        $class_name,
        $this->mk_image_css_for_button(
            $class_name, \%image_info, $main::dir, $main::id
        )
    );
    $this->set_deco( $main::dir, $main::id, \%image_info )
        if %image_info
        && defined $setting::deco_option{$main::dir}
        && $setting::deco_option{$main::dir} eq 'save_all';
}

sub get_button_settings ($$$;$) {
    my ( $this, $dir, $id, $is_image_lib ) = @_;
    my %data;
    $dir
        = ( ($is_image_lib)
        ? $this->{'deco_obj'}->{'image_lib'}
        : $this->{'deco_obj'}->{'image_dir'} )
        . '/'
        . $dir . '/'
        . $id;
    if ( -f "$dir/link.gif" ) {
        $data{'link'} = 'link.gif';
    }
    elsif ( -f "$dir/link.jpg" ) {
        $data{'link'} = 'link.jpg';
    }
    elsif ( -f "$dir/link.png" ) {
        $data{'link'} = 'link.png';
    }
    else {
        return;
    }
    ( $data{'linkwidth'}, $data{'linkheight'} )
        = $cgi::Image->info("$dir/$data{'link'}")
        if $data{'link'};
    if ( -f "$dir/visited.gif" ) {
        $data{'visited'} = 'visited.gif';
    }
    elsif ( -f "$dir/visited.jpg" ) {
        $data{'visited'} = 'visited.jpg';
    }
    elsif ( -f "$dir/visited.png" ) {
        $data{'visited'} = 'visited.png';
    }
    ( $data{'visitedwidth'}, $data{'visitedheight'} )
        = $cgi::Image->info("$dir/$data{'visited'}")
        if $data{'visited'};
    if ( -f "$dir/hover.gif" ) {
        $data{'hover'} = 'hover.gif';
    }
    elsif ( -f "$dir/hover.jpg" ) {
        $data{'hover'} = 'hover.jpg';
    }
    elsif ( -f "$dir/hover.png" ) {
        $data{'hover'} = 'hover.png';
    }
    ( $data{'hoverwidth'}, $data{'hoverheight'} )
        = $cgi::Image->info("$dir/$data{'hover'}")
        if $data{'hover'};
    if ( -f "$dir/active.gif" ) {
        $data{'active'} = 'active.gif';
    }
    elsif ( -f "$dir/active.jpg" ) {
        $data{'active'} = 'active.jpg';
    }
    elsif ( -f "$dir/active.png" ) {
        $data{'active'} = 'active.png';
    }
    ( $data{'activewidth'}, $data{'activeheight'} )
        = $cgi::Image->info("$dir/$data{'active'}")
        if $data{'active'};
    return %data;
}

######################################################################
# 設定フォーム
sub print_setting_form ($) {
    my $this = shift;
    print
        '<fieldset><legend><label for="setting:link">ボタンタイプ</label></legend>';
    print $this->mk_select_menu(
        'setting:link',
        '', '',
        (   (   (   $main::id
                        && grep( $_ eq 'button-'
                            . $main::dir . '-'
                            . $main::id,
                        @{ $this->{'sync_button'} } )
                        || ( !$main::id && $default::button_sync )
                )
            ) ? 1 : 0
        ),
        '0' => '独立型',
        '1' => '連動型'
    );
    print '<input type="submit" value="設定を保存" />'
        if defined $main::id && $main::id ne '';
    print '</fieldset>';
}

######################################################################
# アップロードフォーム
sub print_upload_form ($$) {
    my ( $this, $save_button ) = @_;
    print <<FORM;
<fieldset>
<legend>オリジナル画像</legend>
<table class="field-table">
<tbody>
<tr>
<td class="field-name"><label for="image:link">通常画像</label></td><td class="field"><input type="file" name="image:link" id="image:link" /></td>
</tr>
<tr>
<td class="field-name"><label for="image:visited">訪問済みリンク画像</label></td><td class="field"><input type="file" name="image:visited" id="image:visited" /></td>
</tr>
<tr>
<td class="field-name"><label for="image:hover">マウスオーバー画像</label></td><td class="field"><input type="file" name="image:hover" id="image:hover" /></td>
</tr>
<tr>
<td class="field-name"><label for="image:active">マウスダウン画像</label></td><td class="field"><input type="file" name="image:active" id="image:active" /></td>
</tr>
</tbody>
</table>
<div class="submit"><input type="submit" value="$save_button" /></div>
</fieldset>
FORM
}

######################################################################
# 画像の保存
sub save_images ($) {
    my $this = shift;
    return 0
        if !ref $cgi::input
        || $cgi::Image->is_not_web_image($cgi::input)
        || $this->is_invalid_images;

    #古い画像の削除
    $this->unlink_my_files;

    #保存
    my $image;
    foreach $image ( $this->get_my_files ) {
        $cgi::input->save_file(
            'image:' . $image,
            '!' . $image,
            [   'image/gif:gif',   'image/jpeg:jpg',
                'image/pjpeg:jpg', 'image/x-png:png',
                'image/png:png'
            ],
            1000000,
            "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id",
        );
    }
    push( @{ $cgi::action->{'dialog'} }, $cgi::input->dialog )
        if ref $cgi::action;
    return 1;
}

sub is_invalid_images ($) {
    my $this = shift;
    if ( !$cgi::input->get_posted_file_size_from_param('image:link') ) {
        push(
            @{ $cgi::action->{'dialog'} },
            '通常画像が指定されていません。'
        ) if ref $cgi::action;
        return 1;
    }
    else {
        return 0;
    }
}

sub unlink_my_files ($) {
    my $this = shift;
    $this->unlink_image_files( $this->get_my_files );
}

sub get_my_files ($) {
    my $this = shift;
    return ( 'link', 'visited', 'hover', 'active' );
}

sub unlink_image_files ($@) {
    my ( $this, @files ) = @_;
    foreach (@files) {
        unlink(
            "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id/$_.gif");
        unlink(
            "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id/$_.jpg");
        unlink(
            "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id/$_.png");
    }
}

######################################################################
# ライブラリからのインポート
sub import_images ($) {
    my $this  = shift;
    my $check = 1;
    if (( !defined $main::color || $main::color eq '' )
        && opendir( DIR,
            "$this->{'deco_obj'}->{'image_lib'}/$main::dir/$main::img"
        )
        )
    {
        if (!$this->set_dir(
                "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id")
            )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                '画像格納ディレクトリを開けませんでした。'
            ) if ref $cgi::action;
            return 0;
        }
        $this->unlink_my_files;
        my (@files);
        @files
            = grep(
            -f "$this->{'deco_obj'}->{'image_lib'}/$main::dir/$main::img/$_",
            readdir(DIR) );
        closedir(DIR);
        foreach (@files) {
            $check = 0
                unless $this->file_copy(
                "$this->{'deco_obj'}->{'image_lib'}/$main::dir/$main::img/$_",
                "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id/$_"
                );
        }
        chmod( $env::dir_mode,
            "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id" );
        return $check;
    }
    elsif (
        (      defined $main::color
            && $main::color eq 'slide'
            && $main::slide{'color'} =~ /([0-9a-fA-F]{6}|random)/i
        )
        || (   defined $main::color
            && $main::color eq 'change'
            && $main::change{'color'} =~ /([0-9a-fA-F]{6}|random)/i )
        )
    {
        if (!$this->set_dir(
                "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id")
            )
        {
            push(
                @{ $cgi::action->{'dialog'} },
                '画像格納ディレクトリを開けませんでした。'
            ) if ref $cgi::action;
            return 0;
        }
        $this->unlink_my_files;
        my (@setting);
        @setting = ( $main::color eq 'slide' )
            ? (
            $main::slide{'color'}, $main::slide{'abs'}, $main::slide{'rev'}
            )
            : split( /[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,
            $main::change{'color'} );
        $check = 0
            unless $cgi::Image->change_color_table_multi(
            "$this->{'deco_obj'}->{'image_lib'}/$main::dir/$main::img",
            "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id",
            $main::color,
            @setting
            );
        return $check;
    }
    push( @{ $cgi::action->{'dialog'} }, '設定に不正があります。' )
        if ref $cgi::action;
    return 0;
}

sub change_image_color ($) {
    my $this  = shift;
    my $check = 1;
    if (!$this->set_dir(
            "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id")
        )
    {
        push(
            @{ $cgi::action->{'dialog'} },
            '画像格納ディレクトリを開けませんでした。'
        ) if ref $cgi::action;
        return 0;
    }
    if ((      $main::color eq 'slide'
            && $main::slide{'color'} =~ /([0-9a-fA-F]{6}|random)/i
        )
        || (   $main::color eq 'change'
            && $main::change{'color'} =~ /([0-9a-fA-F]{6}|random)/i )
        )
    {
        my (@setting);
        @setting = ( $main::color eq 'slide' )
            ? (
            $main::slide{'color'}, $main::slide{'abs'}, $main::slide{'rev'}
            )
            : split( /[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,
            $main::change{'color'} );
        $check = 0
            unless $cgi::Image->change_color_table_multi(
            "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id",
            "$this->{'deco_obj'}->{'image_dir'}/$main::dir/$main::id",
            $main::color,
            @setting
            );
        return $check;
    }
    return 1;
}

######################################################################
# 新規作成ID
sub get_id ($$) {
    my ( $this, $dir ) = @_;
    if ( opendir( DIR, $this->{'deco_obj'}->{'image_dir'} . '/' . $dir ) ) {
        my $num = 0;
        while (1) {
            $num++;
            last
                if !-d $this->{'deco_obj'}->{'image_dir'} . '/'
                . $dir . '/'
                . $num;
        }
        return $num;
    }
    else {
        return 1;
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    #クラス登録
    $ImageLib::modules{'button'} = 'ButtonLib'
        if !defined $ImageLib::modules{'button'};
}

1;
