package ExternalFileEditor;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use Fcntl qw(:flock);
use strict;
use utf8;
######################################################################
# 外部ファイル設定モジュール
# ExternalFileEditor.pm ver1.0.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新：2008-08-21
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( $k, $v );

    #書き込みチェック
    if ( defined $main::a && $main::a eq 'w' ) {

        #ターゲットファイル名
        $main::target = $this->z_to_h($main::target)
            if defined $main::target;    #半角化
        if ( !defined $main::target || $main::target eq '' ) {
            push(
                @{ $this->{'dialog'} },
                'ファイル名を設定してください。'
            );
            $main::target = '';

        }
        elsif ( defined $main::target && $main::target !~ /^[0-9a-zA-Z_]+$/ )
        {
            push(
                @{ $this->{'dialog'} },
                'ファイル名を確認してください。半角英数字にしてください。'
            );
            $main::target = '';

        }
        else {
            $this->write_external_file($main::target);
        }

    }
    elsif ( defined $main::target && $main::target ne '' ) {

        #削除
        if ( defined $main::target && defined $main::a && $main::a eq 'del' )
        {
            unlink $env::externalfiles . '/' . $main::target . '.ssi';
            unlink $env::externalfiles . '/' . $main::target . '.js';
            push(
                @{ $this->{'dialog'} },
                $main::target . '.ssi を削除しました。'
            );
            $main::target = '';
        }

        #読み込み
        $this->read_external_file($main::target)
            if defined $main::target && $main::target ne '';
    }

    #外部ファイルメニュー
    if ( opendir( DIR, $env::externalfiles ) ) {
        my @menu
            = grep( $_ =~ /^[0-9a-zA-Z_]+\.ssi$/
                && -f $env::externalfiles . '/' . $_,
            readdir(DIR) );
        closedir(DIR);
        $setting::page{'tools'}{'subMenu'} = $this->form_start(
            'key'      => $setting::security_key,
            'name'     => 'fileselector',
            'required' => \@setting::form
        );
        $setting::page{'tools'}{'subMenu'}
            .= '<select name="target" onchange="submit()"><option value="">新規作成</option>';
        $setting::page{'tools'}{'subMenu'} .= '<option value=""></option>'
            if @menu;
        foreach ( sort @menu ) {
            my $name = $_;
            $name =~ s/\.ssi$//;
            $setting::page{'tools'}{'subMenu'}
                .= '<option value="'
                . $name . '"'
                . ( ( defined $main::target && $main::target eq $name )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $_
                . '</option>';
        }
        $setting::page{'tools'}{'subMenu'}
            .= '</select><input name="subJumpMenu" type="submit" value="選択" /></form>';
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    #タイトル
    print
        '<h1 title="外部ファイルを作成・更新します。">外部ファイル編集</h1>';

    unless ( -d $env::externalfiles ) {
        print
            '<p>スクリプトエラー：外部ファイル保存ディレクトリがみつかりません。</p>';
        $this->print_page_footer('tools');
        return;
    }

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="w" />';

    print
        '<fieldset><legend><label for="target">ターゲットファイル</label></legend>';
    print $this->path_change( $env::externalfiles, $ENV{'SCRIPT_NAME'},
        'abs' );
    print
        '/<input type="text" size="20" maxlength="20" name="target" id="target" value="';
    print( ( !defined $main::target || $main::target eq '' )
        ? ''
        : $this->escape($main::target) );
    print
        '" title="半角英数字のみで設定します。" />.ssi（半角英数字）';
    print
        '<div class="warning">既存ファイルと同名のファイル名にすると既存ファイルが上書きされます。</div>'
        if !defined $main::target || $main::target eq '';
    print '</fieldset>';

    print '<fieldset><legend><label for="html">内容</label></legend>';
    print
        '<textarea cols="70" rows="30" name="html" id="html" wrap="off" class="content-editor-area" title="外部ファイルの内容を記述します。" ondblclick="cmonos.tool.colorPalette.open(\'html+\',\'dom\')">',
        $this->escape($main::html), '</textarea>';
    print '<div class="toolbox">';
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'html\').setAutoFit(this.checked);" />フィット</label> ';
    print
        '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'html\').quotationByMenu(this)">';
    print '<option value="">▼挿入する</option>';
    print
        '<option value="&lt;address&gt;&lt;/address&gt;">&lt;address&gt;&lt;/address&gt;</option>';
    print
        '<option value="&lt;hr size=&quot;&quot; color=&quot;&quot; /&gt;">&lt;hr size=&quot;&quot; color=&quot;&quot; /&gt;</option>';
    print
        '<option value="&lt;a href=&quot;&quot;&gt;&lt;/a&gt;">&lt;a href=&quot;&quot;&gt;&lt;/a&gt;</option>';
    print
        '<option value="&lt;a href=&quot;mailto:?subject=&quot;&gt;&lt;/a&gt;">&lt;a href=&quot;mailto:&quot;&gt;&lt;/a&gt;</option>';
    print '<option value=""></option>';
    print '<option value="&amp;reg;">&reg;[記号]</option>';
    print '<option value="&amp;copy;">&copy;[記号]</option>';
    print '<option value=""></option>';
    print '<option value="&lt;a href=&quot;'
        . $this->path_change( '../../index.xml', $ENV{'SCRIPT_NAME'}, 'abs' )
        . '&quot; title=&quot;このサイトの新着情報&quot;&gt;&lt;img src=&quot;'
        . $this->path_change( '../../icon/feed-icon/feed-icon-28x28.png',
        $ENV{'SCRIPT_NAME'}, 'abs' )
        . '&quot; width=&quot;28&quot; height=&quot;28&quot; border=&quot;0&quot; alt=&quot;このサイトの新着情報&quot; /&gt;&lt;/a&gt;">このサイトの新着情報 RSS アイコン</option>'
        if -f '../../index.xml';
    print '<option value="&lt;a href=&quot;itpc://'
        . $env::server_name
        . ( ( $ENV{'SERVER_PORT'} != 80 ) ? ':' . $ENV{'SERVER_PORT'} : '' )
        . $this->path_change( '../../podcast.xml', $ENV{'SCRIPT_NAME'},
        'abs' )
        . '&quot; title=&quot;このサイトの新着ポッドキャスト情報&quot;&gt;&lt;img src=&quot;'
        . $this->path_change(
        '../../icon/feed-icon/feed-icon-28x28-purple.png',
        $ENV{'SCRIPT_NAME'}, 'abs' )
        . '&quot; width=&quot;28&quot; height=&quot;28&quot; border=&quot;0&quot; alt=&quot;このサイトの新着ポッドキャスト情報&quot; /&gt;&lt;/a&gt;">このサイトの新着ポッドキャスト情報 RSS アイコン</option>'
        if -f '../../podcast.xml';
    print '<option value="&lt;a href=&quot;'
        . $this->path_change( '../../comment.xml', $ENV{'SCRIPT_NAME'},
        'abs' )
        . '&quot; title=&quot;このサイトの新着コメント情報&quot;&gt;&lt;img src=&quot;'
        . $this->path_change( '../../icon/feed-icon/feed-icon-28x28-gray.png',
        $ENV{'SCRIPT_NAME'}, 'abs' )
        . '&quot; width=&quot;28&quot; height=&quot;28&quot; border=&quot;0&quot; alt=&quot;このサイトの新着コメント情報&quot; /&gt;&lt;/a&gt;">このサイトの新着コメント情報 RSS アイコン</option>'
        if -f '../../comment.xml';
    print '<option value=""></option>';
    print '<option value="[[REPLACE]]">[置換]</option>';
    print '<option value=""></option>';
    print '<option value="';
    print $this->jsquot(
              '<form action="'
            . $cgi::path
            . '" id="ysearch-form">\n'
            . '<input type="text" size="16" name="query" />\n'
            . '<input type="submit" value="検索" />\n'
            . '<script type="text/javascript" language="JavaScript" src="'
            . $this->path_change(
            $env::externalfiles . '/ysearch-tip.js', $ENV{'SCRIPT_NAME'},
            'abs'
            )
            . '"></script>\n'
            . '</form>\n'
    );
    print '">Yahoo!サイト検索フォーム</option>';

    if ( $main::target && $main::target eq 'metaindex' ) {
        print '<option value=""></option>';
        print
            '<option value="&lt;!--metaindex--&gt;&lt;!--/metaindex--&gt;">サイト横断最新記事インデックス</option>';
        print
            '<option value="&lt;!--article--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/article--&gt;">記事</option>';
        print
            '<option value="&lt;!--title--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/title--&gt;">記事タイトル</option>';
        print
            '<option value="&lt;!--icon--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/icon--&gt;">記事アイコン</option>';
        print
            '<option value="&lt;!--description--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/description--&gt;">記事要約</option>';
        print
            '<option value="&lt;!--author--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/author--&gt;">記事の筆者</option>';
        print
            '<option value="&lt;!--author email=&quot;yes&quot;--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/author--&gt;">記事の筆者(メールアドレス)</option>';
        print
            '<option value="&lt;!--subtitle--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/subtitle--&gt;">記事サブタイトル</option>';
        print
            '<option value="&lt;!--mtime type="c"--&gt;&lt;!--/mtime--&gt;">更新時</option>';
        print
            '<option value="&lt;!--date type="c"--&gt;&lt;!--/date--&gt;">日付</option>';
        print '<option value=""></option>';
        print '<option value="';
        print $this->jsquot(
            '<!--metaindex sort="mtimedescend" limit="10"--><!--head-->\n'
                . '<ul class="index">\n'
                . '<!--/head-->\n'
                . '<!--article--><!--head-->\n'
                . '<li>\n'
                . '<!--/head-->\n'
                . '<!--title--><!--head-->\n'
                . '<div class="title">\n'
                . '<!--/head--><!--foot-->\n'
                . '</div>\n'
                . '<!--/foot--><!--/title-->\n'
                . '<!--description--><!--head-->\n'
                . '<div class="description">\n'
                . '<!--/head--><!--foot-->\n'
                . '</div>\n'
                . '<!--/foot--><!--/description-->\n'
                . '<ul>\n'
                . '<!--subtitle--><!--head-->\n'
                . '<li>\n'
                . '<!--/head--><!--foot-->\n'
                . '</li>\n'
                . '<!--/foot--><!--/subtitle-->\n'
                . '</ul>\n'
                . '<!--foot-->\n'
                . '</li>\n'
                . '<!--/foot-->\n'
                . '<!--/article--><!--foot-->\n'
                . '</ul>\n'
                . '<!--/foot-->\n'
                . '<!--/metaindex-->\n' );
        print '">メタインデックス例</option>';
    }
    elsif ( $main::target && $main::target eq 'recentcomments' ) {
        print '<option value=""></option>';
        print
            '<option value="&lt;!--recentcomments limit="10"--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/recentcomments--&gt;">最新のコメント &lt;!--recentcomments--&gt;&lt;!--/recentcomments--&gt;</option>';
        print
            '<option value="&lt;!--article--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/article--&gt;">記事 &lt;!--recentcomments--&gt;&lt;!--/recentcomments--&gt;</option>';
        print
            '<option value="&lt;!--name--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/name--&gt;">名前 &lt;!--name--&gt;&lt;!--/name--&gt;</option>';
        print
            '<option value="&lt;!--comment--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/comment--&gt;">コメント &lt;!--comment--&gt;&lt;!--/comment--&gt;</option>';
        print
            '<option value="&lt;!--title--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/title--&gt;">タイトル &lt;!--title--&gt;&lt;!--/title--&gt;</option>';
        print
            '<option value="&lt;!--email--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/email--&gt;">メールアドレスリンク &lt;!--email--&gt;&lt;!--/email--&gt;</option>';
        print
            '<option value="&lt;!--url--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/url--&gt;">URLリンク &lt;!--url--&gt;&lt;!--/url--&gt;</option>';
        print
            '<option value="&lt;!--icon--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/icon--&gt;">アイコン &lt;!--icon--&gt;&lt;!--/icon--&gt;</option>';
        print
            '<option value="&lt;!--ip--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/ip--&gt;">投稿者 IP &lt;!--ip--&gt;&lt;!--/ip--&gt;</option>';
        print
            '<option value="&lt;!--agent--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/agent--&gt;">投稿者環境情報 &lt;!--agent--&gt;&lt;!--/agent--&gt;</option>';
        print
            '<option value="&lt;!--ctime--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/ctime--&gt;">投稿日時 &lt;!--ctime--&gt;&lt;!--/ctime--&gt;</option>';
        print '<option value=""></option>';
        print '<option value="'
            . $this->jsquot( '<!--recentcomments limit="10"-->\n'
                . '<!--head-->\n'
                . '<ul class="index">\n'
                . '<!--/head-->\n'
                . '<!--article--><!--head-->\n'
                . '<li>\n'
                . '<!--/head-->\n'
                . '<!--title maxlength="32"--><!--/title-->\n'
                . '<!--foot-->\n'
                . '</li>\n'
                . '<!--/foot-->\n'
                . '<!--/article--><!--foot-->\n'
                . '</ul>\n'
                . '<!--/foot-->\n'
                . '<!--/recentcomments-->\n' )
            . '">最新コメント一覧テンプレート例</option>';
    }
    elsif ( $main::target && $main::target eq 'recenttrackbacks' ) {
        print '<option value=""></option>';
        print
            '<option value="&lt;!--recenttrackbacks limit="10"--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/recenttrackbacks--&gt;">最新のトラックバック &lt;!--recenttrackbacks--&gt;&lt;!--/recenttrackbacks--&gt;</option>';
        print
            '<option value="&lt;!--article--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/article--&gt;">記事 &lt;!--article--&gt;&lt;!--/article--&gt;</option>';
        print
            '<option value="&lt;!--blog_name--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/blog_name--&gt;">サイト名 &lt;!--blog_name--&gt;&lt;!--/blog_name--&gt;</option>';
        print
            '<option value="&lt;!--title--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/title--&gt;">タイトル &lt;!--title--&gt;&lt;!--/title--&gt;</option>';
        print
            '<option value="&lt;!--excerpt--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/excerpt--&gt;">要約 &lt;!--excerpt--&gt;&lt;!--/excerpt--&gt;</option>';
        print
            '<option value="&lt;!--url--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/url--&gt;">URL &lt;!--url--&gt;&lt;!--/url--&gt;</option>';
        print
            '<option value="&lt;!--ip--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/ip--&gt;">トラックバック元 IP &lt;!--ip--&gt;&lt;!--/ip--&gt;</option>';
        print
            '<option value="&lt;!--ctime--&gt;&lt;!--head--&gt;&lt;!--/head--&gt;&lt;!--foot--&gt;&lt;!--/foot--&gt;&lt;!--/ctime--&gt;">トラックバック日時 &lt;!--ctime--&gt;&lt;!--/ctime--&gt;</option>';
        print '<option value=""></option>';
        print '<option value="'
            . $this->jsquot( '<!--recenttrackbacks limit="10"-->\n'
                . '<!--head-->\n'
                . '<ul class="index">\n'
                . '<!--/head-->\n'
                . '<!--article--><!--head-->\n'
                . '<li>\n'
                . '<!--/head-->\n'
                . '<!--title maxlength="32"--><!--/title-->\n'
                . '<!--foot-->\n'
                . '</li>\n'
                . '<!--/foot-->\n'
                . '<!--/article--><!--foot-->\n'
                . '</ul>\n'
                . '<!--/foot-->\n'
                . '<!--/recenttrackbacks-->\n' )
            . '">最新トラックバック一覧テンプレート例</option>';
    }
    print '</select> ';
    print
        '<a href="JavaScript:cmonos.tool.colorPalette.open(\'html+\')" ondblclick="cmonos.tool.colorPalette.open(\'html+\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>';
    print
        ' <label><input type="checkbox" name="address_protector" value="1"> メールアドレスを JavaScript + CSS + HTML エンティティで保護する</label></div>';
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset></form>';

    if ( defined $main::target && $main::target ne '' ) {

        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'delete',
            'method'   => 'post',
            'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
            'required' => [ @setting::form, 'target' ]
        );
        print
            '<fieldset class="delete"><input type="submit" name="a=del" value="削除" onclick="return confirm(\'この操作は取り消せません。本当に削除しますか？\')" /></fieldset></form>';

        #取り込みタグサンプル
        my $target_uri = $this->http_my_domain
            . $this->path_change(
            $env::externalfiles . '/' . $main::target . '.js',
            $ENV{'SCRIPT_NAME'}, 'abs' );
        print '<form action="#">';
        print
            '<fieldset class="sample"><legend>埋め込みタグ(JavaScript)</legend>';
        print
            '<textarea cols="70" rows="3">&lt;script type=&quot;text/javascript&quot; language=&quot;JavaScript&quot; src=&quot;'
            . $target_uri
            . '&quot; class=&quot;sim-ssi&quot;&gt;&lt;/script&gt;</textarea>';
        print '</fieldset>';
        if ( defined $env::ssi && $env::ssi ) {
            print
                '<fieldset class="sample"><legend>埋め込みタグ(SSI)</legend>';
            print
                '<textarea cols="70" rows="3">&lt;!--#include virtual=&quot;'
                . $this->path_change(
                $env::externalfiles . '/' . $main::target . '.ssi',
                $ENV{'SCRIPT_NAME'}, 'abs' )
                . '&quot;--&gt;</textarea>';
            print '</fieldset>';
        }
        print '</form>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
#スタイルシート書き込み読み込み
sub write_external_file ($$) {
    my ( $this, $target ) = @_;
    my $path = $env::externalfiles . '/' . $main::target . '.ssi';
    $this->autolink( \$main::html, 0, $main::address_protector );
    local (*OUT);
    if ( open( OUT, ">>$path" ) ) {
        binmode FILE, ":utf8";
        flock( OUT, LOCK_EX );
        truncate( OUT, 0 );
        seek( OUT, 0, 0 );
        print OUT $main::html;
        close(OUT);
        push(
            @{ $this->{'dialog'} },
            $main::target . '.ssi を更新しました。'
        );
        $this->mk_external_javascript( $path, '', $main::html );
    }
    else {
        push(
            @{ $this->{'dialog'} },
            $main::target
                . '.ssi の更新に失敗しました。ファイルを開けません。'
        );
    }
}

sub read_external_file ($$) {
    my ( $this, $target ) = @_;
    my $path = $env::externalfiles . '/' . $main::target . '.ssi';

    #読み込み
    local (*FILE);
    if ( open( FILE, $path ) ) {
        binmode FILE;
        flock( FILE, LOCK_EX );
        $main::html = join( '', <FILE> );
        close(FILE);
    }
    else {
        push(
            @{ $this->{'dialog'} },
            $main::target . '.ssi が見つかりませんでした。'
        );
        return;
    }
    $main::html = $this->jcode_convert( $main::html, 'utf8' );
    $main::html =~ s/\x0D\x0A/\x0A/sg;
    $main::html =~ s/\x0D/\x0A/sg;
}

######################################################################
# モジュール読み込み
#sub import {
BEGIN {
    no warnings qw(once);
    unless ( defined $default::subdivided
        && defined $main::c
        && $default::subdivided
        && $main::c ne '' )
    {

        #クラス登録
        Common::class_register( '0120', 'ExternalFileEditor' );

        #ツール登録
        $setting::jump_menu{'0120'} = '外部ファイル編集(共通)';
        $setting::master_check_option{'0120'} = 1;
    }
}

1;
