package Page;
require CMONOS::CGI_Lib::Tools::Preference;
@ISA = qw(Preference);
use strict;
use utf8;
######################################################################
# ページレイアウトモジュール
# Page.pm ver1.1.6
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-02-05
######################################################################
# 設定チェック項目
sub get_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    $this->get_sub_setting_items;

    #チェックボックス
    push( @check::checkbox,
        'defaultcss',           'bgproperties',         'externalHeader',
        'insideExternalHeader', 'insideHeader',         'insideTitleImage',
        'externalFooter',       'insideExternalFooter', 'insideFooter' );
    push( @check::checkbox, 'commonHtmlHeaderDisabled' )
        if !$preference::static_page;
    push( @check::checkbox, 'naviEnabled', 'externalNavi' )
        if defined $env::inline_navi && $env::inline_navi;
    push( @check::checkbox, 'googleAnalytics' )
        if defined $env::google_analytics_uacct
        && $env::google_analytics_uacct;

    #半角英数字チェック
    $check::word{'bodyClass'} = '&lt;body&gt; class';

    #カラー書式チェック
    $check::color{'bgcolor'} = '背景色';
    $check::color{'text'}    = 'テキスト色';
    $check::color{'link'}    = 'リンク色';
    $check::color{'alink'}   = 'アクティブリンク色';
    $check::color{'vlink'}   = '訪問済みリンク色';
    $check::color{'hover'}   = 'オーバーリンク色';

    #数字チェック
    #$check::num{'topmargin'} = '上マージン';
    #$check::num{'leftmargin'} = '左マージン';
    #$check::num{'bottommargin'} = '下マージン';
    #$check::num{'rightmargin'} = '右マージン';
    #$check::num{'marginwidth'} = 'マージン幅';
    #$check::num{'marginheight'} = 'マージンの高さ';
    $check::num{'width'}          = '本文幅';
    $check::num{'titleTextWidth'} = '本文タイトル幅';
    $check::num{'headTextWidth'}  = '序文幅';

    #容量チェック
    $check::maxlength{'title'}     = 'タイトル';
    $check::maxlength{'header'}    = 'ページヘッダ';
    $check::maxlength{'titleText'} = '本文タイトル';
    $check::maxlength{'headText'}  = '序文';
    $check::maxlength{'footer'}    = 'ページフッタ';
    $check::maxlength{'template'}  = 'テンプレート';

    #タグ修正
    push( @check::tagfix, 'titleText', 'headText' );

}

######################################################################
# 設定項目取得
sub get_sub_setting_items ($) {
    my $this = shift;
    return;
}

######################################################################
# 設定環境
sub set_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    $preference::db         = 'page';    # 記録用データベース名
    $preference::submenukey = 'page';    # サブメニューキー
    $this->set_sub_preferences;

    push( @preference::import_key, $preference::id )
        if defined $preference::id && $preference::id ne '';
    if ( defined $main::import && $main::import ne '' ) {
        push( @preference::import_key, $main::import );
        push(
            @{ $this->{'dialog'} },
            '設定をインポートしました。この設定でよければ、設定ページの一番下にある 保存 ボタンを押して下さい。'
        );
    }

    unless ( ref $cgi::Deco ) {
        require CMONOS::CGI_Lib::Deco;
        $cgi::Deco = Deco->new;
    }
    $this->set_onload(
        "cmonos.element.tagClass('fieldset','foldable','tbody','foldable').disappear(true);"
    ) if !defined $main::unfold || !$main::unfold;
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    my $this = shift;
    $preference::id   = '';                            # データキー
    $preference::name = 'ページレイアウト';    # 設定パネル名
    $preference::title = 'ページレイアウトを設定します。';
}

######################################################################
# 設定パネル
sub print_preference_panel ($) {
    my $this = shift;
    my ( $id, $key, $bgp_checked, $defaultcsscheckbox, $unfold_checked );

    $id = ( defined $preference::id ) ? $preference::id : '';
    $id .= '_' if $id ne '';

    #設定のインポート
    if ( defined $main::import && $main::import ne '' && %preference::page ) {
        $key = $main::import . '_';
        $this->escape(
            \$main::data{ $id . 'title' },
            \$main::data{ $id . 'titleText' }
        );
    }
    else {
        $key = $id;
    }

    #デフォルト
    $unfold_checked
        = ( defined $main::unfold && $main::unfold )
        ? ' checked="checked"'
        : '';
    $bgp_checked
        = ( defined $main::data{ $key . 'bgproperties' }
            && $main::data{ $key . 'bgproperties' } eq 'fixed' )
        ? ' checked="checked"'
        : '';

    #エスケープ/初期化
    $this->escape(
        \$main::data{ $key . 'title' },
        \$main::data{ $key . 'header' },
        \$main::data{ $key . 'bodyClass' },
        \$main::data{ $key . 'titleText' },
        \$main::data{ $key . 'headText' },
        \$main::data{ $key . 'footer' },
        \$main::data{ $key . 'bgcolor' },
        \$main::data{ $key . 'text' },
        \$main::data{ $key . 'link' },
        \$main::data{ $key . 'alink' },
        \$main::data{ $key . 'vlink' },
        \$main::data{ $key . 'hover' },
        \$main::data{ $key . 'topmargin' },
        \$main::data{ $key . 'leftmargin' },
        \$main::data{ $key . 'bottommargin' },
        \$main::data{ $key . 'rightmargin' },
        \$main::data{ $key . 'marginwidth' },
        \$main::data{ $key . 'marginheight' },
        \$main::data{ $key . 'CSS' },
        \$main::data{ $key . 'htmlHeader' },
        \$main::data{ $key . 'width' },
        \$main::data{ $key . 'titleTextWidth' },
        \$main::data{ $key . 'headTextWidth' },
        \$main::data{ $key . 'margintop' },
        \$main::data{ $key . 'marginbottom' }
    );
    if ( -f $env::css . '/default.css' ) {
        $defaultcsscheckbox
            = '<input type="checkbox" name="data:'
            . $id
            . 'defaultcss" id="data:'
            . $id
            . 'defaultcss" value="true"'
            . (
            (   defined $main::data{ $key . 'defaultcss' }
                    && $main::data{ $key . 'defaultcss' } eq 'true'
            ) ? ' checked="checked"' : ''
            ) . ' />';
    }
    else {
        $defaultcsscheckbox
            = '<div class="note">ファイルがありません</div>';
    }

    print <<PANEL;
<input type="hidden" name="id" value="$preference::id" />
<fieldset class="toggle">
<label title="「CSSファイル編集(共通)」「外部ファイル編集(共通)」で一括して設定することをお勧めします。"><input type="checkbox" name="unfold" value="1" onclick="cmonos.element.tagClass('fieldset','foldable','tbody','foldable').disappear(!this.checked);"$unfold_checked /> 詳しい設定を表示する (非推奨)</label>
</fieldset>
<fieldset>
<legend>ヘッダ設定</legend>
<fieldset>
<legend><label for="data:${id}title">タイトル</label></legend>
<input type="text" name="data:${id}title" id="data:${id}title" size="64" maxlength="100" value="$main::data{$id . 'title'}" />
</fieldset>
<fieldset>
<legend><label for="data:${id}bodyClass">&lt;body&gt; class</label></legend>
<input type="text" name="data:${id}bodyClass" id="data:${id}bodyClass" size="30" maxlength="100" value="$main::data{$key . 'bodyClass'}" />
</fieldset>
PANEL

    if ( !$preference::static_page ) {
        print <<PANEL;
<fieldset>
<legend>共通 HTML ヘッダ</legend>
PANEL

        print '<label><input type="checkbox" name="data:'
            . $id
            . 'commonHtmlHeaderDisabled" value="1"'
            . ( ( $main::data{ $key . 'commonHtmlHeaderDisabled' } )
            ? ' checked="checked"'
            : '' )
            . ' /> 共通 HTML ヘッダ（下記）を使用しない</label>';
        print '<pre><code class="html-source-code">'
            . ( ($env::html_header)
            ? $this->escape($env::html_header)
            : '※共通 HTML ヘッダがありません。' )
            . '</code></pre>';

        print <<PANEL;
<div class="note">「共通 HTML ヘッダ」はセットアップアシスタントの「管理情報変更」で変更できます。</div>
</fieldset>
PANEL
    }

    print <<PANEL;
<fieldset>
<legend>外部 CSS 設定</legend>
<table class="field-table">
<tbody>
<tr>
<td class="field-name">$defaultcsscheckbox</td><td class="q-field"><label for="data:${id}defaultcss">デフォルトCSSを利用する</label></td>
PANEL
    undef $defaultcsscheckbox;

    #スタイルシートメニュー
    local (*CSS);
    if ( opendir( CSS, $env::css ) ) {
        my @menu
            = grep( -f $env::css . '/' . $_
                && $_ =~ /\.css$/
                && $_ ne 'default.css'
                && $_ ne 'common.css'
                && $_ ne 'tools.css'
                && $_ ne 'image.css'
                && $_ !~ /-/,
            readdir(CSS) );
        closedir(CSS);
        if (@menu) {
            my ($file_name);
            print '<td class="field-name"><label for="data:' . $id
                . 'cssFileName">CSS外部ファイル1</label></td>';
            print '<td class="q-field">';
            print '<select name="data:'
                . $id
                . 'cssFileName" id="data:'
                . $id
                . 'cssFileName"><option value=""></option>';
            foreach $file_name ( sort @menu ) {
                print '<option value="'
                    . $file_name . '"'
                    . (
                    (   defined $main::data{ $key . 'cssFileName' }
                            && $main::data{ $key . 'cssFileName' } eq
                            $file_name
                    ) ? ' selected="selected"' : ''
                    )
                    . '>'
                    . $file_name
                    . '</option>';
            }
            print '</select>';
            print '</td>';
            print '<td class="field-name"><label for="data:' . $id
                . 'cssFileName2">CSS外部ファイル2</label></td>';
            print '<td class="q-field">';
            print '<select name="data:'
                . $id
                . 'cssFileName2" id="data:'
                . $id
                . 'cssFileName2"><option value=""></option>';
            foreach $file_name ( sort @menu ) {
                print '<option value="'
                    . $file_name . '"'
                    . (
                    (   defined $main::data{ $key . 'cssFileName2' }
                            && $main::data{ $key . 'cssFileName2' } eq
                            $file_name
                    ) ? ' selected="selected"' : ''
                    )
                    . '>'
                    . $file_name
                    . '</option>';
            }
            print '</select>';
            print '</td>';
        }
    }

    print <<PANEL;
</tr>
</tbody>
</table>
</fieldset>
<fieldset class="foldable">
<legend><label for="data:${id}htmlHeader">HTMLヘッダ</label></legend>
<textarea name="data:${id}htmlHeader" id="data:${id}htmlHeader" cols="72" rows="6" wrap="off" class="content-editor-area">$main::data{$key . 'htmlHeader'}</textarea>
PANEL

    print '<div class="toolbox">';
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'data:'
        . $id
        . 'htmlHeader\').setAutoFit(this.checked);" />フィット</label> ';
    print
        '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:'
        . $id
        . 'htmlHeader\').quotationByMenu(this)">';
    print '<option value="">▼挿入する</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;meta name=&quot;&quot; content=&quot;&quot; /&gt;')
        . '">&lt;meta name=&quot;&quot; content=&quot;&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;meta name=&quot;author&quot; content=&quot;&quot; /&gt;')
        . '">&lt;meta name=&quot;author&quot; content=&quot;著作者&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;meta name=&quot;description&quot; content=&quot;&quot; /&gt;')
        . '">&lt;meta name=&quot;description&quot; content=&quot;要約&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;meta name=&quot;keywords&quot; content=&quot;&quot; /&gt;')
        . '">&lt;meta name=&quot;keywords&quot; content=&quot;キーワード&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;meta name=&quot;robots&quot; content=&quot;noindex,nofollow&quot; /&gt;'
        )
        . '">&lt;meta name=&quot;robots&quot; content=&quot;登録禁止・リンク追跡禁止&quot; /&gt;</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;&quot; href=&quot;&quot; type=&quot;&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;&quot; href=&quot;&quot; type=&quot;&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;stylesheet&quot; type=&quot;text/css&quot; media=&quot;screen, print, tty&quot; href=&quot;&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;stylesheet&quot; href=&quot;&quot; type=&quot;text/css&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;prev&quot; href=&quot;&quot; /&gt;')
        . '">&lt;link rel=&quot;prev&quot; href=&quot;前ページのURI&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;next&quot; href=&quot;&quot; /&gt;')
        . '">&lt;link rel=&quot;next&quot; href=&quot;次ページのURI&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;contents&quot; href=&quot;&quot; title=&quot;目次&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;contents&quot; href=&quot;目次のURI&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;index&quot; href=&quot;&quot; title=&quot;索引&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;index&quot; href=&quot;索引のURI&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;copyright&quot; href=&quot;&quot; title=&quot;&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;copyright&quot; href=&quot;著作権文書のURI&quot; /&gt;</option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;link rel=&quot;appendix&quot; href=&quot;&quot; title=&quot;&quot; /&gt;'
        )
        . '">&lt;link rel=&quot;appendix&quot; href=&quot;付録文書のURI&quot; /&gt;</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot(
        '\n&lt;script type=&quot;text/javascript&quot; language=&quot;JavaScript&quot; src=&quot;&quot; charset=&quot;UTF-8&quot;&gt;&lt;/script&gt;'
        )
        . '">&lt;script type=&quot;text/javascript&quot; src=&quot;&quot;&gt;&lt;/script&gt;</option>';
    print '<option value=""></option>';
    print '<option value="[[REPLACE]]">[置換]</option>';
    print <<PANEL;
</select></div></fieldset>
<fieldset class="foldable">
<legend>背景画像</legend>
PANEL

    print $cgi::Deco->background_menu( 'data:' . $id . 'background',
        'background', 120, $main::data{ $key . 'background' } );

    print <<PANEL;
<div class="extra-field"><label for="data:${id}bgproperties"><input type="checkbox" name="data:${id}bgproperties" id="data:${id}bgproperties"  value="fixed"$bgp_checked /> 背景画像固定</label></div>
</fieldset>
<fieldset class="foldable">
<legend><label for="data:${id}bgcolor">背景色</label></legend>
<input type="text" name="data:${id}bgcolor" id="data:${id}bgcolor" value="$main::data{$key . 'bgcolor'}" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open('data:${id}bgcolor','dom')" onchange="cmonos.validate.field(this).asColor()" style="background-color: $main::data{$key . 'bgcolor'}" />
 <a href="JavaScript:cmonos.tool.colorPalette.open('data:${id}bgcolor')" ondblclick="cmonos.tool.colorPalette.open('data:${id}bgcolor','window')"><img src="$env::image_dir/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>
 <a href="JavaScript:cmonos.tool.colorSlider.open('data:${id}bgcolor')" ondblclick="cmonos.tool.colorSlider.open('data:${id}bgcolor','window')"><img src="$env::image_dir/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>
</fieldset>
<fieldset class="foldable">
<legend>テキスト設定</legend>
<table class="field-table">
<tbody>
<tr>
<td class="field-name"><label for="data:${id}text">テキスト色</label></td>
<td class="q-field"><input type="text" name="data:${id}text" id="data:${id}text" value="$main::data{$key . 'text'}" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open('data:${id}text','dom')" onchange="cmonos.validate.field(this).asColor()" style="background-color: $main::data{$key . 'text'}" />
 <a href="JavaScript:cmonos.tool.colorPalette.open('data:${id}text')" ondblclick="cmonos.tool.colorPalette.open('data:${id}text','window')"><img src="$env::image_dir/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>
 <a href="JavaScript:cmonos.tool.colorSlider.open('data:${id}text')" ondblclick="cmonos.tool.colorSlider.open('data:${id}text','window')"><img src="$env::image_dir/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>
</td><td class="field-name"><label for="data:${id}lineheight">行間隔</label></td>
<td class="q-field">
PANEL

    print $this->mk_select_menu(
        'data:' . $id . 'lineheight', '', '',
        $main::data{ $key . 'lineheight' },
        ''      => '',
        '1em'   => '1文字',
        '1.2em' => '1.2文字',
        '1.5em' => '1.5文字',
        '1.8em' => '1.8文字',
        '2em'   => '2文字',
        '2.5em' => '2.5文字'
    );

    print <<PANEL;
</td>
</tr>
<tr>
<td class="field-name"><label for="data:${id}link">リンク色</label></td>
<td class="q-field"><input type="text" name="data:${id}link" id="data:${id}link" value="$main::data{$key . 'link'}" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open('data:${id}link','dom')" onchange="cmonos.validate.field(this).asColor()" style="background-color: $main::data{$key . 'link'}" />
 <a href="JavaScript:cmonos.tool.colorPalette.open('data:${id}link')" ondblclick="cmonos.tool.colorPalette.open('data:${id}link','window')"><img src="$env::image_dir/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>
 <a href="JavaScript:cmonos.tool.colorSlider.open('data:${id}link')" ondblclick="cmonos.tool.colorSlider.open('data:${id}link','window')"><img src="$env::image_dir/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>
</td><td class="field-name"><label for="data:${id}linkdecoration">リンク装飾</label></td>
<td class="q-field">
PANEL

    print $this->mk_select_menu(
        'data:' . $id . 'linkdecoration', '', '',
        $main::data{ $key . 'linkdecoration' },
        ''                   => '',
        'none'               => '無',
        'underline'          => '下線',
        'overline'           => '上線',
        'overline underline' => '上線+下線'
    );

    print <<PANEL;
</td>
</tr>
<tr>
<td class="field-name"><label for="data:${id}alink">アクティブリンク色</label></td>
<td class="q-field"><input type="text" name="data:${id}alink" id="data:${id}alink" value="$main::data{$key . 'alink'}" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open('data:${id}alink','dom')" onchange="cmonos.validate.field(this).asColor()" style="background-color: $main::data{$key . 'alink'}" />
 <a href="JavaScript:cmonos.tool.colorPalette.open('data:${id}alink')" ondblclick="cmonos.tool.colorPalette.open('data:${id}alink','window')"><img src="$env::image_dir/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>
 <a href="JavaScript:cmonos.tool.colorSlider.open('data:${id}alink')" ondblclick="cmonos.tool.colorSlider.open('data:${id}alink','window')"><img src="$env::image_dir/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>
</td><td class="field-name"><label for="data:${id}alinkdecoration">アクティブリンク装飾</label></td>
<td class="q-field">
PANEL

    print $this->mk_select_menu(
        'data:' . $id . 'alinkdecoration', '', '',
        $main::data{ $key . 'alinkdecoration' },
        ''                   => '',
        'none'               => '無',
        'underline'          => '下線',
        'overline'           => '上線',
        'overline underline' => '上線+下線'
    );

    print <<PANEL;
</td>
</tr>
<tr>
<td class="field-name"><label for="data:${id}vlink">訪問済みリンク色</label></td>
<td class="q-field"><input type="text" name="data:${id}vlink" id="data:${id}vlink" value="$main::data{$key . 'vlink'}" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open('data:${id}vlink','dom')" onchange="cmonos.validate.field(this).asColor()" style="background-color: $main::data{$key . 'vlink'}" />
 <a href="JavaScript:cmonos.tool.colorPalette.open('data:${id}vlink')" ondblclick="cmonos.tool.colorPalette.open('data:${id}vlink','window')"><img src="$env::image_dir/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>
 <a href="JavaScript:cmonos.tool.colorSlider.open('data:${id}vlink')" ondblclick="cmonos.tool.colorSlider.open('data:${id}vlink','window')"><img src="$env::image_dir/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>
</td><td class="field-name"><label for="data:${id}vlinkdecoration">訪問済みリンク装飾</label></td>
<td class="q-field">
PANEL

    print $this->mk_select_menu(
        'data:' . $id . 'vlinkdecoration', '', '',
        $main::data{ $key . 'vlinkdecoration' },
        ''                   => '',
        'none'               => '無',
        'underline'          => '下線',
        'overline'           => '上線',
        'overline underline' => '上線+下線'
    );

    print <<PANEL;
</td>
</tr>
<tr>
<td class="field-name"><label for="data:${id}hover">オーバーリンク色</label></td>
<td class="q-field"><input type="text" name="data:${id}hover" id="data:${id}hover" value="$main::data{$key . 'hover'}" size="7" maxlength="7" onclick="cmonos.tool.colorPalette.open('data:${id}hover','dom')" onchange="cmonos.validate.field(this).asColor()" style="background-color: $main::data{$key . 'hover'}" />
 <a href="JavaScript:cmonos.tool.colorPalette.open('data:${id}hover')" ondblclick="cmonos.tool.colorPalette.open('data:${id}hover','window')"><img src="$env::image_dir/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a>
 <a href="JavaScript:cmonos.tool.colorSlider.open('data:${id}hover')" ondblclick="cmonos.tool.colorSlider.open('data:${id}hover','window')"><img src="$env::image_dir/tool/picker.gif" alt="カラーピッカー" height="12" width="18" border="0" /></a>
</td><td class="field-name"><label for="data:${id}hoverdecoration">オーバーリンク装飾</label></td>
<td class="q-field">
PANEL

    print $this->mk_select_menu(
        'data:' . $id . 'hoverdecoration', '', '',
        $main::data{ $key . 'hoverdecoration' },
        ''                   => '',
        'none'               => '無',
        'underline'          => '下線',
        'overline'           => '上線',
        'overline underline' => '上線+下線'
    );

    print <<PANEL;
</td>
</tr>
</tbody>
</table>
</fieldset>
<fieldset class="foldable">
<legend>マージン設定</legend>
<table class="field-table">
<tbody>
<tr>
<td class="field-name"><label for="data:${id}topmargin">上マージン</label></td>
<td class="q-field"><input type="text" name="data:${id}topmargin" id="data:${id}topmargin" value="$main::data{$key . 'topmargin'}" size="7" maxlength="7" />
</td><td class="field-name"><label for="data:${id}bottommargin">下マージン</label></td>
<td class="q-field"><input type="text" name="data:${id}bottommargin" id="data:${id}bottommargin" value="$main::data{$key . 'bottommargin'}" size="7" maxlength="7" />
</td></tr>
<tr>
<td class="field-name"><label for="data:${id}leftmargin">左マージン</label></td>
<td class="q-field"><input type="text" name="data:${id}leftmargin" id="data:${id}leftmargin" value="$main::data{$key . 'leftmargin'}" size="7" maxlength="7" />
</td><td class="field-name"><label for="data:${id}rightmargin">右マージン</label></td>
<td class="q-field"><input type="text" name="data:${id}rightmargin" id="data:${id}rightmargin" value="$main::data{$key . 'rightmargin'}" size="7" maxlength="7" />
</td></tr>
<tr>
<td class="field-name"><label for="data:${id}marginwidth">マージン幅</label></td>
<td class="q-field"><input type="text" name="data:${id}marginwidth" id="data:${id}marginwidth" value="$main::data{$key . 'marginwidth'}" size="7" maxlength="7" />
</td><td class="field-name"><label for="data:${id}marginheight">マージンの高さ</label></td>
<td class="q-field"><input type="text" name="data:${id}marginheight" id="data:${id}marginheight" value="$main::data{$key . 'marginheight'}" size="7" maxlength="7" />
</td>
</tr>
</tbody>
</table>
</fieldset>
<fieldset class="foldable">
<legend><label for="data:${id}CSS">CSS 設定</label></legend>
<textarea name="data:${id}CSS" id="data:${id}CSS" cols="72" rows="6" wrap="off" class="content-editor-area" ondblclick="cmonos.tool.colorPalette.open(\'data:${id}CSS+\',\'dom\')">$main::data{$key . 'CSS'}</textarea>
PANEL
    require CMONOS::CGI_Lib::StyleSheetHelper;
    StyleSheetHelper->new->print_menu( 'data:' . $id . 'CSS' );
    print '</fieldset>';

    #GoogleAnalytics
    if ( defined $env::google_analytics_uacct
        && $env::google_analytics_uacct )
    {
        my $googleAnalytics_checked = (
            $main::data{ $key . 'googleAnalytics' }
                && (
                (      $env::google_analytics_embedded
                    && $main::data{ $key . 'googleAnalytics' } eq 'disabled'
                )
                || (  !$env::google_analytics_embedded
                    && $main::data{ $key . 'googleAnalytics' } eq 'enabled' )
                )
        ) ? ' checked="checked"' : '';
        my $googleAnalytics_action
            = ($env::google_analytics_embedded)
            ? 'トラッキングコードを埋め込まない（デフォルトで埋め込みます）'
            : 'トラッキングコードを埋め込む';
        my $googleAnalytics_value
            = ($env::google_analytics_embedded) ? 'disabled' : 'enabled';
        print <<PANEL;
<fieldset>
<legend>Google Analytics</legend>
<label><input type="checkbox" name="data:${id}googleAnalytics" id="data:${id}googleAnalytics" value="$googleAnalytics_value"$googleAnalytics_checked /> $googleAnalytics_action</label> 
</fieldset>
PANEL
    }

    #インラインサイトナビゲーション
    if ( defined $env::inline_navi && $env::inline_navi ) {
        my $naviEnabled_checked
            = ( $main::data{ $key . 'naviEnabled' } )
            ? ' checked="checked"'
            : '';
        my $externalNavi_checked
            = ( $main::data{ $key . 'externalNavi' } )
            ? ' checked="checked"'
            : '';
        my $include_type
            = ( $env::ssi && $env::ssi_ext && $preference::ssi )
            ? 'SSI'
            : 'JavaScript';
        print <<PANEL;
<fieldset>
<legend>ナビゲーションヘッダ</legend>
<label><input type="checkbox" name="data:${id}naviEnabled" id="data:${id}naviEnabled" value="1"$naviEnabled_checked /> 使用する</label> 
<label><input type="checkbox" name="data:${id}externalNavi" value="1"$externalNavi_checked /> $include_type を利用して埋め込む</label>
</fieldset>
PANEL
    }

   #ページヘッダファイル/ページヘッダファイル メニュー
    local (*DIR);
    my ( $header_file_name_menu, $footer_file_name_menu );
    if ( opendir( DIR, $env::externalfiles ) ) {
        my @menu
            = grep( $_ =~ /^[0-9a-zA-Z_]+\.ssi$/
                && -f $env::externalfiles . '/' . $_,
            readdir(DIR) );
        closedir(DIR);
        if (@menu) {
            my ($file_name);
            $header_file_name_menu
                = '<select name="data:'
                . $id
                . 'headerFileName" id="data:'
                . $id
                . 'headerFileName"><option value=""></option>';
            $footer_file_name_menu
                = '<select name="data:'
                . $id
                . 'footerFileName" id="data:'
                . $id
                . 'footerFileName"><option value=""></option>';
            foreach $file_name ( sort @menu ) {
                $header_file_name_menu
                    .= '<option value="'
                    . $file_name . '"'
                    . (
                    (   defined $main::data{ $key . 'headerFileName' }
                            && $main::data{ $key . 'headerFileName' } eq
                            $file_name
                    ) ? ' selected="selected"' : ''
                    )
                    . '>'
                    . $file_name
                    . '</option>';
                $footer_file_name_menu
                    .= '<option value="'
                    . $file_name . '"'
                    . (
                    (   defined $main::data{ $key . 'footerFileName' }
                            && $main::data{ $key . 'footerFileName' } eq
                            $file_name
                    ) ? ' selected="selected"' : ''
                    )
                    . '>'
                    . $file_name
                    . '</option>';
            }
            $header_file_name_menu .= '</select>';
            $footer_file_name_menu .= '</select>';
        }
        else {
            $header_file_name_menu = $footer_file_name_menu
                = '<div class="note">ファイルがありません</div>';
        }
    }
    else {
        $header_file_name_menu = $footer_file_name_menu
            = '<div class="note">ファイルがありません</div>';
    }
    $header_file_name_menu
        .= ' <input type="checkbox" name="data:'
        . $id
        . 'externalHeader" value="1"'
        . ( ( $main::data{ $key . 'externalHeader' } )
        ? ' checked="checked"'
        : '' )
        . ' /> ';
    $header_file_name_menu
        .= ( ( $env::ssi && $env::ssi_ext && $preference::ssi )
        ? 'SSI'
        : 'JavaScript' )
        . ' を利用して埋め込む';
    $footer_file_name_menu
        .= ' <input type="checkbox" name="data:'
        . $id
        . 'externalFooter" value="1"'
        . ( ( $main::data{ $key . 'externalFooter' } )
        ? ' checked="checked"'
        : '' )
        . ' /> ';
    $footer_file_name_menu
        .= ( ( $env::ssi && $env::ssi_ext && $preference::ssi )
        ? 'SSI'
        : 'JavaScript' )
        . 'を利用して埋め込む';

    print <<PANEL;
<fieldset>
<legend><label for="data:${id}headerFileName">ページヘッダ外部ファイル</label></legend>
PANEL
    print $header_file_name_menu;
    undef $header_file_name_menu;
    print ' <label><input type="checkbox" name="data:'
        . $id
        . 'insideExternalHeader" value="1"'
        . ( ( $main::data{ $key . 'insideExternalHeader' } )
        ? ' checked="checked"'
        : '' )
        . ' /> コンテナ（コンテンツフレーム）の中に表示する</label>';
    print <<PANEL;
</fieldset>
<fieldset>
<legend><label for="data:${id}header">ページヘッダ</label></legend>
<textarea name="data:${id}header" id="data:${id}header" cols="72" rows="8" wrap="off" class="content-editor-area">$main::data{$key . 'header'}</textarea>
<div class="toolbox">
PANEL

    $this->print_header_tool( $id, 'header' );
    print ' <label><input type="checkbox" name="data:'
        . $id
        . 'insideHeader" value="1"'
        . (
        ( $main::data{ $key . 'insideHeader' } ) ? ' checked="checked"' : '' )
        . ' /> コンテナ（コンテンツフレーム）の中に表示する</label>';

    print <<PANEL;
</div>
</fieldset>
<fieldset class="foldable">
<legend>タイトル画像 <span class="note">(タイトル画像に代替テキストが設定されていない場合「タイトル」または「本文タイトル」が代替テキストになります)</span></legend>
PANEL

    print $cgi::Deco->image_menu( 'data:' . $id . 'titleImage',
        'title', 120, $main::data{ $key . 'titleImage' } );

    print '<div class="additional-field"><label for="data:' . $id
        . 'titleImageAlign">タイトル画像位置</label>';
    print $this->mk_select_menu(
        'data:' . $id . 'titleImageAlign', '', '',
        $main::data{ $key . 'titleImageAlign' },
        ''       => '',
        'left'   => '左',
        'center' => '中央',
        'right'  => '右'
    );
    print '</div>';
    print '<label><input type="checkbox" name="data:'
        . $id
        . 'insideTitleImage" value="1"'
        . ( ( $main::data{ $key . 'insideTitleImage' } )
        ? ' checked="checked"'
        : '' )
        . ' /> コンテナ（コンテンツフレーム）の中に表示する</label>';

    print <<PANEL;
</fieldset>
</fieldset>

<fieldset class="foldable">
<legend>コンテナ表示設定</legend>
<fieldset>
<legend>コンテナ配置設定</legend>
<table class="field-table">
<tbody>
<tr>
<td class="field-name"><label for="data:${id}margintop">上マージン</label></td>
<td class="q-field"><input type="text" name="data:${id}margintop" id="data:${id}margintop" value="$main::data{$key . 'margintop'}" size="7" maxlength="7" /></td>
<td class="field-name"><label for="data:${id}marginbottom">下マージン</label></td>
<td class="q-field"><input type="text" name="data:${id}marginbottom" id="data:${id}marginbottom" value="$main::data{$key . 'marginbottom'}" size="7" maxlength="7" /></td>
</tr>
<tr>
<td class="field-name"><label for="data:${id}align">コンテナの位置</label></td>
<td class="q-field">
PANEL

    print $this->mk_select_menu(
        'data:' . $id . 'align', '', '', $main::data{ $key . 'align' },
        ''       => '',
        'left'   => '左',
        'center' => '中央',
        'right'  => '右'
    );

    print <<PANEL;
</td>
<td class="field-name"><label for="data:${id}width">コンテナの幅</label></td>
<td class="q-field"><input type="text" name="data:${id}width" id="data:${id}width" value="$main::data{$key . 'width'}" size="7" maxlength="7" /> <span class="unit">ピクセル</span>
</td>
</tr>
</tbody>
</table>
</fieldset>
<fieldset>
<legend>コンテンツを囲うフレーム（コンテナ/table#container）</legend>
PANEL

    print $cgi::Deco->frame_menu( 'data:' . $id . 'deco',
        'frame', $main::data{ $key . 'deco' } );

    print <<PANEL;
</fieldset>
</fieldset>

<fieldset class="foldable">
<legend>本文表示設定</legend>
<label for="data:${id}textAlign" class="datatitle">本文行揃え</label>
PANEL

    print $this->mk_select_menu(
        'data:' . $id . 'textAlign', '', '',
        $main::data{ $key . 'textAlign' },
        ''       => '',
        'left'   => '左寄せ',
        'center' => '中央揃え',
        'right'  => '右寄せ'
    );

    print <<PANEL;
<fieldset>
<legend>本文タイトル</legend>
<table class="field-table">
<tbody>
<tr>
<td class="field-name"><label for="data:${id}titleText">本文タイトル</label></td>
<td class="field" colspan="3"><input type="text" name="data:${id}titleText" id="data:${id}titleText" value="$main::data{$id . 'titleText'}" size="64" />
</td>
</tr>
<tr>
<td class="field-name"><label for="data:${id}titleTextAlign">本文タイトル位置</label></td>
<td class="q-field">
PANEL

    print $this->mk_select_menu(
        'data:' . $id . 'titleTextAlign', '', '',
        $main::data{ $key . 'titleTextAlign' },
        ''       => '',
        'left'   => '左',
        'center' => '中央',
        'right'  => '右'
    );

    print <<PANEL;
</td>
<td class="field-name"><label for="data:${id}titleTextWidth">本文タイトル幅</label></td>
<td class="q-field"><input type="text" name="data:${id}titleTextWidth" id="data:${id}titleTextWidth" value="$main::data{$key . 'titleTextWidth'}" size="7" maxlength="7" /> <span class="unit">ピクセル</span>
</td>
</tr>
</tbody>
</table>
</fieldset>
<fieldset>
<legend>本文タイトルを囲うフレーム</legend>
PANEL

    print $cgi::Deco->frame_menu( 'data:' . $id . 'titleTextDeco',
        'frame', $main::data{ $key . 'titleTextDeco' } );

    print <<PANEL;
</fieldset>
</fieldset>

<fieldset class="foldable">
<legend>序文設定</legend>
<fieldset>
<legend><label for="data:${id}headText">序文</label></legend>
<textarea name="data:${id}headText" id="data:${id}headText" cols="72" rows="8" wrap="off" class="content-editor-area">$main::data{$key . 'headText'}</textarea>
<div class="toolbox">
PANEL

    $this->print_header_tool( $id, 'headText' );

    print <<PANEL;
</div>
</fieldset>
<fieldset>
<legend>序文を囲うフレーム</legend>
PANEL

    print $cgi::Deco->frame_menu( 'data:' . $id . 'headTextDeco',
        'frame', $main::data{ $key . 'headTextDeco' } );

    print <<PANEL;
<div class="additional-field"><label for="data:${id}headTextWidth">序文幅</label><input type="text" name="data:${id}headTextWidth" id="data:${id}headTextWidth" value="$main::data{$key . 'headTextWidth'}" size="7" maxlength="7" /> <span class="unit">ピクセル</span></div>
</fieldset>
</fieldset>

<fieldset>
<legend>フッタ設定</legend>
<fieldset>
<legend><label for="data:${id}footer">ページフッタ</label></legend>
<textarea name="data:${id}footer" id="data:${id}footer" cols="72" rows="8" wrap="off" class="content-editor-area">$main::data{$key . 'footer'}</textarea>
<div class="toolbox">
PANEL

    $this->print_header_tool( $id, 'footer' );
    print ' <label><input type="checkbox" name="data:'
        . $id
        . 'insideFooter" value="1"'
        . (
        ( $main::data{ $key . 'insideFooter' } ) ? ' checked="checked"' : '' )
        . ' /> コンテナ（コンテンツフレーム）の中に表示する</label>';

    print <<PANEL;
</div>
</fieldset>
<fieldset>
<legend>ページフッタ外部ファイル</legend>
PANEL
    print $footer_file_name_menu;
    undef $footer_file_name_menu;
    print ' <label><input type="checkbox" name="data:'
        . $id
        . 'insideExternalFooter" value="1"'
        . ( ( $main::data{ $key . 'insideExternalFooter' } )
        ? ' checked="checked"'
        : '' )
        . ' /> コンテナ（コンテンツフレーム）の中に表示する</label>';
    print <<PANEL;
</fieldset>
</fieldset>
PANEL

    $this->print_sub_preference_panel( $id, $key );

    print '<fieldset class="submit">';
    print
        '<span class="warning">新しい設定は次回書き込みから反映されます。</span>'
        if defined $preference::static_page && $preference::static_page;
    print '<input type="submit" value="保存" class="save" /></fieldset>';
    $this->print_import_menu;
}

#他の設定のインポート
sub print_import_menu ($) {
    my $this = shift;
    if (   defined $preference::id
        && %preference::page
        && $preference::page{$main::k}
        && keys(%preference::page) > 1 )
    {
        print '</form>';
        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'importer',
            'method'   => 'post',
            'required' => \@setting::form
        );
        print
            '<fieldset id="import-menu"><legend>設定のインポート</legend>';
        print '<input type="hidden" name="id" value="'
            . $preference::id . '" />';
        print '<select name="import">';
        my $code = '';
        foreach ( sort keys %preference::page ) {
            if ( $_ =~ /^([0-9a-zA-Z_]{3})/ && $code ne $1 ) {
                print '<option value="' . $preference::id . '"></option>'
                    if $code ne '';
                $code = $1;
            }
            print '<option value="'
                . $preference::page{$_} . '"'
                . (
                (   defined $preference::page{$_}
                        && $preference::page{$_} eq $preference::id
                ) ? ' selected="selected"' : ''
                )
                . '>'
                . $setting::sub_jump_menu{'page'}{$_}
                . '</option>';
        }
        print '</select>';
        print
            '<input type="submit" value="インポート" onclick="return confirm(\'現在の設定が上書きされます。本当に読み込みますか？\');" />';
        print '</fieldset>';
    }
}

sub print_header_tool ($$) {
    my ( $this, $id, $target ) = @_;
    print
        '<label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'data:'
        . $id
        . $target
        . '\').setAutoFit(this.checked);" />フィット</label> ';
    require CMONOS::CGI_Lib::HTMLHelper;
    print HTMLHelper->new->html_helper_menu(
        'data:' . $id . $target,
        '<option value="&lt;!--sitetitle--&gt;&lt;!--/sitetitle--&gt;">サイトタイトル(&lt;!--sitetitle--&gt;)</option>'
            . '<option value="&lt;!--sitedescription--&gt;&lt;!--/sitedescription--&gt;">サイトの説明(&lt;!--sitedescription--&gt;)</option>'
            . '<option value="&lt;!--pagetitle--&gt;&lt;!--/pagetitle--&gt;">タイトル(&lt;!--pagetitle--&gt;)</option>'
            . '<option value="&lt;!--pageicon--&gt;&lt;!--/pageicon--&gt;">アイコン(&lt;!--pageicon--&gt;)</option>'
            . '<option value="&lt;!--cginame--&gt;&lt;!--/cginame--&gt;">CGIタイトル(&lt;!--cginame--&gt;)</option>'
            . (
            ( defined $setting::general{'description'} )
            ? '<option value="&lt;!--cgidescription--&gt;&lt;!--/cgidescription--&gt;">CGIの説明(&lt;!--cgidescription--&gt;)</option>'
            : ''
            )
            . '<option value="&lt;!--ogdescription--&gt;&lt;!--/ogdescription--&gt;">og:description(&lt;!--ogdescription--&gt;)</option>'
            . '<option value="&lt;!--ogimage--&gt;&lt;!--/ogimage--&gt;">og:image(&lt;!--ogimage--&gt;)</option>'
            . '<option value="&lt;!--topicpath--&gt;&lt;!--/topicpath--&gt;">トピックパス(&lt;!--topicpath--&gt;)</option>'
            . '<option value="&lt;!--topicpath type=&quot;list&quot;--&gt;&lt;!--/topicpath--&gt;">トピックパス(&lt;!--topicpath type=&quot;list&quot;--&gt;)</option>'
            . '<option value="&lt;!--masterrss title=&quot;RSS2.0&quot;--&gt;&lt;!--/masterrss--&gt;">サイト全体新着記事 RSS へのリンク(&lt;!--masterrss--&gt;)</option>'
            . '<option value="&lt;!--mastercommentrss title=&quot;RSS2.0(Comment)&quot;--&gt;&lt;!--/mastercommentrss--&gt;">サイト全体新着コメント RSS へのリンク(&lt;!--mastercommentrss--&gt;)</option>'
            . '<option value="&lt;!--mastertrackbackrss title=&quot;RSS2.0(Trackback)&quot;--&gt;&lt;!--/mastertrackbackrss--&gt;">サイト全体ト新着トラックバック RSS へのリンク(&lt;!--mastertrackbackrss--&gt;)</option>'
            . '<option value="&lt;!--masterpodcast title=&quot;RSS2.0(Podcast)&quot;--&gt;&lt;!--/masterpodcast--&gt;">サイト全体新着ポッドキャスト RSS へのリンク(&lt;!--masterpodcast--&gt;)</option>'
            . '<option value="&lt;!--sitemap--&gt;&lt;!--/sitemap--&gt;">サイトマップ(&lt;!--sitemap--&gt;)</option>'
            . '<option value="&lt;!--login--&gt;&lt;!--/login--&gt;">ログインリンク(&lt;!--login--&gt;)</option>'
            . (
            (          !defined $preference::static_page
                    || !$preference::static_page
            )
            ? '<option value="&lt;!--welcome--&gt;&lt;!--/welcome--&gt;">ウェルカムメッセージ(&lt;!--welcome--&gt;)</option>'
                . '<option value="&lt;!--username--&gt;&lt;!--/username--&gt;">ユーザーネーム(&lt;!--username--&gt;)</option>'
            : ''
            )
            . (
            ( defined $env::checkout && $env::checkout )
            ? '<option value="&lt;!--checkout--&gt;&lt;!--/checkout--&gt;">カートを見る ボタン(&lt;!--checkout--&gt;)</option>'
            : ''
            )
            . $this->_print_header_tool
    );
}

sub _print_header_tool ($) {
    my $this = shift;
    return '';
}

######################################################################
# サブ設定パネル
sub print_sub_preference_panel ($$) {
    my ( $this, $id, $key ) = @_;
    return;
}

######################################################################
# 共通する設定
sub print_page_button_setting ($$) {
    my ( $this, $index_enabled ) = @_;
    print '<fieldset>';
    print '<legend>ページ移動ボタン設定</legend>';
    print '<fieldset>';
    print '<legend>戻る ボタン</legend>';
    print $cgi::Deco->button_menu( 'data:returnButton', 'return', 120,
        $main::data{'returnButton'} );
    print
        '<div class="additional-field"><label for="data:returnUrl">戻る先URL</label><input type="text" size="50" maxlength="100" name="data:returnUrl" id="data:returnUrl" value="'
        . $this->escape( $main::data{'returnUrl'} )
        . '" /></div>';
    print '</fieldset>';

    if ($index_enabled) {
        print '<fieldset>';
        print '<legend>インデックス ボタン</legend>';
        print $cgi::Deco->button_menu( 'data:indexButton', 'index', 120,
            $main::data{'indexButton'} );
        print '</fieldset>';
    }
    print '<fieldset>';
    print '<legend>「前」ボタン</legend>';
    print $cgi::Deco->button_menu( 'data:prevButton', 'prev', 120,
        $main::data{'prevButton'} );
    print '</fieldset>';
    print '<fieldset>';
    print '<legend>「次」ボタン</legend>';
    print $cgi::Deco->button_menu( 'data:nextButton', 'next', 120,
        $main::data{'nextButton'} );
    print '</fieldset>';
    print '</fieldset>';
    return;
}

sub print_close_button_setting ($;$) {
    my ( $this, $disabled_option ) = @_;
    print '<fieldset>';
    print '<legend>閉じるボタン設定</legend>';
    print
        '<label><input type="checkbox" name="data:closeButtonDisabled" id="data:closeButtonDisabled"  value="1"'
        . (
        (   defined $main::data{'closeButtonDisabled'}
                && $main::data{'closeButtonDisabled'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> 閉じるボタンを表示しない</label>'
        if $disabled_option;
    print '<fieldset>';
    print '<legend>閉じる ボタン</legend>';
    print $cgi::Deco->button_menu( 'data:closeButton', 'close', 120,
        $main::data{'closeButton'} );
    print '</fieldset>';
    print '<fieldset>';
    print '<legend>戻る ボタン</legend>';
    print $cgi::Deco->button_menu( 'data:backButton', 'return', 120,
        $main::data{'backButton'} );
    print '</fieldset>';
    print '</fieldset>';
    return;
}

sub print_sc_button_setting ($) {
    my $this = shift;
    if ( defined $env::checkout && $env::checkout ) {
        print '<fieldset>';
        print '<legend>ショッピングカートボタン設定</legend>';

#	print '<fieldset>';
#	print '<legend>ログインボタン</legend>';
#	print $cgi::Deco->button_menu('data:loginButton','login',120,$main::data{'loginButton'});
#	print '</fieldset>';
#	print '<fieldset>';
#	print '<legend>ログアウトボタン</legend>';
#	print $cgi::Deco->button_menu('data:logoutButton','logout',120,$main::data{'logoutButton'});
#	print '</fieldset>';
        print '<fieldset>';
        print '<legend>カートを見る ボタン</legend>';
        print $cgi::Deco->button_menu( 'data:checkButton', 'check', 120,
            $main::data{'checkButton'} );
        print
            '<div class="additional-field"><label for="data:checkButtonName">ボタンの名前</label><input type="text" size="50" maxlength="100" name="data:checkButtonName" id="data:checkButtonName" value="'
            . $this->escape( $main::data{'checkButtonName'} )
            . '" /></div>';
        print '</fieldset>';
        print '</fieldset>';
    }
    return;
}

######################################################################
# モジュール読み込み
#sub import {
BEGIN {
    no warnings qw(once);
    $setting::deco_dir{'background'}  = '背景画像';
    $setting::deco_dir{'frame'}       = 'フレーム';
    $setting::deco_dir{'title'}       = 'タイトル画像';
    $setting::deco_type{'background'} = 'background';
    $setting::deco_type{'frame'}      = 'frame';
    $setting::deco_type{'title'}      = 'image';
}

1;
