package TableSetting;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# テーブル設定モジュール
# TableSetting.pm ver1.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;

    #書き込み
    if (   $this->is_master
        && defined $main::id
        && defined $main::data{'name'}
        && $main::id ne ''
        && $main::data{'name'} ne '' )
    {
        my ( $delete, %new );
        if ( defined $main::delete && $main::delete ) {
            $delete   = $main::id;
            $main::id = 'new';
        }
        else {
            $main::data{'setting'} = $this->z_to_h( $main::data{'setting'} )
                if $main::data{'setting'} =~ /[^0-9a-zA-Z_]/;
            $this->detag( \$main::data{'name'} );
            $main::data{'name'} = 'テーブル設定 ' . $main::id
                if !defined $main::data{'name'} || !$main::data{'name'};
            $main::data{'name'} =~ s/[\r\n]//sg;

            #新規ID
            $main::id = $this->new_number( \%table::obj, 0 )
                if $main::id eq 'new';

            my ( $k, $v );
            while ( ( $k, $v ) = each %main::data ) {
                $new{ $main::id . '_' . $k } = $v;
            }
        }

        undef %main::data;
        $this->write_db( $env::preference_dir . '/table',
            \%new, '', $delete, \%setting::table, 1 );
        undef %new;

        #読み込み
    }
    else {
        $this->import_db( $env::preference_dir . '/table',
            \%setting::table, 1 );
        push( @{ $this->{'dialog'} }, '設定名を入力して下さい。' )
            if defined $main::id
            && defined $main::data{'setting'}
            && $main::id ne ''
            && $main::data{'setting'} ne '';

    }

    #ページヘッダ
    $this->jump_menu('tools');

    if ( !$this->is_master ) {
        push( @{ $this->{'dialog'} },
            'アクセス権がありません。' );
        $this->print_page_header('tools');
        $this->print_page_footer('tools');
        return;
    }

    $this->print_page_header('tools');

    #IDなしのとき
    $main::id = 'new' if !defined $main::id || $main::id eq '';

    #前後設定
    print '<fieldset class="toolbox" id="toolnavi">';
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'tableselector',
        'required' => \@setting::form
    );
    print '<select name="id" onchange="submit();">';

    my ( $prev, $next );
    foreach ( sort { $a <=> $b } keys %setting::table ) {
        next if $_ eq '';
        $prev = $_ if $main::id ne 'new' && $_ < $main::id;
        $next = $_ if $main::id ne 'new' && !$next && $_ > $main::id;
        print '<option value="'
            . $_ . '"'
            . ( ( $main::id eq $_ ) ? ' selected="selected"' : '' ) . '>'
            . $setting::table{$_}{'name'}
            . '</option>';
        $prev = $_ if $main::id eq 'new';
    }
    print '<option value="new"'
        . ( ( $main::id eq 'new' ) ? ' selected="selected"' : '' )
        . '>新規</option>';

    print '</select><input type="submit" value="選択" />';
    print '</form>';

    if ( defined $prev && $prev ne '' ) {
        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'prevtable',
            'required' => \@setting::form
        );
        print '<input type="submit" name="id=' . $prev
            . '" value="前のテーブル設定" />';
        print '</form>';
    }
    if ( defined $next && $next ) {
        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'nexttable',
            'required' => \@setting::form
        );
        print '<input type="submit" name="id=' . $next
            . '" value="次のテーブル設定" />';
        print '</form>';
    }
    print '</fieldset>';
    undef $prev;
    undef $next;

    #設定名がない場合
    %{ $setting::table{$main::id} } = %main::data
        if $main::id ne ''
        && defined $main::data{'setting'}
        && $main::data{'setting'} ne '';

    print '<h1>テーブル'
        . (
        (          !defined $setting::table{$main::id}{'name'}
                || !$setting::table{$main::id}{'name'}
        ) ? '新規' : ''
        ) . '設定</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="id" value="'
        . $this->escape($main::id) . '" />';

    $this->escape( \$setting::table{$main::id}{'name'} );
    print
        '<fieldset><legend><label for="data:name">設定名</label></legend>';
    print
        '<input type="text" size="80" maxlength="80" name="data:name" id="data:name" value="'
        . $setting::table{$main::id}{'name'} . '" />';
    print '</fieldset>';

    if ( !defined $setting::table{$main::id}{'setting'}
        || $setting::table{$main::id}{'setting'} eq '' )
    {
        $setting::table{$main::id}{'setting'} = <<SETTING;
tableclass="datatable"
trclass="odd,even"
class0="odd,even"
thenabled="true"
SETTING
    }

    print
        '<fieldset><legend><label for="data:setting">設定</label></legend>';
    print
        '<textarea name="data:setting" id="data:setting" cols="80" rows="15" wrap="off" class="content-editor-area" ondblclick="cmonos.tool.colorPalette.open(\'data:setting+\',\'dom\')">'
        . $this->escape( $setting::table{$main::id}{'setting'} )
        . '</textarea>';

    print
        '<div class="toolbox"><select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:setting\').quotationByMenu(this)">';
    print '<option value="">▼挿入する</option>';
    print '<option value="'
        . $this->jsquot('\nsummary=&quot;&quot;')
        . '">説明/summary=""</option>';
    print '<option value="'
        . $this->jsquot('\ncaption=&quot;&quot;')
        . '">キャプション/caption=""</option>';
    print '<option value="'
        . $this->jsquot('\ntableclass=&quot;&quot;')
        . '">テーブルのスタイルクラス/tableclass=""</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot('\ntrclass=&quot;&quot;')
        . '">trスタイルクラス/trclass="r1,r2"</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot('\nthenabled=&quot;true&quot;')
        . '">thタグ（true=必ず使う/false=絶対使わない/auto=必要なときのみ使う）</option>';
    print '<option value="'
        . $this->jsquot('\nclassH=&quot;&quot;')
        . '">一行目tdスタイルクラス/classH="tdStyle1,tdStyle2"</option>';
    print '<option value="'
        . $this->jsquot('\nclassF=&quot;&quot;')
        . '">一列目tdスタイルクラス/classF="tdStyle1,tdStyle2"</option>';
    print '<option value="'
        . $this->jsquot('\nclass0=&quot;&quot;')
        . '">n行目tdスタイルクラス（これはn=1）/class0="tdStyle1,tdStyle2"</option>';
    print '<option value=""></option>';
    print '<option value="">▽非推奨設定</option>';
    print '<option value="'
        . $this->jsquot('\ncellpadding=&quot;&quot;')
        . '">セル余白/cellpadding=""</option>';
    print '<option value="'
        . $this->jsquot('\ncellspacing=&quot;&quot;')
        . '">罫線幅/cellspacing=""</option>';
    print '<option value="'
        . $this->jsquot('\nborder=&quot;&quot;')
        . '">枠線幅/border=""</option>';
    print '<option value="'
        . $this->jsquot('\nbordercolor=&quot;&quot;')
        . '">罫線色/bordercolor=""</option>';
    print '<option value="'
        . $this->jsquot('\nbackground=&quot;&quot;')
        . '">背景画像パス/background=""</option>';
    print '<option value="'
        . $this->jsquot('\ntablealign=&quot;&quot;')
        . '">表全体の位置揃え/tablealign=""</option>';
    print '<option value="'
        . $this->jsquot('\ntablewidth=&quot;&quot;')
        . '">表幅/tablewidth=""</option>';
    print '<option value="'
        . $this->jsquot('\ntableheight=&quot;&quot;')
        . '">表高さ/tableheight=""</option>';
    print '<option value="'
        . $this->jsquot('\ntablehspace=&quot;&quot;')
        . '">表水平余白/tablehspace=""</option>';
    print '<option value="'
        . $this->jsquot('\ntablevspace=&quot;&quot;')
        . '">表垂直余白/tablevspace=""</option>';
    print '<option value="'
        . $this->jsquot('\ntableoption=&quot;&quot;')
        . '">上記以外の表オプション/tableoption=""</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot('\nwidth=&quot;&quot;')
        . '">セル幅/width="50,50,50"</option>';
    print '<option value="'
        . $this->jsquot('\nheight=&quot;&quot;')
        . '">セル高/height="30,30,30"</option>';
    print '<option value="'
        . $this->jsquot('\nalign0=&quot;&quot;')
        . '">セル水平位置揃え/align0="center,left,right"</option>';
    print '<option value="'
        . $this->jsquot('\nvalign0=&quot;&quot;')
        . '">セル垂直位置揃え/valign0="bottom,top,middle"</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot('\nbgcolorH=&quot;&quot;')
        . '">一行目td背景色/bgcolorH="#000000,#333333"</option>';
    print '<option value="'
        . $this->jsquot('\nbgcolorF=&quot;&quot;')
        . '">一列目td背景色/bgcolorF="#ffffff,#cccccc"</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot('\nbgcolor0=&quot;&quot;')
        . '">n行目td背景色（これはn=1）/bgcolor0="#ffffff,#cccccc"</option>';
    print '<option value="'
        . $this->jsquot('\nbackground0=&quot;&quot;')
        . '">n行目td背景画像（これはn=1）/background0="bg.gif"</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot('\nstyleH=&quot;&quot;')
        . '">一行目tdスタイルシート/styleH="font-size: 9pt;,font-size: 10pt;"</option>';
    print '<option value="'
        . $this->jsquot('\nstyleF=&quot;&quot;')
        . '">一列目tdスタイルシート/styleF="font-size: 10pt;,font-size: 9pt;"</option>';
    print '<option value="'
        . $this->jsquot('\nstyle0=&quot;&quot;')
        . '">n行目tdスタイルシート（これはn=1）/style0="font-size: 10pt;,font-size: 9pt;"</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot('\nfontcolorH=&quot;&quot;')
        . '">一行目td文字色/fontcolorH="#ffffff,#cccccc"</option>';
    print '<option value="'
        . $this->jsquot('\nfontcolorF=&quot;&quot;')
        . '">一列目td文字色/fontcolorF="#000000,#333333"</option>';
    print '<option value="'
        . $this->jsquot('\nfontcolor0=&quot;&quot;')
        . '">n行目td文字色（これはn=1）/fontcolor0="#000000,#333333"</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot('\nfontsizeH=&quot;&quot;')
        . '">一行目td文字サイズ/fontsizeH="4,3,4"</option>';
    print '<option value="'
        . $this->jsquot('\nfontsizeF=&quot;&quot;')
        . '">一列目td文字サイズ/fontsizeF="5,7,1"</option>';
    print '<option value="'
        . $this->jsquot('\nfontsize0=&quot;&quot;')
        . '">n行目td文字サイズ（これはn=1）/fontsize0="3,2"</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot('\nnowrapH=&quot;&quot;')
        . '">一行目td無改行指定/nowrapH="1,0,0"</option>';
    print '<option value="'
        . $this->jsquot('\nnowrapF=&quot;&quot;')
        . '">一列目td無改行指定/nowrapF="1,0,0"</option>';
    print '<option value="'
        . $this->jsquot('\nnowrap0=&quot;&quot;')
        . '">n行目td無改行指定（これはn=1）/nowrap0="1,0,0"</option>';
    print '<option value=""></option>';
    print '<option value="'
        . $this->jsquot('\nfixed=&quot;1&quot;')
        . '">読み込んだところから表示する/fixed="1"</option>';
    print '<option value="'
        . $this->jsquot('\nbg=&quot;1&quot;')
        . '">テーブルの背面を反映させる/bg="1"</option>';
    print '</select> ';
    print
        ' <a href="JavaScript:cmonos.tool.colorPalette.open(\'data:setting+\')" ondblclick="cmonos.tool.colorPalette.open(\'data:setting+\',\'window\')"><img src="'
        . $env::image_dir
        . '/tool/palette.gif" alt="パレット" height="12" width="18" border="0" /></a></div>';

    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';

    print '<fieldset class="sample"><legend>現在の設定</legend>';

    require CMONOS::CGI_Lib::Table;
    my ( $table, $r );
    $table = Table->new( $setting::table{$main::id}{'setting'} );
    print $table->table;
    foreach $r ( 0 .. 5 ) {
        my ($c);
        print $table->tr;
        foreach $c ( 'A', 'B', 'C', 'D', 'E', 'F' ) {
            print $table->td( $c . '-' . $r );
        }
        print $table->_tr;
    }
    print $table->_table;
    print '</fieldset>';

    if ( $main::id ne 'new' ) {
        print
            '<fieldset class="delete"><input type="submit" name="delete=1" value="この設定を削除する" onclick="return confirm(\'この操作は取り消せません。本当に削除しますか？\');" /></fieldset>';
    }

    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');

}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    unless ( defined $default::subdivided
        && defined $main::c
        && $default::subdivided
        && $main::c ne '' )
    {

        $setting::table = 1;

        #クラス登録
        Common::class_register( '0130', 'TableSetting' );

        #メニュー登録
        $setting::jump_menu{'0130'}           = 'テーブル設定(共通)';
        $setting::master_check_option{'0130'} = 1;
    }
}

1;
