package CalCommon;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
eval 'use ' . $env::dbm_file . ';';
######################################################################
# カレンダーセットアップモジュール
# CalCommon.pm ver1.1.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-05-27
######################################################################
use Time::Local;

sub new_date ($) {
    my $this = {};
    bless $this;
    $this;
}

sub dt_stamp ($$$$) {
    my ( $this, $year, $mon, $mday, $hour, $min, $isdate ) = @_;
    return
        ( !$year ) ? ''
        : $year
        . sprintf( "%.2d", $mon )
        . sprintf( "%.2d", $mday )
        . ( ( $isdate || !defined $hour ) ? ''
        : 'T' . sprintf( "%.2d", $hour ) . sprintf( "%.2d", $min ) . "00" );
}

sub dt_to_time ($$) {
    my ( $this, $dt ) = @_;
    if ( $dt
        =~ /^([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2})([0-9]{2})([0-9]{2})Z?$/
        )
    {
        return timelocal(
            ( $6 + 0 ), ( $5 + 0 ), ( $4 + 0 ), ( $3 + 0 ),
            ( $2 - 1 ), $1
        );
    }
    elsif ( $dt =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        return timelocal( 0, 0, 0, ( $3 + 0 ), ( $2 - 1 ), $1 );
    }
    else {
        return 0;
    }
}

sub dt_fix($$) {
    my ( $this, $dt, $isdate ) = @_;
    my ( $y, $m, $d, $h, $min, $sec );
    if ( $dt
        =~ /^([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2})([0-9]{2})([0-9]{2})Z?$/
        )
    {
        ( $y, $m, $d, $h, $min, $sec )
            = ( $1, $2 + 0, $3 + 0, $4 + 0, $5 + 0, $6 + 0 );
        $h   = 23 if $h > 23;
        $min = 59 if $min > 59;
        $sec = 59 if $sec > 59;
    }
    elsif ( $dt =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        ( $y, $m, $d, $h, $min, $sec ) = ( $1, $2 + 0, $3 + 0, 0, 0, 0 );
        if ( !$isdate && $setting::general{'dateDisabled'} ) {
            $h = ( localtime(time) )[2];
        }
        else {
            $isdate = 1;
        }
    }
    else {
        return $this->time_to_dtstamp( time, $isdate );
    }
    if ( $m > 12 ) {
        $m = 12;
    }
    elsif ( $m == 0 ) {
        $m = 1;
    }
    if ( $d > 31 ) {
        $d = 31;
    }
    elsif ( $m == 0 ) {
        $d = 1;
    }
    if ( $this->is_impossible_mday( $y, $m, $d ) ) {
        $d = $this->get_last_mday( $y, $m );
    }
    return $this->dt_stamp( $y, $m, $d, $h, $min, $isdate );
}

sub time_to_dtstamp ($;$$) {
    my ( $this, $time, $isdate ) = @_;
    $time = time if !defined $time || $time eq '';
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday ) = localtime($time);
    $year += 1900;
    $mon++;
    return
          $year
        . sprintf( "%.2d", $mon )
        . sprintf( "%.2d", $mday )
        . ( ($isdate)
        ? ''
        : 'T'
            . sprintf( "%.2d", $hour )
            . sprintf( "%.2d", $min )
            . sprintf( "%.2d", $sec ) );
}

sub get_dt_gap ($$$;$) {
    my ( $this, $s, $e, $isdate ) = @_;
    my $gap = $this->dt_to_time($e) - $this->dt_to_time($s);
    return ($isdate)
        ? int( $gap / 86400 ) + ( ( $gap % 86400 ) ? 1 : 0 )
        : $gap;
}

sub timestamp_to_array ($$) {
    my ( $this, $dt ) = @_;
    if (   $dt
        && $dt
        =~ /^([0-9]{4})([0-9]{2})([0-9]{2})T?([0-9]{2})([0-9]{2})([0-9]{2})Z?$/
        )
    {
        return ( $1, ( $2 + 0 ), ( $3 + 0 ), ( $4 + 0 ), ( $5 + 0 ),
            ( $6 + 0 ) );
    }
    elsif ( $dt && $dt =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        return ( $1, ( $2 + 0 ), ( $3 + 0 ), 0, 0, 0 );
    }
    else {
        return ();
    }
}

sub get_hash_from_rrule ($$) {
    my ( $this, $rrule ) = @_;
    return () if !$rrule;
    my %hash = ();
    while ( $rrule =~ /\G([0-9a-zA-Z_]+)=([0-9a-zA-Z_,]+);?/g ) {
        my ( $k, $v ) = ( $1, $2 );
        $hash{$k} = ( $v =~ /,/ ) ? [ split( /,/, $v ) ] : $v;
    }
    return %hash;
}

sub get_prev_date ($$;$) {
    my ( $this, $dt, $interval ) = @_;
    return $dt if defined $interval && $interval ne '' && $interval == 0;
    $interval = 1 if !$interval;
    if ( $dt =~ /^([0-9]{4})([0-9]{2})([0-9]{2})/ ) {
        my ( $y, $m, $d )
            = $this->get_prev_date_array( $1, $2 + 0, $3 + 0, $interval );
        return $y . sprintf( "%.2d", $m ) . sprintf( "%.2d", $d );
    }
    return $dt;
}

sub get_prev_date_array ($$$$;$) {
    my ( $this, $y, $m, $d, $interval ) = @_;
    return ( $y, $m, $d )
        if defined $interval && $interval ne '' && $interval == 0;
    $interval = 1 if !$interval;
    $d -= $interval;
    while ( $d < 1 ) {
        if ( $m && $m == 1 ) {
            $y--;
            $m = 12;
        }
        else {
            $m--;
        }
        $d += $this->get_last_mday( $y, $m );
    }
    return ( $y, $m, $d );
}

sub get_next_date ($$;$) {
    my ( $this, $dt, $interval ) = @_;
    return $dt if defined $interval && $interval ne '' && $interval == 0;
    $interval = 1 if !$interval;
    if ( $dt =~ /^([0-9]{4})([0-9]{2})([0-9]{2})/ ) {
        my ( $y, $m, $d )
            = $this->get_next_date_array( $1, $2 + 0, $3 + 0, $interval );
        return $y . sprintf( "%.2d", $m ) . sprintf( "%.2d", $d );
    }
    return $dt;
}

sub get_next_date_array ($$$$;$) {
    my ( $this, $y, $m, $d, $interval ) = @_;
    return ( $y, $m, $d )
        if defined $interval && $interval ne '' && $interval == 0;
    $interval = 1 if !$interval;
    $d += $interval;
    while (( ( $m == 4 || $m == 6 || $m == 9 || $m == 11 ) && $d > 30 )
        || ( $m == 2 && $d > ( ( $this->is_leap_year($y) ) ? 29 : 28 ) )
        || $d > 31 )
    {
        $d -= $this->get_last_mday( $y, $m );
        if ( $m && $m == 12 ) {
            $y++;
            $m = 1;
        }
        else {
            $m++;
        }
    }
    return ( $y, $m, $d );
}

sub get_last_mday ($$$) {
    my ( $this, $y, $m ) = @_;
    return ( ( $m == 4 || $m == 6 || $m == 9 || $m == 11 ) ? 30
        : ( $m == 2 && $this->is_leap_year($y) ) ? 29
        : ( $m == 2 ) ? 28
        :               31 );
}

sub number_to_wday ($$) {
    my ( $this, $n ) = @_;
    my @week = ( 'SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA' );
    return $week[$n];
}

sub wday_to_number ($$) {
    my ( $this, $d ) = @_;
    my %w = (
        'SU' => 0,
        'MO' => 1,
        'TU' => 2,
        'WE' => 3,
        'TH' => 4,
        'FR' => 5,
        'SA' => 6
    );
    if ( ref $d && ref $d eq 'ARRAY' ) {
        return map( $w{$_}, @$d );
    }
    else {
        return $w{$d};
    }
}

sub dt_to_wday ($$) {
    my ( $this, $dt ) = @_;
    if ( $dt =~ /^([0-9]{4})([0-9]{2})([0-9]{2})/ ) {
        return $this->get_wday( $1, $2 + 0, $3 + 0 );
    }
    else {
        return 0;
    }
}

sub set_event_data ($$$) {
    my ( $this, $date, $event ) = @_;
    $this->{'date'}  = $date;
    $this->{'event'} = $event;
}

sub get_tool_buttons ($$;$$$) {
    my ( $this, $n, $id, $class, $is_opened ) = @_;
    if ( $this->is_secure('revise') ) {
        $this->use_icon('calendar') if !$cgi::icon;
        my $url
            = $cgi::path
            . '?m=Editor&amp;k=editor'
            . (
            ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
            . '&amp;n='
            . $n
            . '&amp;v='
            . ( ($main::v) ? $main::v : '' );
        my $button = '';
        $button
            = '<div'
            . ( ($id)    ? ' id="' . $id . '"'       : '' )
            . ( ($class) ? ' class="' . $class . '"' : '' ) . '>'
            if $id || $class;
        $button .= $cgi::icon->button(
            'eventedit',
            $setting::page{$setting::key}{'eventedit'},
            $url,
            ' &gt;修正 ',
            (     ($is_opened)
                ? ()
                : ( 'detailViewer',
                    { 'onclick' => $this->detail_viewer_opener }
                )
            )
        );
        if ( $this->is_secure( 'delete', 0, 'revise' ) ) {
            $button .= $cgi::icon->button(
                'eventdelete',
                $setting::page{$setting::key}{'eventdelete'},
                $url . '&amp;delete=1',
                ' x削除 ',
                (   ($is_opened)
                    ? ( '',
                        {   'onclick' =>
                                "return confirm('この操作は取り消せません。本当に削除しますか？')"
                        }
                        )
                    : ( 'detailViewer',
                        {   'onclick' =>
                                "return confirm('この操作は取り消せません。本当に削除しますか？') && "
                                . $this->detail_viewer_opener
                        }
                    )
                )
            );
        }
        $button .= '</div>' if $id || $class;
        return $button;
    }
    else {
        return '';
    }
}

sub detail_viewer_opener ($) {
    my $this = shift;
    return "cmonos.window.open('detailViewer',"
        . ( ( $setting::page{'detailWindowWidth'} )
        ? $setting::page{'detailWindowWidth'}
        : '640' )
        . ","
        . ( ( $setting::page{'detailWindowHeight'} )
        ? $setting::page{'detailWindowHeight'}
        : '480' )
        . ")";
}

sub get_event_dt ($$$$$$$$$$$$$$$$$$$$;$$) {
    my ($this,       $sy,          $sm, $sd,       $sh,
        $smin,       $ey,          $em, $ed,       $eh,
        $emin,       $uy,          $um, $ud,       $date,
        $indefinite, $freq,        $by, $interval, $byday,
        $se_gap,     $for_tooltip, $notag
    ) = @_;
    my ( $dey, $dem, $ded, $event_dt, $spacer, $separator );
    ( $dey, $dem, $ded ) = $this->get_prev_date_array( $ey, $em, $ed )
        if $date;
    $event_dt  = '';
    $separator = ($for_tooltip) ? '-' : ' 〜 ';
    $notag     = 1 if $for_tooltip;

    #イベントの日時
    if ($freq) {
        if ( $freq eq 'daily' || $freq eq 'weekly' ) {
            if ( $freq eq 'daily' ) {
                $event_dt
                    = ( $interval == 1 ) ? '毎日' : $interval . '日ごと';
            }
            elsif ( $freq eq 'weekly' ) {
                $event_dt
                    = ( $interval == 1 ) ? '毎週' : $interval . '週ごと';
                if ( @$byday == 5
                    && !grep( $_ !~ /^(MO|TU|WE|TH|FR)$/, @$byday ) )
                {
                    $event_dt .= ' 平日';
                }
                else {
                    $event_dt .= ' 日' if grep( $_ eq 'SU', @$byday );
                    $event_dt .= ' 月' if grep( $_ eq 'MO', @$byday );
                    $event_dt .= ' 火' if grep( $_ eq 'TU', @$byday );
                    $event_dt .= ' 水' if grep( $_ eq 'WE', @$byday );
                    $event_dt .= ' 木' if grep( $_ eq 'TH', @$byday );
                    $event_dt .= ' 金' if grep( $_ eq 'FR', @$byday );
                    $event_dt .= ' 土' if grep( $_ eq 'SA', @$byday );
                    $event_dt .= '曜日' if !$for_tooltip && @$byday == 1;
                }
            }
            if ( !$date ) {
                $event_dt
                    .= ' '
                    . $sh . ':'
                    . sprintf( "%.2d", $smin )
                    . $separator;
                if ( $sy == $ey && $sm == $em && $sd == $ed ) {
                    $event_dt .= $eh . ':' . sprintf( "%.2d", $emin );
                }
                else {
                    $event_dt
                        .= '翌 ' . $eh . ':' . sprintf( "%.2d", $emin );
                }
            }
        }
        else {
            my $se_dgap
                = int(
                ( ($date) ? $se_gap : $se_gap + $sh * 3600 + $smin * 60 )
                / 86400 );
            if ( $freq eq 'monthly' ) {
                $event_dt
                    = ( $interval == 1 )
                    ? '毎月'
                    : $interval . 'ヶ月ごと';
                if ( $by && $by eq 'bywday' ) {
                    $event_dt
                        .= ' 第'
                        . ( int( $sd / 7 ) + ( ( $sd % 7 ) ? 1 : 0 ) )
                        . $this->get_wday_name( $sy, $sm, $sd )
                        . '曜日';
                }
                else {
                    $event_dt .= ' ' . $sd . '日';
                }
            }
            elsif ( $freq eq 'yearly' ) {
                $event_dt
                    = ( $interval == 1 ) ? '毎年' : $interval . '年ごと';
                $event_dt
                    .= ' '
                    . $sm
                    . ( ($for_tooltip) ? '/' . $sd : '月' . $sd . '日' );
            }
            if ($date) {
                if ( $se_dgap > 1 ) {
                    if ( $freq eq 'yearly' && $by eq 'bymday' ) {
                        $event_dt
                            .= $separator
                            . ( ( $dey > $sy ) ? '翌年 ' : '' )
                            . $dem . '月'
                            . $ded . '日';
                    }
                    else {
                        $event_dt .= $separator . $se_dgap . '日間';
                    }
                }
            }
            else {
                $event_dt
                    .= ' '
                    . $sh . ':'
                    . sprintf( "%.2d", $smin )
                    . $separator;
                if ( $sy == $ey && $sm == $em && $sd == $ed ) {
                    $event_dt
                        .= ' '
                        . $sh . ':'
                        . sprintf( "%.2d", $smin )
                        . $separator
                        . $eh . ':'
                        . sprintf( "%.2d", $emin );
                }
                else {
                    $event_dt
                        .= ( ( $se_dgap == 1 ) ? '翌' : $se_dgap . '日後' )
                        . ' '
                        . $eh . ':'
                        . sprintf( "%.2d", $emin );
                }
            }
        }
        return $event_dt if $for_tooltip;
        if ( !$indefinite ) {
            $event_dt .= ($notag) ? ' ' : '<div class="note">';
            $event_dt .= '[期間:'
                . $this->event_date_text( $sy, $sm, $sd, $for_tooltip );
            ( $uy, $um, $ud ) = $this->get_prev_date_array( $uy, $um, $ud )
                if !$date;
            $event_dt
                .= $separator
                . $uy . '年'
                . $um . '月'
                . $ud . '日('
                . $this->get_wday_name( $uy, $um, $ud ) . ')]';
            $event_dt .= '</div>' if !$notag;
        }
        $event_dt .= ( ($notag) ? ' ' : '<div class="note">' ) . '[開始:';
    }
    my $tz = ( $ENV{'TZ'} && $ENV{'TZ'} =~ /([+\-][0-9]+)$/ ) ? $1 * -1 : 0;
    $tz = ($tz) ? sprintf( "%+.2d", $tz ) . ':00' : 'Z';
    $event_dt
        .= '<time itemprop="startDate" datetime="'
        . $sy . '-'
        . sprintf( "%.2d", $sm ) . '-'
        . sprintf( "%.2d", $sd )
        . ( ($date)
        ? ''
        : 'T' . sprintf( "%.2d", $sh ) . ':' . sprintf( "%.2d", $smin ) )
        . $tz . '">'
        if !$notag;
    $event_dt .= $this->event_date_text( $sy, $sm, $sd, $for_tooltip );
    if ($date) {
        $event_dt
            .= ( ( !$notag ) ? '</time>' : '' )
            . $separator
            . (
            ( !$notag )
            ? '<time itemprop="endDate" datetime="'
                . $ey . '-'
                . sprintf( "%.2d", $em ) . '-'
                . sprintf( "%.2d", $ed )
                . (
                ($date)
                ? ''
                : 'T'
                    . sprintf( "%.2d", $eh ) . ':'
                    . sprintf( "%.2d", $emin )
                )
                . $tz . '">'
            : ''
            )
            . $this->event_date_text( $dey, $dem, $ded, $for_tooltip )
            if $sy != $dey || $sm != $dem || $sd != $ded;

#} elsif ($for_tooltip) {
#	$event_dt .= $separator . $this->event_date_text($ey,$em,$ed,1) if $sy != $ey || $sm != $em || $sd != $ed;
    }
    else {
        $event_dt .= ' ' . $sh . ':' . sprintf( "%.2d", $smin );
        $event_dt .= '</time>' if !$notag;
        $event_dt .= $separator;
        $event_dt
            .= '<time itemprop="endDate" datetime="'
            . $ey . '-'
            . sprintf( "%.2d", $em ) . '-'
            . sprintf( "%.2d", $ed )
            . ( ($date)
            ? ''
            : 'T' . sprintf( "%.2d", $eh ) . ':' . sprintf( "%.2d", $emin ) )
            . $tz . '">'
            if !$notag;
        if ( $sy == $ey && $sm == $em && $sd == $ed ) {
            $event_dt .= $eh . ':' . sprintf( "%.2d", $emin );
        }
        else {
            $event_dt
                .= $this->event_date_text( $ey, $em, $ed, $for_tooltip ) . ' '
                . $eh . ':'
                . sprintf( "%.2d", $emin );
        }
    }
    $event_dt .= '</time>' if !$notag;
    $event_dt .= ']' . ( ($notag) ? '' : '</div>' ) if $freq;
    return $event_dt;
}

sub event_date_text ($$$$;$) {
    my ( $this, $y, $m, $d, $for_tooltip ) = @_;
    if ($for_tooltip) {
        return $m . '/' . $d;
    }
    else {
        return
              $y . '年'
            . $m . '月'
            . $d . '日('
            . $this->get_wday_name( $y, $m, $d ) . ')';
    }
}

sub is_available_wday ($$$$$;$) {
    my ( $this, $available_wday_array, $y, $m, $d, $holiday ) = @_;
    my $w = $this->get_wday( $y, $m, $d );
    if ( !ref $holiday && grep( $_ == 7, @$available_wday_array ) ) {
        require CMONOS::CGI_Lib::Holiday;
        $holiday = Holiday->new;
    }
    my $is_holiday
        = (    $w != 0
            && ref $holiday
            && grep( $_ == 7, @$available_wday_array )
            && $holiday->jp($y)->name( $m, $d ) ) ? 1 : 0;
    if (   ( grep( $_ == $w, @$available_wday_array ) && !$is_holiday )
        || ( !grep( $_ == $w, @$available_wday_array ) && $is_holiday ) )
    {
        return 1;
    }
    else {
        return 0;
    }
}

sub is_required_date ($$$$$$$$$$) {
    my ($this,     $dt,  $rrule_hash, $start_date,
        $end_date, $ymd, $y,          $m,
        $d,        $md,  $prev_ref,   $next_ref
    ) = @_;
    if (   $dt !~ /^[0-9]{8}$/
        && $rrule_hash->{'FREQ'}
        && $rrule_hash->{'INTERVAL'} )
    {
        return 0 if $ymd < $start_date;
        if ( $rrule_hash->{'FREQ'} eq 'DAILY' ) {
            return 0 if $$prev_ref && $ymd == $start_date;
            if ( $rrule_hash->{'INTERVAL'} > 1 ) {
                my $dgap
                    = ($$prev_ref)
                    ? $this->get_dt_gap( $end_date,   $ymd, 1 )
                    : $this->get_dt_gap( $start_date, $ymd, 1 );
                return 0 if $dgap % $rrule_hash->{'INTERVAL'};
            }
        }
        elsif ( $rrule_hash->{'FREQ'} eq 'WEEKLY' ) {
            return 0 if $$prev_ref && $ymd == $start_date;
            if ( $rrule_hash->{'INTERVAL'} > 1 ) {
                my $dgap
                    = ($$prev_ref)
                    ? $this->get_dt_gap( $end_date,   $ymd, 1 )
                    : $this->get_dt_gap( $start_date, $ymd, 1 );
                return 0 if int( $dgap / 7 ) % $rrule_hash->{'INTERVAL'};
            }
        }
        elsif ( $rrule_hash->{'FREQ'} eq 'MONTHLY' ) {
            if ( $start_date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
                my ( $sy, $sm, $sd ) = ( $1, $2, $3 );
                if ( $rrule_hash->{'INTERVAL'} > 1 ) {
                    my $mgap = ( $y - $sy ) * 12 + $m - $sm;
                    $mgap-- if $$prev_ref && $$prev_ref >= $d;
                    return 0 if $mgap % $rrule_hash->{'INTERVAL'};
                }
                if ( $rrule_hash->{'BYMONTHDAY'} ) {
                    if ( $$prev_ref && $$prev_ref >= $d ) {
                        my $gap = 31 - $this->get_last_mday(
                            $this->get_prev_ym( $y, $m ) );
                        $$next_ref += $gap;
                        $$prev_ref -= $gap;
                        return 0 if $$next_ref < 0;
                    }
                }
                elsif ($dt =~ /^[0-9][A-Z]{2}$/
                    && ref $rrule_hash->{'BYDAY'}
                    && $rrule_hash->{'BYDAY'}[0]
                    && $rrule_hash->{'BYDAY'}[0] =~ /^([0-9])([A-Z]{2})$/ )
                {
                    my ( $wn, $wd ) = ( $1 + 0, $this->wday_to_number($2) );
                    my ( $sy, $sm, $sd, $fw );
                    if ( $$prev_ref && $$prev_ref >= $d ) {
                        ( $sy, $sm ) = $this->get_prev_ym( $y, $m );
                    }
                    else {
                        ( $sy, $sm ) = ( $y, $m );
                    }
                    $fw = $this->get_wday( $sy, $sm, 1 );
                    $sd = $wn * 7 + $wd + ( ( $wd < $fw ) ? 1 : -6 ) - $fw;
                    return 0 if $this->is_impossible_mday( $sy, $sm, $sd );
                    my $sdt = $this->dt_stamp( $sy, $sm, $sd );
                    my $dgap = $this->get_dt_gap( $sdt, $ymd, 1 );
                    return 0 if $sdt < $start_date;
                    my $se_dgap = $$prev_ref + $$next_ref;
                    return 0 if $dgap < 0 || $dgap > $se_dgap;
                    $$prev_ref = $dgap;
                    $$next_ref = $se_dgap - $dgap;
                }
            }
        }
        elsif ( $rrule_hash->{'FREQ'} eq 'YEARLY' ) {
            if ( $start_date =~ /^([0-9]{4})([0-9]{4})$/ ) {
                my ( $sy, $smd ) = ( $1, $2 );
                if ( $rrule_hash->{'INTERVAL'} > 1 ) {
                    my $ygap = $y - $sy - ( ( $md < $smd ) ? 1 : 0 );
                    return 0 if $ygap % $rrule_hash->{'INTERVAL'};
                }
                if ($$prev_ref) {
                    my $ly = ( $smd lt "0229" ) ? $sy : ( $sy + 1 );
                    my $ldate = $ly . "0229";
                    if ( $this->is_leap_year($ly) && $ldate < $end_date ) {
                        if ( $ymd < $ldate ) {
                            $$next_ref++;
                        }
                        else {
                            $$prev_ref++;
                        }
                    }
                }
            }
        }
    }
    return 1;
}

sub get_prev_ym ($$$) {
    my ( $this, $y, $m ) = @_;
    $m--;
    if ( $m == 0 ) {
        $m = 12;
        $y--;
    }
    return ( $y, $m );
}

sub get_event_date_info ($$$;$$$) {
    my ( $this, $dt_start, $dt_end, $rrule, $for_tooltip, $notag ) = @_;
    my ($sy,          $sm,       $sd,         $sh,
        $smin,        $ey,       $em,         $ed,
        $eh,          $emin,     $uy,         $um,
        $ud,          $date,     $indefinite, $freq,
        $by,          $interval, @byday,      $dt_start_time,
        $dt_end_time, $se_gap
    );
    use Time::Local;
    return '' if !$dt_start;
    $date = 1 if $dt_start =~ /^[0-9]{8}$/;
    ( $sy, $sm, $sd, $sh, $smin ) = $this->timestamp_to_array($dt_start);
    ( $ey, $em, $ed, $eh, $emin ) = $this->timestamp_to_array($dt_end);
    if ( ref $rrule && %$rrule ) {

        if ( $rrule->{'FREQ'} ) {
            $freq = lc $rrule->{'FREQ'};
            if ( $rrule->{'UNTIL'} ) {
                if ( $rrule->{'UNTIL'} =~ /^([0-9]{8})T[0-9]{6}Z$/ ) {
                    my $udt = $1;
                    if (   $ENV{'TZ'}
                        && $ENV{'TZ'} =~ /(\-?)([0-9]+)$/
                        && !$1
                        && $2 )
                    {
                        $udt = $this->get_prev_date($udt);
                    }
                    ( $uy, $um, $ud ) = $this->timestamp_to_array($udt);
                }
                else {
                    ( $uy, $um, $ud )
                        = $this->timestamp_to_array( $rrule->{'UNTIL'} );
                }
                ( $uy, $um, $ud )
                    = $this->timestamp_to_array( $rrule->{'UNTIL'} );
            }
            else {
                $indefinite = 1;
            }
            $interval = ( $rrule->{'INTERVAL'} ) ? $rrule->{'INTERVAL'} : 1;
            if ( $rrule->{'BYDAY'} ) {
                $this->scalar_to_array( \$rrule->{'BYDAY'} );
                @byday = @{ $rrule->{'BYDAY'} };
                if ( $freq eq 'monthly' ) {
                    $by = 'bywday';
                    @byday
                        = grep( $_ =~ s/^[0-9]+//g, @{ $rrule->{'BYDAY'} } );
                }
            }
            $by = 'bymday' if $rrule->{'BYMONTHDAY'};
        }
    }
    $dt_start_time = timelocal(
        0,
        ( ($date) ? 0 : $smin ),
        ( ($date) ? 0 : $sh ),
        $sd, ( $sm - 1 ), $sy
    );
    $dt_end_time = timelocal(
        0,
        ( ($date) ? 0 : $emin ),
        ( ($date) ? 0 : $eh ),
        $ed, ( $em - 1 ), $ey
    );
    $se_gap = $dt_end_time - $dt_start_time;
    return $this->get_event_dt(
        $sy,          $sm, $sd,       $sh,     $smin,
        $ey,          $em, $ed,       $eh,     $emin,
        $uy,          $um, $ud,       $date,   $indefinite,
        $freq,        $by, $interval, \@byday, $se_gap,
        $for_tooltip, $notag
    );
}

sub get_event_date_info_from_data ($$;$$) {
    my ( $this, $data, $for_tooltip, $notag ) = @_;
    my ( $dt_start, $dt_end, $rrule );
    if (   $data
        && $data
        =~ /^([0-9a-zA-Z_]+)-([0-9a-zA-Z_]+)\x09([^\x09\x0A\x0C\x0D\x20]*)\x09/
        )
    {
        ( $dt_start, $dt_end, $rrule ) = ( $1, $2, $3 );
    }
    $rrule = { $this->get_hash_from_rrule($rrule) };
    return $this->get_event_date_info( $dt_start, $dt_end, $rrule,
        $for_tooltip, $notag );
}

sub event_info_replace ($$$) {
    my ( $this, $template, $n, $event_dt, $title ) = @_;
    $template
        =~ s/<\!--dt(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/dt(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace('dt',$1,$2,$event_dt)/ges;
    $template
        =~ s/<\!--title(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/title(?![0-9a-zA-Z_])[^>]*-->/$this->linktodetail($1,$2,$n,($title || '詳細を見る'))/iges;
    $template
        =~ s/<\!--readmore(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/readmore(?![0-9a-zA-Z_])[^>]*-->/$this->linktodetail($1,$2,$n,'&raquo; もっとくわしく…')/iges;
    $template
        =~ s/<\!--editbutton(?![0-9a-zA-Z_])([^>]*)-->(.*?)<\!--\/editbutton(?![0-9a-zA-Z_])[^>]*-->/$this->edit_buttons($1,$2,$n)/iges;
    return $template;
}

sub edit_buttons ($$$$) {
    my ( $this, $attributes, $content, $n ) = @_;
    my $class
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+class="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    my $id
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+id="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    return $this->item_tag_replace(
        'editbutton',
        $attributes,
        $content,
        $this->get_tool_buttons(
            $n, $id, $class,
            (   (          $setting::key eq 'detail'
                        && $setting::page{'detailInSameWindow'}
                ) ? 0 : 1
            )
        )
    );
}

sub linktodetail ($$$;$$) {
    my ( $this, $attributes, $content, $n, $title ) = @_;
    $content
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+text="([^"]+)"/i )
        ? $this->unescape($1)
        : ''
        if !$content;
    my $class
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+class="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    my $id
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+id="([^"]+)"/i )
        ? $this->unescape($1)
        : '';
    my $query = $this->amp_escape(
        $this->set_query( 'url', [ 'and', 'search' ], 1 ) );
    return
          '<a href="'
        . $cgi::path
        . '?m=DetailViewer'
        . ( ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
        . '&amp;n='
        . $n
        . '&amp;v='
        . ( $main::v || $setting::page{'viewer'}{'defaultView'} || 'w' )
        . $query . '"'
        . ( ( !$setting::page{'detailInSameWindow'} )
        ? ' target="detailViewer" onclick="'
            . $this->detail_viewer_opener . '"'
        : '' )
        . ( ($id)    ? ' id="' . $this->escape($id) . '"'       : '' )
        . ( ($class) ? ' class="' . $this->escape($class) . '"' : '' ) . '>'
        . ( $title || $content ) . '</a>';
}

sub index_sweeper ($$;$) {
    my ( $this, $n, $regexp_enabled ) = @_;
    my ( @index_files, @rss_files );
    $n = [$n] if !ref $n;
    @index_files = (
        $env::externalfiles . '/recentcomments.ssi',
        $env::externalfiles . '/recenttrackbacks.ssi',
        $default::file . '/recentcomments.ssi',
        $default::file . '/recenttrackbacks.ssi',
    );
    @rss_files = (
        '../../index.xml',                 '../../comment.xml',
        '../../trackback.xml',             '../../podcast.xml',
        $default::file . '/index.xml',     $default::file . '/comment.xml',
        $default::file . '/trackback.xml', $default::file . '/podcast.xml',
    );
    if ($default::subdivided) {
        push( @index_files,
            $default::master_file_dir . '/recentcomments.ssi',
            $default::master_file_dir . '/recenttrackbacks.ssi' );
        push( @rss_files,
            $default::master_file_dir . '/index.xml',
            $default::master_file_dir . '/comment.xml',
            $default::master_file_dir . '/trackback.xml',
            $default::master_file_dir . '/podcast.xml' );
    }
    require CMONOS::CGI_Lib::IndexSweeper;
    my $url
        = $this->my_url
        . '?m=DetailViewer'
        . ( ( $default::subdivided && defined $main::c && $main::c ne '' )
        ? '&c=' . $main::c
        : '' )
        . '&n=';
    my $id
        = ( ( $default::subdivided && defined $main::c && $main::c ne '' )
        ? $main::c . '-'
        : '' );
    IndexSweeper->new(
        'index_files' => \@index_files,
        'rss_files'   => \@rss_files,
        )->sweep(
        [ map { $url . $_ } @$n ],
        { $default::id => [ map { $id . $_ } @$n ] },
        ( $regexp_enabled || 0 )
        );
}

sub get_target_month_dates ($$) {
    my ( $this, $date ) = @_;
    if ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        my ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
        my @dates = ();
        my $last = $this->get_last_mday( $y, $m );
        for ( my $i = 1; $i <= $last; $i++ ) {
            push( @dates,
                $y . sprintf( "%.2d", $m ) . sprintf( "%.2d", $i ) );
        }
        return @dates;
    }
    else {
        return ();
    }
}

sub sort_target_events ($$$) {
    my ( $this, $event, $sort ) = @_;
    return sort {
        $this->sort_by( \$event->{$a}->{'start_date'},
            \$event->{$b}->{'start_date'}, $sort )
            || $this->sort_by( \$event->{$a}->{'start_time'},
            \$event->{$b}->{'start_time'}, $sort )
            || $this->sort_by( \$event->{$a}->{'end_date'},
            \$event->{$b}->{'end_date'}, $sort )
            || $this->sort_by( \$event->{$a}->{'end_time'},
            \$event->{$b}->{'end_time'}, $sort )
            || $this->sort_by( \$a, \$b, $sort )
    } keys %$event;
}

sub get_target_events_hash ($$$$) {
    my ( $this, $dates, $listtarget, $boundary ) = @_;
    my ( $pending_time, $preview_time, %event, %match );
    $this->{'DB'} = {} if !$this->{'DB'};
    tie( %{ $this->{'DB'}{'category'} },
        $env::dbm_file, "$default::db/category", O_RDONLY, $env::dbm_mode )
        if $field::for{'category'}
        && !ref tied %{ $this->{'DB'}{'category'} };
    tie( %{ $this->{'DB'}{'dt'} },
        $env::dbm_file, "$default::db/dt", O_RDONLY, $env::dbm_mode )
        if !ref tied %{ $this->{'DB'}{'dt'} };
    tie( %{ $this->{'DB'}{'date'} },
        $env::dbm_file, "$default::db/date", O_RDONLY, $env::dbm_mode )
        if !ref tied %{ $this->{'DB'}{'date'} };
    if (   $setting::general{'pendingTime'}
        && !$this->is_secure('input')
        && !$this->is_secure('delete')
        && !$this->is_secure('revise') )
    {
        tie( %{ $this->{'DB'}{'ctime'} },
            $env::dbm_file, "$default::db/ctime", O_RDONLY, $env::dbm_mode )
            if !ref tied %{ $this->{'DB'}{'ctime'} };
        $pending_time = time - $setting::general{'pendingTime'};
    }
    if (   $setting::general{'previewTime'}
        && !$this->is_secure('input')
        && !$this->is_secure('delete')
        && !$this->is_secure('revise') )
    {
        $preview_time
            = $this->time_id( time + $setting::general{'previewTime'} );
    }
    my $title_field
        = ( $field::for{'title'} ) ? $field::for{'title'}
        : ( $field::for{'key'} )  ? $field::for{'key'}
        : ( $field::for{'name'} ) ? $field::for{'name'}
        :                           "";

    #表示対象となるイベントを読み込み
    my @search_fields = (%main::search) ? keys %main::search : ();
DATE:
    foreach my $ymd (@$dates) {
        my ( $y, $m, $d, $md );

        #関連する日付
        my @dt = ( 'DAILY', $ymd );
        if ( $ymd =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
            $md = $2 . $3;
            ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
            my $w = $this->number_to_wday( $this->get_wday( $y, $m, $d ) );
            my $wn = int( $d / 7 ) + ( ( $d % 7 ) ? 1 : 0 ) . $w;
            push( @dt, $w, $wn, $d, $md );
        }

        #関連する日付のイベント
    DATEID:
        foreach my $dt (@dt) {
            if ( $this->{'DB'}{'date'}->{ $this->encode_utf8($dt) } ) {
                my @n = split(
                    /,/,
                    $this->decode_utf8(
                        $this->{'DB'}{'date'}->{ $this->encode_utf8($dt) }
                    )
                );
            EVENT:
                foreach my $n (@n) {
                    my $category = '';
                    my $prev = ( $n =~ s/^([\-]?[0-9]+)<// ) ? eval($1) : 0;
                    my $next = ( $n =~ s/>([\-]?[0-9]+)$// ) ? eval($1) : 0;
                    next DATEID
                        if $pending_time
                        && $this->{'DB'}{'ctime'}->{ $this->encode_utf8($n) }
                        && $this->{'DB'}{'ctime'}->{ $this->encode_utf8($n) }
                        > $pending_time;

                    #カテゴリーチェック
                    if (   $field::for{'category'}
                        && %setting::event_category
                        && ref $setting::event_category{'ORDER'}
                        && %{ $setting::event_category{'ORDER'} }
                        && $this->{'DB'}{'category'}
                        ->{ $this->encode_utf8($n) } )
                    {
                        $category
                            = $this->decode_utf8( $this->{'DB'}{'category'}
                                ->{ $this->encode_utf8($n) } );
                        if ( !ref $setting::event_category{$category} ) {
                            undef $category;
                        }
                        elsif ( $setting::event_category{$category}{'closed'}
                            && !$this->is_secure('closed') )
                        {
                            next EVENT;
                        }
                    }

                    #検索
                    if (@search_fields) {
                        my ( $id, $is_matched, $search_enabled );
                    SEARCH:
                        foreach my $id (@search_fields) {
                            if ( $id eq 'freq' ) {
                                if ( $main::search{'freq'} ) {
                                    $search_enabled = 1;
                                    my $freq = 0;
                                    if ($this->{'DB'}{'dt'}
                                        ->{ $this->encode_utf8($n) }
                                        && $this->decode_utf8(
                                            $this->{'DB'}{'dt'}
                                                ->{ $this->encode_utf8($n) }
                                        ) =~ /FREQ=\w+/
                                        )
                                    {
                                        $freq = 1;
                                    }
                                    if ((      $main::search{'freq'} eq '1'
                                            && $freq
                                        )
                                        || ( $main::search{'freq'} eq '-1'
                                            && !$freq )
                                        )
                                    {
                                        $is_matched = 1;
                                    }
                                    elsif ($main::and) {
                                        next EVENT;
                                    }
                                }
                            }
                            elsif (
                                   ref $field::obj{$id}
                                && $field::obj{$id}->{'searchEnabled'}
                                && (   $id eq $title_field
                                    || $field::obj{$id}->{'class'} eq
                                    'xcal_category'
                                    || (defined $setting::security{'detail'}
                                        {'passwordCheck'}
                                        && $setting::security{'detail'}
                                        {'passwordCheck'} eq 'nocheck' )
                                    || $this->is_one_of_owners
                                )
                                && !$field::obj{$id}->{'Hidden'}
                                && $field::obj{$id}->match_enabled
                                && (   $field::obj{$id}->{'Open'}
                                    || $this->is_secure('secret') )
                                )
                            {
                                $search_enabled = 1;
                                my $db = $field::obj{$id}->db;
                                next SEARCH if !defined $db || !$db;
                                tie(%{ $this->{'DB'}{$db} }, $env::dbm_file,
                                    "$default::db/$db",      O_RDONLY,
                                    $env::dbm_mode
                                ) if !ref tied %{ $this->{'DB'}{$db} };
                                my $dat
                                    = $this->decode_utf8( $this->{'DB'}{$db}
                                        ->{ $this->encode_utf8($n) } );
                                if ( $field::obj{$id}->match( \$dat ) ) {
                                    $is_matched = 1;
                                }
                                elsif ($main::and) {
                                    next EVENT;
                                }
                            }
                        }
                        next EVENT if $search_enabled && !$is_matched;
                        if ( !$search_enabled ) {
                            undef %main::search;
                            undef @search_fields;
                        }
                    }

                    if ( ref $event{$n} ) {
                        next EVENT
                            if !$this->is_required_date(
                            $dt,
                            $event{$n}->{'rrule'},
                            $event{$n}->{'start_date'},
                            $event{$n}->{'end_date'},
                            $ymd,
                            $y,
                            $m,
                            $d,
                            $md,
                            \$prev,
                            \$next
                            );
                    }
                    elsif (
                        $this->{'DB'}{'dt'}->{ $this->encode_utf8($n) }
                        && $this->decode_utf8(
                            $this->{'DB'}{'dt'}->{ $this->encode_utf8($n) }
                        )
                        =~ /^([0-9]{8})T?([0-9]{6}|)-([0-9]{8})T?([0-9]{6}|)\x09([^\x09\x0A\x0C\x0D\x20]*)/
                        )
                    {
                        my ($start_date, $start_time, $end_date,
                            $end_time,   $rrule
                        ) = ( $1, $2, $3, $4, $5 );
                        next EVENT
                            if $preview_time
                            && $preview_time
                            < $start_date . ( $start_time || '000000' );
                        my $rrule_hash;
                        if ($rrule) {
                            while ( $rrule
                                =~ /\G([0-9a-zA-Z_]+)=([0-9a-zA-Z_,]+);?/g )
                            {
                                my ( $k, $v ) = ( $1, $2 );
                                if ( $v =~ /,/ ) {
                                    $rrule_hash->{$k} = [ split( /,/, $v ) ];
                                }
                                else {
                                    $rrule_hash->{$k} = $v;
                                }
                            }
                            $this->scalar_to_array( \$rrule_hash->{'BYDAY'} )
                                if $rrule_hash->{'BYDAY'};
                            next EVENT
                                if !$this->is_required_date( $dt,
                                $rrule_hash, $start_date, $end_date, $ymd,
                                $y, $m, $d, $md, \$prev, \$next );
                        }
                        if (!$listtarget
                            || (   $listtarget eq 'future'
                                && $boundary
                                <= $start_date . ( $start_time || '000000' ) )
                            || (   $listtarget eq 'past'
                                && $boundary
                                >= $end_date . ( $end_time || '000000' ) )
                            )
                        {
                            $event{$n}->{'start_date'} = $start_date;
                            $event{$n}->{'start_time'} = $start_time
                                if $start_time ne '';
                            $event{$n}->{'end_date'} = $end_date;
                            $event{$n}->{'end_time'} = $end_time
                                if $end_time ne '';
                        }
                    }
                    else {
                        next EVENT;
                    }
                }
            }
        }
    }
    undef @search_fields;
    return \%event;
}

sub set_event_id ($$$$;$) {
    my ( $this, $data, $n, $category, $prefix ) = @_;
    unless ( $data
        =~ s/^(\s*)<(div|section|article|table|tbody|thead|tfoot|tr|th|td|dl|dt|dd|ul|ol|li)(?!\w)([^<>]*?)>/$1 . $this->set_event_category_class($2,$3,$category)/iges
        )
    {
        $data
            = '<div class="detail-event-category-'
            . ( ($category) ? $category : '1' )
            . '" itemscope="itemscope" itemtype="http://data-vocabulary.org/Event">'
            . $data
            . '</div>';
    }
    $data
        =~ s/^(\s*)<(div|section|article|table|tbody|thead|tfoot|tr|th|td|dl|dt|dd|ul|ol|li)(?!\w)([^<>]*?)>/$1 . $this->set_eventdetail_id($2,$3,$n,$prefix)/iges
        if $prefix;
    return $data;
}

sub set_event_category_class ($$$$) {
    my ( $this, $tag, $attributes, $category ) = @_;
    my $new            = '';
    my $class_done     = 0;
    my $itemscope_done = 0;
    my $itemtype_done  = 0;
    while ( $attributes =~ /\s+(\w+)=(["'])(.*?)\2/gs ) {
        my ( $k, $v ) = ( lc $1, $3 );
        if ( $k eq 'class' ) {
            $new
                .= ' '
                . $k . '="'
                . $v
                . ( ($v) ? ' ' : '' )
                . 'detail-event-category-'
                . ( ($category) ? $category : '1' ) . '"';
            $class_done = 1;
        }
        elsif ( $k eq 'itempscope' ) {
            $new .= ' ' . $k . '="' . $k . '"';
            $itemscope_done = 1;
        }
        elsif ( $k eq 'itemptype' ) {
            $new
                .= ' '
                . $k . '="'
                . $v
                . ( ($v) ? ' ' : '' )
                . 'http://data-vocabulary.org/Event"';
            $itemtype_done = 1;
        }
        elsif ( $k eq 'class' ) {
        }
        $new .= ' ' . $k . '="' . $v . '"';
    }
    $new
        .= ' class="detail-event-category-'
        . ( ($category) ? $category : '1' ) . '"'
        if !$class_done;
    $new .= ' itemscope="itemscope"' if !$itemscope_done;
    $new .= ' itemtype="http://data-vocabulary.org/Event"' if !$itemtype_done;
    return '<' . $tag . $new . '>';
}

sub set_eventdetail_id ($$$$) {
    my ( $this, $tag, $attributes, $n, $prefix ) = @_;
    my $id = $prefix . 'event-detail-' . $n;
    $attributes .= ' id="' . $id . '"'
        unless $attributes =~ s/\s+id=(["'])(.*?)\1/ id="$id"/is;
    return '<' . $tag . $attributes . '>';
}

sub get_title_id ($) {
    my $this = shift;
    return
          ( $field::for{'title'} ) ? $field::for{'title'}
        : ( $field::for{'key'} )   ? $field::for{'key'}
        : ( $field::for{'name'} )  ? $field::for{'name'}
        :                            '';
}

sub event_list ($$$$;$$) {
    my ( $this, $attributes, $content, $event_hash, $prefix, $sort ) = @_;
    return '' if !$content || !$event_hash;
    my ( $header, $footer );
    ( $header, $footer, $attributes, $content )
        = $this->get_header_and_footer( $attributes, $content );
    my $list = '';
    $sort
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+sort="([^"]+)"/i )
        ? $this->unescape($1)
        : ( $sort || 'ascend' );
    my $title_id = $this->get_title_id;
    require CMONOS::CGI_Lib::Template;
    my $template_obj = Template->new;
    $template_obj->set(
        'field_style_disabled' => $setting::field_style_disabled,
        'field_tag_enabled'    => $setting::field_tag_enabled
    );
    $template_obj->template_parser( \$content );
    $content = $template_obj->deco($content);

    foreach my $n ( $this->sort_target_events( $event_hash, $sort ) ) {
        $list
            .= $this->get_event_data( $template_obj, $content, $n, $title_id,
            $prefix )
            if $n !~ /\D/;
    }
    return ( $list ne '' ) ? $header . $list . $footer : '';
}

sub get_event_data ($$$$;$$$$) {
    my ( $this, $template_obj, $template, $n, $title_id, $prefix, $event_dt,
        $category )
        = @_;
    return '' if $n =~ /\D/;
    $this->{'DB'} = {}                        if !ref $this->{'DB'};
    $event_dt     = $this->get_event_date($n) if !$event_dt;
    $title_id     = $this->get_title_id       if !$title_id;
    if ( !$category ) {
        tie( %{ $this->{'DB'}{'category'} },
            $env::dbm_file, "$default::db/category", O_RDONLY,
            $env::dbm_mode )
            if !ref tied %{ $this->{'DB'}{'category'} };
        $category = $this->decode_utf8(
            $this->{'DB'}{'category'}{ $this->encode_utf8($n) } );
    }
    my %data;
    while ( my ( $id, $field ) = each %field::obj ) {
        if ( ref $field ) {
            if (   $field->format_for_detail_enabled
                && !$field->{'Hidden'}
                && ( $field->{'Open'} || $this->is_secure('secret') ) )
            {
                my $data = '';
                my $db   = $field->db;
                if ( defined $db && $db ) {
                    tie( %{ $this->{'DB'}{$db} },
                        $env::dbm_file, "$default::db/$db", O_RDONLY,
                        $env::dbm_mode )
                        if !ref tied %{ $this->{'DB'}{$db} };
                    $data = $this->decode_utf8(
                        $this->{'DB'}{$db}{ $this->encode_utf8($n) } );
                }
                $data{$id} = $data;
            }
            else {
                $field->disabled;
            }
        }
    }
    my $data = $template_obj->replace(
        $this->event_info_replace(
            $this->page_item_replace(
                $template, 0, '', $setting::page{$setting::key}{'title'}
            ),
            $n,
            $event_dt,
            (   ($title_id)
                ? $field::obj{$title_id}->format_for_detail(
                    $data{$title_id}, $default::file . '/' . $n,
                    $main::c,         $n
                    )
                : undef
            )
        ),
        \%data,
        'detail',
        $default::file . '/' . $n,
        $main::c, $n
    );
    $data = $this->set_event_id( $data, $n, $category, $prefix );
    return $data;
}

sub get_event_date ($$) {
    my ( $this, $n ) = @_;
    my ( %dt, $dt_start, $dt_end, $rrule );
    tie( %dt, $env::dbm_file, $default::db . '/dt', O_RDONLY,
        $env::dbm_mode );
    if (   ref tied %dt
        && $dt{$n}
        && $this->decode_utf8( $dt{ $this->encode_utf8($n) } )
        =~ /^([0-9a-zA-Z_]+)-([0-9a-zA-Z_]+)\x09([^\x09\x0A\x0C\x0D\x20]*)\x09/
        )
    {
        ( $dt_start, $dt_end, $rrule ) = ( $1, $2, $3 );
    }
    untie %dt;
    $rrule = { $this->get_hash_from_rrule($rrule) };
    return (wantarray)
        ? (
        $this->get_event_date_info( $dt_start, $dt_end, $rrule ),
        (   (          ( !$rrule->{'FREQ'} || $rrule->{'UNTIL'} )
                    && $dt_start
                    && $dt_start =~ /^([0-9]{8})/
            ) ? $1 : ''
        ),
        $dt_start
        )
        : $this->get_event_date_info( $dt_start, $dt_end, $rrule );
}

sub monthlist ($$$) {
    my ( $this, $attributes, $content ) = @_;
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday )
        = localtime(time);
    $year += 1900;
    $mon++;
    my $start
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+start="([^"]+)"/i )
        ? $1
        : 'current';
    my $format
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+format="([^"]+)"/i )
        ? $1
        : '';
    if ( $start =~ /^[0-9]+$/ ) {
        $mon = $start % 12;
    }
    elsif ( $start =~ /^[\-\+][0-9]+$/ ) {
        $mon += $start;
        while ( $mon > 12 ) {
            $year++;
            $mon -= 12;
        }
        while ( $mon < 1 ) {
            $year--;
            $mon += 12;
        }
    }
    elsif ( $start =~ /^view([\-\+][0-9]+|)$/ ) {
        my $s = $1;
        if ( $main::date =~ /^([0-9]{4})([0-9]{2})[0-9]{2}$/ ) {
            ( $year, $mon ) = ( $1, $2 + 0 );
        }
        $mon += $s;
        while ( $mon > 12 ) {
            $year++;
            $mon -= 12;
        }
        while ( $mon < 1 ) {
            $year--;
            $mon += 12;
        }
    }
    my $list  = '<ul>';
    my $query = $this->amp_escape(
        $this->set_query( 'url', [ 'and', 'search' ], 1 ) );
    foreach my $m ( 0 .. 11 ) {
        $m += $mon;
        my $y = $year;
        while ( $m > 12 ) {
            $y++;
            $m -= 12;
        }
        while ( $m < 1 ) {
            $y--;
            $m += 12;
        }
        my $date = $this->dt_stamp( $y, $m, 1 );
        $list
            .= '<li><a href="'
            . $cgi::path
            . '?m=Viewer&amp;v=m'
            . (
            ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
            . '&amp;date='
            . $date
            . $query . '">';
        if ($format) {
            $list .= $this->date( $date, '', '', $format );
        }
        else {
            $list .= $y . '年' . $m . '月';
        }
        $list .= '</a></li>';
    }
    $list .= '</ul>';
    return $this->item_tag_replace( 'monthlist', $attributes, $content,
        $list );
}

sub monthname ($$$) {
    my ( $this, $attributes, $content ) = @_;
    my $nav
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+nav="([^"]+)"/i )
        ? lc $1
        : 'enabled';
    my $format
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+format="([^"]+)"/i )
        ? $1
        : '';
    my ( $year, $mon, $mday );
    if ( $main::date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        ( $year, $mon, $mday ) = ( $1, $2 + 0, $3 + 0 );
    }
    else {
        return '';
    }
    my $name = '';
    if ($format) {
        $name .= $this->date( $main::date, '', '', $format );
    }
    else {
        $name .= $mon . '月';
    }
    my $is_list
        = (
        ( ref $this->{'view'} && $this->{'view'}->calendar_type eq 'list' )
            || ( !ref $this->{'view'} && $this->calendar_type eq 'list' ) )
        ? 1
        : 0;
    if ($nav ne 'disabled'
        && (!$is_list
            || (   $setting::page{$setting::key}{'listUnit'}
                && $setting::page{$setting::key}{'listUnit'} eq 'monthly' )
        )
        )
    {
        my ( $prev_boundary, $next_boundary );
        if ( $is_list && $setting::page{$setting::key}{'listTarget'} ) {
            my ( $ns, $nmin, $nh, $nd, $nm, $ny ) = localtime(time);
            $ny += 1900;
            $nm++;
            $prev_boundary = $ny . sprintf( "%.2d", $nm ) . '01';
            $next_boundary = $ny . sprintf( "%.2d", $nm ) . '32';
        }
        my $query = $this->amp_escape(
            $this->set_query( 'url', [ 'and', 'search' ], 1 ) );
        my $y = $year;
        my $m = $mon - 1;
        my $d = $mday;
        if ( $m == 0 ) {
            $y--;
            $m = 12;
        }
        $d = $this->max_mday( $y, $m ) if $d > $this->max_mday( $y, $m );
        my $date = $this->dt_stamp( $y, $m, $d );
        $name
            = '<a href="'
            . $cgi::path
            . '?m=Viewer&amp;v='
            . $main::v
            . (
            ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
            . '&amp;date='
            . $date
            . $query
            . '" class="cal-prev-mon">&lsaquo;</a> '
            . $name
            if !$is_list
            || !$setting::page{$setting::key}{'listTarget'}
            || $setting::page{$setting::key}{'listTarget'} eq 'past'
            || $date >= $prev_boundary;
        $y = $year;
        $m = $mon + 1;
        $d = $mday;
        if ( $m > 12 ) {
            $y++;
            $m = 1;
        }
        $d = $this->max_mday( $y, $m ) if $d > $this->max_mday( $y, $m );
        $date = $this->dt_stamp( $y, $m, $d );
        $name
            .= ' <a href="'
            . $cgi::path
            . '?m=Viewer&amp;v='
            . $main::v
            . (
            ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
            . '&amp;date='
            . $date
            . $query
            . '" class="cal-next-mon">&rsaquo;</a>'
            if !$is_list
            || !$setting::page{$setting::key}{'listTarget'}
            || $setting::page{$setting::key}{'listTarget'} eq 'future'
            || $date <= $next_boundary;
    }
    return $this->item_tag_replace( 'monthname', $attributes, $content,
        $name );
}

sub yearname ($$$) {
    my ( $this, $attributes, $content ) = @_;
    my $nav
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+nav="([^"]+)"/i )
        ? lc $1
        : 'enabled';
    my $format
        = ( $attributes =~ /[\x09\x0A\x0C\x0D\x20]+format="([^"]+)"/i )
        ? $1
        : '';
    my ( $year, $mon, $mday );
    if ( $main::date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        ( $year, $mon, $mday ) = ( $1, $2 + 0, $3 + 0 );
    }
    else {
        return '';
    }
    my $name = '';
    my $is_list
        = (
        ( ref $this->{'view'} && $this->{'view'}->calendar_type eq 'list' )
            || ( !ref $this->{'view'} && $this->calendar_type eq 'list' ) )
        ? 1
        : 0;
    if (   $is_list
        && $setting::page{$setting::key}{'startMonth'}
        && $setting::page{$setting::key}{'startMonth'} > 1 )
    {
        my $y = $year;
        $y-- if $setting::page{$setting::key}{'startMonth'} > $mon;
        if ($format) {
            $name .= $this->date(
                $this->dt_stamp(
                    $y, $setting::page{$setting::key}{'startMonth'}, 1
                ),
                '', '', $format
            );
        }
        else {
            $name .= $y . '年度';
        }
    }
    else {
        if ($format) {
            $name .= $this->date( $main::date, '', '', $format );
        }
        else {
            $name .= $year . '年';
        }
    }
    if ( $nav ne 'disabled' ) {
        my ( $prev_boundary, $next_boundary );
        if ( $is_list && $setting::page{$setting::key}{'listTarget'} ) {
            my ( $ns, $nmin, $nh, $nd, $nm, $ny ) = localtime(time);
            $ny += 1900;
            $nm++;
            $prev_boundary = (
                (          $setting::page{$setting::key}{'startMonth'}
                        && $setting::page{$setting::key}{'startMonth'} > $nm
                ) ? $ny - 1 : $ny
                )
                . sprintf( "%.2d",
                $setting::page{$setting::key}{'startMonth'} )
                . '00';
            $next_boundary = (
                (          $setting::page{$setting::key}{'startMonth'}
                        && $setting::page{$setting::key}{'startMonth'} > $nm
                ) ? $ny : $ny + 1
                )
                . sprintf( "%.2d",
                $setting::page{$setting::key}{'startMonth'} )
                . '01';
        }
        my $query = $this->amp_escape(
            $this->set_query( 'url', [ 'and', 'search' ], 1 ) );
        my $y = $year - 1;
        my $m = $mon;
        my $d = $mday;
        $d = $this->max_mday( $y, $m ) if $d > $this->max_mday( $y, $m );
        my $date = $this->dt_stamp( $y, $m, $d );
        $name
            = '<a href="'
            . $cgi::path
            . '?m=Viewer&amp;v='
            . $main::v
            . (
            ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
            . '&amp;date='
            . $date
            . $query
            . '" class="cal-prev-year">&lsaquo;</a> '
            . $name
            if !$is_list
            || !$setting::page{$setting::key}{'listTarget'}
            || $setting::page{$setting::key}{'listTarget'} eq 'past'
            || $date >= $prev_boundary;
        $y    = $year + 1;
        $m    = $mon;
        $d    = $mday;
        $d    = $this->max_mday( $y, $m ) if $d > $this->max_mday( $y, $m );
        $date = $this->dt_stamp( $y, $m, $d );
        $name
            .= ' <a href="'
            . $cgi::path
            . '?m=Viewer&amp;v='
            . $main::v
            . (
            ( $default::subdivided && $main::c ) ? '&amp;c=' . $main::c : '' )
            . '&amp;date='
            . $date
            . $query
            . '" class="cal-next-year">&rsaquo;</a>'
            if !$is_list
            || !$setting::page{$setting::key}{'listTarget'}
            || $setting::page{$setting::key}{'listTarget'} eq 'future'
            || $date < $next_boundary;
    }
    return $this->item_tag_replace( 'yearname', $attributes, $content,
        $name );
}

sub untie_all_dbs ($) {
    my $this = shift;
    while ( my ( $k, $db ) = each %{ $this->{'DB'} } ) {
        untie(%$db);
    }
    undef %{ $this->{'DB'} };
}

sub amp_escape ($$) {
    my ( $this, $url ) = @_;
    return '' if !defined $url;
    $url =~ s/\&/\&amp;/g;
    return $url;
}

1;
