package CalMonthlyView;
require CMONOS::Calendar::CalDayView;
@ISA = qw(CalDayView);
use strict;
use utf8;
######################################################################
# カレンダー月表示モジュール
# CalMonthlyView.pm ver1.0.8
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2016-05-26
######################################################################
# オブジェクト作成
sub new ($) {
    my $pkg  = shift;
    my $this = {};
    bless $this, $pkg;
    return $this;
}

sub need_day_events ($) {
    my $this = shift;
    return 1;
}

sub get_target_dates ($$) {
    my ( $this, $date ) = @_;
    if ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        my ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
        my @dates = ();
        my $last  = $this->get_last_mday( $y, $m );
        my $first = $this->get_wday( $y, $m, 1 );
        my $cell
            = $last + $first
            - ( ( $last + $first ) % 7 )
            + ( ( ( $last + $first ) % 7 > 0 ) ? 7 : 0 );
        if ($first) {
            my ( $prev_y, $prev_m, $prev_last );
            if ( $m == 1 ) {
                $prev_y = $y - 1;
                $prev_m = 12;
            }
            else {
                $prev_y = $y;
                $prev_m = $m - 1;
            }
            $prev_last = $this->get_last_mday( $prev_y, $prev_m );
            for (
                my $i = ( $prev_last - $first + 1 );
                $i <= $prev_last;
                $i++
                )
            {
                push( @dates,
                          $prev_y
                        . sprintf( "%.2d", $prev_m )
                        . sprintf( "%.2d", $i ) );
            }
        }
        for ( my $i = 1; $i <= $last; $i++ ) {
            push( @dates,
                $y . sprintf( "%.2d", $m ) . sprintf( "%.2d", $i ) );
        }
        my $rest = $cell - $first - $last;
        if ($rest) {
            my ( $next_y, $next_m );
            if ( $m == 12 ) {
                $next_y = $y + 1;
                $next_m = 1;
            }
            else {
                $next_y = $y;
                $next_m = $m + 1;
            }
            for ( my $i = 1; $i <= $rest; $i++ ) {
                push( @dates,
                          $next_y
                        . sprintf( "%.2d", $next_m )
                        . sprintf( "%.2d", $i ) );
            }
        }
        return @dates;
    }
    else {
        return ();
    }
}

sub prev_calendar_date ($$) {
    my ( $this, $date ) = @_;
    if ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        my ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
        if ( $m == 1 ) {
            $y--;
            $m = 12;
        }
        else {
            $m--;
        }
        return $y . sprintf( "%.2d", $m ) . sprintf( "%.2d", $d );
    }
    else {
        return $date;
    }
}

sub next_calendar_date ($$) {
    my ( $this, $date ) = @_;
    if ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ ) {
        my ( $y, $m, $d ) = ( $1, $2 + 0, $3 + 0 );
        if ( $m == 12 ) {
            $y++;
            $m = 1;
        }
        else {
            $m++;
        }
        return $y . sprintf( "%.2d", $m ) . sprintf( "%.2d", $d );
    }
    else {
        return $date;
    }
}

sub page_title ($$) {
    my ( $this, $date ) = @_;
    if ( $date =~ /^([0-9]{4})([0-9]{2})/ ) {
        my ( $y, $m ) = ( $1, $2 + 0 );
        return $y . '年' . $m . '月';
    }
    return '';
}

sub calendar_type ($) {
    my $this = shift;
    return 'monthly';
}

sub day_limit ($) {
    my $this = shift;
    return 7;
}

sub print_hourly_events_list ($$;$) {
    my ( $this, $date, $query ) = @_;
    my $tz = ( $ENV{'TZ'} && $ENV{'TZ'} =~ /([+\-][0-9]+)$/ ) ? $1 * -1 : 0;
    $tz = ($tz) ? sprintf( "%+.2d", $tz ) . ':00' : 'Z';
    my $ymd
        = ( $date =~ /^([0-9]{4})([0-9]{2})([0-9]{2})$/ )
        ? $1 . '-' . $2 . '-' . $3
        : '';
    my $e = 0;
    no warnings qw(numeric);
    foreach
        my $h ( sort { $a <=> $b } keys %{ $this->{'date'}{$date}{'time'} } )
    {
        foreach my $n (
            sort {
                $this->{'event'}{$a}{'start_date'}
                    <=> $this->{'event'}{$b}{'start_date'}
                    || $this->{'event'}{$a}{'start_time'}
                    <=> $this->{'event'}{$b}{'start_time'}
                    || $a <=> $b
            } keys %{ $this->{'date'}{$date}{'time'}{$h} }
            )
        {
            my $this_event = $this->{'date'}{$date}{'time'}{$h}{$n};
            next if !defined $this_event->{'start_min'};
            next
                if $this_event->{'prev'}
                && !$this_event->{'next'}
                && !$this_event->{'end_time'}
                && !$this->{'event'}{$n}{'end_time'};
            print '<ol class="event-list">' if !$e;
            print '<li'
                . ( ( $n && $this->{'event'}{$n}{'sellout'} )
                ? ' class="event-sellout"'
                : ( $n && $this->{'event'}{$n}{'remaining'} )
                ? ' class="event-remaining"'
                : '' )
                . ' itemscope="itemscope" itemtype="http://data-vocabulary.org/Event">';
            print '<a';
            print ' class="event-category-'
                . ( ( $this->{'event'}{$n}{'category'} )
                ? $this->{'event'}{$n}{'category'}
                : '1' )
                . '"';
            print ' href="'
                . $cgi::path
                . '?m=DetailViewer'
                . ( ( $default::subdivided && $main::c )
                ? '&amp;c=' . $main::c
                : '' )
                . '&amp;n='
                . ( ( $n =~ /^([0-9]+)\.[0-9]+$/ ) ? $1 : $n )
                . '&amp;v='
                . $main::v
                . $query . '"';

            if ( !$setting::page{'detailInSameWindow'} ) {
                print ' target="detailViewer"';
                print ' onclick="' . $this->detail_viewer_opener . '"';
            }
            print ' title="'
                . $this->get_span($n)
                . (
                (   defined $this->{'event'}{$n}{'title'}
                        && $this->{'event'}{$n}{'title'} ne ''
                ) ? ' ' . $this->{'event'}{$n}{'title'} : ''
                ) . '"';
            print ' itemprop="url">';
            print '<del>' if $this->{'event'}{$n}{'sellout'};
            print(
                (          $this_event->{'start_time'}
                        && $this_event->{'start_time'}
                        =~ /^([0-9]{2})([0-9]{2})/
                )
                ? '<meta itemprop="startDate" content="'
                    . $ymd . 'T'
                    . $1 . ':'
                    . $2
                    . $tz
                    . '" /><span class="start-time">'
                    . ( $1 + 0 ) . ':'
                    . $2
                    . '</span>'
                : ( $this_event->{'prev'} )
                ? '<meta itemprop="startDate" content="'
                    . $ymd
                    . 'T00:00'
                    . $tz
                    . '" /><span class="start-time">0:00</span>'
                : ( $this->{'event'}{$n}{'start_time'}
                        =~ /^([0-9]{2})([0-9]{2})/ )
                ? '<meta itemprop="startDate" content="'
                    . $ymd . 'T'
                    . $1 . ':'
                    . $2
                    . $tz
                    . '" /><span class="start-time">'
                    . ( $1 + 0 ) . ':'
                    . $2
                    . '</span>'
                : ''
            );
            print(
                (          $this_event->{'end_time'}
                        && $this_event->{'end_time'}
                        =~ /^([0-9]{2})([0-9]{2})/
                )
                ? '<meta itemprop="endDate" content="'
                    . $ymd . 'T'
                    . $1 . ':'
                    . $2
                    . $tz . '" />'
                : ( $this_event->{'prev'} )
                ? '<meta itemprop="endDate" content="'
                    . $ymd
                    . 'T24:00'
                    . $tz . '" />'
                : ( $this->{'event'}{$n}{'end_time'}
                        =~ /^([0-9]{2})([0-9]{2})/ )
                ? '<meta itemprop="endDate" content="'
                    . $ymd . 'T'
                    . $1 . ':'
                    . $2
                    . $tz . '" />'
                : ''
            );
            print '<span itemprop="summary">'
                . $this->cut_string(
                (   (   defined $this->{'event'}{$n}{'title'}
                            && $this->{'event'}{$n}{'title'} ne ''
                    )                     ? $this->{'event'}{$n}{'title'}
                    : ($default::english) ? 'EVENT'
                    :                       '予定'
                ),
                56, 'i'
                ) . '</span>';
            print '</del>' if $this->{'event'}{$n}{'sellout'};
            print '</a>';
            print '</li>';
            $e = 1;
        }
    }
    print '</ol>' if $e;
}

sub print_table_header ($$$) {
    my ( $this, $wclass, $wname ) = @_;
    if ( $setting::page{$setting::key}{'weekRow'} ) {
        print '<thead>';
        print '<tr>';
        foreach my $w ( 0 .. 6 ) {
            print '<th class="' . $wclass->[$w] . '">';
            print $wname->[$w];
            print '</th>';
        }
        print '</tr>';
        print '</thead>';
    }
}

sub print_hourly_events ($) {
    my $this = shift;
}

sub print_event_separator ($) {
    my $this = shift;
    print '</tr><tr>';
}

1;
