package CategoryEditor;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# カレンダーカテゴリー編集モジュール
# CategoryEditor.pm ver1.1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-09-25
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( $newname, $newid, @folder );

    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';

    unless ( opendir( DIR, $default::db ) ) {
        push(
            @{ $this->{'dialog'} },
            'データベースディレクトリの設定を確認してください。'
        );
    }
    else {
        no warnings qw(numeric);
        @folder
            = sort { $a <=> $b || $a cmp $b }
            grep( -d "$default::db/$_"
                && $_ !~ /^\-/
                && -d "$default::master_preference_dir/$_",
            readdir(DIR) );
        closedir(DIR);
    }

    #書き込み
    if ( defined $main::a && $main::a eq 'e' ) {
        my $is_changed = 0;

        #データ削除
        push( @main::delete, $main::delete )
            if defined $main::delete && $main::delete;
        @main::delete = grep( $_ ne '', @main::delete );
        $this->delete_category( \@main::delete );
        $is_changed = 1 if @main::delete;

        #新規作成
        if ( defined $main::new{'id'} && $main::new{'id'} ne '' ) {
            $main::new{'id'} = $this->z_to_h( $main::new{'id'} );
            if ($this->mk_category(
                    $main::new{'id'},   \@folder,
                    $main::new{'name'}, $main::new{'mastername'},
                    $main::new{'master'}
                )
                )
            {

                #増設通知
                if ( defined $main::notification && $main::notification ) {
                    my %installmsg;
                    $this->import_db(
                        $default::preference_dir . '/installmsg',
                        \%installmsg );
                    $installmsg{'subject'}
                        = '【お知らせ】カレンダーを増設しました。'
                        if !defined $installmsg{'subject'}
                        || !$installmsg{'subject'};
                    $installmsg{'msg'}
                        = '○カレンダーのURL' . "\x0A" . '<uri>'
                        . "\x0A\x0A"
                        . '○カレンダー環境設定ツールのURL'
                        . "\x0A"
                        . '<tool>' . "\x0A"
                        if !defined $installmsg{'msg'} || !$installmsg{'msg'};
                    my $tool
                        = $this->my_url . '?m=Tools&c=' . $main::new{'id'};
                    my $uri = $this->my_url . '?c=' . $main::new{'id'};
                    $installmsg{'msg'} =~ s/<(uri|url)>/$uri/igs;
                    $installmsg{'msg'} =~ s/<tool>/$tool/igs;

                    unless ( ref $cgi::SendMail ) {
                        require CMONOS::CGI_Lib::SendMail;
                        $cgi::SendMail = SendMail->new( $env::sendmail,
                            $env::master_email, '', $cgi::input, 1 );
                    }
                    $cgi::SendMail->send_mail(
                        'To' => (
                            ( $main::new{'master'} )
                            ? ( ( $main::new{'mastername'} )
                                ? $main::new{'mastername'} . ' <'
                                    . $main::new{'master'} . '>'
                                : $main::new{'master'} )
                            : $env::master_email
                        ),
                        'From' => (
                            ( $setting::general{'master'} )
                            ? ( ( $setting::general{'mastername'} )
                                ? $setting::general{'mastername'} . ' <'
                                    . $setting::general{'master'} . '>'
                                : $setting::general{'master'} )
                            : $env::master_email
                        ),
                        'Subject' => $installmsg{'subject'},
                        'message' => $installmsg{'msg'} . "\x0A\x0A"
                    );
                }
                undef %main::new;
                $is_changed = 1;
            }

        }
        elsif ($main::new{'name'} ne ''
            || $main::new{'mastername'} ne ''
            || $main::new{'master'} ne '' )
        {
            push(
                @{ $this->{'dialog'} },
                '新規フォルダ名を記入してください。'
            );
        }
        if ($is_changed) {
            require CMONOS::CGI_Lib::NavUpdate;
            NavUpdate->new->scan->update;
        }
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print
        '<h1 title="カテゴリーの増設や削除を行います。">カテゴリー編集</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="e" />';
    print '<fieldset><legend>カテゴリー編集</legend>';

    #テーブル設定
    require CMONOS::CGI_Lib::Table;
    my $table = Table->new;
    $table->set(
        'tableclass' => 'tooltable',
        'trclass'    => 'odd,even',
        'class0'     => 'odd,even',
        'classF'     => 'caution',
        'thenabled'  => 'true'
    );

    print $table->table;

    #ヘッダ
    print $table->tr;
    print $table->td('削除');
    print $table->td('フォルダ名');
    print $table->td('カテゴリー名');
    print $table->td('管理人名');
    print $table->td('管理人アドレス');
    print $table->td('環境設定');
    print $table->_tr;

    unless ( opendir( DIR, $default::db ) ) {
        print $table->tr
            . $table->td(
            'データベースディレクトリの設定を確認してください。',
            { 'colspan' => '6' }
            ) . $table->_tr;
    }
    else {
        no warnings qw(numeric);
        @folder = sort { $a <=> $b || $a cmp $b }
            grep( -d "$default::db/$_" && $_ !~ /^[\.\-]/, readdir(DIR) );
        closedir(DIR);
    }

    #設定項目
    my (%name);
    foreach my $folder_name (@folder) {
        my (%setting);
        print $table->tr;
        tie(%setting,
            $env::dbm_file,
            $default::master_preference_dir . '/' . $folder_name . '/general',
            O_RDONLY,
            $env::dbm_mode
        );
        $name{$folder_name}
            = ( ref tied %setting && $setting{'cginame'} ne '' )
            ? $this->escape( $this->decode_utf8( $setting{'cginame'} ) )
            : $folder_name;
        print $table->td( '<input type="checkbox" name="delete" value="'
                . $folder_name
                . '" onclick="cmonos.form.field(this).checkboxConfirm(\''
                . $name{$folder_name}
                . ' に関連する全てのデータが削除されます。本当に削除しますか？\')" />'
        );
        print $table->td($folder_name);
        print $table->td( $name{$folder_name} );
        print $table->td(
            (          ref tied %setting
                    && defined $setting{'mastername'}
                    && $setting{'mastername'} ne ''
            )
            ? $this->escape( $this->decode_utf8( $setting{'mastername'} ) )
            : ''
        );
        print $table->td(
            (          ref tied %setting
                    && defined $setting{'master'}
                    && $setting{'master'} ne ''
            )
            ? $this->escape( $this->decode_utf8( $setting{'master'} ) )
            : ''
        );
        untie %setting;
        print $table->td( '<a href="./start.cgi?m=Tools&amp;c='
                . $folder_name
                . '" target="_blank">環境設定</a>' );
        print $table->_tr;
    }

    #新規作成
    print $table->tr;
    print $table->td('新規');
    print $table->td( '<input type="text" name="new:id" size="10" value="'
            . $this->escape( $main::new{'id'} )
            . '" maxlength="16" />' );
    print $table->td( '<input type="text" name="new:name" size="20" value="'
            . $this->escape( $main::new{'name'} )
            . '" maxlength="30" />' );
    print $table->td(
              '<input type="text" name="new:mastername" size="20" value="'
            . $this->escape( $main::new{'mastername'} )
            . '" maxlength="20" />' );
    print $table->td(
              '<input type="email" name="new:master" size="30" value="'
            . $this->escape( $main::new{'master'} )
            . '" />' );
    print $table->td('');
    print $table->_tr;

    print $table->_table;
    print
        '<div class="hints">カテゴリーを追加するには新規フォルダ名を記入します。</div>';
    print '</fieldset>';

    #デフォルト設定選択
    print
        '<fieldset><legend>初期設定の読み込み</legend><table class="field-table"><tbody>';
    if (@folder) {
        print
            '<tr><td class="field-name"><label for="import">設定のインポート</label></td>';
        print
            '<td class="field"><select name="import" id="import" title="既に存在するカテゴリーから一般設定・アクセス権設定以外の設定を読み込みます。"><option value=""></option>';
        foreach my $f (@folder) {
            print '<option value="'
                . $f . '"'
                . ( ( $main::import && $main::import eq $f )
                ? ' selected="selected"'
                : '' )
                . '>'
                . $name{$f}
                . '</option>';
        }
        print '</select>';
        print '</td></tr>';
    }
    print
        '<tr><td class="field-name"><label for="pref">初期設定のデフォルト</label></td><td class="field">';
    $this->print_backup_menu('pref');
    print
        '</td></tr><tr><td class="field-name"><label for="field">フィールド設定のデフォルト</label></td><td class="field">';
    $this->print_backup_menu('field');
    print '</td></tr></tbody></table></fieldset>';

    print
        '<fieldset class="submit"><label><input type="checkbox" name="notification" value="1"'
        . ( ($main::notification) ? ' checked="checked"' : '' )
        . ' /> 新規カテゴリーの管理人にカテゴリー増設通知を送る</label> <input type="submit" value="保存" class="save" /></fieldset>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# カテゴリー削除
sub delete_category ($$) {
    my ( $this, $delete_ids ) = @_;
    return if !@$delete_ids;
    my ( @urls, $home_url );
    $home_url = $this->my_url . '?m=DetailViewer&c=';

    use File::Path;
    foreach my $cid (@$delete_ids) {
        next if !defined $cid || $cid eq '';
        if ( -d "$default::db/$cid" ) {
            rmtree "$default::db/$cid";
        }
        if ( -d "$default::file/$cid" ) {
            rmtree "$default::file/$cid";
        }
        if ( -d "$default::master_preference_dir/$cid" ) {
            rmtree "$default::master_preference_dir/$cid";
        }
        if ( -d "$default::master_field_pref_dir/$cid" ) {
            rmtree "$default::master_field_pref_dir/$cid";
        }
        push( @urls, $home_url . $cid );
    }
    require CMONOS::CGI_Lib::IndexSweeper;
    IndexSweeper->new(
        'index_files' => [
            $env::externalfiles . '/recentcomments.ssi',
            $env::externalfiles . '/recenttrackbacks.ssi',
        ],
        'rss_files' => [
            '../../index.xml',
            '../../comment.xml',
            '../../trackback.xml',
            '../../podcast.xml',
            $default::file . '/index.xml',
            $default::file . '/comment.xml',
            $default::file . '/trackback.xml',
            $default::file . '/podcast.xml',
        ],
    )->sweep( \@urls, { $default::id => $delete_ids } );

    #更新時データの消去
    my (%mtime);
    tie( %mtime, $env::dbm_file, $default::master_db_dir . '/category_mtime',
        O_RDWR, $env::dbm_mode );
    foreach my $cid (@$delete_ids) {
        utf8::encode($cid) if utf8::is_utf8($cid);
        delete $mtime{$cid};
    }
    untie(%mtime);

    return;
}

######################################################################
# カテゴリー作成
sub mk_category ($$$$) {
    my ( $this, $id, $folder, $name, $mastername, $master ) = @_;
    return if !defined $id || $id eq '';
    my ($error);
    if ( $master ne '' && $this->is_invalid_email_address($master) ) {
        push(
            @{ $this->{'dialog'} },
            '新しいカテゴリーの増設に失敗しました。管理人アドレスを確認してください。'
        );
        $error = 1;
    }
    if ( $id =~ /[^0-9a-zA-Z_]/ ) {
        push(
            @{ $this->{'dialog'} },
            '新しいカテゴリーの増設に失敗しました。フォルダ名は半角英数字にしてください。'
        );
        $error = 1;
    }
    if ( length $id > 32 ) {
        push(
            @{ $this->{'dialog'} },
            '新しいカテゴリーの増設に失敗しました。フォルダ名が長すぎます。'
        );
        $error = 1;
    }
    if ( grep( $_ eq $id, @$folder ) ) {
        push(
            @{ $this->{'dialog'} },
            '新しいカテゴリーの増設に失敗しました。このフォルダ名は既に使われています。'
        );
        $error = 1;
    }
    return 0 if defined $error && $error;

    #必要なディレクトリを作成
    my ( $pref, $field );
    $pref = $default::master_preference_dir . '/' . $id;
    ( $field = $default::master_field_pref_dir )
        =~ s!^$default::master_preference_dir/!$pref/!;

    $this->set_dir("$default::file/$id");
    $this->set_dir("$default::db/$id");
    $this->set_dir($pref);
    $this->set_dir($field);

    #設定のインポートorバックアップ読み込み
    if (   defined $main::import
        && $main::import ne ''
        && $main::import !~ /[^0-9a-zA-Z_]/
        && -d $default::master_preference_dir . '/' . $main::import )
    {
        my ( $import, $import_field, $icon );
        $import = $default::master_preference_dir . '/' . $main::import;
        ( $import_field = $default::master_field_pref_dir )
            =~ s!^$default::master_preference_dir/!$import/!;
        $this->directory_copy( $import, $pref, 0,
            [ '^[\.\-]', '\.(html|htm|txt|cgi)$' ] );
        $this->directory_copy( $import_field, $field, 0,
            [ '^[\.\-]', '\.(html|htm|txt|cgi)$' ] );
        $icon
            = $this->fileglob(
            $default::master_file_dir . '/' . $main::import,
            '^icon\.(gif|jpg|png)$' );
        if ( defined $icon && $icon ) {
            my ($icon_name);
            $icon_name = $1 if $icon =~ /(icon\.(gif|jpg|png)$)$/;
            $this->file_copy( $icon,
                $default::master_file_dir . '/' . $id . '/' . $icon_name );
        }
        if (  -f $default::master_file_dir . '/'
            . $main::import
            . '/event-category.css' )
        {
            $this->file_copy(
                $default::master_file_dir . '/'
                    . $main::import
                    . '/event-category.css',
                $default::master_file_dir . '/' . $id . '/event-category.css'
            );
        }
    }
    else {
        $this->restore( $backup::default{'pref'}, $pref, 'bak', $id );
        $this->restore( $backup::default{'field'}, $field, 'bak' );
    }

    #デフォルトの設定を作成
    $this->restore( $backup::default{'pref'},
        $pref, $main::restore{'pref'}, $id );
    $this->restore( $backup::default{'field'},
        $field, $main::restore{'field'} );

    my (%general);
    tie(%general, $env::dbm_file,
        $pref . '/general',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %general ) {
        $general{'cginame'} = $this->encode_utf8($name)
            if defined $name && $name;
        $general{'mastername'} = $this->encode_utf8($mastername)
            if defined $mastername && $mastername;
        $general{'master'} = $this->encode_utf8($master)
            if defined $master && $master;
    }
    untie %general;

    #グループ更新時を登録
    my (%category_mtime);
    tie(%category_mtime, $env::dbm_file,
        $default::master_db_dir . '/category_mtime',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %category_mtime ) {
        $category_mtime{ $this->encode_utf8($id) } = $this->encode_utf8(time);
    }
    untie %category_mtime;
    return 1;
}

######################################################################
# 復原メニュー
sub print_backup_menu ($$) {
    my ( $this, $key ) = @_;
    my (%bak);
    %bak = ();
    $this->get_backup_list_hash( $backup::default{$key}, \%bak )
        if defined $backup::default{$key} && $backup::default{$key};
    if ( !%bak ) {
        print
            '<div class="hints">設定ファイルがありません。</div>';
        return;
    }
    print '<select name="restore:', $key, '" id="restore:', $key,
        '" title="新規作成するカテゴリーに必要な設定を自動作成します。">';
    print '<option value=""></option>';
    print '<option value="bak"'
        . ( ( $main::restore{$key} && $main::restore{$key} eq 'bak' )
        ? ' selected="selected"'
        : '' )
        . '>'
        . $bak{'bak'}
        . '</option>'
        if $bak{'bak'};
    foreach my $ext ( sort keys %bak ) {
        next if $ext eq 'bak';
        print '<option value="', $ext, '"',
            ( ( $main::restore{$key} && $main::restore{$key} eq $ext )
            ? ' selected="selected"'
            : '' ), '>';
        print $bak{$ext};
        print '</option>';
    }
    print '</select>';
}

sub get_backup_list_hash ($$$) {
    my ( $this, $dir, $bak ) = @_;
    $this->_get_backup_list_hash( $env::default_dir . '/' . $dir, $bak )
        if $env::default_dir;
    $this->_get_backup_list_hash( $env::user_default_dir . '/' . $dir,
        $bak, 1 )
        if $env::user_default_dir;
}

sub _get_backup_list_hash ($$$;$) {
    my ( $this, $path, $bak, $user_dir ) = @_;
    my ( $db, @files );
    if ( -d $path ) {
        $db = '.*?';
    }
    else {
        ( $path, $db ) = ( $1, $2 ) if $path =~ m!^(.*)/([^/]+)$!;
        $db =~ s/\.([^\/\.]+)$//;
    }
    $path =~ s!/$!!;
    if ( opendir( DIR, $path ) ) {
        @files = grep( -f "$path/$_" && $_ !~ /^\./, readdir(DIR) );
        closedir(DIR);
        foreach my $file_name (@files) {
            if ( $file_name =~ /^$db\.bak$/ ) {
                next if defined $$bak{'bak'} && $$bak{'bak'} ne '';
                $$bak{'bak'} = '標準設定';
            }
            elsif ( $file_name =~ /^$db\.([0-9a-zA-Z_]+\.default)$/ ) {
                my $ext = ( ($user_dir) ? '~' : '' ) . $1;
                next if defined $$bak{$ext} && $$bak{$ext} ne '';
                if ( open( FILE, "$path/$file_name" ) ) {
                    binmode FILE, ":utf8";
                    $$bak{$ext} = <FILE>;
                    close(FILE);
                    chomp $$bak{$ext};
                    $$bak{$ext} .= '&lt;共有&gt;'
                        if $env::user_default_dir && !$user_dir;
                }
            }
        }
    }
}

######################################################################
# デフォルトを復原
sub restore ($$$$;$) {
    my ( $this, $bakpath, $dbpath, $ext, $id ) = @_;
    my $default_dir;
    if ( $ext && $ext =~ s/^~// ) {
        $default_dir = $env::user_default_dir || $env::default_dir;
        $bakpath = $default_dir . '/' . $bakpath;
    }
    elsif ($bakpath) {
        $default_dir = $env::default_dir;
        $bakpath     = $default_dir . '/' . $bakpath;
    }
    return
           if !$ext
        || ( $ext ne 'bak' && $ext !~ /^[0-9a-zA-Z_]+\.default$/ )
        || !$bakpath
        || !$dbpath;
    local (*DIR);
    return unless opendir( DIR, $bakpath );
    my @db = grep( -f "$bakpath/$_" && $_ !~ /^[\.\-]/, readdir(DIR) );
    closedir(DIR);
    my $user_directory   = $this->home_directory(1);
    my $home_dir_id      = ($env::home_dir_id) ? $env::home_dir_id : '';
    my $home_url         = $this->full_path( '../../', 1 );
    my $home_url_encoded = $this->url_encode($home_url);
    my $error            = 0;

    foreach my $db (@db) {
        my ( %db, $file );
        $file = $bakpath . '/' . $db;
        $db   = $dbpath . '/' . $db;
        next unless -f $file && $db =~ s/\.$ext$//;
        tie( %db, $env::dbm_file, $db, O_RDWR | O_CREAT, $env::dbm_mode );
        if ( ref tied %db ) {
            my $data = $this->read_file($file);
            if ($data) {
                while ( $data
                    =~ /([^\x09\x0A\x0D]+)\x09([^\x0A\x0D]*)[\x0A\x0D]*/sg )
                {
                    my ( $k, $v ) = ( $1, $2 );
                    $k =~ s/\&#09;/\x09/gs;
                    $k =~ s/\x0B/\x0A/gs;
                    $v =~ s/\&#09;/\x09/gs;
                    $v =~ s/\x0B/\x0A/gs;
                    $v =~ s!<home-dir-id>!$home_dir_id!gs;
                    $v =~ s/<home-url>/$home_url/gs;
                    $v =~ s/<home-url-encoded>/$home_url_encoded/gs;
                    $v =~ s!<user-directory>!$user_directory!gs;
                    utf8::encode($k) if utf8::is_utf8($k);
                    utf8::encode($v) if utf8::is_utf8($v);
                    eval '$db{$k} = $v;';
                    $error++ if defined $@ && $@;
                }
            }
        }
        untie %db;
    }
    if ( defined $id && $id ne '' ) {
        $ext =~ s/\.default$//;
        my $css
            = $default::master_file_dir . '/' . $id . '/event-category.css';
        my $path
            = $default_dir
            . '/calendar/default/css/event-category.'
            . $ext . '.css';
        $this->file_copy( $path, $css ) if -f $path;
    }
    push(
        @{ $this->{'dialog'} },
        '容量が大き過ぎる'
            . $error
            . '件の設定復元に失敗しました。'
    ) if $error;
    return;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ( $default::subdivided && ( !defined $main::c || $main::c eq '' ) ) {

        #クラス登録
        Common::class_register( '00c0', 'CategoryEditor' );

        #メニュー登録
        $setting::jump_menu{'00c0'} = 'カテゴリー編集';

    }
}

1;
