package AutoRegister;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# 自動登録モジュール
# AutoRegister.pm ver1.0.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-12-21
######################################################################
# メイン
sub run ($) {
    my $this = shift;
    if ( !$main::path ) {
        $this->print_dialog( $setting::key,
            'ディレクトリ情報を取得できません。' );
    }
    elsif ( !-d $this->path_conv( $main::path, 'path' ) ) {
        $this->print_dialog( $setting::key,
            '指定されたディレクトリは存在しません。' );
    }
    elsif ( $default::target && $main::path !~ /^\Q$default::target\E/ ) {
        $this->print_dialog( $setting::key,
            '指定されたディレクトリではアクセス権の設定ができません。'
        );
    }
    elsif ( $ENV{'SCRIPT_NAME'} =~ /^$main::path/ ) {
        $this->print_dialog( $setting::key,
            '正しいディレクトリ情報を取得できませんでした。'
        );
    }
    my ($current_directory,     $htaccess,
        $auth_enabled,          $auth_dbm_enabled,
        $auto_register_enabled, $auto_reissue_enabled,
        $htauthdb,              $htmaildb,
        $htpasswd,              $htcomment,
        $acc_is_invalid,        $email_is_invalid,
        $email2_is_invalid
    );
    $this->use_deco('');
    $current_directory = $this->path_conv( $main::path, 'path' );
    $htaccess = $current_directory . '/.htaccess';
    (   $auth_enabled,          $auth_dbm_enabled,
        $auto_register_enabled, $auto_reissue_enabled
    ) = $this->get_auth_settings($htaccess);
    if ( !$auth_enabled ) {
        $this->jump( ($main::goal) ? $main::goal : $this->get_referer );
    }
    elsif ( !$auto_register_enabled && !$auto_reissue_enabled ) {
        $this->print_dialog( $setting::key,
            '認証に失敗しました。' );
    }
    elsif ( $main::refresh || !$main::referer ) {
        $this->register_reload(
                  $ENV{'SCRIPT_NAME'}
                . '?m=AutoRegister'
                . $this->add_query(
                'url',
                {   'path'    => $main::path,
                    'referer' => (
                        ($main::referer)
                        ? $main::referer
                        : $this->get_referer
                    ),
                    ( ($main::goal) ? ( 'goal' => $main::goal ) : () )
                }
                )
        );
    }
    elsif ( !$main::goal ) {
        $main::goal = $this->get_referer;
    }
    if (   $main::referer =~ /^$ENV{'SCRIPT_NAME'}/
        && $main::referer =~ /referer=([^\&]+)/ )
    {
        $main::referer = $1;
        utf8::encode($main::referer) if utf8::is_utf8($main::referer);
        $main::referer =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
    }
    if ($auth_dbm_enabled) {
        use AnyDBM_File;
        $htauthdb = $current_directory . '/.htauthdb';
        $htmaildb = $current_directory . '/.htmaildb';
    }
    else {
        $htpasswd  = $current_directory . '/.htpasswd';
        $htcomment = $current_directory . '/.htcomment';
    }
    if ($main::action) {
        my $is_invalid = 0;
        my $expire_time
            = ( $setting::general{'lifetime'} )
            ? ( time + $setting::general{'lifetime'} )
            : '';
        my $passwd = $this->rand_char;
        if ( $this->is_invalid_email_address($main::addr) ) {
            push(
                @{ $this->{'dialog'} },
                (   ($main::addr)
                    ? 'メールアドレスを確認してください。'
                    : 'メールアドレスを入力してください。'
                )
            );
            $email_is_invalid = $is_invalid = 1;
        }
        if ( $auto_register_enabled && $main::action eq 'register' ) {
            if (  !$main::acc
                || $main::acc !~ /^[0-9a-zA-Z]{1}[0-9a-zA-Z_]{1,31}$/ )
            {
                push(
                    @{ $this->{'dialog'} },
                    '希望アカウントを確認してください。アカウントは2文字以上32文字以内の半角英数字にしてください。'
                );
                $acc_is_invalid = $is_invalid = 1;
            }
            if ( !$is_invalid ) {
                if ($auth_dbm_enabled) {
                    if ( $setting::general{'uniqaddr'} ) {
                        my (%htmaildb);
                        tie( %htmaildb, 'AnyDBM_File', $htmaildb, O_RDONLY,
                            $env::dbm_mode );
                        if ( ref tied %htmaildb ) {
                            if ($htmaildb{ $this->encode_utf8($main::addr) } )
                            {
                                push(
                                    @{ $this->{'dialog'} },
                                    'このメールアドレスは既に登録されています。'
                                );
                                $email_is_invalid = $is_invalid = 1;
                            }
                        }
                        untie(%htmaildb);
                    }
                    if ( !$is_invalid ) {
                        my ( %htmaildb, %htauthdb );
                        tie( %htauthdb, 'AnyDBM_File', $htauthdb, O_RDONLY,
                            $env::dbm_mode );
                        if ( ref tied %htauthdb ) {
                            if ( $htauthdb{ $this->encode_utf8($main::acc) } )
                            {
                                push(
                                    @{ $this->{'dialog'} },
                                    'このアカウント('
                                        . $main::acc
                                        . ')は既に登録されています。'
                                );
                                $acc_is_invalid = $is_invalid = 1;
                                my $n = 1;
                                $n++
                                    while defined $htauthdb{
                                    $this->encode_utf8(
                                        $main::acc . '_' . $n
                                    )
                                    };
                                $main::acc = $main::acc . '_' . $n;
                            }
                        }
                        untie(%htauthdb);
                        if ( !$is_invalid ) {
                            tie( %htauthdb, 'AnyDBM_File', $htauthdb, O_RDWR,
                                $env::dbm_mode );
                            if ( ref tied %htauthdb ) {
                                $htauthdb{ $this->encode_utf8($main::acc) }
                                    = $this->encode_utf8(
                                    join( ':',
                                        crypt( $passwd, $this->salt ),
                                        'member',
                                        '',
                                        $main::addr,
                                        '',
                                        $expire_time )
                                    );
                            }
                            else {
                                push(
                                    @{ $this->{'dialog'} },
                                    'アカウントの登録に失敗しました。'
                                );
                                $is_invalid = 1;
                            }
                            untie(%htauthdb);
                            if ( !$is_invalid ) {
                                tie( %htmaildb, 'AnyDBM_File', $htmaildb,
                                    O_RDWR | O_CREAT,
                                    $env::dbm_mode );
                                if ( ref tied %htmaildb ) {
                                    $htmaildb{ $this->encode_utf8($main::addr)
                                        } = $this->encode_utf8(
                                        $this->push_account(
                                            $htmaildb{$main::addr},
                                            $main::acc
                                        )
                                        );
                                }
                                untie(%htmaildb);
                            }
                            else {
                                push(
                                    @{ $this->{'dialog'} },
                                    'メールアドレスの登録に失敗しました。'
                                );
                                $is_invalid = 1;
                            }
                        }
                    }
                }
                else {
                    my ( $line, $acc_no, $addr_count, $acc_count );
                    $acc_no = 0;
                    local ( *CMT, *PASS );
                    if ( open( CMT, $htcomment ) ) {
                        binmode CMT, ":utf8";
                        flock( CMT, LOCK_SH );
                        my ( $line, $acc_no, $addr_count, $acc_count );
                        while ( $line = <CMT> ) {
                            my (@data) = split( /\t/, $line );
                            if ( $data[0] eq $main::acc ) {
                                $acc_count++;
                            }
                            elsif ($data[0] =~ /^${main::acc}_([0-9]+)/
                                && $1 > $acc_no )
                            {
                                $acc_no = $1;
                            }
                            $addr_count++ if $data[2] eq $main::addr;
                        }
                        close(CMT);
                    }
                    elsif ( open( PASS, $htpasswd ) ) {
                        binmode PASS, ":utf8";
                        flock( PASS, LOCK_SH );
                        while ( $line = <PASS> ) {
                            if ( $line =~ /^$main::acc:/ ) {
                                $acc_count++;
                            }
                            elsif ($line =~ /^${main::acc}_([0-9]+):/
                                && $1 > $acc_no )
                            {
                                $acc_no = $1;
                            }
                        }
                        close(PASS);
                    }
                    else {
                        push(
                            @{ $this->{'dialog'} },
                            'アカウント情報の読み取りに失敗しました。'
                        );
                        $is_invalid = 1;
                    }
                    if ( !$is_invalid ) {
                        if ($acc_count) {
                            push(
                                @{ $this->{'dialog'} },
                                'このアカウント('
                                    . $main::acc
                                    . ')は既に登録されています。'
                            );
                            $acc_is_invalid = $is_invalid = 1;
                            $acc_no++;
                            $main::acc = $main::acc . '_' . $acc_no;
                        }
                        elsif ( $setting::general{'uniqaddr'} && $addr_count )
                        {
                            push(
                                @{ $this->{'dialog'} },
                                'このメールアドレスは既に登録されています。'
                            );
                            $email_is_invalid = $is_invalid = 1;
                        }
                        if ( !$is_invalid ) {
                            if ( open( PASS, ">>$htpasswd" ) ) {
                                binmode PASS, ":utf8";
                                flock( PASS, LOCK_EX );
                                print PASS $main::acc . ':'
                                    . crypt( $passwd, $this->salt ) . "\x0A";
                                close(PASS);
                            }
                            else {
                                push(
                                    @{ $this->{'dialog'} },
                                    'アカウントの登録に失敗しました。'
                                );
                                $is_invalid = 1;
                            }
                            if ( !$is_invalid ) {
                                if ( open( CMT, ">>$htcomment" ) ) {
                                    binmode CMT, ":utf8";
                                    flock( CMT, LOCK_EX );
                                    print CMT join( "\x09",
                                        $main::acc, '', $main::addr,
                                        '',         $expire_time )
                                        . "\x0A";
                                    close(CMT);
                                }
                                else {
                                    push(
                                        @{ $this->{'dialog'} },
                                        'メールアドレスの登録に失敗しました。'
                                    );
                                    $is_invalid = 1;
                                }
                            }
                        }
                    }
                }
                if ( !$is_invalid ) {
                    $this->mail_to_user(
                        $setting::general{'autosubject'},
                        $setting::general{'automsg'},
                        $main::addr,
                        $main::acc,
                        $passwd,
                        '',
                        $expire_time
                    );
                    $this->print_auto_register_msg(
                        'アカウントを作成しました',
                        'アカウント情報を登録されたメールアドレスに送信しましたので、ご確認ください。'
                    );
                }
            }
        }
        elsif ( !$is_invalid
            && $auto_reissue_enabled
            && $main::action eq 'reissue' )
        {
            my $account = '';
            if ($auth_dbm_enabled) {
                my ( %htmaildb, @acc );
                tie( %htmaildb, 'AnyDBM_File', $htmaildb, O_RDONLY,
                    $env::dbm_mode );
                if ( ref tied %htmaildb ) {
                    if ( $htmaildb{ $this->encode_utf8($main::addr) } ) {
                        @acc = split(
                            /,/,
                            $this->decode_utf8(
                                $htmaildb{ $this->encode_utf8($main::addr) }
                            )
                        );
                    }
                    else {
                        push(
                            @{ $this->{'dialog'} },
                            'このメールアドレスは登録されていません。'
                        );
                        $email2_is_invalid = $is_invalid = 1;
                    }
                }
                untie(%htmaildb);
                if ( !$is_invalid ) {
                    my (%htauthdb);
                    tie( %htauthdb, 'AnyDBM_File', $htauthdb, O_RDONLY,
                        $env::dbm_mode );
                    if ( ref tied %htauthdb ) {
                        foreach my $n (@acc) {
                            if ( $htauthdb{ $this->encode_utf8($n) } ) {
                                my @data = split(
                                    /:/,
                                    $this->decode_utf8(
                                        $htauthdb{ $this->encode_utf8($n) }
                                    )
                                );
                                if ($main::addr eq $data[3]
                                    && (   $auto_register_enabled
                                        || !$data[5]
                                        || $data[5] > time )
                                    )
                                {
                                    $account = $_;
                                    last;
                                }
                            }
                        }
                    }
                    untie(%htauthdb);
                    if ( !$account ) {
                        push(
                            @{ $this->{'dialog'} },
                            'このメールアドレスは登録されていません。'
                        );
                        $email2_is_invalid = $is_invalid = 1;
                    }
                    if ( !$is_invalid ) {
                        tie( %htauthdb, 'AnyDBM_File', $htauthdb, O_RDWR,
                            $env::dbm_mode );
                        if ( ref tied %htauthdb ) {
                            my @data = split(
                                /:/,
                                $this->decode_utf8(
                                    $htauthdb{ $this->encode_utf8($account) }
                                )
                            );
                            $expire_time = $data[5]
                                if !$auto_register_enabled
                                || !$data[5]
                                || ( $expire_time
                                && $data[5]
                                && $expire_time < $data[5] );
                            $this->encode_utf8( $htauthdb{$account} )
                                = $this->encode_utf8(
                                join( ':',
                                    crypt( $passwd, $this->salt ),
                                    $data[1],
                                    $data[2],
                                    $data[3],
                                    $data[4],
                                    $expire_time )
                                );
                        }
                        else {
                            push(
                                @{ $this->{'dialog'} },
                                'アカウントの登録に失敗しました。'
                            );
                            $is_invalid = 1;
                        }
                        untie(%htauthdb);
                    }
                }
            }
            else {
                my @data = ();
                my $comment_data = $this->read_file( $htcomment, 1 );
                while ( $comment_data =~ /^(.*)$/gm ) {
                    @data = split( /\t/, $1 );
                    if ($data[2] eq $main::addr
                        && (   $auto_register_enabled
                            || !$data[4]
                            || $data[4] > time )
                        )
                    {
                        $account = $data[0];
                        last;
                    }
                }
                if ( !$account ) {
                    push(
                        @{ $this->{'dialog'} },
                        'このメールアドレスは登録されていません。'
                    );
                    $is_invalid = 1;
                }
                if ( !$is_invalid ) {
                    if (   $auto_register_enabled
                        && $data[4]
                        && ( !$expire_time || $expire_time > $data[4] ) )
                    {
                        $data[4] = $expire_time;
                        my $data = join( "\x09", @data );
                        unless (
                            $comment_data =~ s/^$account\t.*$/$data/m
                            && $this->write_file(
                                $htcomment, \$comment_data, 1
                            )
                            )
                        {
                            push(
                                @{ $this->{'dialog'} },
                                'アカウント情報の書き換えに失敗しました。'
                            );
                            $is_invalid = 1;
                        }
                    }
                    else {
                        $expire_time = $data[4];
                    }
                }
                undef $comment_data;
                undef @data;
                if ( !$is_invalid ) {
                    my $passwd_data = $this->read_file( $htpasswd, 1 );
                    my $data = $account . ':' . crypt( $passwd, $this->salt );
                    unless ( $passwd_data =~ s/^$account:.*$/$data/m
                        && $this->write_file( $htpasswd, \$passwd_data, 1 ) )
                    {
                        push(
                            @{ $this->{'dialog'} },
                            'アカウント情報の書き換えに失敗しました。'
                        );
                        $is_invalid = 1;
                    }
                    undef $passwd_data;
                }
            }
            if ( !$is_invalid ) {
                $this->mail_to_user(
                    $setting::general{'autosubject'},
                    $setting::general{'automsg'},
                    $main::addr,
                    $account,
                    $passwd,
                    '',
                    $expire_time
                );
                $this->print_auto_register_msg(
                    'パスワードを再発行しました',
                    'パスワード情報を登録されたメールアドレスに送信しましたので、ご確認ください。'
                );
            }
        }
    }
    $this->print_page_header('register');
    if ($auto_register_enabled) {
        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'autoregister',
            'method'   => 'post',
            'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
            'required' => \@setting::form
        );
        print
            '<fieldset id="auto-register"><legend>パスワード申請</legend>';
        print '<input type="hidden" name="action" value="register" />';
        print '<table class="field-table"><tbody>';
        print '<tr>';
        print
            '<td class="field-name"><label for="addr">メールアドレス</label></td>';
        print
            '<td class="field"><input type="email" name="addr" id="addr" value="'
            . $this->escape($main::addr)
            . '" onchange="cmonos.validate.field(this).asEMailAddress()"'
            . ( ($email_is_invalid) ? ' class="invalid-field"' : '' )
            . ' required="required" /></td>';
        print '</tr>';
        print '<tr>';
        print
            '<td class="field-name"><label for="acc">希望アカウント</label></td>';
        print
            '<td class="field"><input type="text" name="acc" id="acc" value="'
            . $this->escape($main::acc)
            . '" onchange="cmonos.validate.field(this).asWord()"'
            . ( ($acc_is_invalid) ? ' class="invalid-field"' : '' )
            . ' required="required" /></td>';
        print '</tr>';
        print '<tr class="tbody-start">';
        print
            '<td class="submit-box" colspan="2"><input type="submit" value="送信" class="save" /></td>';
        print '</tr>';
        print '</tbody></table>';
        print '</fieldset></form>';
    }
    if ($auto_reissue_enabled) {
        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'autoreissue',
            'method'   => 'post',
            'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
            'required' => \@setting::form
        );
        print
            '<fieldset id="auto-reissue"><legend>パスワード再発行</legend>';
        print '<input type="hidden" name="action" value="reissue" />';
        print
            '<label for="reissueaddr" class="datatitle">メールアドレス</label><input type="email" name="addr" id="reissueaddr" value="'
            . $this->escape($main::addr)
            . '" onchange="cmonos.validate.field(this).asEMailAddress()"'
            . ( ($email2_is_invalid) ? ' class="invalid-field"' : '' )
            . ' required="required" /><input type="submit" value="送信" class="save" />';
        print '</fieldset></form>';
    }
    $this->print_jump_menu;
    $this->print_page_footer('register');
}

sub get_referer ($) {
    my $this    = shift;
    my $referer = $main::path;
    if ( $ENV{'HTTP_REFERER'} ) {
        $referer = $ENV{'HTTP_REFERER'};
        $referer =~ s!^https?://[^/]+!!;
    }
    return $referer;
}

sub print_jump_menu ($) {
    my $this = shift;
    print '<ul class="tools panel" id="auto-register-jump-menu">';
    print '<li>';
    print $cgi::Deco->button( 'return',
        $setting::page{'register'}{'returnButton'},
        $main::referer, '戻る' );
    print '</li>';
    print '<li>';
    print $cgi::Deco->button( 'next',
        $setting::page{'register'}{'nextButton'},
        $main::goal, '次へ', );
    print '</li>';
    print '</ul>';
}

sub print_auto_register_msg ($$$) {
    my ( $this, $title, $msg ) = @_;
    $this->print_page_header('register');
    print '<div class="section" id="auto-register-msg"><h2>' . $title
        . '</h2>';
    print '<div class="body"><p>' . $msg . '</p></div></div>';
    $this->print_jump_menu;
    $this->print_page_footer('register');
    exit;
}

########################################################################
#ジャンプ
sub register_reload ($$;$) {
    my ( $this, $url ) = @_;
    $this->print_http_header;
    print '<html><head><meta http-equiv="Refresh" content="0; URL='
        . $this->escape($url) . '" />';
    print
        '<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" /></head>';
    print '<body onload="location.href=&quot;'
        . $this->escape($url)
        . '&quot;">';

#print '<a href="' . $this->escape($url) . '">自動的に移動しない場合はこちらをクリックしてください。</a>';
    print '</body></html>';
    exit;
}

######################################################################
# Auth DBM チェック
sub get_auth_settings ($$) {
    my ( $this, $htaccess ) = @_;
    return ( 0, 0, 0, 0 ) if !-f $htaccess;
    my $data = $this->read_file( $htaccess, 1 );
    return (
        (   (          defined $data
                    && $data =~ /^Require valid-user$/mi
                    && ( $data =~ /^AuthDBMUserFile[\x09\x0A\x0C\x0D\x20]/mi
                    || $data =~ /^AuthUserFile[\x09\x0A\x0C\x0D\x20]/mi )
            ) ? 1 : 0
        ),
        (   (   defined $data
                    && $data =~ /^AuthDBMUserFile[\x09\x0A\x0C\x0D\x20]/mi
            ) ? 1 : 0
        ),
        (   ( defined $data && $data =~ /^# auto_register_enabled = true$/m )
            ? 1
            : 0
        ),
        (   ( defined $data && $data =~ /^# auto_reissue_enabled = true$/m )
            ? 1
            : 0
        ),
    );
}

######################################################################
# メールアドレス=>アカウント
sub push_account ($$$) {
    my ( $this, $data, $acc ) = @_;
    my @data = split( /,/, ( ( defined $data ) ? $data : '' ) );
    return ( grep( $_ eq $acc, @data ) ) ? $data : join( ',', @data, $acc );
}

sub shift_account ($$$) {
    my ( $this, $data, $acc ) = @_;
    my @data = split( /,/, ( ( defined $data ) ? $data : '' ) );
    @data = grep( $_ ne $acc, @data );
    return join( ',', @data );
}

######################################################################
# メール送信
sub mail_to_user ($$$$$$$$$) {
    my ( $this, $subject, $msg, $mailto, $account, $passwd, $name,
        $expiretime )
        = @_;
    return 0 if !$mailto || !$account;
    my $url = $this->http_my_domain . $main::path;

    if ( !defined $msg || $msg !~ /<account>/i || $msg !~ /<password>/i ) {
        $msg .= <<MSG;
$url
以下の階層にアクセスするときは、下記アカウントおよびパスワードを
入力してください。

────────────────────────────────
アカウント：<account>
パスワード：<password>
────────────────────────────────

MSG
    }
    else {
        $msg =~ s/<url>/$url/ig;
    }
    if ( !defined $subject || !$subject ) {
        $setting::general{'subject'} = 'お知らせ';
    }
    else {
        $subject =~ s/<url>/$url/ig;
    }

    unless ( ref $cgi::SendMail ) {
        require CMONOS::CGI_Lib::SendMail;
        $cgi::SendMail = SendMail->new(
            ( ( defined $env::sendmail )     ? $env::sendmail     : '' ),
            ( ( defined $env::master_email ) ? $env::master_email : '' ),
            '',
            ( ( defined $cgi::input ) ? $cgi::input : '' ),
            1
        );
    }

    my $from = (
        ( $setting::general{'master'} )
        ? ( ( $setting::general{'mastername'} )
            ? $setting::general{'mastername'} . ' <'
                . $setting::general{'master'} . '>'
            : $setting::general{'master'} )
        : $env::master_email
    );

    $passwd
        = ( defined $passwd && $passwd ne '' )
        ? $passwd
        : 'これまでと同じパスワード';
    $name = ( defined $name && $name ne '' ) ? $name : $account;
    $expiretime
        = ( defined $expiretime && $expiretime )
        ? $this->date( $expiretime, 0, 'gmt' . $setting::general{'datetype'} )
        : '無期限';

    $msg =~ s/<name>/$name/ig;
    $msg =~ s/<account>/$account/ig;
    $msg =~ s/<password>/$passwd/ig;
    $msg =~ s/<expire>/$expiretime/ig;

    $subject =~ s/<name>/$name/ig;
    $subject =~ s/<account>/$account/ig;
    $subject =~ s/<password>/$passwd/ig;
    $subject =~ s/<expire>/$expiretime/ig;

    if ($cgi::SendMail->send_mail(
            'To' =>
                ( ( $name ne '' ) ? $name . ' <' . $mailto . '>' : $mailto ),
            'From'    => $from,
            'Subject' => $subject,
            'message' => $msg,
            'info'    => 1
        )
        )
    {
        return 1;
    }
    else {
        push(
            @{ $this->{'dialog'} },
            'メールを送信できませんでした（'
                . $account
                . '）。'
        );
        push( @{ $this->{'dialog'} }, $cgi::SendMail->dialog );
        return 0;
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {

    #設定DB読み込み設定
    $setting::general = 1 if !$setting::general;    #一般設定読み込み
    $setting::deco = 1
        if !$setting::deco;    #テキスト装飾設定読み込み
    $setting::security = 1
        if !$setting::security;    #アクセス権設定読み込み
    $setting::page = 1 if !$setting::page;    #ページ設定読み込み

    #設定種キー設定
    $setting::security_key = 'register'
        if !defined $setting::security_key
        || $setting::security_key ne
        'register';    #デフォルトセキュリティキー
    $setting::key = 'register';

    #クッキー
    $cookie::id_enabled = 1
        if !$cookie::id_enabled;    #ID情報クッキー読み込み

    #デフォルトキー
    $main::k = 'register' if !defined $main::k || !$main::k;

    #必須フォーム
    @setting::form = ( 'm', 'k', 'path', 'referer', 'goal' )
        if !@setting::form;

    #クラス登録
    Common::class_register( 'register', 'AutoRegister' );
}

1;
