package MasterGeneralSetting;
require CMONOS::CGI_Lib::Tools::General;
@ISA = qw(General);
use strict;
use utf8;
no warnings qw(once);
######################################################################
# 一般設定モジュール
# MasterGeneralSetting.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
# 最終更新日：2009-11-09
######################################################################
# 設定チェック項目取得
sub get_sub_setting_items ($) {
    my $this = shift;

    #チェックボックス
    push( @check::checkbox, 'subicon', 'addressProtector' );

    #タグを修正する
    push( @check::tagfix, 'description' );

    #数字チェック
    $check::num{'iconMaxSize'}
        = 'カテゴリーアイコンファイルサイズ制限';
    $check::num{'iconWidthLimit'} = 'カテゴリーアイコン横幅制限';
    $check::num{'iconHeightLimit'}
        = 'カテゴリーアイコン高さ制限';

    $check::imagefile{'og'}   = 'og:image';
    $check::ogimage{'og'}     = 1;
    $check::imagefile{'icon'} = 'アイコン';    #画像の名前
    $check::maxsize{'icon'}
        = ( $setting::general{'iconMaxSize'} )
        ? $setting::general{'iconMaxSize'}
        : 30000;    #ファイルサイズ制限
    $check::widthlimit{'icon'}
        = $setting::general{'iconWidthLimit'};    #ファイル横幅制限
    $check::heightlimit{'icon'}
        = $setting::general{'iconHeightLimit'};    #ファイル高さ制限
}

######################################################################
# 設定環境2
sub is_database_cgi ($) {
    my $this = shift;
    return 1;
}

sub sub_set_preferences ($) {
    my $this = shift;
    $cgi::enctype_multipart = 1;
    $default::home = '.' if !defined $default::home || $default::home eq '';
    $preference::filedir{'og'} = $preference::filedir{'icon'}
        = $default::home;    #画像保存ディレクトリ
}

######################################################################
# サブ設定パネル
sub print_sub_preference_panel ($$) {
    my ( $this, $master ) = @_;

    print
        '<fieldset><legend><label for="data:icon">アイコン</label></legend>';
    $this->print_image_upload_form( 'icon', 'icon', \%main::data );
    print '</fieldset>';

    print '<fieldset><legend>og:image</legend>';
    $this->print_image_upload_form( 'og', 'og', \%main::data );
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:description">説明</label></legend>';
    print
        '<div class="hints">インデックスに表示するページの説明を設定します。ページが更新されるまで設定変更は表示に反映されません。</div>';
    print
        '<textarea name="data:description" id="data:description" cols="56" rows="6" wrap="soft" title="インデックスに表示するページの説明を設定します。">'
        . $main::data{'description'}
        . '</textarea>';
    print
        '<div class="extra-field"><label><input type="checkbox" name="data:addressProtector" value="1"'
        . (
        (   defined $main::data{'addressProtector'}
                && $main::data{'addressProtector'}
        ) ? ' checked="checked"' : ''
        )
        . ' /> メールアドレスを JavaScript + CSS + HTML エンティティで保護する</label></div>';
    print
        '<div class="extra-field"><label><input type="checkbox" name="rebuild" value="1" /> 関連ページをすべて書き換える(処理が完了するまで時間がかかる場合があります)</label></div>';
    print '</fieldset>';

    print '<fieldset><legend>カテゴリーアイコン設定</legend>';

    print '<table class="field-table"><tbody>';

    print '<tr>';
    print
        '<td class="field-name"><span class="datatitle">カテゴリーアイコン</span></td>';
    print
        '<td class="field"><label><input type="checkbox" name="data:subicon" id="data:subicon" value="1"'
        . ( ( defined $main::data{'subicon'} && $main::data{'subicon'} )
        ? ' checked="checked"'
        : '' )
        . ' /> カテゴリーごとにアイコンを作成できるようにする</label></td>';
    print '</tr>';

    print '<tr>';
    print
        '<td class="field-name"><label for="data:iconMaxSize">カテゴリーアイコンファイルサイズ制限</label></td>';
    print
        '<td class="field"><input type="number" name="data:iconMaxSize" id="data:iconMaxSize" value="'
        . $this->escape( $main::data{'iconMaxSize'} )
        . '" size="15" /> <span class="unit">byte</span>';
    print
        '<div class="hints">設定が無い場合、30,000byte（30kB）に設定されます。</div>';
    print '</td>';
    print '</tr>';

    print '<tr>';
    print
        '<td class="field-name"><label for="data:iconWidthLimit">カテゴリーアイコン横幅制限</label></td>';
    print
        '<td class="field"><input type="number" name="data:iconWidthLimit" id="data:iconWidthLimit" value="'
        . $this->escape( $main::data{'iconWidthLimit'} )
        . '" size="4" /> <span class="unit">ピクセル</span></td>';
    print '</tr>';

    print '<tr>';
    print
        '<td class="field-name"><label for="data:iconHeightLimit">カテゴリーアイコン高さ制限</label></td>';
    print
        '<td class="field"><input type="number" name="data:iconHeightLimit" id="data:iconHeightLimit" value="'
        . $this->escape( $main::data{'iconHeightLimit'} )
        . '" size="4" /> <span class="unit">ピクセル</span></td>';
    print '</tr>';

    print '</tbody></table>';

    print '</fieldset>';
}

######################################################################
# サブ設定チェック
sub _is_valid_setting ($) {
    my $this = shift;
    if ($main::rebuild) {
        require CMONOS::CGI_Lib::ProgressBar;
        $cgi::ProgressBar = ProgressBar->new;
        $this->set_javascript( $cgi::ProgressBar->javascript );
    }
    return;
}

######################################################################
# アイコンと説明をリフレッシュ
sub print_progress_bar ($) {
    my $this = shift;
    $cgi::ProgressBar->print_progress_bar('ファイル') if $main::rebuild;
    return;
}

sub site_update ($) {
    my $this = shift;
    $this->refresh_icon_and_description if $main::rebuild;
    return;
}

sub refresh_icon_and_description ($$) {
    my $this = shift;
    return 1 if !defined $main::data{'description'};
    my @categories  = ();
    my @indexes     = ();
    my @top_indexes = ();
    push( @indexes, $default::external_index )
        if $default::external_index && -f $default::external_index;
    push( @indexes, $default::external_navi )
        if $default::external_navi && -f $default::external_navi;
    push( @indexes, $default::external_navi2 )
        if $default::external_navi2 && -f $default::external_navi2;
    my @articles       = ();
    my $icon_attribute = '';
    my $description    = '';
    my $index_id
        = ( defined $default::id && $default::id ne '' )
        ? $default::id
        : 'homepage';
    my $metaindex_target = '';

    if ($default::id) {
        $metaindex_target = $default::id;
        $metaindex_target =~ s/^_+//;
        $metaindex_target =~ s/_/-/g;
    }
    if ( opendir( DIR, $default::home ) ) {
        while ( my $file = readdir(DIR) ) {
            next if $file =~ /^\./;
            if ( -d $default::home . '/' . $file ) {
                push( @categories, $default::home . '/' . $file );
            }
            elsif ( -f $default::home . '/' . $file
                && $file =~ /\.(s?html?|ssi)$/i )
            {
                push( @top_indexes, $default::home . '/' . $file );
            }
        }
        closedir(DIR);
    }
    foreach my $dir (@categories) {
        if ( opendir( DIR, $dir ) ) {
            my @article_folders = ();
            my $history_dir     = $dir . '/history';
            while ( my $file = readdir(DIR) ) {
                next if $file =~ /^\./ && $file ne 'calendar';
                if ( -d $dir . '/' . $file ) {
                    push( @article_folders, $dir . '/' . $file );
                }
                elsif ( -f $dir . '/' . $file
                    && $file =~ /\.(s?html?|ssi)$/i )
                {
                    push( @indexes, $dir . '/' . $file );
                }
            }
            closedir(DIR);
            if (@article_folders) {
                foreach my $d (@article_folders) {
                    if ( opendir( DIR, $d ) ) {
                        if ( $default::date && $d eq $history_dir ) {
                            push(
                                @indexes,
                                map( $d . '/' . $_,
                                    grep( $_ =~ /^([0-9]+|index)\.s?html?$/i
                                            && -f $d . '/' . $_,
                                        readdir(DIR) ) )
                            );
                        }
                        else {
                            push(
                                @articles,
                                map( $d . '/' . $_,
                                    grep( $_ =~ /^([0-9]+|index)\.s?html?$/i
                                            && -f $d . '/' . $_,
                                        readdir(DIR) ) )
                            );
                        }
                        closedir(DIR);
                    }
                }
            }
        }
    }
    if ( $main::data{'icon'} ) {
        $icon_attribute
            = ' width="'
            . $main::data{'iconwidth'}
            . '" height="'
            . $main::data{'iconheight'}
            . '" alt="'
            . $main::data{'cginame'}
            . '" border="0"';
    }
    elsif ( $setting::general{'icon'}
        && -f $default::home . '/' . $setting::general{'icon'} )
    {
        $icon_attribute
            = ' width="'
            . $setting::general{'iconwidth'}
            . '" height="'
            . $setting::general{'iconheight'}
            . '" alt="'
            . $setting::general{'cginame'}
            . '" border="0"';
    }
    $description
        = $this->autolink(
        $this->tag_fix( $this->autolist( $main::data{'description'} ), 2 ),
        0, $main::data{'addressProtector'} )
        if $main::data{'description'};
    my $total = @top_indexes + @indexes + @articles;
    my $count = 0;
    if (@top_indexes) {
        foreach my $path (@top_indexes) {
            my ( $data, $icon, $done );
            $done = 0;
            $count++;
            $icon = '<img src="'
                . $this->base_change(
                $default::home . '/' . $main::data{'icon'},
                $path, ( ( $path =~ /\.ssi$/ ) ? 'abs' : '' ) )
                . '"'
                . $icon_attribute . ' />'
                if $icon_attribute;
            $data = $this->read_file( $path, 1, 1 );
            $data
                =~ s/(<!--index(?![0-9a-zA-Z_])[^>]*id="$index_id"[^>]*-->)(.*?)(<!--\/index-->)/"\x03" . $1 . $this->_index_replace($2,$icon,$description,\$done) . $3 . "\x02"/sge;
            $data
                =~ s/(<!--metaindex(?![0-9a-zA-Z_])[^>]*-->)(.*?)(<!--\/metaindex-->)/"\x03" . $1 . $this->_metaindex_replace($2,$metaindex_target,$icon,$description,\$done) . $3 . "\x02"/sge
                if $metaindex_target;
            $data
                =~ s/(?:^|\x02)(.*?)(?:\x03|$)/$this->_replace_icon_and_description($1,$icon,$description,\$done)/sge;
            $data =~ s/[\x02\x03]//sg;

            if ($done) {
                $this->write_file( $path, \$data, 1, $env::file_mode, 1 );
                $cgi::ProgressBar->update( $count, $total, 20 );
            }
        }
    }
    if (@indexes) {
        foreach my $path (@indexes) {
            my ( $data, $icon, $done );
            $done = 0;
            $count++;
            $icon = '<img src="'
                . $this->base_change(
                $default::home . '/' . $main::data{'icon'},
                $path, ( ( $path =~ /\.ssi$/ ) ? 'abs' : '' ) )
                . '"'
                . $icon_attribute . ' />'
                if $icon_attribute;
            $data = $this->read_file( $path, 1, 1 );
            $data
                =~ s/(<!--index(?![0-9a-zA-Z_])[^>]*id="$index_id"[^>]*-->)(.*?)(<!--\/index-->)/$1 . $this->_index_replace($2,$icon,$description,\$done) . $3/sge;
            $data
                =~ s/(<!--metaindex(?![0-9a-zA-Z_])[^>]*-->)(.*?)(<!--\/metaindex-->)/$1 . $this->_metaindex_replace($2,$metaindex_target,$icon,$description,\$done) . $3/sge
                if $metaindex_target;
            if ($done) {
                $this->write_file( $path, \$data, 1, $env::file_mode, 1 );
                $cgi::ProgressBar->update( $count, $total, 20 );
            }
        }
    }
    if (@articles) {
        foreach my $path (@articles) {
            my ( $data, $icon, $done );
            $done = 0;
            $count++;
            $icon = '<img src="'
                . $this->base_change(
                $default::home . '/' . $main::data{'icon'},
                $path, ( ( $path =~ /\.ssi$/ ) ? 'abs' : '' ) )
                . '"'
                . $icon_attribute . ' />'
                if $icon_attribute;
            $data = $this->read_file( $path, 1, 1 );
            $data = $this->_replace_icon_and_description( $data, $icon,
                $description, \$done, 'master', 'top' );
            if ($done) {
                $this->write_file( $path, \$data, 1, $env::file_mode, 1 );
                $cgi::ProgressBar->update( $count, $total, 20 );
            }
        }
    }
    $cgi::ProgressBar->update( $total, $total );
    $cgi::ProgressBar->remove;
    return 1;
}

sub _index_replace ($$$$$) {
    my ( $this, $data, $icon, $description, $done_ref ) = @_;
    my $subindex
        = ( $data
            =~ s/(<!--subindex(?![0-9a-zA-Z_])[^>]*-->.*<!--\/subindex-->)/<!--subindex--><!--\/subindex-->/sg
        )
        ? $this->_replace_icon_and_description( $1, $icon, $description,
        $done_ref, 'master', 'top' )
        : '';
    $data = $this->_replace_icon_and_description( $data, $icon, $description,
        $done_ref );
    $data =~ s/<!--subindex--><!--\/subindex-->/$subindex/sg;
    return $data;
}

sub _metaindex_replace ($$$$$$) {
    my ( $this, $data, $target, $icon, $description, $done_ref ) = @_;
    $data
        =~ s/(<!--article(?![0-9a-zA-Z_])[^>]*id="$target-[^"]+"[^>]*-->.*?<!--\/article-->)/$this->_replace_icon_and_description($1,$icon,$description,$done_ref,'master','top')/sge;
    return $data;
}

sub _replace_icon_and_description ($$$$;$$) {
    my ( $this, $data, $icon, $description, $done_ref, $prefix, $prefix2 )
        = @_;
    $prefix = '' if !defined $prefix;
    $$done_ref = 1
        if $data
        =~ s/<\!--(${prefix}icon(?![0-9a-zA-Z_]))([^>]*)-->(.*?)<\!--\/\1(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$icon,1)/gies;
    $$done_ref = 1
        if $data
        =~ s/<\!--(${prefix}description(?![0-9a-zA-Z_]))([^>]*)-->(.*?)<\!--\/\1(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$description,1)/gies;
    if ($prefix2) {
        $$done_ref = 1
            if $data
            =~ s/<\!--(${prefix2}icon(?![0-9a-zA-Z_]))([^>]*)-->(.*?)<\!--\/\1(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$icon,1)/gies;
        $$done_ref = 1
            if $data
            =~ s/<\!--(${prefix2}description(?![0-9a-zA-Z_]))([^>]*)-->(.*?)<\!--\/\1(?![0-9a-zA-Z_])[^>]*-->/$this->item_tag_replace($1,$2,$3,$description,1)/gies;
    }
    return $data;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ( $default::subdivided && ( !defined $main::c || $main::c eq '' ) ) {

        #クラス登録
        Common::class_register( '0000', 'MasterGeneralSetting' );

        #ツール登録
        $setting::jump_menu{'0000'} = '一般設定';
    }
}

1;
