package Importer;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use File::Path;
use Fcntl qw(:DEFAULT);
######################################################################
# テキスト読み込みモジュール
# Importer.pm ver1.1.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-07-10
######################################################################
# 実行
sub run ($) {
    my $this          = shift;
    my $file_uploaded = 0;
    my $temp_dir = $env::clipboard . '/temp' . ( $default::id || '_hpm' );
    if ( -d $temp_dir ) {
        if ( -f $temp_dir . '/file' ) {
            $file_uploaded = 1;
        }
        else {
            rmtree $temp_dir;
        }
    }

    if ($file_uploaded) {
        if ($main::cancel) {
            rmtree $temp_dir;
            $file_uploaded = 0;
        }

    }
    elsif ( defined $main::file && $main::file ne '' ) {
        $main::type
            = ( $main::file =~ /\.csv$/i ) ? 'csv'
            : ( $main::file =~ /\.tsv$/i ) ? 'tsv'
            : ( defined $main::type )      ? $main::type
            :                                'csv';

        #大きすぎるファイル
        if ( $cgi::input->get_posted_file_size_from_param('file') > 8000000 )
        {
            push(
                @{ $this->{'dialog'} },
                'ファイルの読み込みに失敗しました。ファイルサイズが大きすぎます。'
            );

        }
        elsif ( $main::type && $main::type eq 'mt' ) {
            my $data = $cgi::input->get_posted_file_data('file');
            $data = $this->jcode_convert( \$data, 'utf8' );
            $this->parse_mt( \$data );
            push(
                @{ $this->{'dialog'} },
                'MT形式ファイルを読み込みました。結果を確認してください。'
            );
            undef $main::file;
            undef $main::type;
        }
        else {
            $this->set_dir($temp_dir);
            $this->write_file( $temp_dir . '/file',
                $cgi::input->get_posted_file_data('file'), 1 );
            if ( -s $temp_dir . '/file' ) {
                $file_uploaded = 1;
                $this->write_file( $temp_dir . '/type', $main::type, 1 );
            }
            else {
                rmtree $temp_dir;
            }
        }
        $cgi::input->clear_clipboard;
    }

    #書き込み
    if ($file_uploaded) {
        my $is_valid = 1;

        require CMONOS::CGI_Lib::Table;
        if ( $is_valid && %main::collate ) {

            foreach my $i ( keys %main::collate ) {
                $this->scalar_to_array( \$main::collate{$i} );
            }

            #テキストをHTMLファイルに変換
            my $import = Table->new;
            my $data   = $this->read_file( $temp_dir . '/file' );
            my $type   = $this->read_file( $temp_dir . '/type', 1 );
            $data =~ s/[\x0A\x0D]+\Z//;
            $import->sv_to_table( \$data, 'no', $type, $this );
            push(
                @{ $this->{'dialog'} },
                'ファイルを読み込みました。結果を確認してください。'
            );
            my %general;
            eval 'use ' . $env::dbm_file . ';';
            tie(%general, $env::dbm_file,
                $default::preference_dir . '/general',
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );

            if ( ref tied %general ) {
                $general{'importSettings'}
                    = $this->set_query( 'url', ['collate'], 1 );
            }
            untie %general;
            rmtree $temp_dir;

            #読み込み設定
        }
        else {
            $this->use_field( 'field',
                $default::field_pref_dir . '/inputfield',
                2, 0, $setting::field_tag_enabled )
                if !%field::obj;
            $this->use_field( 'info', $default::field_pref_dir . '/infofield',
                2, 0, $setting::field_tag_enabled, 'info' )
                if !%info::obj;
            $this->url_decode( $setting::general{'importSettings'}, 'main' )
                if $setting::general{'importSettings'};
            my @sample_row = $this->get_first_row(
                $this->read_file( $temp_dir . '/file' ),
                $this->read_file( $temp_dir . '/type', 1 )
            );

            #ページヘッダ
            $this->jump_menu('tools');
            $this->print_page_header('tools');

            print '<h1>ファイル読み込み</h1>';
            print $this->form_start(
                'key'   => 'tools',
                'name'  => 'input',
                'multi' => 1,
                'onsubmit' =>
                    'return cmonos.form.item(this).isNotSubmitted()',
                'required' => \@setting::form
            );

            print '<fieldset><legend>読み込み設定</legend>';
            my $table = Table->new;
            $table->set(
                'tableclass' => 'tooltable',
                'trclass'    => 'odd,even',
                'class0'     => 'odd,even',
                'style0' =>
                    'text-align:center,text-align:left,text-align:center',
                'thenabled' => 'true'
            );
            print $table->table;

            #ヘッダ
            print $table->tr;
            print $table->td('列番号');
            print $table->td('列の内容');
            print $table->td('読み込み先フィールド');
            print $table->_tr;
            my $no = 1;
            foreach my $i (@sample_row) {
                my $iid = 'i' . ( $info::ORDER{$no} || $no );
                my $fid = $field::ORDER{$no} || $no;
                print $table->tr;
                print $table->td($no);
                print $table->td( $this->cut_string( $i, 32, 'no' ) );
                print $table->td(
                    $this->import_field_menu(
                        $no,
                        (     ( defined $main::collate{$no} )
                            ? ( ref $main::collate{$no} )
                                    ? $main::collate{$no}
                                    : [ $main::collate{$no} ]
                            : [ $iid, $fid ]
                        )
                    )
                );
                print $table->_tr;
                $no++;
            }
            print $table->_table;
            print '</fieldset>';

            print
                '<fieldset class="submit"><input type="submit" value="読み込み" /></fieldset>';

            print
                '<fieldset class="delete"><input type="submit" name="cancel=1" value="アップロードしたファイルを削除して読み込みを中止する" /></fieldset>';
            print '</form>';

            #ページフッタ
            $this->print_page_footer('tools');
            return;
        }
    }
    elsif ($main::type) {
        push(
            @{ $this->{'dialog'} },
            'ファイルの読み込みに失敗しました。ファイルがアップロードされていません。'
        );
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>ファイル読み込み</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'multi'    => 1,
        'name'     => 'input',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );

    print
        '<fieldset><table class="field-table"><tbody><tr><td class="field-name"><label for="file">ファイル選択</label></td>';
    print '<td class="field"><input type="file" name="file" id="file" />';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="type">ファイル形式</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        'type', '', '', $main::type,
        ''    => '',
        'csv' => 'カンマ区切りテキスト（Excel形式）',
        'tsv' => 'タブ区切りテキスト（FileMaker形式）',
        'mt' =>
            'MT形式 (Movable Type のブログ記事インポートフォーマット)',
    );
    print '</td></tr>';
    print '</tbody></table></fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="読み込み" /></fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');

}

######################################################################
# 最初の行を取り出す
sub get_first_row ($$$) {
    my ( $this, $data, $type ) = @_;
    my @data = ();
    my $pat  = '';
    $data =~ s/[\x0D\x0A]+$//;
    $type = 'tsv' if ( !defined $type || $type eq '' ) && $data =~ /\x09/;

    if ( $type && $type eq 'tsv' ) {
        $data =~ s/\x20{3}/\x09/g if $data !~ /\x09/;
        $pat = '([^\x09\x0D\x0A]*)(\x09|\x0D\x0A|\x0D|\x0A|\Z)';
    }
    else {
        $pat = '([^,\x0D\x0A]*)(,|\x0D\x0A|\x0D|\x0A|\Z)';
    }

    my $count = 0;
    my $temp  = '';
    while ( $data =~ /\G($pat)/gs ) {
        next if $1 eq '';
        my ( $v, $s ) = ( $2, $3 );
        if ( $type && $type ne 'tsv' ) {
            $count += ( $v =~ tr/"/"/ )
                if ( defined $v && $v =~ /^"/ )
                || ( defined $temp && $temp =~ /^"/ );
            if ( $count && $count % 2 > 0 ) {
                $temp .= $v . $s;
                next;
            }
            else {
                $v = $temp . $v if defined $temp;
                if ( $v =~ s/^"// ) {
                    $v =~ s/"$//;
                    $v =~ s/""/"/g;
                }
                undef $temp;
                $count = 0;
            }
        }
        else {
            $v =~ s/\x0B/\x0A/gs;
            $v =~ s/\&#09;/\x09/gs;
        }
        $v =~ s/\x0D\x0A/\x0A/gs;
        $v =~ s/\x0D/\x0A/gs;
        push( @data, $v );
        if ( $s eq '' || $s =~ /^(\x0D\x0A|\x0D|\x0A)$/ ) {
            last;
        }
    }
    return @data;
}

sub import_field_menu ($$$$) {
    my ( $this, $no, $id ) = @_;
    $id = ( defined $id ) ? [$id] : [] if !ref $id;
    my $selector
        = '<select name="collate:'
        . $no
        . '" id="collate:'
        . $no
        . '" multiple="multiple" size="8">';
    foreach my $i ( sort { $a <=> $b } keys %info::ORDER ) {
        my $fid = $info::ORDER{$i};
        my $v   = 'i:' . $fid;
        $selector
            .= '<option value="'
            . $v . '"'
            . ( ( ref $id && grep( $_ eq $v, @$id ) )
            ? ' selected="selected"'
            : '' )
            . '>ページ情報フィールド:'
            . $this->cut_string( $info::obj{$fid}->name, 32, 'no' )
            . '</option>'
            if ref $info::obj{$fid};
    }
    $selector .= '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        my $fid = $field::ORDER{$i};
        $selector
            .= '<option value="'
            . $fid . '"'
            . ( ( ref $id && grep( $_ eq $fid, @$id ) )
            ? ' selected="selected"'
            : '' )
            . '>入力フィールド:'
            . $this->cut_string( $field::obj{$fid}->name, 32, 'no' )
            . '</option>'
            if ref $field::obj{$fid};
    }
    $selector
        .= '<option value=""></option><option value=""'
        . ( ( !$id ) ? ' selected="selected"' : '' )
        . '>&lt;なし&gt;</option></select>';
    return $selector;
}

######################################################################
# MT形式
sub parse_mt ($$) {
    my ( $this, $data ) = @_;
    use Time::Local;
    $$data =~ s/\x0D\x0A/\x0A/gs;
    $$data =~ s/\x0D/\x0A/gs;
    $this->table;
    %main::collate = (
        '1' => [ $field::for{'title'} ],
        '2' => [ $field::for{'description'} ],
    );
    my ( $comment_field, $info_comment, $ping_field, $info_ping );
    foreach my $o ( sort { $a <=> $b } keys %field::ORDER ) {
        my $id = $field::ORDER{$o};
        if ( ref $field::obj{$id} ) {
            if ( !defined $comment_field
                && $field::obj{$id}->{'class'} eq 'ug_comment' )
            {
                $comment_field = $id;
            }
            elsif ( !defined $ping_field
                && $field::obj{$id}->{'class'} eq 'ug_trackback' )
            {
                $ping_field = $id;
            }
        }
    }
    if ( !defined $comment_field || !defined $ping_field ) {
        foreach my $o ( sort { $a <=> $b } keys %info::ORDER ) {
            my $id = $info::ORDER{$o};
            if ( ref $info::obj{$id} ) {
                if ( !defined $comment_field
                    && $info::obj{$id}->{'class'} eq 'ug_comment' )
                {
                    $comment_field = $id;
                    $info_comment  = 1;
                }
                elsif ( !defined $ping_field
                    && $info::obj{$id}->{'class'} eq 'ug_trackback' )
                {
                    $ping_field = $id;
                    $info_ping  = 1;
                }
            }
        }
    }

    while ( $$data =~ s/^(.*?)\x0A--------(?:\x0A|$)//s ) {
        my $article = $1;
        my ($date,       $basename,   $sectiontitle,
            $body,       $extended,   $comments,
            $trackbacks, $comment_id, $tb_id
        );
        $comment_id = $tb_id = 1;
        while ( $article =~ s/^(.*?)\x0A-----(?:\x0A|$)//s ) {
            my $fields = $1;
            if ( !defined $date ) {
                $date
                    = ( $fields =~ /^DATE:[\x09\x0A\x0C\x0D\x20]*(.*?)$/m )
                    ? $1
                    : $this->time_id;
                $basename
                    = (
                    $fields =~ /^BASENAME:[\x09\x0A\x0C\x0D\x20]*(.*?)$/m )
                    ? $1
                    : $this->time_id;
                $sectiontitle
                    = ( $fields =~ /^TITLE:[\x09\x0A\x0C\x0D\x20]*(.*?)$/m )
                    ? $1
                    : '';
            }
            elsif ( $fields =~ s/^BODY:[\x09\x0A\x0C\x0D\x20]+//s ) {
                $body = $fields;
            }
            elsif ( $fields =~ s/^EXTENDED BODY:[\x09\x0A\x0C\x0D\x20]+//s ) {
                $extended = $fields;
            }
            elsif ( $fields =~ s/^COMMENT:[\x09\x0A\x0C\x0D\x20]+//s ) {
                my ( $author, $email, $url, $ip, $dt );
                $author
                    = (
                    $fields =~ s/^AUTHOR:[\x09\x0A\x0C\x0D\x20]*(.*?)$//m )
                    ? $1
                    : '';
                $email
                    = ( $fields =~ s/^EMAIL:[\x09\x0A\x0C\x0D\x20]*(.*?)$//m )
                    ? $1
                    : '';
                $url
                    = ( $fields =~ s/^URL:[\x09\x0A\x0C\x0D\x20]*(.*?)$//m )
                    ? $1
                    : '';
                $url = $1 if $url =~ / href=["'](.*?)["']/i;
                $ip
                    = ( $fields =~ s/^IP:[\x09\x0A\x0C\x0D\x20]*(.*?)$//m )
                    ? $1
                    : '';
                $dt
                    = ( $fields =~ s/^DATE:[\x09\x0A\x0C\x0D\x20]*(.*?)$//m )
                    ? $1
                    : '';
                if ( $dt
                    =~ /^([0-9]{2})\/([0-9]{2})\/([0-9]{4})[\x09\x0A\x0C\x0D\x20]+([0-9]{2}):([0-9]{2}):([0-9]{2})[\x09\x0A\x0C\x0D\x20]*([PA]M)?/
                    )
                {
                    $dt
                        = timelocal( $6 + 0, $5 + 0,
                        $4 + ( ( $7 && $7 eq 'PM' ) ? 12 : 0 ),
                        $2 + 0, $1 - 1, $3 );
                }
                $comments
                    .= $this->svjoin( "\x09", $comment_id++, $dt, 0, $fields,
                    $author, '', $email, 1, $url, '', '', '', $ip, '' )
                    . "\x0A";
            }
            elsif ( $fields =~ s/^PING:[\x09\x0A\x0C\x0D\x20]+//s ) {
                my ( $title, $name, $url, $ip, $dt );
                $title
                    = ( $fields =~ s/^TITLE:[\x09\x0A\x0C\x0D\x20]*(.*?)$//m )
                    ? $1
                    : '';
                $name
                    = (
                    $fields =~ s/^BLOG NAME:[\x09\x0A\x0C\x0D\x20]*(.*?)$//m )
                    ? $1
                    : '';
                $url
                    = ( $fields =~ s/^URL:[\x09\x0A\x0C\x0D\x20]*(.*?)$//m )
                    ? $1
                    : '';
                $url = $1 if $url =~ / href=["'](.*?)["']/i;
                $ip
                    = ( $fields =~ s/^IP:[\x09\x0A\x0C\x0D\x20]*(.*?)$//m )
                    ? $1
                    : '';
                $dt
                    = ( $fields =~ s/^DATE:[\x09\x0A\x0C\x0D\x20]*(.*?)$//m )
                    ? $1
                    : '';
                if ( $dt
                    =~ /^([0-9]{2})\/([0-9]{2})\/([0-9]{4})[\x09\x0A\x0C\x0D\x20]+([0-9]{2}):([0-9]{2}):([0-9]{2})[\x09\x0A\x0C\x0D\x20]*([PA]M)?/
                    )
                {
                    $dt
                        = timelocal( $6 + 0, $5 + 0,
                        $4 + ( ( $7 && $7 eq 'PM' ) ? 12 : 0 ),
                        $2 + 0, $1 + 0, $3 );
                }
                $trackbacks .= $this->svjoin(
                    "\x09",  $tb_id++, $dt, 0, $url, $title,
                    $fields, $name,    $ip
                ) . "\x0A";
            }
        }
        $body .= $extended if $extended;
        $this->tr;
        $this->td( $sectiontitle || '' );
        $this->td( $body         || '' );
        $this->_tr(1);
        if ( $comments && defined $comment_field ) {
            my $file_dir
                = ($info_comment)
                ? $default::file . '/i'
                : $this->{'filedir'};
            $this->set_dir($file_dir);
            $this->write_file( $file_dir . '/' . $comment_field . '.dat',
                \$comments, 1 );
        }
        if ( $trackbacks && defined $ping_field ) {
            my $file_dir
                = ($info_ping) ? $default::file . '/i' : $this->{'filedir'};
            $this->set_dir($file_dir);
            $this->write_file( $file_dir . '/' . $ping_field . '.dat',
                \$trackbacks, 1 );
        }
        $this->_tr;
    }
    $this->_table;
    return 1;
}

######################################################################
# 以下はTable.pmでの区切り文字テキスト分解に関係する処理
# TABLE
sub table {
    no strict "subs";
    my $this = shift;
    use File::Path;
    require CMONOS::CGI_Lib::Template;
    require CMONOS::HomePageMakerLite::Editor;
    $this->{'field_editor'} = {};
    bless $this->{'field_editor'}, Editor;
    $this->{'info_editor'} = {};
    bless $this->{'info_editor'}, Editor;

    $setting::field                = 2;
    $setting::field_style_disabled = 0;
    $setting::field_tag_enabled    = 'tag';

    #フィールド設定読み込み
    $this->use_field( 'field', $default::field_pref_dir . '/inputfield',
        2, 0, 'tag' )
        if !%field::obj;
    $this->use_field( 'info', $default::field_pref_dir . '/infofield',
        2, 0, $setting::field_tag_enabled, 'info' )
        if !%info::obj;

    #ページ設定読み込み
    $this->import_db( $default::preference_dir . '/page', \%setting::page,
        1 );

    $main::i = 1;
    $this->{'info_editor'}->mk_template_obj;
    $main::i = 0;
    $this->{'field_editor'}->mk_template_obj;

    #必須項目のデフォルト
    $this->{'required_field'} = {};
    while ( my ( $id, $field ) = each %field::obj ) {
        $this->{'required_field'}{$id} = ( $field->{'required'} ) ? 1 : 0;
    }
    $this->{'required_info'} = {};
    while ( my ( $id, $field ) = each %info::obj ) {
        $this->{'required_info'}{$id} = ( $field->{'required'} ) ? 1 : 0;
    }
    return '';
}

######################################################################
# _TABLE
sub _table {
    my $this = shift;
    delete $this->{'col'};
    delete $this->{'row'};
    undef %{ $this->{'field_editor'} };
    delete $this->{'field_editor'};
    undef %{ $this->{'info_editor'} };
    delete $this->{'info_editor'};
    delete $this->{'field_data'};
    delete $this->{'info_data'};
    delete $this->{'is_valid'};
    delete $this->{'filedir'};
    $this->clear_article_info;
    return '';
}

######################################################################
# TR
sub tr {
    my $this = shift;
    $this->{'col'} = 1;
    $this->{'row'}++;
    $this->{'is_valid'}   = 1;
    $this->{'field_data'} = {};
    $this->{'info_data'}  = {};
    $this->clear_article_info;
    $main::w        = 1;
    $main::i        = 0;
    $main::feed_rss = 1 if $setting::general{'rssChecked'};

    #必須項目のデフォルト
    while ( my ( $id, $required ) = each %{ $this->{'required_field'} } ) {
        $field::obj{$id}->{'required'} = $required;
    }
    while ( my ( $id, $required ) = each %{ $this->{'required_info'} } ) {
        $info::obj{$id}->{'required'} = $required;
    }

    return '';
}

sub clear_article_info {
    my $this = shift;
    undef $main::chap;
    undef $main::sect;
    undef $main::refresh;
    undef $main::pending;
    undef $cgi::output;
    undef $cgi::output_option;
    undef $main::w;
    undef $main::feed_rss;
}

######################################################################
# _TR
sub _tr {
    my ( $this, $get_n ) = @_;
    if ( !$this->{'is_valid'} ) {
        push(
            @{ $this->{'dialog'} },
            '読み込みに失敗しました('
                . $this->{'row'}
                . '行目)。'
        );
        return '';
    }

    $this->{'field_editor'}->{'keep_mtime'}
        = $this->{'info_editor'}->{'keep_mtime'}
        = ( $setting::general{'keepMtimeChecked'} ) ? 1 : 0;
    my $article = $this->{'field_editor'}->read_html_file;
    $main::chap         = '';
    $main::sect         = '';
    $cgi::output        = $default::home;
    $cgi::output_option = "";
    $this->{'info_editor'}->{'template'}->scan( \$article );
    $this->{'field_editor'}->{'template'}
        ->scan( \$article, \$main::chap, \$main::sect );

    if ( $main::chap && $main::chap > 99 ) {
        $main::chap = 99;
    }
    if ( $main::sect && $main::sect > 99 ) {
        $main::sect = 99;
    }

    #アップロードファイル保存先
    $this->{'field_editor'}->{'filedir'} = $this->{'filedir'}
        = $default::file . '/' . $main::chap . '/' . $main::sect;
    $this->{'info_editor'}->{'filedir'} = $default::file . '/i';

    if ( %{ $this->{'info_data'} } ) {
        local ( $main::i, $main::chap, $main::sect, %main::data,
            $cgi::output, $cgi::output_option );
        %main::data = %{ $this->{'info_data'} };
        $main::i    = 1;
        $this->{'info_editor'}->field_settle_up;
        $this->{'info_editor'}->page_info_update($article);
        $article = $this->{'field_editor'}->read_html_file;
    }

    {
        local %main::data = %{ $this->{'field_data'} };
        $this->{'field_editor'}->field_settle_up;
        $this->{'field_editor'}
            ->section_update( $article, $main::chap, $main::sect );
    }
    return '';
}

######################################################################
# TD
sub td {
    my ( $this, $data ) = @_;
    if ( ref $main::collate{ $this->{'col'} } ) {
        my $n = 0;
        foreach my $i ( @{ $main::collate{ $this->{'col'} } } ) {
            if ($i) {
                my $id = $i;
                local %main::data = ();
                if ( $id =~ s/^i:// ) {
                    if ( ref $info::obj{$id} ) {
                        if (   $info::obj{$id}->{'class'} eq 'time_date'
                            && $data =~ /[^0-9]/s )
                        {
                            $data
                                = $this->time_id( $this->date_to_gmt($data) );
                        }
                        %{ $main::data{$id} }
                            = $info::obj{$id}->elements($data);
                        if ( $info::obj{$id}->is_valid_data ) {
                            %{ $this->{'info_data'}{$id} }
                                = %{ $main::data{$id} };
                        }
                        else {
                            $this->{'is_valid'} = 0;
                        }
                    }
                    else {
                        $main::data{$id}{'str'} = $data;
                    }
                }
                else {
                    if ( ref $field::obj{$id} ) {
                        if (   $field::obj{$id}->{'class'} eq 'time_date'
                            && $data =~ /[^0-9]/s )
                        {
                            $data
                                = $this->time_id( $this->date_to_gmt($data) );
                        }
                        %{ $main::data{$id} }
                            = $field::obj{$id}->elements($data);
                        if ( $field::obj{$id}->is_valid_data ) {
                            %{ $this->{'field_data'}{$id} }
                                = %{ $main::data{$id} };
                        }
                        else {
                            $this->{'is_valid'} = 0;
                        }
                    }
                    else {
                        $main::data{$id}{'str'} = $data;
                    }
                }
            }
        }
    }
    $this->{'col'}++;
    return '';
}

sub get_setting ($;$$$$$) {
    my ( $this, $table, $type, $info, $op, $setting ) = @_;
    return '';
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {
        #クラス登録
        Common::class_register( 'h210', 'Importer' );

        #メニュー登録
        $setting::jump_menu{'h210'} = '読み込み'
            if !defined $setting::jump_menu{'H210'};
    }
}

1;
