package RgCommon;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# パスワード登録共通モジュール
# RgCommon.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2009-04-04
######################################################################
# ID情報取得
sub id_info ($$) {
    my ( $this, $email ) = @_;
    my $pswd;
    $this->use_register if !ref $cgi::Register;
    $pswd = $cgi::Register->read_crypt($email)
        if defined $email && $email ne '';
    return (
        $this->is_secure( 'pswd', 1 ),
        ( ( defined $pswd && $pswd ne '' ) ? 1 : 0 ),
        (   (          defined $pswd
                    && $pswd ne ''
                    && $cgi::Register->is_identifier( $email, $pswd )
            ) ? $pswd : ''
        ),
        (   (          $setting::general{'reregistry'}
                    && defined $pswd
                    && $pswd ne ''
            ) ? 1 : 0
        ),
    );
}

sub endpoint_normalize ($$) {
    my ( $this, $url ) = @_;
    if ( !$url ) {
        return '';
    }
    elsif ( $url !~ m!://! ) {
        return 'https://' . $url;
    }
    $url =~ s!^[0-9a-zA-Z_]+://!https://!gi;
    $url =~ s/#.*$//;
    return $url;
}

sub openid_provider_array ($) {
    my $this = shift;
    return [] if !$env::openid_enabled || !$setting::general{'op'};
    my $op = [];
    while ( $setting::general{'op'}
        =~ /^([^=]*)=([^\x0A\x0D]+)[\x09\x0A\x0C\x0D\x20]*$/mg )
    {
        my $name = $1;
        my @info = split( /;/, $2, 3 );
        push(
            @$op,
            {   'name' => ( ($name) ? $name : 'OpenID Provider' ),
                'endpoint_url' => $this->endpoint_normalize( $info[0] ),
                'claimed_id'   => ( ( $info[1] ) ? $info[1] : '' ),
                'identity'     => ( ( $info[2] ) ? $info[2] : '' ),
            }
        ) if $2;
    }
    return $op;
}

sub is_secure_op ($$) {
    my ( $this, $endpoint_url ) = @_;
    return 0 if !$endpoint_url;
    return 1 if $setting::general{'anyOp'};
    foreach my $op ( @{ $this->openid_provider_array } ) {
        return 1 if $op->{'endpoint_url'} eq $endpoint_url;
    }
    return 0;
}

sub op_info_from_id ($$) {
    my ( $this, $id ) = @_;
    my $info = $this->openid_provider_array->[$id];
    return ( defined $info && ref $info ) ? $info : {};
}

sub print_openid_login_panel_opener ($$$) {
    my ( $this, $email, $claimed_id ) = @_;
    my $op = $this->openid_provider_array;
    if ( !$claimed_id && @$op && $setting::general{'anyOp'} ) {
        $this->set_javascript(
            'cmonos.tool.toggleOpenIdForm = function (this_op) {if (this_op == "") { document.getElementById("openid-identifier").style.display = "table-row"; document.getElementById("identifier").focus(); } else { document.getElementById("openid-identifier").style.display = "none"; }}'
        );
    }
    $this->print_page_header($setting::key);

    print $this->form_start(
        'key'    => 'password',
        'name'   => 'input',
        'method' => 'post',
        (     ( $setting::key ne 'rp' && !$env::register_in_same_window )
            ? ( 'target' => '_blank' )
            : ()
        ),
    );
    print '<input type="hidden" name="m" value="RP" />';
    print '<input type="hidden" name="k" value="rp" />';
    print '<input type="hidden" name="transition" value="1" />'
        if $main::transition;
    print '<input type="hidden" name="referer_url" value="'
        . $this->escape($main::referer_url) . '" />'
        if $main::referer_url;

    print '<table class="field-table"><tbody>';
    if ($email) {
        print '<tr>';
        print
            '<td class="field-name"><span class="datatitle">メールアドレス</span></td>';
        print '<td class="field">'
            . $this->escape($email)
            . '<input type="hidden" name="addr" value="'
            . $this->escape($email)
            . '" /></td>';
        print '</tr>';
    }
    else {
        print '<tr>';
        print
            '<td class="field-name"><label for="addr">メールアドレス</label></td>';
        print
            '<td class="field"><input type="email" name="addr" id="addr" size="32" value="'
            . $this->escape($email)
            . '" /></td>';
        print '</tr>';
    }
    if ( !$claimed_id ) {
        if (@$op) {
            print '<tr>';
            print
                '<td class="field-name"><span class="datatitle">OpenID プロバイダー</span></td>';
            print '<td class="field">';
            if ( ( @$op > 1 ) || ( @$op && $setting::general{'anyOp'} ) ) {
                print '<select name="op"'
                    . ( ( $setting::general{'anyOp'} )
                    ? ' onchange="cmonos.tool.toggleOpenIdForm(this.options[this.selectedIndex].value)"'
                    : '' )
                    . '>';
                for ( my $i = 0; $i < @$op; $i++ ) {
                    print '<option value="'
                        . $i . '"'
                        . ( ( defined $main::op && $main::op eq $i )
                        ? ' selected="selected"'
                        : '' )
                        . '>'
                        . $this->escape( $op->[$i]->{'name'} )
                        . '</option>';
                }
                print '<option value=""'
                    . ( ( !defined $main::op || $main::op eq '' )
                    ? ' selected="selected"'
                    : '' )
                    . '>その他の OpenID</option>'
                    if $setting::general{'anyOp'};
                print '</select>';
            }
            elsif ( @$op == 1 ) {
                print $this->escape( $op->[0]->{'name'} );
                print '<input type="hidden" name="op" value="0" />';
            }
            print '</td>';
            print '</tr>';
        }
        if ( $setting::general{'anyOp'} ) {
            print '<tr id="openid-identifier">';
            print
                '<td class="field-name"><label for="identifier">OpenID</label></td>';
            print
                '<td class="field"><input type="text" name="identifier" id="identifier" size="20" value="'
                . $this->escape($main::identifier)
                . '" class="open-id" /></td>';
            print '</tr>';
        }
    }
    print '<tr>';
    $this->print_close_cell;
    print '<input type="submit" value="ログイン" /></td></tr>';
    print '</tbody></table></form>';

    #ツールアイコン
    $this->print_tool_icon if $this->is_secure('tools');

    #ページフッタ
    $this->print_page_footer($setting::key);

    exit;
}

sub print_close_cell ($) {
    my $this = shift;
    if ( $setting::page{'closeButtonDisabled'} ) {
        print '<td class="submit-box" colspan="2">';
    }
    elsif ($env::register_in_same_window) {
        if ($main::referer_url) {
            print '<td class="close-box">';
            print $cgi::Deco->button( 'return', $setting::page{'backButton'},
                $main::referer_url,
                ( ($default::english) ? 'Return.' : '戻る' ) );
            print '</td>';
            print '<td class="submit-box">';
        }
        else {
            print '<td class="submit-box" colspan="2">';
        }
    }
    else {
        print '<td class="close-box">';
        print $this->close_button(
            $main::referer_url,
            $setting::page{'closeButton'},
            $setting::page{'backButton'}
        );
        print '</td>';
        print '<td class="submit-box">';
    }
}

######################################################################
# obj->print_register_dialog(key,@msg)
sub print_register_dialog ($$@) {
    my ( $this, $key, @msg ) = @_;
    $this->print_page_header($key);
    if ( @msg > 1 ) {
        print '<ul class="dialog">';
        foreach (@msg) {
            next if !defined $_ || $_ eq '';
            $_ =~ s/\\r\\n/<br \/>/g;
            $_ =~ s/\\r/<br \/>/g;
            $_ =~ s/\\n/<br \/>/g;
            print '<li>' . $_ . '</li>';
        }
        print '</ul>';
    }
    else {
        $msg[0] =~ s/\\r\\n/<br \/>/g;
        $msg[0] =~ s/\\r/<br \/>/g;
        $msg[0] =~ s/\\n/<br \/>/g;
        print '<p>';
        print $msg[0];
        print '</p>';
    }
    if ($env::register_in_same_window) {
        if ($main::referer_url) {
            print '<div id="close-button">';
            print $cgi::Deco->button( 'return', $setting::page{'backButton'},
                $main::referer_url,
                ( ($default::english) ? 'Return.' : '戻る' ) );
            print '</div>';
        }
    }
    else {
        print '<div id="close-button">';
        print $this->close_button(
            $main::referer_url,
            $setting::page{'closeButton'},
            $setting::page{'backButton'}
        );
        print '</div>';
    }
    $this->print_page_footer($key);
    exit;
}

1;
