package RegisterSetting;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# パスワード登録環境設定モジュール
# RegisterSetting.pm ver1.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my %setting;

    #書き込み
    if ( defined $main::a && $main::a eq 'w' ) {
        $main::data{'nofreemail'} = 0
            if !defined $main::data{'nofreemail'}
            || $main::data{'nofreemail'} eq '';
        $this->write_db( $env::register_db . '/preference',
            \%main::data, '', '', \%setting );

    }
    else {
        #読み込み
        $this->import_db( $env::register_db . '/preference', \%setting );
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>パスワード登録環境設定</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="w" />';
    $this->print_mail_settings( \%setting, 'mail',
        'デフォルトの登録' );
    $this->print_mail_settings( \%setting, 'batchmail',
        '一括登録用登録' );
    $this->print_mail_settings( \%setting, 'batchmail2',
        '予備の一括登録用登録' );
    $this->print_mail_settings( \%setting, 'remove', '登録抹消' );

    print '<fieldset><legend>登録制限設定</legend>';
    print
        '<fieldset><legend><label for="data:good_addr">登録承認メールアドレス</label></legend>';
    print
        '<textarea name="data:good_addr" id="data:good_addr" cols="72" rows="5" wrap="off">'
        . $this->escape( $setting{'good_addr'} )
        . '</textarea>';
    print
        ' <label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'data:good_addr\').setAutoFit(this.checked);" />フィット</label>';
    print
        '<div class="hints">複数指定するときは改行または半角スペースで区切ります。</div>';
    print
        '<div class="hints">指定された文字列にメールアドレスが合致する場合のみ接続を許可します（正規表現可）。</div>';
    print '</fieldset>';
    print
        '<fieldset><legend><label for="data:bad_addr">接続拒絶メールアドレス</label></legend>';
    print
        '<textarea name="data:bad_addr" id="data:bad_addr" cols="72" rows="5" wrap="off">'
        . $this->escape( $setting{'bad_addr'} )
        . '</textarea>';
    print
        ' <label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'data:bad_addr\').setAutoFit(this.checked);" />フィット</label>';
    print
        '<div class="hints">複数指定するときは改行または半角スペースで区切ります。</div>';
    print
        '<div class="hints">指定された文字列にメールアドレスが合致する場合、接続を拒絶します（正規表現可）。</div>';
    print '</fieldset>';
    print
        '<label><input type="checkbox" name="data:nofreemail" id="data:nofreemail" value="1"'
        . ( ( defined $setting{'nofreemail'} && $setting{'nofreemail'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 主要なフリーメールの登録を拒絶する。</label>';
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');

}

sub print_mail_settings ($$$$) {
    my ( $this, $setting, $key, $name ) = @_;
    print '<fieldset><legend>' . $name . '通知メール設定</legend>';
    print '<fieldset><legend><label for="data:' . $key
        . '_subject">表題</label></legend>';
    print '<input type="text" name="data:'
        . $key
        . '_subject" id="data:'
        . $key
        . '_subject" size="72" value="'
        . $this->escape( $setting->{ $key . '_subject' } ) . '" />';
    print '</fieldset>';
    print '<fieldset><legend><label for="data:' . $key
        . '_msg">本文</label></legend>';
    print '<textarea name="data:'
        . $key
        . '_msg" id="data:'
        . $key
        . '_msg" cols="72" rows="8" wrap="soft">'
        . $this->escape( $setting->{ $key . '_msg' } )
        . '</textarea>';
    print '<div class="toolbox">';
    print
        '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:'
        . $key
        . '_msg\').quotationByMenu(this)" title="これらのタグは登録通知メール発送時に対応する値に変換されます。">';
    print '<option value="">▼挿入する</option>';

    if ( $key eq 'remove' ) {
        print
            '<option value="&lt;reason&gt;">&lt;reason&gt;[登録抹消理由]</option>';
    }
    else {
        print
            '<option value="&lt;email&gt;">&lt;email&gt;[登録メールアドレス]</option>';
        print
            '<option value="&lt;password&gt;">&lt;password&gt;[発行パスワード]</option>';
    }
    print '</select>';
    print
        ' <label class="auto-fit" title="高さを内容に合わせる"><input type="checkbox" onclick="cmonos.form.item(this.form).fieldname(\'data:'
        . $key
        . '_msg\').setAutoFit(this.checked);" />フィット</label>';
    print '</div>';
    print '</fieldset>';
    print '</fieldset>';
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 'r200', 'RegisterSetting' );

    #メニュー登録
    $setting::jump_menu{'r200'} = 'パスワード登録環境設定';

}

1;
