package CartSetting;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# 消費税・カート情報変更モジュール
# CartSetting.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ($valid_flag,             $error_flag,
        $tax_rate,               $tax_included,
        $tax_frac,               $point_lifetime,
        $checkout_window_type,   $checkout_window_width,
        $checkout_window_height, $checkout_window_top,
        $checkout_window_left,   $cart_js_changed
    );
    $this->get_setup_env;

    #入力チェック
    if ( defined $main::tax_rate && $main::tax_rate ne '' ) {
        if ( $this->is_nan($main::tax_rate) ) {
            push(
                @{ $this->{'dialog'} },
                '消費税率は半角数字で設定してください。'
            );
            $error_flag = 1;
        }
        else {
            $tax_rate = abs( int($main::tax_rate) ) / 100;
            if ( $tax_rate == $env::tax_rate ) {
                undef $main::tax_rate;
                undef $tax_rate;
            }
            else {
                $valid_flag = 1;
            }
        }
    }
    if ( defined $main::tax_frac
        && $main::tax_frac =~ /^(ceil|round|floor)$/ )
    {
        if ( $main::tax_frac eq $env::tax_frac ) {
            undef $main::tax_frac;
        }
        else {
            $tax_frac   = $main::tax_frac;
            $valid_flag = 1;
        }
    }
    if ( defined $main::tax_included && $main::tax_included ne '' ) {
        if ( $main::tax_included eq $env::tax_included ) {
            undef $main::tax_included;
        }
        else {
            $tax_included = ($main::tax_included) ? 1 : 0;
            $valid_flag = 1;
        }
    }

#if (defined $main::point_lifetime && $main::point_lifetime ne '') {
#	if ($this->is_nan($main::point_lifetime)) {
#		push(@{$this->{'dialog'}},'ポイント有効期限は半角数字で設定してください。');
#		$error_flag = 1;
#	} else {
#		$point_lifetime = $main::point_lifetime;
#		$valid_flag = 1;
#	}
#}
    if (defined $main::checkout_window_type
        && (   $main::checkout_window_type eq 'self'
            || $main::checkout_window_type eq 'iframe'
            || $main::checkout_window_type eq 'modal'
            || $main::checkout_window_type eq 'new' )
        )
    {
        if ( $main::checkout_window_type eq $env::checkout_window_type ) {
            undef $main::checkout_window_type;
        }
        else {
            $checkout_window_type = $main::checkout_window_type;
            $cart_js_changed = $valid_flag = 1;
        }
    }
    if ( defined $main::checkout_window_width
        && $main::checkout_window_width ne '' )
    {
        $main::checkout_window_width = lc $main::checkout_window_width;
        if ( $main::checkout_window_width eq $env::checkout_window_width ) {
            undef $main::checkout_window_width;
        }
        elsif ($main::checkout_window_width ne 'auto'
            && $this->is_nan($main::checkout_window_width) )
        {
            push(
                @{ $this->{'dialog'} },
                'チェックアウトウィンドウ幅は半角数字または auto のいずれかで設定してください。'
            );
            $error_flag = 1;
        }
        else {
            $cart_js_changed = $valid_flag = 1;
            $checkout_window_width
                = ( $main::checkout_window_width eq 'auto' )
                ? $main::checkout_window_width
                : int($main::checkout_window_width);
        }
    }
    if ( defined $main::checkout_window_height
        && $main::checkout_window_height ne '' )
    {
        $main::checkout_window_height = lc $main::checkout_window_height;
        if ( $main::checkout_window_height eq $env::checkout_window_height ) {
            undef $main::checkout_window_height;
        }
        elsif ($main::checkout_window_height ne 'auto'
            && $this->is_nan($main::checkout_window_height) )
        {
            push(
                @{ $this->{'dialog'} },
                'チェックアウトウィンドウ高さは半角数字または auto のいずれかで設定してください。'
            );
            $error_flag = 1;
        }
        else {
            $cart_js_changed = $valid_flag = 1;
            $checkout_window_height
                = ( $main::checkout_window_height eq 'auto' )
                ? $main::checkout_window_height
                : int($main::checkout_window_height);
        }
    }
    if ( defined $main::checkout_window_top
        && $main::checkout_window_top ne '' )
    {
        $main::checkout_window_top = lc $main::checkout_window_top;
        if ( $main::checkout_window_top eq $env::checkout_window_top ) {
            undef $main::checkout_window_top;
        }
        elsif ($main::checkout_window_top !~ /^(?:top|center|bottom)$/
            && $this->is_nan($main::checkout_window_top) )
        {
            push(
                @{ $this->{'dialog'} },
                'チェックアウトフレーム垂直位置は半角数字または top、center、bottom のいずれかで設定してください。'
            );
            $error_flag = 1;
        }
        else {
            $cart_js_changed = $valid_flag = 1;
            $checkout_window_top
                = ( $main::checkout_window_top =~ /^(?:top|center|bottom)$/ )
                ? $main::checkout_window_top
                : int($main::checkout_window_top);
        }
    }
    if ( defined $main::checkout_window_left
        && $main::checkout_window_left ne '' )
    {
        $main::checkout_window_left = lc $main::checkout_window_left;
        if ( $main::checkout_window_left eq $env::checkout_window_left ) {
            undef $main::checkout_window_left;
        }
        elsif ($main::checkout_window_left !~ /^(?:left|center|right)$/
            && $this->is_nan($main::checkout_window_left) )
        {
            push(
                @{ $this->{'dialog'} },
                'チェックアウトフレーム水平位置は半角数字または left、center、right のいずれかで設定してください。'
            );
            $error_flag = 1;
        }
        else {
            $cart_js_changed = $valid_flag = 1;
            $checkout_window_left
                = ( $main::checkout_window_left =~ /^(?:left|center|right)$/ )
                ? $main::checkout_window_left
                : int($main::checkout_window_left);
        }
    }

    #env.pl書き換え
    if ( $valid_flag && !$error_flag ) {
        local (*FILE);
        my $top
            = ( $checkout_window_top && $checkout_window_top !~ /^\-?[0-9]+/ )
            ? "'" . $checkout_window_top . "'"
            : $checkout_window_top;
        my $left
            = (    $checkout_window_left
                && $checkout_window_left !~ /^\-?[0-9]+/ )
            ? "'" . $checkout_window_left . "'"
            : $checkout_window_left;
        my $width
            = (    $checkout_window_width
                && $checkout_window_width !~ /^\-?[0-9]+/ )
            ? "'" . $checkout_window_width . "'"
            : $checkout_window_width;
        my $height
            = (    $checkout_window_height
                && $checkout_window_height !~ /^\-?[0-9]+/ )
            ? "'" . $checkout_window_height . "'"
            : $checkout_window_height;
        my $env_path = $default::cgi_data . '/lib/env.pl';
        my $data = $this->read_file( $env_path, 1 );
        $data
            =~ s/^[\x20\x09]*\$tax_rate[\x20\x09]*=[^;]*;/\$tax_rate = $tax_rate;/gm
            if defined $tax_rate && $tax_rate ne '';
        $data
            =~ s/^[\x20\x09]*\$tax_frac[\x20\x09]*=[^;]*;/\$tax_frac = '$tax_frac';/gm
            if defined $tax_frac && $tax_frac ne '';
        $data
            =~ s/^[\x20\x09]*\$tax_included[\x20\x09]*=[^;]*;/\$tax_included = $tax_included;/gm
            if defined $tax_included && $tax_included ne '';
        $data
            =~ s/^[\x20\x09]*\$point_lifetime[\x20\x09]*=[^;]*;/\$point_lifetime = $point_lifetime;/gm
            if defined $point_lifetime && $point_lifetime ne '';
        $data
            =~ s/^[\x20\x09]*\$checkout_window_type[\x20\x09]*=[^;]*;/\$checkout_window_type = '$checkout_window_type';/gm
            if defined $checkout_window_type && $checkout_window_type ne '';
        $data
            =~ s/^[\x20\x09]*\$checkout_window_width[\x20\x09]*=[^;]*;/\$checkout_window_width = $width;/gm
            if defined $width && $width ne '';
        $data
            =~ s/^[\x20\x09]*\$checkout_window_height[\x20\x09]*=[^;]*;/\$checkout_window_height = $height;/gm
            if defined $height && $height ne '';
        $data
            =~ s/^[\x20\x09]*\$checkout_window_top[\x20\x09]*=[^;]*;/\$checkout_window_top = $top;/gm
            if defined $top && $top ne '';
        $data
            =~ s/^[\x20\x09]*\$checkout_window_left[\x20\x09]*=[^;]*;/\$checkout_window_left = $left;/gm
            if defined $left && $left ne '';

        if ( open( FILE, ">>$env_path" ) ) {
            binmode FILE, ":utf8";
            flock( FILE, LOCK_EX );
            truncate( FILE, 0 );
            seek( FILE, 0, 0 );
            print FILE $data;
            close(FILE);
            push(
                @{ $this->{'dialog'} },
                '消費税・カート情報を更新しました。'
            );
            $env::tax_rate = $tax_rate
                if defined $tax_rate && $tax_rate ne '';
            $env::tax_frac = $tax_frac
                if defined $tax_frac && $tax_frac ne '';
            $env::tax_included = $tax_included
                if defined $tax_included && $tax_included ne '';
            $env::point_lifetime = $point_lifetime
                if defined $point_lifetime && $point_lifetime ne '';
            $env::checkout_window_type = $checkout_window_type
                if defined $checkout_window_type
                && $checkout_window_type ne '';
            $env::checkout_window_width = $checkout_window_width
                if defined $checkout_window_width
                && $checkout_window_width ne '';
            $env::checkout_window_height = $checkout_window_height
                if defined $checkout_window_height
                && $checkout_window_height ne '';
            $env::checkout_window_top = $checkout_window_top
                if defined $checkout_window_top && $checkout_window_top ne '';
            $env::checkout_window_left = $checkout_window_left
                if defined $checkout_window_left
                && $checkout_window_left ne '';
            undef $main::tax_rate;
            undef $main::tax_frac;
            undef $main::tax_included;
            undef $main::point_lifetime;
            undef $main::checkout_window_type;
            undef $main::checkout_window_width;
            undef $main::checkout_window_height;
            undef $main::checkout_window_top;
            undef $main::checkout_window_left;
            $this->mk_cart_js if $cart_js_changed;
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '消費税・カート情報の更新に失敗しました。'
            );
        }
    }
    elsif ( defined $error_flag && $error_flag ) {
        push(
            @{ $this->{'dialog'} },
            '消費税・カート情報の更新に失敗しました。'
        );
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>消費税・カート情報変更</h1>';

    print $this->form_start(
        'key'    => 'tools',
        'name'   => 'input',
        'method' => 'post',
        'onsubmit' =>
            'return confirm(\'この操作は取り消せません。本当に変更しますか？\') &amp;&amp; cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="1" />';
    print '<fieldset><table class="field-table"><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="tax_rate">消費税率</label></td>';
    print
        '<td class="field"><input type="number" name="tax_rate" id="tax_rate" size="4" value="'
        . ( ( defined $main::tax_rate ) ? $main::tax_rate
        : ( defined $env::tax_rate ) ? $env::tax_rate * 100
        :                              '' )
        . '" /> %';
    print
        '<div class="hints">0 %に設定すると消費税の計算をしません。</div>';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="tax_frac">端数処理</label></td>';
    print '<td class="field"><select name="tax_frac" id="tax_frac">';
    print '<option value="ceil"'
        . (
        (          ( defined $main::tax_frac && $main::tax_frac eq 'ceil' )
                || ( !defined $main::tax_frac && $env::tax_frac eq 'ceil' )
        ) ? ' selected="selected"' : ''
        ) . '>切り上げ</option>';
    print '<option value="round"'
        . (
        (          ( defined $main::tax_frac && $main::tax_frac eq 'round' )
                || ( !defined $main::tax_frac && $env::tax_frac eq 'round' )
        ) ? ' selected="selected"' : ''
        ) . '>四捨五入</option>';
    print '<option value="floor"'
        . (
        (          ( defined $main::tax_frac && $main::tax_frac eq 'floor' )
                || ( !defined $main::tax_frac && $env::tax_frac eq 'floor' )
        ) ? ' selected="selected"' : ''
        ) . '>切り捨て</option>';
    print '</select>';
    print '</td>';
    print '</tr>';
    print '<tr class="tbody-end">';
    print
        '<td class="field-name"><label for="tax_included">価格表示</label></td>';
    print '<td class="field"><select name="tax_included" id="tax_included">';
    print '<option value="1"'
        . (
        (   ( defined $main::tax_included && $main::tax_included eq '1' )
                || ( !defined $main::tax_included
                && $env::tax_included eq '1' )
        ) ? ' selected="selected"' : ''
        ) . '>内税方式</option>';
    print '<option value="0"'
        . (
        (   ( defined $main::tax_included && $main::tax_included eq '0' )
                || ( !defined $main::tax_included
                && $env::tax_included eq '0' )
        ) ? ' selected="selected"' : ''
        ) . '>外税方式</option>';
    print '</select>';
    print '</td>';
    print '</tbody><tbody>';

#print '<tr class="tbody-end">';
#print '<td class="field-name"><label for="point_lifetime">ポイント有効期限</label></td>';
#print '<td class="field">';
#print '<input type="text" name="point_lifetime" id="point_lifetime" size="4" value="' . $this->escape((defined $main::point_lifetime) ? $main::point_lifetime : $env::point_lifetime) . '" /> ヶ月間有効';
#print '(端数ポイントは' . (($env::tax_frac eq 'floor') ? '切り上げ' : (($env::tax_frac eq 'round') ? '四捨五入' : '切り下げ')) . ')';
#print '<div class="hints">現在の設定=' . $env::point_lifetime . 'ヶ月</div>';
#print '<div class="hints">無記入の場合変更されません。</div>';
#print '</td>';
#print '</tr>';
#print '</tbody><tbody>';
    print '<tr>';
    print
        '<td class="field-name"><label for="checkout_window_type">チェックアウトツール</label></td><td class="field"><select name="checkout_window_type" id="checkout_window_type">';
    print '<option value="new"'
        . (
        (   (   defined $main::checkout_window_type
                    && $main::checkout_window_type eq 'new'
            )
                || ( !defined $main::checkout_window_type
                && $env::checkout_window_type eq 'new' )
        ) ? ' selected="selected"' : ''
        ) . '>別のウィンドウで開く</option>';
    print '<option value="iframe"'
        . (
        (   (   defined $main::checkout_window_type
                    && $main::checkout_window_type eq 'iframe'
            )
                || ( !defined $main::checkout_window_type
                && $env::checkout_window_type eq 'iframe' )
        ) ? ' selected="selected"' : ''
        ) . '>インラインフレームで開く</option>';
    print '<option value="modal"'
        . (
        (   (   defined $main::checkout_window_type
                    && $main::checkout_window_type eq 'modal'
            )
                || ( !defined $main::checkout_window_type
                && $env::checkout_window_type eq 'modal' )
        ) ? ' selected="selected"' : ''
        ) . '>モーダルウィンドウで開く</option>';
    print '<option value="self"'
        . (
        (   (   defined $main::checkout_window_type
                    && $main::checkout_window_type eq 'self'
            )
                || ( !defined $main::checkout_window_type
                && $env::checkout_window_type eq 'self' )
        ) ? ' selected="selected"' : ''
        ) . '>同じウィンドウで開く</option>';
    print '</select>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="tax_rate">チェックアウト'
        . (
        (          $env::checkout_window_type
                && $env::checkout_window_type eq 'iframe'
        ) ? 'フレーム' : 'ウィンドウ'
        ) . '幅</label></td>';
    print
        '<td class="field"><input type="text" name="checkout_window_width" id="checkout_window_width" size="8" value="'
        . $this->escape( ( defined $main::checkout_window_width )
        ? $main::checkout_window_width
        : $env::checkout_window_width )
        . '" /> (auto|px)';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print '<td class="field-name"><label for="tax_rate">チェックアウト'
        . (
        (          $env::checkout_window_type
                && $env::checkout_window_type eq 'iframe'
        ) ? 'フレーム' : 'ウィンドウ'
        ) . '高さ</label></td>';
    print
        '<td class="field"><input type="text" name="checkout_window_height" id="checkout_window_height" size="8" value="'
        . $this->escape( ( defined $main::checkout_window_height )
        ? $main::checkout_window_height
        : $env::checkout_window_height )
        . '" /> (auto|px)';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="tax_rate">チェックアウトフレーム垂直表示位置</label></td>';
    print
        '<td class="field"><input type="text" name="checkout_window_top" id="checkout_window_top" size="8" value="'
        . $this->escape( ( defined $main::checkout_window_top )
        ? $main::checkout_window_top
        : $env::checkout_window_top )
        . '" /> (top|center|bottom|px)';
    print
        '<div class="hints">マイナスの値を設定するとページ下端からの距離となります。</div>';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="tax_rate">チェックアウトフレーム水平表示位置</label></td>';
    print
        '<td class="field"><input type="text" name="checkout_window_left" id="checkout_window_left" size="8" value="'
        . $this->escape( ( defined $main::checkout_window_left )
        ? $main::checkout_window_left
        : $env::checkout_window_left )
        . '" /> (left|center|right|px)';
    print
        '<div class="hints">マイナスの値を設定するとページ右端からの距離となります。</div>';
    print '</td>';
    print '</tr>';
    print '</tbody></table></fieldset>';
    print
        '<fieldset class="submit"><input type="submit" value="変更" class="save" /></fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

sub mk_cart_js ($) {
    my $this = shift;
    my $cart_js = $this->read_file( $env::javascript . '/cart.js', 1 );
    return 0 if !$cart_js;
    my $top
        = (    $env::checkout_window_top
            && $env::checkout_window_top !~ /^\-?[0-9]+/ )
        ? '"' . $env::checkout_window_top . '"'
        : $env::checkout_window_top;
    my $left
        = (    $env::checkout_window_left
            && $env::checkout_window_left !~ /^\-?[0-9]+/ )
        ? '"' . $env::checkout_window_left . '"'
        : $env::checkout_window_left;
    my $width
        = (    $env::checkout_window_width
            && $env::checkout_window_width !~ /^\-?[0-9]+/ )
        ? '"' . $env::checkout_window_width . '"'
        : $env::checkout_window_width;
    my $height
        = (    $env::checkout_window_height
            && $env::checkout_window_height !~ /^\-?[0-9]+/ )
        ? '"' . $env::checkout_window_height . '"'
        : $env::checkout_window_height;
    $cart_js
        =~ s/([\x09\x0A\x0C\x0D\x20]+)type : .*?,/${1}type : "$env::checkout_window_type",/mg
        if defined $env::checkout_window_type
        && $env::checkout_window_type ne '';
    $cart_js =~ s/([\x09\x0A\x0C\x0D\x20]+)top : .*?,/${1}top : $top,/mg
        if defined $top && $top ne '';
    $cart_js =~ s/([\x09\x0A\x0C\x0D\x20]+)left : .*?,/${1}left : $left,/mg
        if defined $left && $left ne '';
    $cart_js =~ s/([\x09\x0A\x0C\x0D\x20]+)width : .*?,/${1}width : $width,/mg
        if defined $width && $width ne '';
    $cart_js
        =~ s/([\x09\x0A\x0C\x0D\x20]+)height : .*?,/${1}height : $height,/mg
        if defined $height && $height ne '';
    return $this->write_file( $env::javascript . '/cart.js', \$cart_js, 1 );
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ( defined $env::checkout && $env::checkout ) {
        no warnings qw(once);

        #クラス登録
        Common::class_register( '00s1', 'CartSetting' );

        #メニュー登録
        $setting::jump_menu{'00s1'} = '消費税・カート情報変更';

    }
}

1;
