package SessionSweeper;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# セッション削除モジュール
# SessionSweeper.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    local (*STORE);
    my $session_count = 0;
    my $delete_count  = 0;
    my $expired_time  = 0;
    my $stat = ( defined $main::stat && $main::stat eq 'mtime' ) ? 9 : 8;
    if (   !defined $main::sleepdays
        || $main::sleepdays eq ''
        || ( $main::sleepdays && $this->is_nan($main::sleepdays) ) )
    {
        $main::sleepdays = '';
    }
    else {
        $expired_time = time - 86400 * $main::sleepdays;
    }

    if ( opendir( STORE, $env::session_store ) ) {
        while ( my $id = readdir(STORE) ) {
            if ( $id !~ /^\./ ) {
                my $session = $env::session_store . '/' . $id;
                if (   $expired_time
                    && -f $session
                    && ( stat($session) )[$stat] <= $expired_time )
                {
                    unlink $session;
                    $delete_count++;
                }
                else {
                    $session_count++;
                }
            }
        }
        closedir(STORE);
    }
    push(
        @{ $this->{'dialog'} },
        $delete_count . '件のセッションを削除しました。'
    ) if $expired_time;

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>セッション削除</h1>';

    if ($session_count) {
        print '<p>現在、'
            . $session_count
            . '件のセッションが保存されています。</p>';
        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'input',
            'method'   => 'post',
            'required' => \@setting::form,
            'onsubmit' =>
                'return confirm(\'この操作は取り消せません。本当に削除しますか？\');',
        );
        print '<fieldset class="delete">';
        print '<input type="number" size="5" name="sleepdays" value="'
            . $this->escape( $main::sleepdays || $env::cookie_lifetime || 30 )
            . '" title="0 を設定するとすべてのセッションを削除します。" />';
        print '日より長く';
        print '<select name="stat" id="stat">';
        print '<option value="atime"'
            . ( ( defined $main::stat && $main::stat eq 'atime' )
            ? ' selected="selected"'
            : '' )
            . '>アクセス</option>';
        print '<option value="mtime"'
            . ( ( defined $main::stat && $main::stat eq 'mtime' )
            ? ' selected="selected"'
            : '' )
            . '>更新</option>';
        print '</select>';
        print
            'されていないセッションを<input type="submit" value="一括削除する" /></td></tr>';
        print '</fieldset></form>';
    }
    elsif ( !$env::session_enabled ) {
        print
            '<p>セッション管理が無効です。「管理情報変更」で変更できます。</p>';
    }
    else {
        print '<p>セッションが見つかりません。</p>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 's305', 'SessionSweeper' );

    #メニュー登録
    $setting::jump_menu{'s305'} = 'セッション削除'
        if !defined $setting::jump_menu{'s305'};

}

1;
