package SiteInfo;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use strict;
use utf8;
######################################################################
# サイト情報一覧モジュール
# SiteInfo.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( $size, $fcount );
    if ( -f "../servercheck.cgi" ) {
        unlink "../servercheck.cgi";
    }
    $this->get_setup_env;

    my $home = $this->document_root;
    $home =~ s/\/[^\/]+$//
        unless $default::htdocs_ftp ne ''
        && $home =~ s/$default::htdocs_ftp$//;

    use File::Find;
    find( sub { ( $size += -s ) && ( $fcount++ ) if -f }, $home );
    $size = '?' if !defined $size   || !$size;
    $size = '?' if !defined $fcount || !$fcount;

    use Config;

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>サイト情報</h1>';

    print '<fieldset><dl id="site-info">';

    if ($cgi::cmonos_ver) {
        print '<dt>CMONOS.JP のバージョン</dt><dd>';
        print(( $cgi::cmonos_ver > 20110926 ) ? '2.0'
            : ( $cgi::cmonos_ver >= 20101125 ) ? '1.5'
            : ( $cgi::cmonos_ver >= 20100915 ) ? '1.4'
            : ( $cgi::cmonos_ver >= 20091100 ) ? '1.3'
            : ( $cgi::cmonos_ver >= 20081216 ) ? '1.2'
            : ( $cgi::cmonos_ver >= 20080715 ) ? '1.1'
            :                                    '1.0' );
        print '.' . $cgi::cmonos_ver;
        print '</dd>';
    }

    print '<dt>ディスク使用量</dt><dd>';
    print $this->digit($size), ' byte</dd>';

    print '<dt>ファイル数</dt><dd>';
    print $this->digit($fcount), '</dd>';

    print '<dt>OS</dt><dd>';
    print `uname`, '</dd>';

    print '<dt>OSバージョン</dt><dd>';
    print $Config{'osvers'}, '</dd>';

    print '<dt>サーバソフトウェア</dt><dd>';
    print $ENV{'SERVER_SOFTWARE'}, '</dd>';

    print '<dt>Perlバージョン</dt><dd>';
    print $], '</dd>';

    print '<dt>Perlのパス</dt><dd>';
    print $^X, '</dd>';

    print '<dt>CGIの動作</dt><dd>';
    print( ($env::ownerpower)
        ? 'ユーザー権限'
        : 'ユーザー以外の権限' );
    print '</dd>';

    print '<dt>Image::Magick</dt><dd>';
    print( ( eval 'use Image::Magick;1;' )
        ? '利用可能'
        : 'みつかりません' );
    print '</dd>';
    print '<dt>LWP::UserAgent</dt><dd>';
    print( ( eval 'use LWP::UserAgent;1;' )
        ? '利用可能'
        : 'みつかりません' );
    print '</dd>';
    print '<dt>Crypt::SSLeay</dt><dd>';
    print( ( eval 'use Crypt::SSLeay;1;' )
        ? '利用可能'
        : 'みつかりません' );
    print '</dd>';
    print '<dt>Digest::SHA</dt><dd>';
    print( ( eval 'use Digest;use Digest::SHA;1;' )
        ? '利用可能'
        : 'みつかりません' );
    print '</dd>';

    print '<dt>DBM</dt><dd><ul>';

    my $ex = $Config{'extensions'};
    if ( $ex =~ s/NDBM_File//ig ) {
        print
            '<li title="UNIXに標準装備されていることが多いndbmデータベースにアクセスします。容量制限は 1レコードあたり 4,096 byte です。">NDBM_File</li>';
    }
    if ( $ex =~ s/DB_File//ig ) {
        print
            '<li title="BSD系UNIXに標準装備されているdbデータベースパッケージを利用します。">DB_File</li>';
    }
    if ( $ex =~ s/GDBM_File//ig ) {
        print
            '<li title="gdbmデータベースにアクセスします。キーと値の容量に制限がありませんがファイルサイズが大きくなります">GDBM_File</li>';
    }
    if ( $ex =~ s/SDBM_File//ig ) {
        print
            '<li title="Perlに付属しているsdbmデータベースにアクセスします。容量制限は 1レコードあたり 1,024 byte または 4,096 byte です。">SDBM_File</li>';
    }
    if ( $ex =~ s/ODBM_File//ig ) {
        print
            '<li title="古くからUNIXで標準だったdbmデータベースにアクセスします。容量制限は 1レコードあたり 1,024 byte です。">ODBM_File</li>';
    }

    print '</ul></dd>';
    print '<dt>機能拡張</dt><dd><ul><li>';
    $ex =~ s/[\x09\x0A\x0C\x0D\x20]+/<\/li><li>/g;
    print $ex;

    print '</li></ul></dd></dl></fieldset>';

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ( ( !defined $env::home_dir_id || $env::home_dir_id eq '' )
        && !$env::multi_domain )
    {

        #クラス登録
        Common::class_register( 's200', 'SiteInfo' );

        #メニュー登録
        $setting::jump_menu{'s200'} = 'サイト情報一覧';
    }
}

1;
