package SiteTemplateImporter;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use strict;
use utf8;
######################################################################
# サイトテンプレート読み込みモジュール
# SiteTemplateImporter.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2011-04-18
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    $this->get_setup_env;
    unless ( eval 'use Archive::Tar;Archive::Tar->has_zlib_support;' ) {
        $this->jump_menu('tools');
        $this->print_page_header('tools');
        print '<h1>サイトテンプレート読み込み</h1>';
        print
            '<p>Archive::Tar モジュールがみつからないため、この機能は利用できません。こちらのマニュアル「<a href="http://cmonos.jp/manual/setup_assistant/how/site_template_install.html" target="_blank">サイトテンプレートの導入</a>」をご覧ください。</p>';
        $this->print_page_footer('tools');
        return 1;
    }
    use File::Path;
    require CMONOS::CGI_Lib::HTTP_Client;

    #作業フォルダ
    my $temp_dir
        = $env::clipboard
        . '/site_template_importer'
        . ( $env::home_dir_id || '' );
    my $status_file = $temp_dir . '/status';
    my $pending_time
        = ( !-f $status_file ) ? 0 : time - ( stat($status_file) )[9];
    if ( -d $temp_dir && $pending_time > 600 ) {
        push(
            @{ $this->{'dialog'} },
            '前回サイトテンプレートの読み込みまたは解凍に失敗した可能性があります。作業フォルダを削除しました。'
        );
        rmtree $temp_dir;
    }
    my $status = $this->read_file( $status_file, 1 ) || 0;

    #読み込み処理
    if ($main::a) {
        my $tgz_file  = '';
        my $temp_name = '';
        my $file_name = '';
        if ($main::file) {
            if ( $main::file =~ /^(.+?)\.(tar\.gz|tgz)$/i ) {
                $temp_name = $1;
                $temp_name =~ s/\@.*$//s;
                $temp_name =~ s/[^0-9a-zA-Z_\-]//s;
                if ( !$temp_name ) {
                    $temp_name = 'NewSiteTemplate';
                    undef $main::replace;
                }
                $main::file = $temp_name . '.tgz';
                $this->set_dir($temp_dir);
                $this->write_file( $status_file, 1, 1 );
                $cgi::input->save_file( 'file', $main::file, '', '',
                    $temp_dir );
                $tgz_file = $temp_dir . '/' . $main::file;
            }
            else {
                push(
                    @{ $this->{'dialog'} },
                    'サイトテンプレートを追加できませんでした。tar-gzip で圧縮されたファイルをアップロードしてください。拡張子は「.tar.gz」または「.tgz」です。ファイル名は半角英数字にしてください。'
                );
            }
            $cgi::input->clear_clipboard;
        }
        elsif ($main::filename) {
            if ( $main::filename !~ /^[0-9a-zA-Z_]+\.(?:tar\.gz|tgz)$/ ) {
                push(
                    @{ $this->{'dialog'} },
                    'サイトテンプレートを追加できませんでした。指定されたサイトテンプレートは存在しません。'
                );
            }
            else {
                $this->set_dir($temp_dir);
                $this->write_file( $status_file, 1, 1 );
                HTTP_Client->new->request(
                    'GET' => 'http://cmonos.jp/download/site_templates/'
                        . $main::filename,
                    {}, '', '', '', $temp_dir . '/' . $main::filename
                );
                $tgz_file = $temp_dir . '/' . $main::filename;
            }
        }
        else {
            push(
                @{ $this->{'dialog'} },
                'サイトテンプレートを追加できませんでした。サイトテンプレートが指定されていません。'
            );
        }
        if ( $tgz_file && -f $tgz_file ) {
            $this->write_file( $status_file, 2, 1 );
            my $tar = Archive::Tar->new();
            $tar->setcwd($temp_dir);
            $tar->read( $tgz_file, 1 );
            $tar->extract();
            $this->write_file( $status_file, 3, 1 );
            my $new_template = $temp_dir . '/' . $temp_name;
            if ( -f $temp_dir . '/env.bak' ) {
                local (*DIR);
                if ( opendir( DIR, $temp_dir ) ) {
                    my @files
                        = grep( $_ ne 'status'
                            && $_ ne $main::file
                            && $_ !~ /^\./,
                        readdir(DIR) );
                    closedir(DIR);
                    $this->set_dir($new_template);
                    foreach my $f (@files) {
                        rename(
                            $temp_dir . '/' . $f,
                            $new_template . '/' . $f
                        );
                    }
                }
            }
            else {
                $new_template = $this->fileglob( $temp_dir, '', 'd' );
            }
            my $site_template_dir = $env::user_site_template_dir
                || $env::template_dir . '/site';
            if ( !$new_template ) {
                push(
                    @{ $this->{'dialog'} },
                    'サイトテンプレートを追加できませんでした。サイトテンプレートがみつかりません。'
                );
            }
            elsif ( !-f $new_template . '/env.bak' ) {
                push(
                    @{ $this->{'dialog'} },
                    'サイトテンプレートを追加できませんでした。指定されたファイルはサイトテンプレートではありません。'
                );
            }
            elsif ( !$this->set_dir($site_template_dir) ) {
                push(
                    @{ $this->{'dialog'} },
                    'サイトテンプレートを追加できませんでした。サイトテンプレート保存ディレクトリを作成できませんでした。'
                );
            }
            else {
                my $new_name;
                if ( $new_template =~ /\/([a-zA-Z0-9]+)$/ ) {
                    $new_name = $1;
                }
                elsif ($temp_name) {
                    $new_name = $temp_name;
                }
                else {
                    $new_name = 'NewSiteTemplate';
                    undef $main::replace;
                }
                $new_name = substr( $new_name, 0, 32 )
                    if length $new_name > 32;
                my $suffix
                    = ( !$env::user_site_template_dir && $main::closed )
                    ? $this->directory_suffix
                    : '';
                if ( -d $site_template_dir . '/' . $new_name . $suffix ) {
                    if ($main::replace) {
                        rmtree $site_template_dir . '/' . $new_name . $suffix;
                    }
                    else {
                        my $n = ( $new_name =~ s/([0-9]+)$// ) ? $1 : 2;
                        while ( -d $site_template_dir . '/'
                            . $new_name
                            . $n
                            . $suffix )
                        {
                            $n++;
                        }
                        $new_name .= $n;
                    }
                }
                if (rename(
                        $new_template,
                        $site_template_dir . '/' . $new_name . $suffix
                    )
                    )
                {
                    my $temp_label
                        = $new_name
                        . ( ($main::closed)
                        ? ($default::home_dir)
                                ? '@~' . $this->escape($default::home_dir)
                                : '@~/'
                        : '' );
                    my ($title) = split(
                        /\x09/,
                        $this->read_file(
                                  $site_template_dir . '/'
                                . $new_name
                                . $suffix
                                . '/name.txt'
                        )
                    );
                    push(
                        @{ $this->{'dialog'} },
                        'サイトテンプレート「'
                            . $this->site_template_title( $title,
                            $temp_label )
                            . '」を追加しました。'
                    );
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        'サイトテンプレートを追加できませんでした。サイトテンプレートフォルダにコピーできませんでした。'
                    );
                }
            }
        }
        rmtree $temp_dir if -d $temp_dir;
        $status = 0;
    }

    my $res = HTTP_Client->new->request(
        'GET' => 'http://cmonos.jp/download/site_templates/list.txt' );
    my $site_templates_list
        = ( $res->is_success ) ? $res->content('utf8') : '';

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>サイトテンプレート読み込み</h1>';

    if ($status) {
        print '<p>';
        if ( $status == 1 ) {
            print
                'サイトテンプレートファイルを読み込んでいます... (読み込みに失敗している場合、'
                . ( ($pending_time) ? 10 - int( $pending_time / 60 ) : 10 )
                . '分後に作業フォルダが削除されます)';
        }
        elsif ( $status == 2 ) {
            print
                'サイトテンプレートファイルを解凍しています... (圧縮に失敗している場合、'
                . ( ($pending_time) ? 10 - int( $pending_time / 60 ) : 10 )
                . '分後に作業フォルダが削除されます)';
        }
        elsif ( $status == 3 ) {
            print
                'サイトテンプレートファイルを適切な位置に移動させています...';
        }
        else {
            print '処理中...';
        }
        print '</p>';
    }
    else {
        print $this->form_start(
            'key'      => 'tools',
            'name'     => 'input',
            'multi'    => 1,
            'method'   => 'post',
            'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted();',
            'required' => \@setting::form
        );
        print '<input type="hidden" name="a" value="1" />';
        if ($site_templates_list) {
            print
                '<fieldset><legend><a href="http://cmonos.jp/download/" target="_blank">CMONOS.JP</a> からサイトテンプレートを読み込む</legend><ol class="checkbox">';
            while ( $site_templates_list =~ /^(.*?)$/mg ) {
                my $line = $1;
                chomp $line;
                my ( $file, $name, $summary ) = split( /\x09/, $line );
                print '<li><label><input type="radio" name="filename" value="'
                    . $file . '"'
                    . ( ( $main::filename && $main::filename eq $file )
                    ? ' checked="checked"'
                    : '' )
                    . ' />'
                    . $name . ' : '
                    . $summary
                    . '</label></li>';
            }
            print '</ol></fieldset>';
        }
        print
            '<fieldset><legend><label for="file">サイトテンプレートファイル(tar-gzip)をアップロードする</label></legend><input type="file" name="file" id="file" /></fieldset>';
        print
            '<fieldset><legend>同名のサイトテンプレートフォルダが既に存在する場合</legend><ol class="checkbox">';
        print '<li><label><input type="radio" name="replace" value="0" '
            . ( ( !$main::replace ) ? ' checked="checked"' : '' )
            . '/> サイトテンプレートフォルダ名に数字を追加する</label></li>';
        print '<li><label><input type="radio" name="replace" value="1" '
            . ( ($main::replace) ? ' checked="checked"' : '' )
            . '/> 新しいサイトテンプレートフォルダで同名のサイトテンプレートフォルダを上書きする</label></li>';
        print '</ol></fieldset>';
        if ( !$env::user_site_template_dir ) {
            print '<fieldset>';
            print '<legend>公開設定</legend>';
            print
                '<label><input type="checkbox" name="closed" id="closed" value="1"'
                . ( ($main::closed) ? ' checked="checked"' : '' )
                . ' /> 下位ディレクトリからこのサイトテンプレートを隠す。</label>';
            print '</fieldset>';
        }
        print
            '<fieldset class="submit"><input type="submit" value="サイトテンプレートを読み込む" /></fieldset>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 's701', 'SiteTemplateImporter' );

    #メニュー登録
    $setting::jump_menu{'s701'} = 'サイトテンプレート読み込み';

}

1;
