package SubDirSize;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use File::Find;
use strict;
use utf8;
######################################################################
# サブディレクトリサイズ一覧モジュール
# SubDirSize.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    $this->get_setup_env;

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>サブディレクトリサイズ一覧</h1>';

    my ($list)
        = $this->sub_directory_size_list(
        $default::cgi_data_root . ( $default::home_dir || '' ) );
    if ($list) {
        print '<ul>' . $list . '</ul>';
    }
    else {
        print '<p>サブディレクトリがみつかりません。</p>';
    }

    #ページフッタ
    $this->print_page_footer('tools');
}

sub sub_directory_size_list ($;$$) {
    my ( $this, $dir, $home_dir, $folder ) = @_;
    my ( $size, $count ) = ( 0, 0 );
    my @subdirs = ();
    my @files   = ();
    my $list    = '';
    my $sublist = '';
    $home_dir = '' if !defined $home_dir;
    local (*DIR);
    return ( '', 0, 0 ) unless opendir( DIR, $dir );
    @files = grep( $_ !~ /^\.(?![0-9a-zA-Z_])/, readdir(DIR) );
    closedir(DIR);

    foreach my $file (@files) {
        my $path = $dir . '/' . $file;
        if ( -f $path ) {
            $size += -s $path;
            $count++;
        }
        elsif ( -d $path ) {
            my ( $sub, $s, $c ) = ( '', 0, 0 );
            if ( -f $path . '/lib/env.pl' ) {
                ( $sub, $s, $c )
                    = $this->sub_directory_size_list( $path,
                    $home_dir . '/' . $file, $file );
                $sublist .= $sub;
                push( @subdirs, $file );
            }
            else {
                ( $s, $c ) = $this->directory_size($path);
            }
            $size  += $s;
            $count += $c;
        }
    }
    if ( opendir( DIR, '../..' . ( $home_dir || '/' ) ) ) {
        my @files = grep( $_ !~ /^\.(?![0-9a-zA-Z_])/, readdir(DIR) );
        closedir(DIR);
        foreach my $file (@files) {
            my $path = '../..' . ( $home_dir || '/' ) . '/' . $file;
            if ( -f $path ) {
                $size += -s $path;
                $count++;
            }
            elsif ( -d $path && !grep( $_ eq $file, @subdirs ) ) {
                my ( $s, $c ) = $this->directory_size($path);
                $size  += $s;
                $count += $c;
            }
        }
    }
    my $mb = int( $size / 10000 + 0.5 ) / 100;
    return (
        '<li><span style="font-size:'
            . ( 1 + ( ( $mb && $mb > 1000 ) ? 1 : ( int($mb) / 1000 ) ) )
            . 'em">'
            . ( $folder || 'このディレクトリ' )
            . $this->subdir_name( $dir . '/lib/env.pl' ) . ' = '
            . $mb . 'MB/'
            . $count
            . 'ファイル</span>'
            . ( ($sublist) ? '<ul>' . $sublist . '</ul>' : '' ) . '</li>',
        $size, $count
    );
}

sub directory_size ($$) {
    my ( $this, $path ) = @_;
    my ( $size, $count ) = ( 0, 0 );
    find(
        sub {
            if ( -f $_ ) { $size += -s $_; $count++; }
        },
        $path
    );
    return ( $size, $count );
}

sub subdir_name ($$) {
    my ( $this, $path ) = @_;
    my $data = $this->read_file($path);
    if ( $data
        =~ /^[\x20\x09]*\$title[\x20\x09]*=[\x09\x0A\x0C\x0D\x20]*["']([^'"]*)["'];/m
        )
    {
        return ' (' . $1 . ')';
    }
    else {
        return '';
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ( !$env::subdirectory_disabled ) {

        #クラス登録
        Common::class_register( 's503', 'SubDirSize' );

        #メニュー登録
        $setting::jump_menu{'s503'}
            = 'サブディレクトリサイズ一覧';
    }
}

1;
