package DL;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
######################################################################
# ダウンロードリスト表示モジュール
# DL.pm ver1.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2012-09-14
######################################################################
# メイン
sub run ($) {
    my $this = shift;
    $this->set_base( $this->my_url );
    $this->key_is_not_found if !$main::key;
    $setting::general{'dcMaxDownload'}
        = ( $setting::general{'dcMaxDownload'} )
        ? $setting::general{'dcMaxDownload'}
        : 5;
    $setting::general{'dcLifetime'}
        = ( $setting::general{'dcLifetime'} )
        ? $setting::general{'dcLifetime'}
        : 30;
    $this->{'download_folder'} = $default::file . '/download/' . $main::key;
    $this->key_is_not_found if !$this->key_is_available;
    if ( -f $this->{'download_folder'} . '/list.dat' ) {
        $this->{'dc_unmaintainable'} = 1;
        $this->{'dc_archive_enabled'}
            = ( $setting::general{'dcArchiveEnabled'}
                && eval 'use Archive::Zip;1;' ) ? 1 : 0;
    }
    if ( $main::file && $this->file_is_available($main::file) ) {
        if ( $this->{'dc_unmaintainable'} ) {
            $this->print_download_file;
        }
        else {
            $this->jump(
                $this->path_change(
                    $this->{'download_folder'} . '/' . $main::file,
                    $ENV{'SCRIPT_NAME'}, 'abs'
                )
            );
        }
    }
    my %files = $this->file_list;
    $this->print_page_header($setting::key);
    my $list = '<ul id="download-list">';
    foreach my $file ( sort keys %files ) {
        if ( $files{$file} ) {
            $list
                .= '<li><a href="'
                . $cgi::path . '/'
                . $file
                . '?m=DL&amp;key='
                . $main::key
                . '&amp;file='
                . $file . '">'
                . $file
                . '</a> (残り'
                . $files{$file}
                . '回)</li>';
        }
        else {
            $list
                .= '<li><del>'
                . $file
                . '</del> (残り'
                . $files{$file}
                . '回)</li>';
        }
    }
    $list .= '</ul>';
    my $lifetime = int( $setting::general{'dcLifetime'}
            - ( -M $this->{'download_folder'} . '/' . 'checkout.dat' ) );
    my $template = '';
    if ( $setting::page{'downloadMsg'} ) {
        $template = $setting::page{'dl'}{'downloadMsg'};
    }
    else {
        $template = <<TEMP;
<p>ダウンロード期限まであと<strong><lifetime></strong>日です。</p>
<list>
<p>() はダウンロードできる残り回数</p>
TEMP
    }
    $template =~ s/<lifetime>/$lifetime/gis;
    $template .= $list unless $template =~ s/<list>/$list/gis;
    print $template;
    $this->print_page_footer($setting::key);
}

sub key_is_available ($) {
    my $this = shift;
    return 0 if !-d $this->{'download_folder'};
    my $path = $this->{'download_folder'} . '/checkout.dat';
    if ( !-s $path ) {
        $this->write_file( $path, time, 1 );
        return 1;
    }
    elsif ( -M $path > $setting::general{'dcLifetime'} ) {
        return 0;
    }
    else {
        return 1;
    }
}

sub file_is_available ($$;$) {
    my ( $this, $file, $read_only ) = @_;
    my $ip = $ENV{'REMOTE_HOST'} || $this->to_host( $ENV{'REMOTE_ADDR'} );
    if ( $this->{'dc_unmaintainable'} ) {
        $this->{'dc_list'} = {};
        local (*LIST);
        my $remains   = 0;
        my $path      = $this->{'download_folder'} . '/list.dat';
        my $temp_path = $path . '.dat';
        if ( open( LIST, $path ) ) {
            binmode LIST, ":utf8";
            flock( LIST, LOCK_SH );
            if ( open( TMP, ">>$temp_path" ) ) {
                binmode TMP, ":utf8";
                flock( TMP, LOCK_EX );
                truncate( TMP, 0 );
                seek( TMP, 0, 0 );
                while ( my $data = <LIST> ) {
                    if (   $this->{'dc_archive_enabled'}
                        || $data =~ /^$file\x09/ )
                    {
                        chomp $data;
                        my ( $file_name, $path, $count, $prev_ip, $atime,
                            $max )
                            = split( /\x09/, $data );
                        $count   = 0  if !$count;
                        $prev_ip = '' if !$prev_ip;
                        $atime   = '' if !$atime;
                        $max = $setting::general{'dcMaxDownload'} if !$max;
                        if ( !-f $path
                            && $path
                            =~ m!^(.*?)/([^/]+)/([^/]+)/([0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.[0-9a-zA-Z_]+\.dat)$!
                            )
                        {
                            my ( $dir, $c, $n, $dat ) = ( $1, $2, $3, $4 );
                            my $record_id = $this->read_file(
                                $dir . '/record_id_alias.dat', 1 );
                            if ($record_id) {
                                my %record_id;
                                eval 'use ' . $env::dbm_file . ';';
                                tie( %record_id, $env::dbm_file, $record_id,
                                    O_RDONLY, $env::dbm_mode );
                                if (   ref tied %record_id
                                    && $record_id{$n}
                                    && $record_id{$n} ne $c )
                                {
                                    $path
                                        = $dir . '/'
                                        . $record_id{$n} . '/'
                                        . $n . '/'
                                        . $dat;
                                }
                                untie %record_id;
                            }
                        }
                        if ( -f $path ) {
                            $remains = $max - $count;
                            if ($remains) {
                                $count++;
                                $atime                         = time;
                                $prev_ip                       = $ip;
                                $this->{'dc_list'}{$file_name} = $path;
                            }
                            else {
                                push(
                                    @{ $this->{'dialog'} },
                                    $file
                                        . 'のダウンロード回数は上限に達しています。'
                                );
                            }
                        }
                        else {
                            push(
                                @{ $this->{'dialog'} },
                                $file
                                    . 'がみつかりませんでした。'
                            );
                            $count = $max;
                        }
                        print TMP join( "\x09",
                            $file_name, $path, $count, $prev_ip, $atime,
                            $max )
                            . "\x0A";
                    }
                    else {
                        print TMP $data;
                    }
                }
                close(TMP);
            }
            close(LIST);
            rename( $temp_path, $path );
        }
        return $remains;
    }
    else {
        my $path     = $this->{'download_folder'} . '/' . $file;
        my $dat_path = $path . '.dat';
        if ( -f $path && -f $dat_path ) {
            my ( $count, $prev_ip, $atime, $max )
                = split( /\x09/, $this->read_file( $dat_path, 1 ) );
            $count = 0 if !$count;
            $max = $setting::general{'dcMaxDownload'} if !$max;
            my $remains = $max - $count;
            $remains = 0 if $remains < 0;
            if ( !$remains ) {
                push(
                    @{ $this->{'dialog'} },
                    $file
                        . 'のダウンロード回数は上限に達しています。'
                );
                return 0;
            }
            elsif (
                !$read_only
                && (   !$prev_ip
                    || !$atime
                    || $ip ne $prev_ip
                    || ( time - $atime ) > 10800 )
                )
            {
                $this->write_file( $dat_path,
                    join( "\x09", ++$count, $ip, time, $max ), 1 );
            }
            $this->write_file(
                $this->{'download_folder'} . '/.htaccess',
                'Satisfy all' . "\x0A"
                    . 'order deny,allow' . "\x0A"
                    . 'deny from all' . "\x0A"
                    . 'allow from '
                    . $ip . "\x0A"

                    . '<IfModule mod_headers.c>' . "\x0A"
                    . 'Header set content-disposition attachment' . "\x0A"
                    . '</IfModule>' . "\x0A",
                1
            ) if !$read_only;
            return $remains;
        }
        push(
            @{ $this->{'dialog'} },
            $file . 'がみつかりませんでした。'
        );
    }
    return 0;
}

sub file_list ($) {
    my $this   = shift;
    my %files  = ();
    my $nofile = 1;
    if ( $this->{'dc_unmaintainable'} ) {
        local (*LIST);
        if ( open( LIST, $this->{'download_folder'} . '/list.dat' ) ) {
            binmode LIST, ":utf8";
            flock( LIST, LOCK_SH );
            while ( my $data = <LIST> ) {
                chomp $data;
                my ( $file, $path, $count, $prev_ip, $atime, $max )
                    = split( /\x09/, $data );
                $count = 0 if !$count;
                $max = $setting::general{'dcMaxDownload'} if !$max;
                my $remains = $max - $count;
                $nofile = 0 if $remains;
                if ( $this->{'dc_archive_enabled'} ) {
                    if (%files) {
                        %files = ();
                        $files{'download.zip'} = $remains;
                        last;
                    }
                    elsif ( $file =~ /\.zip$/i ) {
                        $files{$file} = $remains;
                    }
                    else {
                        $files{'download.zip'} = $remains;
                    }
                }
                else {
                    $files{$file} = $remains;
                }
            }
            close(LIST);
        }
    }
    else {
        local (*DIR);
        if ( opendir( DIR, $this->{'download_folder'} ) ) {
            while ( my $file = readdir(DIR) ) {
                next
                    if $file =~ /^\./
                    || $file =~ /\.dat$/
                    || $file eq 'index.html';
                my $remains = $this->file_is_available( $file, 1 );
                $files{$file} = $remains;
                $nofile = 0 if $remains;
            }
            closedir(DIR);
        }
    }
    $this->key_is_not_found if $nofile;
    return %files;
}

# Not Found
sub key_is_not_found ($) {
    my $this = shift;
    if ( $this->{'download_folder'} ) {
        use File::Path;
        rmtree $this->{'download_folder'};
    }
    $this->print_page_header($setting::key);
    if ( $setting::page{'dl'}{'errorMsg'} ) {
        print $setting::page{'dl'}{'errorMsg'};
    }
    else {
        print
            '<p>指定されたダウンロードファイルは存在しません。URL に誤りがあるか、ダウンロード期限を過ぎたため、もしくは、ダウンロードが許可された回数を超えたために、ファイルが削除された可能性があります。</p>';
    }
    $this->print_page_footer($setting::key);
    exit;
}

# ダウンロードファイル書き出し
sub print_download_file ($) {
    my $this = shift;
    my ( $download_file, $zip );
    if ( $this->{'dc_archive_enabled'} ) {
        my @list = keys %{ $this->{'dc_list'} };
        if ( @list > 1 || $list[0] !~ /\.zip$/i ) {
            $download_file = $this->{'download_folder'} . '/download.zip';
            $zip           = Archive::Zip->new;
        }
    }
    while ( my ( $file, $path ) = each %{ $this->{'dc_list'} } ) {
        if ($download_file) {
            $zip->addFile( $path, $file );
        }
        else {
            $this->print_stdout_download_file( $path, $file );
            exit;
        }
    }
    if ($download_file) {
        $zip->writeToFileNamed($download_file);
        $this->print_stdout_download_file( $download_file, 'download.zip' );
        unlink $download_file;
    }
    exit;
}

sub print_stdout_download_file ($$$) {
    my ( $this, $path, $file ) = @_;
    my $size = -s $path;
    my $type
        = (    $file
            && %Common::mime_type
            && $file =~ /\.([0-9a-zA-Z_]+)$/
            && $Common::mime_type{ lc $1 } )
        ? $Common::mime_type{ lc $1 }
        : 'application/octet-stream';
    binmode STDOUT;
    print "Content-Type: $type\n";
    print "Content-Length: $size\n";
    print "Content-Disposition: attachment\n";
    print "\n";
    local (*FILE);

    if ( open( FILE, $path ) ) {
        binmode FILE;
        flock( FILE, LOCK_SH );
        my $buf;
        while ( read( FILE, $buf, $env::buffer_size ) ) {
            print $buf;
        }
        close(FILE);
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ( $default::type && $default::type eq 'order' ) {

        #設定DB読み込み設定
        $setting::general = 1 if !$setting::general; #一般設定読み込み
        $setting::deco = 1
            if !$setting::deco;    #テキスト装飾設定読み込み
        $setting::security = 1
            if !$setting::security;    #アクセス権設定読み込み
        $setting::page = 1 if !$setting::page;    #ページ設定読み込み

        #設定種キー設定
        $setting::security_key = 'dl'
            if !defined $setting::security_key
            || $setting::security_key ne
            'dl';    #デフォルトセキュリティキー
        $setting::key = 'dl';

        #クッキー
        $cookie::id_enabled = 1
            if !$cookie::id_enabled;    #ID情報クッキー読み込み

        #デフォルトキー
        $main::k = 'dl' if !defined $main::k || !$main::k;

        #必須フォーム
        @setting::form = ('m') if !@setting::form;

        #クラス登録
        Common::class_register( 'dl', 'DL' );
    }
}

1;
