package SerialNo;
require CMONOS::CGI_Lib::Field::Num;
@ISA = qw(Num);
use strict;
use utf8;
######################################################################
# フィールド定義モジュール（数）
# Num.pm 1.1.3
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2012-06-17
######################################################################
sub _set ($$) {
    my ( $this, $setting ) = @_;
    $this->{'cols'}   = '16' if !$this->{'cols'};
    $this->{'timeid'} = 1    if !defined $this->{'timeid'};
    $this->{'field_for'}{'serialno'} = $this->{'id'};
    return;
}

######################################################################
#	(0)-8 obj->align;
sub align ($) {
    my $this = shift;
    return 'left';
}

sub number_from_data ($$) {
    my ( $this, $str ) = @_;
    return ( defined $str && $str =~ /[0-9]/ ) ? $str + 0 : $str;
}

######################################################################
#	(2) obj->form;
sub input_enabled ($) {
    my $this = shift;
    return 0;
}

sub form {
    my ( $this, $status ) = @_;
    return '';
}

######################################################################
#	(3) obj->is_valid_data(path);
sub is_valid_data ($;$) {
    my $this = shift;
    return 1;
}

sub save_serial_no ($$$) {
    my ( $this, $c, $n ) = @_;
    return 1 if !defined $n || $n eq '' || !defined $c || $c eq '';
    my ( %db, $no, $db );
    $db = $default::db . '/' . $c . '/' . $this->db;
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    tie( %db, $env::dbm_file, $db, O_RDONLY, $env::dbm_mode );
    if ( ref tied %db && defined $db{$n} ) {
        $no = $this->decode_utf8( $db{$n} );
    }
    untie %db;
    if ( !defined $no || $no eq '' ) {
        if ( $this->{'timeid'} ) {
            $no = $this->time_id . sprintf( "%.2d", substr( $n, -2 ) );
        }
        elsif ( $this->{'digits'} ) {
            $no = sprintf( '%.' . $this->{'digits'} . 'd', $n );
        }
        else {
            $no = $n;
        }
        tie( %db, $env::dbm_file, $db, O_RDWR | O_CREAT, $env::dbm_mode );
        if ( ref tied %db ) {
            $db{ $this->encode_utf8($n) } = $this->encode_utf8($no);
        }
        untie %db;
    }
    return 1;
}

######################################################################
sub usual_format {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return ( defined $str && $str ne '' )
        ? $this->field_tag(
        $this->style(
            $this->data_tag( 'str', $str ), $this->{'style'},
            $this->{'fontsize'}, $this->{'fontcolor'},
            $this->{'fontface'}
        )
        )
        : $this->field_tag('');
}

sub format_for_mail {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return ( defined $str && $str ne '' )
        ? '[' . $this->name . "]\x0A" . $str . "\x0A\x0A"
        : '';
}

sub format_for_text {
    my ( $this, $str ) = @_;
    $str = $$str{'str'} if ref $str eq 'HASH';
    return ( defined $str && $str ne '' ) ? $str : '';
}

######################################################################
#	(5) obj->print_setting_form;
sub print_setting_form ($;$$) {
    my ( $this, $html_disabled ) = @_;

    #名前
    my $name = 'data:' . $this->{'id'};

    $this->print_field_info_form( $name, 1 );

    print '<fieldset>';
    print '<legend>番号の形式</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="radio" name="'
        . $name
        . '_timeid" value="0"'
        . ( ( !$this->{'timeid'} ) ? ' checked="checked"' : '' ) . ' /> ';
    print $this->mk_select_menu(
        $name . '_digits', '', '',
        ( ( defined $this->{'digits'} ) ? $this->{'digits'} : '' ),
        '' => '桁揃えなし',
        3  => '3桁',
        4  => '4桁',
        5  => '5桁',
        6  => '6桁',
        7  => '7桁',
        8  => '8桁',
        9  => '9桁',
        10 => '10桁',
        11 => '11桁',
        12 => '12桁',
        13 => '13桁',
        14 => '14桁',
        15 => '15桁',
        16 => '16桁',
    );
    my $no = int( rand(1000) );
    print 'のレコードID (レコードIDが '
        . $no
        . ' のとき '
        . ( ( $this->{'digits'} )
        ? sprintf( '%.' . $this->{'digits'} . 'd', $no )
        : $no )
        . ')。</label></li>';
    print '<li><label><input type="radio" name="'
        . $name
        . '_timeid" value="1"'
        . ( ( $this->{'timeid'} ) ? ' checked="checked"' : '' )
        . ' /> 作成時+レコードIDの下2桁 (レコードIDが '
        . $no
        . ' のとき '
        . $this->time_id
        . sprintf( "%.2d", substr( $no, -2 ) )
        . ')。</label></li>';
    print '</ol></fieldset>';

    if ( !defined $html_disabled || !$html_disabled ) {

        #検索オプション
        $this->print_search_option_settings($name);

        #文字設定
        $this->print_font_setting_form($name);
    }
    $this->print_setting_save_button;
    return;
}

######################################################################
#	(6) obj->is_valid_setting;
sub is_valid_setting ($) {
    my $this = shift;
    $main::data{ $this->{'id'} . '_searchOption' } = 0
        if !$main::data{ $this->{'id'} . '_searchOption' };
    if ( $main::data{ $this->{'id'} . '_searchMenu' } ) {
        $main::data{ $this->{'id'} . '_searchMenu' } = join(
            "\x0A",
            sort { $a <=> $b } grep( !$this->is_nan($_) && $_ ne '',
                split(
                    /[\r\n]+/, $main::data{ $this->{'id'} . '_searchMenu' }
                ) )
        );
    }
    return $this->is_valid_font_setting;
}

######################################################################
#	(9) obj->cookie_enabled;
sub cookie_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#	(10) obj->required_enabled;
sub required_enabled ($) {
    my $this = shift;
    return 0;
}

######################################################################
#sub import {
BEGIN {
    if ( !$setting::field_for_mail ) {
        Field::class_register( 'xtm_serialno', 'SerialNo',
            'レコード番号' );
        push( @field::redundancy_forbidden, 'xtm_serialno' );
    }
}

1;
