package MkPaymentTerms;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# 手数料・送料モジュール
# MkPaymentTerms.pm ver1.1.9
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-03-24
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    require CMONOS::CGI_Lib::JPrefecture;
    $cgi::JPrefecture = JPrefecture->new;
    my %pref_name_table   = $cgi::JPrefecture->get_prefecture_name_hash;
    my %region_name_table = $cgi::JPrefecture->get_region_name_hash;
    my ( %discount, %bank, %other, %weight );

    #書き込み
    if ( defined $main::a && $main::a eq 'w' ) {
        no warnings qw(numeric);
        my $is_valid = 1;
        my ( $no, $num );

        $no = $num = 0;
        foreach my $n (
            sort {
                $main::discount{$b}{'amount'}
                    <=> $main::discount{$a}{'amount'}
                    || $a <=> $b
            } keys %main::discount
            )
        {
            if (   !$this->is_not_digit( $main::discount{$n}{'amount'} )
                && !$this->is_nan( $main::discount{$n}{'ratio'} )
                && $main::discount{$n}{'ratio'} > 0
                && $main::discount{$n}{'ratio'} < 100 )
            {
                $discount{ $main::discount{$n}{'amount'} }
                    = $main::data{ 'discount_'
                        . $main::discount{$n}{'amount'} }
                    = $main::discount{$n}{'ratio'};
            }
        }
        undef %main::discount;
        $main::data{'discountlimited'} = 0
            if !defined $main::data{'discountlimited'}
            || !$main::data{'discountlimited'};
        $main::data{'discountrounddown'} = ''
            if !$main::data{'discountrounddown'}
            || $main::data{'discountrounddown'} =~ /[^0-9]/;

        $is_valid = 0
            if $this->is_not_number( \$main::data{'minimumamount'},
            '購入総額の下限額', 1 );
        $is_valid = 0
            if $this->is_not_number( \$main::data{'maximumamount'},
            '購入総額の上限額', 1 );
        $main::data{'limitlimited'} = 0
            if !defined $main::data{'limitlimited'}
            || !$main::data{'limitlimited'};
        $main::data{'pricerequired'} = 0
            if !defined $main::data{'pricerequired'}
            || !$main::data{'pricerequired'};
        $main::data{'cancelnolimit'} = 0
            if !defined $main::data{'cancelnolimit'}
            || !$main::data{'cancelnolimit'};

        $no = $num = 0;
        foreach my $id (
            sort { $main::order{$a} <=> $main::order{$b} || $a cmp $b }
            keys %main::order )
        {
            $main::data{ 'order_' . $id } = ++$no;
        }

        $main::data{'bankenabled'} = 0
            if !defined $main::data{'bankenabled'}
            || !$main::data{'bankenabled'};
        $no = $num = 0;
        foreach my $n (
            sort {
                $main::bank{$a}{'order'} <=> $main::bank{$b}{'order'}
                    || $a <=> $b
            } keys %main::bank
            )
        {
            next if $main::bank{$n}{'delete'} || !$main::bank{$n}{'bank'};
            $num++;
            $bank{$num}{'bank'}    = $main::bank{$n}{'bank'};
            $bank{$num}{'office'}  = $main::bank{$n}{'office'};
            $bank{$num}{'deposit'} = $main::bank{$n}{'deposit'};
            $bank{$num}{'number'}  = $main::bank{$n}{'number'};
            $bank{$num}{'name'}    = $main::bank{$n}{'name'};
            $bank{$num}{'kana'}    = $main::bank{$n}{'kana'};

            if (!$this->is_not_number( \$main::bank{$n}{'number'},
                    $main::bank{$n}{'bank'} . '口座番号', 1 )
                && !$this->is_not_kana_name( \$main::bank{$n}{'kana'},
                    $main::bank{$n}{'bank'} . 'カタカナ表記' )
                && !$this->is_shortage(
                    $main::bank{$n}{'bank'},   $main::bank{$n}{'bank'},
                    $main::bank{$n}{'office'}, $main::bank{$n}{'number'},
                    $main::bank{$n}{'name'},   $main::bank{$n}{'kana'}
                )
                )
            {
                $no++;
                $main::data{ 'bank_' . $no . '_bank' }
                    = $main::bank{$n}{'bank'};
                $main::data{ 'bank_' . $no . '_office' }
                    = $main::bank{$n}{'office'};
                $main::data{ 'bank_' . $no . '_deposit' }
                    = $main::bank{$n}{'deposit'};
                $main::data{ 'bank_' . $no . '_number' }
                    = $main::bank{$n}{'number'};
                $main::data{ 'bank_' . $no . '_name' }
                    = $main::bank{$n}{'name'};
                $main::data{ 'bank_' . $no . '_kana' }
                    = $main::bank{$n}{'kana'};
            }
            else {
                $is_valid = 0;
            }
        }
        undef %main::bank;

        $main::data{'yubinenabled'} = 0
            if !defined $main::data{'yubinenabled'}
            || !$main::data{'yubinenabled'};
        $is_valid = 0
            if $this->is_not_number( \$main::data{'yubin_mark'},
            'ゆうちょ銀行口座の記号', 1 );
        $is_valid = 0
            if $this->is_not_number( \$main::data{'yubin_digit'},
            'ゆうちょ銀行口座のディジット', 1 );
        $is_valid = 0
            if $this->is_not_number( \$main::data{'yubin_number'},
            'ゆうちょ銀行口座の番号', 1 );
        if (   $main::data{'yubin_mark'}
            && $main::data{'yubin_mark'} !~ /^[01][0-9]{2}/ )
        {
            push(
                @{ $this->{'dialog'} },
                'ゆうちょ銀行口座の記号を確認してください。'
            );
            $is_valid = 0;
        }
        $is_valid = 0
            if $this->is_shortage(
            'ゆうちょ銀行口座',
            $main::data{'yubin_mark'},
            $main::data{'yubin_number'}
            );

        $main::data{'codenabled'} = 0
            if !defined $main::data{'codenabled'}
            || !$main::data{'codenabled'};
        $main::data{'cod_dcUnavailable'} = 0
            if !defined $main::data{'cod_dcUnavailable'}
            || !$main::data{'cod_dcUnavailable'};
        $is_valid = 0
            if $this->is_not_number( \$main::data{'cod_fee1'},
            '代引き手数料(1万円未満)' );
        $is_valid = 0
            if $this->is_not_number( \$main::data{'cod_fee2'},
            '代引き手数料(1万円以上3万円未満)' );
        $is_valid = 0
            if $this->is_not_number( \$main::data{'cod_fee3'},
            '代引き手数料(3万円以上10万円未満)' );
        $is_valid = 0
            if $this->is_not_number( \$main::data{'cod_fee4'},
            '代引き手数料(10万円以上30万円未満)' );

        if ($main::data{'codenabled'}
            && ( !defined $main::data{'cod_fee1'}
                || $main::data{'cod_fee1'} eq '' )
            )
        {
            push(
                @{ $this->{'dialog'} },
                '代引き手数料が設定されていません。'
            );
            $is_valid = 0;
        }
        $main::data{'cod_fee3'} = $main::data{'cod_fee4'} = ''
            if !defined $main::data{'cod_fee3'}
            || $main::data{'cod_fee3'} eq '';
        $main::data{'cod_fee2'} = $main::data{'cod_fee3'}
            = $main::data{'cod_fee4'} = ''
            if !defined $main::data{'cod_fee2'}
            || $main::data{'cod_fee2'} eq '';
        $main::data{'cod_fee1'} = $main::data{'cod_fee2'}
            = $main::data{'cod_fee3'} = $main::data{'cod_fee4'} = ''
            if !defined $main::data{'cod_fee1'}
            || $main::data{'cod_fee1'} eq '';

        $main::data{'moneyorderenabled'} = 0
            if !defined $main::data{'moneyorderenabled'}
            || !$main::data{'moneyorderenabled'};

        $main::data{'paypalenabled'} = 0
            if !defined $main::data{'paypalenabled'}
            || !$main::data{'paypalenabled'};
        $is_valid = 0
            if $this->is_not_number( \$main::data{'paypal_ratio'},
            '取引当たりの手数料率(参考)' );
        $is_valid = 0
            if $this->is_not_number( \$main::data{'paypal_fee'},
            '取引当たりの固定手数料(参考)' );
        $main::data{'paypal_noshippingsale'} = 0
            if !defined $main::data{'paypal_noshippingsale'}
            || !$main::data{'paypal_noshippingsale'};
        $main::data{'paypal_incontext'} = 0
            if !defined $main::data{'paypal_incontext'}
            || !$main::data{'paypal_incontext'};
        $is_valid = 0
            if $this->is_not_number( \$main::data{'paypal_mratio'},
            'マイクロペイメント手数料率(参考)' );
        $is_valid = 0
            if $this->is_not_number( \$main::data{'paypal_mfee'},
            'マイクロペイメント固定手数料(参考)' );
        delete $main::data{'paypal_paymentaction'}
            if $main::data{'paypal_paymentaction'}
            && $main::data{'paypal_paymentaction'} ne 'Order'
            && $main::data{'paypal_paymentaction'} ne 'Sale';
        $main::data{'paypal_username'}
            = $this->z_to_h( $main::data{'paypal_username'} )
            if $main::data{'paypal_username'};
        $main::data{'paypal_password'}
            = $this->z_to_h( $main::paypal{'password'} )
            if $main::paypal{'password'};
        $main::data{'paypal_signature'}
            = $this->z_to_h( $main::paypal{'signature'} )
            if $main::paypal{'signature'};

        if ($main::data{'paypalenabled'}
            && (!$main::data{'paypal_username'}
                || (   !$main::paypal{'password'}
                    && !$main::paypal{'old_password'} )
                || (   !$main::paypal{'signature'}
                    && !$main::paypal{'old_signature'} )
            )
            )
        {
            push(
                @{ $this->{'dialog'} },
                'PayPal を有効にするには「APIユーザー名」「APIパスワード」「API署名」が必要です。'
            );
            $is_valid = 0;
            $main::data{'paypalenabled'} = 0;
        }
        $main::data{'paypal_sandbox'} = 0
            if !defined $main::data{'paypal_sandbox'}
            || !$main::data{'paypal_sandbox'};
        $main::data{'paypal_top'} = 0
            if !defined $main::data{'paypal_top'}
            || !$main::data{'paypal_top'};
        $is_valid = 0
            if $this->is_not_image(
            \$main::data{'paypal_checkoutbutton'},
            'チェックアウトボタン画像'
            );
        $is_valid = 0
            if $this->is_not_image( \$main::data{'paypal_button'},
            '購入ボタン画像' );
        $main::data{'paypal_pagestyle'}
            = $this->z_to_h( $main::data{'paypal_pagestyle'} )
            if $main::data{'paypal_pagestyle'};

        $main::data{'otherenabled'} = 0
            if !defined $main::data{'otherenabled'}
            || !$main::data{'otherenabled'};
        $no = $num = 0;
        foreach my $n (
            sort {
                $main::other{$a}{'order'} <=> $main::other{$b}{'order'}
                    || $a <=> $b
            } keys %main::other
            )
        {
            next if $main::other{$n}{'delete'} || !$main::other{$n}{'name'};
            $num++;
            $other{$num}{'name'}    = $main::other{$n}{'name'};
            $other{$num}{'formula'} = $main::other{$n}{'formula'};
            $other{$num}{'frac'}    = $main::other{$n}{'frac'};
            $other{$num}{'note'}    = $main::other{$n}{'note'};
            if (!$this->is_invalid_formula( \$main::other{$n}{'formula'},
                    $main::other{$n}{'name'} . '手数料計算式' )
                && !$this->is_shortage(
                    $main::other{$n}{'name'},
                    $main::other{$n}{'name'}
                )
                )
            {
                $no++;
                $main::data{ 'other_' . $no . '_name' }
                    = $main::other{$n}{'name'};
                $main::data{ 'other_' . $no . '_formula' }
                    = $main::other{$n}{'formula'};
                $main::data{ 'other_' . $no . '_frac' }
                    = $main::other{$n}{'frac'};
                $main::data{ 'other_' . $no . '_note' }
                    = $main::other{$n}{'note'};
            }
            else {
                $is_valid = 0;
            }
        }
        undef %main::other;

        $is_valid = 0
            if $this->is_not_number( \$main::data{'refundfee'},
            '払い戻し手数料' );
        $main::data{'paypalrefundfee'} = 0 if !$main::data{'paypalrefundfee'};

        if ( !defined $main::data{'carriage'}
            || $main::data{'carriage'} eq '' )
        {
            push(
                @{ $this->{'dialog'} },
                '送料区分が選択されていません。'
            );
            $is_valid = 0;
        }
        $is_valid = 0
            if $this->is_not_number( \$main::data{'nation'},
            '全国一律送料' );
        my ($id);
        foreach $id ( sort { $a <=> $b } keys %region_name_table ) {
            if ((      $main::data{'carriage'}
                    && $main::data{'carriage'} eq 'region'
                    && ( !defined $main::data{ 'region_' . $id }
                        || $main::data{ 'region_' . $id } eq '' )
                )
                || $this->is_not_number(
                    \$main::data{ 'region_' . $id },
                    $region_name_table{$id} . '送料'
                )
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    $region_name_table{$id}
                        . 'の送料設定を確認してください。'
                );
                $is_valid = 0;
            }
        }
        foreach $id ( sort { $a <=> $b } keys %pref_name_table ) {
            if ((      $main::data{'carriage'}
                    && $main::data{'carriage'} eq 'prefecture'
                    && ( !defined $main::data{ 'pref_' . $id }
                        || $main::data{ 'pref_' . $id } eq '' )
                )
                || $this->is_not_number(
                    \$main::data{ 'pref_' . $id },
                    $pref_name_table{$id} . '送料'
                )
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    $pref_name_table{$id}
                        . 'の送料設定を確認してください。'
                );
                $is_valid = 0;
            }
        }
        $main::data{'yuumail'} = 0
            if !defined $main::data{'yuumail'} || !$main::data{'yuumail'};
        $no = $num = 0;
        foreach my $n (
            sort {
                $main::weight{$a}{'weight'} <=> $main::weight{$b}{'weight'}
                    || $a <=> $b
            } keys %main::weight
            )
        {
            next
                if $main::weight{$n}{'delete'}
                || !$main::weight{$n}{'weight'};
            $num++;
            $weight{$num}{'weight'}   = $main::weight{$n}{'weight'};
            $weight{$num}{'carriage'} = $main::weight{$n}{'carriage'};
            if (   !$this->is_nan( $weight{$num}{'weight'} )
                && !$this->is_nan( $weight{$num}{'carriage'} ) )
            {
                $no++;
                $main::data{ 'weight_' . $no . '_weight' }
                    = $main::weight{$n}{'weight'};
                $main::data{ 'weight_' . $no . '_carriage' }
                    = $main::weight{$n}{'carriage'};
            }
            else {
                $is_valid = 0;
            }
        }
        undef %main::weight;
        $is_valid = 0
            if $this->is_not_number( \$main::data{'packageweight'},
            '梱包重量' );

        $is_valid = 0
            if $this->is_not_number(
            \$main::data{'papersize_1A4'},
            'A4/1cmまでのサイズ別料金'
            );
        $is_valid = 0
            if $this->is_not_number(
            \$main::data{'papersize_accessory1'},
            'A4/1cmサイズに入る小物の数'
            );
        $is_valid = 0
            if $this->is_not_number(
            \$main::data{'papersize_2A4'},
            'A4/2cmまでのサイズ別料金'
            );
        $is_valid = 0
            if $this->is_not_number(
            \$main::data{'papersize_accessory2'},
            'A4/2cmサイズに入る小物の数'
            );
        $is_valid = 0
            if $this->is_not_number(
            \$main::data{'papersize_expressdelivery'},
            'サイズ別料金の速達料金'
            );
        $main::data{'mailbin'} = 0
            if !defined $main::data{'mailbin'} || !$main::data{'mailbin'};

        if ($main::data{'packagethickness'}
            && ($this->is_not_number( \$main::data{'packagethickness'},
                    '梱包材の厚さ' )
                || $main::data{'packagethickness'} < 0
                || $main::data{'packagethickness'} > 16
            )
            )
        {
            push(
                @{ $this->{'dialog'} },
                '梱包材の厚さは 20 以下の数字で設定してください。'
            );
            $is_valid = 0;
        }

        $is_valid = 0
            if $this->is_not_number( \$main::data{'expressdelivery'},
            '速達料金' );

        $is_valid = 0
            if $this->is_not_number(
            \$main::data{'specialcarriage_requirements'},
            '送料優遇条件価格' );
        $is_valid = 0
            if $this->is_not_number( \$main::data{'specialcarriage_cost'},
            '優遇全国一律送料' );
        $main::data{'estimatedCarriageRequired'} = 0
            if !defined $main::data{'estimatedCarriageRequired'}
            || !$main::data{'estimatedCarriageRequired'};

        $this->write_db( $default::preference_dir . '/terms',
            \%main::data, 'discount|bank|other|weight', '',
            \%setting::payment, 1 );
        if ($is_valid) {
            $this->print_payment_info_page( \%pref_name_table,
                \%region_name_table );
        }
        else {
            push(
                @{ $this->{'dialog'} },
                'いくつかの設定に問題があったため、お支払い情報ページを更新しませんでした。'
            );
        }

    }
    else {
        #読み込み
        $this->import_db( $default::preference_dir . '/terms',
            \%setting::payment, 1 );
        %bank
            = ( ref $setting::payment{'bank'} )
            ? %{ $setting::payment{'bank'} }
            : ();
        %other
            = ( ref $setting::payment{'other'} )
            ? %{ $setting::payment{'other'} }
            : ();
        %weight
            = ( ref $setting::payment{'weight'} )
            ? %{ $setting::payment{'weight'} }
            : ();
        %discount
            = ( ref $setting::payment{'discount'} )
            ? %{ $setting::payment{'discount'} }
            : ();
    }
    $setting::payment{'order'} = {
        'bank'       => 1,
        'yubin'      => 2,
        'paypal'     => 3,
        'cod'        => 4,
        'moneyorder' => 5,
        'other'      => 6
        }
        if !ref $setting::payment{'order'}
        || !%{ $setting::payment{'order'} };

    #ゆうメール用設定
    if ($main::for_yuumail) {
        undef %weight;
        %weight = (
            1 => { 'weight' => 150,  'carriage' => 180 },
            2 => { 'weight' => 250,  'carriage' => 215 },
            3 => { 'weight' => 500,  'carriage' => 300 },
            4 => { 'weight' => 1000, 'carriage' => 350 },
            5 => { 'weight' => 2000, 'carriage' => 460 },
            6 => { 'weight' => 3000, 'carriage' => 610 }
        );
    }

    #ゆうメール用設定
    if ( $main::for_mailbin || !defined $setting::payment{'papersize'} ) {
        $setting::payment{'papersize'}{'1A4'}             = 82;
        $setting::payment{'papersize'}{'2A4'}             = 164;
        $setting::payment{'papersize'}{'expressdelivery'} = 103;
    }

    #ページヘッダ
    $this->set_javascript(
        <<JAVA
cmonos.tool.prefSync = function () {
	if (document.getElementById('prefsync').checked == false) return;
	var v = arguments[0];
	if (arguments.length > 1) {
		for (var i=1; i<arguments.length; i++) {
			document.getElementById('data:pref_'+arguments[i]).value = v;
		}
	} else {
		for (var i=1; i<=47; i++) {
			document.getElementById('data:pref_'+i).value = v;
		}
	}
};
cmonos.tool.sortPaymentMethods = function () {
	var methods = document.getElementById('payment-methods');
	var temp = [];
	var order = [];
	var new_order = [];
	var n = 0;
	var method_ids = [];
	for (var i=0; i<methods.childNodes.length; i++) {
		var fieldset = methods.childNodes[i];
		if (fieldset.id) {
			var id = fieldset.id;
			method_ids[method_ids.length] = id;
			order[id] = n++;
			new_order[id] = parseInt(cmonos.form.field('sortmenu-'+id).getValue());
			temp[id] = fieldset;
			//methods.removeChild(methods.firstChild);
		}
	}
	while (methods.firstChild) {
		methods.removeChild(methods.firstChild);
	}
	method_ids.sort(function (a,b) {
		if (new_order[a] < new_order[b]) return -1;
		if (new_order[a] > new_order[b]) return 1;
		if (order[a] < order[b]) return 1;
		if (order[a] > order[b]) return -1;
		return 0;
	});
	for (var i=0; i<method_ids.length; i++) {
		methods.appendChild(temp[method_ids[i]]);
		cmonos.form.field('sortmenu-'+method_ids[i]).setValue(i+1)
	}
}
JAVA
    );
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="w" />';

    print '<fieldset><legend>消費税</legend>';
    print(
          ( $env::tax_rate * 100 )
        . '% (端数は'
            . (
            ( $env::tax_frac eq 'floor' ) ? '切り下げ'
            : ( ( $env::tax_frac eq 'round' ) ? '四捨五入'
                : '切り上げ' )
            )
            . ')'
    );
    print
        '<div class="hints">「セットアップアシスタント/消費税・カート情報変更」で変更できます。</div>';
    print '</fieldset>';

    if (  -f $default::file
        . '/payment_info.'
        . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' ) )
    {
        print '<fieldset><legend>お支払い情報ページ</legend>';
        print '<a href="'
            . $default::file
            . '/payment_info.'
            . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' )
            . '" target="_blank">お支払い方法および送料について...</a>';
        print '</fieldset>';
    }

#print '<fieldset><legend>ポイント</legend>';
#print $env::point_lifetime . ' ヶ月間有効';
#print '(端数ポイントは' . (($env::tax_frac eq 'floor') ? '切り上げ' : (($env::tax_frac eq 'round') ? '四捨五入' : '切り下げ')) . ')';
#print '<div class="hints">「セットアップアシスタント/消費税・カート情報変更」で変更できます。</div>';
#print '</fieldset>';

    my ( $count, $cell_limit );
    print '<h1>割引</h1>';

    print '<fieldset><legend>割引設定</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="radio" name="data:discountlimited" value="0"'
        . ( ( !$setting::payment{'discountlimited'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 全商品を割引対象にする</label></li>';
    print
        '<li><label><input type="radio" name="data:discountlimited" value="1"'
        . (
        ( $setting::payment{'discountlimited'} ) ? ' checked="checked"' : '' )
        . ' /> 「まとめ買い割引対象」に指定した商品のみを割引対象にする</label></li>';
    print '</ol>';
    print '<table class="tooltable" id="discount-fields">';
    print
        '<thead><tr class="odd"><th class="odd">購入総額</th><th class="even">割引率</th></tr></thead>';
    print '<tbody>';
    $count      = 0;
    $cell_limit = 1;
    $cell_limit += scalar( keys %discount ) if %discount;

    foreach my $amount ( sort { $a <=> $b || $a cmp $b } keys %discount ) {
        $count++;
        print '<tr class="' . ( ( $count % 2 ) ? 'even' : 'odd' ) . '">';
        print '<td class="odd"><input type="number" name="discount:'
            . $count
            . ':amount" id="discount:'
            . $count
            . ':amount" size="16" value="'
            . $this->escape($amount)
            . '" min="0" />円以上の購入で</td>';
        print '<td class="even"><input type="text" name="discount:'
            . $count
            . ':ratio" id="discount:'
            . $count
            . ':ratio" size="16" value="'
            . $this->escape( $discount{$amount} )
            . '" />% 引きにする</td>';
        print '</tr>';
    }
    $count++;
    print '<tr class="' . ( ( $count % 2 ) ? 'even' : 'odd' ) . '">';
    print '<td class="odd"><input type="number" name="discount:'
        . $count
        . ':amount" id="discount:'
        . $count
        . ':amount" size="16" min="0" />円以上の購入で</td>';
    print '<td class="even"><input type="text" name="discount:'
        . $count
        . ':ratio" id="discount:'
        . $count
        . ':ratio" size="16" />% 引きにする</td>';
    print '</tr>';
    $count = 0;
    print '</tbody></table>';
    print
        '<div class="note">割引率は商品ごとに適用され、商品ごとに端数が切り捨てられます。</div></fieldset>';

    print
        '<fieldset><legend><label for="data:discountrounddown">割引率の表示</label></legend>';
    print
        '小数点 <select name="data:discountrounddown" id="data:discountrounddown"><option value=""></option>';
    foreach my $n ( 1 .. 8 ) {
        print '<option value="'
            . $n . '"'
            . (
            (          $setting::payment{'discountrounddown'}
                    && $setting::payment{'discountrounddown'} eq $n
            ) ? ' selected="selected"' : ''
            )
            . '>第'
            . $n
            . '位</option>';
    }
    print '</select> を切り捨て';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:discountname">割引タイトル</label></legend>';
    print
        '<input name="data:discountname" id="data:discountname" size="56" value="'
        . $this->escape( $setting::payment{'discountname'} ) . '" />';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:discountnote">割引注意事項</label></legend>';
    print
        '<textarea name="data:discountnote" id="data:discountnote" cols="56" rows="5">';
    print $this->escape( $setting::payment{'discountnote'} );
    print '</textarea>';
    print '</fieldset>';

    print '<h1>購入総額制限</h1>';

    print '<fieldset><legend>購入総額制限</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="radio" name="data:limitlimited" value="0"'
        . (
        ( !$setting::payment{'limitlimited'} ) ? ' checked="checked"' : '' )
        . ' /> 全商品を購入総額制限対象にする</label></li>';
    print '<li><label><input type="radio" name="data:limitlimited" value="1"'
        . (
        ( $setting::payment{'limitlimited'} ) ? ' checked="checked"' : '' )
        . ' /> 「購入総額制限対象」に指定した商品のみを購入総額制限対象にする</label></li>';
    print '</ol>';
    print
        '割引前税抜き購入総額が <label><input type="number" name="data:minimumamount" id="data:minimumamount" size="16" value="'
        . $this->escape( $setting::payment{'minimumamount'} )
        . '" min="0" />円以上</label>';
    print
        '<label><input type="number" name="data:maximumamount" id="data:maximumamount" size="16" value="'
        . $this->escape( $setting::payment{'maximumamount'} )
        . '" min="0" />円以下</label> のときのみ購入を受け付ける。';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:pricerequired" value="1"'
        . (
        ( $setting::payment{'pricerequired'} ) ? ' checked="checked"' : '' )
        . ' />購入総額が0円のときは常に購入を受け付ける</label></li>';
    print
        '<li><label><input type="checkbox" name="data:cancelnolimit" value="1"'
        . (
        ( $setting::payment{'cancelnolimit'} ) ? ' checked="checked"' : '' )
        . ' />キャンセル時には購入総額制限を無効化する</label></li>';
    print '</ol>';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:limitname">購入総額制限タイトル</label></legend>';
    print '<input name="data:limitname" id="data:limitname" size="56" value="'
        . $this->escape( $setting::payment{'limitname'} ) . '" />';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:limitnote">購入総額制限注意事項</label></legend>';
    print
        '<textarea name="data:limitnote" id="data:limitnote" cols="56" rows="5">';
    print $this->escape( $setting::payment{'limitnote'} );
    print '</textarea>';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:limiterrormsg">購入総額制限エラーメッセージ</label></legend>';
    print
        '<textarea name="data:limiterrormsg" id="data:limiterrormsg" cols="56" rows="5">';
    print $this->escape( $setting::payment{'limiterrormsg'} );
    print '</textarea>';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:limiterrormsghtml">購入総額制限エラーメッセージ(HTML)</label></legend>';
    print
        '<textarea name="data:limiterrormsghtml" id="data:limiterrormsghtml" cols="56" rows="5">';
    print $this->escape( $setting::payment{'limiterrormsg'} );
    print '</textarea>';
    print '</fieldset>';

    print '<h1>支払い方法</h1>';

    print '<div id="payment-methods">';
    foreach my $id (
        sort {
            $setting::payment{'order'}{$a} <=> $setting::payment{'order'}{$b}
                || $a cmp $b
        } keys %{ $setting::payment{'order'} }
        )
    {
        $this->print_payment_method( $id, \%bank, \%other );
    }
    print '</div>';

    print '<h1>払い戻し</h1>';

    print
        '<fieldset><legend><label for="data:refundfee">払い戻し手数料</label></legend>';
    print
        '<input type="number" name="data:refundfee" id="data:refundfee" size="16" value="'
        . $this->escape( $setting::payment{'refundfee'} )
        . '" min="0" />円';
    print
        ' <label><input type="checkbox" name="data:paypalrefundfee" value="1"'
        . (
        ( $setting::payment{'paypalrefundfee'} ) ? ' checked="checked"' : '' )
        . ' /> 支払い方法が PayPal の場合も手数料を差し引く</label>';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:refundname">払い戻しタイトル</label></legend>';
    print
        '<input name="data:refundname" id="data:refundname" size="56" value="'
        . $this->escape( $setting::payment{'refundname'} ) . '" />';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:refundnote">払い戻し注意事項</label></legend>';
    print
        '<textarea name="data:refundnote" id="data:refundnote" cols="56" rows="5">';
    print $this->escape( $setting::payment{'refundnote'} );
    print '</textarea>';
    print '</fieldset>';

    print '<h1>送料</h1>';
    print '<fieldset><legend>送料</legend>';

    print
        '<fieldset><legend><label><input type="radio" name="data:carriage" value="disabled"'
        . (
        (   defined $setting::payment{'carriage'}
                && $setting::payment{'carriage'} eq 'disabled'
        ) ? ' checked="checked"' : ''
        ) . ' /> 使用停止</label></legend>';
    print '<div class="note">配送が必要ない場合等</div>';
    print '</fieldset>';

    print
        '<fieldset><legend><label><input type="radio" name="data:carriage" value="nation"'
        . (
        (   defined $setting::payment{'carriage'}
                && $setting::payment{'carriage'} eq 'nation'
        ) ? ' checked="checked"' : ''
        ) . ' /> 全国一律</label></legend>';
    print
        '<label><input type="number" name="data:nation" id="data:nation" size="8" value="'
        . $this->escape( $setting::payment{'nation'} )
        . '" onchange="cmonos.tool.prefSync(this.value);" /> <span class="unit">円</span></label>';
    print '</fieldset>';

    print
        '<fieldset><legend><label><input type="radio" name="data:carriage" value="region"'
        . (
        (   defined $setting::payment{'carriage'}
                && $setting::payment{'carriage'} eq 'region'
        ) ? ' checked="checked"' : ''
        ) . ' /> 地方単位</label></legend>';
    print '<table class="tooltable"><tbody>';
    my ($id);

    foreach $id ( sort { $a <=> $b } keys %region_name_table ) {
        print '<tr class="' . ( ( $id % 6 == 1 ) ? 'odd' : 'even' ) . '">'
            if $id % 3 == 1;
        print '<td class="'
            . ( ( ( $id - 1 ) % 3 % 2 ) ? 'even' : 'odd' )
            . '"><label for="data:region_'
            . $id . '">'
            . $region_name_table{$id}
            . '</label> <input type="number" name="data:region_'
            . $id
            . '" id="data:region_'
            . $id
            . '" size="8" value="'
            . $this->escape( $setting::payment{'region'}{$id} )
            . '" onchange="cmonos.tool.prefSync(this.value,'
            . (
            join( ',',
                $cgi::JPrefecture->get_prefecture_id_array_by_region_id($id) )
            ) . ');" min="0" /> <span class="unit">円</span></td>';
        print '</tr>' if $id % 3 == 0;
    }
    print '</tbody></table>';
    print '</fieldset>';

    print
        '<fieldset><legend><label><input type="radio" name="data:carriage" value="prefecture"'
        . (
        (   defined $setting::payment{'carriage'}
                && $setting::payment{'carriage'} eq 'prefecture'
        ) ? ' checked="checked"' : ''
        ) . ' /> 都道府県単位</label></legend>';
    print '<table class="tooltable"><tbody>';
    foreach $id ( sort { $a <=> $b } keys %pref_name_table ) {
        print '<tr class="' . ( ( $id % 6 == 1 ) ? 'odd' : 'even' ) . '">'
            if $id % 3 == 1;
        print '<td class="'
            . ( ( ( $id - 1 ) % 3 % 2 ) ? 'even' : 'odd' )
            . '"><label for="data:pref_'
            . $id . '">'
            . $pref_name_table{$id}
            . '</label> <input type="number" name="data:pref_'
            . $id
            . '" id="data:pref_'
            . $id
            . '" size="8" value="'
            . $this->escape( $setting::payment{'pref'}{$id} )
            . '" min="0" />  <span class="unit">円</span></td>';
        print '</tr>' if $id % 3 == 0;
    }
    print
        '<td><label><input name="prefsync" id="prefsync" type="checkbox" value="1"'
        . ( ( !$main::a || $main::prefsync ) ? ' checked="checked"' : '' )
        . ' /> 連動</label></td></tr>';
    print '</tbody></table>';
    print '</fieldset>';

    print '<fieldset><legend>';
    print
        '<label><input type="radio" name="data:carriage" id="data:carriage" value="weight"'
        . (
        (   defined $setting::payment{'carriage'}
                && $setting::payment{'carriage'} eq 'weight'
        ) ? ' checked="checked"' : ''
        ) . ' /> 重量別</label>';
    print
        ' (<label><input type="checkbox" name="data:yuumail" id="data:yuumail" value="1"'
        . ( ( $setting::payment{'yuumail'} ) ? ' checked="checked"' : '' )
        . ' />「ゆうメール」利用のとき必ず適用する</label>)';
    print '</legend>';
    print '<table class="tooltable">';
    print
        '<thead><tr class="odd"><th class="odd">削除</th><th class="even">重量</th><th class="odd">配送料</th></tr></thead>';
    print '<tbody>';
    $count      = 0;
    $cell_limit = 1;
    $cell_limit += scalar( keys %weight ) if %weight;

    while (1) {
        $count++;
        last if $count > 1 && $count > $cell_limit;
        print '<tr class="' . ( ( $count % 2 ) ? 'even' : 'odd' ) . '">';
        print '<td class="caution">'
            . ( ( $weight{$count}{'weight'} )
            ? '<input type="checkbox" name="weight:'
                . $count
                . ':delete" id="weight:'
                . $count
                . ':delete" value="1" />'
            : '' )
            . '</td>';
        print '<td class="even">〜<input type="number" name="weight:'
            . $count
            . ':weight" id="weight:'
            . $count
            . ':weight" size="16" value="'
            . $this->escape( $weight{$count}{'weight'} )
            . '" min="0" /> <span class="unit">g</span></td>';
        print '<td class="odd"><input type="number" name="weight:'
            . $count
            . ':carriage" id="weight:'
            . $count
            . ':carriage" size="16" value="'
            . $this->escape( $weight{$count}{'carriage'} )
            . '" min="0" /> <span class="unit">円</span></td>';
        print '</tr>';
    }
    print '</tbody></table>';
    print
        '<label><input type="checkbox" name="for_yuumail" id="for_yuumail" value="1" /> ゆうメール設定を読み込む</label><div class="hints">チェックして「保存」するとゆうメール設定が表示されます。この時点では表示されている重量別送料設定は保存されていません。設定に問題なければもう一度「保存」してください。</div>';
    print
        '<fieldset><legend><label for="data:packageweight">梱包重量</label></legend>';
    print
        '<input name="data:packageweight" id="data:packageweight" type="number" size="5" value="'
        . $this->escape( $setting::payment{'packageweight'} )
        . '" title="封筒や納品書、緩衝材など梱包に必要なものの総重量" min="0" />g';
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset><legend>';
    print
        '<label><input type="radio" name="data:carriage" id="data:carriage" value="papersize"'
        . (
        (   defined $setting::payment{'carriage'}
                && $setting::payment{'carriage'} eq 'papersize'
        ) ? ' checked="checked"' : ''
        ) . ' /> サイズ別</label>';
    print
        ' (<label><input type="checkbox" name="data:mailbin" id="data:mailbin" value="1"'
        . ( ( $setting::payment{'mailbin'} ) ? ' checked="checked"' : '' )
        . ' />「クロネコヤマト・メール便」利用のとき必ず適用する</label>)';
    print '</legend>';
    print '<table class="tooltable">';
    print
        '<thead><tr class="odd"><th class="odd">サイズ</th><th class="even">配送料</th><th class="odd">小物類</th></tr></thead>';
    print '<tbody>';
    print '<tr class="even">';
    print '<td class="odd">A4(角2封筒以内)／厚さ=1cm まで</td>';
    print
        '<td class="even"><input type="number" name="data:papersize_1A4" id="data:papersize_1A4" size="16" value="'
        . $this->escape( $setting::payment{'papersize'}{'1A4'} )
        . '" min="0" /> <span class="unit">円</span></td>';
    print
        '<td class="odd"><input type="number" name="data:papersize_accessory1" id="data:papersize_accessory1" size="4" value="'
        . $this->escape( $setting::payment{'papersize'}{'accessory1'} )
        . '" min="0" title="A4/1cmサイズに入る小物の数" /> <span class="unit">個まで</span></td>';
    print '</tr>';
    print '<tr class="odd">';
    print '<td class="odd">A4(角2封筒以内)／厚さ=2cm まで</td>';
    print
        '<td class="even"><input type="number" name="data:papersize_2A4" id="data:papersize_2A4" size="16" value="'
        . $this->escape( $setting::payment{'papersize'}{'2A4'} )
        . '" min="0" /> <span class="unit">円</span></td>';
    print
        '<td class="odd"><input type="number" name="data:papersize_accessory2" id="data:papersize_accessory2" size="4" value="'
        . $this->escape( $setting::payment{'papersize'}{'accessory2'} )
        . '" min="0" title="A4/2cmサイズに入る小物の数" /> <span class="unit">個まで</span></td>';
    print '</tr>';
    print '</tbody></table>';
    print
        '<label><input type="checkbox" name="for_mailbin" id="for_mailbin" value="1" /> クロネコヤマト・メール便設定を読み込む</label><div class="hints">チェックして「保存」するとクロネコヤマト・メール便設定が表示されます。この時点では表示されているサイズ別送料設定は保存されていません。設定に問題なければもう一度「保存」してください。</div>';
    print '<fieldset><legend>速達料金</legend>';
    print
        '<label><input type="number" name="data:papersize_expressdelivery" size="8" value="'
        . $this->escape( $setting::payment{'papersize'}{'expressdelivery'} )
        . '" min="0" /> 円</label>';
    print '</fieldset>';
    print
        '<fieldset><legend><label for="data:packagethickness">梱包材の厚さ</label></legend>';
    print
        '<input name="data:packagethickness" id="data:packagethickness" type="number" size="5" value="'
        . $this->escape( $setting::payment{'packagethickness'} )
        . '" title="封筒や納品書、緩衝材など梱包に必要なものの厚さ" />mm';
    print '</fieldset>';
    print '</fieldset>';

    print '</fieldset>';

    print '<fieldset><legend>送料優遇</legend>';
    print
        '<label><input type="number" name="data:specialcarriage_requirements" id="data:specialcarriage_requirements" size="8" value="'
        . $this->escape(
        $setting::payment{'specialcarriage'}{'requirements'} )
        . '" min="0" /> 円以上の注文で、送料を全国一律</label> ';
    print
        '<label><input type="number" name="data:specialcarriage_cost" size="8" value="'
        . $this->escape( $setting::payment{'specialcarriage'}{'cost'} )
        . '" /> 円にする。</label>';
    print '</fieldset>';

    print '<fieldset><legend>送料の表示</legend>';
    print
        '<label><input type="checkbox" name="data:estimatedCarriageRequired" id="data:estimatedCarriageRequired" value="1"'
        . ( ( $setting::payment{'estimatedCarriageRequired'} )
        ? 'checked="checked"'
        : '' )
        . ' /> 送料の計算に住所が必要なとき、住所取得前に送料の最大額を参考表示する</label>';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:carriagenote">送料注意事項</label></legend>';
    print
        '<textarea name="data:carriagenote" id="data:carriagenote" cols="56" rows="5">';
    print $this->escape( $setting::payment{'carriagenote'} );
    print '</textarea>';
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');

}

sub print_sort_menu ($$$;$) {
    my ( $this, $name ) = @_;
    print '<select name="order:'
        . $name
        . '" id="sortmenu-'
        . $name
        . '" onchange="cmonos.tool.sortPaymentMethods()" title="表示順">';
    foreach my $n ( 1 .. 6 ) {
        print '<option value="'
            . $n . '"'
            . ( ( $setting::payment{'order'}{$name} == $n )
            ? ' selected="selected"'
            : '' )
            . '>'
            . $n
            . '</option>';
    }
    print '</select>';
}

sub print_payment_method ($$$$) {
    my ( $this, $method_id, $bank, $other ) = @_;
    if ( $method_id eq 'bank' ) {
        print '<fieldset id="bank"><legend>';
        print
            '<label><input type="checkbox" name="data:bankenabled" id="data:bankenabled" value="1"'
            . (
            ( $setting::payment{'bankenabled'} ) ? ' checked="checked"' : '' )
            . ' onclick="cmonos.element.item(\'bank-fields\').disappear(!this.checked);" /> 銀行振込</label>';
        $this->print_sort_menu('bank');
        print '</legend>';
        print '<div id="bank-fields">';
        my $count      = 0;
        my $cell_limit = 1;
        $cell_limit += scalar( keys %$bank ) if ref $bank && %$bank;

        while (1) {
            $count++;
            last if $count > 3 && $count > $cell_limit;
            print '<fieldset class="'
                . ( ( $count % 2 ) ? 'odd' : 'even' ) . '">';
            print '<legend>銀行 <input type="number" name="bank:'
                . $count
                . ':order" id="bank:'
                . $count
                . ':order" size="3" value="'
                . $count
                . '" title="表示順" />';
            print ' <label><input type="checkbox" name="bank:'
                . $count
                . ':delete" id="bank:'
                . $count
                . ':delete" value="1" /> 削除</label></legend>';
            print '<table class="field-table"><tbody>';
            print '<tr>';
            print '<td class="field-name"><label for="bank:' . $count
                . ':bank">銀行名</label></td>';
            print '<td class="q-field"><input type="text" name="bank:'
                . $count
                . ':bank" id="bank:'
                . $count
                . ':bank" size="32" value="'
                . $this->escape( $bank->{$count}{'bank'} )
                . '" /></td>';
            print '<td class="field-name"><label for="bank:' . $count
                . ':office">支店名</label></td>';
            print '<td class="q-field"><input type="text" name="bank:'
                . $count
                . ':office" id="bank:'
                . $count
                . ':office" size="32" value="'
                . $this->escape( $bank->{$count}{'office'} )
                . '" /></td>';
            print '</tr>';
            print '<tr>';
            print '<td class="field-name"><label for="bank:' . $count
                . ':number">口座番号</label></td><td class="field" colspan="3">';
            print '<select name="bank:'
                . $count
                . ':deposit" id="bank:'
                . $count
                . ':deposit">';
            print '<option value="ordinary"'
                . (
                (   defined $bank->{$count}{'deposit'}
                        && $bank->{$count}{'deposit'} eq 'ordinary'
                ) ? ' selected="selected"' : ''
                ) . '>普通</option>';
            print '<option value="current"'
                . (
                (   defined $bank->{$count}{'deposit'}
                        && $bank->{$count}{'deposit'} eq 'current'
                ) ? ' selected="selected"' : ''
                ) . '>当座</option>';
            print '</select><input type="text" name="bank:'
                . $count
                . ':number" id="bank:'
                . $count
                . ':number" size="16" value="'
                . $this->escape( $bank->{$count}{'number'} )
                . '" /></td>';
            print '</tr>';
            print '<tr>';
            print '<td class="field-name"><label for="bank:' . $count
                . ':name">名義</label></td>';
            print '<td class="q-field"><input type="text" name="bank:'
                . $count
                . ':name" id="bank:'
                . $count
                . ':name" size="32" value="'
                . $this->escape( $bank->{$count}{'name'} )
                . '" /></td>';
            print '<td class="field-name"><label for="bank:' . $count
                . ':kana">カタカナ表記</label></td>';
            print '<td class="q-field"><input type="text" name="bank:'
                . $count
                . ':kana" id="bank:'
                . $count
                . ':kana" size="32" value="'
                . $this->escape( $bank->{$count}{'kana'} )
                . '" /></td>';
            print '</tr>';
            print '</tbody></table>';
            print '</fieldset>';
        }
        print
            '<fieldset><legend><label for="data:bankinfo">公開範囲</label></legend>';
        print '<select name="data:bankinfo" id="data:bankinfo">';
        print '<option value="0"'
            . (
            ( !$setting::payment{'bankinfo'} ) ? ' selected="selected"' : '' )
            . '>銀行名のみ</option>';
        print '<option value="1"'
            . (
            (          $setting::payment{'bankinfo'}
                    && $setting::payment{'bankinfo'} eq '1'
            ) ? ' selected="selected"' : ''
            ) . '>銀行名・支店名</option>';
        print '<option value="2"'
            . (
            (          $setting::payment{'bankinfo'}
                    && $setting::payment{'bankinfo'} eq '2'
            ) ? ' selected="selected"' : ''
            ) . '>銀行名・支店名・口座番号</option>';
        print '<option value="3"'
            . (
            (          $setting::payment{'bankinfo'}
                    && $setting::payment{'bankinfo'} eq '3'
            ) ? ' selected="selected"' : ''
            ) . '>銀行名・支店名・口座番号・名義</option>';
        print '</select>';
        print '</fieldset>';
        print
            '<fieldset><legend><label for="data:banknote">注意事項</label></legend><textarea name="data:banknote" id="data:banknote" cols="56" rows="5">';
        print $this->escape( $setting::payment{'banknote'} );
        print '</textarea>';
        print '</fieldset>';
        print '</div>';
        print $this->javascript(
            'cmonos.element.item(\'bank-fields\').disappear(true);')
            if !$setting::payment{'bankenabled'};
        print '</fieldset>';

    }
    elsif ( $method_id eq 'yubin' ) {
        print '<fieldset id="yubin"><legend>';
        print
            '<label><input type="checkbox" name="data:yubinenabled" id="data:yubinenabled" value="1"'
            . ( ( $setting::payment{'yubinenabled'} )
            ? ' checked="checked"'
            : '' )
            . ' onclick="cmonos.element.item(\'yubin-fields\').disappear(!this.checked);" /> ゆうちょ銀行 (郵便振替)</label>';
        $this->print_sort_menu('yubin');
        print '</legend>';
        print '<div id="yubin-fields">';
        print '<table class="field-table"><tbody>';
        print '<tr>';
        print
            '<td class="field-name"><span class="datatitle">記号-番号</span></td>';
        print
            '<td class="field" colspan="3"><input type="text" name="data:yubin_mark" id="data:yubin_mark" size="6" value="'
            . $this->escape( $setting::payment{'yubin'}{'mark'} )
            . '" /> - ';
        print
            '<input type="text" name="data:yubin_digit" id="data:yubin_digit" size="2" value="'
            . $this->escape( $setting::payment{'yubin'}{'digit'} )
            . '" /> - ';
        print
            '<input type="text" name="data:yubin_number" id="data:yubin_number" size="8" value="'
            . $this->escape( $setting::payment{'yubin'}{'number'} )
            . '" /> [記号] - [一桁の数字(ない場合は無記入)] - [番号]</td>';
        print '</tr>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:yubin_office">郵便局</label></td>';
        print
            '<td class="field" colspan="3"><input type="text" name="data:yubin_office" id="data:yubin_office" size="32" value="'
            . $this->escape( $setting::payment{'yubin'}{'office'} )
            . '" /></td>';
        print '</tr>';
        print '<tr>';
        print
            '<td class="field-name"><label for="data:yubin_name">名義</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:yubin_name" id="data:yubin_name" size="32" value="'
            . $this->escape( $setting::payment{'yubin'}{'name'} )
            . '" /></td>';
        print
            '<td class="field-name"><label for="data:yubin_kana">カタカナ表記</label></td>';
        print
            '<td class="q-field"><input type="text" name="data:yubin_kana" id="data:yubin_name" size="32" value="'
            . $this->escape( $setting::payment{'yubin'}{'kana'} )
            . '" /></td>';
        print '</tr>';
        print '</tbody></table>';
        print
            '<fieldset><legend><label for="data:yubininfo">公開範囲</label></legend>';
        print '<select name="data:yubininfo" id="data:yubininfo">';
        print '<option value="0"'
            . ( ( !$setting::payment{'yubininfo'} )
            ? ' selected="selected"'
            : '' )
            . '>ゆうちょ銀行</option>';
        print '<option value="1"'
            . (
            (          $setting::payment{'yubininfo'}
                    && $setting::payment{'yubininfo'} eq '1'
            ) ? ' selected="selected"' : ''
            ) . '>ゆうちょ銀行・郵便局名</option>';
        print '<option value="2"'
            . (
            (          $setting::payment{'yubininfo'}
                    && $setting::payment{'yubininfo'} eq '2'
            ) ? ' selected="selected"' : ''
            ) . '>ゆうちょ銀行・郵便局名・口座番号</option>';
        print '<option value="3"'
            . (
            (          $setting::payment{'yubininfo'}
                    && $setting::payment{'yubininfo'} eq '3'
            ) ? ' selected="selected"' : ''
            )
            . '>ゆうちょ銀行・郵便局名・口座番号・名義</option>';
        print '</select>';
        print '</fieldset>';
        print
            '<fieldset><legend><label for="data:yubinnote">注意事項</label></legend><textarea name="data:yubinnote" id="data:yubinnote" cols="56" rows="5">';
        print $this->escape( $setting::payment{'yubinnote'} );
        print '</textarea>';
        print '</fieldset>';
        print '</div>';
        print $this->javascript(
            'cmonos.element.item(\'yubin-fields\').disappear(true);')
            if !$setting::payment{'yubinenabled'};
        print '</fieldset>';

    }
    elsif ( $method_id eq 'paypal' ) {
        my $lwp_is_ready = ( eval { require LWP::UserAgent } ) ? 1 : 0;
        my $ssleay_is_ready = ( eval 'use Crypt::SSLeay;1;' ) ? 1 : 0;
        if ( $lwp_is_ready && $ssleay_is_ready ) {
            $setting::payment{'paypal'}{'ratio'} = 3.6
                if !defined $setting::payment{'paypal'}{'ratio'};
            $setting::payment{'paypal'}{'fee'} = 40
                if !defined $setting::payment{'paypal'}{'fee'};
            $setting::payment{'paypal'}{'mratio'} = 5
                if !defined $setting::payment{'paypal'}{'mratio'};
            $setting::payment{'paypal'}{'mfee'} = 7
                if !defined $setting::payment{'paypal'}{'mfee'};
            print '<fieldset id="paypal"><legend>';
            print
                '<label><input type="checkbox" name="data:paypalenabled" id="data:paypalenabled" value="1"'
                . ( ( $setting::payment{'paypalenabled'} )
                ? ' checked="checked"'
                : '' )
                . ' onclick="cmonos.element.item(\'paypal-fields\').disappear(!this.checked);" /> PayPal</label>';
            $this->print_sort_menu('paypal');
            print '</legend>';
            print '<div id="paypal-fields">';

            print
                '<fieldset><legend><label for"data:paypal_name">表示名</label></legend>';
            print
                '<input type="text" name="data:paypal_name" id="data:paypal_name" size="56" value="'
                . $this->escape( $setting::payment{'paypal'}{'name'} )
                . '" />';
            print '</fieldset>';

            print '<fieldset><legend>決済設定</legend>';
            print '<table class="field-table"><tbody>';
            print '<tr>';
            print
                '<td class="field-name"><span class="datatitle">取引当たりの手数料(参考)</span></td>';
            print '<td class="field">';
            print
                '取引金額&times;<input type="text" name="data:paypal_ratio" id="data:paypal_ratio" size="3" value="'
                . $this->escape( $setting::payment{'paypal'}{'ratio'} )
                . '" />%';
            print
                ' + <input type="text" name="data:paypal_fee" id="data:paypal_fee" size="3" value="'
                . $this->escape( $setting::payment{'paypal'}{'fee'} )
                . '" />円';
            print
                '<div class="hints">サンドボックスを利用しているか、「スタッフ」以上の権限でアクセスした際にのみ表示される、PayPal 手数料と正味金額を計算するために設定します。PayPal の規約により、PayPal の決済手数料を買い手に追加請求することはできません。ご注意下さい。</div>';
            print '</td>';
            print '</tr>';
            print '<tr>';
            print
                '<td class="field-name"><label for="data:paypal_paymentaction">決済のタイミング</label></td>';
            print '<td class="field">';
            print
                '<select name="data:paypal_paymentaction" id="data:paypal_paymentaction">';
            print '<option value="Order"'
                . (
                (          $setting::payment{'paypal'}{'paymentaction'}
                        && $setting::payment{'paypal'}{'paymentaction'} eq
                        'Order'
                ) ? ' selected="selected"' : ''
                ) . '>確認後、決済を承認</option>';
            print '<option value="Sale"'
                . (
                (          $setting::payment{'paypal'}{'paymentaction'}
                        && $setting::payment{'paypal'}{'paymentaction'} eq
                        'Sale'
                ) ? ' selected="selected"' : ''
                ) . '>即時</option>';
            print
                '</select> <label><input type="checkbox" name="data:paypal_noshippingsale" value="1"'
                . ( ( $setting::payment{'paypal'}{'noshippingsale'} )
                ? ' checked="checked"'
                : '' )
                . ' /> 配送を必要としない商品のみのとき、常に「即時」決済する</label>';
            print
                '<div class="hints">購入品目がデジタルコンテンツのみの場合は常に「即時」決済されます。</div>';
            print
                '<div class="hints">「PayPal for Digital Goods」が有効な状態で「即時」を選択すると、すべての商品がデジタルコンテンツとして PayPal に送信されます。ご注意ください。</div></td>';
            print '</tr>';
            print '</tbody></table>';
            print '</fieldset>';

            print
                '<fieldset><legend><label><input type="checkbox" name="data:paypal_incontext" id="data:paypal_incontext" value="1"'
                . ( ( $setting::payment{'paypal'}{'incontext'} )
                ? ' checked="checked"'
                : '' )
                . ' onclick="cmonos.element.item(\'paypal-micropayments\').disappear(!this.checked);" /> PayPal for Digital Goods</label></legend>';
            print
                '<p class="warning">「PayPal for Digital Goods」のマイクロペイメント手数料は、マイクロペイメント手数料の方が通常手数料より安くなり、かつ、<strong>購入商品のすべてがダウンロード商品のとき</strong>、適用されます。</p>';
            print
                '<table class="field-table" id="paypal-micropayments"><tbody>';
            print '<tr>';
            print
                '<td class="field-name"><span class="datatitle">マイクロペイメント手数料(参考)</span></td>';
            print '<td class="field">';
            print
                '取引金額&times;<input type="text" name="data:paypal_mratio" id="data:paypal_mratio" size="3" value="'
                . $this->escape( $setting::payment{'paypal'}{'mratio'} )
                . '" />%';
            print
                ' + <input type="text" name="data:paypal_mfee" id="data:paypal_mfee" size="3" value="'
                . $this->escape( $setting::payment{'paypal'}{'mfee'} )
                . '" />円';
            print
                '<div class="hints">サンドボックスを利用しているか、「スタッフ」以上の権限でアクセスした際にのみ表示される、PayPal 手数料と正味金額を計算するために設定します。PayPal の規約により、PayPal の決済手数料を買い手に追加請求することはできません。ご注意下さい。</div>';
            print '</td>';
            print '</tr>';
            print '</tbody></table>';
            print '</fieldset>';
            print $this->javascript(
                'cmonos.element.item(\'paypal-micropayments\').disappear(true);'
            ) if !$setting::payment{'paypal'}{'incontext'};

            print
                '<fieldset><legend>エクスプレスチェックアウトAPI アクセス設定</legend>';
            print '<table class="field-table"><tbody>';
            print '<tr>';
            print
                '<td class="field-name"><label for="data:paypal_username">APIユーザー名</label></td>';
            print
                '<td class="field"><input type="text" name="data:paypal_username" id="data:paypal_username" size="72" value="'
                . $this->escape( $setting::payment{'paypal'}{'username'} )
                . '" /></td>';
            print '</tr>';
            print '<tr>';
            print
                '<td class="field-name"><label for="paypal:password">APIパスワード</label></td>';
            print '<td class="field">';
            print
                '<input type="text" name="paypal:password" id="paypal:password" size="72" />';
            print
                '<div class="hints">無記入の場合変更されません。</div>';

            if (   $setting::payment{'paypal'}{'password'}
                && $setting::payment{'paypal'}{'password'}
                =~ /([0-9a-zA-Z_]{4})$/ )
            {
                print
                    '<input type="hidden" name="paypal:old_password" id="paypal:old_password" value="1" />';
                print '<div class="hints">現在の設定:****' . $1
                    . '</div>';
            }
            print '</td>';
            print '</tr>';
            print '<tr>';
            print
                '<td class="field-name"><label for="paypal:signature">API署名</label></td>';
            print '<td class="field">';
            print
                '<input type="text" name="paypal:signature" id="paypal:signature" size="72" />';
            print
                '<div class="hints">無記入の場合変更されません。</div>';

            if (   $setting::payment{'paypal'}{'signature'}
                && $setting::payment{'paypal'}{'signature'}
                =~ /([0-9a-zA-Z_]{4})$/ )
            {
                print
                    '<input type="hidden" name="paypal:old_signature" id="paypal:old_signature" value="1" />';
                print '<div class="hints">現在の設定:************' . $1
                    . '</div>';
            }
            print '</td>';
            print '</tr>';
            print '<tr>';
            print
                '<td class="field-name"><span class="datatitle">サンドボックス</span></td>';
            print
                '<td class="field"><label><input type="checkbox" name="data:paypal_sandbox" id="data:paypal_sandbox" value="1"'
                . ( ( $setting::payment{'paypal'}{'sandbox'} )
                ? ' checked="checked"'
                : '' )
                . ' /> 動作テスト専用のサンドボックス API にアクセスする。</label></td>';
            print '</tr>';
            print '</tbody></table>';
            print '</fieldset>';
            print '<fieldset><legend>表示設定</legend>';
            print '<table class="field-table"><tbody>';
            print '<tr>';
            print
                '<td class="field-name"><span class="datatitle">表示順</span></td>';
            print
                '<td class="field"><label><input type="checkbox" name="data:paypal_top" id="data:paypal_top" value="1"'
                . ( ( $setting::payment{'paypal'}{'top'} )
                ? ' checked="checked"'
                : '' )
                . ' /> チェックアウト入力画面で支払い方法フィールドを最初に表示する。</label></td>';
            print '</tr>';
            print '<tr>';
            print
                '<td class="field-name"><label for="data:paypal_button">チェックアウトボタン画像URL</label></td>';
            print
                '<td class="field"><input type="text" name="data:paypal_button" id="data:paypal_button" size="72" value="'
                . $this->escape( $setting::payment{'paypal'}{'button'} )
                . '" /></td>';
            print '</tr>';
            print '<tr>';
            print
                '<td class="field-name"><label for="data:paypal_banner">ソリューション画像HTML</label></td>';
            print
                '<td class="field"><textarea name="data:paypal_banner" id="data:paypal_banner" cols="72" rows="5">'
                . $this->escape( $setting::payment{'paypal'}{'banner'} )
                . '</textarea></td>';
            print '</tr>';
            print '<tr>';
            print
                '<td class="field-name"><label for="data:paypal_pagestyle">「カスタム支払い」ページスタイル名</label></td>';
            print
                '<td class="field"><input type="text" name="data:paypal_pagestyle" id="data:paypal_pagestyle" size="72" value="'
                . $this->escape( $setting::payment{'paypal'}{'pagestyle'} )
                . '" /></td>';
            print '</tr>';
            print '</tbody></table>';
            print '</fieldset>';
            $setting::payment{'paypalnote'} = '手数料は無料です。'
                if !defined $setting::payment{'paypalnote'};
            print
                '<fieldset><legend><label for="data:paypalnote">注意事項</label></legend><textarea name="data:paypalnote" id="data:paypalnote" cols="56" rows="5">';
            print $this->escape( $setting::payment{'paypalnote'} );
            print '</textarea>';
            print '</fieldset>';
            print '</div>';
            print $this->javascript(
                'cmonos.element.item(\'paypal-fields\').disappear(true);')
                if !$setting::payment{'paypalenabled'};
            print '</fieldset>';
        }
        else {
            print '<fieldset id="paypal"><legend><del>PayPal</del>';
            $this->print_sort_menu('paypal');
            print
                '</legend><div class="warning">安全な通信に必要なモジュール(';
            print 'LWP::UserAgent' if !$lwp_is_ready;
            print ' および '    if !$lwp_is_ready && !$ssleay_is_ready;
            print 'Crypt::SSLeay'  if !$ssleay_is_ready;
            print
                ')がみつからないため、PayPal による決済は利用できません。</div></fieldset>';
        }

    }
    elsif ( $method_id eq 'cod' ) {
        print '<fieldset id="cod"><legend>';
        print
            '<label><input type="checkbox" name="data:codenabled" id="data:codenabled" value="1"'
            . (
            ( $setting::payment{'codenabled'} ) ? ' checked="checked"' : '' )
            . ' onclick="cmonos.element.item(\'cod-fields\').disappear(!this.checked);" /> 代金引き換え</label>';
        $this->print_sort_menu('cod');
        print '</legend>';
        print '<div id="cod-fields">';
        print '<fieldset><legend>利用禁止</legend>';
        print
            '<label><input type="checkbox" name="data:cod_dcUnavailable" id="data:cod_dcUnavailable" value="1"'
            . ( ( $setting::payment{'cod'}{'dcUnavailable'} )
            ? ' checked="checked"'
            : '' )
            . ' /> 購入商品にデジタルコンテンツ(ダウンロード商品)が含まれるとき、代金引き換えの利用を禁止する。</label><div class="hints">購入商品に配送商品が含まれないとき(すべての購入商品がデジタルコンテンツのとき)は常に代金引き換えの利用が禁止されます。</div></td>';
        print '</fieldset>';
        print '<fieldset><legend>手数料設定</legend>';
        print '<table class="tooltable">';
        print
            '<thead><tr class="odd"><th class="even">商品代金</th><th class="even">手数料</th></tr></thead>';
        print '<tbody>';
        print
            '<tr class="even"><td class="odd"><label for="data:cod_fee1">1万円未満</label></td><td class="even"><input type="text" name="data:cod_fee1" id="data:cod_fee1" size="8" value="'
            . $this->escape( $setting::payment{'cod'}{'fee1'} )
            . '" />  <span class="unit">円</span></td></tr>';
        print
            '<tr class="odd"><td class="odd"><label for="data:cod_fee2">1万円以上3万円未満</label></td><td class="even"><input type="text" name="data:cod_fee2" id="data:cod_fee2" size="8" value="'
            . $this->escape( $setting::payment{'cod'}{'fee2'} )
            . '" />  <span class="unit">円</span></td></tr>';
        print
            '<tr class="even"><td class="odd"><label for="data:cod_fee3">3万円以上10万円未満</label></td><td class="even"><input type="text" name="data:cod_fee3" id="data:cod_fee3" size="8" value="'
            . $this->escape( $setting::payment{'cod'}{'fee3'} )
            . '" />  <span class="unit">円</span></td></tr>';
        print
            '<tr class="odd"><td class="odd"><label for="data:cod_fee4">10万円以上30万円未満</label></td><td class="even"><input type="text" name="data:cod_fee4" id="data:cod_fee4" size="8" value="'
            . $this->escape( $setting::payment{'cod'}{'fee4'} )
            . '" />  <span class="unit">円</span></td></tr>';
        print '</tbody></table>';
        print
            '<div class="note">手数料を空欄とすると該当する商品代金のとき代金引き換えを受け付けません。</div>';
        print '</fieldset>';
        print
            '<fieldset><legend><label for="data:codnote">注意事項</label></legend><textarea name="data:codnote" id="data:codnote" cols="56" rows="5">';
        print $this->escape( $setting::payment{'codnote'} );
        print '</textarea>';
        print '</fieldset>';
        print '</div>';
        print $this->javascript(
            'cmonos.element.item(\'cod-fields\').disappear(true);')
            if !$setting::payment{'codenabled'};
        print '</fieldset>';

    }
    elsif ( $method_id eq 'moneyorder' ) {
        print '<fieldset id="moneyorder"><legend>';
        print
            '<label><input type="checkbox" name="data:moneyorderenabled" id="data:moneyorderenabled" value="1"'
            . ( ( $setting::payment{'moneyorderenabled'} )
            ? ' checked="checked"'
            : '' )
            . ' onclick="cmonos.element.item(\'moneyorder-fields\').disappear(!this.checked);" /> <a href="http://www.jp-bank.japanpost.jp/kojin/tukau/sokin/hikoza/kj_tk_sk_hkz_kogawase.html" target="_blank">定額小為替</a></label>';
        $this->print_sort_menu('moneyorder');
        print '</legend>';
        print '<div id="moneyorder-fields">';
        print
            '<fieldset><legend><label for="data:moneyordernote">注意事項</label></legend><textarea name="data:moneyordernote" id="data:moneyordernote" cols="56" rows="5">';
        print $this->escape( $setting::payment{'moneyordernote'} );
        print '</textarea>';
        print '</fieldset>';
        print '</div>';
        print $this->javascript(
            'cmonos.element.item(\'moneyorder-fields\').disappear(true);')
            if !$setting::payment{'moneyorderenabled'};
        print '</fieldset>';

    }
    elsif ( $method_id eq 'other' ) {
        print '<fieldset id="other"><legend>';
        print
            '<label><input type="checkbox" name="data:otherenabled" id="data:otherenabled" value="1"'
            . ( ( $setting::payment{'otherenabled'} )
            ? ' checked="checked"'
            : '' )
            . ' onclick="cmonos.element.item(\'other-fields\').disappear(!this.checked);" /> その他</label>';
        $this->print_sort_menu('other');
        print '</legend>';
        print '<table class="tooltable" id="other-fields">';
        print
            '<thead><tr class="odd"><th class="odd">削除</th><th class="even">表示順</th><th class="odd">決済方法名</th><th class="even">手数料計算式(x=支払い総額)</th><th class="odd">端数処理</th><th class="even">説明</th></tr></thead>';
        print '<tbody>';
        my $count      = 0;
        my $cell_limit = 1;
        $cell_limit += scalar( keys %$other ) if ref $other && %$other;

        while (1) {
            $count++;
            last if $count > 1 && $count > $cell_limit;
            print '<tr class="' . ( ( $count % 2 ) ? 'even' : 'odd' ) . '">';
            print '<td class="caution">'
                . ( ( $other->{$count}{'name'} )
                ? '<input type="checkbox" name="other:'
                    . $count
                    . ':delete" id="other:'
                    . $count
                    . ':delete" value="1" />'
                : '' )
                . '</td>';
            print '<td class="even"><input type="text" name="other:'
                . $count
                . ':order" id="other:'
                . $count
                . ':order" size="2" value="'
                . $count
                . '" /></td>';
            print '<td class="odd"><input type="text" name="other:'
                . $count
                . ':name" id="other:'
                . $count
                . ':name" size="16" value="'
                . $this->escape( $other->{$count}{'name'} )
                . '" /></td>';
            print '<td class="even"><input type="text" name="other:'
                . $count
                . ':formula" id="other:'
                . $count
                . ':formula" size="20" value="'
                . $this->escape( $other->{$count}{'formula'} )
                . '" /></td>';
            print '<td class="odd"><select name="other:'
                . $count
                . ':frac" id="other:'
                . $count
                . ':frac">';
            print '<option value="ceil"'
                . (
                (   defined $other->{$count}{'frac'}
                        && $other->{$count}{'frac'} eq 'ceil'
                ) ? ' selected="selected"' : ''
                ) . '>切り上げ</option>';
            print '<option value="round"'
                . (
                (   defined $other->{$count}{'frac'}
                        && $other->{$count}{'frac'} eq 'round'
                ) ? ' selected="selected"' : ''
                ) . '>四捨五入</option>';
            print '<option value="floor"'
                . (
                (   defined $other->{$count}{'frac'}
                        && $other->{$count}{'frac'} eq 'floor'
                ) ? ' selected="selected"' : ''
                ) . '>切り下げ</option>';
            print '</select></td>';
            print '<td class="even"><textarea name="other:'
                . $count
                . ':note" id="other:'
                . $count
                . ':note" cols="36" rows="3">'
                . $this->escape( $other->{$count}{'note'} )
                . '</textarea></td>';
            print '</tr>';
        }
        print '</tbody></table>';
        print $this->javascript(
            'cmonos.element.item(\'other-fields\').disappear(true);')
            if !$setting::payment{'otherenabled'};
        print '</fieldset>';
    }
}

sub is_not_number ($$$;$) {
    my ( $this, $value, $name, $digit ) = @_;
    if (   defined $$value
        && $$value ne ''
        && (   ( !$digit && $this->is_nan($$value) )
            || ( $digit && $this->is_not_digit($$value) ) )
        )
    {
        push(
            @{ $this->{'dialog'} },
            $name
                . 'が数ではありません。半角数字にしてください。'
        );
        undef $$value;
        return 1;
    }
    return 0;
}

sub is_not_image ($$$) {
    my ( $this, $value, $name ) = @_;
    if (   defined $$value
        && $$value ne ''
        && $$value
        !~ m!^s?https?://[0-9a-zA-Z_~\-]+\.[0-9a-zA-Z_~\-/\.#\?\&=\%,]+\.(gif|jpg|jpeg|png)$!i
        )
    {
        push(
            @{ $this->{'dialog'} },
            $name
                . 'が画像URLではありません。画像URLにしてください。'
        );
        undef $$value;
        return 1;
    }
    return 0;
}

# 多くの銀行で入力できるカタカナ名かどうかをチェック
# 全角数字 (０１２３４５６７８９)
# 全角カナ (アイウエオ...ワヲン)
# 全角カナ小文字 (ァ ィ ゥ ェ ォ ャ ュ ョ ッ)
# アルファベット大文字 (ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ)
# 濁点、半濁点 (゛　゜)
# 記号 ，(カンマ)　．(ピリオド)　（）／−ー「」・ 全角スペース
sub is_not_kana_name ($$$) {
    my ( $this, $value, $name ) = @_;
    $$value = $this->to_zkatakana($$value);
    if (   $$value ne ''
        && $$value
        !~ /^[\x{30A1}-\x{30FC}\x{FF10}-\x{FF19}\x{FF21}-\x{FF3A}\x{3000}（），．゛゜／−ー「」・]+$/
        )
    {
        push(
            @{ $this->{'dialog'} },
            $name
                . 'がカタカナ名ではありません。全角カタカナにしてください。'
        );
        undef $$value;
        return 1;
    }
    return 0;
}

sub is_invalid_formula ($$$) {
    my ( $this, $formula, $name ) = @_;
    $$formula = $this->z_to_h($$formula)
        if $$formula !~ /^[\+\-]{0,1}[0-9\.x]+$/;
    $$formula =~ s/[^0-9\.+\-\*\/)(x]//g;
    if ( $$formula =~ /x/ ) {
        my $f = $$formula;
        $f =~ s/x/1/g;
        if ( eval($f) <= 0 ) {
            push(
                @{ $this->{'dialog'} },
                $name . 'を確認してください。'
            );
            undef $$formula;
            return 1;
        }
    }
    elsif ( $$formula ne '' && $this->is_nan($$formula) ) {
        push(
            @{ $this->{'dialog'} },
            $name . 'を確認してください。'
        );
        undef $$formula;
        return 1;
    }
    return 0;
}

sub is_shortage ($$@) {
    my ( $this, $name, @value ) = @_;
    if (   grep( defined $_ && $_ ne '', @value )
        && grep( !defined $_ || $_ eq '', @value ) )
    {
        push(
            @{ $this->{'dialog'} },
            $name
                . '情報に不足があります。すべて記入してください。'
        );
        return 1;
    }
    return 0;
}

sub is_empty_array ($@) {
    my ( $this, @value ) = @_;
    return ( grep( defined $_ && $_ ne '', @value ) == 0 ) ? 1 : 0;
}

sub to_zkatakana ($$) {
    my ( $this, $text ) = @_;
    return '' if !defined $text || $text eq '';
    utf8::decode($text) if !utf8::is_utf8($text);
    $text = $this->h_to_z( $text, 'utf8', 1 );
    $text =~ s/([\x{3041}-\x{309C}])/pack("U",unpack("U",$1) + 96)/ges;
    return $text;
}

sub print_payment_info_page ($$$) {
    my ( $this, $pref_name_table, $region_name_table ) = @_;
    local ( *INFO, $default::static );
    $default::static = 1;
    my $path
        = $default::file
        . '/payment_info.'
        . ( ( $env::ssi && $env::ssi_ext ) ? $env::ssi_ext : 'html' );
    return 0 if !open( INFO, ">>$path" );
    binmode INFO, ":utf8";
    flock( INFO, LOCK_EX );
    truncate( INFO, 0 );
    seek( INFO, 0, 0 );
    $this->import_db( $default::preference_dir . '/page',
        \%setting::page, 1, 'info' );
    my $table = Table->new;

    if ( defined $setting::page{'info'}{'table'}
        && $setting::page{'info'}{'table'} ne '' )
    {
        if ( ref $table::obj{ $setting::page{'info'}{'table'} } ) {
            %$table = %{ $table::obj{ $setting::page{'info'}{'table'} } };
        }
        elsif (
            defined $setting::table{ $setting::page{'info'}{'table'} }
            {'setting'}
            && $setting::table{ $setting::page{'info'}{'table'} }{'setting'} )
        {
            $table->set(
                $setting::table{ $setting::page{'info'}{'table'} }{'setting'}
            );
            $table->{'name'}
                = $setting::table{ $setting::page{'info'}{'table'} }{'name'};
        }
    }
    $table->set( 'thenabled' => 'false' );
    $setting::page{'info'}{'bodyId'}
        = 'document' . ( ($default::id) ? $default::id : '' ) . '_terms';
    $this->print_page_header( 'info', \*INFO, $path );

    if ( ref $setting::payment{'discount'}
        && %{ $setting::payment{'discount'} } )
    {
        print INFO
            '<div class="section" id="discount-info-box"><h2 id="discount-info-header">'
            . ( ( $setting::payment{'discountname'} )
            ? $this->tag_fix( $setting::payment{'discountname'} )
            : 'まとめ買い割引' )
            . '</h2><div class="body">';
        print INFO $table->table;
        print INFO $table->tr;
        print INFO $table->th('購入総額');
        print INFO $table->th('割引率');
        print INFO $table->_tr;
        foreach my $n ( sort { $a <=> $b || $a cmp $b }
            keys %{ $setting::payment{'discount'} } )
        {
            print INFO $table->tr;
            print INFO $table->td( $this->digit($n) . '円以上' );
            print INFO $table->td(
                $this->round_down(
                    $setting::payment{'discount'}{$n},
                    $setting::payment{'discountrounddown'},
                    'floor'
                    )
                    . '%'
            );
            print INFO $table->_tr;
        }
        print INFO $table->_table;
        print INFO '<p id="discount-info-note">'
            . $this->tag_fix( $setting::payment{'discountnote'} ) . '</p>'
            if $setting::payment{'discountnote'};
        print INFO '</div></div>';
    }

    if ((   defined $setting::payment{'minimumamount'}
            && $setting::payment{'minimumamount'} ne ''
        )
        || ( defined $setting::payment{'maximumamount'}
            && $setting::payment{'maximumamount'} ne '' )
        )
    {
        print INFO
            '<div class="section" id="limit-info-box"><h2 id="limit-info-header">'
            . ( ( $setting::payment{'limitname'} )
            ? $this->tag_fix( $setting::payment{'limitname'} )
            : '購入総額制限' )
            . '</h2><div class="body">';
        print INFO '<p id="limit-info-note">'
            . (
            ( $setting::payment{'limitnote'} )
            ? $this->tag_fix( $setting::payment{'limitnote'} )
            : ( ( $setting::payment{'pricerequired'} )
                ? '購入総額制限対象に指定された商品の総額が'
                : '購入総額が' )
                . (
                (   defined $setting::payment{'minimumamount'}
                        && $setting::payment{'minimumamount'} ne ''
                ) ? $setting::payment{'minimumamount'} . '円以上' : ''
                )
                . (
                (   defined $setting::payment{'maximumamount'}
                        && $setting::payment{'maximumamount'} ne ''
                ) ? $setting::payment{'maximumamount'} . '円以下' : ''
                )
                . 'となるようお買い求めください。'
            ) . '</p>';
        print INFO '</div></div>';
    }

    print INFO
        '<div class="section" id="payment-info-box"><h2 id="payment-info-header">お支払い方法</h2><div class="body">';
    print INFO '<dl id="payment-info">';
    foreach my $id (
        sort {
            $setting::payment{'order'}{$a} <=> $setting::payment{'order'}{$b}
                || $a cmp $b
        } keys %{ $setting::payment{'order'} }
        )
    {
        $this->print_payment_method_info( \*INFO, $id, $table );
    }
    print INFO '</div>';

    if ((   defined $setting::payment{'refundfee'}
            && $setting::payment{'refundfee'} ne ''
        )
        || $setting::payment{'refundnote'}
        )
    {
        print INFO
            '<div class="section" id="refund-info-box"><h2 id="refund-info-header">'
            . ( ( $setting::payment{'refundname'} )
            ? $this->tag_fix( $setting::payment{'refundname'} )
            : '払い戻し手数料' )
            . '</h2><div class="body">';
        print INFO '<p id="refundfee">'
            . $this->digit( $setting::payment{'refundfee'} )
            . '円</p>'
            if defined $setting::payment{'refundfee'}
            && $setting::payment{'refundfee'} ne '';
        print INFO '<p id="refund-info-note">'
            . $this->tag_fix( $setting::payment{'refundnote'} ) . '</p>'
            if $setting::payment{'refundnote'};
        print INFO '</div></div>';
    }

    my $carriagetable = Table->new;
    if ( defined $setting::page{'info'}{'carriagetable'}
        && $setting::page{'info'}{'carriagetable'} ne '' )
    {
        if ( ref $table::obj{ $setting::page{'info'}{'carriagetable'} } ) {
            %$carriagetable
                = %{ $table::obj{ $setting::page{'info'}{'carriagetable'} } };
        }
        elsif (
            defined $setting::table{ $setting::page{'info'}{'carriagetable'} }
            {'setting'}
            && $setting::table{ $setting::page{'info'}{'carriagetable'} }
            {'setting'} )
        {
            $carriagetable->set(
                $setting::table{ $setting::page{'info'}{'carriagetable'} }
                    {'setting'} );
            $carriagetable->{'name'}
                = $setting::table{ $setting::page{'info'}{'carriagetable'} }
                {'name'};
        }
    }
    $carriagetable->set( 'thenabled' => 'false' );

    if ( $setting::payment{'carriage'} ne 'disabled' ) {
        print INFO
            '<div class="section" id="carriage-info-box"><h2 id="carriage-info-header">送料</h2><div class="body">';
        print INFO '<dl id="carriage-info">'
            if $setting::payment{'yuumail'} || $setting::payment{'mailbin'};
        if ( $setting::payment{'carriage'} eq 'nation' ) {
            print INFO '<dd>'
                if $setting::payment{'yuumail'}
                || $setting::payment{'mailbin'};
            print INFO '<span class="datatitle">全国一律</span> '
                . $this->digit( $setting::payment{'nation'} )
                . ' <span class="unit">円</span>';
            print INFO '</dd>'
                if $setting::payment{'yuumail'}
                || $setting::payment{'mailbin'};
        }
        elsif ( $setting::payment{'carriage'} eq 'region' ) {
            print INFO '<dt>地域別送料</dt><dd>'
                if $setting::payment{'yuumail'}
                || $setting::payment{'mailbin'};
            my ($id);
            print INFO $carriagetable->table( 'id' => 'regeon-table' );
            foreach $id ( sort { $a <=> $b } keys %$region_name_table ) {
                print INFO $carriagetable->tr if $id % 3 == 1;
                print INFO $carriagetable->th( $region_name_table->{$id} );
                print INFO $carriagetable->td(
                    $this->digit( $setting::payment{'region'}{$id} )
                        . ' <span class="unit">円</span>' );
                print INFO $carriagetable->_tr if $id % 3 == 0;
            }
            print INFO $carriagetable->_table;
            print INFO '</dd>'
                if $setting::payment{'yuumail'}
                || $setting::payment{'mailbin'};
        }
        elsif ( $setting::payment{'carriage'} eq 'prefecture' ) {
            print INFO '<dt>都道府県別送料</dt><dd>'
                if $setting::payment{'yuumail'}
                || $setting::payment{'mailbin'};
            my ($id);
            print INFO $carriagetable->table( 'id' => 'pref-table' );
            foreach $id ( sort { $a <=> $b } keys %$pref_name_table ) {
                print INFO $carriagetable->tr if $id % 3 == 1;
                print INFO $carriagetable->th( $pref_name_table->{$id} );
                print INFO $carriagetable->td(
                    $this->digit( $setting::payment{'pref'}{$id} )
                        . ' <span class="unit">円</span>' );
                print INFO $carriagetable->_tr if $id % 3 == 0;
            }
            print INFO $carriagetable->th('');
            print INFO $carriagetable->td('');
            print INFO $carriagetable->_tr;
            print INFO $carriagetable->_table;
            print INFO '</dd>'
                if $setting::payment{'yuumail'}
                || $setting::payment{'mailbin'};
        }
        if (   $setting::payment{'carriage'} eq 'weight'
            || $setting::payment{'yuumail'} )
        {
            print INFO '<dt>'
                . ( ( $setting::payment{'yuumail'} )
                ? 'ゆうメール'
                : '重量別' )
                . '送料</dt><dd>'
                if $setting::payment{'yuumail'}
                || $setting::payment{'mailbin'};
            my ($id);
            $carriagetable->set( 'thenabled' => 'row' );
            print INFO $carriagetable->table( 'id' => 'weight-table' );
            print INFO $carriagetable->tr;
            print INFO $carriagetable->th('重量');
            print INFO $carriagetable->th('配送料');
            print INFO $carriagetable->_tr;

            foreach $id ( sort { $a <=> $b }
                keys %{ $setting::payment{'weight'} } )
            {
                print INFO $carriagetable->tr;
                print INFO $carriagetable->td(
                    '〜'
                        . $this->digit(
                        $setting::payment{'weight'}{$id}{'weight'}
                        )
                        . '<span class="unit">g</span>'
                );
                print INFO $carriagetable->td(
                    $this->digit(
                        $setting::payment{'weight'}{$id}{'carriage'}
                        )
                        . ' <span class="unit">円</span>'
                );
                print INFO $carriagetable->_tr;
            }
            print INFO $carriagetable->_table;
            print INFO '</dd>'
                if $setting::payment{'yuumail'}
                || $setting::payment{'mailbin'};
        }
        if (   $setting::payment{'carriage'} eq 'papersize'
            || $setting::payment{'mailbin'} )
        {
            print INFO '<dt>'
                . ( ( $setting::payment{'mailbin'} )
                ? 'クロネコヤマト・メール便'
                : 'サイズ別' )
                . '送料</dt><dd>'
                if $setting::payment{'yuumail'}
                || $setting::payment{'mailbin'};
            $carriagetable->set( 'thenabled' => 'row' );
            print INFO $carriagetable->table( 'id' => 'papersize-table' );
            print INFO $carriagetable->tr;
            print INFO $carriagetable->th('サイズ');
            print INFO $carriagetable->th('配送料');
            print INFO $carriagetable->_tr;
            print INFO $carriagetable->tr;
            print INFO $carriagetable->td(
                'A4(角2封筒以内)／厚さ=1cm まで');
            print INFO $carriagetable->td(
                $this->digit( $setting::payment{'papersize'}{'1A4'} )
                    . ' <span class="unit">円</span>' );
            print INFO $carriagetable->_tr;
            print INFO $carriagetable->tr;
            print INFO $carriagetable->td(
                'A4(角2封筒以内)／厚さ=2cm まで');
            print INFO $carriagetable->td(
                $this->digit( $setting::payment{'papersize'}{'2A4'} )
                    . ' <span class="unit">円</span>' );
            print INFO $carriagetable->_tr;
            print INFO $carriagetable->_table;
            print INFO '<div id="express-delivery">速達料金='
                . $this->digit(
                $setting::payment{'papersize'}{'expressdelivery'} )
                . '円</div>'
                if $setting::payment{'papersize'}{'expressdelivery'};
            print INFO '</dd>'
                if $setting::payment{'yuumail'}
                || $setting::payment{'mailbin'};
        }
        print INFO '</dl>'
            if $setting::payment{'yuumail'} || $setting::payment{'mailbin'};
        if ( $setting::payment{'specialcarriage'}{'requirements'} ) {
            print INFO '<p id="spesialcarriage-info">';
            print INFO $this->digit(
                $setting::payment{'specialcarriage'}{'requirements'} )
                . ' 円以上の注文で、全国一律 <span id="spesialcarriage-cost">';
            print INFO $this->digit(
                $setting::payment{'specialcarriage'}{'cost'} )
                . ' 円</span>となります。';
            print INFO '</p>';
        }
        if ( $setting::payment{'carriagenote'} ) {
            print INFO '<p id="carrage-info-note">';
            print INFO $this->tag_fix( $setting::payment{'carriagenote'} );
            print INFO '</p>';
        }
        print INFO '</div></div>';
    }
    $this->print_page_footer( 'info', \*INFO, $path );
    undef %{ $setting::page{'info'} };
    close(INFO);
}

sub print_payment_method_info ($$$$) {
    my ( $this, $handle, $method_id, $table ) = @_;
    if ( $method_id eq 'bank' ) {
        if ( $setting::payment{'bankenabled'} ) {
            print $handle '<dt id="bank-info-name">銀行振込</dt>';
            foreach my $n ( sort { $a <=> $b }
                keys %{ $setting::payment{'bank'} } )
            {
                print $handle '<dd class="bank-info-detail">';
                print $handle $table->table;
                print $handle $table->tr;
                print $handle $table->th('銀行');
                print $handle $table->td(
                    $this->escape( $setting::payment{'bank'}{$n}{'bank'} ) );
                print $handle $table->_tr;
                if ( $setting::payment{'bankinfo'} ) {
                    print $handle $table->tr;
                    print $handle $table->th('支店');
                    print $handle $table->td(
                        $this->escape(
                            $setting::payment{'bank'}{$n}{'office'}
                        )
                    );
                    print $handle $table->_tr;
                    if ( $setting::payment{'bankinfo'} >= 2 ) {
                        print $handle $table->tr;
                        print $handle $table->th('口座');
                        print $handle $table->td(
                            (   (   defined $setting::payment{'bank'}{$n}
                                        {'deposit'}
                                        && $setting::payment{'bank'}{$n}
                                        {'deposit'} eq 'current'
                                ) ? '[当座] ' : '[普通] '
                            )
                            . $setting::payment{'bank'}{$n}{'number'}
                        );
                        print $handle $table->_tr;
                        if ( $setting::payment{'bankinfo'} == 3 ) {
                            print $handle $table->tr;
                            print $handle $table->th('名義');
                            print $handle $table->td(
                                $this->escape(
                                    $setting::payment{'bank'}{$n}{'name'}
                                )
                            );
                            print $handle $table->_tr;
                            print $handle $table->tr;
                            print $handle $table->th('カタカナ表記');
                            print $handle $table->td(
                                $setting::payment{'bank'}{$n}{'kana'} );
                            print $handle $table->_tr;
                        }
                    }
                }
                print $handle $table->_table;
                print $handle '</dd>';
            }
            print $handle '<dd id="bank-info-note">'
                . $this->tag_fix( $setting::payment{'banknote'} ) . '</dd>'
                if $setting::payment{'banknote'};
        }

    }
    elsif ( $method_id eq 'yubin' ) {
        if ( $setting::payment{'yubinenabled'} ) {
            print $handle
                '<dt id="yuubin-info-name">ゆうちょ銀行 (郵便振替)</dt>';
            print $handle '<dd id="yuubin-info-detail">';
            if ( $setting::payment{'yubininfo'} ) {
                print $handle $table->table( 'id' => 'yuubin-info-table' );
                if ( $setting::payment{'yubin'}{'office'} ) {
                    print $handle $table->tr;
                    print $handle $table->th('郵便局');
                    print $handle $table->td(
                        $this->escape( $setting::payment{'yubin'}{'office'} )
                    );
                    print $handle $table->_tr;
                }
                my $banknumber;
                my $bankname;
                my $sougou;
                if ( $setting::payment{'yubin'}{'mark'}
                    =~ /^([01])([0-9]{2})/ )
                {
                    $bankname = $banknumber = $2 . ( ($1) ? '8' : '9' );
                    $sougou = $1;
                    my @suu = (
                        '〇', '一', '二', '三', '四', '五',
                        '六', '七', '八', '九'
                    );
                    $bankname =~ s/([0-9])/$suu[$1]/g;
                    print $handle $table->tr;
                    print $handle $table->th('店番');
                    print $handle $table->td($banknumber);
                    print $handle $table->_tr;
                    print $handle $table->tr;
                    print $handle $table->th('店名');
                    print $handle $table->td($bankname);
                    print $handle $table->_tr;
                }
                if ( $setting::payment{'yubininfo'} >= 2 ) {
                    my $deposit = $setting::payment{'yubin'}{'number'};
                    $deposit =~ s/1$// if $sougou;
                    $deposit = sprintf( "%.7d", $deposit );
                    print $handle $table->tr;
                    print $handle $table->th('記号-番号');
                    print $handle $table->td(
                        (   (   defined $setting::payment{'yubin'}{'mark'}
                                    && $setting::payment{'yubin'}{'mark'} ne
                                    ''
                            ) ? $setting::payment{'yubin'}{'mark'} . '-' : ''
                        )
                        . ( (   defined $setting::payment{'yubin'}{'digit'}
                                    && $setting::payment{'yubin'}{'digit'} ne
                                    ''
                            )
                            ? $setting::payment{'yubin'}{'digit'} . '-'
                            : ' -'
                            )
                            . $setting::payment{'yubin'}{'number'}
                    );
                    print $handle $table->_tr;
                    print $handle $table->tr;
                    print $handle $table->th('口座番号');
                    print $handle $table->td(
                        ( ($sougou) ? '[普通] ' : '[当座] ' )
                        . $deposit );
                    print $handle $table->_tr;

                    if ( $setting::payment{'yubininfo'} >= 3 ) {
                        print $handle $table->tr;
                        print $handle $table->th('名義');
                        print $handle $table->td(
                            $this->escape(
                                $setting::payment{'yubin'}{'name'}
                            )
                        );
                        print $handle $table->_tr;
                        if ( $setting::payment{'yubin'}{'kana'} ) {
                            print $handle $table->tr;
                            print $handle $table->th('カタカナ表記');
                            print $handle $table->td(
                                $this->escape(
                                    $setting::payment{'yubin'}{'kana'}
                                )
                            );
                            print $handle $table->_tr;
                        }
                    }
                }
                print $handle $table->_table;
            }
            else {
                print $handle 'ゆうちょ銀行';
            }
            print $handle '</dd>';
            print $handle '<dd id="yuubin-info-note">'
                . $this->tag_fix( $setting::payment{'yubinnote'} ) . '</dd>'
                if $setting::payment{'yubinnote'};
        }

    }
    elsif ( $method_id eq 'paypal' ) {
        if ( $setting::payment{'paypalenabled'} ) {
            print $handle '<dt id="paypal-info-name">'
                . ( $setting::payment{'paypal'}{'name'} || 'PayPal' )
                . '</dt>';
            print $handle '<dd id="paypal-info-detail">';
            if ( $setting::payment{'paypal'}{'banner'} ) {
                print $handle $setting::payment{'paypal'}{'banner'};
            }
            else {
                print $handle
                    '<a href="#" onclick="javascript:window.open(\'https://www.paypal.com/jp/webapps/mpp/logo/about\',\'olcwhatispaypal\',\'toolbar=no, location=no, directories=no, status=no, menubar=no, scrollbars=yes, resizable=yes, width=900, height=700\');"><img src="https://www.paypalobjects.com/digitalassets/c/website/marketing/apac/jp/developer/Logo_203_80.png" border="0" alt="ペイパル｜Mastercard,VISA,American Express,JCB"></a></a>';
            }
            print $handle '</dd>';
            print $handle '<dd id="paypal-info-note">'
                . $this->tag_fix( $setting::payment{'paypalnote'} ) . '</dd>'
                if $setting::payment{'paypalnote'};
        }

    }
    elsif ( $method_id eq 'cod' ) {
        if ( $setting::payment{'codenabled'} ) {
            $table->set( 'thenabled' => 'true' );
            print $handle '<dt id="cod-info-name">代金引き換え</dt>';
            print $handle '<dd id="cod-info-detail">';
            print $handle $table->table( 'id' => 'cod-info-table' );
            print $handle $table->tr;
            print $handle $table->th(
                ( $setting::general{'itemname'} || '商品' ) . '代金' );
            print $handle $table->th('手数料');
            print $handle $table->_tr;
            print $handle $table->tr;
            print $handle $table->td('1万円未満');
            print $handle $table->td(
                $this->digit( $setting::payment{'cod'}{'fee1'} )
                    . ' <span class="unit">円</span>' );
            print $handle $table->_tr;

            if ( $setting::payment{'cod'}{'fee2'} ne '' ) {
                print $handle $table->tr;
                print $handle $table->td('1万円以上3万円未満');
                print $handle $table->td(
                    $this->digit( $setting::payment{'cod'}{'fee2'} )
                        . ' <span class="unit">円</span>' );
                print $handle $table->_tr;
                if ( $setting::payment{'cod'}{'fee3'} ne '' ) {
                    print $handle $table->tr;
                    print $handle $table->td('3万円以上10万円未満');
                    print $handle $table->td(
                        $this->digit( $setting::payment{'cod'}{'fee3'} )
                            . ' <span class="unit">円</span>' );
                    print $handle $table->_tr;
                    if ( $setting::payment{'cod'}{'fee4'} ne '' ) {
                        print $handle $table->tr;
                        print $handle $table->td(
                            '10万円以上30万円未満');
                        print $handle $table->td(
                            $this->digit( $setting::payment{'cod'}{'fee4'} )
                                . ' <span class="unit">円</span>' );
                        print $handle $table->_tr;
                    }
                }
            }
            print $handle $table->_table;
            print $handle '</dd>';
            print $handle '<dd id="cod-info-note">'
                . $this->tag_fix( $setting::payment{'codnote'} ) . '</dd>'
                if $setting::payment{'codnote'};
            $table->set( 'thenabled' => 'false' );
        }

    }
    elsif ( $method_id eq 'moneyorder' ) {
        if ( $setting::payment{'moneyorderenabled'} ) {
            print $handle
                '<dt id="moneyorder-info-name"><a href="http://www.jp-bank.japanpost.jp/kojin/tukau/sokin/hikoza/kj_tk_sk_hkz_kogawase.html" target="_blank">定額小為替</a></dt>';
            print $handle '<dd id="moneyorder-info-note">'
                . $this->tag_fix( $setting::payment{'moneyordernote'} )
                . '</dd>'
                if $setting::payment{'moneyordernote'};
        }

    }
    elsif ( $method_id eq 'other' ) {
        if ( $setting::payment{'otherenabled'} ) {
            foreach my $n ( sort { $a <=> $b }
                keys %{ $setting::payment{'other'} } )
            {
                print $handle '<dt class="payment-info-name">'
                    . $setting::payment{'other'}{$n}{'name'} . '</dt>';
                print $handle '<dd class="payment-info-detail">'
                    . $this->tag_fix( $setting::payment{'other'}{$n}{'note'} )
                    . '</dd>';
            }
        }
        print $handle '</dl></div>';
    }
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 'c210', 'MkPaymentTerms' );

    #メニュー登録
    $setting::jump_menu{'c210'} = '支払い方法・送料';

}

1;
