package CategoryEditor;
require CMONOS::TableMaker::TMCommon;
@ISA = qw(TMCommon);
use strict;
use utf8;
######################################################################
# カテゴリー編集モジュール
# CategoryEditor.pm ver1.2.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2012-05-05
######################################################################
# 実行
sub run ($) {
    my $this = shift;

    #書き込み
    if ( defined $main::a && $main::a eq 'e' ) {
        no warnings qw(numeric);

        #データ削除
        push( @main::delete, $main::delete )
            if defined $main::delete && $main::delete;
        @main::delete = grep( defined $_ && $_ ne '', @main::delete );
        if (@main::delete) {

            #順序判定への影響を回避
            foreach (@main::delete) {
                delete $main::old{$_};
                delete $main::order{$_};
            }
            $this->delete_category(@main::delete);
        }

        #順序判定
        my $no = 0;
        foreach (
            sort {
                $main::order{$a} <=> $main::order{$b}
                    || $main::old{$b} <=> $main::old{$a}
                    || $main::order{$a} cmp $main::order{$b}
            } keys %main::order
            )
        {
            $no++;
            $main::data{ 'ORDER_' . $no } = $_;
            $main::data{ $_ . '_ORDER' } = $no;
        }

        #不正なデータの除去
        my ( $k, $v, @name );
        while ( ( $k, $v ) = each %main::order ) {
            $this->detag( \$main::data{ $k . '_name' } );
            $main::data{ $k . '_name' } =~ s!/$!!;
            push( @name, $main::data{ $k . '_name' } );
            $main::data{ $k . '_gmap' } = '' if !$main::data{ $k . '_gmap' };
        }
        undef $main::data{'LIMIT'} if $this->is_nan( $main::data{'LIMIT'} );
        $main::data{'MULTIPLE'} = ''
            if !defined $main::data{'MULTIPLE'} || !$main::data{'MULTIPLE'};

        #新規作成
        if ( defined $main::new{'name'} && $main::new{'name'} ne '' ) {
            $this->detag( \$main::new{'name'} );
            $main::new{'name'} =~ s!/$!!;
            push( @name, $main::new{'name'} );
            my ( $k, $v, $n );
            while (1) {
                $n++;
                last if !defined $main::old{$n} || $main::old{$n} eq '';
            }
            $no++;
            $main::data{ 'ORDER_' . $no } = $n;
            $main::data{ $n . '_ORDER' } = $no;
            while ( ( $k, $v ) = each %main::new ) {
                $main::data{ $n . '_' . $k } = $v;
            }
            @main::delete = grep( $_ ne $n, @main::delete );
            $this->mk_category($n);
        }

        #名前チェック
        while ( ( $k, $v ) = each %main::order ) {
            $main::data{ $k . '_name' }
                .= '/' . ( ($default::english) ? 'Others' : 'その他' )
                while ( grep( $_ =~ m!^$main::data{$k . '_name'}/!, @name ) );
        }

        $this->write_db( $default::preference_dir . '/category',
            \%main::data, 'ORDER', \@main::delete, \%setting::category, 1 );
        $this->write_category_list;
        require CMONOS::CGI_Lib::NavUpdate;
        NavUpdate->new->scan->update;

    }
    else {
        #読み込み
        $this->import_db( $default::preference_dir . '/category',
            \%setting::category, 1 );
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>カテゴリー編集</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="e" />';
    print '<fieldset><legend>カテゴリー編集</legend>';

    #テーブル設定
    require CMONOS::CGI_Lib::Table;
    my $table = Table->new;
    $table->set(
        'tableclass' => 'tooltable',
        'trclass'    => 'odd,even',
        'class0'     => 'odd,even',
        'classF'     => 'caution',
        'style0'     => 'white-space: nowrap;text-align: center;',
        'thenabled'  => 'true'
    );

    print $table->table;

    #ヘッダ
    print $table->tr;
    print $table->td('削除');
    print $table->td('表示順');
    print $table->td('カテゴリー名');
    print $table->td('地図');
    print $table->_tr;

    #設定項目
    my ( @categories, %done, $c );
    if (%setting::category) {
        no warnings qw(numeric);
        @categories = sort {
            $setting::category{$a}{'ORDER'}
                <=> $setting::category{$b}{'ORDER'}
                || $a <=> $b
        } grep( defined $_ && $_ !~ /[^0-9]/, keys %setting::category );
    }
    foreach $c (@categories) {
        $setting::category{$c}{'ORDER'} = $c
            if !defined $setting::category{$c}{'ORDER'};
        print $table->tr;
        print $table->td( '<input type="checkbox" name="delete" value="'
                . $c
                . '" onclick="cmonos.form.field(this).checkboxConfirm(\''
                . $setting::category{$c}{'name'}
                . 'に関連する全てのデータが削除されます。本当に削除しますか？\')" />'
        );
        print $table->td( '<input type="hidden" name="old:'
                . $c
                . '" value="'
                . $setting::category{$c}{'ORDER'}
                . '" /><input type="number" name="order:'
                . $c
                . '" value="'
                . $setting::category{$c}{'ORDER'}
                . '" size="3" maxlength="3" />' );
        print $table->td( '<input type="text" name="data:'
                . $c
                . '_name" value="'
                . $setting::category{$c}{'name'}
                . '" size="60" />' );
        print $table->td(
            $this->gmap_settings(
                'data:' . $c . '_', $setting::category{$c}
            )
        );
        print $table->_tr;
        $done{$c}++;
    }

    #新規作成
    print $table->tr;
    print $table->td('');
    print $table->td('新規');
    print $table->td('<input type="text" name="new:name" size="60" />');
    print $table->td( $this->gmap_settings('new:') );
    print $table->_tr;

    print $table->_table;
    print
        '<div class="hints">カテゴリーを追加するには新規カテゴリー名を記入します。</div>';
    print
        '<div class="hints">カテゴリー名を / で区切ると階層を表現できます。</div>';
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';

    print '<fieldset><legend>カテゴリー設定</legend>';
    print '<table class="field-table"><tbody>';

    if ( !ref $setting::category{'DEFAULT'} ) {
        my $temp
            = ( defined $setting::category{'DEFAULT'} )
            ? $setting::category{'DEFAULT'}
            : '';
        $setting::category{'DEFAULT'} = [];
        push( @{ $setting::category{'DEFAULT'} }, $temp ) if $temp ne '';
    }
    my $size = 0;
    my $selector
        = $this->mk_category_selector( \%setting::category, \@categories,
        $setting::category{'DEFAULT'}, \$size );
    print '<tr>';
    print
        '<td class="field-name"><label for="data:DEFAULT">選択済みカテゴリー</label></td>';
    print '<td class="field"><select name="data:DEFAULT" id="data:DEFAULT"';
    if ( defined $setting::category{'MULTIPLE'}
        && $setting::category{'MULTIPLE'} )
    {
        $size = 1 if $size < 1;
        print ' size="' . $size . '" multiple="multiple"';
    }
    print '>';
    print $selector;
    undef $selector;
    undef $size;
    print '</select>';
    print ' <label><input type="checkbox" name="data:MULTIPLE" value="1"'
        . (
        (   defined $setting::category{'MULTIPLE'}
                && $setting::category{'MULTIPLE'}
        ) ? ' checked="checked"' : ''
        ) . ' /> 複数のカテゴリーを選択可</label>';
    print '</td>';
    print '</tr>';

    $setting::category{'LIMIT'} = 2000
        if !defined $setting::category{'LIMIT'}
        || $setting::category{'LIMIT'} eq ''
        || $setting::category{'LIMIT'} =~ /[^0-9]/;
    print '<tr>';
    print
        '<td class="field-name"><label for="data:ADD">新規レコード</label></td>';
    print '<td class="field">';
    print
        '<label for="data:LIMIT">レコード数</label>が <input type="number" size="4" maxlength="4" name="data:LIMIT" id="data:LIMIT" value="'
        . $setting::category{'LIMIT'}
        . '" /> 件を超えたとき ';

    if ( !$default::type
        || ( $default::type ne 'account' && $default::type ne 'order' ) )
    {
        print $this->mk_select_menu(
            'data:ADD', '', '', $setting::category{'ADD'},
            'add' =>
                ' 最も古いレコードを消去して新しいレコードを追加する。',
            'newcategory' =>
                ' 新しいカテゴリーを作成して新しいレコードを追加する。',
            'refuse' =>
                ' 新しいレコードの書き込みを拒絶する。'
        );
    }
    else {
        print
            '新しいカテゴリーを作成して新しいレコードを追加する。';
    }
    print '</td>';
    print '</tr>';

    print '</tbody></table>';
    print '</fieldset>';

    print
        '<fieldset><legend><label for="data:INFOFORMAT">カテゴリーリスト レコード数情報フォーマット</label></legend>';
    print
        '<input type="text" size="90" name="data:INFOFORMAT" id="data:INFOFORMAT" value="'
        . $this->escape( $setting::category{'INFOFORMAT'} ) . '" />';
    print
        '<div class="note">%p=公開レコード数 / %s=非公開レコード数 / %t=レコード総数</div>';
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
    print '</form>';

    print '<form name="sample-codes" id="sample-codes" action="#">';
    my $target
        = $this->path_change( $default::file, $ENV{'SCRIPT_NAME'}, 'abs' )
        . '/category_list';
    if ( defined $env::ssi && $env::ssi ) {
        print
            '<fieldset class="sample"><legend><label for="sample-ssi">カテゴリーリスト埋め込みタグ(静的 SSI)</label></legend>';
        print '<textarea cols="70" rows="3">&lt;!--#include virtual=&quot;'
            . $target
            . '.ssi&quot;--&gt;</textarea>';
        print '</fieldset>';
    }
    $target = $this->http_my_domain . $target;
    print
        '<fieldset class="sample"><legend><label for="sample-js">カテゴリーリスト埋め込みタグ(静的 JavaScript)</label></legend>';
    print
        '<textarea cols="70" rows="3">&lt;script type=&quot;text/javascript&quot; language=&quot;JavaScript&quot; src=&quot;'
        . $target
        . '.js&quot;&gt;&lt;/script&gt;</textarea>';
    print '</fieldset>';
    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');

}

sub delete_category ($@) {
    my ( $this, @delete ) = @_;
    my ( %email_to_id, @urls, $home_url, $email_to_id_tied );
    $home_url = $this->my_url . '?m=DetailViewer&c=';
    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    if ( $setting::general{'oneEmailToOneRecord'} ) {
        $this->use_field(
            'field',
            $default::field_pref_dir . '/inputfield',
            $setting::field,
            $setting::field_style_disabled,
            $setting::field_tag_enabled,
            $setting::field_tag_name
        ) if !%field::obj;
        if ( $field::for{'mail'} ) {
            tie(%email_to_id, $env::dbm_file,
                $default::db . '/email_to_id',
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );
            $email_to_id_tied = 1;
        }
    }
    use File::Path;
    foreach my $c (@delete) {
        if ($email_to_id_tied) {
            my (%email);
            tie(%email,
                $env::dbm_file,
                $default::db . '/'
                    . $c . '/'
                    . $field::obj{ $field::for{'mail'} }->db,
                O_RDONLY,
                $env::dbm_mode
            );
            if ( ref tied %email ) {
                while ( my ( $id, $email ) = each %email ) {
                    utf8::encode($email) if utf8::is_utf8($email);
                    delete $email_to_id{$email};
                }
                untie %email;
            }
        }
        my ( %ctime, %record_id );
        tie( %ctime, $env::dbm_file, $default::db . '/' . $c . '/ctime',
            O_RDONLY, $env::dbm_mode );
        tie( %record_id, $env::dbm_file, $default::db . '/record_id',
            O_RDWR, $env::dbm_mode );
        if ( ref tied %ctime && ref tied %record_id ) {
            while ( my ($n) = each %ctime ) {
                delete $record_id{$n};
            }
        }
        untie %ctime;
        untie %record_id;
        if ( -d "$default::file/$c" ) {
            rmtree "$default::file/$c";
        }
        if ( -d "$default::db/$c" ) {
            rmtree "$default::db/$c";
        }
        push( @urls, $home_url . $c );
    }
    if ($email_to_id_tied) {
        untie %email_to_id;
    }
    require CMONOS::CGI_Lib::IndexSweeper;
    IndexSweeper->new(
        'index_files' => [
            $env::externalfiles . '/recentcomments.ssi',
            $env::externalfiles . '/recenttrackbacks.ssi',
        ],
        'rss_files' => [
            '../../index.xml',
            '../../comment.xml',
            '../../trackback.xml',
            '../../podcast.xml',
            $default::file . '/index.xml',
            $default::file . '/comment.xml',
            $default::file . '/trackback.xml',
            $default::file . '/podcast.xml',
        ],
    )->sweep( \@urls, { $default::id => \@delete } );
    return;
}

sub mk_category ($$) {
    my ( $this, $id ) = @_;
    $this->set_dir("$default::db/$id");
    $this->set_dir("$default::file/$id");
    return;
}

sub gmap_settings ($$;$) {
    my ( $this, $name, $setting ) = @_;
    $this->import_db( $default::preference_dir . '/page',
        \%setting::page, 1, 'gmap' )
        if !%setting::page || !ref $setting::page{'gmap'};
    my $form = '';
    my $url  = $this->escape(
              $env::javascript
            . '/gmap.html?'
            . $this->url_encoded_query(
            {   (   ($env::google_api_key) ? ( 'key' => $env::google_api_key )
                    : ()
                ),
                'z' => ( ( $setting->{'z'} ) ? $setting->{'z'} : 4 ),
                't' => ( ( $setting->{'t'} ) ? $setting->{'t'} : '' ),
                'c' => (
                    ( $setting->{'c'} ) ? $setting->{'c'}
                    : '36.315125,137.504883'
                ),
                'z_field' => $name . 'z',
                't_field' => $name . 't',
                'c_field' => $name . 'c',
                'om'      => 1,
                'sv'      => 1,
                's'       => 1,
            }
            )
    );
    $form
        .= '<input type="checkbox" name="'
        . $name
        . 'gmap" value="1"'
        . ( ( $setting->{'gmap'} ) ? ' checked="checked"' : '' )
        . ' title="このカテゴリーの地図を設定する" onclick="if (this.checked) {cmonos.window.open(\'gmapWindow\','
        . ( ( $setting::page{'gmap'}{'gmapWidth'} )
        ? $setting::page{'gmap'}{'gmapWidth'}
        : 400 )
        . ','
        . ( ( $setting::page{'gmap'}{'gmapHeight'} )
        ? $setting::page{'gmap'}{'gmapHeight'}
        : 400 )
        . ',\''
        . $url
        . '\',true)}" />';
    $form
        .= '<input type="hidden" name="'
        . $name
        . 'z" id="'
        . $name
        . 'z" value="'
        . $this->escape( $setting->{'z'} ) . '" />';
    $form
        .= '<input type="hidden" name="'
        . $name
        . 't" id="'
        . $name
        . 't" value="'
        . $this->escape( $setting->{'t'} ) . '" />';
    $form
        .= '<input type="hidden" name="'
        . $name
        . 'c" id="'
        . $name
        . 'c" value="'
        . $this->escape( $setting->{'c'} ) . '" />';
    return $form;
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( '00t0', 'CategoryEditor' );

    #メニュー登録
    $setting::jump_menu{'00t0'} = 'カテゴリー編集';

}

1;
