package DetailPage;
require CMONOS::CGI_Lib::Tools::Page;
@ISA = qw(Page);
use strict;
use utf8;
no warnings qw(once);
######################################################################
# 詳細ページ設定モジュール
# DetailPage.pm ver1.2.2
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-09-28
######################################################################
# 設定項目取得
sub get_sub_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #チェックボックス
    push( @check::checkbox,
        'fieldTitleOption',   'headerNeighborLink',
        'footerNeighborLink', 'closeButtonDisabled' );

    #数字チェック
    $check::num{'fieldTitleWidth'} = '項目名幅';
    $check::num{'fieldWidth'}      = '項目幅';
    $check::num{'keptIP'}          = '訪問者IP保存数';

    return;
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    my $detail_no;
    $detail_no = ( defined $main::k && $main::k =~ /_([0-9]+)$/ ) ? $1 : '';
    $preference::id = 'detail' . $detail_no;    # データキー
    $preference::name
        = '詳細表示ページ'
        . $detail_no
        . 'レイアウト';                    # 設定パネル名
    $preference::title
        = '詳細表示ページ'
        . $detail_no
        . 'のレイアウトについて設定します。';
    require CMONOS::CGI_Lib::Deco;
    $this->{'icon_obj'} = Deco->new( $env::image_dir . '/tablemaker',
        $env::image_lib . '/tablemaker' );
    $setting::image_css = 'tablemaker-image.css';
}

######################################################################
# 設定パネル
sub _print_header_tool ($) {
    my $this = shift;
    return
        '<option value="&lt;!--categorylist--&gt;&lt;!--/categorylist--&gt;">カテゴリーリスト(&lt;!--categorylist--&gt;)</option><option value="&lt;!--search--&gt;&lt;!--/search--&gt;">検索フォーム (&lt;!--search--&gt;)</option>';
}

sub print_sub_preference_panel ($;$$) {
    my ( $this, $id, $key ) = @_;
    my ($fieldTitleOption_checked,   $headerNeighborLink_checked,
        $footerNeighborLink_checked, $closeButtonDisabled_checked
    );
    $fieldTitleOption_checked
        = ( defined $main::data{ $key . 'fieldTitleOption' }
            && $main::data{ $key . 'fieldTitleOption' } )
        ? ' checked="checked"'
        : '';
    $headerNeighborLink_checked
        = ( defined $main::data{ $key . 'headerNeighborLink' }
            && $main::data{ $key . 'headerNeighborLink' } )
        ? ' checked="checked"'
        : '';
    $footerNeighborLink_checked
        = ( defined $main::data{ $key . 'footerNeighborLink' }
            && $main::data{ $key . 'footerNeighborLink' } )
        ? ' checked="checked"'
        : '';
    $closeButtonDisabled_checked
        = ( defined $main::data{ $key . 'closeButtonDisabled' }
            && $main::data{ $key . 'closeButtonDisabled' } )
        ? ' checked="checked"'
        : '';

    print <<PANEL;
<fieldset>
<legend>詳細表示設定</legend>
<fieldset>
<legend>前後レコードへのリンク</legend>
<ol class="checkbox">
<li><label><input type="checkbox" name="data:${id}headerNeighborLink" id="data:${id}headerNeighborLink" value="1"$headerNeighborLink_checked /> ページ上部に表示する</label></li>
<li><label><input type="checkbox" name="data:${id}footerNeighborLink" id="data:${id}footerNeighborLink" value="1"$footerNeighborLink_checked /> ページ下部に表示する</label>
(<label><input type="checkbox" name="data:${id}closeButtonDisabled" id="data:${id}closeButtonDisabled" value="1"$closeButtonDisabled_checked /> 閉じるボタンを表示しない</label>)</li>
</ol>
</fieldset>
<fieldset>
<legend>一覧表示設定</legend>
<table class="field-table"><tbody>
<tr>
<td class="field-name"><label for="data:${id}table">一覧表示テーブルタイプ</label></td>
<td class="field">
PANEL

    print $this->table_menu( 'data:' . $id . 'table',
        $main::data{ $key . 'table' } );

    print <<PANEL;
</td>
<td><label for="data:${id}tableValign">一覧表示位置</label></td>
<td class="field">
PANEL

    print $this->mk_select_menu(
        'data:' . $id . 'tableValign', '', '',
        $main::data{ $key . 'tableValign' },
        'top'    => '上',
        'bottom' => '下'
    );

    print <<PANEL;
</td>
</tr>
</tbody></table>
</fieldset>
<fieldset class="foldable">
<legend>項目名フレーム</legend>
<label for="data:${id}fieldTitleOption"><input type="checkbox" name="data:${id}fieldTitleOption" id="data:${id}fieldTitleOption" value="1"$fieldTitleOption_checked title="この設定をチェックすると下記の項目名フレーム設定は無視されます。" /> 項目名を項目フレームのタイトルバー部分に表示する</label>
PANEL

    print $cgi::Deco->frame_menu( 'data:' . $id . 'fieldTitleFrame',
        'frame', $main::data{ $key . 'fieldTitleFrame' } );

    print <<PANEL;
<div class="additional-field"><label for="data:${id}fieldTitleWidth">項目名幅</label><input type="text" name="data:${id}fieldTitleWidth" id="data:${id}fieldTitleWidth" value="$main::data{$key . 'fieldTitleWidth'}" size="7" maxlength="7" /> <span class="unit">ピクセル</span></div>
</fieldset>
<fieldset class="foldable">
<legend>項目フレーム</legend>
PANEL

    print $cgi::Deco->frame_menu( 'data:' . $id . 'fieldFrame',
        'frame', $main::data{ $key . 'fieldFrame' } );

    print <<PANEL;
<div class="additional-field"><label for="data:${id}fieldWidth">項目幅</label><input type="text" name="data:${id}fieldWidth" id="data:${id}fieldWidth" value="$main::data{$key . 'fieldWidth'}" size="7" maxlength="7" /> <span class="unit">ピクセル</span> 
<label for="data:${id}boxAlign">項目フレーム内行揃え</label>
PANEL

    print $this->mk_select_menu(
        'data:' . $id . 'boxAlign', '', '', $main::data{ $key . 'boxAlign' },
        ''       => '',
        'left'   => '左寄せ',
        'center' => '中央揃え',
        'right'  => '右寄せ'
    );
    print $this->mk_select_menu(
        'data:' . $id . 'boxValign', '', '',
        $main::data{ $key . 'boxValign' },
        ''       => '',
        'top'    => '上寄せ',
        'middle' => '中央揃え',
        'bottom' => '下寄せ'
    );

    print <<PANEL;
</div>
</fieldset>
</fieldset>
PANEL

    $this->use_field( 'field', $default::field_pref_dir . '/inputfield', 2 );

    require CMONOS::CGI_Lib::Template;
    my $this_template = Template->new( \%field::obj );
    my $tool          = '';
    $tool
        .= '<select class="tool" onchange="cmonos.form.item(this.form).fieldname(\'data:'
        . $id
        . 'template\').quotationByMenu(this)" title="これらのタグは、対応するデータに変換されます。">';
    $tool .= '<option value="">▼特殊タグを挿入する</option>';
    $tool
        .= '<option value="&lt;!--categorylist--&gt;&lt;!--/categorylist--&gt;">カテゴリーリスト(&lt;!--categorylist--&gt;)</option>';
    $tool
        .= '<option value="&lt;!--search--&gt;&lt;!--/search--&gt;">検索フォーム (&lt;!--search--&gt;)</option>';

    if ( $default::type && $default::type eq 'order' ) {
        $tool .= '<option value=""></option>';
        $tool
            .= '<option value="&lt;!--sheet--&gt;&lt;!--/sheet--&gt;">品目 (&lt;!--sheet--&gt;)</option>';
        $tool
            .= '<option value="&lt;!--date--&gt;&lt;!--/date--&gt;">日付 (&lt;!--date--&gt;)</option>';
        $tool .= '<option value=""></option>';
        $tool
            .= '<option value="&lt;!--invoice--&gt;&lt;!--/invoice--&gt;">請求書でのみ表示 (&lt;!--invoice--&gt;)</option>';
        $tool
            .= '<option value="&lt;!--receipt--&gt;&lt;!--/receipt--&gt;">領収書でのみ表示 (&lt;!--receipt--&gt;)</option>';
        $tool .= '<option value=""></option>';
        $tool
            .= '<option value="&lt;!--buyer--&gt;&lt;!--/buyer--&gt;">名前 (&lt;!--buyer--&gt;)</option>';
        $tool
            .= '<option value="&lt;!--buyer_email--&gt;&lt;!--/buyer_email--&gt;">代表メールアドレス (&lt;!--buyer_email--&gt;)</option>';
        $tool
            .= '<option value="&lt;!--buyer_address--&gt;&lt;!--/buyer_address--&gt;">配送先住所 (&lt;!--buyer_address--&gt;)</option>';
        $tool .= '<option value=""></option>';
        $tool
            .= '<option value="&lt;!--seller--&gt;&lt;!--/seller--&gt;">担当者の名前(管理人名) (&lt;!--seller--&gt;)</option>';
        $tool
            .= '<option value="&lt;!--seller_email--&gt;&lt;!--/seller_email--&gt;">担当者のメールアドレス(管理人アドレス) (&lt;!--seller_email--&gt;)</option>';
        $tool
            .= '<option value="&lt;!--seller_sitename--&gt;&lt;!--/seller_sitename--&gt;">このサイトの名前 (&lt;!--seller_sitename--&gt;)</option>';
        $tool
            .= '<option value="&lt;!--seller_url--&gt;&lt;!--/seller_url--&gt;">このサイトのURL (&lt;!--seller_url--&gt;)</option>';
        $tool
            .= '<option value="&lt;!--seller_company--&gt;&lt;!--/seller_company--&gt;">このサイトの著作権者 (&lt;!--seller_company--&gt;)</option>';
    }
    $tool .= '</select>';

    $this_template->print_template_form(
        'data:' . $id . 'template',
        '詳細表示テンプレート（上記詳細表示設定よりテンプレートが優先されます）',
        \$main::data{ $key . 'template' },
        \$tool,
        'ページテンプレート中に &lt;html&gt; タグおよび &lt;body&gt; タグがある場合、各ページレイアウト設定よりページテンプレートが優先されます。'
    );

    print '<fieldset><legend>レコード編集ボタン設定</legend>';
    print '<fieldset><legend>レコード編集ボタン</legend>';
    print $this->{'icon_obj'}->button_menu( 'data:' . $id . 'detailedit',
        'detailedit', 120, $main::data{ $id . 'detailedit' } );
    print '</fieldset>';
    print '<fieldset><legend>レコード削除ボタン</legend>';
    print $this->{'icon_obj'}->button_menu( 'data:' . $id . 'detaildelete',
        'detaildelete', 120, $main::data{ $id . 'detaildelete' } );
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset><legend>訪問者IP保存数</legend>';
    print
        '<label>直前訪問者IPを <input type="text" size="8" name="data:'
        . $id
        . 'keptIP" value="'
        . $this->escape( $main::data{ $id . 'keptIP' } )
        . '" /> 件まで保存し訪問者IPがいずれかに一致する場合アクセスをカウントしない</label>';
    print '</fieldset>';
}

######################################################################
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 't010', 'DetailPage' );

    #ツール登録
    $setting::sub_jump_menu{'page'}{'t010'} = '詳細表示ページ';
    $setting::selected_jump_menu{'t010'} = 't000'
        ; #サブメニューがあるときメインメニューのデフォルトキー
    $preference::page{'t010'} = 'detail';

    if ( $default::detail_variation >= 2 ) {
        foreach my $n ( 2 .. $default::detail_variation ) {
            my $k = 't010_' . $n;
            Common::class_register( $k, 'DetailPage' );
            $setting::sub_jump_menu{'page'}{$k}
                = '詳細表示ページ' . $n;
            $setting::selected_jump_menu{$k} = 't000'
                ; #サブメニューがあるときメインメニューのデフォルトキー
            $preference::page{$k} = 'detail' . $n;
        }
    }
}

1;
