package InputField;
require CMONOS::CGI_Lib::Tools::FieldEditor;
@ISA = qw(FieldEditor);
use strict;
use utf8;
######################################################################
# 入力フィールド編集モジュール
# InputField.pm ver1.2.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2014-11-25
######################################################################
# フィールド読み込み設定
sub set_field ($) {
    no warnings qw(once);
    my $this = shift;
    $setting::field_db_name = 'inputfield';
    $setting::field         = 2;
    push( @check::checkbox,
        'protected',   'Hidden',        'Default',
        'detailTable', 'searchEnabled', 'sortEnabled' );
    if ( $default::type && $default::type eq 'order' ) {
        push( @check::checkbox, 'skippable' );
    }
    $this->{'foldable'} = 1;
}

######################################################################
# 新規フィールドDB
sub mk_field ($$) {
    my ( $this, $id ) = @_;
    return
        if $field::obj{$id}->db ne $id
        || $field::obj{$id}->{'class'} eq 'xtm_sticky';

    my ( @category, $c );
    local ( *CATEGORY, *DIR );

    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';

    #データベース作成
    $default::db =~ s!/$!!;
    opendir( CATEGORY, $default::db );
    @category
        = grep( $_ !~ /^\./ && -d "$default::db/$_", readdir(CATEGORY) );
    closedir(CATEGORY);
    foreach $c (@category) {
        my $dir = $default::db . '/' . $c;
        if ( $this->set_dir($dir) ) {
            my ( %new, %mtime );
            tie(%new, $env::dbm_file,
                $dir . '/' . $id,
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );
            if ( ref tied %new ) {
                tie( %mtime, $env::dbm_file, $dir . '/mtime',
                    O_RDONLY, $env::dbm_mode );
                if ( ref tied %mtime ) {
                    while ( my ($n) = each %mtime ) {
                        utf8::encode($n) if utf8::is_utf8($n);
                        $new{$n} = '' if !defined $new{$n} || $new{$n} eq '';
                    }
                }
                untie(%mtime);
            }
            untie(%new);
        }
    }
}

######################################################################
# 設定項目名
sub print_setting_item_name ($$) {
    my ( $this, $table ) = @_;
    print $table->td('セル幅');
    print $table->td('位置');
    print $table->td('位置');
    print $table->td('保護');
    print $table->td('公開');
    print $table->td('ダウンロード<br />販売時省略')
        if $default::type && $default::type eq 'order';
    print $table->td('隠す');
    print $table->td('テーブル表示<br />デフォルト');
    print $table->td('詳細表示時<br />一覧項目');
    print $table->td('検索対象');
    print $table->td('ソート対象');
}

sub field_setting_check ($$) {
    my ( $this, $k ) = @_;
    if ( defined $main::data{ $k . '_Hidden' }
        && $main::data{ $k . '_Hidden' } )
    {
        $main::data{ $k . '_Default' }     = '';
        $main::data{ $k . '_detailTable' } = '';
    }
    return;
}

sub note ($) {
    my $this = shift;
    return
        '<div class="warning">画像等アップロードされたファイルはアクセス可能な位置に保存されます。「非公開」または「隠す」に設定しても直接アクセスすることは可能です。</div>';
}

######################################################################
# 設定項目
sub print_setting_items ($$$$;$) {
    my ( $this, $table, $field, $name, $id ) = @_;

    $field->{'cellWidth'} = '150'
        if !defined $field->{'cellWidth'} || $field->{'cellWidth'} eq '';
    $field->{'Open'} = '1'
        if !defined $field->{'Open'} || $field->{'Open'} eq '';

    print $table->td(
        $this->mk_select_menu(
            $name . 'cellWidth',
            $this->escape_except_html_entity(
                $field->{'name'} || 'このフィールド'
                )
                . ' のセル幅',
            '',
            $field->{'cellWidth'},
            '50'  => ' 50',
            '100' => '100',
            '150' => '150',
            '200' => '200',
            '250' => '250',
            '300' => '300',
            '350' => '350',
            '400' => '400'
        )
    );

    print $table->td(
        $this->mk_select_menu(
            $name . 'cellAlign',
            $this->escape_except_html_entity(
                $field->{'name'} || 'このフィールド'
                )
                . ' の水平方向位置揃え',
            '',
            $field->{'cellAlign'},
            'left'   => '左',
            'center' => '中央',
            'right'  => '右'
        )
    );

    print $table->td(
        $this->mk_select_menu(
            $name . 'cellValign',
            $this->escape_except_html_entity(
                $field->{'name'} || 'このフィールド'
                )
                . ' の垂直方向位置揃え',
            '',
            $field->{'cellValign'},
            'top'    => '上',
            'middle' => '中央',
            'bottom' => '下'
        )
    );

    if (   $field->{'class'} !~ /^user_/
        && $field->{'class'} ne 'xtm_serialno'
        && $field->{'class'} ne 'xtm_sticky' )
    {
        print $table->td(
                  '<input type="checkbox" name="'
                . $name
                . 'protected" value="1"'
                . ( ( $field->{'protected'} ) ? ' checked="checked"' : '' )
                . ' title="'
                . $this->escape_except_html_entity(
                $field->{'name'} || 'このフィールド'
                )
                . ' を保護フィールドにする" />'
        );
    }
    else {
        print $table->td('');
    }

    print $table->td(
        $this->mk_select_menu(
            $name . 'Open',
            $this->escape_except_html_entity(
                $field->{'name'} || 'このフィールド'
                )
                . ' の公開',
            '',
            $field->{'Open'},
            '1' => '一般公開',
            '0' => '非公開'
        )
    );

    if ( $default::type && $default::type eq 'order' ) {
        if (   $field->input_enabled
            && $field->{'class'} !~ /^(?:ug|xtm|yorder)_/
            && (  !$field->{'field_for'}{'mail'}
                || $field->{'field_for'}{'mail'} ne $field->{'id'} )
            )
        {
            print $table->td(
                      '<input type="checkbox" name="'
                    . $name
                    . 'skippable" value="1"'
                    . (
                    ( $field->{'skippable'} ) ? ' checked="checked"' : '' )
                    . ' title="'
                    . $this->escape_except_html_entity(
                    $field->{'name'} || 'このフィールド'
                    )
                    . ' をダウンロード販売時に省略する" />'
            );
        }
        else {
            print $table->td('');
        }
    }

    print $table->td(
              '<input type="checkbox" name="'
            . $name
            . 'Hidden" value="1"'
            . ( ( $field->{'Hidden'} ) ? ' checked="checked"' : '' )
            . ' onclick="cmonos.form.item(this.form).checkboxSync(this.checked,true,\''
            . $name
            . 'Default\',false,\''
            . $name
            . 'detailTable\',false)" title="'
            . $this->escape_except_html_entity(
            $field->{'name'} || 'このフィールド'
            )
            . ' を隠す" />'
    );

    print $table->td(
              '<input type="checkbox" name="'
            . $name
            . 'Default" value="1"'
            . ( ( $field->{'Default'} ) ? ' checked="checked"' : '' )
            . ' onclick="cmonos.form.item(this.form).checkboxSync(this.checked,true,\''
            . $name
            . 'Hidden\',false)" title="'
            . $this->escape_except_html_entity(
            $field->{'name'} || 'このフィールド'
            )
            . ' をデフォルトのテーブル表示項目にする" />'
    );
    if ( $field->format_for_detail_enabled ) {
        print $table->td(
                  '<input type="checkbox" name="'
                . $name
                . 'detailTable" value="1"'
                . ( ( $field->{'detailTable'} ) ? ' checked="checked"' : '' )
                . ' onclick="cmonos.form.item(this.form).checkboxSync(this.checked,true,\''
                . $name
                . 'Hidden\',false)" title="'
                . $this->escape_except_html_entity(
                $field->{'name'} || 'このフィールド'
                )
                . ' を詳細表示画面の一覧項目にする" />'
        );
    }
    else {
        print $table->td('');
    }
    if ( $field->search_form_enabled ) {
        print $table->td(
                  '<input type="checkbox" name="'
                . $name
                . 'searchEnabled" value="1"'
                . (
                ( $field->{'searchEnabled'} ) ? ' checked="checked"' : '' )
                . ' title="'
                . $this->escape_except_html_entity(
                $field->{'name'} || 'このフィールド'
                )
                . ' を検索対象にする" />'
        );
    }
    else {
        print $table->td('');
    }
    if ( $field->sort_enabled ) {
        print $table->td(
                  '<input type="checkbox" name="'
                . $name
                . 'sortEnabled" value="1"'
                . ( ( $field->{'sortEnabled'} ) ? ' checked="checked"' : '' )
                . ' title="'
                . $this->escape_except_html_entity(
                $field->{'name'} || 'このフィールド'
                )
                . ' をソート対象にする" />'
        );
    }
    else {
        print $table->td('');
    }
}

######################################################################
#サブ設定
sub print_sub_setting_items ($) {
    my $this = shift;
    return if !%field::obj;

    print
        '<fieldset><legend>特殊フィールド</legend><table class="field-table"><tbody>';

    print
        '<tr><td class="field-name"><label for="data:for_key">キーフィールド</label></td>';
    print '<td class="field"><select name="data:for_key" id="data:for_key">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->sort_enabled;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'key'}
                    && $field::for{'key'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_title">タイトルフィールド</label></td>';
    print
        '<td class="field"><select name="data:for_title" id="data:for_title">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->{'Open'}
            && !$field::obj{ $field::ORDER{$i} }->{'Hidden'}
            && $field::obj{ $field::ORDER{$i} }->sort_enabled
            && $field::obj{ $field::ORDER{$i} }->{'class'}
            =~ /^(?:text|addr_pref|time_date|time_span)/;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'title'}
                    && $field::for{'title'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_content">本文フィールド</label></td>';
    print
        '<td class="field"><select name="data:for_content" id="data:for_content">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->{'Open'}
            && !$field::obj{ $field::ORDER{$i} }->{'Hidden'}
            && $field::obj{ $field::ORDER{$i} }->{'class'} =~ /^text/;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'content'}
                    && $field::for{'content'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_name">名前フィールド</label></td>';
    print
        '<td class="field"><select name="data:for_name" id="data:for_name">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->{'class'} =~ /^text/
            && $field::obj{ $field::ORDER{$i} }->sort_enabled;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'name'}
                    && $field::for{'name'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '<ol class="checkbox">';
    print '<li><label><input type="checkbox" name="data:NAMECHECK" value="1"'
        . ( ($field::NAMECHECK) ? ' checked="checked"' : '' )
        . ' /> 名前を重複チェックデータベースに登録する（登録済みの名前と重複していないかチェックする）</label></li>';
    print
        '<li><label><input type="checkbox" name="data:NAMEREGISTER" value="1"'
        . ( ($field::NAMEREGISTER) ? ' checked="checked"' : '' )
        . ' /> 名前をユーザー名データベースに登録する（他の CGI で利用）</label></li>';
    print '</ol>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_mail">代表メールフィールド</label></td>';
    print
        '<td class="field"><select name="data:for_mail" id="data:for_mail">';
    print '<option value=""></option>'
        if !$default::type || $default::type ne 'order';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->{'e-mail'}
            && $field::obj{ $field::ORDER{$i} }->{'class'} =~ /email/;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'mail'}
                    && $field::for{'mail'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_media">メディアフィールド</label></td>';
    print
        '<td class="field"><select name="data:for_media" id="data:for_media">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && ( $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_media'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq
            'file_shipainter'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_epub' );
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'media'}
                    && $field::for{'media'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_media2">メディアフィールド(予備1)</label></td>';
    print
        '<td class="field"><select name="data:for_media2" id="data:for_media2">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && ( $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_media'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq
            'file_shipainter'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_epub' );
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'media2'}
                    && $field::for{'media2'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="data:for_media3">メディアフィールド(予備2)</label></td>';
    print
        '<td class="field"><select name="data:for_media3" id="data:for_media3">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && ( $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_media'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq
            'file_shipainter'
            || $field::obj{ $field::ORDER{$i} }->{'class'} eq 'file_epub' );
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'media3'}
                    && $field::for{'media3'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print
        '<tr class="tbody-end"><td class="field-name"><label for="data:for_coordinates">座標フィールド</label></td>';
    print
        '<td class="field"><select name="data:for_coordinates" id="data:for_coordinates" title="Googleマップにインデックスを表示する際座標とするフィールドを指定します。">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->{'class'} eq 'webapi_gmap';
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'coordinates'}
                    && $field::for{'coordinates'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    if ( $default::type && $default::type eq 'order' ) {
        print '</tbody><tbody>';

        print
            '<tr><td class="field-name"><label for="data:for_shippingzipcode">配送先郵便番号フィールド</label></td>';
        print
            '<td class="field"><select name="data:for_shippingzipcode" id="data:for_shippingzipcode">';
        print '<option value=""></option>';
        foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
            next
                if !ref $field::obj{ $field::ORDER{$i} }
                || $field::obj{ $field::ORDER{$i} }->{'class'} ne
                'addr_zipcode';
            print '<option value="'
                . $field::ORDER{$i} . '"'
                . (
                (   defined $field::for{'shippingzipcode'}
                        && $field::for{'shippingzipcode'} eq $field::ORDER{$i}
                ) ? ' selected="selected"' : ''
                )
                . '>'
                . $this->jfold( $field::obj{ $field::ORDER{$i} }->name,
                32, 'I', 1 )
                . '</option>';
        }
        print '</select>';
        print '</td></tr>';

        print
            '<tr class="tbody-end"><td class="field-name"><label for="data:for_shippingaddress">配送先住所フィールド</label></td>';
        print
            '<td class="field"><select name="data:for_shippingaddress" id="data:for_shippingaddress">';
        print '<option value=""></option>';
        foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
            next
                if !ref $field::obj{ $field::ORDER{$i} }
                || $field::obj{ $field::ORDER{$i} }->{'class'} ne 'addr_j';
            print '<option value="'
                . $field::ORDER{$i} . '"'
                . (
                (   defined $field::for{'shippingaddress'}
                        && $field::for{'shippingaddress'} eq $field::ORDER{$i}
                ) ? ' selected="selected"' : ''
                )
                . '>'
                . $this->jfold( $field::obj{ $field::ORDER{$i} }->name,
                32, 'I', 1 )
                . '</option>';
        }
        print '</select>';
        print '</td></tr>';
    }

    print '</tbody><tbody>';
    print
        '<tr><td class="field-name"><label for="data:for_sort">ソート項目のデフォルト</label></td>';
    print
        '<td class="field"><select name="data:for_sort" id="data:for_sort">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            unless ref $field::obj{ $field::ORDER{$i} }
            && $field::obj{ $field::ORDER{$i} }->sort_enabled
            && $field::obj{ $field::ORDER{$i} }->{'sortEnabled'};
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'sort'}
                    && $field::for{'sort'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';
    print
        '<tr class="tbody-end"><td class="field-name"><label for="data:SORTORDER">ソート方法のデフォルト</label></td>';
    print '<td class="field">';
    print $this->mk_select_menu(
        'data:SORTORDER', '', '', $field::SORTORDER,
        'ascend'  => '昇順',
        'descend' => '降順'
    );
    print '</td></tr>';

    print '</tbody><tbody>';

    print
        '<tr><td class="field-name"><label for="data:for_top">詳細表示ページのトップ項目</label></td>';
    print '<td class="field"><select name="data:for_top" id="data:for_top">';
    print '<option value=""></option>';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        next
            if !ref $field::obj{ $field::ORDER{$i} }
            || $field::obj{ $field::ORDER{$i} }->{'Hidden'}
            || !$field::obj{ $field::ORDER{$i} }->format_for_detail_enabled;
        print '<option value="'
            . $field::ORDER{$i} . '"'
            . (
            (   defined $field::for{'top'}
                    && $field::for{'top'} eq $field::ORDER{$i}
            ) ? ' selected="selected"' : ''
            )
            . '>'
            . $this->jfold( $field::obj{ $field::ORDER{$i} }->name, 32, 'I',
            1 )
            . '</option>';
    }
    print '</select>';
    print '</td></tr>';

    print '</tbody></table>';
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';

    {
        local ( %main::search, $main::and, $main::c, @main::c );
        $this->import_db( $default::preference_dir . '/page',
            \%setting::page, 1, 'index' )
            if !%setting::page || !ref $setting::page{'index'};
        require CMONOS::TableMaker::TMCommon;
        my $tm = {};
        bless $tm, 'TMCommon';

        my $search_form = $tm->aside_search_form(1);
        my $path        = $default::file . '/search_form';
        $this->write_file( $path . '.ssi', \$search_form, 1 );
        $this->mk_external_javascript(
            $path . '.ssi',
            $path . '.js',
            $search_form
        );
        print '<fieldset><legend>埋め込み検索フォーム</legend>';
        print
            '<p>下記をコピーしてテンプレートや外部 HTML ファイルに埋め込みます。</p><fieldset class="sample"><legend>埋め込みタグ(JavaScript)</legend>';
        my $target_uri = $this->http_my_domain
            . $this->path_change( $path . '.js', $ENV{'SCRIPT_NAME'}, 'abs' );
        print
            '<textarea cols="70" rows="3">&lt;script type=&quot;text/javascript&quot; language=&quot;JavaScript&quot; src=&quot;'
            . $target_uri
            . '&quot; class=&quot;sim-ssi&quot;&gt;&lt;/script&gt;</textarea>';
        print '</fieldset>';

        if ( defined $env::ssi && $env::ssi ) {
            print
                '<fieldset class="sample"><legend>埋め込みタグ(SSI)</legend>';
            print
                '<textarea cols="70" rows="3">&lt;!--#include virtual=&quot;'
                . $this->path_change( $path . '.ssi',
                $ENV{'SCRIPT_NAME'}, 'abs' )
                . '&quot;--&gt;</textarea>';
            print '</fieldset>';
        }
        print '</fieldset>';
    }
}

sub field_general_setting_check ($) {
    my $this = shift;
    if (( $default::type && $default::type eq 'order' )
        || (( !$default::type || $default::type ne 'order' )
            && (   $setting::general{'autoRegister'}
                || $setting::general{'oneEmailToOneRecord'}
                || $setting::general{'syncRegister'} )
        )
        )
    {
        if (!$main::data{'for_mail'}
            || ( @main::delete
                && grep( $_ eq $main::data{'for_mail'}, @main::delete ) )
            )
        {
            my @mail_fields = ();
            while ( my ( $k, $v ) = each %main::data ) {
                if ( $k && $v && $k =~ /^([0-9]+)_class$/ ) {
                    my $id = $1;
                    if ($v =~ /email/
                        && (   !@main::delete
                            || !grep( $_ eq $id, @main::delete ) )
                        )
                    {
                        push( @mail_fields, $id );
                    }
                }
            }
            my $reason
                = ( $default::type && $default::type eq 'order' )
                ? '購入情報を送信するために必要となります。'
                : ( $setting::general{'autoRegister'} )
                ? 'メールアドレス自動登録のために必要です。'
                : ( $setting::general{'oneEmailToOneRecord'} )
                ? '一つの登録メールアドレスに一つのレコードとするために必要です。'
                : '';
            if (@mail_fields) {
                if ( @mail_fields == 1 ) {
                    $main::data{'for_mail'} = $mail_fields[0];
                    push(
                        @{ $this->{'dialog'} },
                        '「'
                            . (
                            $main::data{ $mail_fields[0] . '_name' }
                                || 'メールアドレス'
                            )
                            . '」フィールドを「代表メールフィールド」に設定しました。'
                    );
                }
                else {
                    push(
                        @{ $this->{'dialog'} },
                        '「代表メールフィールド」を設定してください。'
                            . $reason
                    );
                }
            }
            else {
                push(
                    @{ $this->{'dialog'} },
                    '「メールアドレス」フィールドタイプを追加して「代表メールフィールド」を設定してください。'
                        . $reason
                );
            }
        }
        if ( $main::data{'for_mail'}
            && !$main::data{ $main::data{'for_mail'} . '_required' } )
        {
            $main::data{ $main::data{'for_mail'} . '_required' } = 1;
            push(
                @{ $this->{'dialog'} },
                '「'
                    . (
                    $main::data{ $main::data{'for_mail'} . '_name' }
                        || 'メールアドレス'
                    )
                    . '」フィールドを必須項目に設定しました。'
            );
        }
    }
    if ( !$default::type || $default::type ne 'order' ) {
        $main::data{'NAMECHECK'} = ''
            if !defined $main::data{'NAMECHECK'} || !$main::data{'NAMECHECK'};
        $main::data{'NAMEREGISTER'} = ''
            if !defined $main::data{'NAMEREGISTER'}
            || !$main::data{'NAMEREGISTER'};
    }
    return;
}

######################################################################
# データベース削除
sub delete_field ($@) {
    my ( $this, @delete ) = @_;
    my ( $category, $dir, $file, $delete_all );
    local ( *CATEGORY, *DIR );

    #フィールドが全部削除された場合
    $delete_all = 1 if keys %main::order == 0;

    #データベース削除
    opendir( CATEGORY, $default::db );
    $default::db =~ s!/$!!;
    foreach $category ( readdir(CATEGORY) ) {
        next unless $category !~ /^\./ && -d "$default::db/$category";
        opendir( DIR, "$default::db/$category" );
        foreach $file ( readdir(DIR) ) {
            next unless $file !~ /^\./ && -f "$default::db/$category/$file";
            if ( $delete_all || grep( $file =~ /^$_(\.|_|$)/, @delete ) > 0 )
            {
                unlink "$default::db/$category/$file";
            }
        }
        closedir(DIR);
    }
    closedir(CATEGORY);

    #添付ファイル削除
    opendir( CATEGORY, $default::file );
    $default::file =~ s!/$!!;
    foreach $category ( readdir(CATEGORY) ) {
        next unless $category !~ /^\./ && -d "$default::file/$category";
        opendir( DIR, "$default::file/$category" );
        foreach $dir ( readdir(DIR) ) {
            next unless $dir !~ /^\./ && -d "$default::file/$category/$dir";
            opendir( NO, "$default::file/$category/$dir" );
            foreach $file ( readdir(NO) ) {
                next
                    unless $file !~ /^\./
                    && -f "$default::file/$category/$dir/$file";
                if ( grep( $file =~ /^$_(\.|$)/, @delete ) > 0 ) {
                    unlink "$default::file/$category/$dir/$file";
                }
            }
            closedir(NO);
        }
        closedir(DIR);
    }
    closedir(CATEGORY);
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 't100', 'InputField' );

    #ツール登録
    $setting::jump_menu{'t100'} = '入力フィールド設定';
}

1;
