package PageObj;
require CMONOS::CGI_Lib::Tools::Page;
@ISA = qw(Page);
use strict;
use utf8;
######################################################################
# 共通部品モジュール
# PageObj.pm ver1.1.6
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2013-09-23
######################################################################
# 設定チェック項目
sub get_setting_items ($) {
    no warnings qw(once);
    my $this = shift;

    #チェックボックス
    push( @check::checkbox,
        'categoryname',       'checkboxForCatList',
        'linkedTitle',        'sameWindowDetailViewer',
        'categoryselector',   'searchbox',
        'sortmenu',           'download',
        'searchIconDisabled', 'vcard',
        'pswdIcon',           'sameWindowNewDataButton' );

    #数字チェック
    $check::num{'detailViewerWidth'} = '詳細ウィンドウサイズ(幅)';
    $check::num{'detailViewerHeight'}
        = '詳細ウィンドウサイズ(高さ)';
    $check::num{'pagelistmax'}    = 'ページリンク最大表示数';
    $check::num{'pagelistmargin'} = '前方ページリンク表示数';
    $check::num{'newIconLimit'}   = '新着アイコン対象期間';
    $check::num{'upIconLimit'}    = '更新アイコン対象期間';

    #URL書式チェック
    $check::url{'returnUrl'} = '戻る先URL';

    if ( $default::type && $default::type eq 'order' ) {
        push( @check::detag, 'emptyCartMsg' );
    }
}

######################################################################
# 設定環境
sub set_sub_preferences ($) {
    no warnings qw(once);
    my $this = shift;
    $preference::id   = '';                      # データキー
    $preference::name = '共通部品設定';    # 設定パネル名
    $preference::title
        = '各ページが共通して利用するボタン等について設定します。';
    require CMONOS::CGI_Lib::Deco;
    $this->{'icon_obj'} = Deco->new( $env::image_dir . '/tablemaker',
        $env::image_lib . '/tablemaker' );
    $setting::image_css = 'tablemaker-image.css';
}

######################################################################
# 設定パネル
sub print_preference_panel ($) {
    my $this = shift;
    print '<fieldset>';
    print '<legend>ページナビゲーション</legend>';
    print '<ol class="checkbox">';
    print '<li><label><input type="radio" name="data:pageinfo" value="1"'
        . ( ( $main::data{'pageinfo'} ) ? ' checked="checked"' : '' )
        . ' /> ページ位置情報のみ表示する</label></li>';
    print '<li><label><input type="radio" name="data:pageinfo" value="0"'
        . ( ( !$main::data{'pageinfo'} ) ? ' checked="checked"' : '' )
        . ' /> 各ページへのリンクを表示する</label>';
    print ' (<span class="additional-field">';
    print
        '<label for="data:pagelistmax">ページリンク最大表示数</label><input type="number" size="3" maxlength="3" name="data:pagelistmax" id="data:pagelistmax" value="'
        . $this->escape( $main::data{'pagelistmax'} )
        . '" /> (7以上)';
    print
        ' / <label for="data:pagelistmargin">前方ページリンク表示数</label><input type="number" size="3" maxlength="3" name="data:pagelistmargin" id="data:pagelistmargin" value="'
        . $this->escape( $main::data{'pagelistmargin'} ) . '" />';
    print '</span>)</li>';
    print '</ol>';
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>カテゴリー名</legend>';
    print
        '<label for="data:categoryname"><input type="checkbox" name="data:categoryname" id="data:categoryname"  value="1"'
        . (
        (   defined $main::data{'categoryname'} && $main::data{'categoryname'}
        ) ? ' checked="checked"' : ''
        ) . ' /> タイトルにカテゴリー名を表示する</label>';
    print '</fieldset>';

    print '<fieldset>';
    print
        '<legend><label for="data:defaultView">表示形式のデフォルト <span class="note">(検索やカテゴリーリストで参照されます)</span></label></legend>';
    print $this->mk_select_menu(
        'data:defaultView', '', '', $main::data{'defaultView'},
        'TableViewer' => 'テーブル表示',
        'GMapViewer'  => 'Googleマップ表示',
        'TileViewer'  => 'タイル表示'
    );
    print '</fieldset>';

    print '<fieldset>';
    print
        '<legend><label for="data:checkboxf">埋め込み検索フォームの「カテゴリー」の表示</label></legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:checkboxForCatList" id="data:checkboxForCatList" value="1"'
        . (
        ( $main::data{'checkboxForCatList'} ) ? ' checked="checked"' : '' )
        . ' /> 選択メニューではなくチェックボックスで表示する</label></li>';
    print '</ol>';
    print '</fieldset>';

    print
        '<fieldset><legend><label><input type="checkbox" name="data:linkedTitle" id="data:linkedTitle"  value="1"'
        . (
        ( defined $main::data{'linkedTitle'} && $main::data{'linkedTitle'} )
        ? ' checked="checked"'
        : '' )
        . ' onclick="cmonos.element.item(\'detail-viewer-settings\').appear(this.checked)" /> タイトルから詳細画面を開く</label></legend>';
    print '<div id="detail-viewer-settings">';

    if ( $default::detail_variation >= 2 ) {
        print
            '<fieldset><legend><label for="data:detailViewerNo">詳細表示ウィンドウ・ページレイアウト</label></legend>';
        print '<select name="data:detailViewerNo" id="data:detailViewerNo">';
        print '<option value="">詳細表示ページ</option>';
        foreach ( 2 .. $default::detail_variation ) {
            print '<option value="'
                . $_ . '"'
                . (
                (   defined $main::data{'detailViewerNo'}
                        && $main::data{'detailViewerNo'} eq $_
                ) ? ' selected="selected"' : ''
                )
                . '>詳細表示ページ'
                . $_
                . '</option>';
        }
        print '</select></fieldset>';
    }
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:sameWindowDetailViewer" id="data:sameWindowDetailViewer" value="1"'
        . (
        (   defined $main::data{'sameWindowDetailViewer'}
                && $main::data{'sameWindowDetailViewer'}
        ) ? ' checked="checked"' : ''
        )
        . ' onclick="cmonos.element.item(\'windowsize\').disappear(this.checked)" /> 別ウィンドウを開かず同じウィンドウに表示する</label></li>';
    print '</ol>';
    print '<fieldset id="windowsize">';
    print
        '<legend>詳細ウィンドウサイズ(同じウィンドウに表示する場合無効)</legend>';
    print
        '<label>幅 <input type="text" name="data:detailViewerWidth" id="data:detailViewerWidth" value="'
        . $this->escape( $main::data{'detailViewerWidth'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span></label> ';
    print
        '&times; <label>高さ <input type="text" name="data:detailViewerHeight" id="data:detailViewerHeight" value="'
        . $this->escape( $main::data{'detailViewerHeight'} )
        . '" size="7" maxlength="7" /> <span class="unit">ピクセル</span></label> ';
    print '</fieldset></div>';
    print $this->javascript(
        'cmonos.element.item(\'windowsize\').disappear(true)')
        if $main::data{'sameWindowDetailViewer'};
    print $this->javascript(
        'cmonos.element.item(\'detail-viewer-settings\').disappear(true)')
        if !$main::data{'linkedTitle'};
    print '</fieldset>';

    print '<fieldset><legend>再表示ボタン設定</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label for="data:categoryselector"><input type="checkbox" name="data:categoryselector" id="data:categoryselector"  value="1"'
        . (
        (   defined $main::data{'categoryselector'}
                && $main::data{'categoryselector'}
        ) ? ' checked="checked"' : ''
        ) . ' /> カテゴリーメニューを表示する</label></li>';
    print
        '<li><label for="data:searchbox"><input type="checkbox" name="data:searchbox" id="data:searchbox"  value="1"'
        . ( ( defined $main::data{'searchbox'} && $main::data{'searchbox'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 検索ボックスを表示する</label></li>';
    print
        '<li><label for="data:sortmenu"><input type="checkbox" name="data:sortmenu" id="data:sortmenu"  value="1"'
        . ( ( defined $main::data{'sortmenu'} && $main::data{'sortmenu'} )
        ? ' checked="checked"'
        : '' )
        . ' /> 再ソートメニューを表示する</label></li>';
    print '</ol>';
    $this->print_button_menu( 'テーブル表示ボタン',    'tableview' );
    $this->print_button_menu( 'Googleマップ表示ボタン', 'gmapview' );
    $this->print_button_menu( 'タイル表示ボタン',       'tileview' );
    $this->print_button_menu( 'お気に入り表示ボタン',
        'tmfavorites' );
    print '</fieldset>';

    print '<fieldset><legend>機能ボタン設定</legend>';
    $this->print_button_menu( 'メール配信フォームボタン',
        'tmmailer' );
    $this->print_button_menu( '検索条件リセットボタン',
        'tmreset' );
    $this->print_button_menu(
        '検索ボタン',
        'tmsearch',
        '<label for="data:searchIconDisabled"><input type="checkbox" name="data:searchIconDisabled" id="data:searchIconDisabled"  value="1"'
            . (
            (   defined $main::data{'searchIconDisabled'}
                    && $main::data{'searchIconDisabled'}
            ) ? ' checked="checked"' : ''
            )
            . ' /> 検索ボタンを表示<strong>しない</strong></label>'
    );
    print '</fieldset>';

    print '<fieldset><legend>編集ボタン設定</legend>';
    $this->print_button_menu( 'データ修正ボタン', 'tmlist' );
    $this->print_button_menu(
        '新規入力ボタン',
        'tminput',
        '<label for="data:sameWindowNewDataButton"><input type="checkbox" name="data:sameWindowNewDataButton" id="data:sameWindowNewDataButton"  value="1"'
            . (
            (   defined $main::data{'sameWindowNewDataButton'}
                    && $main::data{'sameWindowNewDataButton'}
            ) ? ' checked="checked"' : ''
            )
            . ' /> 入力画面を同じウィンドウで開く</label>'
    );
    print '<fieldset>';
    print '<legend>ボタン名</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name">';
    print
        '<label for="data:reviseButton">データ修正ボタン名</label></td>';
    print
        '<td class="field"><input type="text" name="data:reviseButton" id="data:reviseButton" value="'
        . $this->escape( $main::data{'reviseButton'} )
        . '" size="40" />';
    print '</td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:newDataButton">新規入力ボタン名</label></td>';
    print
        '<td class="field"><input type="text" name="data:newDataButton" id="data:newDataButton" value="'
        . $this->escape( $main::data{'newDataButton'} )
        . '" size="40" />';
    print '</td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset><legend>ツールボタン設定</legend>';
    $this->print_button_menu( '環境設定を開く ボタン', 'tmtool' );
    $this->print_button_menu(
        'ログインボタン',
        'tmlogin',
        '<label for="data:pswdIcon"><input type="checkbox" name="data:pswdIcon" id="data:pswdIcon"  value="1"'
            . (
            ( defined $main::data{'pswdIcon'} && $main::data{'pswdIcon'} )
            ? ' checked="checked"'
            : ''
            )
            . ' /> ログインボタンを表示する</label>'
    );
    print '</fieldset>';

    print '<fieldset>';
    print '<legend>データダウンロードボタン</legend>';
    print '<ol class="checkbox">';
    print
        '<li><label><input type="checkbox" name="data:download" id="data:download"  value="1"'
        . ( ( defined $main::data{'download'} && $main::data{'download'} )
        ? ' checked="checked"'
        : '' )
        . ' /> カンマ区切りテキスト(CSV)形式またはタブ区切りテキストでデータをダウンロードするボタンを表示する</label></li>';
    print '<li><label><input type="checkbox" name="data:vcard" value="1"'
        . ( ( $main::data{'vcard'} ) ? ' checked="checked"' : '' )
        . ' /> 上記に加え、vCard形式の書き出しに対応する</label></li>';
    print '</ol>';
    print '</fieldset>';

    $this->print_page_button_setting;
    $this->print_close_button_setting;
    $this->print_sc_button_setting;
    if ( $default::type && $default::type eq 'order' ) {
        print '<fieldset>';
        print
            '<legend><label for="data:emptyCartMsg">カートに何もないときの警告メッセージ</label></legend>';
        print
            '<input type="text" name="data:emptyCartMsg" id="data:emptyCartMsg" value="'
            . $this->escape( $main::data{'emptyCartMsg'} )
            . '" size="72" />';
        print '</fieldset>';
    }

    print '<fieldset><legend>新着アイコン</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name">';
    print '<span class="datatitle">対象</span></td>';
    print
        '<td class="field"><label><input type="text" name="data:newIconLimit" id="data:newIconLimit" value="'
        . $this->escape( $main::data{'newIconLimit'} )
        . '" size="3" /> 日以内に作成されたレコード</label></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:newIconAlt">代替文字列</label></td>';
    print
        '<td class="field"><input type="text" name="data:newIconAlt" id="data:newIconAlt" value="'
        . $this->escape( $main::data{'newIconAlt'} )
        . '" size="40" /></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '<fieldset><legend>アイコン</legend>';
    print $cgi::Deco->image_menu( 'data:newIcon', 'new', 120,
        $main::data{'newIcon'} );
    print '</fieldset>';
    print '</fieldset>';

    print '<fieldset><legend>更新アイコン</legend>';
    print '<table class="field-table">';
    print '<tbody>';
    print '<tr>';
    print '<td class="field-name">';
    print '<span class="datatitle">対象</span></td>';
    print
        '<td class="field"><label><input type="text" name="data:upIconLimit" id="data:upIconLimit" value="'
        . $this->escape( $main::data{'upIconLimit'} )
        . '" size="3" /> 日以内に更新されたレコード</label><div class="warning">「更新アイコン」よりも「新着アイコン」が優先されます。</div></td>';
    print '</tr>';
    print '<tr>';
    print
        '<td class="field-name"><label for="data:upIconAlt">代替文字列</label></td>';
    print
        '<td class="field"><input type="text" name="data:upIconAlt" id="data:upIconAlt" value="'
        . $this->escape( $main::data{'upIconAlt'} )
        . '" size="40" /></td>';
    print '</tr>';
    print '</tbody>';
    print '</table>';
    print '<fieldset><legend>アイコン</legend>';
    print $cgi::Deco->image_menu( 'data:upIcon', 'new', 120,
        $main::data{'upIcon'} );
    print '</fieldset>';
    print '</fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="保存" class="save" /></fieldset>';
}

sub print_button_menu ($$$;$) {
    my ( $this, $title, $name, $additional_field ) = @_;
    print '<fieldset><legend>' . $title . '</legend>';
    print $this->{'icon_obj'}
        ->button_menu( 'data:' . $name, $name, 120, $main::data{$name} );
    print $additional_field if $additional_field;
    print '</fieldset>';
}

######################################################################
# 設定チェック
sub is_valid_setting ($) {
    my $this = shift;
    $main::data{'pagelistmax'} = 7
        if $main::data{'pagelistmax'} && $main::data{'pagelistmax'} < 7;
    return;
}

######################################################################
# 登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 't0zz', 'PageObj' );

    #ツール登録
    $setting::sub_jump_menu{'page'}{'t0zz'} = '共通部品設定';
    $setting::selected_jump_menu{'t0zz'} = 't000'
        ; #サブメニューがあるときメインメニューのデフォルトキー

    #画像
    $setting::deco_dir{'new'}  = '新着アイコン';
    $setting::deco_type{'new'} = 'image';

    $setting::deco_dir{'return'}  = '戻る ボタン';
    $setting::deco_type{'return'} = 'button';
    $setting::deco_dir{'next'}    = '「次」ボタン';
    $setting::deco_type{'next'}   = 'button';
    $setting::deco_dir{'prev'}    = '「前」ボタン';
    $setting::deco_type{'prev'}   = 'button';
    if ( defined $env::checkout && $env::checkout ) {
        $setting::deco_dir{'check'}  = 'カートを見る ボタン';
        $setting::deco_type{'check'} = 'button';
    }
}

1;
