package RecordSweeper;
require CMONOS::TableMaker::TMCommon;
@ISA = qw(TMCommon);
use strict;
use utf8;
######################################################################
# レコード整理モジュール
# RecordSweeper.pm ver1.0.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2012-01-27
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( $error, @folder, %year );

    #カテゴリー設定読み込み
    $this->import_db( $default::preference_dir . '/category',
        \%setting::category, 1 )
        if !%setting::category;

    @main::c = push( @main::c, $main::c ) if !@main::c && defined $main::c;
    undef $main::c;

    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';

    #一括削除
    if (   $main::delete
        && @main::c
        && !$this->is_nan($main::mday)
        && !$this->is_nan($main::mon)
        && !$this->is_nan($main::year)
        && !$this->is_impossible_mday( $main::year, $main::mon, $main::mday )
        )
    {
        my ( $limit, $c, $gcount, $deleted_total );
        $gcount = $deleted_total = 0;

        #必要なモジュール
        use Time::Local;
        $limit
            = timelocal( 0, 0, 0, $main::mday, $main::mon - 1, $main::year );
        foreach $c (@main::c) {
            my ( %expire, %pending, %mtime, %delete );
            next if !$c || $this->is_nan($c) || !-d "$default::db/$c";
            tie( %expire, $env::dbm_file, "$default::db/$c/expire", O_RDONLY,
                $env::dbm_mode );
            if ( ref tied %expire ) {
                my ( $n, $t, $time );
                $time = time;
                while ( ( $n, $t ) = each %expire ) {
                    utf8::decode($n) if !utf8::is_utf8($n);
                    utf8::decode($t) if !utf8::is_utf8($t);
                    $delete{$n} = 1 if $t < $time;
                }
            }
            untie %expire;
            tie( %mtime, $env::dbm_file, "$default::db/$c/mtime", O_RDONLY,
                $env::dbm_mode );
            if ( ref tied %mtime ) {
                my ( $n, $t, $time );
                $time = time;
                while ( ( $n, $t ) = each %mtime ) {
                    utf8::decode($n) if !utf8::is_utf8($n);
                    utf8::decode($t) if !utf8::is_utf8($t);
                    $delete{$n} = 1 if $t < $time;
                }
            }
            untie %mtime;
            if (%delete) {
                my %pending;
                my $nocount = 0;
                my $deleted = 0;
                tie( %pending, $env::dbm_file, "$default::db/$c/pending",
                    O_RDONLY, $env::dbm_mode );
                foreach my $n ( keys %delete ) {
                    $nocount++
                        if ref tied %pending
                        && $pending{ $this->encode_utf8($n) }
                        && $this->decode_utf8(
                        $pending{ $this->encode_utf8($n) } ) =~ /^1/;
                }
                untie %pending;
                foreach my $n ( keys %delete ) {
                    $this->delete_record( $c, $n );
                    $deleted_total++;
                    $deleted++;
                }
                my (%category);
                tie(%category, $env::dbm_file,
                    $default::preference_dir . '/category',
                    O_RDWR | O_CREAT,
                    $env::dbm_mode
                );
                if ( ref tied %category ) {
                    $category{ $this->encode_utf8( $c . '_total' ) }
                        -= $deleted;
                    $category{ $this->encode_utf8( $c . '_count' ) }
                        -= ( $deleted - $nocount );
                    $setting::category{$c}{'total'}
                        = $this->decode_utf8(
                        $category{ $this->encode_utf8( $c . '_total' ) } )
                        || 0;
                    $setting::category{$c}{'count'}
                        = $this->decode_utf8(
                        $category{ $this->encode_utf8( $c . '_count' ) } )
                        || 0;
                }
                untie %category;
            }
            $gcount++;
        }
        push(
            @{ $this->{'dialog'} },
            $gcount
                . ' カテゴリーから合わせて '
                . $deleted_total
                . ' レコードを削除しました。'
        );
    }
    elsif ($main::delete) {
        push(
            @{ $this->{'dialog'} },
            (@main::c)
            ? '日付の指定に問題があります。'
            : 'カテゴリーを指定してください。'
        );
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print
        '<h1 title="レコードの一括削除を行います。">レコードの整理</h1>';
    my ( $mday, $mon, $year ) = ( localtime(time) )[ 3 .. 5 ];
    $year += 1900;
    $mon++;
    if ( !defined $main::year || !$main::year ) {
        $main::year = $year - 3;
        $main::mon  = $mon;
    }

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );

    #カテゴリー選択
    print $this->category_selector(
        '',
        '<fieldset><legend><label for="c">レコードを整理するカテゴリー</label></legend>',
        '</fieldset>'
    );

    print '<fieldset class="delete">';
    print '<select name="by">';
    print '<option value="ctime">作成時</option>';
    print '<option value="mtime"';
    print( ( defined $main::by && $main::by eq 'mtime' )
        ? ' selected="selected"'
        : '' );
    print '>更新時</option>';
    print '</select> が ';

    print '<label><select name="year">';
    foreach ( ( $year - 10 ) .. $year ) {
        print '<option value="', $_, '"';
        print( ( $main::year && $main::year == $_ )
            ? ' selected="selected"'
            : '' );
        print '>', $_, '</option>';
    }
    print '</select>年</label> ';

    print '<label><select name="mon">';
    foreach ( 1 .. 12 ) {
        print '<option value="', $_, '"';
        print( ( $main::mon && $main::mon == $_ )
            ? ' selected="selected"'
            : '' );
        print '>', $_, '</option>';
    }
    print '</select>月</label> ';

    print '<label><select name="mday">';
    foreach ( 1 .. 31 ) {
        print '<option value="', $_, '"';
        print( ( $main::mday && $main::mday == $_ )
            ? ' selected="selected"'
            : '' );
        print '>', $_, '</option>';
    }
    print '</select>日</label> ';

    print
        ' より古いレコードを <input type="submit" name="delete=1" value="一括削除する" onclick="return confirm(\'この操作は取り消せません。本当に削除しますか？\');" />';
    print '</fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);
    if ( !$default::subdivided
        || ( $default::subdivided && defined $main::c && $main::c ne '' ) )
    {

        #クラス登録
        Common::class_register( 't303', 'RecordSweeper' );

        #メニュー登録
        $setting::jump_menu{'t303'} = 'レコードの整理';

    }
}

1;
