package TableExport;
require CMONOS::TableMaker::Tools::TableImport;
@ISA = qw(TableImport);
use strict;
use utf8;
######################################################################
# テキスト書き出しモジュール
# TableExport.pm ver1.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    $this->read_setting;

    #書き出し
    if ( defined $main::a && $main::a eq 'w' ) {
        no warnings qw(untie);
        my ( $export, %db, %name, $dir );
        $main::c = '0' if !$main::c;

        #カテゴリーの新規作成
        $this->mk_category($main::c)
            if ( !defined $setting::category{$main::c}{'name'}
            || $setting::category{$main::c}{'name'} eq '' )
            && ( !defined $setting::category{$main::c}{'ORDER'}
            || $setting::category{$main::c}{'ORDER'} eq '' );

        #テキストをデータベースに変換
        require CMONOS::CGI_Lib::Table;
        $export = Table->new;
        use Fcntl qw(:DEFAULT);
        eval 'use ' . $env::dbm_file . ';';

        #データベースに接続
        $dir = "$default::db/$main::c/";
        my ( $id, $field );
        while ( ( $id, $field ) = each %field::obj ) {
            next unless $id ne '' && ref $field;
            $name{$id} = $field::obj{$id}->db;
            unless ( !$name{$id} || ref tied %{ $db{ $name{$id} } } ) {
                tie(%{ $db{ $name{$id} } },
                    $env::dbm_file, $dir . $name{$id},
                    O_RDONLY, $env::dbm_mode
                );
            }
        }
        unless ( ref tied %{ $db{'mtime'} } ) {
            tie(%{ $db{'mtime'} }, $env::dbm_file, $dir . 'mtime',
                O_RDONLY,          $env::dbm_mode
            );
        }

        #データベース書き出し
        $export->set_encoding();
        binmode STDOUT;
        if ($main::for_excel) {
            $this->print_http_header('application/vnd.ms-excel');
        }
        elsif ( $main::type eq 'tsv' ) {

            #$this->print_http_header('text/tab-separated-values');
            $this->print_http_header('application/tab-separated-values');
        }
        else {
            #$this->print_http_header('text/comma-separated-values');
            $this->print_http_header('application/csv');
        }
        while ( my ($no) = each %{ $db{'mtime'} } ) {
            utf8::decode($no) if !utf8::is_utf8($no);
            my (@data);
            foreach ( sort { $a <=> $b } keys %field::ORDER ) {
                my $db = $name{ $field::ORDER{$_} };
                push( @data,
                    ( ( $db && defined $db{$db}{$no} ) ? $db{$db}{$no} : '' )
                );
            }
            print $export->array_to_sv( $main::type, @data );
            undef @data;
        }

        #データベースの接続を解除
        while ( my ( $k, $v ) = each %db ) {
            if ( ref tied %{$v} ) {
                untie %{$v};
            }
        }

        #終了
        exit;
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>ファイル書き出し</h1>';
    my $filename = $default::id || 'exported';
    $filename =~ s/^_//;
    $filename .= '_data';
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'path'     => '/.dl/' . $filename . '.csv',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="w" />';

    print
        '<fieldset><table class="field-table"><tbody><tr><td class="field-name"><label for="c">カテゴリー選択</label></td><td class="field">';
    $this->print_category_selector();
    print
        '</td></tr><tr><td class="field-name"><label for="type">ファイル形式</label></td><td class="field">';
    print
        '<select name="type" onchange="cmonos.form.item(this.form).field(this).appendDownloadPathByValue({\'csv\':\''
        . $filename
        . '.csv\',\'tsv\':\''
        . $filename
        . '.tsv\'})">';
    print
        '<option value="csv">カンマ区切りテキスト（Excel形式）</option>';
    print
        '<option value="tsv">タブ区切りテキスト（FileMaker形式）</option>';
    print '</select>';
    print ' <label><input type="checkbox" name="for_excel" value="1"'
        . ( ($main::for_excel) ? ' checked="checked"' : '' )
        . ' /> Excel ファイルとして書き出す</label>';
    print '</td></tr>';

    print '</tbody></table></fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="書き出し" /></fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');

}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 't300', 'TableExport' );

    #メニュー登録
    $setting::jump_menu{'t300'} = '書き出し';
}

1;
