package TableImport;
require CMONOS::TableMaker::TMCommon;
@ISA = qw(TMCommon);
use strict;
use utf8;
use File::Path;
use Fcntl qw(:DEFAULT);
eval 'use ' . $env::dbm_file . ';';
no warnings qw(untie);
######################################################################
# テキスト読み込みモジュール
# TableImport.pm ver1.4.1
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2015-03-20
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    $this->read_setting;
    my $file_uploaded = 0;
    my $temp_dir
        = $env::clipboard . '/temp' . ( $default::id || '_tablemaker' );
    if ( -d $temp_dir ) {
        if ( -f $temp_dir . '/file' ) {
            $file_uploaded = 1;
        }
        else {
            rmtree $temp_dir;
        }
    }

    if ($file_uploaded) {
        if ($main::cancel) {
            rmtree $temp_dir;
            $file_uploaded = 0;
        }

    }
    elsif ( defined $main::file && $main::file ne '' ) {
        $main::type
            = ( $main::file =~ /\.csv$/i ) ? 'csv'
            : ( $main::file =~ /\.tsv$/i ) ? 'tsv'
            : ( defined $main::type )      ? $main::type
            :                                'csv';

        #大きすぎるファイル
        if ( $cgi::input->get_posted_file_size_from_param('file') > 8000000 )
        {
            push(
                @{ $this->{'dialog'} },
                'ファイルの読み込みに失敗しました。ファイルサイズが大きすぎます。'
            );

        }
        else {
            $this->set_dir($temp_dir);
            $this->write_file( $temp_dir . '/file',
                $cgi::input->get_posted_file_data('file'), 1 );
            if ( -s $temp_dir . '/file' ) {
                $file_uploaded = 1;
                $this->write_file( $temp_dir . '/type', $main::type, 1 );
            }
            else {
                rmtree $temp_dir;
            }
        }
        $cgi::input->clear_clipboard;
    }

    if ($file_uploaded) {
        my $is_valid = 1;
        if (   $main::check
            && !$main::overwrite
            && defined $main::c
            && $main::c !~ /[^0-9]/
            && ref $field::obj{$main::check}
            && (   !$field::obj{$main::check}->sort_enabled
                || !$field::obj{$main::check}->{'sortEnabled'} )
            )
        {
            push(
                @{ $this->{'dialog'} },
                'このフィールドは照合できません。他のフィールドを指定してください。'
            );
            $is_valid = 0;
        }
        require CMONOS::CGI_Lib::Table;
        if (   $is_valid
            && defined $main::c
            && $main::c ne ''
            && %main::collate )
        {

            foreach my $i ( keys %main::collate ) {
                $this->scalar_to_array( \$main::collate{$i} );
            }

            #カテゴリーの新規作成
            if ( $main::c =~ /[^0-9]/ || !ref $setting::category{$main::c} ) {
                $main::c = $this->new_number( \%setting::category, 1 );
                $this->mk_category($main::c);
                $main::check = '';
            }

            #上書きする場合
            if ($main::overwrite) {
                $this->delete_category($main::c);
                $main::check = '';
            }

            #テキストをデータベースに変換
            my $import = Table->new;
            my $data   = $this->read_file( $temp_dir . '/file' );
            my $type   = $this->read_file( $temp_dir . '/type', 1 );
            $data =~ s/[\x0A\x0D]+\Z//;
            $import->sv_to_table( \$data, 'no', $type, $this );
            push(
                @{ $this->{'dialog'} },
                'ファイルを読み込みました。結果を確認してください。'
            );
            my %general;
            eval 'use ' . $env::dbm_file . ';';
            tie(%general, $env::dbm_file,
                $default::preference_dir . '/general',
                O_RDWR | O_CREAT,
                $env::dbm_mode
            );

            if ( ref tied %general ) {
                $general{'importSettings'}
                    = $this->set_query( 'url', [ 'field', 'collate' ], 1 );
            }
            untie %general;
            rmtree $temp_dir;

            #読み込み設定
        }
        else {
            $this->url_decode( $setting::general{'importSettings'}, 'main' )
                if $setting::general{'importSettings'};
            my @sample_row = $this->get_first_row(
                $this->read_file( $temp_dir . '/file' ),
                $this->read_file( $temp_dir . '/type', 1 )
            );

            #ページヘッダ
            $this->jump_menu('tools');
            $this->print_page_header('tools');

            print '<h1>ファイル読み込み</h1>';

            print $this->form_start(
                'key'   => 'tools',
                'name'  => 'input',
                'multi' => 1,
                'onsubmit' =>
                    'return cmonos.form.item(this).isNotSubmitted()',
                'required' => \@setting::form
            );

            print
                '<fieldset><legend><label for="c">読み込み先カテゴリー</label></legend>';
            $this->print_category_selector(1);
            print
                ' <label for="option"><input type="checkbox" name="overwrite" id="overwrite" value="1" /> カテゴリーを一度消去し読み込みデータに差し替える</label></fieldset>';

            print '<fieldset><legend>照合フィールド</legend>';
            print '<select name="check">';
            print '<option value="">&lt;照合しない&gt;</option>';
            print '<option value=""></option>';
            foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
                my $id = $field::ORDER{$i};
                if (   $field::obj{$id}->sort_enabled
                    && $field::obj{$id}->{'sortEnabled'} )
                {
                    print '<option value="'
                        . $id . '"'
                        . (
                        (   ( !$main::check && $id == $field::for{'key'} )
                                || ( $main::check && $id eq $main::check )
                        ) ? ' selected="selected"' : ''
                        )
                        . '>'
                        . $this->cut_string( $field::obj{$id}->name, 32,
                        'no' )
                        . '</option>'
                        if ref $field::obj{$id};
                }
            }
            print '</select>';
            print '</fieldset>';

            print '<fieldset><legend>読み込み設定</legend>';
            my $table = Table->new;
            $table->set(
                'tableclass' => 'tooltable',
                'trclass'    => 'odd,even',
                'class0'     => 'odd,even',
                'style0' =>
                    'text-align:center,text-align:left,text-align:center',
                'thenabled' => 'true'
            );
            print $table->table;

            #ヘッダ
            print $table->tr;
            print $table->td('列番号');
            print $table->td('列の内容');
            print $table->td('読み込み先フィールド');
            print $table->_tr;
            my $no = 1;
            foreach my $i (@sample_row) {
                my $id = $field::ORDER{$no} || $no;
                print $table->tr;
                print $table->td($no);
                print $table->td( $this->cut_string( $i, 32, 'no' ) );
                print $table->td(
                    $this->import_field_menu(
                        $no,
                        (     ( defined $main::collate{$no} )
                            ? ( ref $main::collate{$no} )
                                    ? $main::collate{$no}
                                    : [ $main::collate{$no} ]
                            : [$id]
                        )
                    )
                );
                print $table->_tr;
                $no++;
            }
            print $table->_table;
            print
                '<div class="warning">レコード数が上限に達すると照合ができなくなります。</div></fieldset>';

            print
                '<fieldset class="submit"><label for="option"><input type="checkbox" name="option" id="option" value="1" /> 一行目をフィールド名として読み込む（フィールド定義未設定時）</label><input type="submit" value="読み込み" /></fieldset>';

            print
                '<fieldset class="delete"><input type="submit" name="cancel=1" value="アップロードしたファイルを削除して読み込みを中止する" /></fieldset>';
            print '</form>';

            #ページフッタ
            $this->print_page_footer('tools');
            return;
        }
    }
    elsif ($main::type) {
        push(
            @{ $this->{'dialog'} },
            'ファイルの読み込みに失敗しました。ファイルがアップロードされていません。'
        );
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>ファイルのアップロード</h1>';

    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'multi'    => 1,
        'onsubmit' => 'return cmonos.form.item(this).isNotSubmitted()',
        'required' => \@setting::form
    );

    print '<fieldset><table class="field-table"><tbody>';

    print
        '<tr><td class="field-name"><label for="file">ファイル選択</label></td>';
    print '<td class="field"><input type="file" name="file" id="file" />';
    print '</td></tr>';

    print
        '<tr><td class="field-name"><label for="type">ファイル形式</label></td>';
    print '<td class="field"><select name="type">';
    print
        '<option value="csv">カンマ区切りテキスト（Excel形式）</option>';
    print
        '<option value="tsv">タブ区切りテキスト（FileMaker形式）</option>';
    print '</select>';
    print '</td></tr>';

    print '</tbody></table></fieldset>';

    print
        '<fieldset class="submit"><input type="submit" value="このファイルをアップロードする" /></fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');

}

######################################################################
# 最初の行を取り出す
sub get_first_row ($$$) {
    my ( $this, $data, $type ) = @_;
    my @data = ();
    my $pat  = '';
    $data =~ s/[\x0D\x0A]+$//;
    $type = 'tsv' if ( !defined $type || $type eq '' ) && $data =~ /\x09/;

    if ( $type && $type eq 'tsv' ) {
        $data =~ s/\x20{3}/\x09/g if $data !~ /\x09/;
        $pat = '([^\x09\x0D\x0A]*)(\x09|\x0D\x0A|\x0D|\x0A|\Z)';
    }
    else {
        $pat = '([^,\x0D\x0A]*)(,|\x0D\x0A|\x0D|\x0A|\Z)';
    }

    my $count = 0;
    my $temp  = '';
    while ( $data =~ /\G($pat)/gs ) {
        next if $1 eq '';
        my ( $v, $s ) = ( $2, $3 );
        if ( $type && $type ne 'tsv' ) {
            $count += ( $v =~ tr/"/"/ )
                if ( defined $v && $v =~ /^"/ )
                || ( defined $temp && $temp =~ /^"/ );
            if ( $count && $count % 2 > 0 ) {
                $temp .= $v . $s;
                next;
            }
            else {
                $v = $temp . $v if defined $temp;
                if ( $v =~ s/^"// ) {
                    $v =~ s/"$//;
                    $v =~ s/""/"/g;
                }
                undef $temp;
                $count = 0;
            }
        }
        else {
            $v =~ s/\x0B/\x0A/gs;
            $v =~ s/\&#09;/\x09/gs;
        }
        $v =~ s/\x0D\x0A/\x0A/gs;
        $v =~ s/\x0D/\x0A/gs;
        push( @data, $v );
        if ( $s eq '' || $s =~ /^(\x0D\x0A|\x0D|\x0A)$/ ) {
            last;
        }
    }
    return @data;
}

sub import_field_menu ($$$$) {
    my ( $this, $no, $id ) = @_;
    $id = ( defined $id ) ? [$id] : [] if !ref $id;
    my $selector
        = '<select name="collate:'
        . $no
        . '" id="collate:'
        . $no
        . '" multiple="multiple" size="8">';
    foreach my $i ( sort { $a <=> $b } keys %field::ORDER ) {
        my $fid = $field::ORDER{$i};
        $selector
            .= '<option value="'
            . $fid . '"'
            . ( ( ref $id && grep( $_ eq $fid, @$id ) )
            ? ' selected="selected"'
            : '' )
            . '>'
            . $this->cut_string( $field::obj{$fid}->name, 32, 'no' )
            . '</option>'
            if ref $field::obj{$fid};
    }
    $selector
        .= '<option value=""></option><option value="new"'
        . ( ( ref $id && grep( $_ eq 'new', @$id ) )
        ? ' selected="selected"'
        : '' )
        . '>&lt;新規フィールド&gt;</option>';
    $selector
        .= '<option value=""></option><option value=""'
        . ( ( !$id ) ? ' selected="selected"' : '' )
        . '>&lt;なし&gt;</option></select>';
    return $selector;
}

######################################################################
# 設定呼出
sub read_setting ($) {
    my $this = shift;

    #カテゴリー設定読み込み
    $this->import_db( $default::preference_dir . '/category',
        \%setting::category, 1 );
    $setting::category{'LIMIT'} = 2000
        if !defined $setting::category{'LIMIT'}
        || !$setting::category{'LIMIT'};
    $setting::category{'ADD'} = '' if !defined $setting::category{'ADD'};

    #フィールド設定読み込み
    $this->use_field( 'field', $default::field_pref_dir . '/inputfield',
        2, 2, 'no' );

}

######################################################################
# カテゴリーメニュー
sub print_category_selector ($$) {
    my ( $this, $option ) = @_;
    print '<select name="c" id="c">';
    foreach my $o ( sort { $a <=> $b } keys %{ $setting::category{'ORDER'} } )
    {
        my $c = $setting::category{'ORDER'}{$o};
        print '<option value="', $c, '"',
            ( ( defined $main::c && $c eq $main::c )
            ? ' selected="selected"'
            : '' ), '>', $setting::category{$c}{'name'}, ' (',
            ( $setting::category{$c}{'count'} || 0 ), '/',
            ( $setting::category{$c}{'total'} || 0 ), ')</option>';
    }
    print '<option value="new">新規作成</option>' if $option;
    print '</select>';
}

######################################################################
# 新規カテゴリー
sub mk_category ($$;$$) {
    my ( $this, $id, $name, $order ) = @_;
    my ( %new, $new_name, $category_count );
    $this->set_dir("$default::db/$id");
    $this->set_dir("$default::file/$id");
    $category_count = keys %{ $setting::category{'ORDER'} };
    $new_name = ( defined $name && $name ) ? $name : '';
    if ( $new_name ne '' ) {
        my ( @names, $name_text, $name_no );
        @names = map( $setting::category{$_}{'name'},
            grep( defined $_ && $_ !~ /[^0-9]/, keys %setting::category ) );
        ( $name_text, $name_no ) = ( $1, $2 )
            if $new_name =~ /^(.*?)([0-9]*)$/;
        while ( grep( $_ eq $name_text . $name_no, @names ) ) {
            $name_no++;
            $name_no++ if $name_no == 1;
        }
        $new_name = $name_text . $name_no;
    }
    if ( $order && $order <= $category_count ) {
        my %order = %{ $setting::category{'ORDER'} };
        foreach ( sort { $a <=> $b } keys %order ) {
            if ( $_ == $order ) {
                $setting::category{$id}{'ORDER'} = $new{ $id . '_ORDER' }
                    = $order;
                $setting::category{'ORDER'}{$order}
                    = $new{ 'ORDER_' . $order } = $id;
                $setting::category{$id}{'name'} = $new{ $id . '_name' }
                    = ( $new_name ne '' ) ? $new_name : $order;
            }
            if ( $new{ 'ORDER_' . $_ } ) {
                my $o = $_ + 1;
                my $i = $order{$_};
                $setting::category{$i}{'ORDER'} = $new{ $i . '_ORDER' } = $o;
                $setting::category{'ORDER'}{$o} = $new{ 'ORDER_' . $o } = $i;
            }
        }
    }
    else {
        my $o = $category_count + 1;
        $setting::category{$id}{'ORDER'} = $new{ $id . '_ORDER' } = $o;
        $setting::category{'ORDER'}{$o}  = $new{ 'ORDER_' . $o }  = $id;
        $setting::category{$id}{'name'}  = $new{ $id . '_name' }
            = ( $new_name ne '' ) ? $new_name : $new{ $id . '_ORDER' };
    }
    $this->write_db( $default::preference_dir . '/category', \%new );
}

sub delete_category ($$) {
    my ( $this, $c ) = @_;

    if ( opendir( DIR, "$default::db/$c" ) ) {
        foreach ( readdir(DIR) ) {
            unlink "$default::db/$c/$_";
        }
        closedir(DIR);
    }

    #添付ファイル削除
    rmtree "$default::file/$main::c";

    return;
}

######################################################################
# 以下はTable.pmでの区切り文字テキスト分解に関係する処理
# TABLE
sub table {
    my $this = shift;
    $this->set_dir("$default::db/$main::c");
    $this->{'db_dir'} = "$default::db/$main::c/";
    $this->{'total'}  = $this->{'row'} = 0;
    $this->{'time'}   = time;

    tie(%{ $this->{'db'}{'record_id'} },
        $env::dbm_file,
        $default::db . '/record_id',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %{ $this->{'db'}{'record_id'} }
        && !$this->{'db'}{'record_id'}{'last'} )
    {
        my $n = 0;
        foreach my $c (
            grep( defined $_ && $_ !~ /[^0-9]/, keys %setting::category ) )
        {
            my (%ctime);
            tie( %ctime, $env::dbm_file, "$default::db/$c/ctime", O_RDONLY,
                $env::dbm_mode );
            if ( ref tied %ctime ) {
                while ( my ($num) = each %ctime ) {
                    $n = $num if $n < $num;
                }
            }
            untie %ctime;
        }
        $this->{'db'}{'record_id'}{'last'} = $n;
    }

    tie(%{ $this->{'db'}{'mtime'} },
        $env::dbm_file,
        $this->{'db_dir'} . 'mtime',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    tie(%{ $this->{'db'}{'ctime'} },
        $env::dbm_file,
        $this->{'db_dir'} . 'ctime',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );

    #レコード数カウント
    while ( my ( $k, $v ) = each %{ $this->{'db'}{'ctime'} } ) {
        $this->{'total'}++;
    }

    #照合フィールド読み込み
    if ($main::check) {
        $this->{'KEY_DB'} = $field::obj{$main::check}->db;
        if ( defined $this->{'KEY_DB'} && $this->{'KEY_DB'} ) {
            unless ( ref tied %{ $this->{'db'}{ $this->{'KEY_DB'} } } ) {
                tie(%{ $this->{'db'}{ $this->{'KEY_DB'} } },
                    $env::dbm_file,
                    $this->{'db_dir'} . $this->{'KEY_DB'},
                    O_RDWR | O_CREAT,
                    $env::dbm_mode
                );
            }
            while ( my ( $k, $v )
                = each %{ $this->{'db'}{ $this->{'KEY_DB'} } } )
            {
                utf8::decode($k) if !utf8::is_utf8($k);
                utf8::decode($v) if !utf8::is_utf8($v);
                $this->{'KEY'}{$v} = $k;
            }
        }
    }

    #必須項目のデフォルト
    $this->{'required'} = {};
    while ( my ( $id, $field ) = each %field::obj ) {
        $this->{'required'}{$id} = ( $field->{'required'} ) ? 1 : 0;
    }
    return '';
}

######################################################################
# _TABLE
sub _table {
    my $this = shift;

    #データベースとの接続を解除
    while ( my ( $k, $v ) = each %{ $this->{'db'} } ) {
        untie %{$v} if ref tied %{$v};
        undef %{$v};
    }
    delete $this->{'db'};
    delete $this->{'REF'};
    delete $this->{'KEY'};
    delete $this->{'KEY_DB'};

    #新規フィールド登録
    $this->write_db( $default::field_pref_dir . '/inputfield',
        $this->{'NewField'} )
        if ref $this->{'NewField'};
    delete $this->{'NewField'};

    my ( %category, $nocount );
    $nocount = 0;
    if ( $field::for{'pending'} ) {
        my (%pending);
        tie( %pending, $env::dbm_file, $this->{'db_dir'} . 'pending',
            O_RDONLY, $env::dbm_mode );
        if ( ref tied %pending ) {
            while ( my ($k) = each %pending ) {
                $nocount++
                    if $pending{ $this->encode_utf8($k) }
                    && $this->decode_utf8(
                    $pending{ $this->encode_utf8($k) } ) =~ /^1/;
            }
        }
        untie %pending;
    }
    $setting::category{$main::c}{'count'} = $this->{'total'} - $nocount;
    $setting::category{$main::c}{'total'} = $this->{'total'};
    tie(%category, $env::dbm_file,
        $default::preference_dir . '/category',
        O_RDWR | O_CREAT,
        $env::dbm_mode
    );
    if ( ref tied %category ) {
        $category{ $this->encode_utf8( $main::c . '_count' ) }
            = $this->encode_utf8( $setting::category{$main::c}{'count'} );
        $category{ $this->encode_utf8( $main::c . '_total' ) }
            = $this->encode_utf8( $setting::category{$main::c}{'total'} );
    }
    untie %category;
    $this->write_category_list;
    delete $this->{'row'};
    delete $this->{'total'};
    return '';
}

######################################################################
# TR
sub tr {
    my $this = shift;
    $this->{'col'}        = 1;
    $this->{'data_count'} = 0;
    $this->{'check_data'} = '';
    $this->{'is_valid'}   = 1;
    $this->{'row_num'}++;

    #必須項目のデフォルト
    while ( my ( $id, $required ) = each %{ $this->{'required'} } ) {
        $field::obj{$id}->{'required'} = $required;
    }

    #レコード数制限オーバー
    if ( $this->{'total'} >= $setting::category{'LIMIT'} ) {

        #特殊なDB
        if ( defined $default::type
            && ( $default::type eq 'account' || $default::type eq 'order' ) )
        {
            $this->_table;
            my ( @categories, $name, $d );
            @categories = sort {
                $setting::category{$a}{'ORDER'}
                    <=> $setting::category{$b}{'ORDER'}
                    || $a <=> $b
                } grep( defined $_ && $_ !~ /[^0-9]/,
                keys %setting::category );
            $name = $setting::category{ $categories[$#categories] }{'name'};
            $main::c = $categories[$#categories] + 1;
            $d       = $1 if $name =~ s/([0-9]+)$//;
            $d       = 1 if $d == 0;
            $this->mk_category( $main::c, $name . $d );
            $this->table;

  #最も古いレコードを消去して新しいレコードを追加する
        }
        elsif ( $setting::category{'ADD'} eq 'add' ) {
            my $first = $this->minimum( $this->{'db'}{'ctime'} );
            $this->delete_record_from_all_db( $main::c, $first );
            $this->{'total'}--;

            #添付ファイル削除
            use File::Path;
            rmtree "$default::file/$main::c/$first";

  #新しいカテゴリーを作成して新しいレコードを追加する
        }
        elsif ( $setting::category{'ADD'} eq 'newcategory' ) {
            my ( $name, $my_name, $d, @categories, @new );
            $this->_table;
            @categories = sort { $a <=> $b || $a cmp $b }
                grep( defined $_ && $_ !~ /[^0-9]/, keys %setting::category );
            $name
                = ( defined $setting::category{$main::c}{'name'} )
                ? $setting::category{$main::c}{'name'}
                : 1;
            $d = $1 if $name =~ s/([0-9]+)$//;
            $d++;
            $d++ if $d == 1;
            while (1) {
                $my_name = $name . $d;
                @new
                    = grep( defined $setting::category{$_}{'name'}
                        && $setting::category{$_}{'name'} eq $my_name,
                    @categories );
                if (@new) {
                    @new
                        = grep( defined $setting::category{$_}{'total'}
                            && $setting::category{$_}{'total'}
                            < $setting::category{'LIMIT'},
                        @new );
                    if (@new) {
                        last;
                    }
                    else {
                        $d++;
                        next;
                    }
                }
                else {
                    last;
                }
            }
            if (@new) {
                $main::c = $new[0];
            }
            else {
                my $order = $setting::category{$main::c}{'ORDER'} + 1;
                $main::c = $this->new_number( \%setting::category, 1 );
                $this->mk_category( $main::c, $my_name, $order );
            }
            $this->table;

            #新しいレコードの書き込みを拒絶する
        }
        else {
            $this->{'exit'} = 1;
            push(
                @{ $this->{'dialog'} },
                'レコード数が '
                    . $setting::category{'LIMIT'}
                    . ' 件を超えたので、一部読み込みに失敗しました。'
            );

        }
    }

    #データ番号割り出し
    if ( ref tied %{ $this->{'db'}{'record_id'} } ) {
        $this->{'row'} = ( $this->{'db'}{'record_id'}{'last'} || 0 ) + 1;
    }
    return '';
}

######################################################################
# _TR
sub _tr {
    my $this = shift;
    if ( !$this->{'exit'} && $this->{'data_count'} ) {
        my ( $no_count, $row );
        if ( $this->{'is_valid'} ) {

            #行番号
            $row = $this->{'row'};

            #照合
            if (   $main::check
                && ref $this->{'record'}
                && %{ $this->{'record'} }
                && $this->{'data_count'} )
            {

#データキー判定（照合フィールド　値→キー　or　読み込み番号）
                if (   defined $this->{'check_data'}
                    && defined $this->{'KEY'}{ $this->{'check_data'} }
                    && $this->{'KEY'}{ $this->{'check_data'} } ne ''
                    && $this->{'db'}{'mtime'}{
                        $this->encode_utf8(
                            $this->{'KEY'}{ $this->{'check_data'} }
                        )
                    }
                    )
                {
                    $row      = $this->{'KEY'}{ $this->{'check_data'} };
                    $no_count = 1;
                }
                if ( $row > 0 || !$main::option ) {
                    while ( my ( $no, $hash ) = each %{ $this->{'record'} } )
                    {
                        while ( my ( $n, $data ) = each %$hash ) {
                            $this->{'REF'}{$no}{$n}{$row}
                                = $this->encode_utf8($data)
                                if ref $this->{'REF'}{$no}{$n};
                        }
                    }
                }
            }

            #更新時刻の書き込み
            $this->{'db'}{'mtime'}{ $this->encode_utf8($row) }
                = $this->{'time'};
            $this->{'db'}{'ctime'}{ $this->encode_utf8($row) }
                = $this->{'time'}
                if !defined $this->{'db'}{'ctime'}{ $this->encode_utf8($row) }
                || !$this->{'db'}{'ctime'}{ $this->encode_utf8($row) };

        }
        else {
            push(
                @{ $this->{'dialog'} },
                '読み込みに失敗しました('
                    . $this->{'row_num'}
                    . '行目)。'
            );
        }

        delete $this->{'check_data'};
        undef %{ $this->{'record'} };
        if ( !$no_count ) {
            $this->{'db'}{'record_id'}{ $this->{'row'} } = $main::c;
            $this->{'db'}{'record_id'}{'last'} = $this->{'row'}
                if !$this->{'db'}{'record_id'}{'last'}
                || $this->{'db'}{'record_id'}{'last'} < $this->{'row'};
            $this->{'total'}++;
        }
    }
    $main::option = '';
    return '';
}

######################################################################
# TD
sub td {
    my ( $this, $data ) = @_;
    return '' if $this->{'exit'};
    if ( ref $main::collate{ $this->{'col'} } ) {
        my $n = 0;
        foreach my $id ( @{ $main::collate{ $this->{'col'} } } ) {
            $this->new_field( $this->{'col'}, $n, $data )
                unless ref tied %{ $this->{'REF'}{ $this->{'col'} }{$n} };
            if ( ref $this->{'REF'}{ $this->{'col'} }{$n} ) {
                my $str = $data;
                if ( ref $field::obj{$id} ) {
                    if (   $field::obj{$id}->{'class'} eq 'time_date'
                        && $str =~ /[^0-9]/s )
                    {
                        $str = $this->time_id( $this->date_to_gmt($str) );
                    }
                    local %main::data = ();
                    %{ $main::data{$id} } = $field::obj{$id}->elements($str);
                    if ( $field::obj{$id}->is_valid_data ) {
                        $str = $main::data{$id}{'str'};
                    }
                    else {
                        $this->{'is_valid'} = 0;
                    }
                }
                if ( $this->{'is_valid'} ) {
                    if ($main::check) {
                        $this->{'record'}{ $this->{'col'} }{$n} = $str;
                        $this->{'check_data'} = $str if $main::check eq $id;
                    }
                    elsif ( $this->{'row'} > 0 || !$main::option ) {
                        $this->{'REF'}{ $this->{'col'} }{$n}{ $this->{'row'} }
                            = $this->encode_utf8($str);
                    }
                    $this->{'data_count'}++
                        if defined $str
                        && $str ne ''
                        && ( $this->{'row'} || !$main::option );
                }
            }
            $n++;
        }
    }
    $this->{'col'}++;
    return '';
}

######################################################################
# フィールドオブジェクト
sub new_field ($$$$) {
    my ( $this, $no, $n, $name ) = @_;
    my ( $id, $db );
    return if !$main::collate{$no}[$n];
    $id = $main::collate{$no}[$n];
    if ( $id && ( $id eq 'new' || !ref $field::obj{$id} ) ) {
        if ( !%field::obj ) {
            %field::obj = ();
            %field::for = ();
        }
        $main::collate{$no}[$n] = $id = $this->new_number( \%field::obj, 1 );
        my $num = ( keys %field::ORDER ) + 1;
        $field::ORDER{$num} = $id;

        $this->{'NewField'}{ 'ORDER_' . $num } = $id;
        $this->{'NewField'}{ $id . '_ORDER' }  = $num;
        $this->{'NewField'}{ $id . '_class' }  = 'text';

        if ( $name eq '' || length $name > 20 ) {
            $name = '項目' . $num;
        }
        else {
            $this->detag( \$name );
        }
        $this->{'NewField'}{ $id . '_name' } = $name;

        $this->{'NewField'}{ $id . '_Open' }          = 1;
        $this->{'NewField'}{ $id . '_searchEnabled' } = 1;
        $this->{'NewField'}{ $id . '_sortEnabled' }   = 1;
        $this->{'NewField'}{ $id . '_Default' }       = 1;
        $this->{'NewField'}{ $id . '_cellWidth' }     = 150;

        $field::obj{$id}
            = Field->new( 'text', $id, 'field', \%field::obj, \%field::for );
        $field::obj{$id}->set(
            'ORDER' => $num,
            'name'  => $name
        );
    }

    $db = $field::obj{$id}->db;
    if ( !$db ) {
        return;
    }
    elsif ( !ref tied %{ $this->{'db'}{$db} } ) {
        tie(%{ $this->{'db'}{$db} },
            $env::dbm_file,
            $this->{'db_dir'} . $db,
            O_RDWR | O_CREAT,
            $env::dbm_mode
        );
    }

    #順序基準リファレンス
    $this->{'REF'}{$no}{$n} = $this->{'db'}{$db};
}

sub get_setting ($;$$$$$) {
    my ( $this, $table, $type, $info, $op, $setting ) = @_;
    return '';
}

sub delete_record_from_all_db ($$@) {
    my ( $this, $c, $n ) = @_;
    my ( $dir, @files, %database );
    $dir = $default::db . '/' . $c;
    if ( $setting::general{'oneEmailToOneRecord'} ) {
        $this->use_field(
            'field',
            $default::field_pref_dir . '/inputfield',
            $setting::field,
            $setting::field_style_disabled,
            $setting::field_tag_enabled,
            $setting::field_tag_name
        ) if !%field::obj;
        if ( $field::for{'mail'} ) {
            my $db = $field::obj{ $field::for{'mail'} }->db;
            if ( !ref tied %{ $this->{'db'}{$db} } ) {
                tie(%{ $this->{'db'}{$db} },
                    $env::dbm_file, $dir . '/' . $db,
                    O_RDWR, $env::dbm_mode
                );
            }
            if ( ref tied %{ $this->{'db'}{$db} } ) {
                my $email = $this->{'db'}{$db}{ $this->encode_utf8($n) };
                if ($email) {
                    if ( !ref tied %{ $this->{'db'}{'email_to_id'} } ) {
                        tie(%{ $this->{'db'}{'email_to_id'} },
                            $env::dbm_file,
                            $default::db . '/email_to_id',
                            O_RDWR,
                            $env::dbm_mode
                        );
                    }
                    if ( ref tied %{ $this->{'db'}{'email_to_id'} } ) {
                        delete $this->{'db'}{'email_to_id'}{$email};
                    }
                    if (   $setting::general{'syncRegister'}
                        && $this->use_register
                        && $cgi::Register->remove( '', $email, 1 ) )
                    {
                        $cgi::Register->success_dialog;
                    }
                }
            }
        }
    }
    opendir( DIR, $dir );
    @files = readdir(DIR);
    closedir(DIR);

    foreach my $db (@files) {
        next if $db =~ /^\./ || $db =~ /\.htm[l]?$/i || !-f $dir . '/' . $db;
        $db =~ s/\.[^\/\.]+$//;
        $database{$db} = 1;
    }

    utf8::encode($n) if utf8::is_utf8($n);
    while ( my ($db) = each %database ) {
        if ( !ref tied %{ $this->{'db'}{$db} } ) {
            tie(%{ $this->{'db'}{$db} },
                $env::dbm_file, $dir . '/' . $db,
                O_RDWR, $env::dbm_mode
            );
        }
        if ( ref tied %{ $this->{'db'}{$db} } ) {
            delete $this->{'db'}{$db}{$n};
        }
    }
    if ( ref tied %{ $this->{'db'}{'record_id'} } ) {
        delete $this->{'db'}{'record_id'}{$n};
    }

    push(
        @{ $this->{'dialog'} },
        'レコード数が '
            . $setting::category{'LIMIT'}
            . ' 件以内に制限されているため、最も古いレコードを削除しました('
            . $n . ')。'
    );
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 't301', 'TableImport' );

    #メニュー登録
    $setting::jump_menu{'t301'} = '読み込み';
}

1;
