package TeaserVariation;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
use Fcntl qw(:flock);
######################################################################
# 埋め込み用リストバリエーション数変更モジュール
# TeaserVariation.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2010-09-04
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my ( $valid_flag, $error_flag );
    $default::teaser_variation = 0 if !$default::teaser_variation;

    #入力チェック
    if ( defined $main::variation ) {
        if ( $this->is_nan($main::variation) ) {
            push( @{ $this->{'dialog'} }, '不正な入力です。' );
            $error_flag = 1;
        }
        elsif ( $main::variation != $default::teaser_variation ) {
            $valid_flag = 1;
        }
    }

    #teaser.pl書き換え
    if ( $valid_flag && !$error_flag ) {
        local (*FILE);
        my ( $data, $default_path );
        $default_path = './default.pl';
        $data = $this->read_file( $default_path, 1 );
        unless ( $data
            =~ s/^[\x20\x09]*\$teaser_variation[\x20\x09]*=[^;]*;/\$teaser_variation = $main::variation;/gm
            || !$main::variation )
        {
            $data
                =~ s/^1;/# 埋め込み用リストのバリエーション数\x0A\$teaser_variation = $main::variation;\x0A\x0A1;/m;
        }
        if ( $default::teaser_variation > $main::variation ) {
            for (
                my $i = $main::variation + 1;
                $i <= $default::teaser_variation;
                $i++
                )
            {
                my $teaser_path
                    = $default::file . '/teaser' . ( ( $i > 1 ) ? $i : '' );
                unlink $teaser_path . '.ssi';
                unlink $teaser_path . '.js';
            }
        }
        if ( open( FILE, ">>$default_path" ) ) {
            binmode FILE, ":utf8";
            flock( FILE, LOCK_EX );
            truncate( FILE, 0 );
            seek( FILE, 0, 0 );
            print FILE $data;
            close(FILE);
            push(
                @{ $this->{'dialog'} },
                '埋め込み用リストバリエーション数を更新しました。'
            );
            $default::teaser_variation = $main::variation;
            undef $main::variation;
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '埋め込み用リストバリエーション数の更新に失敗しました。'
            );
        }
    }
    elsif ( defined $error_flag && $error_flag ) {
        push(
            @{ $this->{'dialog'} },
            '埋め込み用リストバリエーション数の更新に失敗しました。'
        );
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>埋め込み用リストバリエーション数変更</h1>';

    print $this->form_start(
        'key'    => 'tools',
        'name'   => 'input',
        'method' => 'post',
        'onsubmit' =>
            'return confirm(\'この操作は取り消せません。本当に変更しますか？\') &amp;&amp; cmonos.form.item(this).isNotSubmitted();',
        'required' => \@setting::form
    );
    print '<fieldset class="submit"><select name="variation">';
    foreach my $n ( 0 .. 10 ) {
        print '<option value="'
            . $n . '"'
            . ( ( $default::teaser_variation == $n )
            ? ' selected="selected"'
            : '' )
            . '>'
            . $n
            . '</option>';
    }
    print '</select>';
    print '<input type="submit" value="変更" />';
    print '</fieldset></form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if ( !$default::type || $default::type ne 'order' ) {
        no warnings qw(once);

        #クラス登録
        Common::class_register( '00t2', 'TeaserVariation' );

        #メニュー登録
        $setting::jump_menu{'00t2'}
            = '埋め込み用リストバリエーション数';

    }
}

1;
