package SearchDBEditor;
require CMONOS::CGI_Lib::SetUp;
@ISA = qw(SetUp);
use strict;
use utf8;
######################################################################
# 検索ワードデータベースの整理モジュール
# MkZipCodeDB.pm ver1.0
# Copyright (C) 1999-2018 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
#
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    my $mtime;

    use Fcntl qw(:DEFAULT);
    eval 'use ' . $env::dbm_file . ';';
    $this->jump_menu('tools');

    #データベース処理
    if ( $main::words || $main::num ) {
        my ( %words, @delete, @words );
        $main::mode = "and" if !$main::mode;
        if ( defined $main::words && $main::words ne '' ) {
            @words = grep( $_ ne '', split( /\s+/, $main::words ) );
            map( $_ = quotemeta($_), @words );
        }
        tie( %words, $env::dbm_file, $default::db . '/words',
            O_RDWR, $env::dbm_mode );
        if ( ref tied %words ) {
            while ( my ( $k, $v ) = each %words ) {
                utf8::decode($k) if !utf8::is_utf8($k);
                utf8::decode($v) if !utf8::is_utf8($v);
                next             if $k eq 'MAX VALUE';
                my $is_target = 0;
                if ($main::words) {
                    if ( $main::mode eq 'eq' ) {
                        $is_target = 1 if grep( $k eq $_, @words );
                    }
                    elsif ( $main::mode eq 'or' ) {
                        $is_target = 1 if grep( $k =~ /$_/, @words );
                    }
                    else {
                        $is_target = 1 if !grep( $k !~ /$_/, @words );
                    }
                }
                else {
                    $is_target = 1;
                }
                if ( $is_target && $main::num ) {
                    my ( $t, $n ) = split( /\t/, $v );
                    push( @delete, $k ) if $n < $main::num;
                }
            }
            if (@delete) {
                my $count = @delete;
                foreach my $w (@delete) {
                    delete $words{$w};
                    push(
                        @{ $this->{'dialog'} },
                        '「'
                            . $this->escape($w)
                            . '」を削除しました。'
                    ) if $count < 10;
                }
                push(
                    @{ $this->{'dialog'} },
                    $count . ' の検索ワードを削除しました。'
                );
            }
            else {
                push(
                    @{ $this->{'dialog'} },
                    '削除条件に該当する検索ワードはみつかりませんでした。'
                );
            }
        }
        else {
            push(
                @{ $this->{'dialog'} },
                '検索ワードデータベースに接続できませんでした。'
            );
        }
        untie %words;
    }
    elsif ($main::a) {
        push( @{ $this->{'dialog'} },
            '条件を入力してください。' );
    }

    #ページヘッダ
    $this->print_page_header('tools');

    print
        '<h1 title="検索ワードデータベースを整理します">検索ワードデータベースの整理</h1>';

    #フォーム開始
    print $this->form_start(
        'key'      => 'tools',
        'name'     => 'input',
        'method'   => 'post',
        'required' => \@setting::form
    );
    print '<input type="hidden" name="a" value="1" />';
    print '<fieldset class="delete">';
    print
        '<label>検索された数が<input type="number" size="3" name="num" title="必須項目です。" />回以下</label>かつ ';
    print
        '<label>検索ワードが<input type="text" size="20" name="words" title="空欄の場合回数条件に一致する全ての検索ワードが削除されます。" /></label>';
    print $this->mk_select_menu(
        'mode', '', '', '',
        'eq'  => 'である',
        'and' => 'すべてに一致する',
        'or'  => 'いずれかに一致する',
    );
    print
        '検索ワードを<input type="submit" value="削除する" onclick="return confirm(\'この操作は取り消せません。本当に更新しますか？\') &amp;&amp; cmonos.form.item(this).isNotSubmitted();" />';
    print '</fieldset>';

    print '</form>';

    #ページフッタ
    $this->print_page_footer('tools');
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    no warnings qw(once);

    #クラス登録
    Common::class_register( 'xxde', 'SearchDBEditor' );

    #メニュー登録
    $setting::jump_menu{'xxde'}
        = '検索ワードデータベースの整理';

}

1;
