#!/usr/local/bin/perl
# CMONOS.JP 簡単インストーラー ver1.0.3
#use KCatch qw( mode=html );

use strict;
use utf8;
use Fcntl qw(:DEFAULT :flock);
use File::Path;
use Config;
use FileHandle;
STDOUT->autoflush(1);
binmode STDOUT, ":utf8";

my $this = {};
bless $this;

my $temp_dir = './CMONOS_JP_INSTALLER_TEMP';
my $status_file = $temp_dir . '/status';
my $status = $this->read_file($status_file) || 0;
#my $perl_path = $^X;
my $perl_path = '/usr/local/bin/perl';
my @error_msg = ();
my %in;
if ($ENV{'QUERY_STRING'}) {
	while ($ENV{'QUERY_STRING'} =~ /\G([^=\&]*)=([^\&]*)\&?/sgo) {
		my($k,$v) = ($1,$2);
		$k =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
		$v =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C",hex($1))/ge;
		utf8::decode($k);
		utf8::decode($v);
		$in{$k} = $v;
	}
}
my $action = $in{'action'} || (($status) ? 'install' : '');
my $cmonos_dir = $this->get_cmonos_dir($temp_dir);
if ($status < 3 && $cmonos_dir) {
	$action = 'install';
	$status = 4;
}
if ($status == 5 && $action eq 'replace') {
	my $is_valid = 1;
	my $cgidatapath = ($in{'cgidatapath'}) ? $in{'cgidatapath'} : $in{'cgidata'};
	$cgidatapath .= '/' if $cgidatapath !~ m!/$!;
	my $cgidata_dir = $cgidatapath;
	$cgidata_dir =~ s!^\.\./\.\./!!;
	$cgidata_dir = './' if $cgidata_dir eq '';
	my @htdocs_files = $this->get_file_list($cmonos_dir . '/home/htdocs');
	my @cgidata_files = $this->get_file_list($cmonos_dir . '/home/cgi-data');
	if (defined $cgidatapath) {
		if ($cgidatapath !~ /^\.\.\/\.\.\// || $cgidatapath =~ /[^0-9a-zA-Z_\-\/\.]/) {
			push(@error_msg,'「cgi-data」フォルダの位置を確認してください。CGIからみた相対パスで入力してください。');
			$is_valid = 0;
		} elsif (!-d $cgidata_dir) {
			push(@error_msg,'「cgi-data」フォルダの位置を確認してください。指定されたディレクトリは存在しません。');
			$is_valid = 0;
		} else {
			if (-d $cgidata_dir . 'cgi-data') {
				my @conflicts = grep(-e $cgidata_dir . 'cgi-data/' . $_,'.htaccess',@cgidata_files);
				if (@conflicts) {
					push(@error_msg,'「cgi-data」フォルダを確認してください。インストールされるファイルまたはフォルダと同名のファイルまたはフォルダが存在します(' . join(',',@conflicts) . ')。');
					$is_valid = 0;
				}
			}
		}
	}
	if (defined $in{'sendmail'} && $in{'sendmail'} ne '') {
		if ($in{'sendmail'} =~ /[^0-9a-zA-Z_\-\/\.\:]/) {
			push(@error_msg,'sendmail パスを確認してください。半角英数字で入力してください。');
			$is_valid = 0;
		} elsif (!-e $in{'sendmail'}) {
			push(@error_msg,'sendmail パスを確認してください。設定されたパスでは sendmail にアクセスできませんでした。');
			$is_valid = 0;
		}
	}
	if (defined $in{'dbm'}) {
		if ($in{'dbm'} !~ /^[0-9a-zA-Z_]+_File$/) {
			push(@error_msg,'DBM の種類を確認してください。原因不明の入力エラーです。');
			$is_valid = 0;
		} elsif (!eval 'use ' . $in{'dbm'} . ';1;') {
			push(@error_msg,'DBM の種類を確認してください。設定された DBM は使用できません。');
			$is_valid = 0;
		}
	}
	if (defined $in{'filemode'} && $in{'filemode'} ne '' && $in{'filemode'} !~ /^06[0-6][0-6]$/) {
		push(@error_msg,'ファイルのパーミッションを確認してください。');
		$is_valid = 0;
	}
	if (defined $in{'dirmode'} && $in{'dirmode'} ne '' && $in{'dirmode'} !~ /^07[0-7][0-7]$/) {
		push(@error_msg,'ディレクトリのパーミッションを確認してください。');
		$is_valid = 0;
	}
	if (defined $in{'cgimode'} && $in{'cgimode'} ne '' && $in{'cgimode'} !~ /^07[0-7][0-7]$/) {
		push(@error_msg,'CGI ファイルのパーミッションを確認してください。');
		$is_valid = 0;
	}
	my @conflicts = grep(-e './' . $_,@htdocs_files);
	if (@conflicts) {
		push(@error_msg,'インストール先階層を確認してください。インストールされるファイルまたはフォルダと同名のファイルまたはフォルダが存在します(' . join(',',@conflicts) . ')。');
		$is_valid = 0;
	}
	if ($is_valid) {
		unless (-d $cgidata_dir . 'cgi-data' || mkdir $cgidata_dir . 'cgi-data') {
			push(@error_msg,'「cgi-data」フォルダの位置を確認してください。指定されたディレクトリには「cgi-data」フォルダを作成できませんでした。');
			$is_valid = 0;
		}
	}
	if ($is_valid) {
		if (-f './.htaccess') {
			$this->write_file($cmonos_dir . '/home/htdocs/_htaccess',$this->read_file('./.htaccess') . $this->read_file($cmonos_dir . '/home/htdocs/_htaccess'));
		}
		foreach my $file (@cgidata_files) {
			rename($cmonos_dir . '/home/cgi-data/' . $file,$cgidata_dir . 'cgi-data/' . (($file eq '_htaccess') ? '.htaccess' : $file));
		}
		foreach my $file (@htdocs_files) {
			rename($cmonos_dir . '/home/htdocs/' . $file,'./' . (($file eq '_htaccess') ? '.htaccess' : $file));
		}
		if ($in{'FollowSymLinks'} || $in{'Includes'} || $in{'ExecCGI'} || $in{'Indexes'}) {
			$this->write_file(
				'./.htaccess',
				'Options'	 . (($in{'FollowSymLinks'}) ? ' +FollowSymLinks' : '')
							 . (($in{'Includes'}) ? ' +Includes' : '')
							 . (($in{'ExecCGI'}) ? ' +ExecCGI' : '')
							 . (($in{'Indexes'}) ? ' +Indexes' : '') . "\x0A"
						 . (($in{'ExecCGI'}) ? "AddType text/html cgi\x0AAddHandler cgi-script cgi\x0A" : '')
						 . (($in{'Indexes'}) ? "AddType text/html shtml\x0AAddHandler server-parsed shtml\x0A" : '')
						 . $this->read_file('./.htaccess')
			);
		}
		my $data = $this->read_file($cgidata_dir . 'cgi-data/lib/env.pl');
		utf8::decode($data) if !utf8::is_utf8($data);
		$data =~ s/^[\x20\x09]*\$sendmail[\x20\x09]*=[^;]*;/\$sendmail = '$in{'sendmail'}';/gm if defined $in{'sendmail'} && $in{'sendmail'} ne '';
		if (defined $in{'dbm'} && $in{'dbm'} ne '') {
			$data =~ s/^[\x20\x09]*\$dbm_file[\x20\x09]*=[^;]*;/\$dbm_file = '$in{'dbm'}';/gm;
			my $max = ($in{'dbm'} eq 'DB_File' || $in{'dbm'} eq 'GDBM_File') ? 128000 : ($in{'dbm'} eq 'SDBM_File' || $in{'dbm'} eq 'ODBM_File') ? 900 : 4000;
			$data =~ s/^[\x20\x09]*\$db_maxlength[\x20\x09]*=[^;]*;/\$db_maxlength = $max;/gm;
		}
		if (defined $in{'filemode'} && $in{'filemode'} ne '') {
			$data =~ s/^[\x20\x09]*\$dbm_mode[\x20\x09]*=[^;]*;/\$dbm_mode = $in{'filemode'};/gm;
			$data =~ s/^[\x20\x09]*\$file_mode[\x20\x09]*=[^;]*;/\$file_mode = $in{'filemode'};/gm;
		}
		$data =~ s/^[\x20\x09]*\$dir_mode[\x20\x09]*=[^;]*;/\$dir_mode = $in{'dirmode'};/gm if defined $in{'dirmode'} && $in{'dirmode'} ne '';
		$data =~ s/^[\x20\x09]*\$cgi_file_mode[\x20\x09]*=[^;]*;/\$cgi_file_mode = $in{'cgimode'};/gm if defined $in{'cgimode'} && $in{'cgimode'} ne '';
		$data =~ s!\.\./\.\./\.\./cgi-data!${cgidatapath}cgi-data!sg if $cgidatapath && $cgidatapath ne '../../../';
		$this->write_file($cgidata_dir . 'cgi-data/lib/env.pl',$data);
		$data = $this->read_file('./m.cgi');
		utf8::decode($data) if !utf8::is_utf8($data);
		$data =~ s/\A#![^\x09\x0A\x0C\x0D\x20]+/#!$perl_path/m if $perl_path;
		$data =~ s!\.\./cgi-data!${cgidata_dir}cgi-data!sg if $cgidatapath && $cgidatapath ne '../../../';
		$this->write_file('./m.cgi',$data);
		foreach my $pl ($this->get_cgi_list) {
			my $changed = 0;
			$data = $this->read_file($pl);
			utf8::decode($data) if !utf8::is_utf8($data);
			$changed = 1 if $perl_path && $pl =~ /\.cgi$/ && $data =~ s/\A#![^\x09\x0A\x0C\x0D\x20]+/#!$perl_path/m;
			if ($cgidatapath && $cgidatapath ne '../../../') {
				$changed = 1 if $data =~ s!\.\./\.\./\.\./cgi-data!${cgidatapath}cgi-data!sg;
				$changed = 1 if $data =~ s!^[\x20\x09]*\$cgi_data_name[\x20\x09]*=[^\x0A\x0D]*;!\$cgi_data_name = '${cgidatapath}cgi-data';!gm;
				$changed = 1 if $data =~ s!^[\x20\x09]*\$shared_cgi_data_name[\x20\x09]*=[^\x0A\x0D]*;!\$shared_cgi_data_name = '${cgidatapath}cgi-data';!gm;
			}
			$this->write_file($pl,$data) if $changed;
		}
		if (eval 'use Jcode;1;&Jcode::FB_XMLCREF;') {
			unlink $cgidata_dir . 'cgi-data/lib/Jcode.pm';
			rmtree $cgidata_dir . 'cgi-data/lib/Jcode';
		}
		chmod(eval($in{'cgimode'} || 0705),'./cgi/setup/start.cgi');
		my $setup = $ENV{'SCRIPT_NAME'};
		$setup =~ s![^/]+$!cgi/setup/start.cgi!;
		rmtree $temp_dir;
		binmode STDOUT, ":utf8";
		print "Content-Type: text/html\n\n";
		print '<html><head><meta http-equiv="Refresh" content="0; URL=' . $setup . '" /></head>';
		print '<body></body><a href="' . $setup . '">next&raquo;</a></html>';
		exit;
	} else {
		$action = 'install';
	}
}

print <<HTML;
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja">
<head>
<meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />
<meta http-equiv="Content-Style-Type" content="text/css" />
<title lang="ja">CMONOS.JP 簡単インストーラー</title>
<style type="text/css">
<!--
html,body{ height: 100%; margin: 0; padding: 0; }
body, table {
  font: 14px 'Lucida Grande','Hiragino Kaku Gothic Pro','ヒラギノ角ゴ Pro W3','HiraKakuProN-W3',Verdana,'メイリオ','Meiryo','ＭＳ Ｐゴシック','MS PGothic',Osaka,arial,helvetica,sans-serif;
}
body {
  background-color: #FFF;
}
div#content {
  padding: 3em 10%;
}
ol.footer {
  list-style-type: none;
  font-size: 0.8em;
}
dl {
  border-top: solid 1px #CCC;
  border-left: solid 1px #CCC;
  border-right: solid 1px #CCC;
  text-align: left;
}
dt, dd {
  margin: 0;
  padding: 0.7em 1em;
  border-bottom: solid 1px #CCC;
}
dt {
  background-color: #F3F3F3;
}
table#license-table {
  margin: 1em 0;
  border-collapse: collapse;
  empty-cells: show;
  border: 1px solid #CCC;
  font-size: 0.9em
}
table#license-table td, table#license-table th {
  margin: 0;
  padding: 0.7em 1em;
  border: 1px solid #CCC;
  background-color: #F3F3F3;
}
table#license-table th {
  background-color: #EEE;
  white-space: nowrap;
}
table#license-table td.title, table#license-table td.support {
  text-align: center;
  white-space: nowrap;
}
table#license-table td.price {
  text-align: right;
  white-space: nowrap;
}
table#license-table tr:hover td {
  background-color: #FFF;
}
div#progress-bar-box-bg {
  width: 100%;
  height: 100%;
  position: fixed !important;
  position: absolute;
  top: 0;
  left: 0;
  background-color: #000;
  z-index: 100;
  opacity: 0.7;
  -moz-opacity: 0.7;
  -webkit-opacity: 0.7;
  -o-opacity: 0.7;
  -ms-filter:"alpha(opacity=70)";
  filter:alpha(opacity=70);
}
div#progress-bar-box {
  width: 100%;
  height: 100%;
  position: fixed !important;
  position: absolute;
  top: 0;
  left: 0;
  z-index: 101;
  color: #FFF;
  text-align: center;
}
div#progress-bar-box a {
  color: #FFF;
  text-decoration: none;
}
div#progress-bar-dialog {
  text-align: left;
  padding-top: 200px;
  width: 482px;
  margin: 0 auto;
}
div#progress-bar-bg {
  text-align: left;
  width: 480px;
  height: 24px;
  margin: 0 auto;
  background-color: rgba(0, 0, 0, 0.5) !important;
  background-color: transparent;
  border: 1px #CCC solid;
}
div#progress-bar {
  width: 1px;
  height: 24px;
  margin: 0;
  background-color: #69F;
  border: none;
}
table {
  border-collapse: collapse;
  empty-cells: show;
  border: 1px solid #CCC;
  font-size: 0.9em
}
td, th {
  margin: 0;
  padding: 0.7em 1em;
  border: 1px solid #CCC;
  background-color: #F3F3F3;
}
th {
  background-color: #EEE;
  text-align: right;
  white-space: nowrap;
}
tr:hover td, tr:hover th {
  background-color: #FFF;
}
form {
  margin: 3em auto;
  text-align: center;
}
input.submit {
  margin: 1em auto;
  font-size: 1.5em;
}
.warning {
  color: red;
}
-->
</style>
<script type="text/javascript" language="JavaScript">
<!-- <![CDATA[
var progressbar = {
	update : function (ratio) {
		if (typeof document.body.style.maxHeight == "undefined") {
			document.getElementById('progress-bar-box-bg').style.height = (document.body.scrollTop + document.body.clientHeight) + 'px';
			document.getElementById('progress-bar-box-bg').style.width = (document.body.scrollLeft + document.body.clientWidth) + 'px';
		}
		if (ratio >= 1) {
			document.getElementById('progress-bar').style.width = '480px';
			document.getElementById('progress-bar-dialog').innerHTML += '[完了]';
			document.getElementById('progress-bar-box-bg').style.display = 'none';
			document.getElementById('progress-bar-box').style.display = 'none';
			if (this.bar_timer != null) clearInterval(this.bar_timer);
		} else {
			this.ratio = ratio;
			var self = this;
			if (this.bar_timer == null) this.bar_timer = setInterval(function () {
				self.ratio += 0.005;
				if (self.ratio > 0.99) self.ratio = 0.99;
				var w = Math.floor(self.ratio * 480);
				document.getElementById('progress-bar').style.width = ((w > 0) ? w : 1) + 'px';
			},500);
		}
	}
};
// ]]> -->
</script>
</head>
<body>
HTML

if (!$action) {
	$this->print_license;
	print '</body></html>';
	exit;
} elsif (!-d $temp_dir && !mkdir($temp_dir,0705)) {
	my $dl_url = $this->download_path(($perl_path =~ /local/) ? 1 : 0);
print <<CONTENT;
<div id="content">
<p>作業用ディレクトリを作成できませんでした。お使いのサーバでは「<a href="http://cmonos.jp/manual/setup_assistant/operation/easyinstall.html" target="_blank">CMONOS.JP 簡単インストーラー</a>」によるインストールができません。</p>
<p>「<a href="$dl_url">CMONOS.JP フルセット版</a>」をダウンロードし、<a href="http://cmonos.jp/manual/setup_assistant/operation/install.html" target="_blank">インストールマニュアル</a>にしたがってインストールしてください。</p>
</div>
</body>
</html>
CONTENT
	exit;
} elsif ($status >= 4) {
	$this->print_extract_error if !$cmonos_dir;
	$this->write_file($status_file,5);
	print '<div id="content"><h1>システム基本設定</h1>';
	print '<p class="warning">これらの設定はインストール後変更できません。</p>';
	if (@error_msg) {
		print '<ul>';
		foreach my $msg (@error_msg) {
			print '<li>' . $msg . '</li>';
		}
		print '</ul>';
	}
	print '<form action="./installer.cgi" method="GET"><input type="hidden" name="action" value="replace" /><dl>';
	print '<dt><label for="cgidata">CMONOS.JPモジュールのほか、初期設定やデータベースなどを格納する「cgi-data」フォルダの位置(「./cgi/setup/start.cgi」からみた相対パス)</label></dt>';
	print '<dd>';
	my $here = $this->get_cwd;
	my $dir = ($here =~ m!/([^/]+)$!) ? $1 : '?';
	if ($dir eq 'web') {
		print '<p class="warning">※「ロリポップ！」「チカッパ！」など、公開ディレクトリより上の階層にアクセスできないサーバでは、公開ディレクトリより上の階層を絶対に指定しないでください。万が一公開ディレクトリより上の階層を指定すると FTP 等で「cgi-data」フォルダにアクセスできなくなるため、メンテナンスが困難になります。</p>';
		$in{'cgidata'} = '../../' if !$in{'cgidata'};
	} elsif (!$in{'cgidata'}) {
		$in{'cgidata'} = '../../../';
	}
	print '<select name="cgidata" id="cgidata">';
	my $menu = '<option value="../../"' . (($in{'cgidata'} && $in{'cgidata'} eq '../../') ? ' selected="selected"' : '') . '>(' . $dir . ') ../../</option>';
	my $rel = '../../';
	while ($here =~ s!/[^/]+$!!) {
		my $uid = (stat($here))[4];
		last if $uid ne $> && $uid ne $<;
		$rel .= '../';
		$dir = ($here =~ m!/([^/]+)$!) ? $1 : '?';
		$menu = '<option value="' . $rel . '"' . (($in{'cgidata'} && $in{'cgidata'} eq $rel) ? ' selected="selected"' : '') . '>(' . $dir . ') ' . $rel . '</option>' . $menu;
	}
	print $menu;
	print '</select></dd>';
	print '<dd>その他:<input type="text" size="80" name="cgidatapath" value="' . $this->escape($in{'cgidatapath'}) . '" />/cgi-data</dd>';
	print '<dt><label for="sendmail">sendmail の位置</label></dt>';
	if (!$in{'sendmail'}) {
		$in{'sendmail'} = '/usr/sbin/sendmail';
		if (-e '/usr/sbin/sendmail') {
			$in{'sendmail'} = '/usr/sbin/sendmail';
		} elsif (-e '/usr/lib/sendmail') {
			$in{'sendmail'} = '/usr/lib/sendmail';
		}
	}
	print '<dd><input type="text" size="80" name="sendmail" id="sendmail" value="' . $this->escape($in{'sendmail'}) . '" /></dd>';
	print '<dt><label for="dbm">DBM の種類</label></dt>';
	print '<dd><select name="dbm" id="dbm">';
	my $ex = $Config{'extensions'};
	print '<option value="DB_File"' . (($in{'dbm'} && $in{'dbm'} eq 'DB_File') ? ' selected="selected"' : '') . '>DB_File(推奨)</option>' if eval 'use DB_File;1;';
	print '<option value="GDBM_File"' . (($in{'dbm'} && $in{'dbm'} eq 'GDBM_File') ? ' selected="selected"' : '') . '>GDBM</option>' if $ex =~ /GDBM_File/;
	print '<option value="NDBM_File"' . (($in{'dbm'} && $in{'dbm'} eq 'NDBM_File') ? ' selected="selected"' : '') . '>NDBM</option>' if $ex =~ /NDBM_File/;
	print '<option value="SDBM_File"' . (($in{'dbm'} && $in{'dbm'} eq 'SDBM_File') ? ' selected="selected"' : '') . '>SDBM(非推奨)</option>' if $ex =~ /SDBM_File/;
	print '<option value="ODBM_File"' . (($in{'dbm'} && $in{'dbm'} eq 'ODBM_File') ? ' selected="selected"' : '') . '>ODBM(非推奨)</option>' if $ex =~ /ODBM_File/;
	print '</select></dd>';
	print '<dt><label for="filemode">ファイルのパーミッション</label></dt>';
	$in{'filemode'} = '0604' if !$in{'filemode'};
	print '<dd><input type="text" size="5" name="filemode" id="filemode" value="' . $this->escape($in{'filemode'}) . '" /></dd>';
	print '<dt><label for="dirmode">ディレクトリのパーミッション</label></dt>';
	$in{'dirmode'} = '0705' if !$in{'dirmode'};
	print '<dd><input type="text" size="5" name="dirmode" id="dirmode" value="' . $this->escape($in{'dirmode'}) . '" /></dd>';
	print '<dt><label for="cgimode">CGI ファイルのパーミッション</label></dt>';
	$in{'cgimode'} = '0705' if !$in{'cgimode'};
	print '<dd><input type="text" size="5" name="cgimode" id="cgimode" value="' . $this->escape($in{'cgimode'}) . '" /></dd>';
	print '<dt>.htaccess ファイル / Options</dt>';
	print '<dd>';
	print '<label><input type="checkbox" name="FollowSymLinks" value="1"' . (($in{'FollowSymLinks'}) ? ' checked="checked"' : '') . ' />+FollowSymLinks</label><strong style="text-decoration:underline"> (CPIなどで必須)</strong> ';
	print '<label><input type="checkbox" name="Includes" value="1"' . (($in{'Includes'}) ? ' checked="checked"' : '') . ' />+Includes</label> ';
	print '<label><input type="checkbox" name="ExecCGI" value="1"' . (($in{'ExecCGI'}) ? ' checked="checked"' : '') . ' />+ExecCGI</label> ';
	print '<label><input type="checkbox" name="Indexes" value="1"' . (($in{'Indexes'}) ? ' checked="checked"' : '') . ' />+Indexes</label> ';
	print '</dd>';
	print '</dl><input type="submit" class="submit" value="上記設定に基づき CMONOS.JP インストーラーを起動する" /></form>';
	print '</div></body></html>';
} else {
	my $ratio = 0;
	my $interval = 180;
	if ($status == 0) {
		$this->write_file($status_file,1);
	} elsif ($status == 1) {
		my $total = $this->read_file($temp_dir . '/CMONOS_JP.tgz.len');
		$ratio = int((-s $temp_dir . '/CMONOS_JP.tgz') / $total * 1000) / 1000 if $total && -f $temp_dir . '/CMONOS_JP.tgz';
		$interval = 10;
	} elsif ($status == 2) {
		$this->print_extract_error(1) if !-f $temp_dir . '/CMONOS_JP.tgz';
		$this->print_extract_error unless eval 'use Archive::Tar;Archive::Tar->has_zlib_support;';
		$this->write_file($status_file,3);
	} elsif ($status == 3) {
		$interval = 10;
	}
	print '<div id="progress-bar-box-bg"></div>';
	print '<div id="progress-bar-box"><div id="progress-bar-dialog">';
	if ($status < 2) {
		print 'CMONOS.JP を読み込んでいます...';
	} else {
		print 'CMONOS.JP を解凍しています...';
	}
	print '</div><div id="progress-bar-bg"><div id="progress-bar"></div></div></div>';
	print '</body></html>';
	$this->set_refresh_timer($interval);
	$this->bar_update($ratio);
	if ($status == 0) {
		$this->get($this->download_path,$temp_dir . '/CMONOS_JP.tgz');
		$this->write_file($status_file,2);
		$this->bar_update(1);
		$this->refresh;
	} elsif ($status == 2) {
		my $tar = Archive::Tar->new();
		$tar->setcwd($temp_dir);
		$tar->read($temp_dir . '/CMONOS_JP.tgz',1);
		$tar->extract();
		$this->write_file($status_file,4);
		$this->bar_update(1);
		$this->refresh;
	}
}

sub download_path ($;$) {
my($this,$local) = @_;
return(($] >= 5.008001) ? 'http://cmonos.jp/download/CMONOS_JP_2x' : 'http://cmonos.jp/download/CMONOS_JP_1x') . (($local) ? '/CMONOS_JP_local.tgz' : '/CMONOS_JP.tgz');
}

sub print_extract_error ($;$) {
my($this,$dl) = @_;
my $title = ($dl) ? 'ロード' : '解凍';
my $dl_url = $this->download_path;
print <<CONTENT;
<div id="content">
<p>CMONOS.JP を${title}できませんでした。「<a href="$dl_url">CMONOS.JP 全ファイル</a>」をダウンロードし、解凍してできたフォルダ「CMONOS_JP_<em>YYYYMMDD</em>(YYYYMMDDはリリース日を表す数字)」を、このインストーラと同じ階層にある「CMONOS_JP_INSTALLER_TEMP」フォルダにアップロードしてください。</p>
<form action="./installer.cgi" method="GET">
<input type="hidden" name="action" value="install" />
<input type="submit" value="アップロードが完了したのでインストールを再開する" />
</form>
</div>
</body>
</html>
CONTENT
exit;
}

sub get_cmonos_dir ($$) {
my($this,$temp_dir) = @_;
local(*DIR);
if (opendir(DIR,$temp_dir)) {
	my @temp = grep($_ =~ /^CMONOS_JP_[0-9]{8}$/ && -d $temp_dir . '/' . $_,readdir(DIR));
	closedir(DIR);
	return $temp_dir . '/' . $temp[0] if @temp;
}
return '';
}

sub get_file_list ($$) {
my($this,$dir) = @_;
local(*DIR);
if (opendir(DIR,$dir)) {
	my @files = grep($_ !~ /^\.\.?$/ && $_ ne '.DS_Store',readdir(DIR));
	closedir(DIR);
	return @files;
}
return ();
}

sub get_cgi_list ($) {
my $this = shift;
local(*DIR);
my @cgi_files = ();
if (opendir(DIR,'./cgi')) {
	my @cgidir = grep($_ !~ /^\./ && -d './cgi/' . $_ && (-f './cgi/' . $_ . '/start.cgi' || -f './cgi/' . $_ . '/postman.cgi'),readdir(DIR));
	closedir(DIR);
	foreach my $cgi (@cgidir) {
		if (opendir(DIR,'./cgi/' . $cgi)) {
			push(@cgi_files,map { './cgi/' . $cgi . '/' . $_ } grep($_ =~ /^[0-9a-zA-Z_]+\.(?:pl|cgi)$/,readdir(DIR)));
			closedir(DIR);
		}
	}
}
return @cgi_files;
}

sub bar_update ($$) {
my($this,$ratio) = @_;
print <<JAVASCRIPT;
<script type="text/javascript" language="JavaScript">
<!-- <![CDATA[
progressbar.update($ratio);
// ]]> -->
</script>
JAVASCRIPT
}

sub refresh ($) {
my $this = shift;
print <<JAVASCRIPT;
<script type="text/javascript" language="JavaScript">
<!-- <![CDATA[
window.location.reload();
// ]]> -->
</script>
JAVASCRIPT
}

sub set_refresh_timer ($$) {
my($this,$interval) = @_;
$interval *= 1000;
print <<JAVASCRIPT;
<script type="text/javascript" language="JavaScript">
<!-- <![CDATA[
setTimeout(function () { window.location.reload(); },$interval);
// ]]> -->
</script>
JAVASCRIPT
}

sub get_cwd ($) {
my $this = shift;
my $cwd;
if ($ENV{'SCRIPT_FILENAME'}) {
	$cwd = $ENV{'SCRIPT_FILENAME'};
	$cwd =~ s!/[^/]*$!!;
} else {
	chdir;
	use Cwd;
	$cwd = cwd() || getcwd;
}
return $cwd;
}

sub escape ($$) {
my($this,$str) = @_;
return '' if !defined $str;
$str =~ s/&/&amp;/gs;
$str =~ s/"/&quot;/g;
$str =~ s/</&lt;/g;
$str =~ s/>/&gt;/g;
return $str;
}

sub write_file ($$$) {
my($this,$path,$data) = @_;
local(*FH);
if (open(FH, ">>$path")) {
	binmode FH, ":utf8";
	flock(FH, LOCK_EX);
	truncate(FH, 0);
	seek(FH, 0, 0);
	print FH $data;
	close(FH);
	return 1;
} else {
	return 0;
}
}

sub read_file ($$) {
my($this,$path) = @_;
my $data = '';
local(*FH);
if (open(FH, $path)) {
	binmode FH, ":utf8";
	flock(FH, LOCK_SH);
	$data = join('',<FH>);
	close(FH);
}
return $data;
}

sub get ($$$) {
my($this,$url,$content_file) = @_;
if (!defined $this->{'ua'}) {
	if (eval{require LWP::UserAgent}) {
		$this->{'ua'} = LWP::UserAgent->new;
		$this->{'ua'}->agent('CMONOS_JP_INSTALLER');
		$this->{'ua'}->timeout(10);
	} else {
		$this->{'ua'} = '';
	}
}
if (ref $this->{'ua'}) {
	my $request = HTTP::Request->new('GET' => $url);
	my $res = $this->{'ua'}->request($request,$content_file);
	$this->write_file($content_file . '.len',$res->header('Content-Length')) if $content_file;
	return ($content_file) ? (-f $content_file) ? 1 : 0 : $res->content;
} else {
	my($host,$path) = ($url =~ m!^https?://([^/]+)(/?.*)$!is) ? ($1,$2) : ('','');
	my $header = {};
	$header->{'User-Agent'} = 'CMONOS_JP_INSTALLER';
	$header->{'Accept'} = '*/*';
	$header->{'Host'} = $host;
	local(*HTTPCLIENT);
	use Socket;
	my $port = getservbyname('http', 'tcp');
	my $sockaddr = sockaddr_in($port,inet_aton($host));
	socket(HTTPCLIENT, PF_INET, SOCK_STREAM, getprotobyname('tcp'));
	connect(HTTPCLIENT, $sockaddr);
	HTTPCLIENT->autoflush(1);
	print HTTPCLIENT "GET $path HTTP/1.0\x0D\x0A";
	while (my($k,$v) = each %$header) {
		print HTTPCLIENT $k . ': ' . $v . "\x0D\x0A";
	}
	print HTTPCLIENT "\x0D\x0A";
	my $res = <HTTPCLIENT>;
	my $content_flag = 0;
	my $content = '';
	while ($res = <HTTPCLIENT>) {
		if ($content_flag) {
			$content .= $res;
		} else {
			$res =~ s/[\x09\x0A\x0C\x0D\x20]+$//gs;
			if ($res ne '') {
				if ($content_file && $res =~ /Content-Length:[\x09\x0A\x0C\x0D\x20]*([0-9]+)/i) {
					$this->write_file($content_file . '.len',$1);
				}
			} elsif ($content_file) {
				last;
			} else {
				$content_flag = 1;
			}
		}
	}
	if ($content_file) {
		if (open(FILE, ">>$content_file")) {
			binmode FILE;
			flock(FILE, LOCK_EX);
			truncate(FILE, 0);
			seek(FILE, 0, 0);
			my $buf = '';
			while (read(HTTPCLIENT,$buf,8192)) {
				print FILE $buf;
			}
			close(FILE);
		}
	}
	close(HTTPCLIENT);
	return ($content_file) ? (-f $content_file) ? 1 : 0 : $content;
}
}

sub print_license ($) {
my $this = shift;
print <<LICENSE;
<div id="content">

<h1>CMONOS.JP ソフトウェア使用許諾契約</h1>
<p>本ソフトウェアを使用される前に本使用許諾契約（以下「本契約」といいます）をよくお読み下さい。本ソフトウェアの全部又は一部をサーバに設置した場合、本ソフトウェアをサーバに設置する者及び本ソフトウェアを運用する者（以下「ユーザ」といいます）は本契約の各条項の拘束を受けることに合意したものとみなされます。本契約に同意されない場合は、所有する全ての記録メディア及びサーバ又は契約サーバから、本ソフトウェア及び本ソフトウェア関連ファイル並びに本ソフトウェアを利用して作成したファイルを、全て削除しなければなりません。</p>

<h2>第1条（定義）</h2>
<p>「本ソフトウェア」とは（a）本契約書が添付されたファイル、ディスク、CD-ROMその他の媒体に含まれている内容のすべて（（i）<a href="http://openlab.jp/Jcode/index-j.html" target="_blank">Jcode.pm(Jcodeフォルダ以下のスクリプトを含む)</a> を除く Perl スクリプト<sup>※1</sup>（ii）nicEdit.js および nicEditorIcons.gif を除く JavaScript<sup>※2</sup>、（iii）"htdocs/icon" ディレクトリ内の <a href="http://www.feedicons.com/" target="_blank">feed-icon</a> 及び <a href="http://kb2.adobe.com/jp/cps/353/353.html" target="_blank">PDF アイコン</a> 及び "javascript" ディレクトリ内の "player.swf (GrupoSistemas Player)"<sup>※3</sup> を除くアイコンやパレット等のデジタル画像素材、（iv）"cgi-data/template/vicuna" ディレクトリ以下の Vicuna CMS スキンおよび Vicuna CMS テンプレートを元に作成された Vicuna 互換設定ファイル<sup>※4</sup>を除く設定用ファイル並びに（v）関連する説明書又は説明用のファイルを含むものとします）並びに（b）株式会社CMONOS が使用を許諾したソフトウェアのアップグレード、変更されたバージョン、アップデート、追加ファイル及びコピーを指すものとします。
</p>
<ol class="footer">
<li>※1 <a href="http://openlab.jp/Jcode/index-j.html" target="_blank">Jcode.pm</a> は Dan Kogai 氏が作成した、歌代和正氏作の <a href="http://www.srekcah.org/jcode/index.html" target="_blank">jcode.pl</a> 後継となる漢字変換ライブラリです。Jcode.pm の著作権は Dan Kogai 氏が保有します。株式会社CMONOS は、Jcode.pm の利用規約に則り、これを再配布しています。</li>
<li>※2 株式会社CMONOS は、<a href="http://nicedit.com/" target="_blank">NicEdit</a> のライセンス（<a href="http://sourceforge.jp/projects/opensource/wiki/licenses%2FMIT_license" target="_blank">MIT ライセンス</a>）に則り、これを日本語化・カスタマイズし再配布しています。</li>
<li>※3 株式会社CMONOS は、<a href="http://sourceforge.net/projects/gsplayer/">GSPlayer</a> (GrupoSistemas Player - Copyright &copy; 2009, Un Proyecto de <a href="http://www.gruposistemas.com/" target="_blank">GrupoSistemas</a>) のライセンス（<a href="http://sourceforge.jp/projects/opensource/wiki/licenses%2Fnew_BSD_license" target="_blank">BSD ライセンス</a>）に則り、これを再配布しています。</li>
<li>※4 株式会社CMONOS は、Vicuna CMS のライセンス（<a href="http://sourceforge.jp/projects/opensource/wiki/licenses%2FMIT_license" target="_blank">MIT ライセンス</a>）に則り、これらを再配布しています。</li>
</ol>

<h2>第2条（利用形態と料金）</h2>
<p>本ソフトウェアは、1ライセンスごとに、使用許諾された個人又は使用許諾された法人・団体を構成する個人に使用が限定され、1つのウェブページ公開ディレクトリに対してのみ設置することができます。ただし、法人・団体の構成員が別の法人・団体である場合、使用する法人・団体の数だけライセンスが必要です。</p>
<dl class="ex">
<dt>一つのライセンスで良い例</dt>
<dd><ul><li>http://○○○.○○○.jp/ 以下に設置しサブディレクトリを増設したが、それぞれの管理権限を持つ運営者が、使用許諾された個人の直接の友人若しくは三親等内親族若しくは使用許諾された法人・団体を構成する個人で<strong>ある</strong>場合。</li></ul></dd>
<dt>別途ライセンスが必要な例</dt>
<dd><ul>
<li>http://○○○.○○○.jp/ 以下に設置しサブディレクトリを増設したが、それぞれの管理権限を持つ運営者が、使用許諾された個人の直接の友人若しくは三親等内親族では<strong>ない</strong>個人である場合（「団体」とみなします）。</li>
<li>http://○○○.○○○.jp/ 以下に設置しサブディレクトリを増設したが、それぞれの管理権限を持つ運営者が、使用許諾された法人・団体を構成する個人では<strong>ない</strong>場合。</li>
<li>同じサーバであっても、http://○○○.○○○.jp/~△△△/、 http://○○○.○○○.jp/~×××/、http://□□□.○○○.jp/、のようにウェブページ公開領域が別のディレクトリである場合。</li></ul></dd>
</dl>
<p>本ソフトウェアの利用形態に応じた料金は次の通りです。</p>
<table id="license-table">
	<thead>
		<tr>
			<th class="title">利用形態</th>
			<th class="description">説明</th>
			<th class="support">サポート</th>
			<th class="price">料金（税込）</th>
		</tr>
	</thead>
	<tbody>
		<tr id="personal">
			<td class="title">非商用個人</td>
			<td class="description">個人による非商用目的の使用に限られます。ただし、ユーザが個人であっても、コンテンツの半数以上が本人の個人事業又は本人と関わりの深い事業若しくは本人の所属する法人の業務や実績を紹介・宣伝する内容である場合、あるいはコンテンツの半数以上がアフィリエイト目的の商品紹介の羅列やリンク集である場合、このライセンスは適用されず商用とみなされます。また、使用許諾した個人の直接の友人又は三親等内親族ではない「友人の友人」若しくは全く面識のない個人が管理権限を持ち、コンテンツの追加・編集・デザインの変更等、運営に関わる場合も、このライセンスは適用されません。たとえば、メンバーが固定せず、多数に参加を呼びかけるサークルやコミュニティ等で、管理権限をメンバーに分け与えて運用する場合、非商用個人ライセンスではなく商用・法人・団体ライセンスとなります。</td>
			<td class="support">別途有償</td>
			<td class="price"><strong>&yen;0</strong></td>
		</tr>
		<tr id="business">
			<td class="title">商用・法人・団体</td>
			<td class="description">商用目的の使用若しくは商用・非商用にかかわらず法人・組合・自治体・教育機関・多数の個人が参加するサークル・イベント主催団体等各種団体による使用に適用されます。法人・団体を法人・団体が構成している場合、使用する法人・団体の数だけ商用・法人・団体ライセンスが必要です。また、事業者が代行設置する場合又は事業者が本ソフトウェアに含まれる CGI あるいは本ソフトウェアによって増設したサブディレクトリを他者に貸与又は譲与する場合(いわゆるホスティング)若しくは本ソフトウェアの利用を主な目的とする団体で使用する場合は、実際に使用する者が非商用個人であっても、使用する個人・法人・団体の数若しくは設置箇所の数のいずれか数が多い方の数だけ商用・法人・団体ライセンスが必要です。</td>
			<td class="support">別途有償</td>
			<td class="price"><strong>&yen;27,300</strong></td>
		</tr>
	</tbody>
</table>

<h2>第3条（試用期間）</h2>
<p>本ソフトウェアは設置後14日間は前条のいずれの利用形態でも無償で使用できますが、14日間を過ぎた後も使用を続ける場合、ユーザは前条に定める利用形態に応じた料金を、株式会社CMONOS の指定する方法で支払わなければなりません。料金の支払いに同意できない場合は、ただちに、所有する全ての記録メディア及びサーバ若しくは契約サーバから、本ソフトウェア及び本ソフトウェア関連ファイル並びに試用期間中に本ソフトウェアを利用して作成したファイルを、全て削除しなければなりません。</p>

<h2>第4条（設置サーバの登録）</h2>
<p>本ソフトウェアをインターネットから接続できるサーバに設置する場合、商用・法人・団体に該当するユーザは設置から14日以内に、株式会社 CMONOS が定める方法で設置 URL を登録しなければなりません。商用・法人・団体に該当するユーザは、本ソフトウェアを、登録した設置 URL でのみ使用することができます。登録した URL から本ソフトウェア及び本ソフトウェアを利用して作成したファイルを別の URL 移す場合、商用・法人・団体に該当するユーザは株式会社 CMONOS が定める方法で、ただちに設置 URL 変更について届け出なければなりません。その際古い URL のライセンスは失効します。また、インターネットに公開されていないネットワークで使用する場合、商用・法人・団体に該当するユーザは設置から14日以内に、株式会社 CMONOS が定める方法でユーザ名及び連絡先と本ソフトウェアの設置数を、登録しなければなりません。登録されていない URL 又はサーバで商用・法人・団体に該当するユーザによる14日間を越える使用が確認された場合、株式会社 CMONOS は当該ユーザに対し、ただちに第6条に定める違約金を請求します。</p>

<h2>第5条（禁止事項と使用許諾）</h2>
<p>ユーザは本ソフトウェアを記録している媒体の所有権を有しますが、本ソフトウェアに対する全ての権利は 株式会社CMONOS が保有します。ユーザは、有償無償にかかわらず、本ソフトウェア及び本ソフトウェアを翻案若しくは改造したものを、株式会社CMONOS の許諾なく、頒布、転載、貸与、譲渡、販売してはなりません。ユーザは、本ソフトウェアを、日本国の法律に違反する恐れのある情報の公開のために使用してはなりません。ユーザは、本ソフトウェアの欠陥が死傷又は物理的若しくは環境上若しくは財政上の深刻な損害をもたらすような目的のために、本ソフトウェアを使用してはなりません。ユーザが本契約に従う限りにおいて、株式会社CMONOS はユーザに、本ソフトウェアをHTTPサーバーに適切に設置しCGIとして利用することを許諾します。</p>

<h2>第6条（違約金）</h2>
<p>ユーザが本契約に違反し、株式会社CMONOS の許諾なく、本ソフトウェア及び本ソフトウェアを翻案若しくは改造したものを頒布、転載、貸与、譲渡、販売した場合、当該ユーザは、 株式会社CMONOS に対し違約金として、違反件数と本ソフトウェアの商用・法人・団体ライセンスの正価（違反者が第三者に正価より高額に転売したときは、転売金額）を乗じたものの10倍の金額を支払うものとします。また、商用・法人・団体に該当するユーザが本契約に違反し、本ソフトウェアの設置後14日を過ぎても、第2条に定められた料金を支払うことなく、本ソフトウェアの使用を続けた場合は、本ソフトウェアの商用・法人・団体ライセンスの正価の10倍の金額を支払うこととします。</p>

<h2>第7条（責任の制限）</h2>
<p>過失を含むいかなる場合であっても、株式会社CMONOS は本契約に起因する若しくは関連する付随的、間接損害について、一切の責任を負いません。十分な期間本ソフトウェアを無償で試用できることに鑑み、いかなる場合も支払われた料金は返却されません。</p>

<h2>第8条（準拠法）</h2>
<p>本使用許諾契約の成立、効力、履行及び解釈に関しては日本国の法律が適用されるものとします。</p>

<h2>第9条（管轄裁判所）</h2>
<p>本契約に関する一切の紛争に関しては、株式会社 CMONOS の本店が属する地方裁判所を第一審の専属管轄裁判所とします。</p>

<script type="text/javascript" language="JavaScript">
<!-- <![CDATA[
document.write('<form action="./installer.cgi" method="GET"><input type="hidden" name="action" value="install" /><input type="submit" class="submit" value="上記使用許諾契約に同意して CMONOS.JP のインストールを開始する" /></form>');
// ]]> -->
</script>
<noscript><p class="warning">JavaScript を有効にして再読み込みしてください。</p></noscript>

</div>
LICENSE
}
